---
{"dg-publish":true,"permalink":"/bookmarks/knowledge/chess/something-weird-is-happening-with-ll-ms-and-chess/","tags":["ai","llm"]}
---


# [Something weird is happening with LLMs and chess](https://dynomight.net/chess/)

```table-of-contents

```

A year ago, there was a lot of talk about large language models (LLMs) playing chess. Word was that if you trained a big enough model on enough text, then you could send it a partially played game, ask it to predict the next move, and it would play at the level of an advanced amateur.

This seemed important. These are “language” models, after all, designed to predict _language_.

Now, modern LLMs are trained on a sizeable fraction of all the text ever created. This surely includes many chess games. But they weren’t _designed_ to be good at chess. And the games that are available are just _lists of moves_. Yet people found that LLMs could play all the way through to the end game, with never-before-seen boards.

Did the language models build up some kind of internal representation of board state? And how to construct that state from lists of moves in chess’s [extremely confusing notation](https://en.wikipedia.org/wiki/Portable_Game_Notation)? And how valuable different pieces and positions are? And how to force checkmate in an end-game? And they did this all “by accident”, as part of their goal of predicting general text?

If language models can do all that for chess, then maybe it’s a hint of how they deal with other situations too.

So that was very exciting. A year ago.

Since then, there’s mostly been silence. So I decided to check in and see how things are going. Having done that, I can now report: Weirdly.

## What I did

To make LLMs play chess, I sent them prompts like this:

```
You are a chess grandmaster.
Please choose your next move.
Use standard algebraic notation, e.g. "e4" or "Rdf8" or "R1a3".
NEVER give a turn number.
NEVER explain your choice.
Here is a representation of the position:

[Event "Shamkir Chess"]
[White "Anand, Viswanathan"]
[Black "Topalov, Veselin"]
[Result "1-0"]
[WhiteElo "2779"]
[BlackElo "2740"]

1. e4 e6 2. d3 c5 3. Nf3 Nc6 4. g3 Nf6 5.

```

I used the output as a move. I always had the LLM play as white against Stockfish—a standard chess AI—on the lowest difficulty setting.

The first model I tried was [`llama-3.2-3b`](https://huggingface.co/meta-llama/Llama-3.2-3B). This is a “base model”, meaning it is mostly trained to output text, not to chat with you or obey instructions. It’s quite small by modern standards, with only 3 billion parameters. For reference, GPT-2, released back in 2019, had 1.5 billion parameters, and GPT-4 is rumored to have around 1.8 _trillion._

I had it play 50 games, then had a chess engine score each board after each turn in “centipawns”. This is a measure where a pawn is 100 points, but there’s also accounting for position. If the game was over, I assigned a score of +1500 if the LLM won, 0 if there was a tie, and -1500 if it lost.

The results were:

[<?xml version="1.0" encoding="utf-8" standalone="no"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"  "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg xmlns:xlink="http://www.w3.org/1999/xlink" width="432pt" height="252pt" viewBox="0 0 432 252" xmlns="http://www.w3.org/2000/svg" version="1.1"> <metadata>  <rdf:RDF xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:cc="http://creativecommons.org/ns#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">   <cc:Work>    <dc:type rdf:resource="http://purl.org/dc/dcmitype/StillImage"/>    <dc:date>2024-11-14T11:33:39.351541</dc:date>    <dc:format>image/svg+xml</dc:format>    <dc:creator>     <cc:Agent>      <dc:title>Matplotlib v3.8.4, https://matplotlib.org/</dc:title>     </cc:Agent>    </dc:creator>   </cc:Work>  </rdf:RDF> </metadata> <defs>  <style type="text/css">*{stroke-linejoin: round; stroke-linecap: butt}</style> </defs> <g id="figure_1">  <g id="patch_1">   <path d="M 0 252 L 432 252 L 432 0 L 0 0 z" style="fill: #ffffff"/>  </g>  <g id="axes_1">   <g id="patch_2">    <path d="M 54 224.28 L 388.8 224.28 L 388.8 30.24 L 54 30.24 z" style="fill: #ffffff"/>   </g>   <g id="matplotlib.axis_1">    <g id="xtick_1">     <g id="line2d_1"/>     <g id="text_1">      <!-- 0 -->      <g transform="translate(50.689844 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-30" d="M 2118 -38 Q 1600 -38 1193 237 Q 787 512 560 1027 Q 333 1542 333 2240 Q 333 2938 560 3453 Q 787 3968 1193 4243 Q 1600 4518 2118 4518 Q 2637 4518 3043 4243 Q 3450 3968 3680 3453 Q 3910 2938 3910 2240 Q 3910 1542 3680 1027 Q 3450 512 3043 237 Q 2637 -38 2118 -38 zM 2118 384 Q 2509 384 2806 601 Q 3104 819 3270 1235 Q 3437 1651 3437 2240 Q 3437 2829 3270 3245 Q 3104 3661 2806 3878 Q 2509 4096 2118 4096 Q 1728 4096 1430 3878 Q 1133 3661 966 3245 Q 800 2829 800 2240 Q 800 1651 966 1235 Q 1133 819 1430 601 Q 1728 384 2118 384 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="xtick_2">     <g id="line2d_2"/>     <g id="text_2">      <!-- 20 -->      <g transform="translate(114.81 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-32" d="M 3392 410 L 3392 0 L 250 0 L 250 326 L 2112 2150 Q 2464 2496 2589 2749 Q 2714 3002 2714 3258 Q 2714 3654 2442 3875 Q 2170 4096 1664 4096 Q 877 4096 442 3597 L 115 3878 Q 378 4186 784 4352 Q 1190 4518 1702 4518 Q 2387 4518 2787 4195 Q 3187 3872 3187 3309 Q 3187 2963 3033 2643 Q 2880 2323 2451 1907 L 928 410 L 3392 410 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-32"/>       <use xlink:href="#Montserrat-Regular-30" x="56.799988"/>      </g>     </g>    </g>    <g id="xtick_3">     <g id="line2d_3"/>     <g id="text_3">      <!-- 40 -->      <g transform="translate(181.305156 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-34" d="M 4141 1178 L 3245 1178 L 3245 0 L 2784 0 L 2784 1178 L 256 1178 L 256 1510 L 2630 4480 L 3142 4480 L 851 1587 L 2797 1587 L 2797 2624 L 3245 2624 L 3245 1587 L 4141 1587 L 4141 1178 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-34"/>       <use xlink:href="#Montserrat-Regular-30" x="66.099991"/>      </g>     </g>    </g>    <g id="xtick_4">     <g id="line2d_4"/>     <g id="text_4">      <!-- 60 -->      <g transform="translate(248.524531 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-36" d="M 2189 2682 Q 2630 2682 2972 2515 Q 3315 2349 3507 2045 Q 3699 1741 3699 1338 Q 3699 928 3500 614 Q 3302 301 2953 131 Q 2605 -38 2170 -38 Q 1280 -38 806 547 Q 333 1133 333 2202 Q 333 2950 589 3468 Q 845 3987 1305 4252 Q 1766 4518 2381 4518 Q 3046 4518 3443 4275 L 3258 3904 Q 2938 4115 2387 4115 Q 1645 4115 1222 3644 Q 800 3174 800 2278 Q 800 2118 819 1914 Q 986 2278 1350 2480 Q 1715 2682 2189 2682 zM 2150 352 Q 2637 352 2941 617 Q 3245 883 3245 1325 Q 3245 1766 2941 2028 Q 2637 2291 2118 2291 Q 1779 2291 1520 2163 Q 1261 2035 1113 1811 Q 966 1587 966 1312 Q 966 1056 1104 838 Q 1242 621 1510 486 Q 1779 352 2150 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-36"/>       <use xlink:href="#Montserrat-Regular-30" x="60.899994"/>      </g>     </g>    </g>    <g id="xtick_5">     <g id="line2d_5"/>     <g id="text_5">      <!-- 80 -->      <g transform="translate(315.34 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-38" d="M 2899 2349 Q 3322 2208 3549 1926 Q 3776 1645 3776 1248 Q 3776 858 3565 566 Q 3354 275 2960 118 Q 2566 -38 2035 -38 Q 1229 -38 768 307 Q 307 653 307 1248 Q 307 1651 528 1929 Q 749 2208 1171 2349 Q 826 2483 646 2736 Q 467 2989 467 3334 Q 467 3878 892 4198 Q 1318 4518 2035 4518 Q 2509 4518 2864 4374 Q 3219 4230 3417 3961 Q 3616 3693 3616 3334 Q 3616 2989 3430 2736 Q 3245 2483 2899 2349 zM 928 3322 Q 928 2950 1222 2732 Q 1517 2515 2035 2515 Q 2554 2515 2851 2732 Q 3149 2950 3149 3315 Q 3149 3693 2845 3910 Q 2541 4128 2035 4128 Q 1523 4128 1225 3910 Q 928 3693 928 3322 zM 2035 352 Q 2630 352 2966 595 Q 3302 838 3302 1254 Q 3302 1670 2966 1910 Q 2630 2150 2035 2150 Q 1446 2150 1110 1910 Q 774 1670 774 1254 Q 774 832 1107 592 Q 1440 352 2035 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-38"/>       <use xlink:href="#Montserrat-Regular-30" x="63.799988"/>      </g>     </g>    </g>    <g id="xtick_6">     <g id="line2d_6"/>     <g id="text_6">      <!-- 100 -->      <g transform="translate(380.375 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-31" d="M 1574 4480 L 1574 0 L 1114 0 L 1114 4070 L 58 4070 L 58 4480 L 1574 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>      </g>     </g>    </g>    <g id="text_7">     <!-- turns -->     <g transform="translate(208.170312 248.56125) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-74" d="M 2406 205 Q 2278 90 2089 29 Q 1901 -32 1696 -32 Q 1222 -32 966 224 Q 710 480 710 947 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 4102 L 1165 4102 L 1165 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 973 Q 1165 672 1315 515 Q 1466 358 1747 358 Q 1888 358 2019 403 Q 2150 448 2246 531 L 2406 205 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-75" d="M 3667 3366 L 3667 0 L 3232 0 L 3232 614 Q 3053 307 2739 137 Q 2426 -32 2022 -32 Q 1363 -32 982 336 Q 602 704 602 1414 L 602 3366 L 1056 3366 L 1056 1459 Q 1056 928 1318 653 Q 1581 378 2067 378 Q 2598 378 2905 701 Q 3213 1024 3213 1600 L 3213 3366 L 3667 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-72" d="M 1069 2707 Q 1229 3046 1545 3219 Q 1862 3392 2330 3392 L 2330 2950 L 2221 2957 Q 1690 2957 1389 2630 Q 1088 2304 1088 1715 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2707 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-6e" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2746 Q 1254 3053 1584 3222 Q 1914 3392 2349 3392 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-73" d="M 1536 -32 Q 1126 -32 752 86 Q 378 205 166 384 L 371 742 Q 582 576 902 470 Q 1222 365 1568 365 Q 2029 365 2249 509 Q 2470 653 2470 915 Q 2470 1101 2348 1206 Q 2227 1312 2041 1366 Q 1856 1421 1549 1472 Q 1139 1549 889 1629 Q 640 1709 464 1901 Q 288 2093 288 2432 Q 288 2854 640 3123 Q 992 3392 1619 3392 Q 1946 3392 2272 3305 Q 2598 3219 2810 3078 L 2611 2714 Q 2195 3002 1619 3002 Q 1184 3002 963 2848 Q 742 2694 742 2445 Q 742 2253 867 2137 Q 992 2022 1177 1968 Q 1363 1914 1690 1856 Q 2093 1779 2336 1702 Q 2579 1626 2752 1440 Q 2925 1254 2925 928 Q 2925 486 2557 227 Q 2189 -32 1536 -32 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-74"/>      <use xlink:href="#Montserrat-Regular-75" x="40.599991"/>      <use xlink:href="#Montserrat-Regular-72" x="107.899979"/>      <use xlink:href="#Montserrat-Regular-6e" x="147.999969"/>      <use xlink:href="#Montserrat-Regular-73" x="215.699951"/>     </g>    </g>   </g>   <g id="matplotlib.axis_2">    <g id="ytick_1">     <g id="line2d_7"/>     <g id="text_8">      <!-- LOSS -->      <g transform="translate(23.91875 227.667831) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-4c" d="M 730 4480 L 1203 4480 L 1203 410 L 3712 410 L 3712 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4f" d="M 2688 -38 Q 2022 -38 1481 259 Q 941 557 637 1075 Q 333 1594 333 2240 Q 333 2886 637 3404 Q 941 3923 1481 4220 Q 2022 4518 2688 4518 Q 3354 4518 3888 4224 Q 4422 3930 4729 3408 Q 5037 2886 5037 2240 Q 5037 1594 4729 1072 Q 4422 550 3888 256 Q 3354 -38 2688 -38 zM 2688 384 Q 3219 384 3648 624 Q 4077 864 4320 1289 Q 4563 1715 4563 2240 Q 4563 2765 4320 3190 Q 4077 3616 3648 3856 Q 3219 4096 2688 4096 Q 2157 4096 1725 3856 Q 1293 3616 1046 3190 Q 800 2765 800 2240 Q 800 1715 1046 1289 Q 1293 864 1725 624 Q 2157 384 2688 384 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-53" d="M 1965 -38 Q 1466 -38 1008 122 Q 550 282 301 544 L 486 909 Q 730 666 1133 515 Q 1536 365 1965 365 Q 2566 365 2867 585 Q 3168 806 3168 1158 Q 3168 1427 3005 1587 Q 2842 1747 2605 1833 Q 2368 1920 1946 2022 Q 1440 2150 1139 2268 Q 838 2387 624 2630 Q 410 2874 410 3290 Q 410 3629 589 3907 Q 768 4186 1139 4352 Q 1510 4518 2061 4518 Q 2445 4518 2813 4412 Q 3181 4307 3450 4122 L 3290 3744 Q 3008 3930 2688 4022 Q 2368 4115 2061 4115 Q 1472 4115 1174 3888 Q 877 3661 877 3302 Q 877 3034 1040 2870 Q 1203 2707 1449 2620 Q 1696 2534 2112 2432 Q 2605 2310 2909 2192 Q 3213 2074 3424 1834 Q 3635 1594 3635 1184 Q 3635 845 3452 566 Q 3270 288 2892 125 Q 2515 -38 1965 -38 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-4c"/>       <use xlink:href="#Montserrat-Regular-4f" x="58.899994"/>       <use xlink:href="#Montserrat-Regular-53" x="142.799988"/>       <use xlink:href="#Montserrat-Regular-53" x="204.299973"/>      </g>     </g>    </g>    <g id="ytick_2">     <g id="line2d_8"/>     <g id="text_9">      <!-- -1000 -->      <g transform="translate(23.209375 195.435273) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-2d" d="M 384 1933 L 2061 1933 L 2061 1536 L 384 1536 L 384 1933 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-31" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="74.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="140.499985"/>       <use xlink:href="#Montserrat-Regular-30" x="206.699982"/>      </g>     </g>    </g>    <g id="ytick_3">     <g id="line2d_9"/>     <g id="text_10">      <!-- -500 -->      <g transform="translate(27.779687 163.202714) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-35" d="M 1510 2643 Q 2483 2643 2928 2294 Q 3373 1946 3373 1318 Q 3373 928 3190 621 Q 3008 314 2643 138 Q 2278 -38 1747 -38 Q 1267 -38 835 118 Q 403 275 141 538 L 365 902 Q 589 672 953 528 Q 1318 384 1741 384 Q 2298 384 2602 633 Q 2906 883 2906 1299 Q 2906 1760 2563 1997 Q 2221 2234 1408 2234 L 454 2234 L 685 4480 L 3136 4480 L 3136 4070 L 1088 4070 L 934 2643 L 1510 2643 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-35" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="94.799988"/>       <use xlink:href="#Montserrat-Regular-30" x="160.999985"/>      </g>     </g>    </g>    <g id="ytick_4">     <g id="line2d_10"/>     <g id="text_11">      <!-- 0 -->      <g transform="translate(43.879688 130.970156) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="ytick_5">     <g id="line2d_11"/>     <g id="text_12">      <!-- 500 -->      <g transform="translate(31.6 98.737598) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-35"/>       <use xlink:href="#Montserrat-Regular-30" x="56.599991"/>       <use xlink:href="#Montserrat-Regular-30" x="122.799988"/>      </g>     </g>    </g>    <g id="ytick_6">     <g id="line2d_12"/>     <g id="text_13">      <!-- 1000 -->      <g transform="translate(27.029687 66.50504) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="168.499985"/>      </g>     </g>    </g>    <g id="ytick_7">     <g id="line2d_13"/>     <g id="text_14">      <!-- WIN -->      <g transform="translate(28.240625 34.272482) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-57" d="M 6880 4480 L 5370 0 L 4870 0 L 3558 3821 L 2240 0 L 1747 0 L 237 4480 L 723 4480 L 2016 627 L 3354 4480 L 3795 4480 L 5114 608 L 6426 4480 L 6880 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-49" d="M 730 4480 L 1203 4480 L 1203 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4e" d="M 4474 4480 L 4474 0 L 4083 0 L 1203 3635 L 1203 0 L 730 0 L 730 4480 L 1120 4480 L 4006 845 L 4006 4480 L 4474 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-57"/>       <use xlink:href="#Montserrat-Regular-49" x="111.099991"/>       <use xlink:href="#Montserrat-Regular-4e" x="141.299988"/>      </g>     </g>    </g>    <g id="text_15">     <!-- centipawn difference -->     <g transform="translate(17.26875 180.677188) rotate(-90) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-63" d="M 2029 -32 Q 1530 -32 1136 185 Q 742 403 518 796 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1136 3174 Q 1530 3392 2029 3392 Q 2464 3392 2806 3222 Q 3149 3053 3347 2726 L 3008 2496 Q 2842 2746 2586 2870 Q 2330 2995 2029 2995 Q 1664 2995 1373 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1293 918 995 Q 1082 698 1373 534 Q 1664 371 2029 371 Q 2330 371 2586 492 Q 2842 614 3008 864 L 3347 634 Q 3149 307 2803 137 Q 2458 -32 2029 -32 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-65" d="M 3565 1542 L 749 1542 Q 787 1018 1152 694 Q 1517 371 2074 371 Q 2387 371 2649 483 Q 2912 595 3104 813 L 3360 518 Q 3136 250 2800 109 Q 2464 -32 2061 -32 Q 1542 -32 1142 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 508 2566 Q 723 2957 1097 3174 Q 1472 3392 1939 3392 Q 2406 3392 2777 3174 Q 3149 2957 3360 2569 Q 3571 2182 3571 1683 L 3565 1542 zM 1939 3002 Q 1453 3002 1123 2691 Q 794 2381 749 1882 L 3136 1882 Q 3091 2381 2761 2691 Q 2432 3002 1939 3002 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-69" d="M 634 3366 L 1088 3366 L 1088 0 L 634 0 L 634 3366 zM 864 4102 Q 723 4102 627 4198 Q 531 4294 531 4429 Q 531 4557 627 4653 Q 723 4749 864 4749 Q 1005 4749 1101 4656 Q 1197 4563 1197 4435 Q 1197 4294 1101 4198 Q 1005 4102 864 4102 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-70" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1178 3827 787 Q 3610 397 3229 182 Q 2848 -32 2368 -32 Q 1958 -32 1628 137 Q 1299 307 1088 634 L 1088 -1242 L 634 -1242 L 634 3366 L 1069 3366 L 1069 2701 Q 1274 3034 1610 3213 Q 1946 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2364 Q 3264 2662 2976 2828 Q 2688 2995 2336 2995 Q 1978 2995 1693 2828 Q 1408 2662 1245 2364 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-61" d="M 1811 3392 Q 2470 3392 2822 3062 Q 3174 2733 3174 2086 L 3174 0 L 2739 0 L 2739 525 Q 2586 262 2288 115 Q 1990 -32 1581 -32 Q 1018 -32 685 237 Q 352 506 352 947 Q 352 1376 662 1638 Q 973 1901 1651 1901 L 2720 1901 L 2720 2106 Q 2720 2541 2477 2768 Q 2234 2995 1766 2995 Q 1446 2995 1152 2889 Q 858 2784 646 2598 L 442 2938 Q 698 3155 1056 3273 Q 1414 3392 1811 3392 zM 1651 326 Q 2035 326 2310 502 Q 2586 678 2720 1011 L 2720 1562 L 1664 1562 Q 800 1562 800 960 Q 800 666 1024 496 Q 1248 326 1651 326 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-77" d="M 5574 3366 L 4301 0 L 3872 0 L 2810 2758 L 1747 0 L 1318 0 L 51 3366 L 486 3366 L 1542 506 L 2624 3366 L 3014 3366 L 4090 506 L 5158 3366 L 5574 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-20" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-64" d="M 3706 4749 L 3706 0 L 3270 0 L 3270 666 Q 3066 326 2730 147 Q 2394 -32 1971 -32 Q 1498 -32 1114 185 Q 730 403 512 793 Q 294 1184 294 1683 Q 294 2182 512 2572 Q 730 2963 1114 3177 Q 1498 3392 1971 3392 Q 2381 3392 2710 3222 Q 3040 3053 3251 2726 L 3251 4749 L 3706 4749 zM 2010 371 Q 2362 371 2646 534 Q 2931 698 3094 998 Q 3258 1299 3258 1683 Q 3258 2067 3094 2368 Q 2931 2669 2646 2832 Q 2362 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-66" d="M 1747 4397 Q 1453 4397 1302 4237 Q 1152 4077 1152 3763 L 1152 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 0 L 710 0 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 3782 Q 710 4243 976 4512 Q 1242 4781 1728 4781 Q 1920 4781 2099 4726 Q 2278 4672 2400 4570 L 2246 4230 Q 2048 4397 1747 4397 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-63"/>      <use xlink:href="#Montserrat-Regular-65" x="56.299988"/>      <use xlink:href="#Montserrat-Regular-6e" x="116.699982"/>      <use xlink:href="#Montserrat-Regular-74" x="184.399963"/>      <use xlink:href="#Montserrat-Regular-69" x="224.999954"/>      <use xlink:href="#Montserrat-Regular-70" x="251.899948"/>      <use xlink:href="#Montserrat-Regular-61" x="319.699936"/>      <use xlink:href="#Montserrat-Regular-77" x="378.699921"/>      <use xlink:href="#Montserrat-Regular-6e" x="466.599915"/>      <use xlink:href="#Montserrat-Regular-20" x="534.299896"/>      <use xlink:href="#Montserrat-Regular-64" x="560.499893"/>      <use xlink:href="#Montserrat-Regular-69" x="628.299881"/>      <use xlink:href="#Montserrat-Regular-66" x="655.199875"/>      <use xlink:href="#Montserrat-Regular-66" x="689.099869"/>      <use xlink:href="#Montserrat-Regular-65" x="722.999863"/>      <use xlink:href="#Montserrat-Regular-72" x="783.399857"/>      <use xlink:href="#Montserrat-Regular-65" x="823.499847"/>      <use xlink:href="#Montserrat-Regular-6e" x="883.899841"/>      <use xlink:href="#Montserrat-Regular-63" x="951.599823"/>      <use xlink:href="#Montserrat-Regular-65" x="1007.899811"/>     </g>    </g>   </g>   <g id="line2d_14">    <path d="M 54 127.26 L 57.348 121.006884 L 60.696 124.35907 L 64.044 122.811907 L 67.392 122.811907 L 70.74 123.456558 L 74.088 124.488 L 77.436 155.624651 L 80.784 131.772558 L 84.132 162.329023 L 87.48 161.619907 L 90.828 167.744093 L 94.176 191.016 L 97.524 223.893209 L 100.872 223.957674 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_15">    <path d="M 54 127.26 L 57.348 123.521023 L 60.696 123.778884 L 64.044 108.242791 L 67.392 122.489581 L 70.74 118.557209 L 74.088 113.52893 L 77.436 118.234884 L 80.784 114.753767 L 84.132 111.981767 L 87.48 109.725488 L 90.828 112.755349 L 94.176 113.27107 L 97.524 137.703349 L 100.872 161.555442 L 104.22 159.492558 L 107.568 166.454791 L 110.916 168.195349 L 114.264 177.156 L 117.612 175.479907 L 120.96 172.643442 L 124.308 184.827349 L 127.656 180.250326 L 131.004 190.435814 L 134.352 190.306884 L 137.7 188.37293 L 141.048 199.396465 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_16">    <path d="M 54 127.26 L 57.348 121.071349 L 60.696 122.554047 L 64.044 119.395256 L 67.392 122.102791 L 70.74 121.071349 L 74.088 127.195535 L 77.436 129.193953 L 80.784 130.612186 L 84.132 137.832279 L 87.48 138.412465 L 90.828 148.726884 L 94.176 129.838605 L 97.524 164.069581 L 100.872 223.893209 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_17">    <path d="M 54 127.26 L 57.348 124.874791 L 60.696 123.263163 L 64.044 124.165674 L 67.392 123.907814 L 70.74 124.165674 L 74.088 118.621674 L 77.436 108.049395 L 80.784 147.695442 L 84.132 152.46586 L 87.48 167.357302 L 90.828 157.365209 L 94.176 194.110326 L 97.524 191.016 L 100.872 190.951535 L 104.22 223.893209 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_18">    <path d="M 54 127.26 L 57.348 125.648372 L 60.696 125.648372 L 64.044 124.23014 L 67.392 124.036744 L 70.74 128.613767 L 74.088 124.681395 L 77.436 144.214326 L 80.784 149.887256 L 84.132 155.624651 L 87.48 166.261395 L 90.828 166.261395 L 94.176 188.37293 L 97.524 184.440558 L 100.872 184.182698 L 104.22 199.847721 L 107.568 206.358698 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_19">    <path d="M 54 127.26 L 57.348 125.003721 L 60.696 126.808744 L 64.044 125.777302 L 67.392 126.679814 L 70.74 127.066605 L 74.088 128.549302 L 77.436 127.840186 L 80.784 126.035163 L 84.132 119.588651 L 87.48 127.453395 L 90.828 128.936093 L 94.176 126.550884 L 97.524 128.936093 L 100.872 156.720558 L 104.22 159.814884 L 107.568 177.28493 L 110.916 182.764465 L 114.264 186.696837 L 117.612 184.956279 L 120.96 188.37293 L 124.308 186.181116 L 127.656 195.786419 L 131.004 201.072558 L 134.352 189.017581 L 137.7 208.099256 L 141.048 206.809953 L 144.396 207.196744 L 147.744 223.893209 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_20">    <path d="M 54 127.26 L 57.348 122.360651 L 60.696 123.778884 L 64.044 126.164093 L 67.392 122.940837 L 70.74 125.777302 L 74.088 125.197116 L 77.436 155.044465 L 80.784 167.034977 L 84.132 177.542791 L 87.48 179.218884 L 90.828 178.445302 L 94.176 183.602512 L 97.524 186.696837 L 100.872 191.982977 L 104.22 223.893209 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_21">    <path d="M 54 127.26 L 57.348 124.61693 L 60.696 125.777302 L 64.044 126.486419 L 67.392 126.035163 L 70.74 123.907814 L 74.088 127.195535 L 77.436 127.26 L 80.784 126.937674 L 84.132 124.681395 L 87.48 126.421953 L 90.828 127.582326 L 94.176 121.393674 L 97.524 136.994233 L 100.872 168.646605 L 104.22 177.220465 L 107.568 179.67014 L 110.916 192.047442 L 114.264 199.912186 L 117.612 201.265953 L 120.96 197.269116 L 124.308 196.237674 L 127.656 196.68893 L 131.004 196.624465 L 134.352 190.049023 L 137.7 206.165302 L 141.048 206.036372 L 144.396 198.816279 L 147.744 203.780093 L 151.092 203.328837 L 154.44 198.171628 L 157.788 205.327256 L 161.136 201.717209 L 164.484 191.531721 L 167.832 181.668558 L 171.18 198.107163 L 174.528 181.668558 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_22">    <path d="M 54 127.26 L 57.348 122.102791 L 60.696 125.454977 L 64.044 120.297767 L 67.392 122.425116 L 70.74 120.620093 L 74.088 123.649953 L 77.436 124.488 L 80.784 122.554047 L 84.132 128.355907 L 87.48 131.965953 L 90.828 160.65293 L 94.176 160.65293 L 97.524 178.122977 L 100.872 179.412279 L 104.22 184.182698 L 107.568 185.020744 L 110.916 183.344651 L 114.264 199.912186 L 117.612 196.56 L 120.96 198.236093 L 124.308 207.583535 L 127.656 211.967163 L 131.004 213.772186 L 134.352 223.893209 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_23">    <path d="M 54 127.26 L 57.348 123.843349 L 60.696 127.26 L 64.044 125.583907 L 67.392 125.519442 L 70.74 125.970698 L 74.088 118.944 L 77.436 121.909395 L 80.784 148.662419 L 84.132 158.138791 L 87.48 156.720558 L 90.828 168.066419 L 94.176 160.846326 L 97.524 154.464279 L 100.872 161.168651 L 104.22 151.563349 L 107.568 154.206419 L 110.916 174.448465 L 114.264 185.278605 L 117.612 191.660651 L 120.96 192.240837 L 124.308 191.016 L 127.656 193.336744 L 131.004 188.824186 L 134.352 190.88707 L 137.7 191.982977 L 141.048 194.497116 L 144.396 193.91693 L 147.744 192.369767 L 151.092 195.012837 L 154.44 192.756558 L 157.788 190.435814 L 161.136 188.308465 L 164.484 200.621302 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_24">    <path d="M 54 127.26 L 57.348 125.777302 L 60.696 122.489581 L 64.044 127.195535 L 67.392 120.813488 L 70.74 124.552465 L 74.088 142.860558 L 77.436 145.052372 L 80.784 146.986326 L 84.132 142.731628 L 87.48 143.376279 L 90.828 147.953302 L 94.176 165.101023 L 97.524 175.479907 L 100.872 177.220465 L 104.22 179.605674 L 107.568 190.242419 L 110.916 188.50186 L 114.264 189.984558 L 117.612 191.016 L 120.96 196.108744 L 124.308 185.149674 L 127.656 186.954698 L 131.004 184.311628 L 134.352 181.281767 L 137.7 181.926419 L 141.048 187.405953 L 144.396 195.850884 L 147.744 181.475163 L 151.092 199.976651 L 154.44 223.699814 L 157.788 223.828744 L 161.136 223.893209 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_25">    <path d="M 54 127.26 L 57.348 122.038326 L 60.696 116.816651 L 64.044 126.486419 L 67.392 125.970698 L 70.74 122.489581 L 74.088 145.439163 L 77.436 154.206419 L 80.784 153.690698 L 84.132 157.042884 L 87.48 158.267721 L 90.828 156.075907 L 94.176 156.398233 L 97.524 159.041302 L 100.872 166.841581 L 104.22 163.231535 L 107.568 167.615163 L 110.916 172.578977 L 114.264 170.387163 L 117.612 179.734605 L 120.96 182.248744 L 124.308 181.668558 L 127.656 184.182698 L 131.004 175.157581 L 134.352 179.992465 L 137.7 185.21414 L 141.048 187.277023 L 144.396 197.591442 L 147.744 196.946791 L 151.092 192.885488 L 154.44 199.783256 L 157.788 199.976651 L 161.136 198.171628 L 164.484 201.459349 L 167.832 205.00493 L 171.18 199.654326 L 174.528 200.814698 L 177.876 208.872837 L 181.224 202.039535 L 184.572 192.627628 L 187.92 223.893209 L 191.268 223.893209 L 194.616 223.893209 L 197.964 223.893209 L 201.312 223.893209 L 204.66 223.828744 L 208.008 223.893209 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_26">    <path d="M 54 127.26 L 57.348 122.360651 L 60.696 119.975442 L 64.044 121.522605 L 67.392 120.104372 L 70.74 137.767814 L 74.088 134.480093 L 77.436 130.483256 L 80.784 134.480093 L 84.132 132.481674 L 87.48 97.348186 L 90.828 92.642233 L 94.176 94.25386 L 97.524 102.827721 L 100.872 104.310419 L 104.22 174.770791 L 107.568 175.544372 L 110.916 186.116651 L 114.264 176.382419 L 117.612 185.72986 L 120.96 188.953116 L 124.308 192.047442 L 127.656 184.762884 L 131.004 184.569488 L 134.352 185.020744 L 137.7 186.245581 L 141.048 184.698419 L 144.396 200.234512 L 147.744 198.042698 L 151.092 198.365023 L 154.44 199.332 L 157.788 202.23293 L 161.136 185.72986 L 164.484 195.721953 L 167.832 220.927814 L 171.18 203.909023 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_27">    <path d="M 54 127.26 L 57.348 122.360651 L 60.696 120.813488 L 64.044 119.717581 L 67.392 115.398419 L 70.74 119.782047 L 74.088 122.231721 L 77.436 122.296186 L 80.784 119.910977 L 84.132 125.906233 L 87.48 127.646791 L 90.828 158.654512 L 94.176 168.130884 L 97.524 178.122977 L 100.872 168.453209 L 104.22 146.664 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_28">    <path d="M 54 127.26 L 57.348 125.777302 L 60.696 125.261581 L 64.044 125.970698 L 67.392 124.74586 L 70.74 133.384186 L 74.088 123.972279 L 77.436 126.486419 L 80.784 125.906233 L 84.132 130.032 L 87.48 131.450233 L 90.828 131.192372 L 94.176 135.060279 L 97.524 130.870047 L 100.872 145.954884 L 104.22 143.827535 L 107.568 145.632558 L 110.916 148.275628 L 114.264 152.208 L 117.612 152.014605 L 120.96 157.816465 L 124.308 162.329023 L 127.656 163.55386 L 131.004 167.808558 L 134.352 160.717395 L 137.7 160.137209 L 141.048 179.863535 L 144.396 177.02707 L 147.744 177.736186 L 151.092 186.116651 L 154.44 184.762884 L 157.788 187.212558 L 161.136 192.111907 L 164.484 195.657488 L 167.832 191.14493 L 171.18 182.377674 L 174.528 183.860372 L 177.876 186.761302 L 181.224 187.277023 L 184.572 191.080465 L 187.92 179.79907 L 191.268 223.893209 L 194.616 223.893209 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_29">    <path d="M 54 127.26 L 57.348 119.395256 L 60.696 121.716 L 64.044 120.362233 L 67.392 121.45814 L 70.74 121.200279 L 74.088 120.297767 L 77.436 122.940837 L 80.784 124.165674 L 84.132 123.649953 L 87.48 124.165674 L 90.828 123.069767 L 94.176 126.293023 L 97.524 128.742698 L 100.872 129.065023 L 104.22 171.418605 L 107.568 165.165488 L 110.916 164.520837 L 114.264 168.001953 L 117.612 167.034977 L 120.96 140.99107 L 124.308 162.006698 L 127.656 155.882512 L 131.004 163.102605 L 134.352 165.874605 L 137.7 169.226791 L 141.048 164.714233 L 144.396 179.734605 L 147.744 172.514512 L 151.092 201.394884 L 154.44 210.871256 L 157.788 223.828744 L 161.136 223.828744 L 164.484 223.893209 L 167.832 223.893209 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_30">    <path d="M 54 127.26 L 57.348 121.522605 L 60.696 118.621674 L 64.044 119.717581 L 67.392 121.135814 L 70.74 119.20186 L 74.088 111.143721 L 77.436 112.368558 L 80.784 111.208186 L 84.132 112.046233 L 87.48 117.783628 L 90.828 113.206605 L 94.176 109.725488 L 97.524 105.857581 L 100.872 110.563535 L 104.22 133.319721 L 107.568 157.752 L 110.916 162.780279 L 114.264 170.000372 L 117.612 163.231535 L 120.96 181.410698 L 124.308 194.819442 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_31">    <path d="M 54 127.26 L 57.348 124.61693 L 60.696 119.524186 L 64.044 121.135814 L 67.392 116.172 L 70.74 118.68614 L 74.088 116.752186 L 77.436 119.395256 L 80.784 116.42986 L 84.132 120.813488 L 87.48 121.522605 L 90.828 104.117023 L 94.176 121.200279 L 97.524 113.786791 L 100.872 136.156186 L 104.22 172.192186 L 107.568 183.215721 L 110.916 182.7 L 114.264 181.281767 L 117.612 188.953116 L 120.96 185.472 L 124.308 182.184279 L 127.656 190.951535 L 131.004 201.97507 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_32">    <path d="M 54 127.26 L 57.348 122.554047 L 60.696 125.648372 L 64.044 126.099628 L 67.392 124.74586 L 70.74 122.682977 L 74.088 119.782047 L 77.436 123.327628 L 80.784 121.006884 L 84.132 118.299349 L 87.48 138.412465 L 90.828 163.682791 L 94.176 175.608837 L 97.524 190.177953 L 100.872 192.563163 L 104.22 189.984558 L 107.568 192.176372 L 110.916 177.736186 L 114.264 195.335163 L 117.612 196.56 L 120.96 195.593023 L 124.308 187.148093 L 127.656 187.148093 L 131.004 202.619721 L 134.352 223.893209 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_33">    <path d="M 54 127.26 L 57.348 126.293023 L 60.696 123.649953 L 64.044 119.266326 L 67.392 122.425116 L 70.74 121.071349 L 74.088 115.398419 L 77.436 223.957674 L 80.784 223.957674 L 84.132 223.957674 L 87.48 223.957674 L 90.828 223.957674 L 94.176 223.957674 L 97.524 223.957674 L 100.872 223.957674 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_34">    <path d="M 54 127.26 L 57.348 125.777302 L 60.696 124.294605 L 64.044 120.233302 L 67.392 125.197116 L 70.74 169.484651 L 74.088 167.679628 L 77.436 161.362047 L 80.784 162.909209 L 84.132 161.426512 L 87.48 163.811721 L 90.828 162.329023 L 94.176 166.003535 L 97.524 175.028651 L 100.872 181.023907 L 104.22 186.310047 L 107.568 184.182698 L 110.916 188.824186 L 114.264 188.244 L 117.612 188.888651 L 120.96 187.857209 L 124.308 163.03814 L 127.656 194.626047 L 131.004 195.335163 L 134.352 199.332 L 137.7 198.880744 L 141.048 212.160558 L 144.396 208.228186 L 147.744 219.380651 L 151.092 216.608651 L 154.44 214.610233 L 157.788 218.284744 L 161.136 218.026884 L 164.484 223.828744 L 167.832 212.225023 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_35">    <path d="M 54 127.26 L 57.348 124.874791 L 60.696 123.263163 L 64.044 118.557209 L 67.392 121.200279 L 70.74 119.266326 L 74.088 140.604279 L 77.436 140.604279 L 80.784 141.506791 L 84.132 140.217488 L 87.48 149.500465 L 90.828 157.49414 L 94.176 155.23786 L 97.524 159.170233 L 100.872 161.426512 L 104.22 181.410698 L 107.568 176.382419 L 110.916 191.338326 L 114.264 188.630791 L 117.612 192.434233 L 120.96 182.7 L 124.308 210.42 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_36">    <path d="M 54 127.26 L 57.348 125.777302 L 60.696 125.261581 L 64.044 125.970698 L 67.392 123.198698 L 70.74 125.068186 L 74.088 125.197116 L 77.436 124.74586 L 80.784 125.712837 L 84.132 123.198698 L 87.48 119.782047 L 90.828 125.261581 L 94.176 127.775721 L 97.524 154.593209 L 100.872 147.244186 L 104.22 147.244186 L 107.568 141.829116 L 110.916 152.014605 L 114.264 158.912372 L 117.612 158.203256 L 120.96 152.014605 L 124.308 146.148279 L 127.656 163.55386 L 131.004 159.492558 L 134.352 179.992465 L 137.7 206.681023 L 141.048 204.166884 L 144.396 198.107163 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_37">    <path d="M 54 127.26 L 57.348 122.554047 L 60.696 118.299349 L 64.044 123.392093 L 67.392 122.038326 L 70.74 125.970698 L 74.088 120.555628 L 77.436 115.785209 L 80.784 117.525767 L 84.132 110.047814 L 87.48 111.981767 L 90.828 110.37014 L 94.176 148.726884 L 97.524 137.767814 L 100.872 157.752 L 104.22 155.753581 L 107.568 158.847907 L 110.916 158.654512 L 114.264 166.841581 L 117.612 168.84 L 120.96 177.478326 L 124.308 174.190605 L 127.656 168.066419 L 131.004 164.520837 L 134.352 176.382419 L 137.7 173.352558 L 141.048 169.291256 L 144.396 172.772372 L 147.744 168.388744 L 151.092 168.904465 L 154.44 169.484651 L 157.788 169.871442 L 161.136 177.220465 L 164.484 168.324279 L 167.832 174.899721 L 171.18 183.860372 L 174.528 203.522233 L 177.876 182.57107 L 181.224 223.893209 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_38">    <path d="M 54 127.26 L 57.348 123.392093 L 60.696 119.588651 L 64.044 117.654698 L 67.392 117.074512 L 70.74 124.165674 L 74.088 117.590233 L 77.436 122.425116 L 80.784 131.708093 L 84.132 123.005302 L 87.48 134.222233 L 90.828 136.027256 L 94.176 148.597953 L 97.524 146.019349 L 100.872 120.426698 L 104.22 158.396651 L 107.568 154.98 L 110.916 159.557023 L 114.264 161.684372 L 117.612 157.042884 L 120.96 155.560186 L 124.308 157.752 L 127.656 164.069581 L 131.004 165.487814 L 134.352 173.932744 L 137.7 168.066419 L 141.048 168.453209 L 144.396 181.410698 L 147.744 179.089953 L 151.092 192.434233 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_39">    <path d="M 54 127.26 L 57.348 127.26 L 60.696 126.808744 L 64.044 126.550884 L 67.392 122.554047 L 70.74 124.036744 L 74.088 126.035163 L 77.436 121.651535 L 80.784 123.714419 L 84.132 125.132651 L 87.48 128.033581 L 90.828 130.354326 L 94.176 119.782047 L 97.524 136.671907 L 100.872 147.502047 L 104.22 143.440744 L 107.568 155.882512 L 110.916 164.714233 L 114.264 170.645023 L 117.612 176.769209 L 120.96 223.893209 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_40">    <path d="M 54 127.26 L 57.348 126.808744 L 60.696 123.263163 L 64.044 120.491163 L 67.392 124.294605 L 70.74 126.550884 L 74.088 125.648372 L 77.436 125.777302 L 80.784 125.777302 L 84.132 127.38893 L 87.48 100.378047 L 90.828 92.577767 L 94.176 119.717581 L 97.524 109.854419 L 100.872 119.524186 L 104.22 146.986326 L 107.568 146.083814 L 110.916 143.247349 L 114.264 140.604279 L 117.612 157.365209 L 120.96 154.85107 L 124.308 162.586884 L 127.656 166.712651 L 131.004 166.132465 L 134.352 145.052372 L 137.7 149.887256 L 141.048 183.731442 L 144.396 161.813302 L 147.744 185.278605 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_41">    <path d="M 54 127.26 L 57.348 123.005302 L 60.696 116.107535 L 64.044 117.074512 L 67.392 121.329209 L 70.74 120.039907 L 74.088 120.491163 L 77.436 120.104372 L 80.784 150.016186 L 84.132 164.198512 L 87.48 168.904465 L 90.828 180.830512 L 94.176 185.472 L 97.524 198.687349 L 100.872 204.037953 L 104.22 202.104 L 107.568 203.586698 L 110.916 202.490791 L 114.264 223.893209 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_42">    <path d="M 54 127.26 L 57.348 125.777302 L 60.696 125.261581 L 64.044 118.299349 L 67.392 124.35907 L 70.74 142.215907 L 74.088 135.511535 L 77.436 146.341674 L 80.784 168.195349 L 84.132 167.292837 L 87.48 173.674884 L 90.828 174.64186 L 94.176 167.615163 L 97.524 164.907628 L 100.872 173.417023 L 104.22 183.215721 L 107.568 169.355721 L 110.916 180.830512 L 114.264 185.60093 L 117.612 191.016 L 120.96 192.821023 L 124.308 195.593023 L 127.656 193.91693 L 131.004 200.298977 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_43">    <path d="M 54 127.26 L 57.348 122.360651 L 60.696 122.231721 L 64.044 121.58707 L 67.392 124.488 L 70.74 125.906233 L 74.088 124.23014 L 77.436 126.035163 L 80.784 150.918698 L 84.132 152.530326 L 87.48 168.517674 L 90.828 160.330605 L 94.176 171.998791 L 97.524 170.580558 L 100.872 172.321116 L 104.22 171.805395 L 107.568 172.192186 L 110.916 182.377674 L 114.264 178.961023 L 117.612 180.959442 L 120.96 200.621302 L 124.308 198.687349 L 127.656 192.756558 L 131.004 194.368186 L 134.352 191.531721 L 137.7 190.822605 L 141.048 189.275442 L 144.396 190.693674 L 147.744 184.182698 L 151.092 186.632372 L 154.44 185.149674 L 157.788 190.177953 L 161.136 223.893209 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_44">    <path d="M 54 127.26 L 57.348 125.777302 L 60.696 125.326047 L 64.044 127.00214 L 67.392 127.195535 L 70.74 128.033581 L 74.088 127.840186 L 77.436 126.937674 L 80.784 127.13107 L 84.132 128.678233 L 87.48 129.258419 L 90.828 128.613767 L 94.176 129.967535 L 97.524 139.766233 L 100.872 126.615349 L 104.22 125.906233 L 107.568 130.870047 L 110.916 158.912372 L 114.264 162.780279 L 117.612 177.800651 L 120.96 186.696837 L 124.308 174.964186 L 127.656 173.932744 L 131.004 182.764465 L 134.352 170.645023 L 137.7 186.567907 L 141.048 201.652744 L 144.396 202.104 L 147.744 203.522233 L 151.092 200.814698 L 154.44 205.13386 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_45">    <path d="M 54 127.26 L 57.348 122.167256 L 60.696 123.069767 L 64.044 119.653116 L 67.392 118.363814 L 70.74 126.099628 L 74.088 126.228558 L 77.436 125.261581 L 80.784 128.678233 L 84.132 131.192372 L 87.48 134.544558 L 90.828 131.063442 L 94.176 179.154419 L 97.524 168.195349 L 100.872 175.802233 L 104.22 169.226791 L 107.568 175.608837 L 110.916 181.668558 L 114.264 186.245581 L 117.612 186.632372 L 120.96 186.632372 L 124.308 208.486047 L 127.656 194.497116 L 131.004 197.913767 L 134.352 198.107163 L 137.7 200.363442 L 141.048 200.685767 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_46">    <path d="M 54 127.26 L 57.348 125.068186 L 60.696 123.972279 L 64.044 123.649953 L 67.392 123.134233 L 70.74 115.205023 L 74.088 117.912558 L 77.436 126.937674 L 80.784 135.70493 L 84.132 136.736372 L 87.48 168.453209 L 90.828 169.871442 L 94.176 163.811721 L 97.524 166.519256 L 100.872 157.558605 L 104.22 162.780279 L 107.568 170.967349 L 110.916 179.992465 L 114.264 182.7 L 117.612 182.506605 L 120.96 173.352558 L 124.308 175.995628 L 127.656 191.402791 L 131.004 223.893209 L 134.352 223.893209 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_47">    <path d="M 54 127.26 L 57.348 122.554047 L 60.696 122.554047 L 64.044 123.907814 L 67.392 124.294605 L 70.74 127.775721 L 74.088 127.26 L 77.436 129.77414 L 80.784 129.516279 L 84.132 130.934512 L 87.48 126.679814 L 90.828 126.099628 L 94.176 161.748837 L 97.524 166.519256 L 100.872 175.415442 L 104.22 171.418605 L 107.568 179.863535 L 110.916 182.506605 L 114.264 194.690512 L 117.612 173.932744 L 120.96 195.979814 L 124.308 196.495535 L 127.656 197.720372 L 131.004 192.692093 L 134.352 206.358698 L 137.7 207.39014 L 141.048 192.821023 L 144.396 214.932558 L 147.744 210.033209 L 151.092 208.099256 L 154.44 223.828744 L 157.788 223.893209 L 161.136 209.388558 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_48">    <path d="M 54 127.26 L 57.348 124.874791 L 60.696 123.972279 L 64.044 125.068186 L 67.392 144.987907 L 70.74 146.341674 L 74.088 146.212744 L 77.436 147.759907 L 80.784 149.500465 L 84.132 149.500465 L 87.48 147.502047 L 90.828 143.76307 L 94.176 159.492558 L 97.524 165.165488 L 100.872 169.742512 L 104.22 175.608837 L 107.568 180.830512 L 110.916 177.28493 L 114.264 177.41386 L 117.612 179.79907 L 120.96 180.637116 L 124.308 189.855628 L 127.656 188.888651 L 131.004 178.896558 L 134.352 194.561581 L 137.7 193.336744 L 141.048 192.434233 L 144.396 223.828744 L 147.744 223.893209 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_49">    <path d="M 54 127.26 L 57.348 119.846512 L 60.696 114.818233 L 64.044 119.782047 L 67.392 113.722326 L 70.74 114.753767 L 74.088 119.975442 L 77.436 118.81507 L 80.784 118.621674 L 84.132 119.653116 L 87.48 119.975442 L 90.828 125.068186 L 94.176 132.417209 L 97.524 129.967535 L 100.872 133.255256 L 104.22 168.71107 L 107.568 163.940651 L 110.916 162.329023 L 114.264 162.780279 L 117.612 164.520837 L 120.96 164.520837 L 124.308 167.034977 L 127.656 162.522419 L 131.004 162.522419 L 134.352 201.137023 L 137.7 197.720372 L 141.048 197.591442 L 144.396 194.561581 L 147.744 195.464093 L 151.092 196.173209 L 154.44 202.490791 L 157.788 193.53014 L 161.136 195.979814 L 164.484 183.731442 L 167.832 188.244 L 171.18 186.374512 L 174.528 184.505023 L 177.876 223.893209 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_50">    <path d="M 54 127.26 L 57.348 120.426698 L 60.696 124.552465 L 64.044 126.035163 L 67.392 113.335535 L 70.74 104.052558 L 74.088 164.005116 L 77.436 179.734605 L 80.784 182.248744 L 84.132 183.602512 L 87.48 164.520837 L 90.828 159.814884 L 94.176 182.7 L 97.524 182.248744 L 100.872 182.377674 L 104.22 185.21414 L 107.568 184.311628 L 110.916 189.146512 L 114.264 186.890233 L 117.612 190.242419 L 120.96 186.374512 L 124.308 194.110326 L 127.656 195.593023 L 131.004 191.854047 L 134.352 201.97507 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_51">    <path d="M 54 127.26 L 57.348 125.777302 L 60.696 122.489581 L 64.044 127.26 L 67.392 143.698605 L 70.74 140.281953 L 74.088 144.730047 L 77.436 145.954884 L 80.784 147.308651 L 84.132 159.492558 L 87.48 166.583721 L 90.828 174.835256 L 94.176 184.376093 L 97.524 154.72214 L 100.872 156.011442 L 104.22 153.690698 L 107.568 163.876186 L 110.916 179.79907 L 114.264 176.962605 L 117.612 182.184279 L 120.96 172.772372 L 124.308 177.478326 L 127.656 173.739349 L 131.004 173.481488 L 134.352 180.572651 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_52">    <path d="M 54 127.26 L 57.348 124.294605 L 60.696 125.777302 L 64.044 124.874791 L 67.392 125.003721 L 70.74 125.390512 L 74.088 125.648372 L 77.436 125.648372 L 80.784 123.778884 L 84.132 122.554047 L 87.48 126.421953 L 90.828 119.717581 L 94.176 126.679814 L 97.524 126.937674 L 100.872 129.065023 L 104.22 146.470605 L 107.568 144.14986 L 110.916 164.520837 L 114.264 172.127721 L 117.612 163.489395 L 120.96 165.165488 L 124.308 174.706326 L 127.656 191.14493 L 131.004 191.14493 L 134.352 190.629209 L 137.7 184.762884 L 141.048 190.629209 L 144.396 195.270698 L 147.744 200.943628 L 151.092 203.264372 L 154.44 201.717209 L 157.788 191.531721 L 161.136 201.523814 L 164.484 212.611814 L 167.832 209.453023 L 171.18 206.487628 L 174.528 204.876 L 177.876 199.20307 L 181.224 203.457767 L 184.572 205.198326 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_53">    <path d="M 54 127.26 L 57.348 124.874791 L 60.696 123.972279 L 64.044 125.068186 L 67.392 142.151442 L 70.74 140.410884 L 74.088 148.920279 L 77.436 171.225209 L 80.784 189.662233 L 84.132 182.506605 L 87.48 223.957674 L 90.828 223.957674 L 94.176 223.957674 L 97.524 223.957674 L 100.872 223.957674 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_54">    <path d="M 54 127.26 L 57.348 126.744279 L 60.696 125.906233 L 64.044 125.970698 L 67.392 127.646791 L 70.74 124.165674 L 74.088 144.794512 L 77.436 143.053953 L 80.784 139.057116 L 84.132 139.250512 L 87.48 148.404558 L 90.828 159.492558 L 94.176 179.734605 L 97.524 173.997209 L 100.872 164.198512 L 104.22 180.121395 L 107.568 213.965581 L 110.916 222.926233 L 114.264 227.245395 L 117.612 231.629023 L 120.96 227.052 L 124.308 225.053581 L 127.656 229.69507 L 131.004 223.893209 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_55">    <path d="M 54 127.26 L 57.348 124.874791 L 60.696 123.134233 L 64.044 124.488 L 67.392 127.453395 L 70.74 166.906047 L 74.088 176.060093 L 77.436 175.093116 L 80.784 149.822791 L 84.132 179.67014 L 87.48 175.028651 L 90.828 168.646605 L 94.176 178.058512 L 97.524 177.220465 L 100.872 175.608837 L 104.22 179.154419 L 107.568 185.794326 L 110.916 197.011256 L 114.264 188.437395 L 117.612 185.21414 L 120.96 190.629209 L 124.308 195.141767 L 127.656 198.880744 L 131.004 199.912186 L 134.352 197.913767 L 137.7 194.754977 L 141.048 197.269116 L 144.396 195.850884 L 147.744 201.201488 L 151.092 201.459349 L 154.44 202.877581 L 157.788 187.341488 L 161.136 186.567907 L 164.484 202.490791 L 167.832 218.091349 L 171.18 217.188837 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_56">    <path d="M 54 127.26 L 57.348 127.00214 L 60.696 123.972279 L 64.044 125.519442 L 67.392 120.297767 L 70.74 121.006884 L 74.088 118.428279 L 77.436 121.97386 L 80.784 124.74586 L 84.132 124.35907 L 87.48 125.003721 L 90.828 120.942419 L 94.176 123.263163 L 97.524 115.527349 L 100.872 123.714419 L 104.22 129.129488 L 107.568 164.262977 L 110.916 158.847907 L 114.264 159.685953 L 117.612 168.130884 L 120.96 172.514512 L 124.308 172.772372 L 127.656 176.446884 L 131.004 164.134047 L 134.352 173.868279 L 137.7 177.865116 L 141.048 173.030233 L 144.396 176.640279 L 147.744 171.418605 L 151.092 168.646605 L 154.44 173.674884 L 157.788 169.162326 L 161.136 171.934326 L 164.484 175.028651 L 167.832 178.574233 L 171.18 171.676465 L 174.528 162.200093 L 177.876 174.899721 L 181.224 180.701581 L 184.572 176.382419 L 187.92 183.860372 L 191.268 186.374512 L 194.616 223.893209 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_57">    <path d="M 54 127.26 L 57.348 124.294605 L 60.696 124.874791 L 64.044 119.782047 L 67.392 123.198698 L 70.74 120.039907 L 74.088 120.620093 L 77.436 124.294605 L 80.784 123.263163 L 84.132 161.168651 L 87.48 162.006698 L 90.828 163.618326 L 94.176 168.646605 L 97.524 165.036558 L 100.872 172.385581 L 104.22 179.476744 L 107.568 181.023907 L 110.916 186.310047 L 114.264 185.149674 L 117.612 176.640279 L 120.96 176.833674 L 124.308 183.409116 L 127.656 191.080465 L 131.004 173.997209 L 134.352 184.182698 L 137.7 187.663814 L 141.048 187.98614 L 144.396 179.79907 L 147.744 182.893395 L 151.092 180.508186 L 154.44 199.20307 L 157.788 187.470419 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_58">    <path d="M 54 127.26 L 57.348 120.813488 L 60.696 119.846512 L 64.044 118.299349 L 67.392 122.811907 L 70.74 118.750605 L 74.088 143.63414 L 77.436 159.428093 L 80.784 176.962605 L 84.132 169.742512 L 87.48 178.638698 L 90.828 179.734605 L 94.176 195.464093 L 97.524 196.044279 L 100.872 223.957674 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_59">    <path d="M 54 127.26 L 57.348 121.45814 L 60.696 115.785209 L 64.044 120.426698 L 67.392 123.714419 L 70.74 123.263163 L 74.088 143.118419 L 77.436 145.890419 L 80.784 146.53507 L 84.132 144.536651 L 87.48 145.890419 L 90.828 156.204837 L 94.176 154.206419 L 97.524 158.267721 L 100.872 160.330605 L 104.22 157.558605 L 107.568 157.558605 L 110.916 165.358884 L 114.264 148.662419 L 117.612 178.896558 L 120.96 178.251907 L 124.308 179.992465 L 127.656 165.358884 L 131.004 163.231535 L 134.352 165.101023 L 137.7 166.390326 L 141.048 173.159163 L 144.396 166.003535 L 147.744 188.50186 L 151.092 194.239256 L 154.44 193.272279 L 157.788 191.531721 L 161.136 186.245581 L 164.484 223.893209 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_60">    <path d="M 54 127.26 L 57.348 113.851256 L 60.696 115.011628 L 64.044 117.396837 L 67.392 148.662419 L 70.74 148.855814 L 74.088 149.758326 L 77.436 155.044465 L 80.784 151.369953 L 84.132 155.431256 L 87.48 155.560186 L 90.828 154.915535 L 94.176 153.303907 L 97.524 179.412279 L 100.872 166.390326 L 104.22 163.42493 L 107.568 159.685953 L 110.916 164.134047 L 114.264 163.03814 L 117.612 169.226791 L 120.96 176.446884 L 124.308 170.193767 L 127.656 167.486233 L 131.004 174.899721 L 134.352 174.12614 L 137.7 179.67014 L 141.048 223.828744 L 144.396 223.893209 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_61">    <path d="M 54 127.26 L 57.348 123.843349 L 60.696 125.777302 L 64.044 124.74586 L 67.392 123.778884 L 70.74 125.326047 L 74.088 123.907814 L 77.436 123.907814 L 80.784 120.297767 L 84.132 145.245767 L 87.48 152.07907 L 90.828 154.270884 L 94.176 149.436 L 97.524 157.236279 L 100.872 151.627814 L 104.22 154.335349 L 107.568 151.821209 L 110.916 154.98 L 114.264 172.321116 L 117.612 178.251907 L 120.96 192.305302 L 124.308 194.239256 L 127.656 191.27386 L 131.004 194.561581 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_62">    <path d="M 54 127.26 L 57.348 121.716 L 60.696 120.039907 L 64.044 120.233302 L 67.392 150.983163 L 70.74 160.330605 L 74.088 160.524 L 77.436 156.591628 L 80.784 166.648186 L 84.132 165.616744 L 87.48 167.808558 L 90.828 175.350977 L 94.176 176.833674 L 97.524 170.773953 L 100.872 178.316372 L 104.22 180.959442 L 107.568 177.929581 L 110.916 175.479907 L 114.264 190.306884 L 117.612 181.604093 L 120.96 184.698419 L 124.308 183.989302 L 127.656 189.920093 L 131.004 178.316372 L 134.352 181.926419 L 137.7 186.310047 L 141.048 192.434233 L 144.396 191.467256 L 147.744 185.60093 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_63">    <path d="M 54 127.26 L 57.348 118.68614 L 60.696 112.626419 L 64.044 118.944 L 67.392 107.211349 L 70.74 123.392093 L 74.088 134.609023 L 77.436 129.065023 L 80.784 130.870047 L 84.132 134.286698 L 87.48 144.472186 L 90.828 148.017767 L 94.176 148.404558 L 97.524 151.047628 L 100.872 167.163907 L 104.22 157.236279 L 107.568 170.451628 L 110.916 159.170233 L 114.264 158.847907 L 117.612 174.061674 L 120.96 170.709488 L 124.308 172.965767 L 127.656 171.35414 L 131.004 168.775535 L 134.352 179.218884 L 137.7 179.992465 L 141.048 186.245581 L 144.396 190.435814 L 147.744 178.574233 L 151.092 195.077302 L 154.44 182.248744 L 157.788 192.756558 L 161.136 198.236093 L 164.484 200.041116 L 167.832 201.072558 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #7f7f7f; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_64">    <path d="M 54 127.26 L 57.348 123.843349 L 60.696 123.456558 L 64.044 123.102 L 67.392 123.198698 L 70.74 124.165674 L 74.088 125.648372 L 77.436 127.098837 L 80.784 131.740326 L 84.132 138.541395 L 87.48 145.181302 L 90.828 155.495721 L 94.176 160.072744 L 97.524 164.488605 L 100.872 167.002744 L 104.22 171.612 L 107.568 175.995628 L 110.916 181.249535 L 114.264 185.21414 L 117.612 183.731442 L 120.96 187.277023 L 124.308 190.435814 L 127.656 192.079674 L 131.004 194.464884 L 134.352 198.010465 L 137.7 206.423163 L 141.048 206.423163 L 144.396 223.860977 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p770b955d50)" style="fill: none; stroke: #000000; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="patch_3">    <path d="M 54 224.28 L 54 30.24 " style="fill: none"/>   </g>   <g id="patch_4">    <path d="M 388.8 224.28 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="patch_5">    <path d="M 54 224.28 L 388.8 224.28 " style="fill: none"/>   </g>   <g id="patch_6">    <path d="M 54 30.24 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="text_16">    <g id="patch_7">     <a xlink:href="https://dynomight.net/chess">      <path d="M 323.8664 225.676606 L 388.881244 225.676606 L 388.881244 213.343012 L 323.8664 213.343012 z" style="fill: #ffffff; opacity: 0.000001; stroke: #ffffff; stroke-linejoin: miter"/>     </a>    </g>    <a xlink:href="https://dynomight.net/chess">     <!-- dynomight.net/chess -->     <g style="opacity: 0.3" transform="translate(327.8664 220.734419) scale(0.05 -0.05)">      <defs>       <path id="CourierNewPSMT-64" d="M 3206 3922 L 3206 263 L 3556 263 Q 3653 263 3693 227 Q 3734 191 3734 131 Q 3734 75 3693 37 Q 3653 0 3556 0 L 2941 0 L 2941 522 Q 2484 -103 1781 -103 Q 1425 -103 1098 86 Q 772 275 583 625 Q 394 975 394 1347 Q 394 1722 583 2070 Q 772 2419 1098 2609 Q 1425 2800 1784 2800 Q 2472 2800 2941 2175 L 2941 3656 L 2591 3656 Q 2494 3656 2453 3693 Q 2413 3731 2413 3791 Q 2413 3847 2453 3884 Q 2494 3922 2591 3922 L 3206 3922 zM 2941 1347 Q 2941 1847 2603 2190 Q 2266 2534 1800 2534 Q 1331 2534 993 2190 Q 656 1847 656 1347 Q 656 850 993 504 Q 1331 159 1800 159 Q 2266 159 2603 504 Q 2941 850 2941 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-79" d="M 1938 0 L 719 2441 L 641 2441 Q 547 2441 506 2478 Q 466 2516 466 2572 Q 466 2613 486 2645 Q 506 2678 539 2692 Q 572 2706 641 2706 L 1359 2706 Q 1453 2706 1493 2668 Q 1534 2631 1534 2572 Q 1534 2516 1493 2478 Q 1453 2441 1359 2441 L 1006 2441 L 2078 288 L 3134 2441 L 2781 2441 Q 2688 2441 2647 2478 Q 2606 2516 2606 2575 Q 2606 2631 2647 2668 Q 2688 2706 2781 2706 L 3497 2706 Q 3594 2706 3634 2668 Q 3675 2631 3675 2572 Q 3675 2531 3650 2497 Q 3625 2463 3594 2452 Q 3563 2441 3419 2441 L 1756 -944 L 2166 -944 Q 2259 -944 2300 -980 Q 2341 -1016 2341 -1075 Q 2341 -1131 2300 -1168 Q 2259 -1206 2166 -1206 L 653 -1206 Q 559 -1206 518 -1170 Q 478 -1134 478 -1075 Q 478 -1016 518 -980 Q 559 -944 653 -944 L 1475 -944 L 1938 0 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6e" d="M 1072 2706 L 1072 2309 Q 1347 2588 1569 2694 Q 1791 2800 2069 2800 Q 2369 2800 2616 2672 Q 2791 2578 2933 2361 Q 3075 2144 3075 1916 L 3075 263 L 3297 263 Q 3391 263 3431 227 Q 3472 191 3472 131 Q 3472 75 3431 37 Q 3391 0 3297 0 L 2594 0 Q 2497 0 2456 37 Q 2416 75 2416 131 Q 2416 191 2456 227 Q 2497 263 2594 263 L 2813 263 L 2813 1872 Q 2813 2150 2609 2342 Q 2406 2534 2066 2534 Q 1806 2534 1615 2429 Q 1425 2325 1072 1909 L 1072 263 L 1369 263 Q 1463 263 1503 227 Q 1544 191 1544 131 Q 1544 75 1503 37 Q 1463 0 1369 0 L 513 0 Q 419 0 378 37 Q 338 75 338 131 Q 338 191 378 227 Q 419 263 513 263 L 809 263 L 809 2441 L 588 2441 Q 494 2441 453 2478 Q 413 2516 413 2575 Q 413 2631 453 2668 Q 494 2706 588 2706 L 1072 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6f" d="M 3388 1347 Q 3388 747 2958 322 Q 2528 -103 1922 -103 Q 1309 -103 881 323 Q 453 750 453 1347 Q 453 1947 881 2373 Q 1309 2800 1922 2800 Q 2528 2800 2958 2375 Q 3388 1950 3388 1347 zM 3122 1347 Q 3122 1841 2770 2187 Q 2419 2534 1919 2534 Q 1419 2534 1067 2186 Q 716 1838 716 1347 Q 716 859 1067 509 Q 1419 159 1919 159 Q 2419 159 2770 507 Q 3122 856 3122 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6d" d="M 725 2706 L 725 2441 Q 1059 2800 1397 2800 Q 1600 2800 1753 2692 Q 1906 2584 2009 2366 Q 2184 2584 2364 2692 Q 2544 2800 2725 2800 Q 3009 2800 3178 2616 Q 3400 2378 3400 2097 L 3400 263 L 3622 263 Q 3716 263 3756 227 Q 3797 191 3797 131 Q 3797 75 3756 37 Q 3716 0 3622 0 L 3138 0 L 3138 2072 Q 3138 2272 3016 2403 Q 2894 2534 2734 2534 Q 2591 2534 2431 2426 Q 2272 2319 2069 2003 L 2069 263 L 2288 263 Q 2381 263 2422 227 Q 2463 191 2463 131 Q 2463 75 2422 37 Q 2381 0 2288 0 L 1803 0 L 1803 2053 Q 1803 2263 1679 2398 Q 1556 2534 1403 2534 Q 1263 2534 1125 2444 Q 934 2316 725 2003 L 725 263 L 947 263 Q 1041 263 1081 227 Q 1122 191 1122 131 Q 1122 75 1081 37 Q 1041 0 947 0 L 241 0 Q 147 0 106 37 Q 66 75 66 131 Q 66 191 106 227 Q 147 263 241 263 L 463 263 L 463 2441 L 241 2441 Q 147 2441 106 2478 Q 66 2516 66 2575 Q 66 2631 106 2668 Q 147 2706 241 2706 L 725 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-69" d="M 2050 4053 L 2050 3375 L 1663 3375 L 1663 4053 L 2050 4053 zM 2059 2706 L 2059 263 L 3091 263 Q 3188 263 3228 227 Q 3269 191 3269 131 Q 3269 75 3228 37 Q 3188 0 3091 0 L 766 0 Q 672 0 631 37 Q 591 75 591 131 Q 591 191 631 227 Q 672 263 766 263 L 1797 263 L 1797 2441 L 1031 2441 Q 938 2441 895 2478 Q 853 2516 853 2572 Q 853 2631 893 2668 Q 934 2706 1031 2706 L 2059 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-67" d="M 2803 2225 L 2803 2706 L 3419 2706 Q 3513 2706 3553 2668 Q 3594 2631 3594 2572 Q 3594 2516 3553 2478 Q 3513 2441 3419 2441 L 3066 2441 L 3066 -181 Q 3066 -444 2953 -650 Q 2878 -788 2703 -934 Q 2528 -1081 2384 -1143 Q 2241 -1206 2000 -1206 L 1256 -1206 Q 1163 -1206 1122 -1170 Q 1081 -1134 1081 -1075 Q 1081 -1016 1122 -978 Q 1163 -941 1256 -941 L 2009 -944 Q 2241 -944 2427 -825 Q 2613 -706 2734 -478 Q 2803 -347 2803 -144 L 2803 644 Q 2381 66 1716 66 Q 1175 66 783 467 Q 391 869 391 1434 Q 391 2000 783 2400 Q 1175 2800 1716 2800 Q 2381 2800 2803 2225 zM 2803 1434 Q 2803 1897 2486 2215 Q 2169 2534 1728 2534 Q 1288 2534 972 2214 Q 656 1894 656 1434 Q 656 972 972 651 Q 1288 331 1728 331 Q 2169 331 2486 651 Q 2803 972 2803 1434 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-68" d="M 1056 3922 L 1056 2306 Q 1306 2578 1536 2689 Q 1766 2800 2050 2800 Q 2356 2800 2570 2692 Q 2784 2584 2928 2361 Q 3072 2138 3072 1891 L 3072 263 L 3366 263 Q 3463 263 3502 227 Q 3541 191 3541 131 Q 3541 75 3502 37 Q 3463 0 3366 0 L 2513 0 Q 2416 0 2375 37 Q 2334 75 2334 131 Q 2334 191 2375 227 Q 2416 263 2513 263 L 2806 263 L 2806 1872 Q 2806 2156 2601 2347 Q 2397 2538 2031 2538 Q 1744 2538 1541 2397 Q 1394 2297 1056 1925 L 1056 263 L 1353 263 Q 1447 263 1487 227 Q 1528 191 1528 131 Q 1528 75 1487 37 Q 1447 0 1353 0 L 497 0 Q 403 0 362 37 Q 322 75 322 131 Q 322 191 362 227 Q 403 263 497 263 L 794 263 L 794 3656 L 441 3656 Q 347 3656 306 3693 Q 266 3731 266 3791 Q 266 3847 306 3884 Q 347 3922 441 3922 L 1056 3922 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-74" d="M 1381 2706 L 2806 2706 Q 2900 2706 2940 2668 Q 2981 2631 2981 2572 Q 2981 2516 2940 2478 Q 2900 2441 2806 2441 L 1381 2441 L 1381 694 Q 1381 466 1564 312 Q 1747 159 2100 159 Q 2366 159 2675 239 Q 2984 319 3156 419 Q 3219 459 3259 459 Q 3309 459 3346 420 Q 3384 381 3384 328 Q 3384 281 3344 241 Q 3244 138 2858 17 Q 2472 -103 2119 -103 Q 1659 -103 1387 112 Q 1116 328 1116 694 L 1116 2441 L 631 2441 Q 538 2441 497 2478 Q 456 2516 456 2575 Q 456 2631 497 2668 Q 538 2706 631 2706 L 1116 2706 L 1116 3481 Q 1116 3575 1153 3615 Q 1191 3656 1247 3656 Q 1306 3656 1343 3615 Q 1381 3575 1381 3481 L 1381 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2e" d="M 1866 753 L 1978 753 Q 2153 753 2278 629 Q 2403 506 2403 331 Q 2403 150 2276 28 Q 2150 -94 1978 -94 L 1866 -94 Q 1691 -94 1566 29 Q 1441 153 1441 328 Q 1441 509 1567 631 Q 1694 753 1866 753 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-65" d="M 3341 1291 L 666 1291 Q 734 781 1092 470 Q 1450 159 1978 159 Q 2272 159 2594 256 Q 2916 353 3119 513 Q 3178 559 3222 559 Q 3272 559 3309 520 Q 3347 481 3347 428 Q 3347 375 3297 325 Q 3147 169 2764 33 Q 2381 -103 1978 -103 Q 1303 -103 851 339 Q 400 781 400 1409 Q 400 1981 823 2390 Q 1247 2800 1872 2800 Q 2516 2800 2931 2379 Q 3347 1959 3341 1291 zM 3075 1556 Q 2997 1991 2664 2262 Q 2331 2534 1872 2534 Q 1413 2534 1081 2265 Q 750 1997 669 1556 L 3075 1556 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2f" d="M 3100 4113 L 988 -413 Q 934 -528 850 -528 Q 797 -528 758 -489 Q 719 -450 719 -403 Q 719 -366 750 -300 L 2859 4225 Q 2894 4297 2925 4319 Q 2956 4341 3000 4341 Q 3053 4341 3092 4302 Q 3131 4263 3131 4216 Q 3131 4181 3100 4113 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-63" d="M 3016 2441 L 3016 2528 Q 3016 2625 3053 2665 Q 3091 2706 3147 2706 Q 3206 2706 3243 2665 Q 3281 2625 3281 2528 L 3281 1934 Q 3278 1838 3242 1797 Q 3206 1756 3147 1756 Q 3094 1756 3058 1792 Q 3022 1828 3016 1913 Q 2997 2134 2723 2334 Q 2450 2534 1988 2534 Q 1403 2534 1100 2168 Q 797 1803 797 1331 Q 797 822 1131 490 Q 1466 159 1997 159 Q 2303 159 2620 271 Q 2938 384 3194 634 Q 3259 697 3309 697 Q 3363 697 3398 661 Q 3434 625 3434 572 Q 3434 438 3119 231 Q 2609 -103 1984 -103 Q 1350 -103 942 301 Q 534 706 534 1328 Q 534 1963 951 2381 Q 1369 2800 2003 2800 Q 2606 2800 3016 2441 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-73" d="M 2788 2534 Q 2788 2625 2825 2665 Q 2863 2706 2919 2706 Q 2978 2706 3015 2665 Q 3053 2625 3053 2528 L 3053 2078 Q 3053 1984 3015 1943 Q 2978 1903 2919 1903 Q 2866 1903 2830 1937 Q 2794 1972 2788 2050 Q 2769 2238 2594 2359 Q 2338 2534 1916 2534 Q 1475 2534 1231 2356 Q 1047 2222 1047 2056 Q 1047 1869 1266 1744 Q 1416 1656 1834 1609 Q 2381 1550 2594 1475 Q 2897 1366 3045 1172 Q 3194 978 3194 753 Q 3194 419 2872 158 Q 2550 -103 1928 -103 Q 1306 -103 909 213 Q 909 106 896 75 Q 884 44 851 22 Q 819 0 778 0 Q 722 0 684 40 Q 647 81 647 175 L 647 716 Q 647 809 683 850 Q 719 891 778 891 Q 834 891 873 852 Q 913 813 913 747 Q 913 603 984 506 Q 1094 356 1333 257 Q 1572 159 1919 159 Q 2431 159 2681 350 Q 2931 541 2931 753 Q 2931 997 2678 1144 Q 2422 1291 1933 1341 Q 1444 1391 1231 1472 Q 1019 1553 900 1715 Q 781 1878 781 2066 Q 781 2403 1112 2601 Q 1444 2800 1903 2800 Q 2447 2800 2788 2534 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#CourierNewPSMT-64"/>      <use xlink:href="#CourierNewPSMT-79" x="60.009766"/>      <use xlink:href="#CourierNewPSMT-6e" x="120.019531"/>      <use xlink:href="#CourierNewPSMT-6f" x="180.029297"/>      <use xlink:href="#CourierNewPSMT-6d" x="240.039062"/>      <use xlink:href="#CourierNewPSMT-69" x="300.048828"/>      <use xlink:href="#CourierNewPSMT-67" x="360.058594"/>      <use xlink:href="#CourierNewPSMT-68" x="420.068359"/>      <use xlink:href="#CourierNewPSMT-74" x="480.078125"/>      <use xlink:href="#CourierNewPSMT-2e" x="540.087891"/>      <use xlink:href="#CourierNewPSMT-6e" x="600.097656"/>      <use xlink:href="#CourierNewPSMT-65" x="660.107422"/>      <use xlink:href="#CourierNewPSMT-74" x="720.117188"/>      <use xlink:href="#CourierNewPSMT-2f" x="780.126953"/>      <use xlink:href="#CourierNewPSMT-63" x="840.136719"/>      <use xlink:href="#CourierNewPSMT-68" x="900.146484"/>      <use xlink:href="#CourierNewPSMT-65" x="960.15625"/>      <use xlink:href="#CourierNewPSMT-73" x="1020.166016"/>      <use xlink:href="#CourierNewPSMT-73" x="1080.175781"/>     </g>    </a>   </g>   <g id="text_17">    <!-- llama-3.2-3b vs stockfish -->    <g transform="translate(148.532813 24.24) scale(0.12 -0.12)">     <defs>      <path id="Montserrat-Regular-6c" d="M 634 4749 L 1088 4749 L 1088 0 L 634 0 L 634 4749 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6d" d="M 4832 3392 Q 5466 3392 5827 3027 Q 6189 2662 6189 1952 L 6189 0 L 5734 0 L 5734 1907 Q 5734 2432 5481 2707 Q 5229 2982 4768 2982 Q 4243 2982 3942 2659 Q 3642 2336 3642 1766 L 3642 0 L 3187 0 L 3187 1907 Q 3187 2432 2934 2707 Q 2682 2982 2214 2982 Q 1696 2982 1392 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2752 Q 1248 3059 1568 3225 Q 1888 3392 2304 3392 Q 2726 3392 3036 3213 Q 3347 3034 3501 2682 Q 3686 3014 4035 3203 Q 4384 3392 4832 3392 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-33" d="M 1856 2560 Q 2560 2522 2928 2179 Q 3296 1837 3296 1286 Q 3296 902 3110 604 Q 2925 307 2560 134 Q 2195 -38 1670 -38 Q 1184 -38 752 118 Q 320 275 58 538 L 282 902 Q 506 672 877 528 Q 1248 384 1670 384 Q 2221 384 2521 624 Q 2822 864 2822 1286 Q 2822 1702 2521 1939 Q 2221 2176 1613 2176 L 1293 2176 L 1293 2515 L 2534 4070 L 243 4070 L 243 4480 L 3123 4480 L 3123 4154 L 1856 2560 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-2e" d="M 678 -32 Q 538 -32 438 70 Q 339 173 339 320 Q 339 467 438 566 Q 538 666 678 666 Q 819 666 921 566 Q 1024 467 1024 320 Q 1024 173 921 70 Q 819 -32 678 -32 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-62" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1184 3827 793 Q 3610 403 3226 185 Q 2842 -32 2368 -32 Q 1946 -32 1610 147 Q 1274 326 1069 666 L 1069 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2726 Q 1299 3053 1628 3222 Q 1958 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2368 Q 3264 2669 2976 2832 Q 2688 2995 2336 2995 Q 1978 2995 1693 2832 Q 1408 2669 1245 2368 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-76" d="M 3462 3366 L 1965 0 L 1498 0 L 6 3366 L 480 3366 L 1741 499 L 3014 3366 L 3462 3366 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6f" d="M 2010 -32 Q 1523 -32 1132 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1132 3174 Q 1523 3392 2010 3392 Q 2496 3392 2886 3174 Q 3277 2957 3497 2566 Q 3718 2176 3718 1683 Q 3718 1190 3497 800 Q 3277 410 2886 189 Q 2496 -32 2010 -32 zM 2010 371 Q 2368 371 2653 534 Q 2938 698 3098 998 Q 3258 1299 3258 1683 Q 3258 2067 3098 2368 Q 2938 2669 2653 2832 Q 2368 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6b" d="M 1882 1690 L 1088 960 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 1530 L 3098 3366 L 3661 3366 L 2221 1990 L 3795 0 L 3238 0 L 1882 1690 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-68" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2778 Q 1274 3072 1600 3232 Q 1926 3392 2349 3392 z" transform="scale(0.015625)"/>     </defs>     <use xlink:href="#Montserrat-Regular-6c"/>     <use xlink:href="#Montserrat-Regular-6c" x="26.899994"/>     <use xlink:href="#Montserrat-Regular-61" x="53.799988"/>     <use xlink:href="#Montserrat-Regular-6d" x="112.799973"/>     <use xlink:href="#Montserrat-Regular-61" x="218.899963"/>     <use xlink:href="#Montserrat-Regular-2d" x="277.899948"/>     <use xlink:href="#Montserrat-Regular-33" x="316.099945"/>     <use xlink:href="#Montserrat-Regular-2e" x="372.499939"/>     <use xlink:href="#Montserrat-Regular-32" x="393.699936"/>     <use xlink:href="#Montserrat-Regular-2d" x="450.499924"/>     <use xlink:href="#Montserrat-Regular-33" x="488.699921"/>     <use xlink:href="#Montserrat-Regular-62" x="545.099915"/>     <use xlink:href="#Montserrat-Regular-20" x="612.899902"/>     <use xlink:href="#Montserrat-Regular-76" x="639.099899"/>     <use xlink:href="#Montserrat-Regular-73" x="693.299896"/>     <use xlink:href="#Montserrat-Regular-20" x="742.19989"/>     <use xlink:href="#Montserrat-Regular-73" x="768.399887"/>     <use xlink:href="#Montserrat-Regular-74" x="817.299881"/>     <use xlink:href="#Montserrat-Regular-6f" x="857.899872"/>     <use xlink:href="#Montserrat-Regular-63" x="920.599869"/>     <use xlink:href="#Montserrat-Regular-6b" x="976.899857"/>     <use xlink:href="#Montserrat-Regular-66" x="1036.999847"/>     <use xlink:href="#Montserrat-Regular-69" x="1070.899841"/>     <use xlink:href="#Montserrat-Regular-73" x="1097.799835"/>     <use xlink:href="#Montserrat-Regular-68" x="1146.699829"/>    </g>   </g>  </g> </g> <defs>  <clipPath id="p770b955d50">   <rect x="54" y="30.24" width="334.8" height="194.04"/>  </clipPath> </defs></svg>](https://dynomight.net/img/chess/llama-3.2-3b_stockfish.pdf)

[Terrible](https://lichess.org/AAsQ2Ydi). (Click to see a game. Click the above image to zoom in.)

In the above figure, there’s one light line for each game, and the black line shows the per-turn median. The LLM can play standard openings for a few moves but then quickly starts throwing away pieces. It lost every single game, even though Stockfish was on the lowest setting.

Maybe that model is too small? So I got [`llama-3.1-70b`](https://huggingface.co/meta-llama/Llama-3.1-70B), which is a similar model but with 70 billion parameters instead of 3 billion. The results were:

[<?xml version="1.0" encoding="utf-8" standalone="no"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"  "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg xmlns:xlink="http://www.w3.org/1999/xlink" width="432pt" height="252pt" viewBox="0 0 432 252" xmlns="http://www.w3.org/2000/svg" version="1.1"> <metadata>  <rdf:RDF xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:cc="http://creativecommons.org/ns#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">   <cc:Work>    <dc:type rdf:resource="http://purl.org/dc/dcmitype/StillImage"/>    <dc:date>2024-11-14T11:33:18.374206</dc:date>    <dc:format>image/svg+xml</dc:format>    <dc:creator>     <cc:Agent>      <dc:title>Matplotlib v3.8.4, https://matplotlib.org/</dc:title>     </cc:Agent>    </dc:creator>   </cc:Work>  </rdf:RDF> </metadata> <defs>  <style type="text/css">*{stroke-linejoin: round; stroke-linecap: butt}</style> </defs> <g id="figure_1">  <g id="patch_1">   <path d="M 0 252 L 432 252 L 432 0 L 0 0 z" style="fill: #ffffff"/>  </g>  <g id="axes_1">   <g id="patch_2">    <path d="M 54 224.28 L 388.8 224.28 L 388.8 30.24 L 54 30.24 z" style="fill: #ffffff"/>   </g>   <g id="matplotlib.axis_1">    <g id="xtick_1">     <g id="line2d_1"/>     <g id="text_1">      <!-- 0 -->      <g transform="translate(50.689844 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-30" d="M 2118 -38 Q 1600 -38 1193 237 Q 787 512 560 1027 Q 333 1542 333 2240 Q 333 2938 560 3453 Q 787 3968 1193 4243 Q 1600 4518 2118 4518 Q 2637 4518 3043 4243 Q 3450 3968 3680 3453 Q 3910 2938 3910 2240 Q 3910 1542 3680 1027 Q 3450 512 3043 237 Q 2637 -38 2118 -38 zM 2118 384 Q 2509 384 2806 601 Q 3104 819 3270 1235 Q 3437 1651 3437 2240 Q 3437 2829 3270 3245 Q 3104 3661 2806 3878 Q 2509 4096 2118 4096 Q 1728 4096 1430 3878 Q 1133 3661 966 3245 Q 800 2829 800 2240 Q 800 1651 966 1235 Q 1133 819 1430 601 Q 1728 384 2118 384 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="xtick_2">     <g id="line2d_2"/>     <g id="text_2">      <!-- 20 -->      <g transform="translate(114.81 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-32" d="M 3392 410 L 3392 0 L 250 0 L 250 326 L 2112 2150 Q 2464 2496 2589 2749 Q 2714 3002 2714 3258 Q 2714 3654 2442 3875 Q 2170 4096 1664 4096 Q 877 4096 442 3597 L 115 3878 Q 378 4186 784 4352 Q 1190 4518 1702 4518 Q 2387 4518 2787 4195 Q 3187 3872 3187 3309 Q 3187 2963 3033 2643 Q 2880 2323 2451 1907 L 928 410 L 3392 410 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-32"/>       <use xlink:href="#Montserrat-Regular-30" x="56.799988"/>      </g>     </g>    </g>    <g id="xtick_3">     <g id="line2d_3"/>     <g id="text_3">      <!-- 40 -->      <g transform="translate(181.305156 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-34" d="M 4141 1178 L 3245 1178 L 3245 0 L 2784 0 L 2784 1178 L 256 1178 L 256 1510 L 2630 4480 L 3142 4480 L 851 1587 L 2797 1587 L 2797 2624 L 3245 2624 L 3245 1587 L 4141 1587 L 4141 1178 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-34"/>       <use xlink:href="#Montserrat-Regular-30" x="66.099991"/>      </g>     </g>    </g>    <g id="xtick_4">     <g id="line2d_4"/>     <g id="text_4">      <!-- 60 -->      <g transform="translate(248.524531 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-36" d="M 2189 2682 Q 2630 2682 2972 2515 Q 3315 2349 3507 2045 Q 3699 1741 3699 1338 Q 3699 928 3500 614 Q 3302 301 2953 131 Q 2605 -38 2170 -38 Q 1280 -38 806 547 Q 333 1133 333 2202 Q 333 2950 589 3468 Q 845 3987 1305 4252 Q 1766 4518 2381 4518 Q 3046 4518 3443 4275 L 3258 3904 Q 2938 4115 2387 4115 Q 1645 4115 1222 3644 Q 800 3174 800 2278 Q 800 2118 819 1914 Q 986 2278 1350 2480 Q 1715 2682 2189 2682 zM 2150 352 Q 2637 352 2941 617 Q 3245 883 3245 1325 Q 3245 1766 2941 2028 Q 2637 2291 2118 2291 Q 1779 2291 1520 2163 Q 1261 2035 1113 1811 Q 966 1587 966 1312 Q 966 1056 1104 838 Q 1242 621 1510 486 Q 1779 352 2150 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-36"/>       <use xlink:href="#Montserrat-Regular-30" x="60.899994"/>      </g>     </g>    </g>    <g id="xtick_5">     <g id="line2d_5"/>     <g id="text_5">      <!-- 80 -->      <g transform="translate(315.34 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-38" d="M 2899 2349 Q 3322 2208 3549 1926 Q 3776 1645 3776 1248 Q 3776 858 3565 566 Q 3354 275 2960 118 Q 2566 -38 2035 -38 Q 1229 -38 768 307 Q 307 653 307 1248 Q 307 1651 528 1929 Q 749 2208 1171 2349 Q 826 2483 646 2736 Q 467 2989 467 3334 Q 467 3878 892 4198 Q 1318 4518 2035 4518 Q 2509 4518 2864 4374 Q 3219 4230 3417 3961 Q 3616 3693 3616 3334 Q 3616 2989 3430 2736 Q 3245 2483 2899 2349 zM 928 3322 Q 928 2950 1222 2732 Q 1517 2515 2035 2515 Q 2554 2515 2851 2732 Q 3149 2950 3149 3315 Q 3149 3693 2845 3910 Q 2541 4128 2035 4128 Q 1523 4128 1225 3910 Q 928 3693 928 3322 zM 2035 352 Q 2630 352 2966 595 Q 3302 838 3302 1254 Q 3302 1670 2966 1910 Q 2630 2150 2035 2150 Q 1446 2150 1110 1910 Q 774 1670 774 1254 Q 774 832 1107 592 Q 1440 352 2035 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-38"/>       <use xlink:href="#Montserrat-Regular-30" x="63.799988"/>      </g>     </g>    </g>    <g id="xtick_6">     <g id="line2d_6"/>     <g id="text_6">      <!-- 100 -->      <g transform="translate(380.375 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-31" d="M 1574 4480 L 1574 0 L 1114 0 L 1114 4070 L 58 4070 L 58 4480 L 1574 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>      </g>     </g>    </g>    <g id="text_7">     <!-- turns -->     <g transform="translate(208.170312 248.56125) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-74" d="M 2406 205 Q 2278 90 2089 29 Q 1901 -32 1696 -32 Q 1222 -32 966 224 Q 710 480 710 947 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 4102 L 1165 4102 L 1165 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 973 Q 1165 672 1315 515 Q 1466 358 1747 358 Q 1888 358 2019 403 Q 2150 448 2246 531 L 2406 205 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-75" d="M 3667 3366 L 3667 0 L 3232 0 L 3232 614 Q 3053 307 2739 137 Q 2426 -32 2022 -32 Q 1363 -32 982 336 Q 602 704 602 1414 L 602 3366 L 1056 3366 L 1056 1459 Q 1056 928 1318 653 Q 1581 378 2067 378 Q 2598 378 2905 701 Q 3213 1024 3213 1600 L 3213 3366 L 3667 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-72" d="M 1069 2707 Q 1229 3046 1545 3219 Q 1862 3392 2330 3392 L 2330 2950 L 2221 2957 Q 1690 2957 1389 2630 Q 1088 2304 1088 1715 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2707 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-6e" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2746 Q 1254 3053 1584 3222 Q 1914 3392 2349 3392 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-73" d="M 1536 -32 Q 1126 -32 752 86 Q 378 205 166 384 L 371 742 Q 582 576 902 470 Q 1222 365 1568 365 Q 2029 365 2249 509 Q 2470 653 2470 915 Q 2470 1101 2348 1206 Q 2227 1312 2041 1366 Q 1856 1421 1549 1472 Q 1139 1549 889 1629 Q 640 1709 464 1901 Q 288 2093 288 2432 Q 288 2854 640 3123 Q 992 3392 1619 3392 Q 1946 3392 2272 3305 Q 2598 3219 2810 3078 L 2611 2714 Q 2195 3002 1619 3002 Q 1184 3002 963 2848 Q 742 2694 742 2445 Q 742 2253 867 2137 Q 992 2022 1177 1968 Q 1363 1914 1690 1856 Q 2093 1779 2336 1702 Q 2579 1626 2752 1440 Q 2925 1254 2925 928 Q 2925 486 2557 227 Q 2189 -32 1536 -32 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-74"/>      <use xlink:href="#Montserrat-Regular-75" x="40.599991"/>      <use xlink:href="#Montserrat-Regular-72" x="107.899979"/>      <use xlink:href="#Montserrat-Regular-6e" x="147.999969"/>      <use xlink:href="#Montserrat-Regular-73" x="215.699951"/>     </g>    </g>   </g>   <g id="matplotlib.axis_2">    <g id="ytick_1">     <g id="line2d_7"/>     <g id="text_8">      <!-- LOSS -->      <g transform="translate(23.91875 227.667831) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-4c" d="M 730 4480 L 1203 4480 L 1203 410 L 3712 410 L 3712 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4f" d="M 2688 -38 Q 2022 -38 1481 259 Q 941 557 637 1075 Q 333 1594 333 2240 Q 333 2886 637 3404 Q 941 3923 1481 4220 Q 2022 4518 2688 4518 Q 3354 4518 3888 4224 Q 4422 3930 4729 3408 Q 5037 2886 5037 2240 Q 5037 1594 4729 1072 Q 4422 550 3888 256 Q 3354 -38 2688 -38 zM 2688 384 Q 3219 384 3648 624 Q 4077 864 4320 1289 Q 4563 1715 4563 2240 Q 4563 2765 4320 3190 Q 4077 3616 3648 3856 Q 3219 4096 2688 4096 Q 2157 4096 1725 3856 Q 1293 3616 1046 3190 Q 800 2765 800 2240 Q 800 1715 1046 1289 Q 1293 864 1725 624 Q 2157 384 2688 384 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-53" d="M 1965 -38 Q 1466 -38 1008 122 Q 550 282 301 544 L 486 909 Q 730 666 1133 515 Q 1536 365 1965 365 Q 2566 365 2867 585 Q 3168 806 3168 1158 Q 3168 1427 3005 1587 Q 2842 1747 2605 1833 Q 2368 1920 1946 2022 Q 1440 2150 1139 2268 Q 838 2387 624 2630 Q 410 2874 410 3290 Q 410 3629 589 3907 Q 768 4186 1139 4352 Q 1510 4518 2061 4518 Q 2445 4518 2813 4412 Q 3181 4307 3450 4122 L 3290 3744 Q 3008 3930 2688 4022 Q 2368 4115 2061 4115 Q 1472 4115 1174 3888 Q 877 3661 877 3302 Q 877 3034 1040 2870 Q 1203 2707 1449 2620 Q 1696 2534 2112 2432 Q 2605 2310 2909 2192 Q 3213 2074 3424 1834 Q 3635 1594 3635 1184 Q 3635 845 3452 566 Q 3270 288 2892 125 Q 2515 -38 1965 -38 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-4c"/>       <use xlink:href="#Montserrat-Regular-4f" x="58.899994"/>       <use xlink:href="#Montserrat-Regular-53" x="142.799988"/>       <use xlink:href="#Montserrat-Regular-53" x="204.299973"/>      </g>     </g>    </g>    <g id="ytick_2">     <g id="line2d_8"/>     <g id="text_9">      <!-- -1000 -->      <g transform="translate(23.209375 195.435273) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-2d" d="M 384 1933 L 2061 1933 L 2061 1536 L 384 1536 L 384 1933 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-31" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="74.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="140.499985"/>       <use xlink:href="#Montserrat-Regular-30" x="206.699982"/>      </g>     </g>    </g>    <g id="ytick_3">     <g id="line2d_9"/>     <g id="text_10">      <!-- -500 -->      <g transform="translate(27.779687 163.202714) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-35" d="M 1510 2643 Q 2483 2643 2928 2294 Q 3373 1946 3373 1318 Q 3373 928 3190 621 Q 3008 314 2643 138 Q 2278 -38 1747 -38 Q 1267 -38 835 118 Q 403 275 141 538 L 365 902 Q 589 672 953 528 Q 1318 384 1741 384 Q 2298 384 2602 633 Q 2906 883 2906 1299 Q 2906 1760 2563 1997 Q 2221 2234 1408 2234 L 454 2234 L 685 4480 L 3136 4480 L 3136 4070 L 1088 4070 L 934 2643 L 1510 2643 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-35" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="94.799988"/>       <use xlink:href="#Montserrat-Regular-30" x="160.999985"/>      </g>     </g>    </g>    <g id="ytick_4">     <g id="line2d_10"/>     <g id="text_11">      <!-- 0 -->      <g transform="translate(43.879688 130.970156) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="ytick_5">     <g id="line2d_11"/>     <g id="text_12">      <!-- 500 -->      <g transform="translate(31.6 98.737598) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-35"/>       <use xlink:href="#Montserrat-Regular-30" x="56.599991"/>       <use xlink:href="#Montserrat-Regular-30" x="122.799988"/>      </g>     </g>    </g>    <g id="ytick_6">     <g id="line2d_12"/>     <g id="text_13">      <!-- 1000 -->      <g transform="translate(27.029687 66.50504) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="168.499985"/>      </g>     </g>    </g>    <g id="ytick_7">     <g id="line2d_13"/>     <g id="text_14">      <!-- WIN -->      <g transform="translate(28.240625 34.272482) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-57" d="M 6880 4480 L 5370 0 L 4870 0 L 3558 3821 L 2240 0 L 1747 0 L 237 4480 L 723 4480 L 2016 627 L 3354 4480 L 3795 4480 L 5114 608 L 6426 4480 L 6880 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-49" d="M 730 4480 L 1203 4480 L 1203 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4e" d="M 4474 4480 L 4474 0 L 4083 0 L 1203 3635 L 1203 0 L 730 0 L 730 4480 L 1120 4480 L 4006 845 L 4006 4480 L 4474 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-57"/>       <use xlink:href="#Montserrat-Regular-49" x="111.099991"/>       <use xlink:href="#Montserrat-Regular-4e" x="141.299988"/>      </g>     </g>    </g>    <g id="text_15">     <!-- centipawn difference -->     <g transform="translate(17.26875 180.677188) rotate(-90) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-63" d="M 2029 -32 Q 1530 -32 1136 185 Q 742 403 518 796 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1136 3174 Q 1530 3392 2029 3392 Q 2464 3392 2806 3222 Q 3149 3053 3347 2726 L 3008 2496 Q 2842 2746 2586 2870 Q 2330 2995 2029 2995 Q 1664 2995 1373 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1293 918 995 Q 1082 698 1373 534 Q 1664 371 2029 371 Q 2330 371 2586 492 Q 2842 614 3008 864 L 3347 634 Q 3149 307 2803 137 Q 2458 -32 2029 -32 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-65" d="M 3565 1542 L 749 1542 Q 787 1018 1152 694 Q 1517 371 2074 371 Q 2387 371 2649 483 Q 2912 595 3104 813 L 3360 518 Q 3136 250 2800 109 Q 2464 -32 2061 -32 Q 1542 -32 1142 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 508 2566 Q 723 2957 1097 3174 Q 1472 3392 1939 3392 Q 2406 3392 2777 3174 Q 3149 2957 3360 2569 Q 3571 2182 3571 1683 L 3565 1542 zM 1939 3002 Q 1453 3002 1123 2691 Q 794 2381 749 1882 L 3136 1882 Q 3091 2381 2761 2691 Q 2432 3002 1939 3002 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-69" d="M 634 3366 L 1088 3366 L 1088 0 L 634 0 L 634 3366 zM 864 4102 Q 723 4102 627 4198 Q 531 4294 531 4429 Q 531 4557 627 4653 Q 723 4749 864 4749 Q 1005 4749 1101 4656 Q 1197 4563 1197 4435 Q 1197 4294 1101 4198 Q 1005 4102 864 4102 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-70" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1178 3827 787 Q 3610 397 3229 182 Q 2848 -32 2368 -32 Q 1958 -32 1628 137 Q 1299 307 1088 634 L 1088 -1242 L 634 -1242 L 634 3366 L 1069 3366 L 1069 2701 Q 1274 3034 1610 3213 Q 1946 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2364 Q 3264 2662 2976 2828 Q 2688 2995 2336 2995 Q 1978 2995 1693 2828 Q 1408 2662 1245 2364 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-61" d="M 1811 3392 Q 2470 3392 2822 3062 Q 3174 2733 3174 2086 L 3174 0 L 2739 0 L 2739 525 Q 2586 262 2288 115 Q 1990 -32 1581 -32 Q 1018 -32 685 237 Q 352 506 352 947 Q 352 1376 662 1638 Q 973 1901 1651 1901 L 2720 1901 L 2720 2106 Q 2720 2541 2477 2768 Q 2234 2995 1766 2995 Q 1446 2995 1152 2889 Q 858 2784 646 2598 L 442 2938 Q 698 3155 1056 3273 Q 1414 3392 1811 3392 zM 1651 326 Q 2035 326 2310 502 Q 2586 678 2720 1011 L 2720 1562 L 1664 1562 Q 800 1562 800 960 Q 800 666 1024 496 Q 1248 326 1651 326 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-77" d="M 5574 3366 L 4301 0 L 3872 0 L 2810 2758 L 1747 0 L 1318 0 L 51 3366 L 486 3366 L 1542 506 L 2624 3366 L 3014 3366 L 4090 506 L 5158 3366 L 5574 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-20" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-64" d="M 3706 4749 L 3706 0 L 3270 0 L 3270 666 Q 3066 326 2730 147 Q 2394 -32 1971 -32 Q 1498 -32 1114 185 Q 730 403 512 793 Q 294 1184 294 1683 Q 294 2182 512 2572 Q 730 2963 1114 3177 Q 1498 3392 1971 3392 Q 2381 3392 2710 3222 Q 3040 3053 3251 2726 L 3251 4749 L 3706 4749 zM 2010 371 Q 2362 371 2646 534 Q 2931 698 3094 998 Q 3258 1299 3258 1683 Q 3258 2067 3094 2368 Q 2931 2669 2646 2832 Q 2362 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-66" d="M 1747 4397 Q 1453 4397 1302 4237 Q 1152 4077 1152 3763 L 1152 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 0 L 710 0 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 3782 Q 710 4243 976 4512 Q 1242 4781 1728 4781 Q 1920 4781 2099 4726 Q 2278 4672 2400 4570 L 2246 4230 Q 2048 4397 1747 4397 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-63"/>      <use xlink:href="#Montserrat-Regular-65" x="56.299988"/>      <use xlink:href="#Montserrat-Regular-6e" x="116.699982"/>      <use xlink:href="#Montserrat-Regular-74" x="184.399963"/>      <use xlink:href="#Montserrat-Regular-69" x="224.999954"/>      <use xlink:href="#Montserrat-Regular-70" x="251.899948"/>      <use xlink:href="#Montserrat-Regular-61" x="319.699936"/>      <use xlink:href="#Montserrat-Regular-77" x="378.699921"/>      <use xlink:href="#Montserrat-Regular-6e" x="466.599915"/>      <use xlink:href="#Montserrat-Regular-20" x="534.299896"/>      <use xlink:href="#Montserrat-Regular-64" x="560.499893"/>      <use xlink:href="#Montserrat-Regular-69" x="628.299881"/>      <use xlink:href="#Montserrat-Regular-66" x="655.199875"/>      <use xlink:href="#Montserrat-Regular-66" x="689.099869"/>      <use xlink:href="#Montserrat-Regular-65" x="722.999863"/>      <use xlink:href="#Montserrat-Regular-72" x="783.399857"/>      <use xlink:href="#Montserrat-Regular-65" x="823.499847"/>      <use xlink:href="#Montserrat-Regular-6e" x="883.899841"/>      <use xlink:href="#Montserrat-Regular-63" x="951.599823"/>      <use xlink:href="#Montserrat-Regular-65" x="1007.899811"/>     </g>    </g>   </g>   <g id="line2d_14">    <path d="M 54 127.26 L 57.348 121.84493 L 60.696 123.585488 L 64.044 124.23014 L 67.392 121.006884 L 70.74 111.917302 L 74.088 110.49907 L 77.436 114.173581 L 80.784 117.010047 L 84.132 121.716 L 87.48 112.755349 L 90.828 112.239628 L 94.176 90.901674 L 97.524 75.816837 L 100.872 84.132837 L 104.22 76.010233 L 107.568 100.313581 L 110.916 103.730233 L 114.264 106.437767 L 117.612 99.54 L 120.96 111.852837 L 124.308 112.368558 L 127.656 161.297581 L 131.004 142.731628 L 134.352 150.402977 L 137.7 171.031814 L 141.048 170.000372 L 144.396 169.678047 L 147.744 166.906047 L 151.092 182.95786 L 154.44 223.893209 L 157.788 223.893209 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_15">    <path d="M 54 127.26 L 57.348 124.939256 L 60.696 125.712837 L 64.044 118.621674 L 67.392 118.105953 L 70.74 125.261581 L 74.088 126.808744 L 77.436 122.360651 L 80.784 135.83386 L 84.132 129.77414 L 87.48 159.041302 L 90.828 158.590047 L 94.176 158.847907 L 97.524 150.918698 L 100.872 170.258233 L 104.22 163.811721 L 107.568 175.608837 L 110.916 171.225209 L 114.264 179.089953 L 117.612 189.662233 L 120.96 204.811535 L 124.308 215.577209 L 127.656 223.893209 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_16">    <path d="M 54 127.26 L 57.348 120.039907 L 60.696 120.039907 L 64.044 116.172 L 67.392 118.363814 L 70.74 120.426698 L 74.088 119.330791 L 77.436 122.425116 L 80.784 115.076093 L 84.132 120.233302 L 87.48 125.841767 L 90.828 135.898326 L 94.176 131.321302 L 97.524 135.576 L 100.872 136.414047 L 104.22 136.736372 L 107.568 137.058698 L 110.916 149.049209 L 114.264 141.571256 L 117.612 112.884279 L 120.96 127.646791 L 124.308 157.687535 L 127.656 166.32586 L 131.004 156.849488 L 134.352 160.524 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_17">    <path d="M 54 127.26 L 57.348 122.231721 L 60.696 118.428279 L 64.044 117.267907 L 67.392 118.750605 L 70.74 119.459721 L 74.088 122.940837 L 77.436 123.843349 L 80.784 116.687721 L 84.132 130.032 L 87.48 136.349581 L 90.828 139.508372 L 94.176 128.807163 L 97.524 122.940837 L 100.872 139.186047 L 104.22 141.12 L 107.568 143.376279 L 110.916 151.434419 L 114.264 150.467442 L 117.612 157.042884 L 120.96 176.124558 L 124.308 171.998791 L 127.656 163.231535 L 131.004 163.102605 L 134.352 166.068 L 137.7 190.564744 L 141.048 189.404372 L 144.396 193.401209 L 147.744 178.058512 L 151.092 185.21414 L 154.44 190.88707 L 157.788 186.245581 L 161.136 190.822605 L 164.484 191.209395 L 167.832 191.918512 L 171.18 196.044279 L 174.528 198.558419 L 177.876 196.30214 L 181.224 198.558419 L 184.572 196.624465 L 187.92 202.168465 L 191.268 204.61814 L 194.616 190.629209 L 197.964 197.462512 L 201.312 223.893209 L 204.66 197.720372 L 208.008 223.893209 L 211.356 223.893209 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_18">    <path d="M 54 127.26 L 57.348 122.554047 L 60.696 111.079256 L 64.044 117.203442 L 67.392 119.266326 L 70.74 119.846512 L 74.088 121.264744 L 77.436 123.972279 L 80.784 126.099628 L 84.132 121.97386 L 87.48 125.132651 L 90.828 131.450233 L 94.176 129.645209 L 97.524 133.706512 L 100.872 137.252093 L 104.22 139.443907 L 107.568 142.215907 L 110.916 173.803814 L 114.264 173.094698 L 117.612 164.134047 L 120.96 171.86986 L 124.308 171.74093 L 127.656 172.192186 L 131.004 178.509767 L 134.352 178.187442 L 137.7 185.794326 L 141.048 176.511349 L 144.396 190.242419 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_19">    <path d="M 54 127.26 L 57.348 124.810326 L 60.696 124.488 L 64.044 121.393674 L 67.392 124.423535 L 70.74 125.841767 L 74.088 124.35907 L 77.436 127.840186 L 80.784 122.682977 L 84.132 124.423535 L 87.48 107.275814 L 90.828 117.719163 L 94.176 161.426512 L 97.524 162.071163 L 100.872 163.682791 L 104.22 160.846326 L 107.568 172.192186 L 110.916 168.58214 L 114.264 170.516093 L 117.612 169.678047 L 120.96 156.269302 L 124.308 155.431256 L 127.656 155.10893 L 131.004 159.621488 L 134.352 180.959442 L 137.7 176.962605 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_20">    <path d="M 54 127.26 L 57.348 121.393674 L 60.696 120.813488 L 64.044 125.003721 L 67.392 123.778884 L 70.74 124.810326 L 74.088 122.038326 L 77.436 126.228558 L 80.784 126.550884 L 84.132 126.164093 L 87.48 161.426512 L 90.828 168.517674 L 94.176 168.517674 L 97.524 172.321116 L 100.872 171.805395 L 104.22 172.578977 L 107.568 178.961023 L 110.916 177.02707 L 114.264 182.184279 L 117.612 185.72986 L 120.96 188.953116 L 124.308 186.954698 L 127.656 198.687349 L 131.004 195.206233 L 134.352 193.336744 L 137.7 192.305302 L 141.048 195.012837 L 144.396 198.042698 L 147.744 199.976651 L 151.092 193.014419 L 154.44 200.685767 L 157.788 197.462512 L 161.136 190.88707 L 164.484 203.522233 L 167.832 223.893209 L 171.18 223.893209 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_21">    <path d="M 54 127.26 L 57.348 125.197116 L 60.696 119.975442 L 64.044 121.97386 L 67.392 118.557209 L 70.74 118.557209 L 74.088 123.907814 L 77.436 125.906233 L 80.784 125.003721 L 84.132 118.68614 L 87.48 123.714419 L 90.828 120.297767 L 94.176 132.54614 L 97.524 119.330791 L 100.872 135.83386 L 104.22 136.542977 L 107.568 133.642047 L 110.916 141.442326 L 114.264 140.86214 L 117.612 145.052372 L 120.96 153.948558 L 124.308 154.786605 L 127.656 152.014605 L 131.004 176.060093 L 134.352 173.288093 L 137.7 164.520837 L 141.048 182.119814 L 144.396 183.924837 L 147.744 177.02707 L 151.092 172.707907 L 154.44 177.607256 L 157.788 182.055349 L 161.136 181.475163 L 164.484 181.346233 L 167.832 186.632372 L 171.18 223.893209 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_22">    <path d="M 54 127.26 L 57.348 121.393674 L 60.696 115.656279 L 64.044 115.333953 L 67.392 120.362233 L 70.74 118.170419 L 74.088 120.942419 L 77.436 115.011628 L 80.784 114.366977 L 84.132 119.395256 L 87.48 121.071349 L 90.828 122.489581 L 94.176 119.330791 L 97.524 126.293023 L 100.872 126.679814 L 104.22 123.907814 L 107.568 127.195535 L 110.916 127.195535 L 114.264 139.250512 L 117.612 136.800837 L 120.96 137.703349 L 124.308 137.123163 L 127.656 143.182884 L 131.004 148.404558 L 134.352 152.014605 L 137.7 134.737953 L 141.048 151.176558 L 144.396 152.788186 L 147.744 168.904465 L 151.092 223.893209 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_23">    <path d="M 54 127.26 L 57.348 124.101209 L 60.696 123.649953 L 64.044 124.294605 L 67.392 125.197116 L 70.74 127.00214 L 74.088 118.944 L 77.436 121.651535 L 80.784 126.808744 L 84.132 121.071349 L 87.48 118.621674 L 90.828 100.829302 L 94.176 127.00214 L 97.524 148.146698 L 100.872 183.731442 L 104.22 180.701581 L 107.568 169.09786 L 110.916 174.384 L 114.264 172.707907 L 117.612 174.12614 L 120.96 164.649767 L 124.308 169.935907 L 127.656 159.170233 L 131.004 168.324279 L 134.352 167.034977 L 137.7 172.127721 L 141.048 173.739349 L 144.396 170.000372 L 147.744 173.481488 L 151.092 182.7 L 154.44 180.18586 L 157.788 178.058512 L 161.136 179.605674 L 164.484 178.445302 L 167.832 177.929581 L 171.18 178.638698 L 174.528 187.98614 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_24">    <path d="M 54 127.26 L 57.348 121.84493 L 60.696 115.527349 L 64.044 118.363814 L 67.392 118.750605 L 70.74 95.349767 L 74.088 107.662605 L 77.436 108.951907 L 80.784 139.959628 L 84.132 175.093116 L 87.48 185.72986 L 90.828 178.445302 L 94.176 179.79907 L 97.524 178.380837 L 100.872 180.250326 L 104.22 186.052186 L 107.568 193.207814 L 110.916 196.624465 L 114.264 203.780093 L 117.612 208.421581 L 120.96 210.097674 L 124.308 207.067814 L 127.656 206.681023 L 131.004 209.581953 L 134.352 209.839814 L 137.7 202.168465 L 141.048 210.42 L 144.396 210.871256 L 147.744 209.195163 L 151.092 207.39014 L 154.44 205.585116 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_25">    <path d="M 54 127.26 L 57.348 123.327628 L 60.696 122.102791 L 64.044 125.326047 L 67.392 125.068186 L 70.74 123.714419 L 74.088 124.35907 L 77.436 123.327628 L 80.784 123.263163 L 84.132 124.681395 L 87.48 150.016186 L 90.828 157.752 L 94.176 168.066419 L 97.524 176.382419 L 100.872 177.736186 L 104.22 223.828744 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_26">    <path d="M 54 127.26 L 57.348 124.036744 L 60.696 124.35907 L 64.044 115.91414 L 67.392 106.437767 L 70.74 100.506977 L 74.088 118.557209 L 77.436 125.841767 L 80.784 117.525767 L 84.132 156.978419 L 87.48 155.044465 L 90.828 164.456372 L 94.176 165.294419 L 97.524 172.643442 L 100.872 180.508186 L 104.22 195.012837 L 107.568 189.339907 L 110.916 186.052186 L 114.264 189.662233 L 117.612 198.493953 L 120.96 182.95786 L 124.308 199.525395 L 127.656 150.080651 L 131.004 175.608837 L 134.352 185.923256 L 137.7 186.310047 L 141.048 147.244186 L 144.396 187.792744 L 147.744 186.696837 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_27">    <path d="M 54 127.26 L 57.348 124.036744 L 60.696 124.61693 L 64.044 124.165674 L 67.392 125.132651 L 70.74 124.35907 L 74.088 126.679814 L 77.436 124.681395 L 80.784 124.423535 L 84.132 124.74586 L 87.48 123.456558 L 90.828 123.456558 L 94.176 120.426698 L 97.524 130.354326 L 100.872 124.681395 L 104.22 127.13107 L 107.568 92.835628 L 110.916 93.286884 L 114.264 79.813674 L 117.612 73.560558 L 120.96 142.022512 L 124.308 161.039721 L 127.656 161.748837 L 131.004 153.690698 L 134.352 148.791349 L 137.7 144.472186 L 141.048 142.344837 L 144.396 158.718977 L 147.744 155.431256 L 151.092 157.558605 L 154.44 166.132465 L 157.788 154.528744 L 161.136 161.748837 L 164.484 162.586884 L 167.832 158.525581 L 171.18 161.426512 L 174.528 155.560186 L 177.876 189.920093 L 181.224 191.660651 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_28">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 122.618512 L 64.044 122.747442 L 67.392 124.294605 L 70.74 123.134233 L 74.088 123.907814 L 77.436 123.907814 L 80.784 123.907814 L 84.132 127.26 L 87.48 129.322884 L 90.828 135.83386 L 94.176 132.610605 L 97.524 126.293023 L 100.872 120.491163 L 104.22 124.810326 L 107.568 112.046233 L 110.916 112.046233 L 114.264 110.241209 L 117.612 125.583907 L 120.96 144.14986 L 124.308 133.06186 L 127.656 128.613767 L 131.004 158.267721 L 134.352 164.069581 L 137.7 167.744093 L 141.048 137.574419 L 144.396 175.995628 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_29">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 124.294605 L 64.044 126.486419 L 67.392 125.583907 L 70.74 124.294605 L 74.088 122.296186 L 77.436 118.492744 L 80.784 115.333953 L 84.132 117.977023 L 87.48 126.164093 L 90.828 155.366791 L 94.176 166.19693 L 97.524 180.766047 L 100.872 180.766047 L 104.22 176.89814 L 107.568 177.349395 L 110.916 178.445302 L 114.264 183.666977 L 117.612 173.932744 L 120.96 177.800651 L 124.308 181.217302 L 127.656 182.248744 L 131.004 180.250326 L 134.352 165.874605 L 137.7 175.286512 L 141.048 179.992465 L 144.396 174.190605 L 147.744 169.484651 L 151.092 163.102605 L 154.44 155.366791 L 157.788 166.390326 L 161.136 177.736186 L 164.484 179.992465 L 167.832 185.21414 L 171.18 181.861953 L 174.528 181.088372 L 177.876 183.538047 L 181.224 181.088372 L 184.572 174.12614 L 187.92 194.110326 L 191.268 190.88707 L 194.616 187.470419 L 197.964 191.725116 L 201.312 191.725116 L 204.66 188.630791 L 208.008 177.478326 L 211.356 201.781674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_30">    <path d="M 54 127.26 L 57.348 120.813488 L 60.696 120.877953 L 64.044 121.264744 L 67.392 119.846512 L 70.74 121.651535 L 74.088 119.266326 L 77.436 121.97386 L 80.784 122.231721 L 84.132 128.033581 L 87.48 127.26 L 90.828 126.937674 L 94.176 125.326047 L 97.524 129.193953 L 100.872 128.355907 L 104.22 134.480093 L 107.568 172.901302 L 110.916 156.720558 L 114.264 166.454791 L 117.612 166.519256 L 120.96 160.137209 L 124.308 179.025488 L 127.656 179.79907 L 131.004 176.124558 L 134.352 180.830512 L 137.7 181.990884 L 141.048 181.088372 L 144.396 184.311628 L 147.744 181.733023 L 151.092 184.698419 L 154.44 188.953116 L 157.788 193.207814 L 161.136 193.014419 L 164.484 196.43107 L 167.832 194.174791 L 171.18 194.110326 L 174.528 195.786419 L 177.876 223.893209 L 181.224 200.170047 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_31">    <path d="M 54 127.26 L 57.348 123.907814 L 60.696 120.749023 L 64.044 115.849674 L 67.392 119.910977 L 70.74 128.098047 L 74.088 121.071349 L 77.436 130.28986 L 80.784 130.612186 L 84.132 137.574419 L 87.48 140.797674 L 90.828 140.797674 L 94.176 140.088558 L 97.524 140.346419 L 100.872 134.157767 L 104.22 138.09014 L 107.568 139.250512 L 110.916 187.663814 L 114.264 179.089953 L 117.612 190.951535 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_32">    <path d="M 54 127.26 L 57.348 123.714419 L 60.696 122.102791 L 64.044 117.074512 L 67.392 116.945581 L 70.74 120.168837 L 74.088 124.036744 L 77.436 120.813488 L 80.784 123.456558 L 84.132 121.909395 L 87.48 125.197116 L 90.828 126.099628 L 94.176 118.041488 L 97.524 105.664186 L 100.872 114.109116 L 104.22 112.497488 L 107.568 113.077674 L 110.916 123.198698 L 114.264 153.948558 L 117.612 156.462698 L 120.96 170.387163 L 124.308 185.794326 L 127.656 196.68893 L 131.004 203.973488 L 134.352 199.976651 L 137.7 196.882326 L 141.048 191.725116 L 144.396 176.89814 L 147.744 187.98614 L 151.092 189.404372 L 154.44 193.594605 L 157.788 207.648 L 161.136 200.234512 L 164.484 223.893209 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_33">    <path d="M 54 127.26 L 57.348 123.005302 L 60.696 122.747442 L 64.044 122.554047 L 67.392 125.583907 L 70.74 154.528744 L 74.088 152.981581 L 77.436 154.786605 L 80.784 152.208 L 84.132 152.014605 L 87.48 150.725302 L 90.828 167.163907 L 94.176 167.034977 L 97.524 168.130884 L 100.872 186.696837 L 104.22 187.277023 L 107.568 188.308465 L 110.916 172.772372 L 114.264 180.830512 L 117.612 178.703163 L 120.96 185.278605 L 124.308 184.762884 L 127.656 186.310047 L 131.004 187.083628 L 134.352 223.893209 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_34">    <path d="M 54 127.26 L 57.348 117.074512 L 60.696 121.97386 L 64.044 122.296186 L 67.392 122.618512 L 70.74 121.651535 L 74.088 122.876372 L 77.436 119.459721 L 80.784 116.752186 L 84.132 117.332372 L 87.48 114.238047 L 90.828 116.881116 L 94.176 123.327628 L 97.524 164.005116 L 100.872 175.222047 L 104.22 180.379256 L 107.568 180.701581 L 110.916 187.921674 L 114.264 184.827349 L 117.612 195.915349 L 120.96 192.305302 L 124.308 186.181116 L 127.656 208.937302 L 131.004 204.424744 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_35">    <path d="M 54 127.26 L 57.348 123.907814 L 60.696 124.036744 L 64.044 120.620093 L 67.392 121.909395 L 70.74 124.939256 L 74.088 122.489581 L 77.436 126.937674 L 80.784 126.421953 L 84.132 124.810326 L 87.48 128.678233 L 90.828 123.778884 L 94.176 127.646791 L 97.524 124.939256 L 100.872 129.065023 L 104.22 103.021116 L 107.568 128.613767 L 110.916 127.26 L 114.264 107.469209 L 117.612 121.45814 L 120.96 130.354326 L 124.308 136.865302 L 127.656 169.291256 L 131.004 169.033395 L 134.352 166.003535 L 137.7 167.421767 L 141.048 154.270884 L 144.396 166.841581 L 147.744 176.575814 L 151.092 167.937488 L 154.44 162.006698 L 157.788 167.873023 L 161.136 182.055349 L 164.484 193.788 L 167.832 223.893209 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_36">    <path d="M 54 127.26 L 57.348 120.426698 L 60.696 124.74586 L 64.044 123.263163 L 67.392 126.421953 L 70.74 128.420372 L 74.088 126.164093 L 77.436 125.583907 L 80.784 125.583907 L 84.132 127.38893 L 87.48 128.226977 L 90.828 127.324465 L 94.176 125.068186 L 97.524 126.679814 L 100.872 124.74586 L 104.22 144.472186 L 107.568 146.083814 L 110.916 158.074326 L 114.264 182.764465 L 117.612 182.119814 L 120.96 182.635535 L 124.308 172.643442 L 127.656 182.82893 L 131.004 197.269116 L 134.352 201.394884 L 137.7 199.718791 L 141.048 189.275442 L 144.396 207.067814 L 147.744 210.613395 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_37">    <path d="M 54 127.26 L 57.348 124.036744 L 60.696 120.684558 L 64.044 118.621674 L 67.392 113.077674 L 70.74 127.51786 L 74.088 126.937674 L 77.436 125.454977 L 80.784 127.453395 L 84.132 152.981581 L 87.48 161.490977 L 90.828 152.208 L 94.176 165.229953 L 97.524 135.382605 L 100.872 154.335349 L 104.22 155.431256 L 107.568 163.747256 L 110.916 168.259814 L 114.264 165.745674 L 117.612 165.745674 L 120.96 169.549116 L 124.308 158.718977 L 127.656 162.457953 L 131.004 163.618326 L 134.352 191.918512 L 137.7 187.148093 L 141.048 196.753395 L 144.396 197.269116 L 147.744 223.893209 L 151.092 223.893209 L 154.44 223.893209 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_38">    <path d="M 54 127.26 L 57.348 121.97386 L 60.696 109.145302 L 64.044 106.437767 L 67.392 115.333953 L 70.74 112.368558 L 74.088 122.296186 L 77.436 130.032 L 80.784 119.588651 L 84.132 127.195535 L 87.48 134.157767 L 90.828 152.788186 L 94.176 161.039721 L 97.524 171.676465 L 100.872 179.347814 L 104.22 191.725116 L 107.568 187.019163 L 110.916 189.017581 L 114.264 188.824186 L 117.612 187.663814 L 120.96 190.371349 L 124.308 192.305302 L 127.656 191.467256 L 131.004 190.629209 L 134.352 192.240837 L 137.7 196.044279 L 141.048 194.561581 L 144.396 185.085209 L 147.744 187.728279 L 151.092 189.146512 L 154.44 176.89814 L 157.788 192.692093 L 161.136 196.624465 L 164.484 197.011256 L 167.832 197.269116 L 171.18 196.56 L 174.528 197.526977 L 177.876 201.008093 L 181.224 199.138605 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_39">    <path d="M 54 127.26 L 57.348 122.102791 L 60.696 118.621674 L 64.044 122.811907 L 67.392 125.068186 L 70.74 125.003721 L 74.088 119.717581 L 77.436 105.793116 L 80.784 110.047814 L 84.132 110.176744 L 87.48 116.494326 L 90.828 123.972279 L 94.176 127.00214 L 97.524 112.626419 L 100.872 125.519442 L 104.22 129.065023 L 107.568 139.443907 L 110.916 138.60586 L 114.264 139.314977 L 117.612 141.184465 L 120.96 140.024093 L 124.308 143.76307 L 127.656 141.958047 L 131.004 155.882512 L 134.352 155.946977 L 137.7 158.976837 L 141.048 152.143535 L 144.396 160.39507 L 147.744 167.228372 L 151.092 165.616744 L 154.44 163.102605 L 157.788 166.003535 L 161.136 165.681209 L 164.484 168.84 L 167.832 171.031814 L 171.18 171.160744 L 174.528 166.583721 L 177.876 168.388744 L 181.224 165.552279 L 184.572 163.747256 L 187.92 173.932744 L 191.268 173.288093 L 194.616 176.511349 L 197.964 171.805395 L 201.312 162.651349 L 204.66 177.220465 L 208.008 153.690698 L 211.356 163.296 L 214.704 162.071163 L 218.052 175.737767 L 221.4 172.385581 L 224.748 173.159163 L 228.096 171.86986 L 231.444 174.061674 L 234.792 175.544372 L 238.14 178.316372 L 241.488 180.508186 L 244.836 178.251907 L 248.184 171.48307 L 251.532 181.088372 L 254.88 182.635535 L 258.228 184.827349 L 261.576 185.794326 L 264.924 188.050605 L 268.272 187.792744 L 271.62 187.148093 L 274.968 189.597767 L 278.316 189.662233 L 281.664 192.434233 L 285.012 183.409116 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_40">    <path d="M 54 127.26 L 57.348 123.972279 L 60.696 121.909395 L 64.044 123.069767 L 67.392 125.132651 L 70.74 120.877953 L 74.088 123.069767 L 77.436 127.38893 L 80.784 125.906233 L 84.132 129.193953 L 87.48 122.747442 L 90.828 151.369953 L 94.176 157.300744 L 97.524 165.616744 L 100.872 173.030233 L 104.22 172.965767 L 107.568 171.418605 L 110.916 177.929581 L 114.264 188.630791 L 117.612 182.893395 L 120.96 186.245581 L 124.308 186.310047 L 127.656 186.310047 L 131.004 184.569488 L 134.352 180.701581 L 137.7 178.961023 L 141.048 174.770791 L 144.396 187.405953 L 147.744 187.405953 L 151.092 191.27386 L 154.44 194.110326 L 157.788 209.517488 L 161.136 203.393302 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_41">    <path d="M 54 127.26 L 57.348 123.263163 L 60.696 122.425116 L 64.044 124.036744 L 67.392 124.036744 L 70.74 125.197116 L 74.088 121.071349 L 77.436 121.071349 L 80.784 122.876372 L 84.132 125.003721 L 87.48 118.68614 L 90.828 110.37014 L 94.176 118.105953 L 97.524 106.824558 L 100.872 116.365395 L 104.22 113.27107 L 107.568 117.074512 L 110.916 117.912558 L 114.264 127.066605 L 117.612 152.401395 L 120.96 179.734605 L 124.308 180.766047 L 127.656 180.250326 L 131.004 186.954698 L 134.352 190.822605 L 137.7 199.718791 L 141.048 195.464093 L 144.396 202.619721 L 147.744 196.882326 L 151.092 198.171628 L 154.44 206.938884 L 157.788 204.74707 L 161.136 201.781674 L 164.484 201.781674 L 167.832 199.20307 L 171.18 201.717209 L 174.528 193.91693 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_42">    <path d="M 54 127.26 L 57.348 121.135814 L 60.696 116.558791 L 64.044 110.821395 L 67.392 113.335535 L 70.74 110.434605 L 74.088 112.819814 L 77.436 115.269488 L 80.784 116.30093 L 84.132 113.915721 L 87.48 112.239628 L 90.828 116.816651 L 94.176 118.879535 L 97.524 121.264744 L 100.872 113.980186 L 104.22 111.466047 L 107.568 113.786791 L 110.916 109.854419 L 114.264 114.947163 L 117.612 113.077674 L 120.96 120.039907 L 124.308 128.355907 L 127.656 149.69386 L 131.004 147.308651 L 134.352 149.500465 L 137.7 155.044465 L 141.048 157.816465 L 144.396 170.645023 L 147.744 179.347814 L 151.092 182.377674 L 154.44 176.253488 L 157.788 181.475163 L 161.136 192.885488 L 164.484 195.850884 L 167.832 198.816279 L 171.18 180.379256 L 174.528 191.982977 L 177.876 190.371349 L 181.224 184.891814 L 184.572 188.37293 L 187.92 182.248744 L 191.268 195.270698 L 194.616 198.042698 L 197.964 196.753395 L 201.312 169.291256 L 204.66 191.080465 L 208.008 180.443721 L 211.356 182.248744 L 214.704 186.890233 L 218.052 185.21414 L 221.4 194.948372 L 224.748 197.978233 L 228.096 223.893209 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_43">    <path d="M 54 127.26 L 57.348 120.426698 L 60.696 102.698791 L 64.044 138.734791 L 67.392 139.508372 L 70.74 139.443907 L 74.088 138.670326 L 77.436 139.443907 L 80.784 134.995814 L 84.132 134.286698 L 87.48 130.16093 L 90.828 134.931349 L 94.176 138.60586 L 97.524 142.989488 L 100.872 169.033395 L 104.22 163.55386 L 107.568 176.89814 L 110.916 166.454791 L 114.264 158.00986 L 117.612 160.330605 L 120.96 152.014605 L 124.308 157.171814 L 127.656 158.912372 L 131.004 143.892 L 134.352 151.95014 L 137.7 154.593209 L 141.048 152.917116 L 144.396 152.852651 L 147.744 152.788186 L 151.092 153.368372 L 154.44 151.241023 L 157.788 154.593209 L 161.136 142.796093 L 164.484 169.549116 L 167.832 169.355721 L 171.18 169.613581 L 174.528 163.03814 L 177.876 223.893209 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_44">    <path d="M 54 127.26 L 57.348 123.392093 L 60.696 122.167256 L 64.044 124.165674 L 67.392 123.327628 L 70.74 121.45814 L 74.088 121.84493 L 77.436 120.942419 L 80.784 119.588651 L 84.132 120.233302 L 87.48 120.297767 L 90.828 120.813488 L 94.176 123.585488 L 97.524 122.811907 L 100.872 113.65786 L 104.22 117.590233 L 107.568 124.036744 L 110.916 124.488 L 114.264 122.876372 L 117.612 124.294605 L 120.96 125.970698 L 124.308 129.645209 L 127.656 146.470605 L 131.004 156.527163 L 134.352 153.432837 L 137.7 156.913953 L 141.048 154.206419 L 144.396 146.728465 L 147.744 146.40614 L 151.092 153.303907 L 154.44 138.992651 L 157.788 137.252093 L 161.136 151.112093 L 164.484 156.978419 L 167.832 157.752 L 171.18 150.467442 L 174.528 160.588465 L 177.876 163.876186 L 181.224 167.421767 L 184.572 176.253488 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_45">    <path d="M 54 127.26 L 57.348 127.646791 L 60.696 125.712837 L 64.044 125.003721 L 67.392 124.101209 L 70.74 120.620093 L 74.088 123.263163 L 77.436 107.662605 L 80.784 106.631163 L 84.132 114.044651 L 87.48 112.755349 L 90.828 109.661023 L 94.176 148.726884 L 97.524 156.849488 L 100.872 148.275628 L 104.22 149.436 L 107.568 153.755163 L 110.916 159.299163 L 114.264 161.168651 L 117.612 154.528744 L 120.96 155.624651 L 124.308 155.882512 L 127.656 163.940651 L 131.004 170.258233 L 134.352 155.818047 L 137.7 158.461116 L 141.048 168.904465 L 144.396 174.64186 L 147.744 175.286512 L 151.092 184.633953 L 154.44 186.954698 L 157.788 178.574233 L 161.136 178.187442 L 164.484 180.18586 L 167.832 165.874605 L 171.18 166.068 L 174.528 181.281767 L 177.876 166.841581 L 181.224 166.583721 L 184.572 173.868279 L 187.92 192.240837 L 191.268 183.280186 L 194.616 191.596186 L 197.964 179.476744 L 201.312 176.833674 L 204.66 184.440558 L 208.008 186.503442 L 211.356 223.893209 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_46">    <path d="M 54 127.26 L 57.348 117.912558 L 60.696 120.620093 L 64.044 118.621674 L 67.392 121.006884 L 70.74 121.97386 L 74.088 124.23014 L 77.436 120.555628 L 80.784 129.77414 L 84.132 128.291442 L 87.48 129.451814 L 90.828 132.030419 L 94.176 168.96893 L 97.524 166.132465 L 100.872 162.522419 L 104.22 170.193767 L 107.568 163.42493 L 110.916 159.814884 L 114.264 161.813302 L 117.612 155.366791 L 120.96 153.819628 L 124.308 151.95014 L 127.656 165.745674 L 131.004 169.806977 L 134.352 160.846326 L 137.7 162.844744 L 141.048 165.358884 L 144.396 152.788186 L 147.744 170.129302 L 151.092 162.973674 L 154.44 183.795907 L 157.788 184.053767 L 161.136 180.314791 L 164.484 181.281767 L 167.832 180.443721 L 171.18 179.734605 L 174.528 179.541209 L 177.876 180.572651 L 181.224 181.604093 L 184.572 176.124558 L 187.92 182.82893 L 191.268 180.18586 L 194.616 182.57107 L 197.964 181.926419 L 201.312 179.734605 L 204.66 181.346233 L 208.008 176.253488 L 211.356 178.445302 L 214.704 175.286512 L 218.052 178.445302 L 221.4 175.479907 L 224.748 172.772372 L 228.096 174.835256 L 231.444 180.05693 L 234.792 167.357302 L 238.14 169.935907 L 241.488 223.893209 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_47">    <path d="M 54 127.26 L 57.348 121.97386 L 60.696 121.45814 L 64.044 122.360651 L 67.392 124.488 L 70.74 125.326047 L 74.088 124.810326 L 77.436 120.491163 L 80.784 122.489581 L 84.132 123.263163 L 87.48 127.324465 L 90.828 125.390512 L 94.176 126.099628 L 97.524 127.324465 L 100.872 126.550884 L 104.22 128.936093 L 107.568 127.26 L 110.916 126.035163 L 114.264 125.132651 L 117.612 122.876372 L 120.96 127.453395 L 124.308 138.025674 L 127.656 142.989488 L 131.004 128.484837 L 134.352 148.146698 L 137.7 145.503628 L 141.048 147.824372 L 144.396 163.360465 L 147.744 157.816465 L 151.092 181.604093 L 154.44 180.314791 L 157.788 179.412279 L 161.136 181.281767 L 164.484 176.640279 L 167.832 174.706326 L 171.18 183.215721 L 174.528 189.920093 L 177.876 185.472 L 181.224 184.053767 L 184.572 180.05693 L 187.92 182.44214 L 191.268 188.953116 L 194.616 199.138605 L 197.964 187.534884 L 201.312 181.797488 L 204.66 188.630791 L 208.008 186.890233 L 211.356 223.893209 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_48">    <path d="M 54 127.26 L 57.348 114.947163 L 60.696 123.134233 L 64.044 111.981767 L 67.392 126.421953 L 70.74 91.997581 L 74.088 100.378047 L 77.436 115.140558 L 80.784 131.579163 L 84.132 156.978419 L 87.48 153.046047 L 90.828 159.105767 L 94.176 164.520837 L 97.524 178.896558 L 100.872 176.124558 L 104.22 183.666977 L 107.568 180.508186 L 110.916 201.330419 L 114.264 223.828744 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_49">    <path d="M 54 127.26 L 57.348 123.327628 L 60.696 122.489581 L 64.044 121.200279 L 67.392 118.299349 L 70.74 121.97386 L 74.088 122.811907 L 77.436 123.327628 L 80.784 123.649953 L 84.132 113.786791 L 87.48 120.620093 L 90.828 148.017767 L 94.176 156.333767 L 97.524 153.626233 L 100.872 159.750419 L 104.22 168.066419 L 107.568 168.388744 L 110.916 182.44214 L 114.264 176.382419 L 117.612 223.893209 L 120.96 223.893209 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_50">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 120.942419 L 64.044 123.649953 L 67.392 122.554047 L 70.74 122.489581 L 74.088 124.23014 L 77.436 121.329209 L 80.784 121.651535 L 84.132 124.74586 L 87.48 123.714419 L 90.828 116.816651 L 94.176 125.841767 L 97.524 127.775721 L 100.872 128.807163 L 104.22 132.223814 L 107.568 131.708093 L 110.916 136.994233 L 114.264 142.538233 L 117.612 158.912372 L 120.96 150.918698 L 124.308 157.816465 L 127.656 157.816465 L 131.004 149.951721 L 134.352 152.143535 L 137.7 156.913953 L 141.048 158.138791 L 144.396 174.061674 L 147.744 223.893209 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_51">    <path d="M 54 127.26 L 57.348 123.327628 L 60.696 120.877953 L 64.044 118.621674 L 67.392 116.752186 L 70.74 111.401581 L 74.088 119.459721 L 77.436 114.302512 L 80.784 123.972279 L 84.132 145.310233 L 87.48 151.369953 L 90.828 129.193953 L 94.176 141.893581 L 97.524 165.229953 L 100.872 164.005116 L 104.22 164.585302 L 107.568 165.681209 L 110.916 152.272465 L 114.264 162.715814 L 117.612 159.299163 L 120.96 163.42493 L 124.308 168.646605 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_52">    <path d="M 54 127.26 L 57.348 124.036744 L 60.696 123.521023 L 64.044 120.233302 L 67.392 121.329209 L 70.74 120.426698 L 74.088 124.810326 L 77.436 123.907814 L 80.784 123.456558 L 84.132 120.877953 L 87.48 117.654698 L 90.828 120.039907 L 94.176 113.14214 L 97.524 109.145302 L 100.872 121.135814 L 104.22 164.972093 L 107.568 163.682791 L 110.916 160.524 L 114.264 156.978419 L 117.612 152.659256 L 120.96 171.289674 L 124.308 165.165488 L 127.656 168.388744 L 131.004 174.448465 L 134.352 176.060093 L 137.7 172.707907 L 141.048 173.545953 L 144.396 179.67014 L 147.744 187.98614 L 151.092 212.547349 L 154.44 214.932558 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_53">    <path d="M 54 127.26 L 57.348 121.329209 L 60.696 120.749023 L 64.044 125.003721 L 67.392 125.390512 L 70.74 126.615349 L 74.088 127.324465 L 77.436 125.454977 L 80.784 121.200279 L 84.132 126.744279 L 87.48 122.231721 L 90.828 122.811907 L 94.176 136.929767 L 97.524 148.469023 L 100.872 174.64186 L 104.22 176.89814 L 107.568 175.222047 L 110.916 183.344651 L 114.264 186.825767 L 117.612 194.368186 L 120.96 189.984558 L 124.308 188.50186 L 127.656 188.37293 L 131.004 186.761302 L 134.352 182.893395 L 137.7 194.368186 L 141.048 189.339907 L 144.396 184.311628 L 147.744 192.047442 L 151.092 188.179535 L 154.44 185.472 L 157.788 182.506605 L 161.136 181.733023 L 164.484 177.800651 L 167.832 184.569488 L 171.18 188.37293 L 174.528 185.085209 L 177.876 187.083628 L 181.224 189.726698 L 184.572 181.023907 L 187.92 190.242419 L 191.268 189.339907 L 194.616 192.434233 L 197.964 193.336744 L 201.312 192.176372 L 204.66 189.791163 L 208.008 196.108744 L 211.356 192.821023 L 214.704 187.534884 L 218.052 195.721953 L 221.4 223.893209 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_54">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 125.132651 L 64.044 123.907814 L 67.392 120.749023 L 70.74 123.714419 L 74.088 124.35907 L 77.436 123.649953 L 80.784 122.811907 L 84.132 123.198698 L 87.48 126.099628 L 90.828 148.662419 L 94.176 147.759907 L 97.524 153.046047 L 100.872 149.822791 L 104.22 158.074326 L 107.568 153.110512 L 110.916 160.201674 L 114.264 186.052186 L 117.612 186.632372 L 120.96 186.954698 L 124.308 190.242419 L 127.656 191.338326 L 131.004 190.564744 L 134.352 188.50186 L 137.7 190.88707 L 141.048 194.04586 L 144.396 195.979814 L 147.744 202.490791 L 151.092 197.784837 L 154.44 201.910605 L 157.788 206.874419 L 161.136 187.277023 L 164.484 186.890233 L 167.832 198.945209 L 171.18 186.052186 L 174.528 201.910605 L 177.876 187.405953 L 181.224 206.616558 L 184.572 211.193581 L 187.92 210.54893 L 191.268 204.61814 L 194.616 209.839814 L 197.964 223.893209 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_55">    <path d="M 54 127.26 L 57.348 122.167256 L 60.696 122.167256 L 64.044 124.101209 L 67.392 132.54614 L 70.74 122.425116 L 74.088 148.146698 L 77.436 170.258233 L 80.784 177.542791 L 84.132 178.767628 L 87.48 181.926419 L 90.828 181.088372 L 94.176 177.736186 L 97.524 179.412279 L 100.872 179.734605 L 104.22 164.972093 L 107.568 167.744093 L 110.916 171.096279 L 114.264 168.517674 L 117.612 172.707907 L 120.96 162.200093 L 124.308 131.579163 L 127.656 168.001953 L 131.004 170.193767 L 134.352 160.717395 L 137.7 161.748837 L 141.048 178.509767 L 144.396 186.954698 L 147.744 187.599349 L 151.092 223.893209 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_56">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 116.816651 L 64.044 123.521023 L 67.392 119.975442 L 70.74 121.97386 L 74.088 121.135814 L 77.436 115.849674 L 80.784 121.97386 L 84.132 122.811907 L 87.48 114.173581 L 90.828 126.099628 L 94.176 130.612186 L 97.524 131.256837 L 100.872 130.354326 L 104.22 130.096465 L 107.568 132.094884 L 110.916 135.83386 L 114.264 149.30707 L 117.612 156.075907 L 120.96 158.525581 L 124.308 153.368372 L 127.656 172.192186 L 131.004 152.917116 L 134.352 175.866698 L 137.7 185.407535 L 141.048 181.926419 L 144.396 188.824186 L 147.744 186.632372 L 151.092 174.12614 L 154.44 198.429488 L 157.788 204.61814 L 161.136 205.585116 L 164.484 212.353953 L 167.832 223.828744 L 171.18 223.828744 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_57">    <path d="M 54 127.26 L 57.348 125.583907 L 60.696 123.198698 L 64.044 121.522605 L 67.392 119.653116 L 70.74 121.716 L 74.088 122.747442 L 77.436 120.555628 L 80.784 120.039907 L 84.132 121.135814 L 87.48 115.333953 L 90.828 117.074512 L 94.176 139.057116 L 97.524 136.800837 L 100.872 131.127907 L 104.22 138.60586 L 107.568 145.890419 L 110.916 160.008279 L 114.264 160.201674 L 117.612 168.58214 L 120.96 161.942233 L 124.308 180.572651 L 127.656 168.646605 L 131.004 161.104186 L 134.352 175.866698 L 137.7 173.417023 L 141.048 169.355721 L 144.396 174.190605 L 147.744 180.443721 L 151.092 194.174791 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_58">    <path d="M 54 127.26 L 57.348 117.654698 L 60.696 125.003721 L 64.044 123.907814 L 67.392 154.206419 L 70.74 157.300744 L 74.088 149.30707 L 77.436 152.46586 L 80.784 154.013023 L 84.132 154.528744 L 87.48 147.759907 L 90.828 165.036558 L 94.176 160.910791 L 97.524 156.978419 L 100.872 159.428093 L 104.22 170.129302 L 107.568 184.440558 L 110.916 174.384 L 114.264 205.649581 L 117.612 187.341488 L 120.96 193.852465 L 124.308 195.528558 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_59">    <path d="M 54 127.26 L 57.348 123.069767 L 60.696 116.558791 L 64.044 108.178326 L 67.392 116.04307 L 70.74 109.983349 L 74.088 115.720744 L 77.436 111.917302 L 80.784 107.856 L 84.132 108.049395 L 87.48 121.716 L 90.828 119.395256 L 94.176 125.970698 L 97.524 125.519442 L 100.872 127.38893 L 104.22 123.456558 L 107.568 178.832093 L 110.916 179.154419 L 114.264 175.222047 L 117.612 169.806977 L 120.96 156.591628 L 124.308 150.145116 L 127.656 149.17814 L 131.004 174.899721 L 134.352 182.82893 L 137.7 179.412279 L 141.048 171.418605 L 144.396 173.868279 L 147.744 175.544372 L 151.092 177.220465 L 154.44 169.678047 L 157.788 223.893209 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_60">    <path d="M 54 127.26 L 57.348 125.068186 L 60.696 124.874791 L 64.044 121.651535 L 67.392 124.939256 L 70.74 127.26 L 74.088 103.278977 L 77.436 122.489581 L 80.784 123.392093 L 84.132 120.104372 L 87.48 136.285116 L 90.828 163.16707 L 94.176 153.819628 L 97.524 190.435814 L 100.872 188.888651 L 104.22 197.913767 L 107.568 200.427907 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_61">    <path d="M 54 127.26 L 57.348 123.327628 L 60.696 122.876372 L 64.044 122.360651 L 67.392 124.74586 L 70.74 123.907814 L 74.088 118.428279 L 77.436 121.393674 L 80.784 120.491163 L 84.132 121.84493 L 87.48 119.524186 L 90.828 118.557209 L 94.176 108.951907 L 97.524 95.865488 L 100.872 100.184651 L 104.22 124.23014 L 107.568 127.582326 L 110.916 128.033581 L 114.264 127.711256 L 117.612 148.082233 L 120.96 145.890419 L 124.308 139.443907 L 127.656 128.807163 L 131.004 130.741116 L 134.352 164.005116 L 137.7 183.215721 L 141.048 223.893209 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_62">    <path d="M 54 127.26 L 57.348 120.104372 L 60.696 118.363814 L 64.044 113.4 L 67.392 120.039907 L 70.74 121.716 L 74.088 118.621674 L 77.436 116.172 L 80.784 115.591814 L 84.132 114.109116 L 87.48 119.588651 L 90.828 121.58707 L 94.176 146.40614 L 97.524 157.365209 L 100.872 150.660837 L 104.22 176.060093 L 107.568 185.278605 L 110.916 178.316372 L 114.264 161.619907 L 117.612 177.091535 L 120.96 148.404558 L 124.308 178.574233 L 127.656 173.159163 L 131.004 168.517674 L 134.352 175.608837 L 137.7 152.723721 L 141.048 177.28493 L 144.396 177.156 L 147.744 186.567907 L 151.092 186.567907 L 154.44 194.110326 L 157.788 198.751814 L 161.136 198.493953 L 164.484 200.041116 L 167.832 199.46093 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_63">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 124.488 L 64.044 122.811907 L 67.392 122.231721 L 70.74 124.35907 L 74.088 122.425116 L 77.436 151.95014 L 80.784 162.329023 L 84.132 166.003535 L 87.48 161.362047 L 90.828 150.531907 L 94.176 182.893395 L 97.524 187.277023 L 100.872 188.37293 L 104.22 183.860372 L 107.568 188.695256 L 110.916 194.432651 L 114.264 202.104 L 117.612 202.619721 L 120.96 202.104 L 124.308 198.171628 L 127.656 196.108744 L 131.004 200.105581 L 134.352 200.685767 L 137.7 200.685767 L 141.048 191.789581 L 144.396 214.352372 L 147.744 207.841395 L 151.092 223.893209 L 154.44 223.893209 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #2ca02c; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_64">    <path d="M 54 127.26 L 57.348 123.327628 L 60.696 122.102791 L 64.044 122.135023 L 67.392 122.070558 L 70.74 122.199488 L 74.088 122.618512 L 77.436 122.457349 L 80.784 123.327628 L 84.132 124.713628 L 87.48 125.970698 L 90.828 128.259209 L 94.176 137.767814 L 97.524 141.667953 L 100.872 149.049209 L 104.22 156.752791 L 107.568 163.715023 L 110.916 163.489395 L 114.264 166.100233 L 117.612 167.550698 L 120.96 167.099442 L 124.308 171.86986 L 127.656 170.741721 L 131.004 175.254279 L 134.352 179.444512 L 137.7 184.311628 L 141.048 182.023116 L 144.396 187.180326 L 147.744 187.98614 L 151.092 197.978233 L 154.44 210.935721 L 157.788 223.925442 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p28428d9891)" style="fill: none; stroke: #000000; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="patch_3">    <path d="M 54 224.28 L 54 30.24 " style="fill: none"/>   </g>   <g id="patch_4">    <path d="M 388.8 224.28 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="patch_5">    <path d="M 54 224.28 L 388.8 224.28 " style="fill: none"/>   </g>   <g id="patch_6">    <path d="M 54 30.24 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="text_16">    <g id="patch_7">     <a xlink:href="https://dynomight.net/chess">      <path d="M 323.8664 225.676606 L 388.881244 225.676606 L 388.881244 213.343012 L 323.8664 213.343012 z" style="fill: #ffffff; opacity: 0.000001; stroke: #ffffff; stroke-linejoin: miter"/>     </a>    </g>    <a xlink:href="https://dynomight.net/chess">     <!-- dynomight.net/chess -->     <g style="opacity: 0.3" transform="translate(327.8664 220.734419) scale(0.05 -0.05)">      <defs>       <path id="CourierNewPSMT-64" d="M 3206 3922 L 3206 263 L 3556 263 Q 3653 263 3693 227 Q 3734 191 3734 131 Q 3734 75 3693 37 Q 3653 0 3556 0 L 2941 0 L 2941 522 Q 2484 -103 1781 -103 Q 1425 -103 1098 86 Q 772 275 583 625 Q 394 975 394 1347 Q 394 1722 583 2070 Q 772 2419 1098 2609 Q 1425 2800 1784 2800 Q 2472 2800 2941 2175 L 2941 3656 L 2591 3656 Q 2494 3656 2453 3693 Q 2413 3731 2413 3791 Q 2413 3847 2453 3884 Q 2494 3922 2591 3922 L 3206 3922 zM 2941 1347 Q 2941 1847 2603 2190 Q 2266 2534 1800 2534 Q 1331 2534 993 2190 Q 656 1847 656 1347 Q 656 850 993 504 Q 1331 159 1800 159 Q 2266 159 2603 504 Q 2941 850 2941 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-79" d="M 1938 0 L 719 2441 L 641 2441 Q 547 2441 506 2478 Q 466 2516 466 2572 Q 466 2613 486 2645 Q 506 2678 539 2692 Q 572 2706 641 2706 L 1359 2706 Q 1453 2706 1493 2668 Q 1534 2631 1534 2572 Q 1534 2516 1493 2478 Q 1453 2441 1359 2441 L 1006 2441 L 2078 288 L 3134 2441 L 2781 2441 Q 2688 2441 2647 2478 Q 2606 2516 2606 2575 Q 2606 2631 2647 2668 Q 2688 2706 2781 2706 L 3497 2706 Q 3594 2706 3634 2668 Q 3675 2631 3675 2572 Q 3675 2531 3650 2497 Q 3625 2463 3594 2452 Q 3563 2441 3419 2441 L 1756 -944 L 2166 -944 Q 2259 -944 2300 -980 Q 2341 -1016 2341 -1075 Q 2341 -1131 2300 -1168 Q 2259 -1206 2166 -1206 L 653 -1206 Q 559 -1206 518 -1170 Q 478 -1134 478 -1075 Q 478 -1016 518 -980 Q 559 -944 653 -944 L 1475 -944 L 1938 0 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6e" d="M 1072 2706 L 1072 2309 Q 1347 2588 1569 2694 Q 1791 2800 2069 2800 Q 2369 2800 2616 2672 Q 2791 2578 2933 2361 Q 3075 2144 3075 1916 L 3075 263 L 3297 263 Q 3391 263 3431 227 Q 3472 191 3472 131 Q 3472 75 3431 37 Q 3391 0 3297 0 L 2594 0 Q 2497 0 2456 37 Q 2416 75 2416 131 Q 2416 191 2456 227 Q 2497 263 2594 263 L 2813 263 L 2813 1872 Q 2813 2150 2609 2342 Q 2406 2534 2066 2534 Q 1806 2534 1615 2429 Q 1425 2325 1072 1909 L 1072 263 L 1369 263 Q 1463 263 1503 227 Q 1544 191 1544 131 Q 1544 75 1503 37 Q 1463 0 1369 0 L 513 0 Q 419 0 378 37 Q 338 75 338 131 Q 338 191 378 227 Q 419 263 513 263 L 809 263 L 809 2441 L 588 2441 Q 494 2441 453 2478 Q 413 2516 413 2575 Q 413 2631 453 2668 Q 494 2706 588 2706 L 1072 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6f" d="M 3388 1347 Q 3388 747 2958 322 Q 2528 -103 1922 -103 Q 1309 -103 881 323 Q 453 750 453 1347 Q 453 1947 881 2373 Q 1309 2800 1922 2800 Q 2528 2800 2958 2375 Q 3388 1950 3388 1347 zM 3122 1347 Q 3122 1841 2770 2187 Q 2419 2534 1919 2534 Q 1419 2534 1067 2186 Q 716 1838 716 1347 Q 716 859 1067 509 Q 1419 159 1919 159 Q 2419 159 2770 507 Q 3122 856 3122 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6d" d="M 725 2706 L 725 2441 Q 1059 2800 1397 2800 Q 1600 2800 1753 2692 Q 1906 2584 2009 2366 Q 2184 2584 2364 2692 Q 2544 2800 2725 2800 Q 3009 2800 3178 2616 Q 3400 2378 3400 2097 L 3400 263 L 3622 263 Q 3716 263 3756 227 Q 3797 191 3797 131 Q 3797 75 3756 37 Q 3716 0 3622 0 L 3138 0 L 3138 2072 Q 3138 2272 3016 2403 Q 2894 2534 2734 2534 Q 2591 2534 2431 2426 Q 2272 2319 2069 2003 L 2069 263 L 2288 263 Q 2381 263 2422 227 Q 2463 191 2463 131 Q 2463 75 2422 37 Q 2381 0 2288 0 L 1803 0 L 1803 2053 Q 1803 2263 1679 2398 Q 1556 2534 1403 2534 Q 1263 2534 1125 2444 Q 934 2316 725 2003 L 725 263 L 947 263 Q 1041 263 1081 227 Q 1122 191 1122 131 Q 1122 75 1081 37 Q 1041 0 947 0 L 241 0 Q 147 0 106 37 Q 66 75 66 131 Q 66 191 106 227 Q 147 263 241 263 L 463 263 L 463 2441 L 241 2441 Q 147 2441 106 2478 Q 66 2516 66 2575 Q 66 2631 106 2668 Q 147 2706 241 2706 L 725 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-69" d="M 2050 4053 L 2050 3375 L 1663 3375 L 1663 4053 L 2050 4053 zM 2059 2706 L 2059 263 L 3091 263 Q 3188 263 3228 227 Q 3269 191 3269 131 Q 3269 75 3228 37 Q 3188 0 3091 0 L 766 0 Q 672 0 631 37 Q 591 75 591 131 Q 591 191 631 227 Q 672 263 766 263 L 1797 263 L 1797 2441 L 1031 2441 Q 938 2441 895 2478 Q 853 2516 853 2572 Q 853 2631 893 2668 Q 934 2706 1031 2706 L 2059 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-67" d="M 2803 2225 L 2803 2706 L 3419 2706 Q 3513 2706 3553 2668 Q 3594 2631 3594 2572 Q 3594 2516 3553 2478 Q 3513 2441 3419 2441 L 3066 2441 L 3066 -181 Q 3066 -444 2953 -650 Q 2878 -788 2703 -934 Q 2528 -1081 2384 -1143 Q 2241 -1206 2000 -1206 L 1256 -1206 Q 1163 -1206 1122 -1170 Q 1081 -1134 1081 -1075 Q 1081 -1016 1122 -978 Q 1163 -941 1256 -941 L 2009 -944 Q 2241 -944 2427 -825 Q 2613 -706 2734 -478 Q 2803 -347 2803 -144 L 2803 644 Q 2381 66 1716 66 Q 1175 66 783 467 Q 391 869 391 1434 Q 391 2000 783 2400 Q 1175 2800 1716 2800 Q 2381 2800 2803 2225 zM 2803 1434 Q 2803 1897 2486 2215 Q 2169 2534 1728 2534 Q 1288 2534 972 2214 Q 656 1894 656 1434 Q 656 972 972 651 Q 1288 331 1728 331 Q 2169 331 2486 651 Q 2803 972 2803 1434 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-68" d="M 1056 3922 L 1056 2306 Q 1306 2578 1536 2689 Q 1766 2800 2050 2800 Q 2356 2800 2570 2692 Q 2784 2584 2928 2361 Q 3072 2138 3072 1891 L 3072 263 L 3366 263 Q 3463 263 3502 227 Q 3541 191 3541 131 Q 3541 75 3502 37 Q 3463 0 3366 0 L 2513 0 Q 2416 0 2375 37 Q 2334 75 2334 131 Q 2334 191 2375 227 Q 2416 263 2513 263 L 2806 263 L 2806 1872 Q 2806 2156 2601 2347 Q 2397 2538 2031 2538 Q 1744 2538 1541 2397 Q 1394 2297 1056 1925 L 1056 263 L 1353 263 Q 1447 263 1487 227 Q 1528 191 1528 131 Q 1528 75 1487 37 Q 1447 0 1353 0 L 497 0 Q 403 0 362 37 Q 322 75 322 131 Q 322 191 362 227 Q 403 263 497 263 L 794 263 L 794 3656 L 441 3656 Q 347 3656 306 3693 Q 266 3731 266 3791 Q 266 3847 306 3884 Q 347 3922 441 3922 L 1056 3922 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-74" d="M 1381 2706 L 2806 2706 Q 2900 2706 2940 2668 Q 2981 2631 2981 2572 Q 2981 2516 2940 2478 Q 2900 2441 2806 2441 L 1381 2441 L 1381 694 Q 1381 466 1564 312 Q 1747 159 2100 159 Q 2366 159 2675 239 Q 2984 319 3156 419 Q 3219 459 3259 459 Q 3309 459 3346 420 Q 3384 381 3384 328 Q 3384 281 3344 241 Q 3244 138 2858 17 Q 2472 -103 2119 -103 Q 1659 -103 1387 112 Q 1116 328 1116 694 L 1116 2441 L 631 2441 Q 538 2441 497 2478 Q 456 2516 456 2575 Q 456 2631 497 2668 Q 538 2706 631 2706 L 1116 2706 L 1116 3481 Q 1116 3575 1153 3615 Q 1191 3656 1247 3656 Q 1306 3656 1343 3615 Q 1381 3575 1381 3481 L 1381 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2e" d="M 1866 753 L 1978 753 Q 2153 753 2278 629 Q 2403 506 2403 331 Q 2403 150 2276 28 Q 2150 -94 1978 -94 L 1866 -94 Q 1691 -94 1566 29 Q 1441 153 1441 328 Q 1441 509 1567 631 Q 1694 753 1866 753 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-65" d="M 3341 1291 L 666 1291 Q 734 781 1092 470 Q 1450 159 1978 159 Q 2272 159 2594 256 Q 2916 353 3119 513 Q 3178 559 3222 559 Q 3272 559 3309 520 Q 3347 481 3347 428 Q 3347 375 3297 325 Q 3147 169 2764 33 Q 2381 -103 1978 -103 Q 1303 -103 851 339 Q 400 781 400 1409 Q 400 1981 823 2390 Q 1247 2800 1872 2800 Q 2516 2800 2931 2379 Q 3347 1959 3341 1291 zM 3075 1556 Q 2997 1991 2664 2262 Q 2331 2534 1872 2534 Q 1413 2534 1081 2265 Q 750 1997 669 1556 L 3075 1556 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2f" d="M 3100 4113 L 988 -413 Q 934 -528 850 -528 Q 797 -528 758 -489 Q 719 -450 719 -403 Q 719 -366 750 -300 L 2859 4225 Q 2894 4297 2925 4319 Q 2956 4341 3000 4341 Q 3053 4341 3092 4302 Q 3131 4263 3131 4216 Q 3131 4181 3100 4113 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-63" d="M 3016 2441 L 3016 2528 Q 3016 2625 3053 2665 Q 3091 2706 3147 2706 Q 3206 2706 3243 2665 Q 3281 2625 3281 2528 L 3281 1934 Q 3278 1838 3242 1797 Q 3206 1756 3147 1756 Q 3094 1756 3058 1792 Q 3022 1828 3016 1913 Q 2997 2134 2723 2334 Q 2450 2534 1988 2534 Q 1403 2534 1100 2168 Q 797 1803 797 1331 Q 797 822 1131 490 Q 1466 159 1997 159 Q 2303 159 2620 271 Q 2938 384 3194 634 Q 3259 697 3309 697 Q 3363 697 3398 661 Q 3434 625 3434 572 Q 3434 438 3119 231 Q 2609 -103 1984 -103 Q 1350 -103 942 301 Q 534 706 534 1328 Q 534 1963 951 2381 Q 1369 2800 2003 2800 Q 2606 2800 3016 2441 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-73" d="M 2788 2534 Q 2788 2625 2825 2665 Q 2863 2706 2919 2706 Q 2978 2706 3015 2665 Q 3053 2625 3053 2528 L 3053 2078 Q 3053 1984 3015 1943 Q 2978 1903 2919 1903 Q 2866 1903 2830 1937 Q 2794 1972 2788 2050 Q 2769 2238 2594 2359 Q 2338 2534 1916 2534 Q 1475 2534 1231 2356 Q 1047 2222 1047 2056 Q 1047 1869 1266 1744 Q 1416 1656 1834 1609 Q 2381 1550 2594 1475 Q 2897 1366 3045 1172 Q 3194 978 3194 753 Q 3194 419 2872 158 Q 2550 -103 1928 -103 Q 1306 -103 909 213 Q 909 106 896 75 Q 884 44 851 22 Q 819 0 778 0 Q 722 0 684 40 Q 647 81 647 175 L 647 716 Q 647 809 683 850 Q 719 891 778 891 Q 834 891 873 852 Q 913 813 913 747 Q 913 603 984 506 Q 1094 356 1333 257 Q 1572 159 1919 159 Q 2431 159 2681 350 Q 2931 541 2931 753 Q 2931 997 2678 1144 Q 2422 1291 1933 1341 Q 1444 1391 1231 1472 Q 1019 1553 900 1715 Q 781 1878 781 2066 Q 781 2403 1112 2601 Q 1444 2800 1903 2800 Q 2447 2800 2788 2534 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#CourierNewPSMT-64"/>      <use xlink:href="#CourierNewPSMT-79" x="60.009766"/>      <use xlink:href="#CourierNewPSMT-6e" x="120.019531"/>      <use xlink:href="#CourierNewPSMT-6f" x="180.029297"/>      <use xlink:href="#CourierNewPSMT-6d" x="240.039062"/>      <use xlink:href="#CourierNewPSMT-69" x="300.048828"/>      <use xlink:href="#CourierNewPSMT-67" x="360.058594"/>      <use xlink:href="#CourierNewPSMT-68" x="420.068359"/>      <use xlink:href="#CourierNewPSMT-74" x="480.078125"/>      <use xlink:href="#CourierNewPSMT-2e" x="540.087891"/>      <use xlink:href="#CourierNewPSMT-6e" x="600.097656"/>      <use xlink:href="#CourierNewPSMT-65" x="660.107422"/>      <use xlink:href="#CourierNewPSMT-74" x="720.117188"/>      <use xlink:href="#CourierNewPSMT-2f" x="780.126953"/>      <use xlink:href="#CourierNewPSMT-63" x="840.136719"/>      <use xlink:href="#CourierNewPSMT-68" x="900.146484"/>      <use xlink:href="#CourierNewPSMT-65" x="960.15625"/>      <use xlink:href="#CourierNewPSMT-73" x="1020.166016"/>      <use xlink:href="#CourierNewPSMT-73" x="1080.175781"/>     </g>    </a>   </g>   <g id="text_17">    <!-- llama-3.1-70b vs stockfish -->    <g transform="translate(145.652813 24.24) scale(0.12 -0.12)">     <defs>      <path id="Montserrat-Regular-6c" d="M 634 4749 L 1088 4749 L 1088 0 L 634 0 L 634 4749 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6d" d="M 4832 3392 Q 5466 3392 5827 3027 Q 6189 2662 6189 1952 L 6189 0 L 5734 0 L 5734 1907 Q 5734 2432 5481 2707 Q 5229 2982 4768 2982 Q 4243 2982 3942 2659 Q 3642 2336 3642 1766 L 3642 0 L 3187 0 L 3187 1907 Q 3187 2432 2934 2707 Q 2682 2982 2214 2982 Q 1696 2982 1392 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2752 Q 1248 3059 1568 3225 Q 1888 3392 2304 3392 Q 2726 3392 3036 3213 Q 3347 3034 3501 2682 Q 3686 3014 4035 3203 Q 4384 3392 4832 3392 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-33" d="M 1856 2560 Q 2560 2522 2928 2179 Q 3296 1837 3296 1286 Q 3296 902 3110 604 Q 2925 307 2560 134 Q 2195 -38 1670 -38 Q 1184 -38 752 118 Q 320 275 58 538 L 282 902 Q 506 672 877 528 Q 1248 384 1670 384 Q 2221 384 2521 624 Q 2822 864 2822 1286 Q 2822 1702 2521 1939 Q 2221 2176 1613 2176 L 1293 2176 L 1293 2515 L 2534 4070 L 243 4070 L 243 4480 L 3123 4480 L 3123 4154 L 1856 2560 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-2e" d="M 678 -32 Q 538 -32 438 70 Q 339 173 339 320 Q 339 467 438 566 Q 538 666 678 666 Q 819 666 921 566 Q 1024 467 1024 320 Q 1024 173 921 70 Q 819 -32 678 -32 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-37" d="M 3494 4480 L 3494 4154 L 1619 0 L 1120 0 L 2963 4070 L 659 4070 L 659 3232 L 205 3232 L 205 4480 L 3494 4480 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-62" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1184 3827 793 Q 3610 403 3226 185 Q 2842 -32 2368 -32 Q 1946 -32 1610 147 Q 1274 326 1069 666 L 1069 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2726 Q 1299 3053 1628 3222 Q 1958 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2368 Q 3264 2669 2976 2832 Q 2688 2995 2336 2995 Q 1978 2995 1693 2832 Q 1408 2669 1245 2368 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-76" d="M 3462 3366 L 1965 0 L 1498 0 L 6 3366 L 480 3366 L 1741 499 L 3014 3366 L 3462 3366 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6f" d="M 2010 -32 Q 1523 -32 1132 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1132 3174 Q 1523 3392 2010 3392 Q 2496 3392 2886 3174 Q 3277 2957 3497 2566 Q 3718 2176 3718 1683 Q 3718 1190 3497 800 Q 3277 410 2886 189 Q 2496 -32 2010 -32 zM 2010 371 Q 2368 371 2653 534 Q 2938 698 3098 998 Q 3258 1299 3258 1683 Q 3258 2067 3098 2368 Q 2938 2669 2653 2832 Q 2368 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6b" d="M 1882 1690 L 1088 960 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 1530 L 3098 3366 L 3661 3366 L 2221 1990 L 3795 0 L 3238 0 L 1882 1690 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-68" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2778 Q 1274 3072 1600 3232 Q 1926 3392 2349 3392 z" transform="scale(0.015625)"/>     </defs>     <use xlink:href="#Montserrat-Regular-6c"/>     <use xlink:href="#Montserrat-Regular-6c" x="26.899994"/>     <use xlink:href="#Montserrat-Regular-61" x="53.799988"/>     <use xlink:href="#Montserrat-Regular-6d" x="112.799973"/>     <use xlink:href="#Montserrat-Regular-61" x="218.899963"/>     <use xlink:href="#Montserrat-Regular-2d" x="277.899948"/>     <use xlink:href="#Montserrat-Regular-33" x="316.099945"/>     <use xlink:href="#Montserrat-Regular-2e" x="372.499939"/>     <use xlink:href="#Montserrat-Regular-31" x="393.699936"/>     <use xlink:href="#Montserrat-Regular-2d" x="429.799927"/>     <use xlink:href="#Montserrat-Regular-37" x="467.999924"/>     <use xlink:href="#Montserrat-Regular-30" x="526.899918"/>     <use xlink:href="#Montserrat-Regular-62" x="593.099915"/>     <use xlink:href="#Montserrat-Regular-20" x="660.899902"/>     <use xlink:href="#Montserrat-Regular-76" x="687.099899"/>     <use xlink:href="#Montserrat-Regular-73" x="741.299896"/>     <use xlink:href="#Montserrat-Regular-20" x="790.19989"/>     <use xlink:href="#Montserrat-Regular-73" x="816.399887"/>     <use xlink:href="#Montserrat-Regular-74" x="865.299881"/>     <use xlink:href="#Montserrat-Regular-6f" x="905.899872"/>     <use xlink:href="#Montserrat-Regular-63" x="968.599869"/>     <use xlink:href="#Montserrat-Regular-6b" x="1024.899857"/>     <use xlink:href="#Montserrat-Regular-66" x="1084.999847"/>     <use xlink:href="#Montserrat-Regular-69" x="1118.899841"/>     <use xlink:href="#Montserrat-Regular-73" x="1145.799835"/>     <use xlink:href="#Montserrat-Regular-68" x="1194.699829"/>    </g>   </g>  </g> </g> <defs>  <clipPath id="p28428d9891">   <rect x="54" y="30.24" width="334.8" height="194.04"/>  </clipPath> </defs></svg>](https://dynomight.net/img/chess/llama-3.1-70b_stockfish.pdf)

[Terrible](https://lichess.org/56nPlnP3). A _little_ better, but still extremely bad.

Next I tried [`llama-3.1-70b-instruct`](https://huggingface.co/meta-llama/Llama-3.1-70B-Instruct), a similar model, except trained to be better at following instructions. The results were:

[<?xml version="1.0" encoding="utf-8" standalone="no"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"  "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg xmlns:xlink="http://www.w3.org/1999/xlink" width="432pt" height="252pt" viewBox="0 0 432 252" xmlns="http://www.w3.org/2000/svg" version="1.1"> <metadata>  <rdf:RDF xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:cc="http://creativecommons.org/ns#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">   <cc:Work>    <dc:type rdf:resource="http://purl.org/dc/dcmitype/StillImage"/>    <dc:date>2024-11-14T11:33:27.647431</dc:date>    <dc:format>image/svg+xml</dc:format>    <dc:creator>     <cc:Agent>      <dc:title>Matplotlib v3.8.4, https://matplotlib.org/</dc:title>     </cc:Agent>    </dc:creator>   </cc:Work>  </rdf:RDF> </metadata> <defs>  <style type="text/css">*{stroke-linejoin: round; stroke-linecap: butt}</style> </defs> <g id="figure_1">  <g id="patch_1">   <path d="M 0 252 L 432 252 L 432 0 L 0 0 z" style="fill: #ffffff"/>  </g>  <g id="axes_1">   <g id="patch_2">    <path d="M 54 224.28 L 388.8 224.28 L 388.8 30.24 L 54 30.24 z" style="fill: #ffffff"/>   </g>   <g id="matplotlib.axis_1">    <g id="xtick_1">     <g id="line2d_1"/>     <g id="text_1">      <!-- 0 -->      <g transform="translate(50.689844 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-30" d="M 2118 -38 Q 1600 -38 1193 237 Q 787 512 560 1027 Q 333 1542 333 2240 Q 333 2938 560 3453 Q 787 3968 1193 4243 Q 1600 4518 2118 4518 Q 2637 4518 3043 4243 Q 3450 3968 3680 3453 Q 3910 2938 3910 2240 Q 3910 1542 3680 1027 Q 3450 512 3043 237 Q 2637 -38 2118 -38 zM 2118 384 Q 2509 384 2806 601 Q 3104 819 3270 1235 Q 3437 1651 3437 2240 Q 3437 2829 3270 3245 Q 3104 3661 2806 3878 Q 2509 4096 2118 4096 Q 1728 4096 1430 3878 Q 1133 3661 966 3245 Q 800 2829 800 2240 Q 800 1651 966 1235 Q 1133 819 1430 601 Q 1728 384 2118 384 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="xtick_2">     <g id="line2d_2"/>     <g id="text_2">      <!-- 20 -->      <g transform="translate(114.81 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-32" d="M 3392 410 L 3392 0 L 250 0 L 250 326 L 2112 2150 Q 2464 2496 2589 2749 Q 2714 3002 2714 3258 Q 2714 3654 2442 3875 Q 2170 4096 1664 4096 Q 877 4096 442 3597 L 115 3878 Q 378 4186 784 4352 Q 1190 4518 1702 4518 Q 2387 4518 2787 4195 Q 3187 3872 3187 3309 Q 3187 2963 3033 2643 Q 2880 2323 2451 1907 L 928 410 L 3392 410 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-32"/>       <use xlink:href="#Montserrat-Regular-30" x="56.799988"/>      </g>     </g>    </g>    <g id="xtick_3">     <g id="line2d_3"/>     <g id="text_3">      <!-- 40 -->      <g transform="translate(181.305156 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-34" d="M 4141 1178 L 3245 1178 L 3245 0 L 2784 0 L 2784 1178 L 256 1178 L 256 1510 L 2630 4480 L 3142 4480 L 851 1587 L 2797 1587 L 2797 2624 L 3245 2624 L 3245 1587 L 4141 1587 L 4141 1178 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-34"/>       <use xlink:href="#Montserrat-Regular-30" x="66.099991"/>      </g>     </g>    </g>    <g id="xtick_4">     <g id="line2d_4"/>     <g id="text_4">      <!-- 60 -->      <g transform="translate(248.524531 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-36" d="M 2189 2682 Q 2630 2682 2972 2515 Q 3315 2349 3507 2045 Q 3699 1741 3699 1338 Q 3699 928 3500 614 Q 3302 301 2953 131 Q 2605 -38 2170 -38 Q 1280 -38 806 547 Q 333 1133 333 2202 Q 333 2950 589 3468 Q 845 3987 1305 4252 Q 1766 4518 2381 4518 Q 3046 4518 3443 4275 L 3258 3904 Q 2938 4115 2387 4115 Q 1645 4115 1222 3644 Q 800 3174 800 2278 Q 800 2118 819 1914 Q 986 2278 1350 2480 Q 1715 2682 2189 2682 zM 2150 352 Q 2637 352 2941 617 Q 3245 883 3245 1325 Q 3245 1766 2941 2028 Q 2637 2291 2118 2291 Q 1779 2291 1520 2163 Q 1261 2035 1113 1811 Q 966 1587 966 1312 Q 966 1056 1104 838 Q 1242 621 1510 486 Q 1779 352 2150 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-36"/>       <use xlink:href="#Montserrat-Regular-30" x="60.899994"/>      </g>     </g>    </g>    <g id="xtick_5">     <g id="line2d_5"/>     <g id="text_5">      <!-- 80 -->      <g transform="translate(315.34 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-38" d="M 2899 2349 Q 3322 2208 3549 1926 Q 3776 1645 3776 1248 Q 3776 858 3565 566 Q 3354 275 2960 118 Q 2566 -38 2035 -38 Q 1229 -38 768 307 Q 307 653 307 1248 Q 307 1651 528 1929 Q 749 2208 1171 2349 Q 826 2483 646 2736 Q 467 2989 467 3334 Q 467 3878 892 4198 Q 1318 4518 2035 4518 Q 2509 4518 2864 4374 Q 3219 4230 3417 3961 Q 3616 3693 3616 3334 Q 3616 2989 3430 2736 Q 3245 2483 2899 2349 zM 928 3322 Q 928 2950 1222 2732 Q 1517 2515 2035 2515 Q 2554 2515 2851 2732 Q 3149 2950 3149 3315 Q 3149 3693 2845 3910 Q 2541 4128 2035 4128 Q 1523 4128 1225 3910 Q 928 3693 928 3322 zM 2035 352 Q 2630 352 2966 595 Q 3302 838 3302 1254 Q 3302 1670 2966 1910 Q 2630 2150 2035 2150 Q 1446 2150 1110 1910 Q 774 1670 774 1254 Q 774 832 1107 592 Q 1440 352 2035 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-38"/>       <use xlink:href="#Montserrat-Regular-30" x="63.799988"/>      </g>     </g>    </g>    <g id="xtick_6">     <g id="line2d_6"/>     <g id="text_6">      <!-- 100 -->      <g transform="translate(380.375 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-31" d="M 1574 4480 L 1574 0 L 1114 0 L 1114 4070 L 58 4070 L 58 4480 L 1574 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>      </g>     </g>    </g>    <g id="text_7">     <!-- turns -->     <g transform="translate(208.170312 248.56125) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-74" d="M 2406 205 Q 2278 90 2089 29 Q 1901 -32 1696 -32 Q 1222 -32 966 224 Q 710 480 710 947 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 4102 L 1165 4102 L 1165 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 973 Q 1165 672 1315 515 Q 1466 358 1747 358 Q 1888 358 2019 403 Q 2150 448 2246 531 L 2406 205 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-75" d="M 3667 3366 L 3667 0 L 3232 0 L 3232 614 Q 3053 307 2739 137 Q 2426 -32 2022 -32 Q 1363 -32 982 336 Q 602 704 602 1414 L 602 3366 L 1056 3366 L 1056 1459 Q 1056 928 1318 653 Q 1581 378 2067 378 Q 2598 378 2905 701 Q 3213 1024 3213 1600 L 3213 3366 L 3667 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-72" d="M 1069 2707 Q 1229 3046 1545 3219 Q 1862 3392 2330 3392 L 2330 2950 L 2221 2957 Q 1690 2957 1389 2630 Q 1088 2304 1088 1715 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2707 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-6e" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2746 Q 1254 3053 1584 3222 Q 1914 3392 2349 3392 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-73" d="M 1536 -32 Q 1126 -32 752 86 Q 378 205 166 384 L 371 742 Q 582 576 902 470 Q 1222 365 1568 365 Q 2029 365 2249 509 Q 2470 653 2470 915 Q 2470 1101 2348 1206 Q 2227 1312 2041 1366 Q 1856 1421 1549 1472 Q 1139 1549 889 1629 Q 640 1709 464 1901 Q 288 2093 288 2432 Q 288 2854 640 3123 Q 992 3392 1619 3392 Q 1946 3392 2272 3305 Q 2598 3219 2810 3078 L 2611 2714 Q 2195 3002 1619 3002 Q 1184 3002 963 2848 Q 742 2694 742 2445 Q 742 2253 867 2137 Q 992 2022 1177 1968 Q 1363 1914 1690 1856 Q 2093 1779 2336 1702 Q 2579 1626 2752 1440 Q 2925 1254 2925 928 Q 2925 486 2557 227 Q 2189 -32 1536 -32 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-74"/>      <use xlink:href="#Montserrat-Regular-75" x="40.599991"/>      <use xlink:href="#Montserrat-Regular-72" x="107.899979"/>      <use xlink:href="#Montserrat-Regular-6e" x="147.999969"/>      <use xlink:href="#Montserrat-Regular-73" x="215.699951"/>     </g>    </g>   </g>   <g id="matplotlib.axis_2">    <g id="ytick_1">     <g id="line2d_7"/>     <g id="text_8">      <!-- LOSS -->      <g transform="translate(23.91875 227.667831) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-4c" d="M 730 4480 L 1203 4480 L 1203 410 L 3712 410 L 3712 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4f" d="M 2688 -38 Q 2022 -38 1481 259 Q 941 557 637 1075 Q 333 1594 333 2240 Q 333 2886 637 3404 Q 941 3923 1481 4220 Q 2022 4518 2688 4518 Q 3354 4518 3888 4224 Q 4422 3930 4729 3408 Q 5037 2886 5037 2240 Q 5037 1594 4729 1072 Q 4422 550 3888 256 Q 3354 -38 2688 -38 zM 2688 384 Q 3219 384 3648 624 Q 4077 864 4320 1289 Q 4563 1715 4563 2240 Q 4563 2765 4320 3190 Q 4077 3616 3648 3856 Q 3219 4096 2688 4096 Q 2157 4096 1725 3856 Q 1293 3616 1046 3190 Q 800 2765 800 2240 Q 800 1715 1046 1289 Q 1293 864 1725 624 Q 2157 384 2688 384 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-53" d="M 1965 -38 Q 1466 -38 1008 122 Q 550 282 301 544 L 486 909 Q 730 666 1133 515 Q 1536 365 1965 365 Q 2566 365 2867 585 Q 3168 806 3168 1158 Q 3168 1427 3005 1587 Q 2842 1747 2605 1833 Q 2368 1920 1946 2022 Q 1440 2150 1139 2268 Q 838 2387 624 2630 Q 410 2874 410 3290 Q 410 3629 589 3907 Q 768 4186 1139 4352 Q 1510 4518 2061 4518 Q 2445 4518 2813 4412 Q 3181 4307 3450 4122 L 3290 3744 Q 3008 3930 2688 4022 Q 2368 4115 2061 4115 Q 1472 4115 1174 3888 Q 877 3661 877 3302 Q 877 3034 1040 2870 Q 1203 2707 1449 2620 Q 1696 2534 2112 2432 Q 2605 2310 2909 2192 Q 3213 2074 3424 1834 Q 3635 1594 3635 1184 Q 3635 845 3452 566 Q 3270 288 2892 125 Q 2515 -38 1965 -38 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-4c"/>       <use xlink:href="#Montserrat-Regular-4f" x="58.899994"/>       <use xlink:href="#Montserrat-Regular-53" x="142.799988"/>       <use xlink:href="#Montserrat-Regular-53" x="204.299973"/>      </g>     </g>    </g>    <g id="ytick_2">     <g id="line2d_8"/>     <g id="text_9">      <!-- -1000 -->      <g transform="translate(23.209375 195.435273) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-2d" d="M 384 1933 L 2061 1933 L 2061 1536 L 384 1536 L 384 1933 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-31" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="74.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="140.499985"/>       <use xlink:href="#Montserrat-Regular-30" x="206.699982"/>      </g>     </g>    </g>    <g id="ytick_3">     <g id="line2d_9"/>     <g id="text_10">      <!-- -500 -->      <g transform="translate(27.779687 163.202714) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-35" d="M 1510 2643 Q 2483 2643 2928 2294 Q 3373 1946 3373 1318 Q 3373 928 3190 621 Q 3008 314 2643 138 Q 2278 -38 1747 -38 Q 1267 -38 835 118 Q 403 275 141 538 L 365 902 Q 589 672 953 528 Q 1318 384 1741 384 Q 2298 384 2602 633 Q 2906 883 2906 1299 Q 2906 1760 2563 1997 Q 2221 2234 1408 2234 L 454 2234 L 685 4480 L 3136 4480 L 3136 4070 L 1088 4070 L 934 2643 L 1510 2643 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-35" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="94.799988"/>       <use xlink:href="#Montserrat-Regular-30" x="160.999985"/>      </g>     </g>    </g>    <g id="ytick_4">     <g id="line2d_10"/>     <g id="text_11">      <!-- 0 -->      <g transform="translate(43.879688 130.970156) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="ytick_5">     <g id="line2d_11"/>     <g id="text_12">      <!-- 500 -->      <g transform="translate(31.6 98.737598) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-35"/>       <use xlink:href="#Montserrat-Regular-30" x="56.599991"/>       <use xlink:href="#Montserrat-Regular-30" x="122.799988"/>      </g>     </g>    </g>    <g id="ytick_6">     <g id="line2d_12"/>     <g id="text_13">      <!-- 1000 -->      <g transform="translate(27.029687 66.50504) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="168.499985"/>      </g>     </g>    </g>    <g id="ytick_7">     <g id="line2d_13"/>     <g id="text_14">      <!-- WIN -->      <g transform="translate(28.240625 34.272482) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-57" d="M 6880 4480 L 5370 0 L 4870 0 L 3558 3821 L 2240 0 L 1747 0 L 237 4480 L 723 4480 L 2016 627 L 3354 4480 L 3795 4480 L 5114 608 L 6426 4480 L 6880 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-49" d="M 730 4480 L 1203 4480 L 1203 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4e" d="M 4474 4480 L 4474 0 L 4083 0 L 1203 3635 L 1203 0 L 730 0 L 730 4480 L 1120 4480 L 4006 845 L 4006 4480 L 4474 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-57"/>       <use xlink:href="#Montserrat-Regular-49" x="111.099991"/>       <use xlink:href="#Montserrat-Regular-4e" x="141.299988"/>      </g>     </g>    </g>    <g id="text_15">     <!-- centipawn difference -->     <g transform="translate(17.26875 180.677188) rotate(-90) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-63" d="M 2029 -32 Q 1530 -32 1136 185 Q 742 403 518 796 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1136 3174 Q 1530 3392 2029 3392 Q 2464 3392 2806 3222 Q 3149 3053 3347 2726 L 3008 2496 Q 2842 2746 2586 2870 Q 2330 2995 2029 2995 Q 1664 2995 1373 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1293 918 995 Q 1082 698 1373 534 Q 1664 371 2029 371 Q 2330 371 2586 492 Q 2842 614 3008 864 L 3347 634 Q 3149 307 2803 137 Q 2458 -32 2029 -32 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-65" d="M 3565 1542 L 749 1542 Q 787 1018 1152 694 Q 1517 371 2074 371 Q 2387 371 2649 483 Q 2912 595 3104 813 L 3360 518 Q 3136 250 2800 109 Q 2464 -32 2061 -32 Q 1542 -32 1142 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 508 2566 Q 723 2957 1097 3174 Q 1472 3392 1939 3392 Q 2406 3392 2777 3174 Q 3149 2957 3360 2569 Q 3571 2182 3571 1683 L 3565 1542 zM 1939 3002 Q 1453 3002 1123 2691 Q 794 2381 749 1882 L 3136 1882 Q 3091 2381 2761 2691 Q 2432 3002 1939 3002 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-69" d="M 634 3366 L 1088 3366 L 1088 0 L 634 0 L 634 3366 zM 864 4102 Q 723 4102 627 4198 Q 531 4294 531 4429 Q 531 4557 627 4653 Q 723 4749 864 4749 Q 1005 4749 1101 4656 Q 1197 4563 1197 4435 Q 1197 4294 1101 4198 Q 1005 4102 864 4102 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-70" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1178 3827 787 Q 3610 397 3229 182 Q 2848 -32 2368 -32 Q 1958 -32 1628 137 Q 1299 307 1088 634 L 1088 -1242 L 634 -1242 L 634 3366 L 1069 3366 L 1069 2701 Q 1274 3034 1610 3213 Q 1946 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2364 Q 3264 2662 2976 2828 Q 2688 2995 2336 2995 Q 1978 2995 1693 2828 Q 1408 2662 1245 2364 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-61" d="M 1811 3392 Q 2470 3392 2822 3062 Q 3174 2733 3174 2086 L 3174 0 L 2739 0 L 2739 525 Q 2586 262 2288 115 Q 1990 -32 1581 -32 Q 1018 -32 685 237 Q 352 506 352 947 Q 352 1376 662 1638 Q 973 1901 1651 1901 L 2720 1901 L 2720 2106 Q 2720 2541 2477 2768 Q 2234 2995 1766 2995 Q 1446 2995 1152 2889 Q 858 2784 646 2598 L 442 2938 Q 698 3155 1056 3273 Q 1414 3392 1811 3392 zM 1651 326 Q 2035 326 2310 502 Q 2586 678 2720 1011 L 2720 1562 L 1664 1562 Q 800 1562 800 960 Q 800 666 1024 496 Q 1248 326 1651 326 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-77" d="M 5574 3366 L 4301 0 L 3872 0 L 2810 2758 L 1747 0 L 1318 0 L 51 3366 L 486 3366 L 1542 506 L 2624 3366 L 3014 3366 L 4090 506 L 5158 3366 L 5574 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-20" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-64" d="M 3706 4749 L 3706 0 L 3270 0 L 3270 666 Q 3066 326 2730 147 Q 2394 -32 1971 -32 Q 1498 -32 1114 185 Q 730 403 512 793 Q 294 1184 294 1683 Q 294 2182 512 2572 Q 730 2963 1114 3177 Q 1498 3392 1971 3392 Q 2381 3392 2710 3222 Q 3040 3053 3251 2726 L 3251 4749 L 3706 4749 zM 2010 371 Q 2362 371 2646 534 Q 2931 698 3094 998 Q 3258 1299 3258 1683 Q 3258 2067 3094 2368 Q 2931 2669 2646 2832 Q 2362 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-66" d="M 1747 4397 Q 1453 4397 1302 4237 Q 1152 4077 1152 3763 L 1152 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 0 L 710 0 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 3782 Q 710 4243 976 4512 Q 1242 4781 1728 4781 Q 1920 4781 2099 4726 Q 2278 4672 2400 4570 L 2246 4230 Q 2048 4397 1747 4397 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-63"/>      <use xlink:href="#Montserrat-Regular-65" x="56.299988"/>      <use xlink:href="#Montserrat-Regular-6e" x="116.699982"/>      <use xlink:href="#Montserrat-Regular-74" x="184.399963"/>      <use xlink:href="#Montserrat-Regular-69" x="224.999954"/>      <use xlink:href="#Montserrat-Regular-70" x="251.899948"/>      <use xlink:href="#Montserrat-Regular-61" x="319.699936"/>      <use xlink:href="#Montserrat-Regular-77" x="378.699921"/>      <use xlink:href="#Montserrat-Regular-6e" x="466.599915"/>      <use xlink:href="#Montserrat-Regular-20" x="534.299896"/>      <use xlink:href="#Montserrat-Regular-64" x="560.499893"/>      <use xlink:href="#Montserrat-Regular-69" x="628.299881"/>      <use xlink:href="#Montserrat-Regular-66" x="655.199875"/>      <use xlink:href="#Montserrat-Regular-66" x="689.099869"/>      <use xlink:href="#Montserrat-Regular-65" x="722.999863"/>      <use xlink:href="#Montserrat-Regular-72" x="783.399857"/>      <use xlink:href="#Montserrat-Regular-65" x="823.499847"/>      <use xlink:href="#Montserrat-Regular-6e" x="883.899841"/>      <use xlink:href="#Montserrat-Regular-63" x="951.599823"/>      <use xlink:href="#Montserrat-Regular-65" x="1007.899811"/>     </g>    </g>   </g>   <g id="line2d_14">    <path d="M 54 127.26 L 57.348 121.780465 L 60.696 118.621674 L 64.044 122.231721 L 67.392 118.81507 L 70.74 121.329209 L 74.088 116.42986 L 77.436 121.135814 L 80.784 114.882698 L 84.132 122.425116 L 87.48 125.519442 L 90.828 124.294605 L 94.176 125.583907 L 97.524 123.263163 L 100.872 127.840186 L 104.22 131.450233 L 107.568 140.668744 L 110.916 143.182884 L 114.264 156.140372 L 117.612 160.524 L 120.96 157.429674 L 124.308 158.590047 L 127.656 155.560186 L 131.004 116.752186 L 134.352 161.619907 L 137.7 189.339907 L 141.048 187.148093 L 144.396 173.545953 L 147.744 186.438977 L 151.092 189.339907 L 154.44 199.912186 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_15">    <path d="M 54 127.26 L 57.348 123.069767 L 60.696 119.459721 L 64.044 123.134233 L 67.392 123.134233 L 70.74 125.454977 L 74.088 122.554047 L 77.436 119.266326 L 80.784 120.684558 L 84.132 117.138977 L 87.48 118.621674 L 90.828 113.4 L 94.176 121.84493 L 97.524 122.554047 L 100.872 127.38893 L 104.22 127.453395 L 107.568 136.285116 L 110.916 175.866698 L 114.264 147.566512 L 117.612 183.473581 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_16">    <path d="M 54 127.26 L 57.348 122.167256 L 60.696 119.910977 L 64.044 119.717581 L 67.392 115.785209 L 70.74 119.846512 L 74.088 123.649953 L 77.436 118.428279 L 80.784 118.105953 L 84.132 121.522605 L 87.48 148.340093 L 90.828 148.211163 L 94.176 157.62307 L 97.524 162.393488 L 100.872 164.843163 L 104.22 165.745674 L 107.568 174.448465 L 110.916 166.841581 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_17">    <path d="M 54 127.26 L 57.348 121.909395 L 60.696 121.071349 L 64.044 125.906233 L 67.392 124.552465 L 70.74 122.682977 L 74.088 124.552465 L 77.436 126.293023 L 80.784 95.865488 L 84.132 166.390326 L 87.48 158.590047 L 90.828 175.673302 L 94.176 173.159163 L 97.524 170.516093 L 100.872 167.744093 L 104.22 173.352558 L 107.568 192.305302 L 110.916 182.313209 L 114.264 184.247163 L 117.612 185.987721 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_18">    <path d="M 54 127.26 L 57.348 122.554047 L 60.696 106.437767 L 64.044 136.929767 L 67.392 156.527163 L 70.74 156.720558 L 74.088 156.591628 L 77.436 154.270884 L 80.784 162.006698 L 84.132 165.616744 L 87.48 156.462698 L 90.828 162.006698 L 94.176 140.281953 L 97.524 169.742512 L 100.872 172.707907 L 104.22 169.484651 L 107.568 162.780279 L 110.916 165.81014 L 114.264 191.467256 L 117.612 193.272279 L 120.96 181.926419 L 124.308 188.308465 L 127.656 191.596186 L 131.004 204.037953 L 134.352 196.56 L 137.7 196.237674 L 141.048 192.563163 L 144.396 193.401209 L 147.744 188.695256 L 151.092 182.57107 L 154.44 203.457767 L 157.788 204.166884 L 161.136 197.333581 L 164.484 193.014419 L 167.832 197.849302 L 171.18 186.890233 L 174.528 193.014419 L 177.876 223.893209 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_19">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 125.841767 L 64.044 124.165674 L 67.392 119.07293 L 70.74 124.165674 L 74.088 122.618512 L 77.436 124.165674 L 80.784 123.585488 L 84.132 122.038326 L 87.48 117.010047 L 90.828 122.682977 L 94.176 125.583907 L 97.524 150.016186 L 100.872 152.014605 L 104.22 175.415442 L 107.568 179.412279 L 110.916 160.588465 L 114.264 163.618326 L 117.612 160.65293 L 120.96 167.615163 L 124.308 161.877767 L 127.656 183.924837 L 131.004 178.445302 L 134.352 180.314791 L 137.7 184.247163 L 141.048 188.179535 L 144.396 190.500279 L 147.744 188.244 L 151.092 185.278605 L 154.44 187.599349 L 157.788 193.465674 L 161.136 192.434233 L 164.484 189.275442 L 167.832 183.473581 L 171.18 223.764279 L 174.528 223.764279 L 177.876 223.828744 L 181.224 223.828744 L 184.572 223.893209 L 187.92 223.893209 L 191.268 223.893209 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_20">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 124.35907 L 64.044 122.554047 L 67.392 123.972279 L 70.74 124.488 L 74.088 122.876372 L 77.436 125.326047 L 80.784 126.421953 L 84.132 126.228558 L 87.48 151.047628 L 90.828 150.080651 L 94.176 151.563349 L 97.524 158.654512 L 100.872 159.234698 L 104.22 163.876186 L 107.568 178.380837 L 110.916 188.824186 L 114.264 192.949953 L 117.612 188.050605 L 120.96 187.98614 L 124.308 183.538047 L 127.656 182.57107 L 131.004 199.46093 L 134.352 200.814698 L 137.7 188.37293 L 141.048 179.605674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_21">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 127.775721 L 64.044 126.164093 L 67.392 124.61693 L 70.74 120.426698 L 74.088 127.066605 L 77.436 140.604279 L 80.784 138.348 L 84.132 134.931349 L 87.48 140.733209 L 90.828 165.552279 L 94.176 166.132465 L 97.524 165.423349 L 100.872 223.893209 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_22">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 127.775721 L 64.044 126.164093 L 67.392 124.488 L 70.74 127.711256 L 74.088 139.766233 L 77.436 124.810326 L 80.784 149.242605 L 84.132 150.918698 L 87.48 163.55386 L 90.828 159.814884 L 94.176 146.986326 L 97.524 171.031814 L 100.872 167.034977 L 104.22 170.838419 L 107.568 163.03814 L 110.916 167.421767 L 114.264 173.288093 L 117.612 178.703163 L 120.96 169.09786 L 124.308 180.18586 L 127.656 172.707907 L 131.004 163.16707 L 134.352 179.089953 L 137.7 176.89814 L 141.048 181.217302 L 144.396 184.505023 L 147.744 189.791163 L 151.092 191.854047 L 154.44 195.979814 L 157.788 188.759721 L 161.136 189.662233 L 164.484 199.396465 L 167.832 202.748651 L 171.18 223.828744 L 174.528 223.893209 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_23">    <path d="M 54 127.26 L 57.348 121.135814 L 60.696 119.524186 L 64.044 119.20186 L 67.392 118.234884 L 70.74 108.049395 L 74.088 112.755349 L 77.436 117.848093 L 80.784 113.077674 L 84.132 117.783628 L 87.48 121.84493 L 90.828 121.651535 L 94.176 119.653116 L 97.524 134.480093 L 100.872 138.670326 L 104.22 138.992651 L 107.568 164.649767 L 110.916 157.300744 L 114.264 167.292837 L 117.612 155.560186 L 120.96 223.893209 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_24">    <path d="M 54 127.26 L 57.348 125.068186 L 60.696 124.874791 L 64.044 121.651535 L 67.392 125.841767 L 70.74 122.296186 L 74.088 121.45814 L 77.436 131.450233 L 80.784 116.494326 L 84.132 143.118419 L 87.48 160.588465 L 90.828 160.330605 L 94.176 161.426512 L 97.524 166.132465 L 100.872 169.291256 L 104.22 174.190605 L 107.568 189.404372 L 110.916 185.34307 L 114.264 178.896558 L 117.612 169.420186 L 120.96 178.638698 L 124.308 179.476744 L 127.656 162.522419 L 131.004 181.861953 L 134.352 199.58986 L 137.7 212.611814 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_25">    <path d="M 54 127.26 L 57.348 125.390512 L 60.696 124.165674 L 64.044 125.519442 L 67.392 125.261581 L 70.74 157.042884 L 74.088 155.366791 L 77.436 155.173395 L 80.784 151.692279 L 84.132 161.748837 L 87.48 183.924837 L 90.828 172.256651 L 94.176 178.896558 L 97.524 183.086791 L 100.872 184.182698 L 104.22 185.60093 L 107.568 193.336744 L 110.916 181.668558 L 114.264 195.206233 L 117.612 199.58986 L 120.96 206.036372 L 124.308 202.942047 L 127.656 191.789581 L 131.004 205.907442 L 134.352 211.773767 L 137.7 223.893209 L 141.048 212.096093 L 144.396 206.874419 L 147.744 200.943628 L 151.092 212.096093 L 154.44 223.893209 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_26">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 129.193953 L 64.044 129.77414 L 67.392 135.124744 L 70.74 130.805581 L 74.088 134.609023 L 77.436 139.959628 L 80.784 137.767814 L 84.132 147.566512 L 87.48 135.253674 L 90.828 145.310233 L 94.176 142.796093 L 97.524 142.667163 L 100.872 149.049209 L 104.22 152.852651 L 107.568 148.404558 L 110.916 149.500465 L 114.264 170.516093 L 117.612 160.846326 L 120.96 167.550698 L 124.308 160.39507 L 127.656 173.288093 L 131.004 189.855628 L 134.352 193.401209 L 137.7 190.75814 L 141.048 196.753395 L 144.396 200.492372 L 147.744 223.828744 L 151.092 223.893209 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_27">    <path d="M 54 127.26 L 57.348 122.296186 L 60.696 122.940837 L 64.044 124.23014 L 67.392 123.907814 L 70.74 126.615349 L 74.088 116.42986 L 77.436 126.937674 L 80.784 122.296186 L 84.132 127.38893 L 87.48 156.849488 L 90.828 148.920279 L 94.176 158.525581 L 97.524 163.360465 L 100.872 163.03814 L 104.22 160.78186 L 107.568 167.808558 L 110.916 166.583721 L 114.264 165.487814 L 117.612 171.612 L 120.96 169.355721 L 124.308 171.031814 L 127.656 182.893395 L 131.004 189.275442 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_28">    <path d="M 54 127.26 L 57.348 121.780465 L 60.696 119.20186 L 64.044 118.234884 L 67.392 124.294605 L 70.74 120.813488 L 74.088 120.297767 L 77.436 116.558791 L 80.784 124.101209 L 84.132 130.870047 L 87.48 175.415442 L 90.828 166.454791 L 94.176 167.808558 L 97.524 160.588465 L 100.872 163.16707 L 104.22 164.585302 L 107.568 175.995628 L 110.916 178.767628 L 114.264 182.893395 L 117.612 182.57107 L 120.96 181.861953 L 124.308 183.860372 L 127.656 179.67014 L 131.004 174.12614 L 134.352 179.992465 L 137.7 182.764465 L 141.048 185.149674 L 144.396 186.632372 L 147.744 188.566326 L 151.092 187.341488 L 154.44 175.737767 L 157.788 181.346233 L 161.136 185.858791 L 164.484 192.885488 L 167.832 198.687349 L 171.18 194.239256 L 174.528 200.105581 L 177.876 200.492372 L 181.224 197.526977 L 184.572 198.558419 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_29">    <path d="M 54 127.26 L 57.348 122.231721 L 60.696 117.396837 L 64.044 119.07293 L 67.392 118.428279 L 70.74 116.752186 L 74.088 116.04307 L 77.436 120.877953 L 80.784 117.912558 L 84.132 123.456558 L 87.48 133.577581 L 90.828 139.057116 L 94.176 128.355907 L 97.524 126.228558 L 100.872 132.610605 L 104.22 133.964372 L 107.568 135.31814 L 110.916 136.736372 L 114.264 144.14986 L 117.612 144.794512 L 120.96 142.925023 L 124.308 140.539814 L 127.656 141.571256 L 131.004 139.637302 L 134.352 157.816465 L 137.7 179.218884 L 141.048 182.184279 L 144.396 188.11507 L 147.744 191.402791 L 151.092 182.82893 L 154.44 196.43107 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_30">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 124.488 L 64.044 124.036744 L 67.392 127.324465 L 70.74 127.51786 L 74.088 125.712837 L 77.436 126.099628 L 80.784 125.068186 L 84.132 123.392093 L 87.48 122.682977 L 90.828 122.876372 L 94.176 149.500465 L 97.524 159.105767 L 100.872 172.321116 L 104.22 184.311628 L 107.568 169.935907 L 110.916 169.484651 L 114.264 185.60093 L 117.612 180.250326 L 120.96 179.67014 L 124.308 192.111907 L 127.656 189.082047 L 131.004 177.156 L 134.352 171.289674 L 137.7 162.973674 L 141.048 166.970512 L 144.396 163.03814 L 147.744 172.192186 L 151.092 175.093116 L 154.44 149.436 L 157.788 181.346233 L 161.136 184.247163 L 164.484 186.438977 L 167.832 186.374512 L 171.18 182.377674 L 174.528 188.50186 L 177.876 187.663814 L 181.224 189.533302 L 184.572 193.014419 L 187.92 188.695256 L 191.268 184.053767 L 194.616 223.893209 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_31">    <path d="M 54 127.26 L 57.348 122.231721 L 60.696 116.494326 L 64.044 127.324465 L 67.392 125.841767 L 70.74 126.099628 L 74.088 125.906233 L 77.436 124.552465 L 80.784 128.484837 L 84.132 136.156186 L 87.48 140.281953 L 90.828 135.70493 L 94.176 145.697023 L 97.524 156.527163 L 100.872 179.992465 L 104.22 185.536465 L 107.568 187.921674 L 110.916 187.98614 L 114.264 206.487628 L 117.612 200.685767 L 120.96 185.923256 L 124.308 198.042698 L 127.656 199.718791 L 131.004 193.078884 L 134.352 202.039535 L 137.7 206.745488 L 141.048 207.067814 L 144.396 194.04586 L 147.744 211.773767 L 151.092 208.550512 L 154.44 192.434233 L 157.788 202.942047 L 161.136 202.619721 L 164.484 206.874419 L 167.832 209.130698 L 171.18 203.199907 L 174.528 200.814698 L 177.876 192.627628 L 181.224 223.893209 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_32">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 124.35907 L 64.044 122.876372 L 67.392 124.35907 L 70.74 120.942419 L 74.088 124.294605 L 77.436 124.61693 L 80.784 120.297767 L 84.132 118.363814 L 87.48 126.937674 L 90.828 171.031814 L 94.176 167.550698 L 97.524 178.509767 L 100.872 191.338326 L 104.22 192.885488 L 107.568 196.882326 L 110.916 195.141767 L 114.264 174.706326 L 117.612 203.393302 L 120.96 205.714047 L 124.308 203.522233 L 127.656 199.009674 L 131.004 201.330419 L 134.352 211.129116 L 137.7 189.597767 L 141.048 223.893209 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_33">    <path d="M 54 127.26 L 57.348 125.712837 L 60.696 125.197116 L 64.044 124.488 L 67.392 125.454977 L 70.74 124.681395 L 74.088 123.392093 L 77.436 114.624837 L 80.784 124.036744 L 84.132 131.192372 L 87.48 157.558605 L 90.828 176.382419 L 94.176 173.803814 L 97.524 171.934326 L 100.872 171.74093 L 104.22 187.534884 L 107.568 205.13386 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_34">    <path d="M 54 127.26 L 57.348 116.42986 L 60.696 116.687721 L 64.044 116.881116 L 67.392 117.783628 L 70.74 126.486419 L 74.088 123.649953 L 77.436 123.649953 L 80.784 149.049209 L 84.132 151.692279 L 87.48 167.228372 L 90.828 156.140372 L 94.176 158.203256 L 97.524 176.89814 L 100.872 183.989302 L 104.22 181.668558 L 107.568 189.662233 L 110.916 193.014419 L 114.264 197.591442 L 117.612 190.693674 L 120.96 194.754977 L 124.308 179.605674 L 127.656 201.652744 L 131.004 192.111907 L 134.352 199.332 L 137.7 202.039535 L 141.048 196.56 L 144.396 203.522233 L 147.744 223.893209 L 151.092 223.828744 L 154.44 223.893209 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_35">    <path d="M 54 127.26 L 57.348 121.780465 L 60.696 119.20186 L 64.044 121.716 L 67.392 124.681395 L 70.74 118.750605 L 74.088 124.23014 L 77.436 126.293023 L 80.784 127.51786 L 84.132 126.486419 L 87.48 126.679814 L 90.828 126.357488 L 94.176 130.612186 L 97.524 139.766233 L 100.872 129.709674 L 104.22 138.154605 L 107.568 138.928186 L 110.916 153.239442 L 114.264 125.841767 L 117.612 152.07907 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_36">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 122.811907 L 64.044 121.135814 L 67.392 121.135814 L 70.74 119.008465 L 74.088 116.494326 L 77.436 126.679814 L 80.784 124.74586 L 84.132 120.426698 L 87.48 79.87814 L 90.828 163.03814 L 94.176 141.635721 L 97.524 158.590047 L 100.872 150.596372 L 104.22 144.665581 L 107.568 159.879349 L 110.916 153.239442 L 114.264 156.204837 L 117.612 158.074326 L 120.96 163.618326 L 124.308 172.514512 L 127.656 175.802233 L 131.004 160.330605 L 134.352 173.030233 L 137.7 183.666977 L 141.048 178.574233 L 144.396 179.67014 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_37">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 123.263163 L 64.044 121.45814 L 67.392 105.793116 L 70.74 129.516279 L 74.088 118.879535 L 77.436 151.369953 L 80.784 161.748837 L 84.132 172.321116 L 87.48 154.464279 L 90.828 154.013023 L 94.176 160.008279 L 97.524 159.234698 L 100.872 173.997209 L 104.22 171.160744 L 107.568 183.924837 L 110.916 182.248744 L 114.264 189.984558 L 117.612 191.918512 L 120.96 197.462512 L 124.308 200.750233 L 127.656 194.819442 L 131.004 197.978233 L 134.352 210.484465 L 137.7 213.44986 L 141.048 223.893209 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_38">    <path d="M 54 127.26 L 57.348 125.326047 L 60.696 124.810326 L 64.044 123.907814 L 67.392 128.098047 L 70.74 127.00214 L 74.088 126.615349 L 77.436 126.357488 L 80.784 127.711256 L 84.132 126.873209 L 87.48 121.522605 L 90.828 112.948744 L 94.176 112.819814 L 97.524 105.084 L 100.872 115.140558 L 104.22 112.175163 L 107.568 120.620093 L 110.916 127.582326 L 114.264 136.994233 L 117.612 141.12 L 120.96 151.885674 L 124.308 160.459535 L 127.656 155.302326 L 131.004 174.061674 L 134.352 179.476744 L 137.7 179.605674 L 141.048 176.575814 L 144.396 177.091535 L 147.744 182.184279 L 151.092 185.149674 L 154.44 181.604093 L 157.788 182.377674 L 161.136 184.891814 L 164.484 181.604093 L 167.832 187.083628 L 171.18 186.116651 L 174.528 167.099442 L 177.876 189.791163 L 181.224 193.143349 L 184.572 192.434233 L 187.92 190.500279 L 191.268 187.98614 L 194.616 188.37293 L 197.964 191.467256 L 201.312 189.404372 L 204.66 189.726698 L 208.008 191.918512 L 211.356 191.016 L 214.704 191.080465 L 218.052 186.116651 L 221.4 185.536465 L 224.748 190.75814 L 228.096 182.184279 L 231.444 187.341488 L 234.792 181.281767 L 238.14 188.11507 L 241.488 186.954698 L 244.836 184.118233 L 248.184 185.987721 L 251.532 223.893209 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_39">    <path d="M 54 127.26 L 57.348 125.326047 L 60.696 124.61693 L 64.044 121.97386 L 67.392 118.428279 L 70.74 120.620093 L 74.088 121.393674 L 77.436 124.488 L 80.784 114.109116 L 84.132 118.621674 L 87.48 113.27107 L 90.828 113.52893 L 94.176 118.363814 L 97.524 129.77414 L 100.872 133.384186 L 104.22 157.042884 L 107.568 167.292837 L 110.916 184.247163 L 114.264 183.344651 L 117.612 195.528558 L 120.96 197.140186 L 124.308 197.204651 L 127.656 201.008093 L 131.004 192.627628 L 134.352 206.809953 L 137.7 204.424744 L 141.048 212.611814 L 144.396 214.545767 L 147.744 216.737581 L 151.092 215.641674 L 154.44 214.610233 L 157.788 224.215535 L 161.136 228.341302 L 164.484 223.893209 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_40">    <path d="M 54 127.26 L 57.348 125.326047 L 60.696 125.068186 L 64.044 124.165674 L 67.392 123.327628 L 70.74 123.521023 L 74.088 121.58707 L 77.436 122.360651 L 80.784 123.069767 L 84.132 116.881116 L 87.48 120.942419 L 90.828 121.45814 L 94.176 123.649953 L 97.524 129.580744 L 100.872 126.808744 L 104.22 148.082233 L 107.568 151.563349 L 110.916 183.666977 L 114.264 179.283349 L 117.612 185.794326 L 120.96 176.640279 L 124.308 206.229767 L 127.656 209.839814 L 131.004 233.691907 L 134.352 223.893209 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_41">    <path d="M 54 127.26 L 57.348 120.813488 L 60.696 120.620093 L 64.044 122.038326 L 67.392 123.521023 L 70.74 122.618512 L 74.088 124.74586 L 77.436 123.585488 L 80.784 119.137395 L 84.132 118.363814 L 87.48 121.006884 L 90.828 124.552465 L 94.176 154.013023 L 97.524 157.558605 L 100.872 153.432837 L 104.22 179.79907 L 107.568 179.089953 L 110.916 178.767628 L 114.264 158.138791 L 117.612 178.703163 L 120.96 179.67014 L 124.308 179.863535 L 127.656 182.57107 L 131.004 173.610419 L 134.352 179.412279 L 137.7 181.281767 L 141.048 180.766047 L 144.396 165.487814 L 147.744 170.064837 L 151.092 177.220465 L 154.44 179.734605 L 157.788 167.228372 L 161.136 188.695256 L 164.484 185.536465 L 167.832 184.311628 L 171.18 182.313209 L 174.528 175.479907 L 177.876 172.127721 L 181.224 181.926419 L 184.572 176.89814 L 187.92 180.121395 L 191.268 177.736186 L 194.616 177.671721 L 197.964 187.470419 L 201.312 187.857209 L 204.66 223.828744 L 208.008 223.893209 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_42">    <path d="M 54 127.26 L 57.348 117.977023 L 60.696 116.42986 L 64.044 118.363814 L 67.392 114.173581 L 70.74 96.63907 L 74.088 119.330791 L 77.436 129.77414 L 80.784 127.195535 L 84.132 124.552465 L 87.48 125.519442 L 90.828 128.678233 L 94.176 152.530326 L 97.524 173.932744 L 100.872 170.580558 L 104.22 170.902884 L 107.568 172.836837 L 110.916 182.248744 L 114.264 188.759721 L 117.612 184.247163 L 120.96 187.277023 L 124.308 194.432651 L 127.656 185.536465 L 131.004 189.468837 L 134.352 193.594605 L 137.7 202.748651 L 141.048 201.523814 L 144.396 189.082047 L 147.744 203.844558 L 151.092 192.498698 L 154.44 177.220465 L 157.788 198.300558 L 161.136 195.270698 L 164.484 201.652744 L 167.832 195.335163 L 171.18 206.938884 L 174.528 215.770605 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_43">    <path d="M 54 127.26 L 57.348 120.877953 L 60.696 120.039907 L 64.044 115.205023 L 67.392 108.951907 L 70.74 114.882698 L 74.088 110.37014 L 77.436 112.368558 L 80.784 118.621674 L 84.132 135.769395 L 87.48 133.964372 L 90.828 137.058698 L 94.176 157.042884 L 97.524 149.69386 L 100.872 149.242605 L 104.22 161.748837 L 107.568 185.085209 L 110.916 181.475163 L 114.264 175.157581 L 117.612 170.838419 L 120.96 186.116651 L 124.308 196.044279 L 127.656 191.725116 L 131.004 191.596186 L 134.352 223.893209 L 137.7 197.140186 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_44">    <path d="M 54 127.26 L 57.348 121.909395 L 60.696 120.749023 L 64.044 120.942419 L 67.392 117.203442 L 70.74 120.297767 L 74.088 121.716 L 77.436 125.003721 L 80.784 126.744279 L 84.132 122.231721 L 87.48 121.200279 L 90.828 147.759907 L 94.176 148.791349 L 97.524 223.957674 L 100.872 223.957674 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_45">    <path d="M 54 127.26 L 57.348 125.326047 L 60.696 125.841767 L 64.044 125.519442 L 67.392 124.74586 L 70.74 124.488 L 74.088 126.099628 L 77.436 128.162512 L 80.784 129.580744 L 84.132 152.33693 L 87.48 168.066419 L 90.828 168.904465 L 94.176 173.094698 L 97.524 171.934326 L 100.872 173.610419 L 104.22 183.538047 L 107.568 185.21414 L 110.916 180.250326 L 114.264 182.506605 L 117.612 190.564744 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_46">    <path d="M 54 127.26 L 57.348 121.200279 L 60.696 124.35907 L 64.044 122.489581 L 67.392 122.167256 L 70.74 119.395256 L 74.088 118.557209 L 77.436 120.362233 L 80.784 117.783628 L 84.132 122.747442 L 87.48 120.684558 L 90.828 126.615349 L 94.176 161.233116 L 97.524 161.748837 L 100.872 160.588465 L 104.22 160.072744 L 107.568 138.670326 L 110.916 150.274047 L 114.264 156.978419 L 117.612 155.882512 L 120.96 165.294419 L 124.308 173.803814 L 127.656 180.766047 L 131.004 177.929581 L 134.352 190.88707 L 137.7 195.850884 L 141.048 199.46093 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_47">    <path d="M 54 127.26 L 57.348 125.197116 L 60.696 123.972279 L 64.044 125.970698 L 67.392 140.346419 L 70.74 140.733209 L 74.088 143.053953 L 77.436 148.855814 L 80.784 149.113674 L 84.132 152.014605 L 87.48 167.486233 L 90.828 161.877767 L 94.176 174.25507 L 97.524 174.448465 L 100.872 186.567907 L 104.22 154.98 L 107.568 223.893209 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_48">    <path d="M 54 127.26 L 57.348 122.296186 L 60.696 123.327628 L 64.044 124.101209 L 67.392 117.977023 L 70.74 125.454977 L 74.088 130.547721 L 77.436 135.060279 L 80.784 112.819814 L 84.132 165.616744 L 87.48 170.000372 L 90.828 172.127721 L 94.176 172.772372 L 97.524 170.387163 L 100.872 168.066419 L 104.22 169.09786 L 107.568 167.421767 L 110.916 175.222047 L 114.264 180.637116 L 117.612 185.149674 L 120.96 180.121395 L 124.308 187.599349 L 127.656 179.928 L 131.004 186.761302 L 134.352 192.434233 L 137.7 186.696837 L 141.048 223.893209 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_49">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 116.816651 L 64.044 109.145302 L 67.392 116.30093 L 70.74 115.398419 L 74.088 118.68614 L 77.436 121.393674 L 80.784 122.489581 L 84.132 122.038326 L 87.48 122.038326 L 90.828 125.906233 L 94.176 131.256837 L 97.524 120.749023 L 100.872 112.497488 L 104.22 120.877953 L 107.568 122.102791 L 110.916 112.368558 L 114.264 117.461302 L 117.612 138.47693 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_50">    <path d="M 54 127.26 L 57.348 119.653116 L 60.696 124.23014 L 64.044 124.23014 L 67.392 124.165674 L 70.74 126.099628 L 74.088 124.74586 L 77.436 128.033581 L 80.784 158.138791 L 84.132 158.396651 L 87.48 156.462698 L 90.828 151.563349 L 94.176 138.734791 L 97.524 158.912372 L 100.872 159.363628 L 104.22 163.55386 L 107.568 163.876186 L 110.916 177.02707 L 114.264 167.486233 L 117.612 160.910791 L 120.96 167.744093 L 124.308 185.149674 L 127.656 171.86986 L 131.004 183.022326 L 134.352 187.277023 L 137.7 192.240837 L 141.048 187.277023 L 144.396 182.82893 L 147.744 193.91693 L 151.092 195.270698 L 154.44 207.970326 L 157.788 192.434233 L 161.136 191.531721 L 164.484 223.893209 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_51">    <path d="M 54 127.26 L 57.348 120.877953 L 60.696 108.822977 L 64.044 119.524186 L 67.392 116.30093 L 70.74 121.97386 L 74.088 116.558791 L 77.436 123.456558 L 80.784 124.294605 L 84.132 124.488 L 87.48 125.261581 L 90.828 126.164093 L 94.176 125.390512 L 97.524 112.884279 L 100.872 113.077674 L 104.22 117.719163 L 107.568 123.392093 L 110.916 123.907814 L 114.264 125.777302 L 117.612 171.74093 L 120.96 162.651349 L 124.308 165.165488 L 127.656 173.159163 L 131.004 173.674884 L 134.352 175.350977 L 137.7 169.09786 L 141.048 177.28493 L 144.396 178.251907 L 147.744 180.121395 L 151.092 182.57107 L 154.44 184.505023 L 157.788 180.379256 L 161.136 182.764465 L 164.484 181.346233 L 167.832 179.67014 L 171.18 176.640279 L 174.528 183.215721 L 177.876 175.931163 L 181.224 179.154419 L 184.572 170.000372 L 187.92 186.245581 L 191.268 185.987721 L 194.616 184.762884 L 197.964 190.371349 L 201.312 190.629209 L 204.66 186.761302 L 208.008 187.728279 L 211.356 189.726698 L 214.704 189.404372 L 218.052 188.566326 L 221.4 179.476744 L 224.748 189.662233 L 228.096 189.468837 L 231.444 193.272279 L 234.792 223.893209 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_52">    <path d="M 54 127.26 L 57.348 119.846512 L 60.696 124.810326 L 64.044 116.687721 L 67.392 119.395256 L 70.74 115.849674 L 74.088 115.398419 L 77.436 107.340279 L 80.784 126.099628 L 84.132 139.701767 L 87.48 136.156186 L 90.828 139.895163 L 94.176 140.024093 L 97.524 133.964372 L 100.872 157.49414 L 104.22 165.552279 L 107.568 169.420186 L 110.916 181.604093 L 114.264 185.72986 L 117.612 192.111907 L 120.96 190.435814 L 124.308 176.640279 L 127.656 192.240837 L 131.004 187.405953 L 134.352 171.805395 L 137.7 179.089953 L 141.048 182.635535 L 144.396 218.99386 L 147.744 206.423163 L 151.092 223.893209 L 154.44 223.893209 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_53">    <path d="M 54 127.26 L 57.348 125.261581 L 60.696 125.068186 L 64.044 121.200279 L 67.392 124.810326 L 70.74 124.61693 L 74.088 122.231721 L 77.436 115.076093 L 80.784 123.778884 L 84.132 124.488 L 87.48 121.200279 L 90.828 130.418791 L 94.176 174.25507 L 97.524 149.56493 L 100.872 143.247349 L 104.22 156.140372 L 107.568 153.432837 L 110.916 150.725302 L 114.264 150.531907 L 117.612 150.531907 L 120.96 150.016186 L 124.308 142.344837 L 127.656 147.244186 L 131.004 173.545953 L 134.352 169.162326 L 137.7 178.574233 L 141.048 176.189023 L 144.396 177.156 L 147.744 173.417023 L 151.092 188.050605 L 154.44 186.761302 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_54">    <path d="M 54 127.26 L 57.348 122.038326 L 60.696 120.168837 L 64.044 117.977023 L 67.392 119.717581 L 70.74 116.365395 L 74.088 114.431442 L 77.436 115.076093 L 80.784 118.621674 L 84.132 143.118419 L 87.48 147.115256 L 90.828 147.695442 L 94.176 138.283535 L 97.524 150.209581 L 100.872 145.697023 L 104.22 160.459535 L 107.568 167.679628 L 110.916 172.836837 L 114.264 171.86986 L 117.612 184.053767 L 120.96 183.989302 L 124.308 182.7 L 127.656 196.108744 L 131.004 196.81786 L 134.352 195.786419 L 137.7 196.43107 L 141.048 197.978233 L 144.396 194.883907 L 147.744 195.464093 L 151.092 201.910605 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_55">    <path d="M 54 127.26 L 57.348 125.197116 L 60.696 116.687721 L 64.044 110.563535 L 67.392 223.957674 L 70.74 223.957674 L 74.088 223.957674 L 77.436 223.957674 L 80.784 223.957674 L 84.132 223.957674 L 87.48 223.957674 L 90.828 223.957674 L 94.176 223.957674 L 97.524 223.957674 L 100.872 223.957674 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_56">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 124.165674 L 64.044 121.84493 L 67.392 125.003721 L 70.74 125.003721 L 74.088 124.939256 L 77.436 123.069767 L 80.784 124.488 L 84.132 116.816651 L 87.48 125.583907 L 90.828 147.308651 L 94.176 147.244186 L 97.524 179.476744 L 100.872 181.990884 L 104.22 178.574233 L 107.568 176.124558 L 110.916 195.850884 L 114.264 184.182698 L 117.612 223.893209 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_57">    <path d="M 54 127.26 L 57.348 118.363814 L 60.696 114.173581 L 64.044 112.368558 L 67.392 119.20186 L 70.74 122.940837 L 74.088 111.014791 L 77.436 125.712837 L 80.784 126.164093 L 84.132 128.098047 L 87.48 130.354326 L 90.828 128.807163 L 94.176 129.516279 L 97.524 134.737953 L 100.872 133.964372 L 104.22 141.313395 L 107.568 133.319721 L 110.916 152.530326 L 114.264 182.893395 L 117.612 192.627628 L 120.96 193.981395 L 124.308 195.077302 L 127.656 191.918512 L 131.004 176.962605 L 134.352 186.696837 L 137.7 184.633953 L 141.048 223.893209 L 144.396 223.893209 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_58">    <path d="M 54 127.26 L 57.348 122.231721 L 60.696 128.936093 L 64.044 130.032 L 67.392 124.61693 L 70.74 122.038326 L 74.088 118.234884 L 77.436 97.670512 L 80.784 138.025674 L 84.132 127.453395 L 87.48 139.186047 L 90.828 164.907628 L 94.176 182.95786 L 97.524 184.762884 L 100.872 185.665395 L 104.22 188.11507 L 107.568 187.470419 L 110.916 187.083628 L 114.264 172.578977 L 117.612 184.376093 L 120.96 183.666977 L 124.308 192.305302 L 127.656 195.464093 L 131.004 196.624465 L 134.352 202.942047 L 137.7 203.006512 L 141.048 204.61814 L 144.396 201.588279 L 147.744 207.325674 L 151.092 207.067814 L 154.44 210.613395 L 157.788 210.742326 L 161.136 214.481302 L 164.484 217.253302 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_59">    <path d="M 54 127.26 L 57.348 124.810326 L 60.696 123.972279 L 64.044 120.233302 L 67.392 117.783628 L 70.74 126.486419 L 74.088 119.008465 L 77.436 130.612186 L 80.784 127.646791 L 84.132 117.654698 L 87.48 119.588651 L 90.828 156.720558 L 94.176 179.734605 L 97.524 190.564744 L 100.872 190.822605 L 104.22 179.605674 L 107.568 178.574233 L 110.916 177.542791 L 114.264 180.508186 L 117.612 189.339907 L 120.96 188.953116 L 124.308 169.935907 L 127.656 193.981395 L 131.004 191.596186 L 134.352 193.788 L 137.7 190.693674 L 141.048 191.725116 L 144.396 200.750233 L 147.744 203.264372 L 151.092 208.614977 L 154.44 211.644837 L 157.788 210.226605 L 161.136 202.813116 L 164.484 205.971907 L 167.832 213.32093 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_60">    <path d="M 54 127.26 L 57.348 122.231721 L 60.696 121.651535 L 64.044 122.747442 L 67.392 114.882698 L 70.74 120.039907 L 74.088 120.877953 L 77.436 117.848093 L 80.784 124.74586 L 84.132 122.747442 L 87.48 118.68614 L 90.828 150.080651 L 94.176 166.648186 L 97.524 169.226791 L 100.872 168.71107 L 104.22 172.192186 L 107.568 174.706326 L 110.916 176.640279 L 114.264 194.819442 L 117.612 192.434233 L 120.96 189.146512 L 124.308 191.596186 L 127.656 190.822605 L 131.004 198.751814 L 134.352 194.626047 L 137.7 223.893209 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_61">    <path d="M 54 127.26 L 57.348 124.810326 L 60.696 124.101209 L 64.044 121.97386 L 67.392 123.005302 L 70.74 123.649953 L 74.088 124.294605 L 77.436 125.712837 L 80.784 126.615349 L 84.132 127.51786 L 87.48 121.200279 L 90.828 122.554047 L 94.176 143.182884 L 97.524 160.459535 L 100.872 160.910791 L 104.22 167.099442 L 107.568 150.596372 L 110.916 153.497302 L 114.264 169.226791 L 117.612 168.001953 L 120.96 175.222047 L 124.308 172.321116 L 127.656 191.596186 L 131.004 189.726698 L 134.352 194.303721 L 137.7 187.470419 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_62">    <path d="M 54 127.26 L 57.348 120.877953 L 60.696 119.07293 L 64.044 123.778884 L 67.392 124.488 L 70.74 123.585488 L 74.088 123.263163 L 77.436 121.071349 L 80.784 128.098047 L 84.132 124.939256 L 87.48 128.420372 L 90.828 157.49414 L 94.176 161.233116 L 97.524 168.453209 L 100.872 161.168651 L 104.22 160.975256 L 107.568 165.552279 L 110.916 171.160744 L 114.264 171.35414 L 117.612 166.712651 L 120.96 170.645023 L 124.308 173.739349 L 127.656 175.093116 L 131.004 191.918512 L 134.352 199.654326 L 137.7 189.468837 L 141.048 181.668558 L 144.396 165.616744 L 147.744 191.338326 L 151.092 204.102419 L 154.44 223.893209 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_63">    <path d="M 54 127.26 L 57.348 124.101209 L 60.696 119.459721 L 64.044 117.332372 L 67.392 117.654698 L 70.74 117.590233 L 74.088 119.137395 L 77.436 119.975442 L 80.784 123.327628 L 84.132 124.488 L 87.48 128.098047 L 90.828 153.432837 L 94.176 152.852651 L 97.524 160.072744 L 100.872 161.813302 L 104.22 159.234698 L 107.568 162.909209 L 110.916 162.522419 L 114.264 160.008279 L 117.612 167.357302 L 120.96 155.689116 L 124.308 173.739349 L 127.656 174.706326 L 131.004 165.552279 L 134.352 171.225209 L 137.7 184.440558 L 141.048 197.333581 L 144.396 193.336744 L 147.744 196.173209 L 151.092 194.239256 L 154.44 179.79907 L 157.788 202.555256 L 161.136 210.742326 L 164.484 206.165302 L 167.832 223.893209 L 171.18 205.069395 L 174.528 223.893209 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #8c564b; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_64">    <path d="M 54 127.26 L 57.348 122.811907 L 60.696 123.102 L 64.044 122.135023 L 67.392 123.424326 L 70.74 123.23093 L 74.088 122.586279 L 77.436 124.584698 L 80.784 124.74586 L 84.132 126.679814 L 87.48 131.965953 L 90.828 147.985535 L 94.176 152.046837 L 97.524 160.26614 L 100.872 162.425721 L 104.22 165.648977 L 107.568 169.678047 L 110.916 176.833674 L 114.264 179.089953 L 117.612 184.150465 L 120.96 186.019953 L 124.308 189.952326 L 127.656 191.757349 L 131.004 192.015209 L 134.352 199.46093 L 137.7 196.785628 L 141.048 209.581953 L 144.396 221.443535 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pc1f59d1ee6)" style="fill: none; stroke: #000000; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="patch_3">    <path d="M 54 224.28 L 54 30.24 " style="fill: none"/>   </g>   <g id="patch_4">    <path d="M 388.8 224.28 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="patch_5">    <path d="M 54 224.28 L 388.8 224.28 " style="fill: none"/>   </g>   <g id="patch_6">    <path d="M 54 30.24 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="text_16">    <g id="patch_7">     <a xlink:href="https://dynomight.net/chess">      <path d="M 323.8664 225.676606 L 388.881244 225.676606 L 388.881244 213.343012 L 323.8664 213.343012 z" style="fill: #ffffff; opacity: 0.000001; stroke: #ffffff; stroke-linejoin: miter"/>     </a>    </g>    <a xlink:href="https://dynomight.net/chess">     <!-- dynomight.net/chess -->     <g style="opacity: 0.3" transform="translate(327.8664 220.734419) scale(0.05 -0.05)">      <defs>       <path id="CourierNewPSMT-64" d="M 3206 3922 L 3206 263 L 3556 263 Q 3653 263 3693 227 Q 3734 191 3734 131 Q 3734 75 3693 37 Q 3653 0 3556 0 L 2941 0 L 2941 522 Q 2484 -103 1781 -103 Q 1425 -103 1098 86 Q 772 275 583 625 Q 394 975 394 1347 Q 394 1722 583 2070 Q 772 2419 1098 2609 Q 1425 2800 1784 2800 Q 2472 2800 2941 2175 L 2941 3656 L 2591 3656 Q 2494 3656 2453 3693 Q 2413 3731 2413 3791 Q 2413 3847 2453 3884 Q 2494 3922 2591 3922 L 3206 3922 zM 2941 1347 Q 2941 1847 2603 2190 Q 2266 2534 1800 2534 Q 1331 2534 993 2190 Q 656 1847 656 1347 Q 656 850 993 504 Q 1331 159 1800 159 Q 2266 159 2603 504 Q 2941 850 2941 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-79" d="M 1938 0 L 719 2441 L 641 2441 Q 547 2441 506 2478 Q 466 2516 466 2572 Q 466 2613 486 2645 Q 506 2678 539 2692 Q 572 2706 641 2706 L 1359 2706 Q 1453 2706 1493 2668 Q 1534 2631 1534 2572 Q 1534 2516 1493 2478 Q 1453 2441 1359 2441 L 1006 2441 L 2078 288 L 3134 2441 L 2781 2441 Q 2688 2441 2647 2478 Q 2606 2516 2606 2575 Q 2606 2631 2647 2668 Q 2688 2706 2781 2706 L 3497 2706 Q 3594 2706 3634 2668 Q 3675 2631 3675 2572 Q 3675 2531 3650 2497 Q 3625 2463 3594 2452 Q 3563 2441 3419 2441 L 1756 -944 L 2166 -944 Q 2259 -944 2300 -980 Q 2341 -1016 2341 -1075 Q 2341 -1131 2300 -1168 Q 2259 -1206 2166 -1206 L 653 -1206 Q 559 -1206 518 -1170 Q 478 -1134 478 -1075 Q 478 -1016 518 -980 Q 559 -944 653 -944 L 1475 -944 L 1938 0 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6e" d="M 1072 2706 L 1072 2309 Q 1347 2588 1569 2694 Q 1791 2800 2069 2800 Q 2369 2800 2616 2672 Q 2791 2578 2933 2361 Q 3075 2144 3075 1916 L 3075 263 L 3297 263 Q 3391 263 3431 227 Q 3472 191 3472 131 Q 3472 75 3431 37 Q 3391 0 3297 0 L 2594 0 Q 2497 0 2456 37 Q 2416 75 2416 131 Q 2416 191 2456 227 Q 2497 263 2594 263 L 2813 263 L 2813 1872 Q 2813 2150 2609 2342 Q 2406 2534 2066 2534 Q 1806 2534 1615 2429 Q 1425 2325 1072 1909 L 1072 263 L 1369 263 Q 1463 263 1503 227 Q 1544 191 1544 131 Q 1544 75 1503 37 Q 1463 0 1369 0 L 513 0 Q 419 0 378 37 Q 338 75 338 131 Q 338 191 378 227 Q 419 263 513 263 L 809 263 L 809 2441 L 588 2441 Q 494 2441 453 2478 Q 413 2516 413 2575 Q 413 2631 453 2668 Q 494 2706 588 2706 L 1072 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6f" d="M 3388 1347 Q 3388 747 2958 322 Q 2528 -103 1922 -103 Q 1309 -103 881 323 Q 453 750 453 1347 Q 453 1947 881 2373 Q 1309 2800 1922 2800 Q 2528 2800 2958 2375 Q 3388 1950 3388 1347 zM 3122 1347 Q 3122 1841 2770 2187 Q 2419 2534 1919 2534 Q 1419 2534 1067 2186 Q 716 1838 716 1347 Q 716 859 1067 509 Q 1419 159 1919 159 Q 2419 159 2770 507 Q 3122 856 3122 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6d" d="M 725 2706 L 725 2441 Q 1059 2800 1397 2800 Q 1600 2800 1753 2692 Q 1906 2584 2009 2366 Q 2184 2584 2364 2692 Q 2544 2800 2725 2800 Q 3009 2800 3178 2616 Q 3400 2378 3400 2097 L 3400 263 L 3622 263 Q 3716 263 3756 227 Q 3797 191 3797 131 Q 3797 75 3756 37 Q 3716 0 3622 0 L 3138 0 L 3138 2072 Q 3138 2272 3016 2403 Q 2894 2534 2734 2534 Q 2591 2534 2431 2426 Q 2272 2319 2069 2003 L 2069 263 L 2288 263 Q 2381 263 2422 227 Q 2463 191 2463 131 Q 2463 75 2422 37 Q 2381 0 2288 0 L 1803 0 L 1803 2053 Q 1803 2263 1679 2398 Q 1556 2534 1403 2534 Q 1263 2534 1125 2444 Q 934 2316 725 2003 L 725 263 L 947 263 Q 1041 263 1081 227 Q 1122 191 1122 131 Q 1122 75 1081 37 Q 1041 0 947 0 L 241 0 Q 147 0 106 37 Q 66 75 66 131 Q 66 191 106 227 Q 147 263 241 263 L 463 263 L 463 2441 L 241 2441 Q 147 2441 106 2478 Q 66 2516 66 2575 Q 66 2631 106 2668 Q 147 2706 241 2706 L 725 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-69" d="M 2050 4053 L 2050 3375 L 1663 3375 L 1663 4053 L 2050 4053 zM 2059 2706 L 2059 263 L 3091 263 Q 3188 263 3228 227 Q 3269 191 3269 131 Q 3269 75 3228 37 Q 3188 0 3091 0 L 766 0 Q 672 0 631 37 Q 591 75 591 131 Q 591 191 631 227 Q 672 263 766 263 L 1797 263 L 1797 2441 L 1031 2441 Q 938 2441 895 2478 Q 853 2516 853 2572 Q 853 2631 893 2668 Q 934 2706 1031 2706 L 2059 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-67" d="M 2803 2225 L 2803 2706 L 3419 2706 Q 3513 2706 3553 2668 Q 3594 2631 3594 2572 Q 3594 2516 3553 2478 Q 3513 2441 3419 2441 L 3066 2441 L 3066 -181 Q 3066 -444 2953 -650 Q 2878 -788 2703 -934 Q 2528 -1081 2384 -1143 Q 2241 -1206 2000 -1206 L 1256 -1206 Q 1163 -1206 1122 -1170 Q 1081 -1134 1081 -1075 Q 1081 -1016 1122 -978 Q 1163 -941 1256 -941 L 2009 -944 Q 2241 -944 2427 -825 Q 2613 -706 2734 -478 Q 2803 -347 2803 -144 L 2803 644 Q 2381 66 1716 66 Q 1175 66 783 467 Q 391 869 391 1434 Q 391 2000 783 2400 Q 1175 2800 1716 2800 Q 2381 2800 2803 2225 zM 2803 1434 Q 2803 1897 2486 2215 Q 2169 2534 1728 2534 Q 1288 2534 972 2214 Q 656 1894 656 1434 Q 656 972 972 651 Q 1288 331 1728 331 Q 2169 331 2486 651 Q 2803 972 2803 1434 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-68" d="M 1056 3922 L 1056 2306 Q 1306 2578 1536 2689 Q 1766 2800 2050 2800 Q 2356 2800 2570 2692 Q 2784 2584 2928 2361 Q 3072 2138 3072 1891 L 3072 263 L 3366 263 Q 3463 263 3502 227 Q 3541 191 3541 131 Q 3541 75 3502 37 Q 3463 0 3366 0 L 2513 0 Q 2416 0 2375 37 Q 2334 75 2334 131 Q 2334 191 2375 227 Q 2416 263 2513 263 L 2806 263 L 2806 1872 Q 2806 2156 2601 2347 Q 2397 2538 2031 2538 Q 1744 2538 1541 2397 Q 1394 2297 1056 1925 L 1056 263 L 1353 263 Q 1447 263 1487 227 Q 1528 191 1528 131 Q 1528 75 1487 37 Q 1447 0 1353 0 L 497 0 Q 403 0 362 37 Q 322 75 322 131 Q 322 191 362 227 Q 403 263 497 263 L 794 263 L 794 3656 L 441 3656 Q 347 3656 306 3693 Q 266 3731 266 3791 Q 266 3847 306 3884 Q 347 3922 441 3922 L 1056 3922 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-74" d="M 1381 2706 L 2806 2706 Q 2900 2706 2940 2668 Q 2981 2631 2981 2572 Q 2981 2516 2940 2478 Q 2900 2441 2806 2441 L 1381 2441 L 1381 694 Q 1381 466 1564 312 Q 1747 159 2100 159 Q 2366 159 2675 239 Q 2984 319 3156 419 Q 3219 459 3259 459 Q 3309 459 3346 420 Q 3384 381 3384 328 Q 3384 281 3344 241 Q 3244 138 2858 17 Q 2472 -103 2119 -103 Q 1659 -103 1387 112 Q 1116 328 1116 694 L 1116 2441 L 631 2441 Q 538 2441 497 2478 Q 456 2516 456 2575 Q 456 2631 497 2668 Q 538 2706 631 2706 L 1116 2706 L 1116 3481 Q 1116 3575 1153 3615 Q 1191 3656 1247 3656 Q 1306 3656 1343 3615 Q 1381 3575 1381 3481 L 1381 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2e" d="M 1866 753 L 1978 753 Q 2153 753 2278 629 Q 2403 506 2403 331 Q 2403 150 2276 28 Q 2150 -94 1978 -94 L 1866 -94 Q 1691 -94 1566 29 Q 1441 153 1441 328 Q 1441 509 1567 631 Q 1694 753 1866 753 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-65" d="M 3341 1291 L 666 1291 Q 734 781 1092 470 Q 1450 159 1978 159 Q 2272 159 2594 256 Q 2916 353 3119 513 Q 3178 559 3222 559 Q 3272 559 3309 520 Q 3347 481 3347 428 Q 3347 375 3297 325 Q 3147 169 2764 33 Q 2381 -103 1978 -103 Q 1303 -103 851 339 Q 400 781 400 1409 Q 400 1981 823 2390 Q 1247 2800 1872 2800 Q 2516 2800 2931 2379 Q 3347 1959 3341 1291 zM 3075 1556 Q 2997 1991 2664 2262 Q 2331 2534 1872 2534 Q 1413 2534 1081 2265 Q 750 1997 669 1556 L 3075 1556 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2f" d="M 3100 4113 L 988 -413 Q 934 -528 850 -528 Q 797 -528 758 -489 Q 719 -450 719 -403 Q 719 -366 750 -300 L 2859 4225 Q 2894 4297 2925 4319 Q 2956 4341 3000 4341 Q 3053 4341 3092 4302 Q 3131 4263 3131 4216 Q 3131 4181 3100 4113 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-63" d="M 3016 2441 L 3016 2528 Q 3016 2625 3053 2665 Q 3091 2706 3147 2706 Q 3206 2706 3243 2665 Q 3281 2625 3281 2528 L 3281 1934 Q 3278 1838 3242 1797 Q 3206 1756 3147 1756 Q 3094 1756 3058 1792 Q 3022 1828 3016 1913 Q 2997 2134 2723 2334 Q 2450 2534 1988 2534 Q 1403 2534 1100 2168 Q 797 1803 797 1331 Q 797 822 1131 490 Q 1466 159 1997 159 Q 2303 159 2620 271 Q 2938 384 3194 634 Q 3259 697 3309 697 Q 3363 697 3398 661 Q 3434 625 3434 572 Q 3434 438 3119 231 Q 2609 -103 1984 -103 Q 1350 -103 942 301 Q 534 706 534 1328 Q 534 1963 951 2381 Q 1369 2800 2003 2800 Q 2606 2800 3016 2441 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-73" d="M 2788 2534 Q 2788 2625 2825 2665 Q 2863 2706 2919 2706 Q 2978 2706 3015 2665 Q 3053 2625 3053 2528 L 3053 2078 Q 3053 1984 3015 1943 Q 2978 1903 2919 1903 Q 2866 1903 2830 1937 Q 2794 1972 2788 2050 Q 2769 2238 2594 2359 Q 2338 2534 1916 2534 Q 1475 2534 1231 2356 Q 1047 2222 1047 2056 Q 1047 1869 1266 1744 Q 1416 1656 1834 1609 Q 2381 1550 2594 1475 Q 2897 1366 3045 1172 Q 3194 978 3194 753 Q 3194 419 2872 158 Q 2550 -103 1928 -103 Q 1306 -103 909 213 Q 909 106 896 75 Q 884 44 851 22 Q 819 0 778 0 Q 722 0 684 40 Q 647 81 647 175 L 647 716 Q 647 809 683 850 Q 719 891 778 891 Q 834 891 873 852 Q 913 813 913 747 Q 913 603 984 506 Q 1094 356 1333 257 Q 1572 159 1919 159 Q 2431 159 2681 350 Q 2931 541 2931 753 Q 2931 997 2678 1144 Q 2422 1291 1933 1341 Q 1444 1391 1231 1472 Q 1019 1553 900 1715 Q 781 1878 781 2066 Q 781 2403 1112 2601 Q 1444 2800 1903 2800 Q 2447 2800 2788 2534 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#CourierNewPSMT-64"/>      <use xlink:href="#CourierNewPSMT-79" x="60.009766"/>      <use xlink:href="#CourierNewPSMT-6e" x="120.019531"/>      <use xlink:href="#CourierNewPSMT-6f" x="180.029297"/>      <use xlink:href="#CourierNewPSMT-6d" x="240.039062"/>      <use xlink:href="#CourierNewPSMT-69" x="300.048828"/>      <use xlink:href="#CourierNewPSMT-67" x="360.058594"/>      <use xlink:href="#CourierNewPSMT-68" x="420.068359"/>      <use xlink:href="#CourierNewPSMT-74" x="480.078125"/>      <use xlink:href="#CourierNewPSMT-2e" x="540.087891"/>      <use xlink:href="#CourierNewPSMT-6e" x="600.097656"/>      <use xlink:href="#CourierNewPSMT-65" x="660.107422"/>      <use xlink:href="#CourierNewPSMT-74" x="720.117188"/>      <use xlink:href="#CourierNewPSMT-2f" x="780.126953"/>      <use xlink:href="#CourierNewPSMT-63" x="840.136719"/>      <use xlink:href="#CourierNewPSMT-68" x="900.146484"/>      <use xlink:href="#CourierNewPSMT-65" x="960.15625"/>      <use xlink:href="#CourierNewPSMT-73" x="1020.166016"/>      <use xlink:href="#CourierNewPSMT-73" x="1080.175781"/>     </g>    </a>   </g>   <g id="text_17">    <!-- llama-3.1-70b-instruct vs stockfish -->    <g transform="translate(120.057188 24.24) scale(0.12 -0.12)">     <defs>      <path id="Montserrat-Regular-6c" d="M 634 4749 L 1088 4749 L 1088 0 L 634 0 L 634 4749 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6d" d="M 4832 3392 Q 5466 3392 5827 3027 Q 6189 2662 6189 1952 L 6189 0 L 5734 0 L 5734 1907 Q 5734 2432 5481 2707 Q 5229 2982 4768 2982 Q 4243 2982 3942 2659 Q 3642 2336 3642 1766 L 3642 0 L 3187 0 L 3187 1907 Q 3187 2432 2934 2707 Q 2682 2982 2214 2982 Q 1696 2982 1392 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2752 Q 1248 3059 1568 3225 Q 1888 3392 2304 3392 Q 2726 3392 3036 3213 Q 3347 3034 3501 2682 Q 3686 3014 4035 3203 Q 4384 3392 4832 3392 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-33" d="M 1856 2560 Q 2560 2522 2928 2179 Q 3296 1837 3296 1286 Q 3296 902 3110 604 Q 2925 307 2560 134 Q 2195 -38 1670 -38 Q 1184 -38 752 118 Q 320 275 58 538 L 282 902 Q 506 672 877 528 Q 1248 384 1670 384 Q 2221 384 2521 624 Q 2822 864 2822 1286 Q 2822 1702 2521 1939 Q 2221 2176 1613 2176 L 1293 2176 L 1293 2515 L 2534 4070 L 243 4070 L 243 4480 L 3123 4480 L 3123 4154 L 1856 2560 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-2e" d="M 678 -32 Q 538 -32 438 70 Q 339 173 339 320 Q 339 467 438 566 Q 538 666 678 666 Q 819 666 921 566 Q 1024 467 1024 320 Q 1024 173 921 70 Q 819 -32 678 -32 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-37" d="M 3494 4480 L 3494 4154 L 1619 0 L 1120 0 L 2963 4070 L 659 4070 L 659 3232 L 205 3232 L 205 4480 L 3494 4480 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-62" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1184 3827 793 Q 3610 403 3226 185 Q 2842 -32 2368 -32 Q 1946 -32 1610 147 Q 1274 326 1069 666 L 1069 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2726 Q 1299 3053 1628 3222 Q 1958 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2368 Q 3264 2669 2976 2832 Q 2688 2995 2336 2995 Q 1978 2995 1693 2832 Q 1408 2669 1245 2368 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-76" d="M 3462 3366 L 1965 0 L 1498 0 L 6 3366 L 480 3366 L 1741 499 L 3014 3366 L 3462 3366 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6f" d="M 2010 -32 Q 1523 -32 1132 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1132 3174 Q 1523 3392 2010 3392 Q 2496 3392 2886 3174 Q 3277 2957 3497 2566 Q 3718 2176 3718 1683 Q 3718 1190 3497 800 Q 3277 410 2886 189 Q 2496 -32 2010 -32 zM 2010 371 Q 2368 371 2653 534 Q 2938 698 3098 998 Q 3258 1299 3258 1683 Q 3258 2067 3098 2368 Q 2938 2669 2653 2832 Q 2368 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6b" d="M 1882 1690 L 1088 960 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 1530 L 3098 3366 L 3661 3366 L 2221 1990 L 3795 0 L 3238 0 L 1882 1690 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-68" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2778 Q 1274 3072 1600 3232 Q 1926 3392 2349 3392 z" transform="scale(0.015625)"/>     </defs>     <use xlink:href="#Montserrat-Regular-6c"/>     <use xlink:href="#Montserrat-Regular-6c" x="26.899994"/>     <use xlink:href="#Montserrat-Regular-61" x="53.799988"/>     <use xlink:href="#Montserrat-Regular-6d" x="112.799973"/>     <use xlink:href="#Montserrat-Regular-61" x="218.899963"/>     <use xlink:href="#Montserrat-Regular-2d" x="277.899948"/>     <use xlink:href="#Montserrat-Regular-33" x="316.099945"/>     <use xlink:href="#Montserrat-Regular-2e" x="372.499939"/>     <use xlink:href="#Montserrat-Regular-31" x="393.699936"/>     <use xlink:href="#Montserrat-Regular-2d" x="429.799927"/>     <use xlink:href="#Montserrat-Regular-37" x="467.999924"/>     <use xlink:href="#Montserrat-Regular-30" x="526.899918"/>     <use xlink:href="#Montserrat-Regular-62" x="593.099915"/>     <use xlink:href="#Montserrat-Regular-2d" x="660.899902"/>     <use xlink:href="#Montserrat-Regular-69" x="699.099899"/>     <use xlink:href="#Montserrat-Regular-6e" x="725.999893"/>     <use xlink:href="#Montserrat-Regular-73" x="793.699875"/>     <use xlink:href="#Montserrat-Regular-74" x="842.599869"/>     <use xlink:href="#Montserrat-Regular-72" x="883.19986"/>     <use xlink:href="#Montserrat-Regular-75" x="923.29985"/>     <use xlink:href="#Montserrat-Regular-63" x="990.599838"/>     <use xlink:href="#Montserrat-Regular-74" x="1046.899826"/>     <use xlink:href="#Montserrat-Regular-20" x="1087.499817"/>     <use xlink:href="#Montserrat-Regular-76" x="1113.699814"/>     <use xlink:href="#Montserrat-Regular-73" x="1167.899811"/>     <use xlink:href="#Montserrat-Regular-20" x="1216.799805"/>     <use xlink:href="#Montserrat-Regular-73" x="1242.999802"/>     <use xlink:href="#Montserrat-Regular-74" x="1291.899796"/>     <use xlink:href="#Montserrat-Regular-6f" x="1332.499786"/>     <use xlink:href="#Montserrat-Regular-63" x="1395.199783"/>     <use xlink:href="#Montserrat-Regular-6b" x="1451.499771"/>     <use xlink:href="#Montserrat-Regular-66" x="1511.599762"/>     <use xlink:href="#Montserrat-Regular-69" x="1545.499756"/>     <use xlink:href="#Montserrat-Regular-73" x="1572.39975"/>     <use xlink:href="#Montserrat-Regular-68" x="1621.299744"/>    </g>   </g>  </g> </g> <defs>  <clipPath id="pc1f59d1ee6">   <rect x="54" y="30.24" width="334.8" height="194.04"/>  </clipPath> </defs></svg>](https://dynomight.net/img/chess/llama-3.1-70b-instruct_stockfish.pdf)

[Terrible](https://lichess.org/uJu3RKRR).

Maybe there’s something wrong with the Llama models or datasets? So I tried [`Qwen-2.5-72b`](https://huggingface.co/Qwen/Qwen2.5-72B).

[<?xml version="1.0" encoding="utf-8" standalone="no"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"  "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg xmlns:xlink="http://www.w3.org/1999/xlink" width="432pt" height="252pt" viewBox="0 0 432 252" xmlns="http://www.w3.org/2000/svg" version="1.1"> <metadata>  <rdf:RDF xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:cc="http://creativecommons.org/ns#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">   <cc:Work>    <dc:type rdf:resource="http://purl.org/dc/dcmitype/StillImage"/>    <dc:date>2024-11-14T11:33:33.542939</dc:date>    <dc:format>image/svg+xml</dc:format>    <dc:creator>     <cc:Agent>      <dc:title>Matplotlib v3.8.4, https://matplotlib.org/</dc:title>     </cc:Agent>    </dc:creator>   </cc:Work>  </rdf:RDF> </metadata> <defs>  <style type="text/css">*{stroke-linejoin: round; stroke-linecap: butt}</style> </defs> <g id="figure_1">  <g id="patch_1">   <path d="M 0 252 L 432 252 L 432 0 L 0 0 z" style="fill: #ffffff"/>  </g>  <g id="axes_1">   <g id="patch_2">    <path d="M 54 224.28 L 388.8 224.28 L 388.8 30.24 L 54 30.24 z" style="fill: #ffffff"/>   </g>   <g id="matplotlib.axis_1">    <g id="xtick_1">     <g id="line2d_1"/>     <g id="text_1">      <!-- 0 -->      <g transform="translate(50.689844 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-30" d="M 2118 -38 Q 1600 -38 1193 237 Q 787 512 560 1027 Q 333 1542 333 2240 Q 333 2938 560 3453 Q 787 3968 1193 4243 Q 1600 4518 2118 4518 Q 2637 4518 3043 4243 Q 3450 3968 3680 3453 Q 3910 2938 3910 2240 Q 3910 1542 3680 1027 Q 3450 512 3043 237 Q 2637 -38 2118 -38 zM 2118 384 Q 2509 384 2806 601 Q 3104 819 3270 1235 Q 3437 1651 3437 2240 Q 3437 2829 3270 3245 Q 3104 3661 2806 3878 Q 2509 4096 2118 4096 Q 1728 4096 1430 3878 Q 1133 3661 966 3245 Q 800 2829 800 2240 Q 800 1651 966 1235 Q 1133 819 1430 601 Q 1728 384 2118 384 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="xtick_2">     <g id="line2d_2"/>     <g id="text_2">      <!-- 20 -->      <g transform="translate(114.81 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-32" d="M 3392 410 L 3392 0 L 250 0 L 250 326 L 2112 2150 Q 2464 2496 2589 2749 Q 2714 3002 2714 3258 Q 2714 3654 2442 3875 Q 2170 4096 1664 4096 Q 877 4096 442 3597 L 115 3878 Q 378 4186 784 4352 Q 1190 4518 1702 4518 Q 2387 4518 2787 4195 Q 3187 3872 3187 3309 Q 3187 2963 3033 2643 Q 2880 2323 2451 1907 L 928 410 L 3392 410 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-32"/>       <use xlink:href="#Montserrat-Regular-30" x="56.799988"/>      </g>     </g>    </g>    <g id="xtick_3">     <g id="line2d_3"/>     <g id="text_3">      <!-- 40 -->      <g transform="translate(181.305156 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-34" d="M 4141 1178 L 3245 1178 L 3245 0 L 2784 0 L 2784 1178 L 256 1178 L 256 1510 L 2630 4480 L 3142 4480 L 851 1587 L 2797 1587 L 2797 2624 L 3245 2624 L 3245 1587 L 4141 1587 L 4141 1178 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-34"/>       <use xlink:href="#Montserrat-Regular-30" x="66.099991"/>      </g>     </g>    </g>    <g id="xtick_4">     <g id="line2d_4"/>     <g id="text_4">      <!-- 60 -->      <g transform="translate(248.524531 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-36" d="M 2189 2682 Q 2630 2682 2972 2515 Q 3315 2349 3507 2045 Q 3699 1741 3699 1338 Q 3699 928 3500 614 Q 3302 301 2953 131 Q 2605 -38 2170 -38 Q 1280 -38 806 547 Q 333 1133 333 2202 Q 333 2950 589 3468 Q 845 3987 1305 4252 Q 1766 4518 2381 4518 Q 3046 4518 3443 4275 L 3258 3904 Q 2938 4115 2387 4115 Q 1645 4115 1222 3644 Q 800 3174 800 2278 Q 800 2118 819 1914 Q 986 2278 1350 2480 Q 1715 2682 2189 2682 zM 2150 352 Q 2637 352 2941 617 Q 3245 883 3245 1325 Q 3245 1766 2941 2028 Q 2637 2291 2118 2291 Q 1779 2291 1520 2163 Q 1261 2035 1113 1811 Q 966 1587 966 1312 Q 966 1056 1104 838 Q 1242 621 1510 486 Q 1779 352 2150 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-36"/>       <use xlink:href="#Montserrat-Regular-30" x="60.899994"/>      </g>     </g>    </g>    <g id="xtick_5">     <g id="line2d_5"/>     <g id="text_5">      <!-- 80 -->      <g transform="translate(315.34 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-38" d="M 2899 2349 Q 3322 2208 3549 1926 Q 3776 1645 3776 1248 Q 3776 858 3565 566 Q 3354 275 2960 118 Q 2566 -38 2035 -38 Q 1229 -38 768 307 Q 307 653 307 1248 Q 307 1651 528 1929 Q 749 2208 1171 2349 Q 826 2483 646 2736 Q 467 2989 467 3334 Q 467 3878 892 4198 Q 1318 4518 2035 4518 Q 2509 4518 2864 4374 Q 3219 4230 3417 3961 Q 3616 3693 3616 3334 Q 3616 2989 3430 2736 Q 3245 2483 2899 2349 zM 928 3322 Q 928 2950 1222 2732 Q 1517 2515 2035 2515 Q 2554 2515 2851 2732 Q 3149 2950 3149 3315 Q 3149 3693 2845 3910 Q 2541 4128 2035 4128 Q 1523 4128 1225 3910 Q 928 3693 928 3322 zM 2035 352 Q 2630 352 2966 595 Q 3302 838 3302 1254 Q 3302 1670 2966 1910 Q 2630 2150 2035 2150 Q 1446 2150 1110 1910 Q 774 1670 774 1254 Q 774 832 1107 592 Q 1440 352 2035 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-38"/>       <use xlink:href="#Montserrat-Regular-30" x="63.799988"/>      </g>     </g>    </g>    <g id="xtick_6">     <g id="line2d_6"/>     <g id="text_6">      <!-- 100 -->      <g transform="translate(380.375 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-31" d="M 1574 4480 L 1574 0 L 1114 0 L 1114 4070 L 58 4070 L 58 4480 L 1574 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>      </g>     </g>    </g>    <g id="text_7">     <!-- turns -->     <g transform="translate(208.170312 248.56125) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-74" d="M 2406 205 Q 2278 90 2089 29 Q 1901 -32 1696 -32 Q 1222 -32 966 224 Q 710 480 710 947 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 4102 L 1165 4102 L 1165 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 973 Q 1165 672 1315 515 Q 1466 358 1747 358 Q 1888 358 2019 403 Q 2150 448 2246 531 L 2406 205 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-75" d="M 3667 3366 L 3667 0 L 3232 0 L 3232 614 Q 3053 307 2739 137 Q 2426 -32 2022 -32 Q 1363 -32 982 336 Q 602 704 602 1414 L 602 3366 L 1056 3366 L 1056 1459 Q 1056 928 1318 653 Q 1581 378 2067 378 Q 2598 378 2905 701 Q 3213 1024 3213 1600 L 3213 3366 L 3667 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-72" d="M 1069 2707 Q 1229 3046 1545 3219 Q 1862 3392 2330 3392 L 2330 2950 L 2221 2957 Q 1690 2957 1389 2630 Q 1088 2304 1088 1715 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2707 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-6e" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2746 Q 1254 3053 1584 3222 Q 1914 3392 2349 3392 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-73" d="M 1536 -32 Q 1126 -32 752 86 Q 378 205 166 384 L 371 742 Q 582 576 902 470 Q 1222 365 1568 365 Q 2029 365 2249 509 Q 2470 653 2470 915 Q 2470 1101 2348 1206 Q 2227 1312 2041 1366 Q 1856 1421 1549 1472 Q 1139 1549 889 1629 Q 640 1709 464 1901 Q 288 2093 288 2432 Q 288 2854 640 3123 Q 992 3392 1619 3392 Q 1946 3392 2272 3305 Q 2598 3219 2810 3078 L 2611 2714 Q 2195 3002 1619 3002 Q 1184 3002 963 2848 Q 742 2694 742 2445 Q 742 2253 867 2137 Q 992 2022 1177 1968 Q 1363 1914 1690 1856 Q 2093 1779 2336 1702 Q 2579 1626 2752 1440 Q 2925 1254 2925 928 Q 2925 486 2557 227 Q 2189 -32 1536 -32 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-74"/>      <use xlink:href="#Montserrat-Regular-75" x="40.599991"/>      <use xlink:href="#Montserrat-Regular-72" x="107.899979"/>      <use xlink:href="#Montserrat-Regular-6e" x="147.999969"/>      <use xlink:href="#Montserrat-Regular-73" x="215.699951"/>     </g>    </g>   </g>   <g id="matplotlib.axis_2">    <g id="ytick_1">     <g id="line2d_7"/>     <g id="text_8">      <!-- LOSS -->      <g transform="translate(23.91875 227.667831) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-4c" d="M 730 4480 L 1203 4480 L 1203 410 L 3712 410 L 3712 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4f" d="M 2688 -38 Q 2022 -38 1481 259 Q 941 557 637 1075 Q 333 1594 333 2240 Q 333 2886 637 3404 Q 941 3923 1481 4220 Q 2022 4518 2688 4518 Q 3354 4518 3888 4224 Q 4422 3930 4729 3408 Q 5037 2886 5037 2240 Q 5037 1594 4729 1072 Q 4422 550 3888 256 Q 3354 -38 2688 -38 zM 2688 384 Q 3219 384 3648 624 Q 4077 864 4320 1289 Q 4563 1715 4563 2240 Q 4563 2765 4320 3190 Q 4077 3616 3648 3856 Q 3219 4096 2688 4096 Q 2157 4096 1725 3856 Q 1293 3616 1046 3190 Q 800 2765 800 2240 Q 800 1715 1046 1289 Q 1293 864 1725 624 Q 2157 384 2688 384 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-53" d="M 1965 -38 Q 1466 -38 1008 122 Q 550 282 301 544 L 486 909 Q 730 666 1133 515 Q 1536 365 1965 365 Q 2566 365 2867 585 Q 3168 806 3168 1158 Q 3168 1427 3005 1587 Q 2842 1747 2605 1833 Q 2368 1920 1946 2022 Q 1440 2150 1139 2268 Q 838 2387 624 2630 Q 410 2874 410 3290 Q 410 3629 589 3907 Q 768 4186 1139 4352 Q 1510 4518 2061 4518 Q 2445 4518 2813 4412 Q 3181 4307 3450 4122 L 3290 3744 Q 3008 3930 2688 4022 Q 2368 4115 2061 4115 Q 1472 4115 1174 3888 Q 877 3661 877 3302 Q 877 3034 1040 2870 Q 1203 2707 1449 2620 Q 1696 2534 2112 2432 Q 2605 2310 2909 2192 Q 3213 2074 3424 1834 Q 3635 1594 3635 1184 Q 3635 845 3452 566 Q 3270 288 2892 125 Q 2515 -38 1965 -38 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-4c"/>       <use xlink:href="#Montserrat-Regular-4f" x="58.899994"/>       <use xlink:href="#Montserrat-Regular-53" x="142.799988"/>       <use xlink:href="#Montserrat-Regular-53" x="204.299973"/>      </g>     </g>    </g>    <g id="ytick_2">     <g id="line2d_8"/>     <g id="text_9">      <!-- -1000 -->      <g transform="translate(23.209375 195.435273) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-2d" d="M 384 1933 L 2061 1933 L 2061 1536 L 384 1536 L 384 1933 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-31" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="74.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="140.499985"/>       <use xlink:href="#Montserrat-Regular-30" x="206.699982"/>      </g>     </g>    </g>    <g id="ytick_3">     <g id="line2d_9"/>     <g id="text_10">      <!-- -500 -->      <g transform="translate(27.779687 163.202714) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-35" d="M 1510 2643 Q 2483 2643 2928 2294 Q 3373 1946 3373 1318 Q 3373 928 3190 621 Q 3008 314 2643 138 Q 2278 -38 1747 -38 Q 1267 -38 835 118 Q 403 275 141 538 L 365 902 Q 589 672 953 528 Q 1318 384 1741 384 Q 2298 384 2602 633 Q 2906 883 2906 1299 Q 2906 1760 2563 1997 Q 2221 2234 1408 2234 L 454 2234 L 685 4480 L 3136 4480 L 3136 4070 L 1088 4070 L 934 2643 L 1510 2643 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-35" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="94.799988"/>       <use xlink:href="#Montserrat-Regular-30" x="160.999985"/>      </g>     </g>    </g>    <g id="ytick_4">     <g id="line2d_10"/>     <g id="text_11">      <!-- 0 -->      <g transform="translate(43.879688 130.970156) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="ytick_5">     <g id="line2d_11"/>     <g id="text_12">      <!-- 500 -->      <g transform="translate(31.6 98.737598) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-35"/>       <use xlink:href="#Montserrat-Regular-30" x="56.599991"/>       <use xlink:href="#Montserrat-Regular-30" x="122.799988"/>      </g>     </g>    </g>    <g id="ytick_6">     <g id="line2d_12"/>     <g id="text_13">      <!-- 1000 -->      <g transform="translate(27.029687 66.50504) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="168.499985"/>      </g>     </g>    </g>    <g id="ytick_7">     <g id="line2d_13"/>     <g id="text_14">      <!-- WIN -->      <g transform="translate(28.240625 34.272482) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-57" d="M 6880 4480 L 5370 0 L 4870 0 L 3558 3821 L 2240 0 L 1747 0 L 237 4480 L 723 4480 L 2016 627 L 3354 4480 L 3795 4480 L 5114 608 L 6426 4480 L 6880 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-49" d="M 730 4480 L 1203 4480 L 1203 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4e" d="M 4474 4480 L 4474 0 L 4083 0 L 1203 3635 L 1203 0 L 730 0 L 730 4480 L 1120 4480 L 4006 845 L 4006 4480 L 4474 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-57"/>       <use xlink:href="#Montserrat-Regular-49" x="111.099991"/>       <use xlink:href="#Montserrat-Regular-4e" x="141.299988"/>      </g>     </g>    </g>    <g id="text_15">     <!-- centipawn difference -->     <g transform="translate(17.26875 180.677188) rotate(-90) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-63" d="M 2029 -32 Q 1530 -32 1136 185 Q 742 403 518 796 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1136 3174 Q 1530 3392 2029 3392 Q 2464 3392 2806 3222 Q 3149 3053 3347 2726 L 3008 2496 Q 2842 2746 2586 2870 Q 2330 2995 2029 2995 Q 1664 2995 1373 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1293 918 995 Q 1082 698 1373 534 Q 1664 371 2029 371 Q 2330 371 2586 492 Q 2842 614 3008 864 L 3347 634 Q 3149 307 2803 137 Q 2458 -32 2029 -32 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-65" d="M 3565 1542 L 749 1542 Q 787 1018 1152 694 Q 1517 371 2074 371 Q 2387 371 2649 483 Q 2912 595 3104 813 L 3360 518 Q 3136 250 2800 109 Q 2464 -32 2061 -32 Q 1542 -32 1142 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 508 2566 Q 723 2957 1097 3174 Q 1472 3392 1939 3392 Q 2406 3392 2777 3174 Q 3149 2957 3360 2569 Q 3571 2182 3571 1683 L 3565 1542 zM 1939 3002 Q 1453 3002 1123 2691 Q 794 2381 749 1882 L 3136 1882 Q 3091 2381 2761 2691 Q 2432 3002 1939 3002 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-69" d="M 634 3366 L 1088 3366 L 1088 0 L 634 0 L 634 3366 zM 864 4102 Q 723 4102 627 4198 Q 531 4294 531 4429 Q 531 4557 627 4653 Q 723 4749 864 4749 Q 1005 4749 1101 4656 Q 1197 4563 1197 4435 Q 1197 4294 1101 4198 Q 1005 4102 864 4102 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-70" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1178 3827 787 Q 3610 397 3229 182 Q 2848 -32 2368 -32 Q 1958 -32 1628 137 Q 1299 307 1088 634 L 1088 -1242 L 634 -1242 L 634 3366 L 1069 3366 L 1069 2701 Q 1274 3034 1610 3213 Q 1946 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2364 Q 3264 2662 2976 2828 Q 2688 2995 2336 2995 Q 1978 2995 1693 2828 Q 1408 2662 1245 2364 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-61" d="M 1811 3392 Q 2470 3392 2822 3062 Q 3174 2733 3174 2086 L 3174 0 L 2739 0 L 2739 525 Q 2586 262 2288 115 Q 1990 -32 1581 -32 Q 1018 -32 685 237 Q 352 506 352 947 Q 352 1376 662 1638 Q 973 1901 1651 1901 L 2720 1901 L 2720 2106 Q 2720 2541 2477 2768 Q 2234 2995 1766 2995 Q 1446 2995 1152 2889 Q 858 2784 646 2598 L 442 2938 Q 698 3155 1056 3273 Q 1414 3392 1811 3392 zM 1651 326 Q 2035 326 2310 502 Q 2586 678 2720 1011 L 2720 1562 L 1664 1562 Q 800 1562 800 960 Q 800 666 1024 496 Q 1248 326 1651 326 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-77" d="M 5574 3366 L 4301 0 L 3872 0 L 2810 2758 L 1747 0 L 1318 0 L 51 3366 L 486 3366 L 1542 506 L 2624 3366 L 3014 3366 L 4090 506 L 5158 3366 L 5574 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-20" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-64" d="M 3706 4749 L 3706 0 L 3270 0 L 3270 666 Q 3066 326 2730 147 Q 2394 -32 1971 -32 Q 1498 -32 1114 185 Q 730 403 512 793 Q 294 1184 294 1683 Q 294 2182 512 2572 Q 730 2963 1114 3177 Q 1498 3392 1971 3392 Q 2381 3392 2710 3222 Q 3040 3053 3251 2726 L 3251 4749 L 3706 4749 zM 2010 371 Q 2362 371 2646 534 Q 2931 698 3094 998 Q 3258 1299 3258 1683 Q 3258 2067 3094 2368 Q 2931 2669 2646 2832 Q 2362 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-66" d="M 1747 4397 Q 1453 4397 1302 4237 Q 1152 4077 1152 3763 L 1152 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 0 L 710 0 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 3782 Q 710 4243 976 4512 Q 1242 4781 1728 4781 Q 1920 4781 2099 4726 Q 2278 4672 2400 4570 L 2246 4230 Q 2048 4397 1747 4397 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-63"/>      <use xlink:href="#Montserrat-Regular-65" x="56.299988"/>      <use xlink:href="#Montserrat-Regular-6e" x="116.699982"/>      <use xlink:href="#Montserrat-Regular-74" x="184.399963"/>      <use xlink:href="#Montserrat-Regular-69" x="224.999954"/>      <use xlink:href="#Montserrat-Regular-70" x="251.899948"/>      <use xlink:href="#Montserrat-Regular-61" x="319.699936"/>      <use xlink:href="#Montserrat-Regular-77" x="378.699921"/>      <use xlink:href="#Montserrat-Regular-6e" x="466.599915"/>      <use xlink:href="#Montserrat-Regular-20" x="534.299896"/>      <use xlink:href="#Montserrat-Regular-64" x="560.499893"/>      <use xlink:href="#Montserrat-Regular-69" x="628.299881"/>      <use xlink:href="#Montserrat-Regular-66" x="655.199875"/>      <use xlink:href="#Montserrat-Regular-66" x="689.099869"/>      <use xlink:href="#Montserrat-Regular-65" x="722.999863"/>      <use xlink:href="#Montserrat-Regular-72" x="783.399857"/>      <use xlink:href="#Montserrat-Regular-65" x="823.499847"/>      <use xlink:href="#Montserrat-Regular-6e" x="883.899841"/>      <use xlink:href="#Montserrat-Regular-63" x="951.599823"/>      <use xlink:href="#Montserrat-Regular-65" x="1007.899811"/>     </g>    </g>   </g>   <g id="line2d_14">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 124.810326 L 64.044 123.649953 L 67.392 122.231721 L 70.74 121.58707 L 74.088 125.777302 L 77.436 124.101209 L 80.784 126.293023 L 84.132 112.626419 L 87.48 146.212744 L 90.828 156.591628 L 94.176 136.478512 L 97.524 109.596558 L 100.872 152.917116 L 104.22 181.410698 L 107.568 182.313209 L 110.916 191.016 L 114.264 207.90586 L 117.612 197.784837 L 120.96 198.945209 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_15">    <path d="M 54 127.26 L 57.348 123.843349 L 60.696 122.554047 L 64.044 123.327628 L 67.392 121.522605 L 70.74 123.327628 L 74.088 124.35907 L 77.436 122.554047 L 80.784 116.107535 L 84.132 120.749023 L 87.48 117.590233 L 90.828 121.135814 L 94.176 119.717581 L 97.524 123.198698 L 100.872 125.326047 L 104.22 124.23014 L 107.568 124.939256 L 110.916 125.648372 L 114.264 125.068186 L 117.612 124.74586 L 120.96 178.638698 L 124.308 181.088372 L 127.656 188.37293 L 131.004 183.022326 L 134.352 184.956279 L 137.7 196.173209 L 141.048 200.750233 L 144.396 203.522233 L 147.744 207.261209 L 151.092 203.264372 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_16">    <path d="M 54 127.26 L 57.348 119.330791 L 60.696 120.749023 L 64.044 123.778884 L 67.392 114.689302 L 70.74 115.076093 L 74.088 128.484837 L 77.436 129.90307 L 80.784 127.904651 L 84.132 165.036558 L 87.48 154.141953 L 90.828 167.744093 L 94.176 168.066419 L 97.524 176.382419 L 100.872 171.805395 L 104.22 170.580558 L 107.568 174.770791 L 110.916 190.564744 L 114.264 191.725116 L 117.612 202.684186 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_17">    <path d="M 54 127.26 L 57.348 125.132651 L 60.696 125.390512 L 64.044 123.907814 L 67.392 125.712837 L 70.74 125.648372 L 74.088 125.583907 L 77.436 127.195535 L 80.784 123.005302 L 84.132 121.780465 L 87.48 121.651535 L 90.828 123.907814 L 94.176 123.714419 L 97.524 122.618512 L 100.872 157.816465 L 104.22 150.918698 L 107.568 165.874605 L 110.916 190.500279 L 114.264 196.044279 L 117.612 195.721953 L 120.96 200.105581 L 124.308 203.457767 L 127.656 203.780093 L 131.004 203.199907 L 134.352 203.715628 L 137.7 202.36186 L 141.048 198.687349 L 144.396 193.078884 L 147.744 205.456186 L 151.092 202.877581 L 154.44 204.489209 L 157.788 199.912186 L 161.136 183.989302 L 164.484 204.102419 L 167.832 203.264372 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_18">    <path d="M 54 127.26 L 57.348 122.554047 L 60.696 120.168837 L 64.044 117.977023 L 67.392 119.395256 L 70.74 121.264744 L 74.088 118.879535 L 77.436 113.593395 L 80.784 119.524186 L 84.132 122.231721 L 87.48 126.937674 L 90.828 128.291442 L 94.176 153.110512 L 97.524 147.179721 L 100.872 154.72214 L 104.22 171.676465 L 107.568 160.588465 L 110.916 162.393488 L 114.264 170.000372 L 117.612 168.646605 L 120.96 164.391907 L 124.308 166.777116 L 127.656 167.744093 L 131.004 163.03814 L 134.352 156.720558 L 137.7 158.590047 L 141.048 152.981581 L 144.396 136.478512 L 147.744 177.800651 L 151.092 168.259814 L 154.44 178.896558 L 157.788 175.802233 L 161.136 174.64186 L 164.484 176.253488 L 167.832 180.701581 L 171.18 178.703163 L 174.528 173.610419 L 177.876 175.286512 L 181.224 181.604093 L 184.572 223.893209 L 187.92 223.893209 L 191.268 223.893209 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_19">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 123.972279 L 64.044 119.395256 L 67.392 115.656279 L 70.74 112.368558 L 74.088 112.755349 L 77.436 123.843349 L 80.784 125.261581 L 84.132 119.459721 L 87.48 128.807163 L 90.828 132.610605 L 94.176 137.703349 L 97.524 139.766233 L 100.872 129.451814 L 104.22 157.945395 L 107.568 180.508186 L 110.916 193.852465 L 114.264 198.300558 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_20">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 126.808744 L 64.044 112.239628 L 67.392 125.648372 L 70.74 124.488 L 74.088 126.421953 L 77.436 127.26 L 80.784 127.775721 L 84.132 127.582326 L 87.48 122.038326 L 90.828 106.050977 L 94.176 114.302512 L 97.524 121.329209 L 100.872 125.132651 L 104.22 114.173581 L 107.568 135.83386 L 110.916 160.459535 L 114.264 157.236279 L 117.612 159.234698 L 120.96 161.233116 L 124.308 152.723721 L 127.656 153.368372 L 131.004 154.464279 L 134.352 169.742512 L 137.7 168.517674 L 141.048 169.678047 L 144.396 183.409116 L 147.744 182.893395 L 151.092 181.604093 L 154.44 197.204651 L 157.788 183.086791 L 161.136 223.893209 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_21">    <path d="M 54 127.26 L 57.348 118.041488 L 60.696 124.810326 L 64.044 124.810326 L 67.392 125.132651 L 70.74 122.682977 L 74.088 123.649953 L 77.436 117.912558 L 80.784 161.555442 L 84.132 160.975256 L 87.48 174.706326 L 90.828 193.53014 L 94.176 198.107163 L 97.524 193.272279 L 100.872 190.75814 L 104.22 187.212558 L 107.568 181.668558 L 110.916 184.118233 L 114.264 187.405953 L 117.612 196.366605 L 120.96 200.170047 L 124.308 194.819442 L 127.656 192.305302 L 131.004 196.81786 L 134.352 200.170047 L 137.7 197.011256 L 141.048 204.811535 L 144.396 208.034791 L 147.744 223.893209 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_22">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 124.423535 L 64.044 125.390512 L 67.392 125.003721 L 70.74 120.168837 L 74.088 115.398419 L 77.436 96.187814 L 80.784 94.576186 L 84.132 140.539814 L 87.48 138.60586 L 90.828 142.473767 L 94.176 147.759907 L 97.524 144.472186 L 100.872 146.40614 L 104.22 146.92186 L 107.568 150.016186 L 110.916 146.986326 L 114.264 130.28986 L 117.612 159.170233 L 120.96 133.577581 L 124.308 149.758326 L 127.656 162.071163 L 131.004 194.626047 L 134.352 183.795907 L 137.7 223.893209 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_23">    <path d="M 54 127.26 L 57.348 117.461302 L 60.696 117.848093 L 64.044 125.326047 L 67.392 127.324465 L 70.74 128.484837 L 74.088 134.222233 L 77.436 166.583721 L 80.784 177.41386 L 84.132 177.800651 L 87.48 171.998791 L 90.828 168.775535 L 94.176 155.431256 L 97.524 163.811721 L 100.872 158.461116 L 104.22 155.495721 L 107.568 182.764465 L 110.916 177.671721 L 114.264 179.992465 L 117.612 183.280186 L 120.96 177.736186 L 124.308 184.827349 L 127.656 189.339907 L 131.004 193.465674 L 134.352 179.605674 L 137.7 190.500279 L 141.048 195.206233 L 144.396 223.893209 L 147.744 223.893209 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_24">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 117.590233 L 64.044 122.102791 L 67.392 119.846512 L 70.74 123.134233 L 74.088 151.95014 L 77.436 150.725302 L 80.784 150.725302 L 84.132 153.110512 L 87.48 152.981581 L 90.828 152.659256 L 94.176 163.102605 L 97.524 157.300744 L 100.872 168.324279 L 104.22 180.18586 L 107.568 181.604093 L 110.916 182.184279 L 114.264 184.956279 L 117.612 195.399628 L 120.96 195.206233 L 124.308 201.265953 L 127.656 196.044279 L 131.004 196.56 L 134.352 194.883907 L 137.7 195.141767 L 141.048 184.376093 L 144.396 198.816279 L 147.744 194.04586 L 151.092 154.528744 L 154.44 177.929581 L 157.788 194.303721 L 161.136 200.685767 L 164.484 206.809953 L 167.832 201.910605 L 171.18 210.742326 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_25">    <path d="M 54 127.26 L 57.348 120.877953 L 60.696 121.200279 L 64.044 119.717581 L 67.392 117.074512 L 70.74 118.170419 L 74.088 118.234884 L 77.436 120.555628 L 80.784 143.311814 L 84.132 150.338512 L 87.48 152.07907 L 90.828 161.555442 L 94.176 163.682791 L 97.524 155.624651 L 100.872 160.588465 L 104.22 157.429674 L 107.568 180.443721 L 110.916 180.508186 L 114.264 187.792744 L 117.612 192.692093 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_26">    <path d="M 54 127.26 L 57.348 119.846512 L 60.696 115.205023 L 64.044 122.102791 L 67.392 124.74586 L 70.74 126.421953 L 74.088 122.231721 L 77.436 120.039907 L 80.784 132.54614 L 84.132 128.936093 L 87.48 125.583907 L 90.828 131.965953 L 94.176 134.480093 L 97.524 141.506791 L 100.872 144.665581 L 104.22 145.310233 L 107.568 150.338512 L 110.916 149.500465 L 114.264 157.49414 L 117.612 165.616744 L 120.96 159.041302 L 124.308 174.64186 L 127.656 180.05693 L 131.004 191.27386 L 134.352 179.928 L 137.7 190.371349 L 141.048 223.893209 L 144.396 223.893209 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_27">    <path d="M 54 127.26 L 57.348 125.132651 L 60.696 125.003721 L 64.044 126.421953 L 67.392 115.011628 L 70.74 114.173581 L 74.088 115.91414 L 77.436 109.918884 L 80.784 122.489581 L 84.132 121.58707 L 87.48 121.006884 L 90.828 124.552465 L 94.176 118.041488 L 97.524 122.747442 L 100.872 123.392093 L 104.22 131.772558 L 107.568 131.708093 L 110.916 131.772558 L 114.264 155.946977 L 117.612 161.104186 L 120.96 160.137209 L 124.308 163.811721 L 127.656 179.67014 L 131.004 189.146512 L 134.352 185.60093 L 137.7 185.085209 L 141.048 186.438977 L 144.396 185.536465 L 147.744 171.096279 L 151.092 174.12614 L 154.44 183.344651 L 157.788 190.306884 L 161.136 201.97507 L 164.484 201.394884 L 167.832 197.978233 L 171.18 194.819442 L 174.528 192.627628 L 177.876 197.011256 L 181.224 189.082047 L 184.572 191.854047 L 187.92 191.338326 L 191.268 191.596186 L 194.616 193.788 L 197.964 188.37293 L 201.312 198.945209 L 204.66 191.209395 L 208.008 199.138605 L 211.356 223.893209 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_28">    <path d="M 54 127.26 L 57.348 126.744279 L 60.696 127.13107 L 64.044 124.874791 L 67.392 126.937674 L 70.74 126.421953 L 74.088 125.326047 L 77.436 123.069767 L 80.784 122.360651 L 84.132 120.813488 L 87.48 120.555628 L 90.828 157.042884 L 94.176 150.660837 L 97.524 157.042884 L 100.872 159.234698 L 104.22 159.814884 L 107.568 166.19693 L 110.916 166.261395 L 114.264 165.616744 L 117.612 161.813302 L 120.96 160.910791 L 124.308 165.81014 L 127.656 168.904465 L 131.004 174.899721 L 134.352 182.119814 L 137.7 190.049023 L 141.048 187.405953 L 144.396 223.764279 L 147.744 223.828744 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_29">    <path d="M 54 127.26 L 57.348 120.877953 L 60.696 122.876372 L 64.044 124.165674 L 67.392 123.392093 L 70.74 118.363814 L 74.088 114.495907 L 77.436 119.588651 L 80.784 119.266326 L 84.132 123.714419 L 87.48 117.074512 L 90.828 120.233302 L 94.176 120.491163 L 97.524 118.492744 L 100.872 123.649953 L 104.22 129.838605 L 107.568 132.030419 L 110.916 155.624651 L 114.264 145.052372 L 117.612 149.887256 L 120.96 149.69386 L 124.308 190.88707 L 127.656 198.042698 L 131.004 223.893209 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_30">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 124.165674 L 64.044 124.036744 L 67.392 125.261581 L 70.74 122.167256 L 74.088 122.682977 L 77.436 131.127907 L 80.784 123.843349 L 84.132 126.035163 L 87.48 188.50186 L 90.828 174.12614 L 94.176 184.698419 L 97.524 190.177953 L 100.872 186.374512 L 104.22 191.854047 L 107.568 186.052186 L 110.916 193.465674 L 114.264 197.526977 L 117.612 201.717209 L 120.96 193.336744 L 124.308 204.231349 L 127.656 204.553674 L 131.004 207.51907 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_31">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 124.488 L 64.044 124.552465 L 67.392 125.519442 L 70.74 124.036744 L 74.088 123.714419 L 77.436 124.810326 L 80.784 123.327628 L 84.132 123.327628 L 87.48 123.327628 L 90.828 125.906233 L 94.176 124.552465 L 97.524 150.596372 L 100.872 154.72214 L 104.22 156.527163 L 107.568 149.951721 L 110.916 148.855814 L 114.264 152.723721 L 117.612 155.495721 L 120.96 161.297581 L 124.308 152.401395 L 127.656 150.402977 L 131.004 148.920279 L 134.352 143.182884 L 137.7 146.470605 L 141.048 144.085395 L 144.396 152.594791 L 147.744 163.231535 L 151.092 167.421767 L 154.44 167.873023 L 157.788 177.349395 L 161.136 178.961023 L 164.484 173.997209 L 167.832 174.64186 L 171.18 190.049023 L 174.528 190.371349 L 177.876 196.495535 L 181.224 189.533302 L 184.572 189.468837 L 187.92 196.237674 L 191.268 195.399628 L 194.616 197.269116 L 197.964 187.792744 L 201.312 187.470419 L 204.66 223.893209 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_32">    <path d="M 54 127.26 L 57.348 121.200279 L 60.696 120.813488 L 64.044 120.233302 L 67.392 133.06186 L 70.74 134.931349 L 74.088 148.211163 L 77.436 146.148279 L 80.784 169.420186 L 84.132 177.02707 L 87.48 176.060093 L 90.828 173.352558 L 94.176 178.703163 L 97.524 186.438977 L 100.872 183.924837 L 104.22 178.380837 L 107.568 183.731442 L 110.916 191.402791 L 114.264 185.020744 L 117.612 185.794326 L 120.96 188.630791 L 124.308 191.338326 L 127.656 194.626047 L 131.004 204.424744 L 134.352 206.036372 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_33">    <path d="M 54 127.26 L 57.348 124.810326 L 60.696 120.426698 L 64.044 117.396837 L 67.392 111.143721 L 70.74 107.72707 L 74.088 104.890605 L 77.436 140.475349 L 80.784 142.602698 L 84.132 153.110512 L 87.48 159.492558 L 90.828 165.616744 L 94.176 195.270698 L 97.524 200.750233 L 100.872 208.808372 L 104.22 211.193581 L 107.568 202.684186 L 110.916 198.107163 L 114.264 201.459349 L 117.612 189.920093 L 120.96 203.780093 L 124.308 205.13386 L 127.656 208.679442 L 131.004 214.545767 L 134.352 212.031628 L 137.7 204.295814 L 141.048 209.710884 L 144.396 223.313023 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_34">    <path d="M 54 127.26 L 57.348 116.42986 L 60.696 121.200279 L 64.044 120.555628 L 67.392 118.750605 L 70.74 122.554047 L 74.088 122.425116 L 77.436 125.068186 L 80.784 121.264744 L 84.132 117.396837 L 87.48 140.604279 L 90.828 138.348 L 94.176 152.659256 L 97.524 154.141953 L 100.872 159.879349 L 104.22 165.423349 L 107.568 166.777116 L 110.916 161.684372 L 114.264 163.55386 L 117.612 155.173395 L 120.96 152.401395 L 124.308 167.228372 L 127.656 166.261395 L 131.004 165.681209 L 134.352 157.107349 L 137.7 165.874605 L 141.048 172.450047 L 144.396 166.32586 L 147.744 154.85107 L 151.092 171.998791 L 154.44 223.828744 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_35">    <path d="M 54 127.26 L 57.348 122.554047 L 60.696 125.068186 L 64.044 125.648372 L 67.392 116.687721 L 70.74 127.00214 L 74.088 127.646791 L 77.436 152.852651 L 80.784 151.821209 L 84.132 159.492558 L 87.48 156.140372 L 90.828 160.846326 L 94.176 130.805581 L 97.524 166.583721 L 100.872 165.101023 L 104.22 176.446884 L 107.568 182.506605 L 110.916 182.57107 L 114.264 186.245581 L 117.612 180.379256 L 120.96 172.063256 L 124.308 184.311628 L 127.656 189.984558 L 131.004 193.143349 L 134.352 196.366605 L 137.7 196.81786 L 141.048 199.976651 L 144.396 201.459349 L 147.744 199.267535 L 151.092 223.893209 L 154.44 223.893209 L 157.788 223.893209 L 161.136 223.893209 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_36">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 124.74586 L 64.044 123.585488 L 67.392 123.198698 L 70.74 123.392093 L 74.088 121.651535 L 77.436 125.841767 L 80.784 124.552465 L 84.132 129.645209 L 87.48 131.708093 L 90.828 140.604279 L 94.176 150.080651 L 97.524 169.484651 L 100.872 169.162326 L 104.22 172.514512 L 107.568 178.380837 L 110.916 183.086791 L 114.264 187.019163 L 117.612 189.468837 L 120.96 188.37293 L 124.308 181.861953 L 127.656 183.086791 L 131.004 176.060093 L 134.352 168.58214 L 137.7 175.802233 L 141.048 186.116651 L 144.396 187.857209 L 147.744 188.308465 L 151.092 194.110326 L 154.44 194.948372 L 157.788 196.946791 L 161.136 193.143349 L 164.484 210.67786 L 167.832 204.74707 L 171.18 206.165302 L 174.528 205.327256 L 177.876 205.520651 L 181.224 199.009674 L 184.572 208.421581 L 187.92 210.355535 L 191.268 207.067814 L 194.616 206.229767 L 197.964 202.426326 L 201.312 204.940465 L 204.66 210.613395 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_37">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 124.294605 L 64.044 120.877953 L 67.392 121.45814 L 70.74 107.211349 L 74.088 122.167256 L 77.436 124.488 L 80.784 126.035163 L 84.132 122.747442 L 87.48 123.198698 L 90.828 125.583907 L 94.176 127.13107 L 97.524 136.671907 L 100.872 141.571256 L 104.22 142.796093 L 107.568 160.975256 L 110.916 158.00986 L 114.264 181.023907 L 117.612 168.195349 L 120.96 171.096279 L 124.308 201.137023 L 127.656 204.74707 L 131.004 204.231349 L 134.352 196.495535 L 137.7 193.465674 L 141.048 205.778512 L 144.396 203.393302 L 147.744 204.811535 L 151.092 201.97507 L 154.44 204.876 L 157.788 214.352372 L 161.136 223.893209 L 164.484 223.893209 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_38">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 123.843349 L 64.044 98.766419 L 67.392 118.299349 L 70.74 104.632744 L 74.088 126.679814 L 77.436 128.936093 L 80.784 103.536837 L 84.132 157.365209 L 87.48 144.085395 L 90.828 186.761302 L 94.176 193.272279 L 97.524 197.333581 L 100.872 223.893209 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_39">    <path d="M 54 127.26 L 57.348 121.393674 L 60.696 119.330791 L 64.044 123.649953 L 67.392 120.491163 L 70.74 125.197116 L 74.088 122.554047 L 77.436 121.393674 L 80.784 125.519442 L 84.132 125.583907 L 87.48 129.580744 L 90.828 164.391907 L 94.176 152.208 L 97.524 142.215907 L 100.872 158.332186 L 104.22 162.329023 L 107.568 164.907628 L 110.916 165.681209 L 114.264 164.907628 L 117.612 172.901302 L 120.96 168.646605 L 124.308 166.390326 L 127.656 161.942233 L 131.004 161.426512 L 134.352 169.226791 L 137.7 168.71107 L 141.048 176.124558 L 144.396 190.75814 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_40">    <path d="M 54 127.26 L 57.348 121.135814 L 60.696 121.522605 L 64.044 119.782047 L 67.392 123.005302 L 70.74 121.716 L 74.088 115.785209 L 77.436 131.837023 L 80.784 159.685953 L 84.132 175.028651 L 87.48 191.918512 L 90.828 183.924837 L 94.176 198.687349 L 97.524 197.720372 L 100.872 223.957674 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_41">    <path d="M 54 127.26 L 57.348 121.393674 L 60.696 115.011628 L 64.044 123.972279 L 67.392 124.101209 L 70.74 125.970698 L 74.088 123.392093 L 77.436 124.165674 L 80.784 124.74586 L 84.132 106.502233 L 87.48 117.138977 L 90.828 138.154605 L 94.176 144.02093 L 97.524 159.234698 L 100.872 178.961023 L 104.22 180.894977 L 107.568 182.635535 L 110.916 178.832093 L 114.264 186.116651 L 117.612 186.696837 L 120.96 184.698419 L 124.308 198.816279 L 127.656 198.236093 L 131.004 186.245581 L 134.352 223.893209 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_42">    <path d="M 54 127.26 L 57.348 124.874791 L 60.696 123.585488 L 64.044 119.717581 L 67.392 118.879535 L 70.74 118.68614 L 74.088 120.813488 L 77.436 127.324465 L 80.784 127.775721 L 84.132 130.032 L 87.48 126.357488 L 90.828 126.357488 L 94.176 128.936093 L 97.524 125.906233 L 100.872 123.005302 L 104.22 125.197116 L 107.568 160.846326 L 110.916 168.517674 L 114.264 173.739349 L 117.612 172.643442 L 120.96 166.906047 L 124.308 186.374512 L 127.656 186.116651 L 131.004 178.058512 L 134.352 189.146512 L 137.7 185.858791 L 141.048 186.696837 L 144.396 187.728279 L 147.744 185.472 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_43">    <path d="M 54 127.26 L 57.348 123.778884 L 60.696 123.843349 L 64.044 125.906233 L 67.392 124.488 L 70.74 124.101209 L 74.088 127.324465 L 77.436 124.23014 L 80.784 127.00214 L 84.132 127.066605 L 87.48 125.326047 L 90.828 140.088558 L 94.176 138.025674 L 97.524 152.852651 L 100.872 158.138791 L 104.22 173.094698 L 107.568 175.415442 L 110.916 173.417023 L 114.264 169.484651 L 117.612 183.280186 L 120.96 175.802233 L 124.308 160.846326 L 127.656 175.028651 L 131.004 181.990884 L 134.352 179.218884 L 137.7 181.539628 L 141.048 182.82893 L 144.396 184.891814 L 147.744 185.21414 L 151.092 183.538047 L 154.44 192.369767 L 157.788 187.98614 L 161.136 190.951535 L 164.484 188.824186 L 167.832 196.366605 L 171.18 196.366605 L 174.528 199.009674 L 177.876 223.828744 L 181.224 191.596186 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_44">    <path d="M 54 127.26 L 57.348 119.846512 L 60.696 121.97386 L 64.044 123.069767 L 67.392 126.357488 L 70.74 124.74586 L 74.088 118.68614 L 77.436 123.198698 L 80.784 145.632558 L 84.132 158.783442 L 87.48 157.558605 L 90.828 157.49414 L 94.176 149.629395 L 97.524 158.847907 L 100.872 162.651349 L 104.22 191.338326 L 107.568 200.105581 L 110.916 198.236093 L 114.264 196.108744 L 117.612 182.44214 L 120.96 195.335163 L 124.308 195.141767 L 127.656 200.170047 L 131.004 189.275442 L 134.352 201.330419 L 137.7 196.43107 L 141.048 198.365023 L 144.396 183.344651 L 147.744 200.234512 L 151.092 209.066233 L 154.44 200.492372 L 157.788 207.77693 L 161.136 207.003349 L 164.484 187.083628 L 167.832 188.759721 L 171.18 209.839814 L 174.528 201.265953 L 177.876 202.36186 L 181.224 203.457767 L 184.572 203.393302 L 187.92 198.429488 L 191.268 223.893209 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_45">    <path d="M 54 127.26 L 57.348 123.843349 L 60.696 123.843349 L 64.044 124.552465 L 67.392 124.552465 L 70.74 125.003721 L 74.088 124.488 L 77.436 127.195535 L 80.784 128.936093 L 84.132 133.770977 L 87.48 130.096465 L 90.828 147.953302 L 94.176 171.35414 L 97.524 171.35414 L 100.872 193.465674 L 104.22 196.366605 L 107.568 195.657488 L 110.916 195.593023 L 114.264 197.913767 L 117.612 197.913767 L 120.96 210.871256 L 124.308 201.330419 L 127.656 186.310047 L 131.004 210.484465 L 134.352 226.79414 L 137.7 217.640093 L 141.048 218.155814 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_46">    <path d="M 54 127.26 L 57.348 121.200279 L 60.696 121.909395 L 64.044 117.654698 L 67.392 121.97386 L 70.74 120.620093 L 74.088 117.719163 L 77.436 124.874791 L 80.784 125.003721 L 84.132 126.164093 L 87.48 123.392093 L 90.828 123.392093 L 94.176 129.580744 L 97.524 160.910791 L 100.872 163.03814 L 104.22 185.21414 L 107.568 191.467256 L 110.916 186.567907 L 114.264 185.472 L 117.612 181.023907 L 120.96 198.365023 L 124.308 223.893209 L 127.656 205.327256 L 131.004 210.613395 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_47">    <path d="M 54 127.26 L 57.348 124.35907 L 60.696 125.841767 L 64.044 122.876372 L 67.392 122.940837 L 70.74 125.970698 L 74.088 125.712837 L 77.436 122.296186 L 80.784 118.363814 L 84.132 122.618512 L 87.48 125.068186 L 90.828 122.940837 L 94.176 123.907814 L 97.524 118.492744 L 100.872 119.782047 L 104.22 121.135814 L 107.568 122.811907 L 110.916 138.670326 L 114.264 139.121581 L 117.612 135.060279 L 120.96 157.816465 L 124.308 160.846326 L 127.656 180.18586 L 131.004 175.157581 L 134.352 184.827349 L 137.7 205.262791 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_48">    <path d="M 54 127.26 L 57.348 124.874791 L 60.696 122.811907 L 64.044 119.975442 L 67.392 124.810326 L 70.74 124.810326 L 74.088 150.274047 L 77.436 164.456372 L 80.784 175.028651 L 84.132 160.39507 L 87.48 223.957674 L 90.828 223.957674 L 94.176 223.957674 L 97.524 223.957674 L 100.872 223.957674 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_49">    <path d="M 54 127.26 L 57.348 124.874791 L 60.696 123.649953 L 64.044 130.354326 L 67.392 148.662419 L 70.74 146.470605 L 74.088 152.788186 L 77.436 162.973674 L 80.784 169.678047 L 84.132 161.426512 L 87.48 157.88093 L 90.828 146.40614 L 94.176 158.654512 L 97.524 155.753581 L 100.872 161.684372 L 104.22 157.816465 L 107.568 183.924837 L 110.916 176.89814 L 114.264 191.467256 L 117.612 189.726698 L 120.96 201.588279 L 124.308 190.049023 L 127.656 194.303721 L 131.004 194.432651 L 134.352 195.012837 L 137.7 201.394884 L 141.048 209.517488 L 144.396 204.74707 L 147.744 205.971907 L 151.092 223.893209 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_50">    <path d="M 54 127.26 L 57.348 121.200279 L 60.696 114.689302 L 64.044 122.102791 L 67.392 121.071349 L 70.74 120.362233 L 74.088 119.524186 L 77.436 117.074512 L 80.784 120.362233 L 84.132 125.583907 L 87.48 123.327628 L 90.828 127.840186 L 94.176 128.226977 L 97.524 140.604279 L 100.872 125.132651 L 104.22 152.33693 L 107.568 173.545953 L 110.916 160.588465 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_51">    <path d="M 54 127.26 L 57.348 113.851256 L 60.696 129.516279 L 64.044 128.936093 L 67.392 121.071349 L 70.74 120.039907 L 74.088 153.819628 L 77.436 156.398233 L 80.784 152.594791 L 84.132 170.258233 L 87.48 174.25507 L 90.828 178.380837 L 94.176 178.961023 L 97.524 178.574233 L 100.872 180.894977 L 104.22 176.317953 L 107.568 177.671721 L 110.916 190.435814 L 114.264 194.561581 L 117.612 197.204651 L 120.96 179.928 L 124.308 192.176372 L 127.656 187.148093 L 131.004 187.405953 L 134.352 181.604093 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_52">    <path d="M 54 127.26 L 57.348 120.877953 L 60.696 121.522605 L 64.044 123.972279 L 67.392 123.327628 L 70.74 126.679814 L 74.088 124.874791 L 77.436 124.165674 L 80.784 126.486419 L 84.132 125.390512 L 87.48 125.583907 L 90.828 124.74586 L 94.176 120.362233 L 97.524 124.74586 L 100.872 129.451814 L 104.22 84.777488 L 107.568 76.719349 L 110.916 155.302326 L 114.264 152.981581 L 117.612 153.561767 L 120.96 151.95014 L 124.308 170.838419 L 127.656 175.544372 L 131.004 190.049023 L 134.352 187.019163 L 137.7 179.476744 L 141.048 208.614977 L 144.396 223.764279 L 147.744 223.828744 L 151.092 223.893209 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_53">    <path d="M 54 127.26 L 57.348 121.200279 L 60.696 120.877953 L 64.044 118.750605 L 67.392 116.816651 L 70.74 120.749023 L 74.088 122.296186 L 77.436 150.918698 L 80.784 157.300744 L 84.132 163.16707 L 87.48 166.906047 L 90.828 165.81014 L 94.176 167.486233 L 97.524 167.615163 L 100.872 167.808558 L 104.22 179.154419 L 107.568 181.926419 L 110.916 180.766047 L 114.264 175.286512 L 117.612 186.181116 L 120.96 186.052186 L 124.308 187.921674 L 127.656 190.75814 L 131.004 193.272279 L 134.352 199.07414 L 137.7 171.612 L 141.048 223.828744 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_54">    <path d="M 54 127.26 L 57.348 124.874791 L 60.696 123.972279 L 64.044 125.068186 L 67.392 129.387349 L 70.74 135.31814 L 74.088 161.555442 L 77.436 171.35414 L 80.784 170.129302 L 84.132 160.588465 L 87.48 165.93907 L 90.828 168.324279 L 94.176 166.132465 L 97.524 171.676465 L 100.872 174.770791 L 104.22 174.964186 L 107.568 173.868279 L 110.916 175.673302 L 114.264 184.698419 L 117.612 176.511349 L 120.96 175.673302 L 124.308 159.105767 L 127.656 160.459535 L 131.004 174.448465 L 134.352 173.417023 L 137.7 177.607256 L 141.048 180.637116 L 144.396 185.407535 L 147.744 190.242419 L 151.092 223.893209 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_55">    <path d="M 54 127.26 L 57.348 124.874791 L 60.696 123.649953 L 64.044 125.197116 L 67.392 126.228558 L 70.74 121.45814 L 74.088 123.585488 L 77.436 125.068186 L 80.784 125.454977 L 84.132 124.23014 L 87.48 126.228558 L 90.828 129.77414 L 94.176 136.285116 L 97.524 138.992651 L 100.872 153.884093 L 104.22 161.748837 L 107.568 174.061674 L 110.916 184.311628 L 114.264 184.440558 L 117.612 196.173209 L 120.96 202.942047 L 124.308 209.001767 L 127.656 178.316372 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_56">    <path d="M 54 127.26 L 57.348 121.006884 L 60.696 117.396837 L 64.044 121.97386 L 67.392 123.069767 L 70.74 121.135814 L 74.088 103.085581 L 77.436 122.554047 L 80.784 119.459721 L 84.132 129.065023 L 87.48 130.032 L 90.828 156.978419 L 94.176 223.957674 L 97.524 223.957674 L 100.872 223.957674 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_57">    <path d="M 54 127.26 L 57.348 122.940837 L 60.696 120.362233 L 64.044 121.716 L 67.392 117.654698 L 70.74 124.036744 L 74.088 154.206419 L 77.436 160.072744 L 80.784 166.712651 L 84.132 168.646605 L 87.48 170.064837 L 90.828 178.122977 L 94.176 180.637116 L 97.524 223.828744 L 100.872 223.893209 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_58">    <path d="M 54 127.26 L 57.348 121.071349 L 60.696 123.843349 L 64.044 124.294605 L 67.392 115.462884 L 70.74 122.554047 L 74.088 125.197116 L 77.436 128.033581 L 80.784 122.167256 L 84.132 122.167256 L 87.48 121.200279 L 90.828 125.906233 L 94.176 121.97386 L 97.524 122.425116 L 100.872 162.909209 L 104.22 147.759907 L 107.568 153.626233 L 110.916 152.46586 L 114.264 155.882512 L 117.612 182.764465 L 120.96 193.65907 L 124.308 191.918512 L 127.656 192.498698 L 131.004 183.022326 L 134.352 180.250326 L 137.7 195.915349 L 141.048 205.13386 L 144.396 201.588279 L 147.744 198.880744 L 151.092 206.616558 L 154.44 212.225023 L 157.788 209.324093 L 161.136 223.893209 L 164.484 223.893209 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_59">    <path d="M 54 127.26 L 57.348 123.843349 L 60.696 124.294605 L 64.044 122.876372 L 67.392 123.392093 L 70.74 123.907814 L 74.088 125.841767 L 77.436 125.583907 L 80.784 123.972279 L 84.132 159.170233 L 87.48 162.522419 L 90.828 165.93907 L 94.176 180.314791 L 97.524 178.380837 L 100.872 175.802233 L 104.22 171.74093 L 107.568 172.127721 L 110.916 179.992465 L 114.264 190.177953 L 117.612 193.65907 L 120.96 205.069395 L 124.308 195.850884 L 127.656 206.745488 L 131.004 192.756558 L 134.352 204.102419 L 137.7 205.649581 L 141.048 204.553674 L 144.396 202.684186 L 147.744 202.942047 L 151.092 199.267535 L 154.44 168.130884 L 157.788 189.855628 L 161.136 206.294233 L 164.484 210.29107 L 167.832 192.369767 L 171.18 202.684186 L 174.528 206.487628 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_60">    <path d="M 54 127.26 L 57.348 125.648372 L 60.696 125.712837 L 64.044 124.294605 L 67.392 123.456558 L 70.74 124.61693 L 74.088 125.583907 L 77.436 123.649953 L 80.784 125.003721 L 84.132 125.648372 L 87.48 120.813488 L 90.828 122.296186 L 94.176 120.362233 L 97.524 123.069767 L 100.872 122.231721 L 104.22 123.778884 L 107.568 125.326047 L 110.916 141.571256 L 114.264 181.926419 L 117.612 196.237674 L 120.96 183.022326 L 124.308 197.784837 L 127.656 197.011256 L 131.004 182.635535 L 134.352 195.593023 L 137.7 191.080465 L 141.048 190.306884 L 144.396 183.473581 L 147.744 197.978233 L 151.092 199.58986 L 154.44 197.398047 L 157.788 197.720372 L 161.136 212.805209 L 164.484 223.828744 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_61">    <path d="M 54 127.26 L 57.348 124.294605 L 60.696 123.521023 L 64.044 122.618512 L 67.392 125.648372 L 70.74 122.231721 L 74.088 128.613767 L 77.436 127.51786 L 80.784 123.778884 L 84.132 142.473767 L 87.48 151.176558 L 90.828 145.825953 L 94.176 145.503628 L 97.524 147.115256 L 100.872 138.09014 L 104.22 140.99107 L 107.568 137.896744 L 110.916 154.528744 L 114.264 146.53507 L 117.612 176.511349 L 120.96 162.522419 L 124.308 190.049023 L 127.656 194.948372 L 131.004 198.300558 L 134.352 189.404372 L 137.7 198.300558 L 141.048 207.90586 L 144.396 191.467256 L 147.744 223.893209 L 151.092 223.893209 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_62">    <path d="M 54 127.26 L 57.348 119.588651 L 60.696 109.983349 L 64.044 119.975442 L 67.392 123.198698 L 70.74 120.233302 L 74.088 121.780465 L 77.436 118.557209 L 80.784 124.74586 L 84.132 123.134233 L 87.48 162.522419 L 90.828 156.849488 L 94.176 189.146512 L 97.524 180.443721 L 100.872 173.674884 L 104.22 183.473581 L 107.568 179.089953 L 110.916 185.794326 L 114.264 189.791163 L 117.612 184.118233 L 120.96 183.795907 L 124.308 192.563163 L 127.656 189.791163 L 131.004 193.143349 L 134.352 200.621302 L 137.7 200.234512 L 141.048 208.163721 L 144.396 223.893209 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_63">    <path d="M 54 127.26 L 57.348 120.555628 L 60.696 119.137395 L 64.044 122.167256 L 67.392 107.72707 L 70.74 118.621674 L 74.088 112.239628 L 77.436 123.649953 L 80.784 134.351163 L 84.132 135.44707 L 87.48 136.156186 L 90.828 133.384186 L 94.176 135.83386 L 97.524 147.566512 L 100.872 152.014605 L 104.22 169.484651 L 107.568 164.069581 L 110.916 184.440558 L 114.264 184.956279 L 117.612 181.861953 L 120.96 178.187442 L 124.308 175.222047 L 127.656 182.313209 L 131.004 178.380837 L 134.352 180.443721 L 137.7 184.053767 L 141.048 182.119814 L 144.396 192.434233 L 147.744 193.465674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #e377c2; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_64">    <path d="M 54 127.26 L 57.348 123.811116 L 60.696 123.553256 L 64.044 123.198698 L 67.392 123.134233 L 70.74 122.618512 L 74.088 123.682186 L 77.436 124.971488 L 80.784 125.777302 L 84.132 129.000558 L 87.48 133.93214 L 90.828 146.116047 L 94.176 149.855023 L 97.524 155.689116 L 100.872 160.233907 L 104.22 170.032605 L 107.568 175.093116 L 110.916 180.250326 L 114.264 184.988512 L 117.612 184.956279 L 120.96 185.375302 L 124.308 192.047442 L 127.656 192.402 L 131.004 193.368977 L 134.352 196.43107 L 137.7 199.267535 L 141.048 208.389349 L 144.396 223.764279 L 147.744 223.925442 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pea2631c811)" style="fill: none; stroke: #000000; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="patch_3">    <path d="M 54 224.28 L 54 30.24 " style="fill: none"/>   </g>   <g id="patch_4">    <path d="M 388.8 224.28 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="patch_5">    <path d="M 54 224.28 L 388.8 224.28 " style="fill: none"/>   </g>   <g id="patch_6">    <path d="M 54 30.24 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="text_16">    <g id="patch_7">     <a xlink:href="https://dynomight.net/chess">      <path d="M 323.8664 225.676606 L 388.881244 225.676606 L 388.881244 213.343012 L 323.8664 213.343012 z" style="fill: #ffffff; opacity: 0.000001; stroke: #ffffff; stroke-linejoin: miter"/>     </a>    </g>    <a xlink:href="https://dynomight.net/chess">     <!-- dynomight.net/chess -->     <g style="opacity: 0.3" transform="translate(327.8664 220.734419) scale(0.05 -0.05)">      <defs>       <path id="CourierNewPSMT-64" d="M 3206 3922 L 3206 263 L 3556 263 Q 3653 263 3693 227 Q 3734 191 3734 131 Q 3734 75 3693 37 Q 3653 0 3556 0 L 2941 0 L 2941 522 Q 2484 -103 1781 -103 Q 1425 -103 1098 86 Q 772 275 583 625 Q 394 975 394 1347 Q 394 1722 583 2070 Q 772 2419 1098 2609 Q 1425 2800 1784 2800 Q 2472 2800 2941 2175 L 2941 3656 L 2591 3656 Q 2494 3656 2453 3693 Q 2413 3731 2413 3791 Q 2413 3847 2453 3884 Q 2494 3922 2591 3922 L 3206 3922 zM 2941 1347 Q 2941 1847 2603 2190 Q 2266 2534 1800 2534 Q 1331 2534 993 2190 Q 656 1847 656 1347 Q 656 850 993 504 Q 1331 159 1800 159 Q 2266 159 2603 504 Q 2941 850 2941 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-79" d="M 1938 0 L 719 2441 L 641 2441 Q 547 2441 506 2478 Q 466 2516 466 2572 Q 466 2613 486 2645 Q 506 2678 539 2692 Q 572 2706 641 2706 L 1359 2706 Q 1453 2706 1493 2668 Q 1534 2631 1534 2572 Q 1534 2516 1493 2478 Q 1453 2441 1359 2441 L 1006 2441 L 2078 288 L 3134 2441 L 2781 2441 Q 2688 2441 2647 2478 Q 2606 2516 2606 2575 Q 2606 2631 2647 2668 Q 2688 2706 2781 2706 L 3497 2706 Q 3594 2706 3634 2668 Q 3675 2631 3675 2572 Q 3675 2531 3650 2497 Q 3625 2463 3594 2452 Q 3563 2441 3419 2441 L 1756 -944 L 2166 -944 Q 2259 -944 2300 -980 Q 2341 -1016 2341 -1075 Q 2341 -1131 2300 -1168 Q 2259 -1206 2166 -1206 L 653 -1206 Q 559 -1206 518 -1170 Q 478 -1134 478 -1075 Q 478 -1016 518 -980 Q 559 -944 653 -944 L 1475 -944 L 1938 0 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6e" d="M 1072 2706 L 1072 2309 Q 1347 2588 1569 2694 Q 1791 2800 2069 2800 Q 2369 2800 2616 2672 Q 2791 2578 2933 2361 Q 3075 2144 3075 1916 L 3075 263 L 3297 263 Q 3391 263 3431 227 Q 3472 191 3472 131 Q 3472 75 3431 37 Q 3391 0 3297 0 L 2594 0 Q 2497 0 2456 37 Q 2416 75 2416 131 Q 2416 191 2456 227 Q 2497 263 2594 263 L 2813 263 L 2813 1872 Q 2813 2150 2609 2342 Q 2406 2534 2066 2534 Q 1806 2534 1615 2429 Q 1425 2325 1072 1909 L 1072 263 L 1369 263 Q 1463 263 1503 227 Q 1544 191 1544 131 Q 1544 75 1503 37 Q 1463 0 1369 0 L 513 0 Q 419 0 378 37 Q 338 75 338 131 Q 338 191 378 227 Q 419 263 513 263 L 809 263 L 809 2441 L 588 2441 Q 494 2441 453 2478 Q 413 2516 413 2575 Q 413 2631 453 2668 Q 494 2706 588 2706 L 1072 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6f" d="M 3388 1347 Q 3388 747 2958 322 Q 2528 -103 1922 -103 Q 1309 -103 881 323 Q 453 750 453 1347 Q 453 1947 881 2373 Q 1309 2800 1922 2800 Q 2528 2800 2958 2375 Q 3388 1950 3388 1347 zM 3122 1347 Q 3122 1841 2770 2187 Q 2419 2534 1919 2534 Q 1419 2534 1067 2186 Q 716 1838 716 1347 Q 716 859 1067 509 Q 1419 159 1919 159 Q 2419 159 2770 507 Q 3122 856 3122 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6d" d="M 725 2706 L 725 2441 Q 1059 2800 1397 2800 Q 1600 2800 1753 2692 Q 1906 2584 2009 2366 Q 2184 2584 2364 2692 Q 2544 2800 2725 2800 Q 3009 2800 3178 2616 Q 3400 2378 3400 2097 L 3400 263 L 3622 263 Q 3716 263 3756 227 Q 3797 191 3797 131 Q 3797 75 3756 37 Q 3716 0 3622 0 L 3138 0 L 3138 2072 Q 3138 2272 3016 2403 Q 2894 2534 2734 2534 Q 2591 2534 2431 2426 Q 2272 2319 2069 2003 L 2069 263 L 2288 263 Q 2381 263 2422 227 Q 2463 191 2463 131 Q 2463 75 2422 37 Q 2381 0 2288 0 L 1803 0 L 1803 2053 Q 1803 2263 1679 2398 Q 1556 2534 1403 2534 Q 1263 2534 1125 2444 Q 934 2316 725 2003 L 725 263 L 947 263 Q 1041 263 1081 227 Q 1122 191 1122 131 Q 1122 75 1081 37 Q 1041 0 947 0 L 241 0 Q 147 0 106 37 Q 66 75 66 131 Q 66 191 106 227 Q 147 263 241 263 L 463 263 L 463 2441 L 241 2441 Q 147 2441 106 2478 Q 66 2516 66 2575 Q 66 2631 106 2668 Q 147 2706 241 2706 L 725 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-69" d="M 2050 4053 L 2050 3375 L 1663 3375 L 1663 4053 L 2050 4053 zM 2059 2706 L 2059 263 L 3091 263 Q 3188 263 3228 227 Q 3269 191 3269 131 Q 3269 75 3228 37 Q 3188 0 3091 0 L 766 0 Q 672 0 631 37 Q 591 75 591 131 Q 591 191 631 227 Q 672 263 766 263 L 1797 263 L 1797 2441 L 1031 2441 Q 938 2441 895 2478 Q 853 2516 853 2572 Q 853 2631 893 2668 Q 934 2706 1031 2706 L 2059 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-67" d="M 2803 2225 L 2803 2706 L 3419 2706 Q 3513 2706 3553 2668 Q 3594 2631 3594 2572 Q 3594 2516 3553 2478 Q 3513 2441 3419 2441 L 3066 2441 L 3066 -181 Q 3066 -444 2953 -650 Q 2878 -788 2703 -934 Q 2528 -1081 2384 -1143 Q 2241 -1206 2000 -1206 L 1256 -1206 Q 1163 -1206 1122 -1170 Q 1081 -1134 1081 -1075 Q 1081 -1016 1122 -978 Q 1163 -941 1256 -941 L 2009 -944 Q 2241 -944 2427 -825 Q 2613 -706 2734 -478 Q 2803 -347 2803 -144 L 2803 644 Q 2381 66 1716 66 Q 1175 66 783 467 Q 391 869 391 1434 Q 391 2000 783 2400 Q 1175 2800 1716 2800 Q 2381 2800 2803 2225 zM 2803 1434 Q 2803 1897 2486 2215 Q 2169 2534 1728 2534 Q 1288 2534 972 2214 Q 656 1894 656 1434 Q 656 972 972 651 Q 1288 331 1728 331 Q 2169 331 2486 651 Q 2803 972 2803 1434 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-68" d="M 1056 3922 L 1056 2306 Q 1306 2578 1536 2689 Q 1766 2800 2050 2800 Q 2356 2800 2570 2692 Q 2784 2584 2928 2361 Q 3072 2138 3072 1891 L 3072 263 L 3366 263 Q 3463 263 3502 227 Q 3541 191 3541 131 Q 3541 75 3502 37 Q 3463 0 3366 0 L 2513 0 Q 2416 0 2375 37 Q 2334 75 2334 131 Q 2334 191 2375 227 Q 2416 263 2513 263 L 2806 263 L 2806 1872 Q 2806 2156 2601 2347 Q 2397 2538 2031 2538 Q 1744 2538 1541 2397 Q 1394 2297 1056 1925 L 1056 263 L 1353 263 Q 1447 263 1487 227 Q 1528 191 1528 131 Q 1528 75 1487 37 Q 1447 0 1353 0 L 497 0 Q 403 0 362 37 Q 322 75 322 131 Q 322 191 362 227 Q 403 263 497 263 L 794 263 L 794 3656 L 441 3656 Q 347 3656 306 3693 Q 266 3731 266 3791 Q 266 3847 306 3884 Q 347 3922 441 3922 L 1056 3922 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-74" d="M 1381 2706 L 2806 2706 Q 2900 2706 2940 2668 Q 2981 2631 2981 2572 Q 2981 2516 2940 2478 Q 2900 2441 2806 2441 L 1381 2441 L 1381 694 Q 1381 466 1564 312 Q 1747 159 2100 159 Q 2366 159 2675 239 Q 2984 319 3156 419 Q 3219 459 3259 459 Q 3309 459 3346 420 Q 3384 381 3384 328 Q 3384 281 3344 241 Q 3244 138 2858 17 Q 2472 -103 2119 -103 Q 1659 -103 1387 112 Q 1116 328 1116 694 L 1116 2441 L 631 2441 Q 538 2441 497 2478 Q 456 2516 456 2575 Q 456 2631 497 2668 Q 538 2706 631 2706 L 1116 2706 L 1116 3481 Q 1116 3575 1153 3615 Q 1191 3656 1247 3656 Q 1306 3656 1343 3615 Q 1381 3575 1381 3481 L 1381 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2e" d="M 1866 753 L 1978 753 Q 2153 753 2278 629 Q 2403 506 2403 331 Q 2403 150 2276 28 Q 2150 -94 1978 -94 L 1866 -94 Q 1691 -94 1566 29 Q 1441 153 1441 328 Q 1441 509 1567 631 Q 1694 753 1866 753 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-65" d="M 3341 1291 L 666 1291 Q 734 781 1092 470 Q 1450 159 1978 159 Q 2272 159 2594 256 Q 2916 353 3119 513 Q 3178 559 3222 559 Q 3272 559 3309 520 Q 3347 481 3347 428 Q 3347 375 3297 325 Q 3147 169 2764 33 Q 2381 -103 1978 -103 Q 1303 -103 851 339 Q 400 781 400 1409 Q 400 1981 823 2390 Q 1247 2800 1872 2800 Q 2516 2800 2931 2379 Q 3347 1959 3341 1291 zM 3075 1556 Q 2997 1991 2664 2262 Q 2331 2534 1872 2534 Q 1413 2534 1081 2265 Q 750 1997 669 1556 L 3075 1556 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2f" d="M 3100 4113 L 988 -413 Q 934 -528 850 -528 Q 797 -528 758 -489 Q 719 -450 719 -403 Q 719 -366 750 -300 L 2859 4225 Q 2894 4297 2925 4319 Q 2956 4341 3000 4341 Q 3053 4341 3092 4302 Q 3131 4263 3131 4216 Q 3131 4181 3100 4113 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-63" d="M 3016 2441 L 3016 2528 Q 3016 2625 3053 2665 Q 3091 2706 3147 2706 Q 3206 2706 3243 2665 Q 3281 2625 3281 2528 L 3281 1934 Q 3278 1838 3242 1797 Q 3206 1756 3147 1756 Q 3094 1756 3058 1792 Q 3022 1828 3016 1913 Q 2997 2134 2723 2334 Q 2450 2534 1988 2534 Q 1403 2534 1100 2168 Q 797 1803 797 1331 Q 797 822 1131 490 Q 1466 159 1997 159 Q 2303 159 2620 271 Q 2938 384 3194 634 Q 3259 697 3309 697 Q 3363 697 3398 661 Q 3434 625 3434 572 Q 3434 438 3119 231 Q 2609 -103 1984 -103 Q 1350 -103 942 301 Q 534 706 534 1328 Q 534 1963 951 2381 Q 1369 2800 2003 2800 Q 2606 2800 3016 2441 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-73" d="M 2788 2534 Q 2788 2625 2825 2665 Q 2863 2706 2919 2706 Q 2978 2706 3015 2665 Q 3053 2625 3053 2528 L 3053 2078 Q 3053 1984 3015 1943 Q 2978 1903 2919 1903 Q 2866 1903 2830 1937 Q 2794 1972 2788 2050 Q 2769 2238 2594 2359 Q 2338 2534 1916 2534 Q 1475 2534 1231 2356 Q 1047 2222 1047 2056 Q 1047 1869 1266 1744 Q 1416 1656 1834 1609 Q 2381 1550 2594 1475 Q 2897 1366 3045 1172 Q 3194 978 3194 753 Q 3194 419 2872 158 Q 2550 -103 1928 -103 Q 1306 -103 909 213 Q 909 106 896 75 Q 884 44 851 22 Q 819 0 778 0 Q 722 0 684 40 Q 647 81 647 175 L 647 716 Q 647 809 683 850 Q 719 891 778 891 Q 834 891 873 852 Q 913 813 913 747 Q 913 603 984 506 Q 1094 356 1333 257 Q 1572 159 1919 159 Q 2431 159 2681 350 Q 2931 541 2931 753 Q 2931 997 2678 1144 Q 2422 1291 1933 1341 Q 1444 1391 1231 1472 Q 1019 1553 900 1715 Q 781 1878 781 2066 Q 781 2403 1112 2601 Q 1444 2800 1903 2800 Q 2447 2800 2788 2534 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#CourierNewPSMT-64"/>      <use xlink:href="#CourierNewPSMT-79" x="60.009766"/>      <use xlink:href="#CourierNewPSMT-6e" x="120.019531"/>      <use xlink:href="#CourierNewPSMT-6f" x="180.029297"/>      <use xlink:href="#CourierNewPSMT-6d" x="240.039062"/>      <use xlink:href="#CourierNewPSMT-69" x="300.048828"/>      <use xlink:href="#CourierNewPSMT-67" x="360.058594"/>      <use xlink:href="#CourierNewPSMT-68" x="420.068359"/>      <use xlink:href="#CourierNewPSMT-74" x="480.078125"/>      <use xlink:href="#CourierNewPSMT-2e" x="540.087891"/>      <use xlink:href="#CourierNewPSMT-6e" x="600.097656"/>      <use xlink:href="#CourierNewPSMT-65" x="660.107422"/>      <use xlink:href="#CourierNewPSMT-74" x="720.117188"/>      <use xlink:href="#CourierNewPSMT-2f" x="780.126953"/>      <use xlink:href="#CourierNewPSMT-63" x="840.136719"/>      <use xlink:href="#CourierNewPSMT-68" x="900.146484"/>      <use xlink:href="#CourierNewPSMT-65" x="960.15625"/>      <use xlink:href="#CourierNewPSMT-73" x="1020.166016"/>      <use xlink:href="#CourierNewPSMT-73" x="1080.175781"/>     </g>    </a>   </g>   <g id="text_17">    <!-- qwen-2.5-72b vs stockfish -->    <g transform="translate(144.609375 24.24) scale(0.12 -0.12)">     <defs>      <path id="Montserrat-Regular-71" d="M 3706 3366 L 3706 -1242 L 3251 -1242 L 3251 634 Q 3040 307 2710 137 Q 2381 -32 1971 -32 Q 1491 -32 1110 182 Q 730 397 512 787 Q 294 1178 294 1683 Q 294 2182 512 2572 Q 730 2963 1114 3177 Q 1498 3392 1971 3392 Q 2394 3392 2730 3213 Q 3066 3034 3270 2701 L 3270 3366 L 3706 3366 zM 2010 371 Q 2362 371 2646 534 Q 2931 698 3094 998 Q 3258 1299 3258 1683 Q 3258 2067 3094 2364 Q 2931 2662 2646 2828 Q 2362 2995 2010 2995 Q 1651 2995 1366 2828 Q 1082 2662 918 2364 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-2e" d="M 678 -32 Q 538 -32 438 70 Q 339 173 339 320 Q 339 467 438 566 Q 538 666 678 666 Q 819 666 921 566 Q 1024 467 1024 320 Q 1024 173 921 70 Q 819 -32 678 -32 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-37" d="M 3494 4480 L 3494 4154 L 1619 0 L 1120 0 L 2963 4070 L 659 4070 L 659 3232 L 205 3232 L 205 4480 L 3494 4480 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-62" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1184 3827 793 Q 3610 403 3226 185 Q 2842 -32 2368 -32 Q 1946 -32 1610 147 Q 1274 326 1069 666 L 1069 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2726 Q 1299 3053 1628 3222 Q 1958 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2368 Q 3264 2669 2976 2832 Q 2688 2995 2336 2995 Q 1978 2995 1693 2832 Q 1408 2669 1245 2368 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-76" d="M 3462 3366 L 1965 0 L 1498 0 L 6 3366 L 480 3366 L 1741 499 L 3014 3366 L 3462 3366 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6f" d="M 2010 -32 Q 1523 -32 1132 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1132 3174 Q 1523 3392 2010 3392 Q 2496 3392 2886 3174 Q 3277 2957 3497 2566 Q 3718 2176 3718 1683 Q 3718 1190 3497 800 Q 3277 410 2886 189 Q 2496 -32 2010 -32 zM 2010 371 Q 2368 371 2653 534 Q 2938 698 3098 998 Q 3258 1299 3258 1683 Q 3258 2067 3098 2368 Q 2938 2669 2653 2832 Q 2368 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6b" d="M 1882 1690 L 1088 960 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 1530 L 3098 3366 L 3661 3366 L 2221 1990 L 3795 0 L 3238 0 L 1882 1690 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-68" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2778 Q 1274 3072 1600 3232 Q 1926 3392 2349 3392 z" transform="scale(0.015625)"/>     </defs>     <use xlink:href="#Montserrat-Regular-71"/>     <use xlink:href="#Montserrat-Regular-77" x="67.799988"/>     <use xlink:href="#Montserrat-Regular-65" x="155.699982"/>     <use xlink:href="#Montserrat-Regular-6e" x="216.099976"/>     <use xlink:href="#Montserrat-Regular-2d" x="283.799957"/>     <use xlink:href="#Montserrat-Regular-32" x="321.999954"/>     <use xlink:href="#Montserrat-Regular-2e" x="378.799942"/>     <use xlink:href="#Montserrat-Regular-35" x="399.999939"/>     <use xlink:href="#Montserrat-Regular-2d" x="456.59993"/>     <use xlink:href="#Montserrat-Regular-37" x="494.799927"/>     <use xlink:href="#Montserrat-Regular-32" x="553.699921"/>     <use xlink:href="#Montserrat-Regular-62" x="610.499908"/>     <use xlink:href="#Montserrat-Regular-20" x="678.299896"/>     <use xlink:href="#Montserrat-Regular-76" x="704.499893"/>     <use xlink:href="#Montserrat-Regular-73" x="758.69989"/>     <use xlink:href="#Montserrat-Regular-20" x="807.599884"/>     <use xlink:href="#Montserrat-Regular-73" x="833.799881"/>     <use xlink:href="#Montserrat-Regular-74" x="882.699875"/>     <use xlink:href="#Montserrat-Regular-6f" x="923.299866"/>     <use xlink:href="#Montserrat-Regular-63" x="985.999863"/>     <use xlink:href="#Montserrat-Regular-6b" x="1042.29985"/>     <use xlink:href="#Montserrat-Regular-66" x="1102.399841"/>     <use xlink:href="#Montserrat-Regular-69" x="1136.299835"/>     <use xlink:href="#Montserrat-Regular-73" x="1163.199829"/>     <use xlink:href="#Montserrat-Regular-68" x="1212.099823"/>    </g>   </g>  </g> </g> <defs>  <clipPath id="pea2631c811">   <rect x="54" y="30.24" width="334.8" height="194.04"/>  </clipPath> </defs></svg>](https://dynomight.net/img/chess/qwen-2.5-72b_stockfish.pdf)

[Terrible.](https://lichess.org/5ZMxoM3T)

Maybe Qwen is somehow defective too? So I tried [`command-r-v01`](https://huggingface.co/CohereForAI/c4ai-command-r-v01), a 35 billion parameter model.

[<?xml version="1.0" encoding="utf-8" standalone="no"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"  "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg xmlns:xlink="http://www.w3.org/1999/xlink" width="432pt" height="252pt" viewBox="0 0 432 252" xmlns="http://www.w3.org/2000/svg" version="1.1"> <metadata>  <rdf:RDF xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:cc="http://creativecommons.org/ns#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">   <cc:Work>    <dc:type rdf:resource="http://purl.org/dc/dcmitype/StillImage"/>    <dc:date>2024-11-14T11:33:56.173251</dc:date>    <dc:format>image/svg+xml</dc:format>    <dc:creator>     <cc:Agent>      <dc:title>Matplotlib v3.8.4, https://matplotlib.org/</dc:title>     </cc:Agent>    </dc:creator>   </cc:Work>  </rdf:RDF> </metadata> <defs>  <style type="text/css">*{stroke-linejoin: round; stroke-linecap: butt}</style> </defs> <g id="figure_1">  <g id="patch_1">   <path d="M 0 252 L 432 252 L 432 0 L 0 0 z" style="fill: #ffffff"/>  </g>  <g id="axes_1">   <g id="patch_2">    <path d="M 54 224.28 L 388.8 224.28 L 388.8 30.24 L 54 30.24 z" style="fill: #ffffff"/>   </g>   <g id="matplotlib.axis_1">    <g id="xtick_1">     <g id="line2d_1"/>     <g id="text_1">      <!-- 0 -->      <g transform="translate(50.689844 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-30" d="M 2118 -38 Q 1600 -38 1193 237 Q 787 512 560 1027 Q 333 1542 333 2240 Q 333 2938 560 3453 Q 787 3968 1193 4243 Q 1600 4518 2118 4518 Q 2637 4518 3043 4243 Q 3450 3968 3680 3453 Q 3910 2938 3910 2240 Q 3910 1542 3680 1027 Q 3450 512 3043 237 Q 2637 -38 2118 -38 zM 2118 384 Q 2509 384 2806 601 Q 3104 819 3270 1235 Q 3437 1651 3437 2240 Q 3437 2829 3270 3245 Q 3104 3661 2806 3878 Q 2509 4096 2118 4096 Q 1728 4096 1430 3878 Q 1133 3661 966 3245 Q 800 2829 800 2240 Q 800 1651 966 1235 Q 1133 819 1430 601 Q 1728 384 2118 384 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="xtick_2">     <g id="line2d_2"/>     <g id="text_2">      <!-- 20 -->      <g transform="translate(114.81 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-32" d="M 3392 410 L 3392 0 L 250 0 L 250 326 L 2112 2150 Q 2464 2496 2589 2749 Q 2714 3002 2714 3258 Q 2714 3654 2442 3875 Q 2170 4096 1664 4096 Q 877 4096 442 3597 L 115 3878 Q 378 4186 784 4352 Q 1190 4518 1702 4518 Q 2387 4518 2787 4195 Q 3187 3872 3187 3309 Q 3187 2963 3033 2643 Q 2880 2323 2451 1907 L 928 410 L 3392 410 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-32"/>       <use xlink:href="#Montserrat-Regular-30" x="56.799988"/>      </g>     </g>    </g>    <g id="xtick_3">     <g id="line2d_3"/>     <g id="text_3">      <!-- 40 -->      <g transform="translate(181.305156 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-34" d="M 4141 1178 L 3245 1178 L 3245 0 L 2784 0 L 2784 1178 L 256 1178 L 256 1510 L 2630 4480 L 3142 4480 L 851 1587 L 2797 1587 L 2797 2624 L 3245 2624 L 3245 1587 L 4141 1587 L 4141 1178 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-34"/>       <use xlink:href="#Montserrat-Regular-30" x="66.099991"/>      </g>     </g>    </g>    <g id="xtick_4">     <g id="line2d_4"/>     <g id="text_4">      <!-- 60 -->      <g transform="translate(248.524531 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-36" d="M 2189 2682 Q 2630 2682 2972 2515 Q 3315 2349 3507 2045 Q 3699 1741 3699 1338 Q 3699 928 3500 614 Q 3302 301 2953 131 Q 2605 -38 2170 -38 Q 1280 -38 806 547 Q 333 1133 333 2202 Q 333 2950 589 3468 Q 845 3987 1305 4252 Q 1766 4518 2381 4518 Q 3046 4518 3443 4275 L 3258 3904 Q 2938 4115 2387 4115 Q 1645 4115 1222 3644 Q 800 3174 800 2278 Q 800 2118 819 1914 Q 986 2278 1350 2480 Q 1715 2682 2189 2682 zM 2150 352 Q 2637 352 2941 617 Q 3245 883 3245 1325 Q 3245 1766 2941 2028 Q 2637 2291 2118 2291 Q 1779 2291 1520 2163 Q 1261 2035 1113 1811 Q 966 1587 966 1312 Q 966 1056 1104 838 Q 1242 621 1510 486 Q 1779 352 2150 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-36"/>       <use xlink:href="#Montserrat-Regular-30" x="60.899994"/>      </g>     </g>    </g>    <g id="xtick_5">     <g id="line2d_5"/>     <g id="text_5">      <!-- 80 -->      <g transform="translate(315.34 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-38" d="M 2899 2349 Q 3322 2208 3549 1926 Q 3776 1645 3776 1248 Q 3776 858 3565 566 Q 3354 275 2960 118 Q 2566 -38 2035 -38 Q 1229 -38 768 307 Q 307 653 307 1248 Q 307 1651 528 1929 Q 749 2208 1171 2349 Q 826 2483 646 2736 Q 467 2989 467 3334 Q 467 3878 892 4198 Q 1318 4518 2035 4518 Q 2509 4518 2864 4374 Q 3219 4230 3417 3961 Q 3616 3693 3616 3334 Q 3616 2989 3430 2736 Q 3245 2483 2899 2349 zM 928 3322 Q 928 2950 1222 2732 Q 1517 2515 2035 2515 Q 2554 2515 2851 2732 Q 3149 2950 3149 3315 Q 3149 3693 2845 3910 Q 2541 4128 2035 4128 Q 1523 4128 1225 3910 Q 928 3693 928 3322 zM 2035 352 Q 2630 352 2966 595 Q 3302 838 3302 1254 Q 3302 1670 2966 1910 Q 2630 2150 2035 2150 Q 1446 2150 1110 1910 Q 774 1670 774 1254 Q 774 832 1107 592 Q 1440 352 2035 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-38"/>       <use xlink:href="#Montserrat-Regular-30" x="63.799988"/>      </g>     </g>    </g>    <g id="xtick_6">     <g id="line2d_6"/>     <g id="text_6">      <!-- 100 -->      <g transform="translate(380.375 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-31" d="M 1574 4480 L 1574 0 L 1114 0 L 1114 4070 L 58 4070 L 58 4480 L 1574 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>      </g>     </g>    </g>    <g id="text_7">     <!-- turns -->     <g transform="translate(208.170312 248.56125) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-74" d="M 2406 205 Q 2278 90 2089 29 Q 1901 -32 1696 -32 Q 1222 -32 966 224 Q 710 480 710 947 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 4102 L 1165 4102 L 1165 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 973 Q 1165 672 1315 515 Q 1466 358 1747 358 Q 1888 358 2019 403 Q 2150 448 2246 531 L 2406 205 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-75" d="M 3667 3366 L 3667 0 L 3232 0 L 3232 614 Q 3053 307 2739 137 Q 2426 -32 2022 -32 Q 1363 -32 982 336 Q 602 704 602 1414 L 602 3366 L 1056 3366 L 1056 1459 Q 1056 928 1318 653 Q 1581 378 2067 378 Q 2598 378 2905 701 Q 3213 1024 3213 1600 L 3213 3366 L 3667 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-72" d="M 1069 2707 Q 1229 3046 1545 3219 Q 1862 3392 2330 3392 L 2330 2950 L 2221 2957 Q 1690 2957 1389 2630 Q 1088 2304 1088 1715 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2707 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-6e" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2746 Q 1254 3053 1584 3222 Q 1914 3392 2349 3392 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-73" d="M 1536 -32 Q 1126 -32 752 86 Q 378 205 166 384 L 371 742 Q 582 576 902 470 Q 1222 365 1568 365 Q 2029 365 2249 509 Q 2470 653 2470 915 Q 2470 1101 2348 1206 Q 2227 1312 2041 1366 Q 1856 1421 1549 1472 Q 1139 1549 889 1629 Q 640 1709 464 1901 Q 288 2093 288 2432 Q 288 2854 640 3123 Q 992 3392 1619 3392 Q 1946 3392 2272 3305 Q 2598 3219 2810 3078 L 2611 2714 Q 2195 3002 1619 3002 Q 1184 3002 963 2848 Q 742 2694 742 2445 Q 742 2253 867 2137 Q 992 2022 1177 1968 Q 1363 1914 1690 1856 Q 2093 1779 2336 1702 Q 2579 1626 2752 1440 Q 2925 1254 2925 928 Q 2925 486 2557 227 Q 2189 -32 1536 -32 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-74"/>      <use xlink:href="#Montserrat-Regular-75" x="40.599991"/>      <use xlink:href="#Montserrat-Regular-72" x="107.899979"/>      <use xlink:href="#Montserrat-Regular-6e" x="147.999969"/>      <use xlink:href="#Montserrat-Regular-73" x="215.699951"/>     </g>    </g>   </g>   <g id="matplotlib.axis_2">    <g id="ytick_1">     <g id="line2d_7"/>     <g id="text_8">      <!-- LOSS -->      <g transform="translate(23.91875 227.667831) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-4c" d="M 730 4480 L 1203 4480 L 1203 410 L 3712 410 L 3712 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4f" d="M 2688 -38 Q 2022 -38 1481 259 Q 941 557 637 1075 Q 333 1594 333 2240 Q 333 2886 637 3404 Q 941 3923 1481 4220 Q 2022 4518 2688 4518 Q 3354 4518 3888 4224 Q 4422 3930 4729 3408 Q 5037 2886 5037 2240 Q 5037 1594 4729 1072 Q 4422 550 3888 256 Q 3354 -38 2688 -38 zM 2688 384 Q 3219 384 3648 624 Q 4077 864 4320 1289 Q 4563 1715 4563 2240 Q 4563 2765 4320 3190 Q 4077 3616 3648 3856 Q 3219 4096 2688 4096 Q 2157 4096 1725 3856 Q 1293 3616 1046 3190 Q 800 2765 800 2240 Q 800 1715 1046 1289 Q 1293 864 1725 624 Q 2157 384 2688 384 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-53" d="M 1965 -38 Q 1466 -38 1008 122 Q 550 282 301 544 L 486 909 Q 730 666 1133 515 Q 1536 365 1965 365 Q 2566 365 2867 585 Q 3168 806 3168 1158 Q 3168 1427 3005 1587 Q 2842 1747 2605 1833 Q 2368 1920 1946 2022 Q 1440 2150 1139 2268 Q 838 2387 624 2630 Q 410 2874 410 3290 Q 410 3629 589 3907 Q 768 4186 1139 4352 Q 1510 4518 2061 4518 Q 2445 4518 2813 4412 Q 3181 4307 3450 4122 L 3290 3744 Q 3008 3930 2688 4022 Q 2368 4115 2061 4115 Q 1472 4115 1174 3888 Q 877 3661 877 3302 Q 877 3034 1040 2870 Q 1203 2707 1449 2620 Q 1696 2534 2112 2432 Q 2605 2310 2909 2192 Q 3213 2074 3424 1834 Q 3635 1594 3635 1184 Q 3635 845 3452 566 Q 3270 288 2892 125 Q 2515 -38 1965 -38 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-4c"/>       <use xlink:href="#Montserrat-Regular-4f" x="58.899994"/>       <use xlink:href="#Montserrat-Regular-53" x="142.799988"/>       <use xlink:href="#Montserrat-Regular-53" x="204.299973"/>      </g>     </g>    </g>    <g id="ytick_2">     <g id="line2d_8"/>     <g id="text_9">      <!-- -1000 -->      <g transform="translate(23.209375 195.435273) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-2d" d="M 384 1933 L 2061 1933 L 2061 1536 L 384 1536 L 384 1933 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-31" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="74.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="140.499985"/>       <use xlink:href="#Montserrat-Regular-30" x="206.699982"/>      </g>     </g>    </g>    <g id="ytick_3">     <g id="line2d_9"/>     <g id="text_10">      <!-- -500 -->      <g transform="translate(27.779687 163.202714) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-35" d="M 1510 2643 Q 2483 2643 2928 2294 Q 3373 1946 3373 1318 Q 3373 928 3190 621 Q 3008 314 2643 138 Q 2278 -38 1747 -38 Q 1267 -38 835 118 Q 403 275 141 538 L 365 902 Q 589 672 953 528 Q 1318 384 1741 384 Q 2298 384 2602 633 Q 2906 883 2906 1299 Q 2906 1760 2563 1997 Q 2221 2234 1408 2234 L 454 2234 L 685 4480 L 3136 4480 L 3136 4070 L 1088 4070 L 934 2643 L 1510 2643 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-35" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="94.799988"/>       <use xlink:href="#Montserrat-Regular-30" x="160.999985"/>      </g>     </g>    </g>    <g id="ytick_4">     <g id="line2d_10"/>     <g id="text_11">      <!-- 0 -->      <g transform="translate(43.879688 130.970156) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="ytick_5">     <g id="line2d_11"/>     <g id="text_12">      <!-- 500 -->      <g transform="translate(31.6 98.737598) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-35"/>       <use xlink:href="#Montserrat-Regular-30" x="56.599991"/>       <use xlink:href="#Montserrat-Regular-30" x="122.799988"/>      </g>     </g>    </g>    <g id="ytick_6">     <g id="line2d_12"/>     <g id="text_13">      <!-- 1000 -->      <g transform="translate(27.029687 66.50504) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="168.499985"/>      </g>     </g>    </g>    <g id="ytick_7">     <g id="line2d_13"/>     <g id="text_14">      <!-- WIN -->      <g transform="translate(28.240625 34.272482) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-57" d="M 6880 4480 L 5370 0 L 4870 0 L 3558 3821 L 2240 0 L 1747 0 L 237 4480 L 723 4480 L 2016 627 L 3354 4480 L 3795 4480 L 5114 608 L 6426 4480 L 6880 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-49" d="M 730 4480 L 1203 4480 L 1203 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4e" d="M 4474 4480 L 4474 0 L 4083 0 L 1203 3635 L 1203 0 L 730 0 L 730 4480 L 1120 4480 L 4006 845 L 4006 4480 L 4474 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-57"/>       <use xlink:href="#Montserrat-Regular-49" x="111.099991"/>       <use xlink:href="#Montserrat-Regular-4e" x="141.299988"/>      </g>     </g>    </g>    <g id="text_15">     <!-- centipawn difference -->     <g transform="translate(17.26875 180.677188) rotate(-90) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-63" d="M 2029 -32 Q 1530 -32 1136 185 Q 742 403 518 796 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1136 3174 Q 1530 3392 2029 3392 Q 2464 3392 2806 3222 Q 3149 3053 3347 2726 L 3008 2496 Q 2842 2746 2586 2870 Q 2330 2995 2029 2995 Q 1664 2995 1373 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1293 918 995 Q 1082 698 1373 534 Q 1664 371 2029 371 Q 2330 371 2586 492 Q 2842 614 3008 864 L 3347 634 Q 3149 307 2803 137 Q 2458 -32 2029 -32 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-65" d="M 3565 1542 L 749 1542 Q 787 1018 1152 694 Q 1517 371 2074 371 Q 2387 371 2649 483 Q 2912 595 3104 813 L 3360 518 Q 3136 250 2800 109 Q 2464 -32 2061 -32 Q 1542 -32 1142 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 508 2566 Q 723 2957 1097 3174 Q 1472 3392 1939 3392 Q 2406 3392 2777 3174 Q 3149 2957 3360 2569 Q 3571 2182 3571 1683 L 3565 1542 zM 1939 3002 Q 1453 3002 1123 2691 Q 794 2381 749 1882 L 3136 1882 Q 3091 2381 2761 2691 Q 2432 3002 1939 3002 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-69" d="M 634 3366 L 1088 3366 L 1088 0 L 634 0 L 634 3366 zM 864 4102 Q 723 4102 627 4198 Q 531 4294 531 4429 Q 531 4557 627 4653 Q 723 4749 864 4749 Q 1005 4749 1101 4656 Q 1197 4563 1197 4435 Q 1197 4294 1101 4198 Q 1005 4102 864 4102 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-70" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1178 3827 787 Q 3610 397 3229 182 Q 2848 -32 2368 -32 Q 1958 -32 1628 137 Q 1299 307 1088 634 L 1088 -1242 L 634 -1242 L 634 3366 L 1069 3366 L 1069 2701 Q 1274 3034 1610 3213 Q 1946 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2364 Q 3264 2662 2976 2828 Q 2688 2995 2336 2995 Q 1978 2995 1693 2828 Q 1408 2662 1245 2364 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-61" d="M 1811 3392 Q 2470 3392 2822 3062 Q 3174 2733 3174 2086 L 3174 0 L 2739 0 L 2739 525 Q 2586 262 2288 115 Q 1990 -32 1581 -32 Q 1018 -32 685 237 Q 352 506 352 947 Q 352 1376 662 1638 Q 973 1901 1651 1901 L 2720 1901 L 2720 2106 Q 2720 2541 2477 2768 Q 2234 2995 1766 2995 Q 1446 2995 1152 2889 Q 858 2784 646 2598 L 442 2938 Q 698 3155 1056 3273 Q 1414 3392 1811 3392 zM 1651 326 Q 2035 326 2310 502 Q 2586 678 2720 1011 L 2720 1562 L 1664 1562 Q 800 1562 800 960 Q 800 666 1024 496 Q 1248 326 1651 326 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-77" d="M 5574 3366 L 4301 0 L 3872 0 L 2810 2758 L 1747 0 L 1318 0 L 51 3366 L 486 3366 L 1542 506 L 2624 3366 L 3014 3366 L 4090 506 L 5158 3366 L 5574 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-20" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-64" d="M 3706 4749 L 3706 0 L 3270 0 L 3270 666 Q 3066 326 2730 147 Q 2394 -32 1971 -32 Q 1498 -32 1114 185 Q 730 403 512 793 Q 294 1184 294 1683 Q 294 2182 512 2572 Q 730 2963 1114 3177 Q 1498 3392 1971 3392 Q 2381 3392 2710 3222 Q 3040 3053 3251 2726 L 3251 4749 L 3706 4749 zM 2010 371 Q 2362 371 2646 534 Q 2931 698 3094 998 Q 3258 1299 3258 1683 Q 3258 2067 3094 2368 Q 2931 2669 2646 2832 Q 2362 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-66" d="M 1747 4397 Q 1453 4397 1302 4237 Q 1152 4077 1152 3763 L 1152 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 0 L 710 0 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 3782 Q 710 4243 976 4512 Q 1242 4781 1728 4781 Q 1920 4781 2099 4726 Q 2278 4672 2400 4570 L 2246 4230 Q 2048 4397 1747 4397 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-63"/>      <use xlink:href="#Montserrat-Regular-65" x="56.299988"/>      <use xlink:href="#Montserrat-Regular-6e" x="116.699982"/>      <use xlink:href="#Montserrat-Regular-74" x="184.399963"/>      <use xlink:href="#Montserrat-Regular-69" x="224.999954"/>      <use xlink:href="#Montserrat-Regular-70" x="251.899948"/>      <use xlink:href="#Montserrat-Regular-61" x="319.699936"/>      <use xlink:href="#Montserrat-Regular-77" x="378.699921"/>      <use xlink:href="#Montserrat-Regular-6e" x="466.599915"/>      <use xlink:href="#Montserrat-Regular-20" x="534.299896"/>      <use xlink:href="#Montserrat-Regular-64" x="560.499893"/>      <use xlink:href="#Montserrat-Regular-69" x="628.299881"/>      <use xlink:href="#Montserrat-Regular-66" x="655.199875"/>      <use xlink:href="#Montserrat-Regular-66" x="689.099869"/>      <use xlink:href="#Montserrat-Regular-65" x="722.999863"/>      <use xlink:href="#Montserrat-Regular-72" x="783.399857"/>      <use xlink:href="#Montserrat-Regular-65" x="823.499847"/>      <use xlink:href="#Montserrat-Regular-6e" x="883.899841"/>      <use xlink:href="#Montserrat-Regular-63" x="951.599823"/>      <use xlink:href="#Montserrat-Regular-65" x="1007.899811"/>     </g>    </g>   </g>   <g id="line2d_14">    <path d="M 54 127.26 L 57.348 124.61693 L 60.696 123.714419 L 64.044 121.84493 L 67.392 124.35907 L 70.74 120.684558 L 74.088 118.299349 L 77.436 123.005302 L 80.784 125.906233 L 84.132 151.112093 L 87.48 147.888837 L 90.828 159.879349 L 94.176 155.431256 L 97.524 146.986326 L 100.872 157.042884 L 104.22 160.459535 L 107.568 159.041302 L 110.916 176.446884 L 114.264 186.632372 L 117.612 190.629209 L 120.96 193.014419 L 124.308 201.265953 L 127.656 203.651163 L 131.004 189.404372 L 134.352 181.668558 L 137.7 188.759721 L 141.048 195.464093 L 144.396 193.272279 L 147.744 199.396465 L 151.092 200.492372 L 154.44 194.174791 L 157.788 223.893209 L 161.136 223.893209 L 164.484 223.893209 L 167.832 223.893209 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_15">    <path d="M 54 127.26 L 57.348 120.813488 L 60.696 121.97386 L 64.044 123.649953 L 67.392 120.749023 L 70.74 122.038326 L 74.088 124.874791 L 77.436 116.558791 L 80.784 113.722326 L 84.132 116.881116 L 87.48 115.591814 L 90.828 115.462884 L 94.176 112.626419 L 97.524 112.433023 L 100.872 114.624837 L 104.22 119.524186 L 107.568 127.646791 L 110.916 144.601116 L 114.264 156.398233 L 117.612 166.519256 L 120.96 173.030233 L 124.308 183.280186 L 127.656 182.764465 L 131.004 180.379256 L 134.352 180.959442 L 137.7 176.575814 L 141.048 171.805395 L 144.396 175.157581 L 147.744 172.385581 L 151.092 183.666977 L 154.44 181.023907 L 157.788 183.409116 L 161.136 188.308465 L 164.484 182.635535 L 167.832 191.016 L 171.18 195.335163 L 174.528 197.591442 L 177.876 200.685767 L 181.224 223.893209 L 184.572 223.893209 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_16">    <path d="M 54 127.26 L 57.348 125.777302 L 60.696 124.23014 L 64.044 123.521023 L 67.392 123.521023 L 70.74 118.105953 L 74.088 125.648372 L 77.436 126.293023 L 80.784 125.519442 L 84.132 116.816651 L 87.48 127.00214 L 90.828 125.519442 L 94.176 127.26 L 97.524 129.77414 L 100.872 129.065023 L 104.22 126.357488 L 107.568 126.421953 L 110.916 129.258419 L 114.264 138.47693 L 117.612 141.893581 L 120.96 177.736186 L 124.308 181.023907 L 127.656 186.567907 L 131.004 185.794326 L 134.352 161.684372 L 137.7 186.310047 L 141.048 181.152837 L 144.396 182.893395 L 147.744 193.91693 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_17">    <path d="M 54 127.26 L 57.348 124.61693 L 60.696 123.714419 L 64.044 124.23014 L 67.392 123.972279 L 70.74 124.61693 L 74.088 121.716 L 77.436 156.591628 L 80.784 161.039721 L 84.132 161.748837 L 87.48 161.233116 L 90.828 164.456372 L 94.176 164.778698 L 97.524 170.645023 L 100.872 160.717395 L 104.22 157.042884 L 107.568 165.552279 L 110.916 160.65293 L 114.264 166.068 L 117.612 171.096279 L 120.96 174.319535 L 124.308 223.893209 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_18">    <path d="M 54 127.26 L 57.348 122.296186 L 60.696 122.940837 L 64.044 123.198698 L 67.392 119.395256 L 70.74 125.068186 L 74.088 125.390512 L 77.436 125.390512 L 80.784 127.840186 L 84.132 127.711256 L 87.48 128.291442 L 90.828 130.612186 L 94.176 125.906233 L 97.524 119.717581 L 100.872 158.976837 L 104.22 162.844744 L 107.568 171.676465 L 110.916 175.286512 L 114.264 173.610419 L 117.612 205.971907 L 120.96 197.849302 L 124.308 186.825767 L 127.656 223.893209 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_19">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 123.972279 L 64.044 131.965953 L 67.392 160.26614 L 70.74 155.10893 L 74.088 158.590047 L 77.436 159.750419 L 80.784 162.329023 L 84.132 178.638698 L 87.48 187.341488 L 90.828 183.344651 L 94.176 179.412279 L 97.524 185.72986 L 100.872 187.277023 L 104.22 193.336744 L 107.568 202.104 L 110.916 191.918512 L 114.264 189.339907 L 117.612 194.04586 L 120.96 199.267535 L 124.308 202.555256 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_20">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 123.714419 L 64.044 127.324465 L 67.392 125.068186 L 70.74 128.484837 L 74.088 122.618512 L 77.436 119.846512 L 80.784 152.594791 L 84.132 158.074326 L 87.48 161.877767 L 90.828 159.492558 L 94.176 168.066419 L 97.524 178.896558 L 100.872 185.536465 L 104.22 187.212558 L 107.568 187.857209 L 110.916 183.022326 L 114.264 195.270698 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_21">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 126.550884 L 64.044 126.744279 L 67.392 124.61693 L 70.74 124.23014 L 74.088 126.293023 L 77.436 135.898326 L 80.784 155.173395 L 84.132 162.006698 L 87.48 166.777116 L 90.828 162.006698 L 94.176 164.262977 L 97.524 169.613581 L 100.872 193.594605 L 104.22 188.953116 L 107.568 183.086791 L 110.916 179.089953 L 114.264 184.762884 L 117.612 173.674884 L 120.96 182.313209 L 124.308 183.022326 L 127.656 178.703163 L 131.004 183.344651 L 134.352 192.369767 L 137.7 197.462512 L 141.048 196.753395 L 144.396 198.042698 L 147.744 198.751814 L 151.092 199.58986 L 154.44 201.781674 L 157.788 199.654326 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_22">    <path d="M 54 127.26 L 57.348 125.906233 L 60.696 125.390512 L 64.044 128.936093 L 67.392 127.38893 L 70.74 128.162512 L 74.088 127.51786 L 77.436 126.873209 L 80.784 122.231721 L 84.132 119.717581 L 87.48 119.524186 L 90.828 116.881116 L 94.176 134.737953 L 97.524 134.286698 L 100.872 137.574419 L 104.22 149.69386 L 107.568 162.457953 L 110.916 160.330605 L 114.264 175.866698 L 117.612 195.270698 L 120.96 199.58986 L 124.308 208.034791 L 127.656 223.893209 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_23">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 123.649953 L 64.044 126.357488 L 67.392 130.741116 L 70.74 132.868465 L 74.088 145.374698 L 77.436 147.695442 L 80.784 150.338512 L 84.132 154.013023 L 87.48 156.204837 L 90.828 156.398233 L 94.176 157.752 L 97.524 164.005116 L 100.872 169.549116 L 104.22 173.417023 L 107.568 158.912372 L 110.916 158.976837 L 114.264 152.594791 L 117.612 142.925023 L 120.96 157.945395 L 124.308 187.663814 L 127.656 191.402791 L 131.004 191.338326 L 134.352 191.918512 L 137.7 194.174791 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_24">    <path d="M 54 127.26 L 57.348 123.585488 L 60.696 124.488 L 64.044 122.554047 L 67.392 116.752186 L 70.74 117.719163 L 74.088 123.649953 L 77.436 125.003721 L 80.784 125.648372 L 84.132 123.843349 L 87.48 121.909395 L 90.828 124.74586 L 94.176 126.550884 L 97.524 126.099628 L 100.872 179.412279 L 104.22 180.830512 L 107.568 171.805395 L 110.916 172.127721 L 114.264 173.932744 L 117.612 167.873023 L 120.96 182.055349 L 124.308 194.883907 L 127.656 187.212558 L 131.004 189.275442 L 134.352 189.920093 L 137.7 180.379256 L 141.048 182.313209 L 144.396 172.063256 L 147.744 193.53014 L 151.092 198.300558 L 154.44 205.842977 L 157.788 223.893209 L 161.136 216.930977 L 164.484 207.712465 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_25">    <path d="M 54 127.26 L 57.348 120.555628 L 60.696 124.874791 L 64.044 122.360651 L 67.392 121.329209 L 70.74 119.524186 L 74.088 121.97386 L 77.436 120.684558 L 80.784 120.813488 L 84.132 119.459721 L 87.48 120.555628 L 90.828 121.45814 L 94.176 121.651535 L 97.524 117.396837 L 100.872 149.887256 L 104.22 174.770791 L 107.568 155.753581 L 110.916 156.978419 L 114.264 164.714233 L 117.612 168.646605 L 120.96 167.228372 L 124.308 174.964186 L 127.656 185.72986 L 131.004 179.218884 L 134.352 173.868279 L 137.7 223.828744 L 141.048 185.020744 L 144.396 223.893209 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_26">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 124.165674 L 64.044 126.099628 L 67.392 125.390512 L 70.74 122.811907 L 74.088 118.105953 L 77.436 165.294419 L 80.784 170.902884 L 84.132 164.714233 L 87.48 162.780279 L 90.828 171.998791 L 94.176 183.538047 L 97.524 182.506605 L 100.872 187.212558 L 104.22 184.698419 L 107.568 180.121395 L 110.916 180.894977 L 114.264 182.635535 L 117.612 212.998605 L 120.96 207.712465 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_27">    <path d="M 54 127.26 L 57.348 123.327628 L 60.696 123.263163 L 64.044 122.167256 L 67.392 118.041488 L 70.74 119.717581 L 74.088 115.011628 L 77.436 120.491163 L 80.784 120.555628 L 84.132 114.818233 L 87.48 154.85107 L 90.828 149.758326 L 94.176 163.618326 L 97.524 167.421767 L 100.872 167.744093 L 104.22 173.610419 L 107.568 164.907628 L 110.916 165.93907 L 114.264 173.094698 L 117.612 162.844744 L 120.96 170.064837 L 124.308 167.550698 L 127.656 167.486233 L 131.004 180.766047 L 134.352 179.79907 L 137.7 178.703163 L 141.048 179.79907 L 144.396 174.319535 L 147.744 175.737767 L 151.092 176.89814 L 154.44 181.152837 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_28">    <path d="M 54 127.26 L 57.348 125.906233 L 60.696 125.906233 L 64.044 125.970698 L 67.392 127.00214 L 70.74 127.195535 L 74.088 125.712837 L 77.436 159.105767 L 80.784 161.490977 L 84.132 160.846326 L 87.48 162.393488 L 90.828 172.514512 L 94.176 165.93907 L 97.524 168.96893 L 100.872 165.101023 L 104.22 184.827349 L 107.568 181.861953 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_29">    <path d="M 54 127.26 L 57.348 123.585488 L 60.696 125.583907 L 64.044 125.777302 L 67.392 126.357488 L 70.74 131.127907 L 74.088 131.772558 L 77.436 136.285116 L 80.784 136.736372 L 84.132 164.585302 L 87.48 164.456372 L 90.828 187.277023 L 94.176 185.21414 L 97.524 163.360465 L 100.872 166.132465 L 104.22 193.852465 L 107.568 184.505023 L 110.916 197.462512 L 114.264 172.063256 L 117.612 185.858791 L 120.96 188.695256 L 124.308 176.253488 L 127.656 174.577395 L 131.004 193.594605 L 134.352 194.948372 L 137.7 202.426326 L 141.048 204.037953 L 144.396 223.893209 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_30">    <path d="M 54 127.26 L 57.348 120.942419 L 60.696 121.84493 L 64.044 117.396837 L 67.392 118.105953 L 70.74 120.168837 L 74.088 121.84493 L 77.436 123.778884 L 80.784 128.291442 L 84.132 177.28493 L 87.48 223.957674 L 90.828 223.957674 L 94.176 223.957674 L 97.524 223.957674 L 100.872 223.957674 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_31">    <path d="M 54 127.26 L 57.348 122.167256 L 60.696 119.459721 L 64.044 117.783628 L 67.392 120.039907 L 70.74 120.620093 L 74.088 121.84493 L 77.436 123.005302 L 80.784 121.97386 L 84.132 124.488 L 87.48 124.488 L 90.828 125.132651 L 94.176 127.38893 L 97.524 124.23014 L 100.872 122.940837 L 104.22 127.195535 L 107.568 129.258419 L 110.916 131.965953 L 114.264 153.239442 L 117.612 165.745674 L 120.96 172.514512 L 124.308 176.640279 L 127.656 194.948372 L 131.004 205.649581 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_32">    <path d="M 54 127.26 L 57.348 125.519442 L 60.696 125.777302 L 64.044 124.35907 L 67.392 125.003721 L 70.74 124.294605 L 74.088 124.552465 L 77.436 124.939256 L 80.784 129.258419 L 84.132 127.13107 L 87.48 127.711256 L 90.828 135.253674 L 94.176 169.742512 L 97.524 174.964186 L 100.872 176.060093 L 104.22 173.545953 L 107.568 172.707907 L 110.916 168.453209 L 114.264 182.44214 L 117.612 185.085209 L 120.96 195.206233 L 124.308 204.553674 L 127.656 195.786419 L 131.004 201.330419 L 134.352 181.604093 L 137.7 187.341488 L 141.048 189.920093 L 144.396 195.077302 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_33">    <path d="M 54 127.26 L 57.348 122.940837 L 60.696 126.228558 L 64.044 128.420372 L 67.392 125.519442 L 70.74 126.421953 L 74.088 125.648372 L 77.436 124.423535 L 80.784 126.293023 L 84.132 126.550884 L 87.48 123.585488 L 90.828 129.387349 L 94.176 161.490977 L 97.524 165.294419 L 100.872 160.330605 L 104.22 168.195349 L 107.568 171.225209 L 110.916 167.486233 L 114.264 205.714047 L 117.612 209.904279 L 120.96 188.566326 L 124.308 190.242419 L 127.656 196.946791 L 131.004 195.141767 L 134.352 192.756558 L 137.7 206.745488 L 141.048 197.398047 L 144.396 192.498698 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_34">    <path d="M 54 127.26 L 57.348 125.777302 L 60.696 123.649953 L 64.044 124.35907 L 67.392 128.033581 L 70.74 128.484837 L 74.088 127.840186 L 77.436 131.127907 L 80.784 137.509953 L 84.132 142.925023 L 87.48 145.374698 L 90.828 145.245767 L 94.176 141.055535 L 97.524 165.101023 L 100.872 153.819628 L 104.22 153.626233 L 107.568 159.299163 L 110.916 161.748837 L 114.264 165.294419 L 117.612 173.417023 L 120.96 171.998791 L 124.308 171.998791 L 127.656 162.006698 L 131.004 182.377674 L 134.352 217.124372 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_35">    <path d="M 54 127.26 L 57.348 120.813488 L 60.696 118.492744 L 64.044 116.945581 L 67.392 145.890419 L 70.74 144.923442 L 74.088 149.56493 L 77.436 151.821209 L 80.784 173.997209 L 84.132 171.547535 L 87.48 177.091535 L 90.828 165.358884 L 94.176 174.448465 L 97.524 167.615163 L 100.872 168.259814 L 104.22 175.157581 L 107.568 166.454791 L 110.916 177.865116 L 114.264 195.464093 L 117.612 223.893209 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_36">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 123.972279 L 64.044 117.074512 L 67.392 113.65786 L 70.74 111.337116 L 74.088 121.97386 L 77.436 124.23014 L 80.784 121.329209 L 84.132 129.387349 L 87.48 129.709674 L 90.828 130.28986 L 94.176 129.967535 L 97.524 134.480093 L 100.872 129.77414 L 104.22 129.193953 L 107.568 143.376279 L 110.916 120.168837 L 114.264 137.123163 L 117.612 151.369953 L 120.96 163.811721 L 124.308 223.893209 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_37">    <path d="M 54 127.26 L 57.348 122.102791 L 60.696 121.97386 L 64.044 121.329209 L 67.392 119.395256 L 70.74 120.039907 L 74.088 152.33693 L 77.436 162.457953 L 80.784 154.270884 L 84.132 150.660837 L 87.48 150.016186 L 90.828 147.244186 L 94.176 153.948558 L 97.524 175.479907 L 100.872 173.545953 L 104.22 170.258233 L 107.568 187.277023 L 110.916 181.926419 L 114.264 181.088372 L 117.612 177.28493 L 120.96 177.736186 L 124.308 205.262791 L 127.656 201.588279 L 131.004 215.964 L 134.352 203.135442 L 137.7 207.970326 L 141.048 205.391721 L 144.396 210.42 L 147.744 200.298977 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_38">    <path d="M 54 127.26 L 57.348 120.555628 L 60.696 123.198698 L 64.044 124.488 L 67.392 140.86214 L 70.74 136.671907 L 74.088 142.215907 L 77.436 147.824372 L 80.784 146.019349 L 84.132 149.887256 L 87.48 162.973674 L 90.828 181.152837 L 94.176 181.797488 L 97.524 168.388744 L 100.872 176.382419 L 104.22 169.291256 L 107.568 174.964186 L 110.916 180.830512 L 114.264 172.578977 L 117.612 169.871442 L 120.96 186.503442 L 124.308 186.052186 L 127.656 191.982977 L 131.004 184.440558 L 134.352 191.725116 L 137.7 189.404372 L 141.048 191.789581 L 144.396 191.596186 L 147.744 189.984558 L 151.092 193.91693 L 154.44 179.025488 L 157.788 195.012837 L 161.136 189.082047 L 164.484 190.242419 L 167.832 188.308465 L 171.18 190.75814 L 174.528 189.920093 L 177.876 191.854047 L 181.224 192.692093 L 184.572 193.723535 L 187.92 192.176372 L 191.268 187.148093 L 194.616 191.531721 L 197.964 190.693674 L 201.312 187.148093 L 204.66 183.280186 L 208.008 198.622884 L 211.356 200.621302 L 214.704 195.012837 L 218.052 179.089953 L 221.4 172.063256 L 224.748 166.906047 L 228.096 166.132465 L 231.444 173.610419 L 234.792 177.091535 L 238.14 171.86986 L 241.488 178.638698 L 244.836 175.608837 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_39">    <path d="M 54 127.26 L 57.348 123.778884 L 60.696 123.778884 L 64.044 120.813488 L 67.392 149.436 L 70.74 152.401395 L 74.088 153.110512 L 77.436 168.96893 L 80.784 162.264558 L 84.132 175.673302 L 87.48 179.025488 L 90.828 173.223628 L 94.176 185.407535 L 97.524 195.012837 L 100.872 188.37293 L 104.22 203.844558 L 107.568 188.308465 L 110.916 202.36186 L 114.264 205.262791 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_40">    <path d="M 54 127.26 L 57.348 123.327628 L 60.696 120.104372 L 64.044 123.456558 L 67.392 120.684558 L 70.74 118.299349 L 74.088 121.329209 L 77.436 126.293023 L 80.784 130.032 L 84.132 97.348186 L 87.48 125.390512 L 90.828 127.582326 L 94.176 132.997395 L 97.524 134.222233 L 100.872 136.027256 L 104.22 141.12 L 107.568 172.578977 L 110.916 172.063256 L 114.264 172.514512 L 117.612 171.805395 L 120.96 170.580558 L 124.308 174.25507 L 127.656 175.028651 L 131.004 173.932744 L 134.352 192.692093 L 137.7 194.174791 L 141.048 181.733023 L 144.396 176.704744 L 147.744 184.698419 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_41">    <path d="M 54 127.26 L 57.348 123.069767 L 60.696 121.071349 L 64.044 123.456558 L 67.392 123.456558 L 70.74 119.395256 L 74.088 122.489581 L 77.436 121.135814 L 80.784 125.841767 L 84.132 126.099628 L 87.48 129.129488 L 90.828 128.291442 L 94.176 162.715814 L 97.524 161.362047 L 100.872 159.685953 L 104.22 154.786605 L 107.568 160.524 L 110.916 161.684372 L 114.264 175.157581 L 117.612 223.893209 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_42">    <path d="M 54 127.26 L 57.348 120.813488 L 60.696 125.454977 L 64.044 124.681395 L 67.392 126.035163 L 70.74 123.521023 L 74.088 127.26 L 77.436 110.241209 L 80.784 116.816651 L 84.132 110.434605 L 87.48 108.565116 L 90.828 156.849488 L 94.176 152.208 L 97.524 155.366791 L 100.872 153.626233 L 104.22 158.074326 L 107.568 159.041302 L 110.916 154.013023 L 114.264 171.096279 L 117.612 171.418605 L 120.96 136.414047 L 124.308 181.475163 L 127.656 183.473581 L 131.004 171.418605 L 134.352 179.218884 L 137.7 195.012837 L 141.048 189.146512 L 144.396 189.404372 L 147.744 181.346233 L 151.092 179.089953 L 154.44 172.127721 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_43">    <path d="M 54 127.26 L 57.348 123.907814 L 60.696 122.618512 L 64.044 123.972279 L 67.392 119.20186 L 70.74 122.876372 L 74.088 124.681395 L 77.436 126.421953 L 80.784 132.030419 L 84.132 130.16093 L 87.48 121.45814 L 90.828 118.621674 L 94.176 114.173581 L 97.524 127.711256 L 100.872 135.640465 L 104.22 141.24893 L 107.568 166.841581 L 110.916 168.84 L 114.264 223.828744 L 117.612 223.893209 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_44">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 123.714419 L 64.044 125.841767 L 67.392 127.00214 L 70.74 145.116837 L 74.088 179.928 L 77.436 173.352558 L 80.784 181.861953 L 84.132 171.612 L 87.48 183.538047 L 90.828 176.382419 L 94.176 179.734605 L 97.524 185.020744 L 100.872 181.733023 L 104.22 173.481488 L 107.568 181.088372 L 110.916 204.295814 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_45">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 122.038326 L 64.044 127.00214 L 67.392 124.423535 L 70.74 116.04307 L 74.088 121.264744 L 77.436 124.681395 L 80.784 153.110512 L 84.132 159.234698 L 87.48 166.32586 L 90.828 192.756558 L 94.176 197.462512 L 97.524 195.786419 L 100.872 182.764465 L 104.22 196.946791 L 107.568 194.948372 L 110.916 197.655907 L 114.264 195.399628 L 117.612 199.525395 L 120.96 205.456186 L 124.308 199.138605 L 127.656 209.581953 L 131.004 216.479721 L 134.352 213.44986 L 137.7 215.448279 L 141.048 201.265953 L 144.396 216.673116 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_46">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 122.038326 L 64.044 125.003721 L 67.392 115.076093 L 70.74 112.497488 L 74.088 118.105953 L 77.436 121.909395 L 80.784 141.764651 L 84.132 155.753581 L 87.48 168.388744 L 90.828 181.539628 L 94.176 178.316372 L 97.524 223.828744 L 100.872 223.893209 L 104.22 197.398047 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_47">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 122.038326 L 64.044 124.423535 L 67.392 134.415628 L 70.74 131.321302 L 74.088 125.712837 L 77.436 139.508372 L 80.784 139.508372 L 84.132 158.461116 L 87.48 171.225209 L 90.828 169.742512 L 94.176 181.990884 L 97.524 167.421767 L 100.872 175.093116 L 104.22 169.291256 L 107.568 180.250326 L 110.916 182.184279 L 114.264 195.721953 L 117.612 204.61814 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_48">    <path d="M 54 127.26 L 57.348 122.811907 L 60.696 116.42986 L 64.044 114.560372 L 67.392 118.81507 L 70.74 117.783628 L 74.088 123.134233 L 77.436 158.461116 L 80.784 175.995628 L 84.132 172.965767 L 87.48 188.630791 L 90.828 185.858791 L 94.176 196.108744 L 97.524 198.300558 L 100.872 201.910605 L 104.22 192.498698 L 107.568 195.270698 L 110.916 207.841395 L 114.264 198.107163 L 117.612 200.427907 L 120.96 199.009674 L 124.308 201.523814 L 127.656 209.259628 L 131.004 208.550512 L 134.352 214.158977 L 137.7 202.490791 L 141.048 215.70614 L 144.396 219.187256 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_49">    <path d="M 54 127.26 L 57.348 122.167256 L 60.696 123.005302 L 64.044 119.330791 L 67.392 119.717581 L 70.74 120.877953 L 74.088 146.728465 L 77.436 148.662419 L 80.784 151.176558 L 84.132 149.113674 L 87.48 183.602512 L 90.828 192.176372 L 94.176 188.308465 L 97.524 195.077302 L 100.872 197.913767 L 104.22 200.234512 L 107.568 223.828744 L 110.916 223.893209 L 114.264 223.893209 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_50">    <path d="M 54 127.26 L 57.348 119.20186 L 60.696 119.266326 L 64.044 120.942419 L 67.392 120.233302 L 70.74 125.454977 L 74.088 119.07293 L 77.436 120.426698 L 80.784 114.238047 L 84.132 115.656279 L 87.48 126.357488 L 90.828 145.116837 L 94.176 145.632558 L 97.524 152.272465 L 100.872 158.976837 L 104.22 162.586884 L 107.568 115.720744 L 110.916 162.780279 L 114.264 155.044465 L 117.612 174.384 L 120.96 182.313209 L 124.308 198.945209 L 127.656 195.464093 L 131.004 193.143349 L 134.352 193.401209 L 137.7 197.784837 L 141.048 202.942047 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_51">    <path d="M 54 127.26 L 57.348 122.811907 L 60.696 125.197116 L 64.044 124.23014 L 67.392 124.23014 L 70.74 124.488 L 74.088 123.778884 L 77.436 125.326047 L 80.784 125.454977 L 84.132 125.519442 L 87.48 120.877953 L 90.828 122.231721 L 94.176 126.293023 L 97.524 127.26 L 100.872 148.533488 L 104.22 167.421767 L 107.568 177.28493 L 110.916 171.160744 L 114.264 172.385581 L 117.612 176.769209 L 120.96 182.506605 L 124.308 177.671721 L 127.656 183.666977 L 131.004 185.020744 L 134.352 180.894977 L 137.7 174.12614 L 141.048 190.242419 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_52">    <path d="M 54 127.26 L 57.348 123.198698 L 60.696 121.84493 L 64.044 122.360651 L 67.392 117.138977 L 70.74 115.849674 L 74.088 115.205023 L 77.436 125.519442 L 80.784 152.208 L 84.132 173.739349 L 87.48 173.803814 L 90.828 167.937488 L 94.176 170.516093 L 97.524 172.385581 L 100.872 180.894977 L 104.22 181.539628 L 107.568 186.696837 L 110.916 177.607256 L 114.264 183.473581 L 117.612 190.951535 L 120.96 191.209395 L 124.308 190.822605 L 127.656 189.662233 L 131.004 184.376093 L 134.352 185.020744 L 137.7 183.924837 L 141.048 193.788 L 144.396 194.754977 L 147.744 195.979814 L 151.092 193.91693 L 154.44 198.236093 L 157.788 202.490791 L 161.136 194.368186 L 164.484 196.044279 L 167.832 197.462512 L 171.18 197.398047 L 174.528 223.893209 L 177.876 223.893209 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_53">    <path d="M 54 127.26 L 57.348 120.813488 L 60.696 115.011628 L 64.044 111.788372 L 67.392 112.368558 L 70.74 111.723907 L 74.088 115.140558 L 77.436 114.238047 L 80.784 162.071163 L 84.132 165.552279 L 87.48 181.539628 L 90.828 169.484651 L 94.176 171.676465 L 97.524 179.347814 L 100.872 177.800651 L 104.22 180.572651 L 107.568 176.89814 L 110.916 173.932744 L 114.264 166.132465 L 117.612 174.770791 L 120.96 185.794326 L 124.308 185.278605 L 127.656 182.764465 L 131.004 189.404372 L 134.352 187.534884 L 137.7 187.212558 L 141.048 187.212558 L 144.396 191.016 L 147.744 197.398047 L 151.092 202.36186 L 154.44 205.520651 L 157.788 196.108744 L 161.136 209.710884 L 164.484 213.643256 L 167.832 223.893209 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_54">    <path d="M 54 127.26 L 57.348 123.005302 L 60.696 124.810326 L 64.044 120.426698 L 67.392 123.327628 L 70.74 124.294605 L 74.088 125.906233 L 77.436 135.83386 L 80.784 146.664 L 84.132 145.568093 L 87.48 176.446884 L 90.828 160.072744 L 94.176 112.561953 L 97.524 175.415442 L 100.872 173.030233 L 104.22 171.676465 L 107.568 195.206233 L 110.916 188.244 L 114.264 196.946791 L 117.612 185.536465 L 120.96 223.893209 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_55">    <path d="M 54 127.26 L 57.348 120.813488 L 60.696 115.849674 L 64.044 120.749023 L 67.392 110.821395 L 70.74 114.173581 L 74.088 114.947163 L 77.436 112.690884 L 80.784 117.590233 L 84.132 119.717581 L 87.48 136.994233 L 90.828 125.519442 L 94.176 160.78186 L 97.524 154.657674 L 100.872 179.218884 L 104.22 182.764465 L 107.568 179.734605 L 110.916 186.567907 L 114.264 223.893209 L 117.612 223.828744 L 120.96 223.893209 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_56">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 123.714419 L 64.044 122.038326 L 67.392 104.374884 L 70.74 149.242605 L 74.088 151.047628 L 77.436 153.432837 L 80.784 150.467442 L 84.132 171.225209 L 87.48 169.935907 L 90.828 176.962605 L 94.176 171.35414 L 97.524 177.671721 L 100.872 178.703163 L 104.22 184.698419 L 107.568 186.761302 L 110.916 188.308465 L 114.264 190.822605 L 117.612 185.278605 L 120.96 181.152837 L 124.308 184.118233 L 127.656 196.68893 L 131.004 196.68893 L 134.352 189.533302 L 137.7 199.07414 L 141.048 190.435814 L 144.396 199.58986 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_57">    <path d="M 54 127.26 L 57.348 123.198698 L 60.696 121.84493 L 64.044 121.651535 L 67.392 118.879535 L 70.74 122.811907 L 74.088 120.555628 L 77.436 124.294605 L 80.784 118.68614 L 84.132 122.296186 L 87.48 140.410884 L 90.828 151.434419 L 94.176 184.182698 L 97.524 189.404372 L 100.872 202.748651 L 104.22 201.652744 L 107.568 221.894791 L 110.916 223.119628 L 114.264 222.797302 L 117.612 223.893209 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_58">    <path d="M 54 127.26 L 57.348 120.813488 L 60.696 124.681395 L 64.044 124.165674 L 67.392 125.390512 L 70.74 123.263163 L 74.088 120.813488 L 77.436 125.261581 L 80.784 124.74586 L 84.132 126.937674 L 87.48 152.401395 L 90.828 147.373116 L 94.176 154.72214 L 97.524 159.621488 L 100.872 190.435814 L 104.22 188.695256 L 107.568 189.210977 L 110.916 189.146512 L 114.264 189.210977 L 117.612 193.981395 L 120.96 191.596186 L 124.308 191.789581 L 127.656 182.893395 L 131.004 186.954698 L 134.352 181.539628 L 137.7 189.146512 L 141.048 188.759721 L 144.396 196.237674 L 147.744 192.692093 L 151.092 190.629209 L 154.44 200.105581 L 157.788 201.717209 L 161.136 199.654326 L 164.484 198.880744 L 167.832 201.781674 L 171.18 208.292651 L 174.528 201.652744 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_59">    <path d="M 54 127.26 L 57.348 122.811907 L 60.696 122.940837 L 64.044 120.168837 L 67.392 124.74586 L 70.74 118.879535 L 74.088 112.948744 L 77.436 120.233302 L 80.784 123.907814 L 84.132 130.805581 L 87.48 160.330605 L 90.828 172.578977 L 94.176 170.516093 L 97.524 187.212558 L 100.872 183.538047 L 104.22 168.066419 L 107.568 188.566326 L 110.916 195.077302 L 114.264 179.412279 L 117.612 176.317953 L 120.96 199.58986 L 124.308 200.170047 L 127.656 197.398047 L 131.004 195.979814 L 134.352 209.775349 L 137.7 198.300558 L 141.048 188.11507 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_60">    <path d="M 54 127.26 L 57.348 118.363814 L 60.696 121.071349 L 64.044 121.84493 L 67.392 124.61693 L 70.74 115.462884 L 74.088 119.330791 L 77.436 120.491163 L 80.784 133.06186 L 84.132 144.923442 L 87.48 133.319721 L 90.828 175.028651 L 94.176 170.838419 L 97.524 180.637116 L 100.872 181.604093 L 104.22 155.23786 L 107.568 185.858791 L 110.916 185.858791 L 114.264 179.992465 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_61">    <path d="M 54 127.26 L 57.348 123.198698 L 60.696 122.940837 L 64.044 124.874791 L 67.392 124.165674 L 70.74 124.23014 L 74.088 117.203442 L 77.436 122.747442 L 80.784 128.420372 L 84.132 120.749023 L 87.48 123.456558 L 90.828 125.390512 L 94.176 162.006698 L 97.524 163.102605 L 100.872 173.803814 L 104.22 179.089953 L 107.568 178.187442 L 110.916 191.402791 L 114.264 223.893209 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_62">    <path d="M 54 127.26 L 57.348 121.716 L 60.696 112.755349 L 64.044 116.687721 L 67.392 115.527349 L 70.74 159.814884 L 74.088 154.399814 L 77.436 160.65293 L 80.784 164.262977 L 84.132 164.972093 L 87.48 154.464279 L 90.828 151.176558 L 94.176 162.200093 L 97.524 182.764465 L 100.872 175.350977 L 104.22 177.542791 L 107.568 176.382419 L 110.916 184.633953 L 114.264 210.097674 L 117.612 185.34307 L 120.96 181.475163 L 124.308 199.718791 L 127.656 187.663814 L 131.004 182.44214 L 134.352 185.407535 L 137.7 200.685767 L 141.048 202.104 L 144.396 202.23293 L 147.744 213.06307 L 151.092 214.287907 L 154.44 223.893209 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_63">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 123.714419 L 64.044 124.35907 L 67.392 121.58707 L 70.74 125.454977 L 74.088 124.810326 L 77.436 127.066605 L 80.784 143.376279 L 84.132 158.654512 L 87.48 160.910791 L 90.828 171.289674 L 94.176 168.517674 L 97.524 166.068 L 100.872 168.388744 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_64">    <path d="M 54 127.26 L 57.348 123.263163 L 60.696 123.456558 L 64.044 123.488791 L 67.392 123.746651 L 70.74 123.392093 L 74.088 124.165674 L 77.436 125.454977 L 80.784 134.899116 L 84.132 147.340884 L 87.48 154.657674 L 90.828 158.171023 L 94.176 163.940651 L 97.524 167.518465 L 100.872 173.674884 L 104.22 173.578186 L 107.568 177.736186 L 110.916 179.960233 L 114.264 183.054558 L 117.612 190.790372 L 120.96 194.110326 L 124.308 201.394884 L 127.656 206.455395 L 131.004 212.257256 L 134.352 215.641674 L 137.7 223.893209 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pb354075671)" style="fill: none; stroke: #000000; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="patch_3">    <path d="M 54 224.28 L 54 30.24 " style="fill: none"/>   </g>   <g id="patch_4">    <path d="M 388.8 224.28 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="patch_5">    <path d="M 54 224.28 L 388.8 224.28 " style="fill: none"/>   </g>   <g id="patch_6">    <path d="M 54 30.24 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="text_16">    <g id="patch_7">     <a xlink:href="https://dynomight.net/chess">      <path d="M 323.8664 225.676606 L 388.881244 225.676606 L 388.881244 213.343012 L 323.8664 213.343012 z" style="fill: #ffffff; opacity: 0.000001; stroke: #ffffff; stroke-linejoin: miter"/>     </a>    </g>    <a xlink:href="https://dynomight.net/chess">     <!-- dynomight.net/chess -->     <g style="opacity: 0.3" transform="translate(327.8664 220.734419) scale(0.05 -0.05)">      <defs>       <path id="CourierNewPSMT-64" d="M 3206 3922 L 3206 263 L 3556 263 Q 3653 263 3693 227 Q 3734 191 3734 131 Q 3734 75 3693 37 Q 3653 0 3556 0 L 2941 0 L 2941 522 Q 2484 -103 1781 -103 Q 1425 -103 1098 86 Q 772 275 583 625 Q 394 975 394 1347 Q 394 1722 583 2070 Q 772 2419 1098 2609 Q 1425 2800 1784 2800 Q 2472 2800 2941 2175 L 2941 3656 L 2591 3656 Q 2494 3656 2453 3693 Q 2413 3731 2413 3791 Q 2413 3847 2453 3884 Q 2494 3922 2591 3922 L 3206 3922 zM 2941 1347 Q 2941 1847 2603 2190 Q 2266 2534 1800 2534 Q 1331 2534 993 2190 Q 656 1847 656 1347 Q 656 850 993 504 Q 1331 159 1800 159 Q 2266 159 2603 504 Q 2941 850 2941 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-79" d="M 1938 0 L 719 2441 L 641 2441 Q 547 2441 506 2478 Q 466 2516 466 2572 Q 466 2613 486 2645 Q 506 2678 539 2692 Q 572 2706 641 2706 L 1359 2706 Q 1453 2706 1493 2668 Q 1534 2631 1534 2572 Q 1534 2516 1493 2478 Q 1453 2441 1359 2441 L 1006 2441 L 2078 288 L 3134 2441 L 2781 2441 Q 2688 2441 2647 2478 Q 2606 2516 2606 2575 Q 2606 2631 2647 2668 Q 2688 2706 2781 2706 L 3497 2706 Q 3594 2706 3634 2668 Q 3675 2631 3675 2572 Q 3675 2531 3650 2497 Q 3625 2463 3594 2452 Q 3563 2441 3419 2441 L 1756 -944 L 2166 -944 Q 2259 -944 2300 -980 Q 2341 -1016 2341 -1075 Q 2341 -1131 2300 -1168 Q 2259 -1206 2166 -1206 L 653 -1206 Q 559 -1206 518 -1170 Q 478 -1134 478 -1075 Q 478 -1016 518 -980 Q 559 -944 653 -944 L 1475 -944 L 1938 0 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6e" d="M 1072 2706 L 1072 2309 Q 1347 2588 1569 2694 Q 1791 2800 2069 2800 Q 2369 2800 2616 2672 Q 2791 2578 2933 2361 Q 3075 2144 3075 1916 L 3075 263 L 3297 263 Q 3391 263 3431 227 Q 3472 191 3472 131 Q 3472 75 3431 37 Q 3391 0 3297 0 L 2594 0 Q 2497 0 2456 37 Q 2416 75 2416 131 Q 2416 191 2456 227 Q 2497 263 2594 263 L 2813 263 L 2813 1872 Q 2813 2150 2609 2342 Q 2406 2534 2066 2534 Q 1806 2534 1615 2429 Q 1425 2325 1072 1909 L 1072 263 L 1369 263 Q 1463 263 1503 227 Q 1544 191 1544 131 Q 1544 75 1503 37 Q 1463 0 1369 0 L 513 0 Q 419 0 378 37 Q 338 75 338 131 Q 338 191 378 227 Q 419 263 513 263 L 809 263 L 809 2441 L 588 2441 Q 494 2441 453 2478 Q 413 2516 413 2575 Q 413 2631 453 2668 Q 494 2706 588 2706 L 1072 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6f" d="M 3388 1347 Q 3388 747 2958 322 Q 2528 -103 1922 -103 Q 1309 -103 881 323 Q 453 750 453 1347 Q 453 1947 881 2373 Q 1309 2800 1922 2800 Q 2528 2800 2958 2375 Q 3388 1950 3388 1347 zM 3122 1347 Q 3122 1841 2770 2187 Q 2419 2534 1919 2534 Q 1419 2534 1067 2186 Q 716 1838 716 1347 Q 716 859 1067 509 Q 1419 159 1919 159 Q 2419 159 2770 507 Q 3122 856 3122 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6d" d="M 725 2706 L 725 2441 Q 1059 2800 1397 2800 Q 1600 2800 1753 2692 Q 1906 2584 2009 2366 Q 2184 2584 2364 2692 Q 2544 2800 2725 2800 Q 3009 2800 3178 2616 Q 3400 2378 3400 2097 L 3400 263 L 3622 263 Q 3716 263 3756 227 Q 3797 191 3797 131 Q 3797 75 3756 37 Q 3716 0 3622 0 L 3138 0 L 3138 2072 Q 3138 2272 3016 2403 Q 2894 2534 2734 2534 Q 2591 2534 2431 2426 Q 2272 2319 2069 2003 L 2069 263 L 2288 263 Q 2381 263 2422 227 Q 2463 191 2463 131 Q 2463 75 2422 37 Q 2381 0 2288 0 L 1803 0 L 1803 2053 Q 1803 2263 1679 2398 Q 1556 2534 1403 2534 Q 1263 2534 1125 2444 Q 934 2316 725 2003 L 725 263 L 947 263 Q 1041 263 1081 227 Q 1122 191 1122 131 Q 1122 75 1081 37 Q 1041 0 947 0 L 241 0 Q 147 0 106 37 Q 66 75 66 131 Q 66 191 106 227 Q 147 263 241 263 L 463 263 L 463 2441 L 241 2441 Q 147 2441 106 2478 Q 66 2516 66 2575 Q 66 2631 106 2668 Q 147 2706 241 2706 L 725 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-69" d="M 2050 4053 L 2050 3375 L 1663 3375 L 1663 4053 L 2050 4053 zM 2059 2706 L 2059 263 L 3091 263 Q 3188 263 3228 227 Q 3269 191 3269 131 Q 3269 75 3228 37 Q 3188 0 3091 0 L 766 0 Q 672 0 631 37 Q 591 75 591 131 Q 591 191 631 227 Q 672 263 766 263 L 1797 263 L 1797 2441 L 1031 2441 Q 938 2441 895 2478 Q 853 2516 853 2572 Q 853 2631 893 2668 Q 934 2706 1031 2706 L 2059 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-67" d="M 2803 2225 L 2803 2706 L 3419 2706 Q 3513 2706 3553 2668 Q 3594 2631 3594 2572 Q 3594 2516 3553 2478 Q 3513 2441 3419 2441 L 3066 2441 L 3066 -181 Q 3066 -444 2953 -650 Q 2878 -788 2703 -934 Q 2528 -1081 2384 -1143 Q 2241 -1206 2000 -1206 L 1256 -1206 Q 1163 -1206 1122 -1170 Q 1081 -1134 1081 -1075 Q 1081 -1016 1122 -978 Q 1163 -941 1256 -941 L 2009 -944 Q 2241 -944 2427 -825 Q 2613 -706 2734 -478 Q 2803 -347 2803 -144 L 2803 644 Q 2381 66 1716 66 Q 1175 66 783 467 Q 391 869 391 1434 Q 391 2000 783 2400 Q 1175 2800 1716 2800 Q 2381 2800 2803 2225 zM 2803 1434 Q 2803 1897 2486 2215 Q 2169 2534 1728 2534 Q 1288 2534 972 2214 Q 656 1894 656 1434 Q 656 972 972 651 Q 1288 331 1728 331 Q 2169 331 2486 651 Q 2803 972 2803 1434 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-68" d="M 1056 3922 L 1056 2306 Q 1306 2578 1536 2689 Q 1766 2800 2050 2800 Q 2356 2800 2570 2692 Q 2784 2584 2928 2361 Q 3072 2138 3072 1891 L 3072 263 L 3366 263 Q 3463 263 3502 227 Q 3541 191 3541 131 Q 3541 75 3502 37 Q 3463 0 3366 0 L 2513 0 Q 2416 0 2375 37 Q 2334 75 2334 131 Q 2334 191 2375 227 Q 2416 263 2513 263 L 2806 263 L 2806 1872 Q 2806 2156 2601 2347 Q 2397 2538 2031 2538 Q 1744 2538 1541 2397 Q 1394 2297 1056 1925 L 1056 263 L 1353 263 Q 1447 263 1487 227 Q 1528 191 1528 131 Q 1528 75 1487 37 Q 1447 0 1353 0 L 497 0 Q 403 0 362 37 Q 322 75 322 131 Q 322 191 362 227 Q 403 263 497 263 L 794 263 L 794 3656 L 441 3656 Q 347 3656 306 3693 Q 266 3731 266 3791 Q 266 3847 306 3884 Q 347 3922 441 3922 L 1056 3922 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-74" d="M 1381 2706 L 2806 2706 Q 2900 2706 2940 2668 Q 2981 2631 2981 2572 Q 2981 2516 2940 2478 Q 2900 2441 2806 2441 L 1381 2441 L 1381 694 Q 1381 466 1564 312 Q 1747 159 2100 159 Q 2366 159 2675 239 Q 2984 319 3156 419 Q 3219 459 3259 459 Q 3309 459 3346 420 Q 3384 381 3384 328 Q 3384 281 3344 241 Q 3244 138 2858 17 Q 2472 -103 2119 -103 Q 1659 -103 1387 112 Q 1116 328 1116 694 L 1116 2441 L 631 2441 Q 538 2441 497 2478 Q 456 2516 456 2575 Q 456 2631 497 2668 Q 538 2706 631 2706 L 1116 2706 L 1116 3481 Q 1116 3575 1153 3615 Q 1191 3656 1247 3656 Q 1306 3656 1343 3615 Q 1381 3575 1381 3481 L 1381 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2e" d="M 1866 753 L 1978 753 Q 2153 753 2278 629 Q 2403 506 2403 331 Q 2403 150 2276 28 Q 2150 -94 1978 -94 L 1866 -94 Q 1691 -94 1566 29 Q 1441 153 1441 328 Q 1441 509 1567 631 Q 1694 753 1866 753 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-65" d="M 3341 1291 L 666 1291 Q 734 781 1092 470 Q 1450 159 1978 159 Q 2272 159 2594 256 Q 2916 353 3119 513 Q 3178 559 3222 559 Q 3272 559 3309 520 Q 3347 481 3347 428 Q 3347 375 3297 325 Q 3147 169 2764 33 Q 2381 -103 1978 -103 Q 1303 -103 851 339 Q 400 781 400 1409 Q 400 1981 823 2390 Q 1247 2800 1872 2800 Q 2516 2800 2931 2379 Q 3347 1959 3341 1291 zM 3075 1556 Q 2997 1991 2664 2262 Q 2331 2534 1872 2534 Q 1413 2534 1081 2265 Q 750 1997 669 1556 L 3075 1556 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2f" d="M 3100 4113 L 988 -413 Q 934 -528 850 -528 Q 797 -528 758 -489 Q 719 -450 719 -403 Q 719 -366 750 -300 L 2859 4225 Q 2894 4297 2925 4319 Q 2956 4341 3000 4341 Q 3053 4341 3092 4302 Q 3131 4263 3131 4216 Q 3131 4181 3100 4113 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-63" d="M 3016 2441 L 3016 2528 Q 3016 2625 3053 2665 Q 3091 2706 3147 2706 Q 3206 2706 3243 2665 Q 3281 2625 3281 2528 L 3281 1934 Q 3278 1838 3242 1797 Q 3206 1756 3147 1756 Q 3094 1756 3058 1792 Q 3022 1828 3016 1913 Q 2997 2134 2723 2334 Q 2450 2534 1988 2534 Q 1403 2534 1100 2168 Q 797 1803 797 1331 Q 797 822 1131 490 Q 1466 159 1997 159 Q 2303 159 2620 271 Q 2938 384 3194 634 Q 3259 697 3309 697 Q 3363 697 3398 661 Q 3434 625 3434 572 Q 3434 438 3119 231 Q 2609 -103 1984 -103 Q 1350 -103 942 301 Q 534 706 534 1328 Q 534 1963 951 2381 Q 1369 2800 2003 2800 Q 2606 2800 3016 2441 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-73" d="M 2788 2534 Q 2788 2625 2825 2665 Q 2863 2706 2919 2706 Q 2978 2706 3015 2665 Q 3053 2625 3053 2528 L 3053 2078 Q 3053 1984 3015 1943 Q 2978 1903 2919 1903 Q 2866 1903 2830 1937 Q 2794 1972 2788 2050 Q 2769 2238 2594 2359 Q 2338 2534 1916 2534 Q 1475 2534 1231 2356 Q 1047 2222 1047 2056 Q 1047 1869 1266 1744 Q 1416 1656 1834 1609 Q 2381 1550 2594 1475 Q 2897 1366 3045 1172 Q 3194 978 3194 753 Q 3194 419 2872 158 Q 2550 -103 1928 -103 Q 1306 -103 909 213 Q 909 106 896 75 Q 884 44 851 22 Q 819 0 778 0 Q 722 0 684 40 Q 647 81 647 175 L 647 716 Q 647 809 683 850 Q 719 891 778 891 Q 834 891 873 852 Q 913 813 913 747 Q 913 603 984 506 Q 1094 356 1333 257 Q 1572 159 1919 159 Q 2431 159 2681 350 Q 2931 541 2931 753 Q 2931 997 2678 1144 Q 2422 1291 1933 1341 Q 1444 1391 1231 1472 Q 1019 1553 900 1715 Q 781 1878 781 2066 Q 781 2403 1112 2601 Q 1444 2800 1903 2800 Q 2447 2800 2788 2534 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#CourierNewPSMT-64"/>      <use xlink:href="#CourierNewPSMT-79" x="60.009766"/>      <use xlink:href="#CourierNewPSMT-6e" x="120.019531"/>      <use xlink:href="#CourierNewPSMT-6f" x="180.029297"/>      <use xlink:href="#CourierNewPSMT-6d" x="240.039062"/>      <use xlink:href="#CourierNewPSMT-69" x="300.048828"/>      <use xlink:href="#CourierNewPSMT-67" x="360.058594"/>      <use xlink:href="#CourierNewPSMT-68" x="420.068359"/>      <use xlink:href="#CourierNewPSMT-74" x="480.078125"/>      <use xlink:href="#CourierNewPSMT-2e" x="540.087891"/>      <use xlink:href="#CourierNewPSMT-6e" x="600.097656"/>      <use xlink:href="#CourierNewPSMT-65" x="660.107422"/>      <use xlink:href="#CourierNewPSMT-74" x="720.117188"/>      <use xlink:href="#CourierNewPSMT-2f" x="780.126953"/>      <use xlink:href="#CourierNewPSMT-63" x="840.136719"/>      <use xlink:href="#CourierNewPSMT-68" x="900.146484"/>      <use xlink:href="#CourierNewPSMT-65" x="960.15625"/>      <use xlink:href="#CourierNewPSMT-73" x="1020.166016"/>      <use xlink:href="#CourierNewPSMT-73" x="1080.175781"/>     </g>    </a>   </g>   <g id="text_17">    <!-- command-r-v01 vs stockfish -->    <g transform="translate(137.386875 24.24) scale(0.12 -0.12)">     <defs>      <path id="Montserrat-Regular-6f" d="M 2010 -32 Q 1523 -32 1132 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1132 3174 Q 1523 3392 2010 3392 Q 2496 3392 2886 3174 Q 3277 2957 3497 2566 Q 3718 2176 3718 1683 Q 3718 1190 3497 800 Q 3277 410 2886 189 Q 2496 -32 2010 -32 zM 2010 371 Q 2368 371 2653 534 Q 2938 698 3098 998 Q 3258 1299 3258 1683 Q 3258 2067 3098 2368 Q 2938 2669 2653 2832 Q 2368 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6d" d="M 4832 3392 Q 5466 3392 5827 3027 Q 6189 2662 6189 1952 L 6189 0 L 5734 0 L 5734 1907 Q 5734 2432 5481 2707 Q 5229 2982 4768 2982 Q 4243 2982 3942 2659 Q 3642 2336 3642 1766 L 3642 0 L 3187 0 L 3187 1907 Q 3187 2432 2934 2707 Q 2682 2982 2214 2982 Q 1696 2982 1392 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2752 Q 1248 3059 1568 3225 Q 1888 3392 2304 3392 Q 2726 3392 3036 3213 Q 3347 3034 3501 2682 Q 3686 3014 4035 3203 Q 4384 3392 4832 3392 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-76" d="M 3462 3366 L 1965 0 L 1498 0 L 6 3366 L 480 3366 L 1741 499 L 3014 3366 L 3462 3366 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6b" d="M 1882 1690 L 1088 960 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 1530 L 3098 3366 L 3661 3366 L 2221 1990 L 3795 0 L 3238 0 L 1882 1690 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-68" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2778 Q 1274 3072 1600 3232 Q 1926 3392 2349 3392 z" transform="scale(0.015625)"/>     </defs>     <use xlink:href="#Montserrat-Regular-63"/>     <use xlink:href="#Montserrat-Regular-6f" x="56.299988"/>     <use xlink:href="#Montserrat-Regular-6d" x="118.999985"/>     <use xlink:href="#Montserrat-Regular-6d" x="225.099976"/>     <use xlink:href="#Montserrat-Regular-61" x="331.199966"/>     <use xlink:href="#Montserrat-Regular-6e" x="390.199951"/>     <use xlink:href="#Montserrat-Regular-64" x="457.899933"/>     <use xlink:href="#Montserrat-Regular-2d" x="525.699921"/>     <use xlink:href="#Montserrat-Regular-72" x="563.899918"/>     <use xlink:href="#Montserrat-Regular-2d" x="603.999908"/>     <use xlink:href="#Montserrat-Regular-76" x="642.199905"/>     <use xlink:href="#Montserrat-Regular-30" x="696.399902"/>     <use xlink:href="#Montserrat-Regular-31" x="762.599899"/>     <use xlink:href="#Montserrat-Regular-20" x="798.69989"/>     <use xlink:href="#Montserrat-Regular-76" x="824.899887"/>     <use xlink:href="#Montserrat-Regular-73" x="879.099884"/>     <use xlink:href="#Montserrat-Regular-20" x="927.999878"/>     <use xlink:href="#Montserrat-Regular-73" x="954.199875"/>     <use xlink:href="#Montserrat-Regular-74" x="1003.099869"/>     <use xlink:href="#Montserrat-Regular-6f" x="1043.69986"/>     <use xlink:href="#Montserrat-Regular-63" x="1106.399857"/>     <use xlink:href="#Montserrat-Regular-6b" x="1162.699844"/>     <use xlink:href="#Montserrat-Regular-66" x="1222.799835"/>     <use xlink:href="#Montserrat-Regular-69" x="1256.699829"/>     <use xlink:href="#Montserrat-Regular-73" x="1283.599823"/>     <use xlink:href="#Montserrat-Regular-68" x="1332.499817"/>    </g>   </g>  </g> </g> <defs>  <clipPath id="pb354075671">   <rect x="54" y="30.24" width="334.8" height="194.04"/>  </clipPath> </defs></svg>](https://dynomight.net/img/chess/command-r-v01_stockfish.pdf)

[Terrible.](https://lichess.org/TrkfFSTA)

And then I tried [`gemma-2-27b`](https://huggingface.co/google/gemma-2-27b).

[<?xml version="1.0" encoding="utf-8" standalone="no"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"  "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg xmlns:xlink="http://www.w3.org/1999/xlink" width="432pt" height="252pt" viewBox="0 0 432 252" xmlns="http://www.w3.org/2000/svg" version="1.1"> <metadata>  <rdf:RDF xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:cc="http://creativecommons.org/ns#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">   <cc:Work>    <dc:type rdf:resource="http://purl.org/dc/dcmitype/StillImage"/>    <dc:date>2024-11-14T11:33:45.614717</dc:date>    <dc:format>image/svg+xml</dc:format>    <dc:creator>     <cc:Agent>      <dc:title>Matplotlib v3.8.4, https://matplotlib.org/</dc:title>     </cc:Agent>    </dc:creator>   </cc:Work>  </rdf:RDF> </metadata> <defs>  <style type="text/css">*{stroke-linejoin: round; stroke-linecap: butt}</style> </defs> <g id="figure_1">  <g id="patch_1">   <path d="M 0 252 L 432 252 L 432 0 L 0 0 z" style="fill: #ffffff"/>  </g>  <g id="axes_1">   <g id="patch_2">    <path d="M 54 224.28 L 388.8 224.28 L 388.8 30.24 L 54 30.24 z" style="fill: #ffffff"/>   </g>   <g id="matplotlib.axis_1">    <g id="xtick_1">     <g id="line2d_1"/>     <g id="text_1">      <!-- 0 -->      <g transform="translate(50.689844 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-30" d="M 2118 -38 Q 1600 -38 1193 237 Q 787 512 560 1027 Q 333 1542 333 2240 Q 333 2938 560 3453 Q 787 3968 1193 4243 Q 1600 4518 2118 4518 Q 2637 4518 3043 4243 Q 3450 3968 3680 3453 Q 3910 2938 3910 2240 Q 3910 1542 3680 1027 Q 3450 512 3043 237 Q 2637 -38 2118 -38 zM 2118 384 Q 2509 384 2806 601 Q 3104 819 3270 1235 Q 3437 1651 3437 2240 Q 3437 2829 3270 3245 Q 3104 3661 2806 3878 Q 2509 4096 2118 4096 Q 1728 4096 1430 3878 Q 1133 3661 966 3245 Q 800 2829 800 2240 Q 800 1651 966 1235 Q 1133 819 1430 601 Q 1728 384 2118 384 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="xtick_2">     <g id="line2d_2"/>     <g id="text_2">      <!-- 20 -->      <g transform="translate(114.81 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-32" d="M 3392 410 L 3392 0 L 250 0 L 250 326 L 2112 2150 Q 2464 2496 2589 2749 Q 2714 3002 2714 3258 Q 2714 3654 2442 3875 Q 2170 4096 1664 4096 Q 877 4096 442 3597 L 115 3878 Q 378 4186 784 4352 Q 1190 4518 1702 4518 Q 2387 4518 2787 4195 Q 3187 3872 3187 3309 Q 3187 2963 3033 2643 Q 2880 2323 2451 1907 L 928 410 L 3392 410 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-32"/>       <use xlink:href="#Montserrat-Regular-30" x="56.799988"/>      </g>     </g>    </g>    <g id="xtick_3">     <g id="line2d_3"/>     <g id="text_3">      <!-- 40 -->      <g transform="translate(181.305156 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-34" d="M 4141 1178 L 3245 1178 L 3245 0 L 2784 0 L 2784 1178 L 256 1178 L 256 1510 L 2630 4480 L 3142 4480 L 851 1587 L 2797 1587 L 2797 2624 L 3245 2624 L 3245 1587 L 4141 1587 L 4141 1178 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-34"/>       <use xlink:href="#Montserrat-Regular-30" x="66.099991"/>      </g>     </g>    </g>    <g id="xtick_4">     <g id="line2d_4"/>     <g id="text_4">      <!-- 60 -->      <g transform="translate(248.524531 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-36" d="M 2189 2682 Q 2630 2682 2972 2515 Q 3315 2349 3507 2045 Q 3699 1741 3699 1338 Q 3699 928 3500 614 Q 3302 301 2953 131 Q 2605 -38 2170 -38 Q 1280 -38 806 547 Q 333 1133 333 2202 Q 333 2950 589 3468 Q 845 3987 1305 4252 Q 1766 4518 2381 4518 Q 3046 4518 3443 4275 L 3258 3904 Q 2938 4115 2387 4115 Q 1645 4115 1222 3644 Q 800 3174 800 2278 Q 800 2118 819 1914 Q 986 2278 1350 2480 Q 1715 2682 2189 2682 zM 2150 352 Q 2637 352 2941 617 Q 3245 883 3245 1325 Q 3245 1766 2941 2028 Q 2637 2291 2118 2291 Q 1779 2291 1520 2163 Q 1261 2035 1113 1811 Q 966 1587 966 1312 Q 966 1056 1104 838 Q 1242 621 1510 486 Q 1779 352 2150 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-36"/>       <use xlink:href="#Montserrat-Regular-30" x="60.899994"/>      </g>     </g>    </g>    <g id="xtick_5">     <g id="line2d_5"/>     <g id="text_5">      <!-- 80 -->      <g transform="translate(315.34 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-38" d="M 2899 2349 Q 3322 2208 3549 1926 Q 3776 1645 3776 1248 Q 3776 858 3565 566 Q 3354 275 2960 118 Q 2566 -38 2035 -38 Q 1229 -38 768 307 Q 307 653 307 1248 Q 307 1651 528 1929 Q 749 2208 1171 2349 Q 826 2483 646 2736 Q 467 2989 467 3334 Q 467 3878 892 4198 Q 1318 4518 2035 4518 Q 2509 4518 2864 4374 Q 3219 4230 3417 3961 Q 3616 3693 3616 3334 Q 3616 2989 3430 2736 Q 3245 2483 2899 2349 zM 928 3322 Q 928 2950 1222 2732 Q 1517 2515 2035 2515 Q 2554 2515 2851 2732 Q 3149 2950 3149 3315 Q 3149 3693 2845 3910 Q 2541 4128 2035 4128 Q 1523 4128 1225 3910 Q 928 3693 928 3322 zM 2035 352 Q 2630 352 2966 595 Q 3302 838 3302 1254 Q 3302 1670 2966 1910 Q 2630 2150 2035 2150 Q 1446 2150 1110 1910 Q 774 1670 774 1254 Q 774 832 1107 592 Q 1440 352 2035 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-38"/>       <use xlink:href="#Montserrat-Regular-30" x="63.799988"/>      </g>     </g>    </g>    <g id="xtick_6">     <g id="line2d_6"/>     <g id="text_6">      <!-- 100 -->      <g transform="translate(380.375 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-31" d="M 1574 4480 L 1574 0 L 1114 0 L 1114 4070 L 58 4070 L 58 4480 L 1574 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>      </g>     </g>    </g>    <g id="text_7">     <!-- turns -->     <g transform="translate(208.170312 248.56125) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-74" d="M 2406 205 Q 2278 90 2089 29 Q 1901 -32 1696 -32 Q 1222 -32 966 224 Q 710 480 710 947 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 4102 L 1165 4102 L 1165 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 973 Q 1165 672 1315 515 Q 1466 358 1747 358 Q 1888 358 2019 403 Q 2150 448 2246 531 L 2406 205 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-75" d="M 3667 3366 L 3667 0 L 3232 0 L 3232 614 Q 3053 307 2739 137 Q 2426 -32 2022 -32 Q 1363 -32 982 336 Q 602 704 602 1414 L 602 3366 L 1056 3366 L 1056 1459 Q 1056 928 1318 653 Q 1581 378 2067 378 Q 2598 378 2905 701 Q 3213 1024 3213 1600 L 3213 3366 L 3667 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-72" d="M 1069 2707 Q 1229 3046 1545 3219 Q 1862 3392 2330 3392 L 2330 2950 L 2221 2957 Q 1690 2957 1389 2630 Q 1088 2304 1088 1715 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2707 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-6e" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2746 Q 1254 3053 1584 3222 Q 1914 3392 2349 3392 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-73" d="M 1536 -32 Q 1126 -32 752 86 Q 378 205 166 384 L 371 742 Q 582 576 902 470 Q 1222 365 1568 365 Q 2029 365 2249 509 Q 2470 653 2470 915 Q 2470 1101 2348 1206 Q 2227 1312 2041 1366 Q 1856 1421 1549 1472 Q 1139 1549 889 1629 Q 640 1709 464 1901 Q 288 2093 288 2432 Q 288 2854 640 3123 Q 992 3392 1619 3392 Q 1946 3392 2272 3305 Q 2598 3219 2810 3078 L 2611 2714 Q 2195 3002 1619 3002 Q 1184 3002 963 2848 Q 742 2694 742 2445 Q 742 2253 867 2137 Q 992 2022 1177 1968 Q 1363 1914 1690 1856 Q 2093 1779 2336 1702 Q 2579 1626 2752 1440 Q 2925 1254 2925 928 Q 2925 486 2557 227 Q 2189 -32 1536 -32 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-74"/>      <use xlink:href="#Montserrat-Regular-75" x="40.599991"/>      <use xlink:href="#Montserrat-Regular-72" x="107.899979"/>      <use xlink:href="#Montserrat-Regular-6e" x="147.999969"/>      <use xlink:href="#Montserrat-Regular-73" x="215.699951"/>     </g>    </g>   </g>   <g id="matplotlib.axis_2">    <g id="ytick_1">     <g id="line2d_7"/>     <g id="text_8">      <!-- LOSS -->      <g transform="translate(23.91875 227.667831) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-4c" d="M 730 4480 L 1203 4480 L 1203 410 L 3712 410 L 3712 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4f" d="M 2688 -38 Q 2022 -38 1481 259 Q 941 557 637 1075 Q 333 1594 333 2240 Q 333 2886 637 3404 Q 941 3923 1481 4220 Q 2022 4518 2688 4518 Q 3354 4518 3888 4224 Q 4422 3930 4729 3408 Q 5037 2886 5037 2240 Q 5037 1594 4729 1072 Q 4422 550 3888 256 Q 3354 -38 2688 -38 zM 2688 384 Q 3219 384 3648 624 Q 4077 864 4320 1289 Q 4563 1715 4563 2240 Q 4563 2765 4320 3190 Q 4077 3616 3648 3856 Q 3219 4096 2688 4096 Q 2157 4096 1725 3856 Q 1293 3616 1046 3190 Q 800 2765 800 2240 Q 800 1715 1046 1289 Q 1293 864 1725 624 Q 2157 384 2688 384 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-53" d="M 1965 -38 Q 1466 -38 1008 122 Q 550 282 301 544 L 486 909 Q 730 666 1133 515 Q 1536 365 1965 365 Q 2566 365 2867 585 Q 3168 806 3168 1158 Q 3168 1427 3005 1587 Q 2842 1747 2605 1833 Q 2368 1920 1946 2022 Q 1440 2150 1139 2268 Q 838 2387 624 2630 Q 410 2874 410 3290 Q 410 3629 589 3907 Q 768 4186 1139 4352 Q 1510 4518 2061 4518 Q 2445 4518 2813 4412 Q 3181 4307 3450 4122 L 3290 3744 Q 3008 3930 2688 4022 Q 2368 4115 2061 4115 Q 1472 4115 1174 3888 Q 877 3661 877 3302 Q 877 3034 1040 2870 Q 1203 2707 1449 2620 Q 1696 2534 2112 2432 Q 2605 2310 2909 2192 Q 3213 2074 3424 1834 Q 3635 1594 3635 1184 Q 3635 845 3452 566 Q 3270 288 2892 125 Q 2515 -38 1965 -38 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-4c"/>       <use xlink:href="#Montserrat-Regular-4f" x="58.899994"/>       <use xlink:href="#Montserrat-Regular-53" x="142.799988"/>       <use xlink:href="#Montserrat-Regular-53" x="204.299973"/>      </g>     </g>    </g>    <g id="ytick_2">     <g id="line2d_8"/>     <g id="text_9">      <!-- -1000 -->      <g transform="translate(23.209375 195.435273) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-2d" d="M 384 1933 L 2061 1933 L 2061 1536 L 384 1536 L 384 1933 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-31" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="74.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="140.499985"/>       <use xlink:href="#Montserrat-Regular-30" x="206.699982"/>      </g>     </g>    </g>    <g id="ytick_3">     <g id="line2d_9"/>     <g id="text_10">      <!-- -500 -->      <g transform="translate(27.779687 163.202714) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-35" d="M 1510 2643 Q 2483 2643 2928 2294 Q 3373 1946 3373 1318 Q 3373 928 3190 621 Q 3008 314 2643 138 Q 2278 -38 1747 -38 Q 1267 -38 835 118 Q 403 275 141 538 L 365 902 Q 589 672 953 528 Q 1318 384 1741 384 Q 2298 384 2602 633 Q 2906 883 2906 1299 Q 2906 1760 2563 1997 Q 2221 2234 1408 2234 L 454 2234 L 685 4480 L 3136 4480 L 3136 4070 L 1088 4070 L 934 2643 L 1510 2643 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-35" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="94.799988"/>       <use xlink:href="#Montserrat-Regular-30" x="160.999985"/>      </g>     </g>    </g>    <g id="ytick_4">     <g id="line2d_10"/>     <g id="text_11">      <!-- 0 -->      <g transform="translate(43.879688 130.970156) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="ytick_5">     <g id="line2d_11"/>     <g id="text_12">      <!-- 500 -->      <g transform="translate(31.6 98.737598) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-35"/>       <use xlink:href="#Montserrat-Regular-30" x="56.599991"/>       <use xlink:href="#Montserrat-Regular-30" x="122.799988"/>      </g>     </g>    </g>    <g id="ytick_6">     <g id="line2d_12"/>     <g id="text_13">      <!-- 1000 -->      <g transform="translate(27.029687 66.50504) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="168.499985"/>      </g>     </g>    </g>    <g id="ytick_7">     <g id="line2d_13"/>     <g id="text_14">      <!-- WIN -->      <g transform="translate(28.240625 34.272482) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-57" d="M 6880 4480 L 5370 0 L 4870 0 L 3558 3821 L 2240 0 L 1747 0 L 237 4480 L 723 4480 L 2016 627 L 3354 4480 L 3795 4480 L 5114 608 L 6426 4480 L 6880 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-49" d="M 730 4480 L 1203 4480 L 1203 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4e" d="M 4474 4480 L 4474 0 L 4083 0 L 1203 3635 L 1203 0 L 730 0 L 730 4480 L 1120 4480 L 4006 845 L 4006 4480 L 4474 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-57"/>       <use xlink:href="#Montserrat-Regular-49" x="111.099991"/>       <use xlink:href="#Montserrat-Regular-4e" x="141.299988"/>      </g>     </g>    </g>    <g id="text_15">     <!-- centipawn difference -->     <g transform="translate(17.26875 180.677188) rotate(-90) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-63" d="M 2029 -32 Q 1530 -32 1136 185 Q 742 403 518 796 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1136 3174 Q 1530 3392 2029 3392 Q 2464 3392 2806 3222 Q 3149 3053 3347 2726 L 3008 2496 Q 2842 2746 2586 2870 Q 2330 2995 2029 2995 Q 1664 2995 1373 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1293 918 995 Q 1082 698 1373 534 Q 1664 371 2029 371 Q 2330 371 2586 492 Q 2842 614 3008 864 L 3347 634 Q 3149 307 2803 137 Q 2458 -32 2029 -32 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-65" d="M 3565 1542 L 749 1542 Q 787 1018 1152 694 Q 1517 371 2074 371 Q 2387 371 2649 483 Q 2912 595 3104 813 L 3360 518 Q 3136 250 2800 109 Q 2464 -32 2061 -32 Q 1542 -32 1142 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 508 2566 Q 723 2957 1097 3174 Q 1472 3392 1939 3392 Q 2406 3392 2777 3174 Q 3149 2957 3360 2569 Q 3571 2182 3571 1683 L 3565 1542 zM 1939 3002 Q 1453 3002 1123 2691 Q 794 2381 749 1882 L 3136 1882 Q 3091 2381 2761 2691 Q 2432 3002 1939 3002 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-69" d="M 634 3366 L 1088 3366 L 1088 0 L 634 0 L 634 3366 zM 864 4102 Q 723 4102 627 4198 Q 531 4294 531 4429 Q 531 4557 627 4653 Q 723 4749 864 4749 Q 1005 4749 1101 4656 Q 1197 4563 1197 4435 Q 1197 4294 1101 4198 Q 1005 4102 864 4102 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-70" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1178 3827 787 Q 3610 397 3229 182 Q 2848 -32 2368 -32 Q 1958 -32 1628 137 Q 1299 307 1088 634 L 1088 -1242 L 634 -1242 L 634 3366 L 1069 3366 L 1069 2701 Q 1274 3034 1610 3213 Q 1946 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2364 Q 3264 2662 2976 2828 Q 2688 2995 2336 2995 Q 1978 2995 1693 2828 Q 1408 2662 1245 2364 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-61" d="M 1811 3392 Q 2470 3392 2822 3062 Q 3174 2733 3174 2086 L 3174 0 L 2739 0 L 2739 525 Q 2586 262 2288 115 Q 1990 -32 1581 -32 Q 1018 -32 685 237 Q 352 506 352 947 Q 352 1376 662 1638 Q 973 1901 1651 1901 L 2720 1901 L 2720 2106 Q 2720 2541 2477 2768 Q 2234 2995 1766 2995 Q 1446 2995 1152 2889 Q 858 2784 646 2598 L 442 2938 Q 698 3155 1056 3273 Q 1414 3392 1811 3392 zM 1651 326 Q 2035 326 2310 502 Q 2586 678 2720 1011 L 2720 1562 L 1664 1562 Q 800 1562 800 960 Q 800 666 1024 496 Q 1248 326 1651 326 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-77" d="M 5574 3366 L 4301 0 L 3872 0 L 2810 2758 L 1747 0 L 1318 0 L 51 3366 L 486 3366 L 1542 506 L 2624 3366 L 3014 3366 L 4090 506 L 5158 3366 L 5574 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-20" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-64" d="M 3706 4749 L 3706 0 L 3270 0 L 3270 666 Q 3066 326 2730 147 Q 2394 -32 1971 -32 Q 1498 -32 1114 185 Q 730 403 512 793 Q 294 1184 294 1683 Q 294 2182 512 2572 Q 730 2963 1114 3177 Q 1498 3392 1971 3392 Q 2381 3392 2710 3222 Q 3040 3053 3251 2726 L 3251 4749 L 3706 4749 zM 2010 371 Q 2362 371 2646 534 Q 2931 698 3094 998 Q 3258 1299 3258 1683 Q 3258 2067 3094 2368 Q 2931 2669 2646 2832 Q 2362 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-66" d="M 1747 4397 Q 1453 4397 1302 4237 Q 1152 4077 1152 3763 L 1152 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 0 L 710 0 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 3782 Q 710 4243 976 4512 Q 1242 4781 1728 4781 Q 1920 4781 2099 4726 Q 2278 4672 2400 4570 L 2246 4230 Q 2048 4397 1747 4397 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-63"/>      <use xlink:href="#Montserrat-Regular-65" x="56.299988"/>      <use xlink:href="#Montserrat-Regular-6e" x="116.699982"/>      <use xlink:href="#Montserrat-Regular-74" x="184.399963"/>      <use xlink:href="#Montserrat-Regular-69" x="224.999954"/>      <use xlink:href="#Montserrat-Regular-70" x="251.899948"/>      <use xlink:href="#Montserrat-Regular-61" x="319.699936"/>      <use xlink:href="#Montserrat-Regular-77" x="378.699921"/>      <use xlink:href="#Montserrat-Regular-6e" x="466.599915"/>      <use xlink:href="#Montserrat-Regular-20" x="534.299896"/>      <use xlink:href="#Montserrat-Regular-64" x="560.499893"/>      <use xlink:href="#Montserrat-Regular-69" x="628.299881"/>      <use xlink:href="#Montserrat-Regular-66" x="655.199875"/>      <use xlink:href="#Montserrat-Regular-66" x="689.099869"/>      <use xlink:href="#Montserrat-Regular-65" x="722.999863"/>      <use xlink:href="#Montserrat-Regular-72" x="783.399857"/>      <use xlink:href="#Montserrat-Regular-65" x="823.499847"/>      <use xlink:href="#Montserrat-Regular-6e" x="883.899841"/>      <use xlink:href="#Montserrat-Regular-63" x="951.599823"/>      <use xlink:href="#Montserrat-Regular-65" x="1007.899811"/>     </g>    </g>   </g>   <g id="line2d_14">    <path d="M 54 127.26 L 57.348 118.428279 L 60.696 131.965953 L 64.044 129.709674 L 67.392 131.321302 L 70.74 162.264558 L 74.088 151.563349 L 77.436 151.434419 L 80.784 140.86214 L 84.132 162.393488 L 87.48 158.074326 L 90.828 167.937488 L 94.176 173.481488 L 97.524 173.932744 L 100.872 180.830512 L 104.22 178.896558 L 107.568 196.624465 L 110.916 191.789581 L 114.264 193.788 L 117.612 202.297395 L 120.96 223.893209 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_15">    <path d="M 54 127.26 L 57.348 125.454977 L 60.696 125.261581 L 64.044 124.810326 L 67.392 124.294605 L 70.74 125.841767 L 74.088 125.583907 L 77.436 124.552465 L 80.784 121.45814 L 84.132 125.390512 L 87.48 121.84493 L 90.828 131.450233 L 94.176 142.989488 L 97.524 154.464279 L 100.872 149.242605 L 104.22 138.670326 L 107.568 162.586884 L 110.916 160.459535 L 114.264 172.063256 L 117.612 184.247163 L 120.96 174.964186 L 124.308 173.030233 L 127.656 187.792744 L 131.004 176.769209 L 134.352 201.910605 L 137.7 201.84614 L 141.048 201.008093 L 144.396 205.00493 L 147.744 203.909023 L 151.092 223.893209 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_16">    <path d="M 54 127.26 L 57.348 126.744279 L 60.696 123.521023 L 64.044 122.618512 L 67.392 119.20186 L 70.74 117.396837 L 74.088 119.137395 L 77.436 122.682977 L 80.784 121.393674 L 84.132 123.972279 L 87.48 127.066605 L 90.828 127.51786 L 94.176 154.270884 L 97.524 154.915535 L 100.872 176.189023 L 104.22 145.116837 L 107.568 199.138605 L 110.916 207.583535 L 114.264 199.976651 L 117.612 212.225023 L 120.96 209.324093 L 124.308 212.096093 L 127.656 212.418419 L 131.004 209.646419 L 134.352 206.552093 L 137.7 206.036372 L 141.048 206.358698 L 144.396 208.614977 L 147.744 208.614977 L 151.092 206.487628 L 154.44 205.327256 L 157.788 223.828744 L 161.136 223.893209 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_17">    <path d="M 54 127.26 L 57.348 121.716 L 60.696 120.684558 L 64.044 121.651535 L 67.392 123.198698 L 70.74 121.716 L 74.088 121.522605 L 77.436 127.26 L 80.784 116.236465 L 84.132 126.744279 L 87.48 130.612186 L 90.828 130.032 L 94.176 162.006698 L 97.524 170.709488 L 100.872 164.843163 L 104.22 162.071163 L 107.568 161.748837 L 110.916 160.137209 L 114.264 186.954698 L 117.612 193.272279 L 120.96 199.718791 L 124.308 199.718791 L 127.656 198.622884 L 131.004 204.231349 L 134.352 223.893209 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_18">    <path d="M 54 127.26 L 57.348 118.428279 L 60.696 119.524186 L 64.044 125.970698 L 67.392 127.646791 L 70.74 127.646791 L 74.088 131.579163 L 77.436 141.055535 L 80.784 144.343256 L 84.132 142.989488 L 87.48 143.892 L 90.828 145.632558 L 94.176 146.728465 L 97.524 148.211163 L 100.872 145.697023 L 104.22 163.42493 L 107.568 156.785023 L 110.916 162.522419 L 114.264 161.233116 L 117.612 168.904465 L 120.96 157.107349 L 124.308 165.93907 L 127.656 161.684372 L 131.004 176.833674 L 134.352 177.800651 L 137.7 189.146512 L 141.048 192.821023 L 144.396 199.396465 L 147.744 199.20307 L 151.092 186.696837 L 154.44 185.72986 L 157.788 185.278605 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_19">    <path d="M 54 127.26 L 57.348 123.521023 L 60.696 123.327628 L 64.044 125.841767 L 67.392 125.454977 L 70.74 126.486419 L 74.088 127.453395 L 77.436 127.26 L 80.784 138.412465 L 84.132 146.212744 L 87.48 155.946977 L 90.828 184.440558 L 94.176 173.610419 L 97.524 190.242419 L 100.872 186.181116 L 104.22 184.633953 L 107.568 195.012837 L 110.916 187.98614 L 114.264 183.795907 L 117.612 193.272279 L 120.96 202.297395 L 124.308 200.621302 L 127.656 212.611814 L 131.004 207.648 L 134.352 224.602326 L 137.7 224.40893 L 141.048 223.828744 L 144.396 223.893209 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_20">    <path d="M 54 127.26 L 57.348 123.843349 L 60.696 123.456558 L 64.044 120.555628 L 67.392 121.97386 L 70.74 121.071349 L 74.088 123.327628 L 77.436 120.620093 L 80.784 119.524186 L 84.132 120.104372 L 87.48 135.382605 L 90.828 126.164093 L 94.176 147.759907 L 97.524 150.854233 L 100.872 153.884093 L 104.22 154.077488 L 107.568 163.16707 L 110.916 165.165488 L 114.264 169.355721 L 117.612 169.162326 L 120.96 170.322698 L 124.308 160.588465 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_21">    <path d="M 54 127.26 L 57.348 123.521023 L 60.696 122.554047 L 64.044 119.846512 L 67.392 118.299349 L 70.74 132.030419 L 74.088 127.969116 L 77.436 137.445488 L 80.784 136.027256 L 84.132 136.800837 L 87.48 136.285116 L 90.828 136.671907 L 94.176 129.580744 L 97.524 140.217488 L 100.872 138.734791 L 104.22 157.687535 L 107.568 160.588465 L 110.916 152.981581 L 114.264 167.679628 L 117.612 167.615163 L 120.96 164.649767 L 124.308 162.780279 L 127.656 166.712651 L 131.004 168.96893 L 134.352 174.835256 L 137.7 183.860372 L 141.048 183.602512 L 144.396 182.248744 L 147.744 179.992465 L 151.092 182.57107 L 154.44 190.564744 L 157.788 194.368186 L 161.136 197.462512 L 164.484 197.978233 L 167.832 194.239256 L 171.18 223.893209 L 174.528 195.528558 L 177.876 191.209395 L 181.224 223.893209 L 184.572 223.893209 L 187.92 223.893209 L 191.268 223.893209 L 194.616 223.893209 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_22">    <path d="M 54 127.26 L 57.348 125.261581 L 60.696 124.810326 L 64.044 124.423535 L 67.392 125.970698 L 70.74 153.690698 L 74.088 157.945395 L 77.436 156.978419 L 80.784 159.750419 L 84.132 156.656093 L 87.48 149.758326 L 90.828 156.398233 L 94.176 154.72214 L 97.524 157.49414 L 100.872 173.094698 L 104.22 183.344651 L 107.568 183.215721 L 110.916 182.82893 L 114.264 191.402791 L 117.612 189.146512 L 120.96 180.314791 L 124.308 184.440558 L 127.656 193.272279 L 131.004 198.622884 L 134.352 196.56 L 137.7 185.665395 L 141.048 191.531721 L 144.396 171.160744 L 147.744 193.788 L 151.092 197.333581 L 154.44 200.943628 L 157.788 223.828744 L 161.136 223.893209 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_23">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 125.003721 L 64.044 125.132651 L 67.392 125.841767 L 70.74 127.324465 L 74.088 126.035163 L 77.436 126.873209 L 80.784 122.296186 L 84.132 126.679814 L 87.48 126.937674 L 90.828 127.582326 L 94.176 127.13107 L 97.524 149.822791 L 100.872 142.731628 L 104.22 165.874605 L 107.568 162.909209 L 110.916 165.294419 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_24">    <path d="M 54 127.26 L 57.348 125.261581 L 60.696 120.877953 L 64.044 120.168837 L 67.392 120.620093 L 70.74 122.876372 L 74.088 122.876372 L 77.436 126.937674 L 80.784 142.280372 L 84.132 138.21907 L 87.48 177.156 L 90.828 175.866698 L 94.176 179.476744 L 97.524 175.737767 L 100.872 168.71107 L 104.22 161.813302 L 107.568 154.077488 L 110.916 169.484651 L 114.264 187.857209 L 117.612 199.654326 L 120.96 218.60707 L 124.308 204.489209 L 127.656 209.646419 L 131.004 211.193581 L 134.352 223.893209 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_25">    <path d="M 54 127.26 L 57.348 118.105953 L 60.696 123.714419 L 64.044 128.226977 L 67.392 125.326047 L 70.74 127.26 L 74.088 134.802419 L 77.436 133.319721 L 80.784 136.156186 L 84.132 148.404558 L 87.48 126.099628 L 90.828 118.105953 L 94.176 122.554047 L 97.524 125.777302 L 100.872 164.456372 L 104.22 166.648186 L 107.568 195.979814 L 110.916 171.805395 L 114.264 223.828744 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_26">    <path d="M 54 127.26 L 57.348 117.267907 L 60.696 122.811907 L 64.044 121.071349 L 67.392 120.813488 L 70.74 114.302512 L 74.088 116.236465 L 77.436 134.544558 L 80.784 159.492558 L 84.132 153.432837 L 87.48 164.198512 L 90.828 157.88093 L 94.176 153.110512 L 97.524 188.888651 L 100.872 180.637116 L 104.22 183.086791 L 107.568 177.220465 L 110.916 196.108744 L 114.264 205.00493 L 117.612 207.712465 L 120.96 202.36186 L 124.308 200.750233 L 127.656 210.742326 L 131.004 210.097674 L 134.352 215.577209 L 137.7 211.193581 L 141.048 223.893209 L 144.396 223.893209 L 147.744 223.893209 L 151.092 223.893209 L 154.44 223.893209 L 157.788 223.893209 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_27">    <path d="M 54 127.26 L 57.348 124.294605 L 60.696 121.522605 L 64.044 125.777302 L 67.392 118.234884 L 70.74 121.780465 L 74.088 120.426698 L 77.436 123.456558 L 80.784 123.585488 L 84.132 123.069767 L 87.48 121.97386 L 90.828 127.26 L 94.176 141.764651 L 97.524 140.153023 L 100.872 137.381023 L 104.22 134.351163 L 107.568 124.939256 L 110.916 136.414047 L 114.264 142.086977 L 117.612 151.369953 L 120.96 149.629395 L 124.308 151.627814 L 127.656 169.935907 L 131.004 171.805395 L 134.352 174.706326 L 137.7 171.35414 L 141.048 178.058512 L 144.396 162.071163 L 147.744 164.907628 L 151.092 161.684372 L 154.44 170.064837 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_28">    <path d="M 54 127.26 L 57.348 124.874791 L 60.696 126.937674 L 64.044 112.884279 L 67.392 113.52893 L 70.74 125.777302 L 74.088 125.841767 L 77.436 132.739535 L 80.784 136.156186 L 84.132 133.706512 L 87.48 137.832279 L 90.828 145.052372 L 94.176 140.99107 L 97.524 160.137209 L 100.872 180.701581 L 104.22 177.800651 L 107.568 181.281767 L 110.916 181.281767 L 114.264 184.762884 L 117.612 183.086791 L 120.96 170.645023 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_29">    <path d="M 54 127.26 L 57.348 124.874791 L 60.696 124.874791 L 64.044 124.61693 L 67.392 123.843349 L 70.74 119.008465 L 74.088 123.843349 L 77.436 160.459535 L 80.784 156.978419 L 84.132 157.687535 L 87.48 160.910791 L 90.828 156.656093 L 94.176 154.464279 L 97.524 163.55386 L 100.872 169.291256 L 104.22 158.203256 L 107.568 161.555442 L 110.916 156.978419 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_30">    <path d="M 54 127.26 L 57.348 126.873209 L 60.696 127.066605 L 64.044 125.841767 L 67.392 126.873209 L 70.74 121.716 L 74.088 125.197116 L 77.436 126.357488 L 80.784 122.425116 L 84.132 127.00214 L 87.48 131.127907 L 90.828 122.038326 L 94.176 127.38893 L 97.524 128.807163 L 100.872 156.398233 L 104.22 166.003535 L 107.568 168.453209 L 110.916 169.291256 L 114.264 168.84 L 117.612 170.773953 L 120.96 168.71107 L 124.308 164.391907 L 127.656 165.745674 L 131.004 171.612 L 134.352 177.929581 L 137.7 178.251907 L 141.048 175.608837 L 144.396 204.61814 L 147.744 223.893209 L 151.092 223.893209 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_31">    <path d="M 54 127.26 L 57.348 119.653116 L 60.696 119.524186 L 64.044 114.882698 L 67.392 119.910977 L 70.74 122.489581 L 74.088 125.261581 L 77.436 121.651535 L 80.784 108.436186 L 84.132 157.687535 L 87.48 163.360465 L 90.828 160.975256 L 94.176 166.003535 L 97.524 167.421767 L 100.872 167.421767 L 104.22 167.421767 L 107.568 187.148093 L 110.916 187.728279 L 114.264 189.726698 L 117.612 188.824186 L 120.96 190.629209 L 124.308 191.596186 L 127.656 202.104 L 131.004 194.303721 L 134.352 212.998605 L 137.7 210.806791 L 141.048 212.869674 L 144.396 213.707721 L 147.744 209.001767 L 151.092 219.380651 L 154.44 219.122791 L 157.788 211.902698 L 161.136 223.957674 L 164.484 220.798884 L 167.832 218.929395 L 171.18 217.317767 L 174.528 210.54893 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_32">    <path d="M 54 127.26 L 57.348 121.522605 L 60.696 122.940837 L 64.044 119.459721 L 67.392 123.198698 L 70.74 121.135814 L 74.088 111.530512 L 77.436 116.623256 L 80.784 117.332372 L 84.132 118.363814 L 87.48 113.464465 L 90.828 114.366977 L 94.176 117.332372 L 97.524 119.137395 L 100.872 128.807163 L 104.22 131.256837 L 107.568 139.959628 L 110.916 153.046047 L 114.264 153.626233 L 117.612 153.497302 L 120.96 161.362047 L 124.308 167.873023 L 127.656 168.58214 L 131.004 178.509767 L 134.352 178.509767 L 137.7 181.990884 L 141.048 195.721953 L 144.396 206.487628 L 147.744 202.104 L 151.092 190.564744 L 154.44 206.036372 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_33">    <path d="M 54 127.26 L 57.348 125.003721 L 60.696 126.228558 L 64.044 117.977023 L 67.392 125.326047 L 70.74 127.582326 L 74.088 125.712837 L 77.436 162.973674 L 80.784 157.687535 L 84.132 149.30707 L 87.48 144.665581 L 90.828 165.101023 L 94.176 164.778698 L 97.524 171.547535 L 100.872 168.388744 L 104.22 175.608837 L 107.568 178.380837 L 110.916 179.218884 L 114.264 203.070977 L 117.612 205.13386 L 120.96 200.492372 L 124.308 206.294233 L 127.656 209.904279 L 131.004 223.893209 L 134.352 223.893209 L 137.7 223.893209 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_34">    <path d="M 54 127.26 L 57.348 118.557209 L 60.696 121.522605 L 64.044 121.909395 L 67.392 132.610605 L 70.74 132.804 L 74.088 161.104186 L 77.436 158.976837 L 80.784 164.714233 L 84.132 162.586884 L 87.48 168.066419 L 90.828 168.453209 L 94.176 168.453209 L 97.524 163.16707 L 100.872 173.481488 L 104.22 199.138605 L 107.568 189.920093 L 110.916 193.465674 L 114.264 194.239256 L 117.612 195.335163 L 120.96 195.528558 L 124.308 196.366605 L 127.656 201.588279 L 131.004 206.036372 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_35">    <path d="M 54 127.26 L 57.348 116.687721 L 60.696 122.360651 L 64.044 117.590233 L 67.392 121.651535 L 70.74 119.137395 L 74.088 121.006884 L 77.436 122.425116 L 80.784 127.26 L 84.132 131.450233 L 87.48 146.212744 L 90.828 150.725302 L 94.176 159.170233 L 97.524 178.445302 L 100.872 177.736186 L 104.22 177.671721 L 107.568 192.821023 L 110.916 195.657488 L 114.264 187.857209 L 117.612 182.184279 L 120.96 186.052186 L 124.308 186.181116 L 127.656 188.50186 L 131.004 189.662233 L 134.352 186.181116 L 137.7 188.824186 L 141.048 195.335163 L 144.396 195.206233 L 147.744 186.890233 L 151.092 195.399628 L 154.44 195.915349 L 157.788 197.784837 L 161.136 198.945209 L 164.484 198.687349 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_36">    <path d="M 54 127.26 L 57.348 124.423535 L 60.696 122.747442 L 64.044 122.618512 L 67.392 122.940837 L 70.74 94.834047 L 74.088 110.88586 L 77.436 97.477116 L 80.784 159.750419 L 84.132 155.753581 L 87.48 146.664 L 90.828 149.500465 L 94.176 169.226791 L 97.524 165.358884 L 100.872 159.170233 L 104.22 183.151256 L 107.568 185.60093 L 110.916 189.726698 L 114.264 201.717209 L 117.612 223.828744 L 120.96 223.893209 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_37">    <path d="M 54 127.26 L 57.348 124.874791 L 60.696 123.714419 L 64.044 128.226977 L 67.392 144.536651 L 70.74 150.467442 L 74.088 143.376279 L 77.436 148.469023 L 80.784 159.814884 L 84.132 173.417023 L 87.48 167.034977 L 90.828 175.286512 L 94.176 178.122977 L 97.524 184.440558 L 100.872 178.251907 L 104.22 191.918512 L 107.568 210.484465 L 110.916 209.324093 L 114.264 214.868093 L 117.612 215.383814 L 120.96 206.358698 L 124.308 220.669953 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_38">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 124.874791 L 64.044 122.231721 L 67.392 124.294605 L 70.74 124.552465 L 74.088 122.489581 L 77.436 120.684558 L 80.784 152.530326 L 84.132 148.662419 L 87.48 149.17814 L 90.828 161.619907 L 94.176 158.912372 L 97.524 157.945395 L 100.872 137.574419 L 104.22 143.569674 L 107.568 147.695442 L 110.916 156.591628 L 114.264 150.467442 L 117.612 166.777116 L 120.96 173.159163 L 124.308 179.218884 L 127.656 180.830512 L 131.004 190.177953 L 134.352 187.857209 L 137.7 191.402791 L 141.048 193.014419 L 144.396 191.402791 L 147.744 188.37293 L 151.092 189.146512 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_39">    <path d="M 54 127.26 L 57.348 120.362233 L 60.696 121.84493 L 64.044 119.588651 L 67.392 119.395256 L 70.74 118.879535 L 74.088 115.011628 L 77.436 157.88093 L 80.784 155.044465 L 84.132 159.492558 L 87.48 159.492558 L 90.828 174.12614 L 94.176 174.64186 L 97.524 182.764465 L 100.872 188.759721 L 104.22 191.209395 L 107.568 197.333581 L 110.916 194.948372 L 114.264 200.685767 L 117.612 195.850884 L 120.96 192.176372 L 124.308 199.783256 L 127.656 198.107163 L 131.004 200.363442 L 134.352 198.622884 L 137.7 201.394884 L 141.048 201.781674 L 144.396 193.143349 L 147.744 205.778512 L 151.092 204.037953 L 154.44 207.454605 L 157.788 210.355535 L 161.136 205.842977 L 164.484 215.770605 L 167.832 223.893209 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_40">    <path d="M 54 127.26 L 57.348 124.61693 L 60.696 124.874791 L 64.044 121.071349 L 67.392 113.915721 L 70.74 113.27107 L 74.088 122.811907 L 77.436 125.261581 L 80.784 121.264744 L 84.132 122.940837 L 87.48 124.036744 L 90.828 122.425116 L 94.176 126.615349 L 97.524 127.904651 L 100.872 126.228558 L 104.22 151.885674 L 107.568 138.670326 L 110.916 145.503628 L 114.264 143.569674 L 117.612 147.179721 L 120.96 152.981581 L 124.308 152.401395 L 127.656 152.659256 L 131.004 164.778698 L 134.352 160.330605 L 137.7 155.882512 L 141.048 167.873023 L 144.396 170.129302 L 147.744 169.162326 L 151.092 183.086791 L 154.44 193.981395 L 157.788 185.085209 L 161.136 191.209395 L 164.484 196.81786 L 167.832 184.311628 L 171.18 188.888651 L 174.528 191.789581 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_41">    <path d="M 54 127.26 L 57.348 124.874791 L 60.696 123.972279 L 64.044 117.719163 L 67.392 150.660837 L 70.74 149.822791 L 74.088 124.681395 L 77.436 135.898326 L 80.784 181.797488 L 84.132 183.280186 L 87.48 187.148093 L 90.828 187.599349 L 94.176 181.539628 L 97.524 194.368186 L 100.872 191.660651 L 104.22 194.626047 L 107.568 197.462512 L 110.916 192.885488 L 114.264 191.531721 L 117.612 193.981395 L 120.96 196.30214 L 124.308 211.129116 L 127.656 204.295814 L 131.004 187.470419 L 134.352 192.240837 L 137.7 193.594605 L 141.048 194.626047 L 144.396 196.68893 L 147.744 193.723535 L 151.092 205.520651 L 154.44 193.723535 L 157.788 190.371349 L 161.136 201.008093 L 164.484 184.053767 L 167.832 205.714047 L 171.18 200.170047 L 174.528 194.819442 L 177.876 203.844558 L 181.224 210.355535 L 184.572 209.517488 L 187.92 210.16214 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_42">    <path d="M 54 127.26 L 57.348 124.61693 L 60.696 124.681395 L 64.044 119.910977 L 67.392 119.782047 L 70.74 111.401581 L 74.088 134.737953 L 77.436 134.609023 L 80.784 162.200093 L 84.132 157.107349 L 87.48 178.316372 L 90.828 180.314791 L 94.176 176.317953 L 97.524 185.020744 L 100.872 182.248744 L 104.22 163.55386 L 107.568 157.752 L 110.916 154.593209 L 114.264 173.288093 L 117.612 173.610419 L 120.96 165.93907 L 124.308 175.093116 L 127.656 194.819442 L 131.004 184.633953 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_43">    <path d="M 54 127.26 L 57.348 122.811907 L 60.696 124.552465 L 64.044 124.74586 L 67.392 118.944 L 70.74 152.852651 L 74.088 158.976837 L 77.436 164.198512 L 80.784 162.780279 L 84.132 167.357302 L 87.48 163.811721 L 90.828 163.360465 L 94.176 162.200093 L 97.524 158.203256 L 100.872 163.876186 L 104.22 164.391907 L 107.568 141.571256 L 110.916 162.973674 L 114.264 167.744093 L 117.612 167.937488 L 120.96 172.127721 L 124.308 171.160744 L 127.656 147.888837 L 131.004 167.292837 L 134.352 182.44214 L 137.7 189.597767 L 141.048 181.023907 L 144.396 181.346233 L 147.744 196.81786 L 151.092 194.368186 L 154.44 191.531721 L 157.788 192.692093 L 161.136 190.951535 L 164.484 183.602512 L 167.832 223.893209 L 171.18 192.305302 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_44">    <path d="M 54 127.26 L 57.348 115.849674 L 60.696 121.651535 L 64.044 121.522605 L 67.392 128.484837 L 70.74 159.041302 L 74.088 183.344651 L 77.436 185.536465 L 80.784 223.957674 L 84.132 223.957674 L 87.48 223.957674 L 90.828 223.957674 L 94.176 223.957674 L 97.524 223.957674 L 100.872 223.957674 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_45">    <path d="M 54 127.26 L 57.348 123.521023 L 60.696 122.102791 L 64.044 119.266326 L 67.392 121.200279 L 70.74 125.068186 L 74.088 133.899907 L 77.436 165.93907 L 80.784 168.84 L 84.132 139.572837 L 87.48 144.085395 L 90.828 139.830698 L 94.176 143.311814 L 97.524 137.123163 L 100.872 137.445488 L 104.22 138.60586 L 107.568 157.945395 L 110.916 172.063256 L 114.264 171.096279 L 117.612 167.550698 L 120.96 184.633953 L 124.308 187.663814 L 127.656 182.764465 L 131.004 186.761302 L 134.352 190.371349 L 137.7 195.270698 L 141.048 192.821023 L 144.396 193.788 L 147.744 182.893395 L 151.092 193.272279 L 154.44 186.954698 L 157.788 191.596186 L 161.136 195.077302 L 164.484 191.27386 L 167.832 202.490791 L 171.18 196.366605 L 174.528 196.624465 L 177.876 196.81786 L 181.224 202.23293 L 184.572 202.23293 L 187.92 202.877581 L 191.268 203.844558 L 194.616 202.877581 L 197.964 205.069395 L 201.312 207.90586 L 204.66 205.585116 L 208.008 205.198326 L 211.356 206.100837 L 214.704 206.745488 L 218.052 206.616558 L 221.4 207.648 L 224.748 213.772186 L 228.096 223.893209 L 231.444 223.893209 L 234.792 223.893209 L 238.14 217.511163 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_46">    <path d="M 54 127.26 L 57.348 121.58707 L 60.696 122.167256 L 64.044 119.717581 L 67.392 117.332372 L 70.74 118.944 L 74.088 113.14214 L 77.436 119.137395 L 80.784 140.281953 L 84.132 145.632558 L 87.48 145.181302 L 90.828 144.601116 L 94.176 146.148279 L 97.524 166.712651 L 100.872 174.64186 L 104.22 168.388744 L 107.568 172.772372 L 110.916 179.476744 L 114.264 184.182698 L 117.612 175.995628 L 120.96 190.306884 L 124.308 177.800651 L 127.656 189.404372 L 131.004 191.918512 L 134.352 175.157581 L 137.7 188.824186 L 141.048 186.567907 L 144.396 187.341488 L 147.744 190.629209 L 151.092 200.879163 L 154.44 195.270698 L 157.788 200.685767 L 161.136 208.228186 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_47">    <path d="M 54 127.26 L 57.348 118.428279 L 60.696 130.418791 L 64.044 132.094884 L 67.392 136.220651 L 70.74 153.626233 L 74.088 173.932744 L 77.436 187.277023 L 80.784 201.330419 L 84.132 197.526977 L 87.48 205.971907 L 90.828 213.06307 L 94.176 205.262791 L 97.524 219.831907 L 100.872 208.872837 L 104.22 223.764279 L 107.568 223.828744 L 110.916 223.893209 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_48">    <path d="M 54 127.26 L 57.348 122.167256 L 60.696 122.682977 L 64.044 120.813488 L 67.392 124.35907 L 70.74 127.324465 L 74.088 117.267907 L 77.436 123.069767 L 80.784 127.775721 L 84.132 130.870047 L 87.48 165.358884 L 90.828 168.066419 L 94.176 164.005116 L 97.524 177.28493 L 100.872 169.742512 L 104.22 175.995628 L 107.568 190.693674 L 110.916 198.751814 L 114.264 197.269116 L 117.612 207.261209 L 120.96 216.22186 L 124.308 223.893209 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_49">    <path d="M 54 127.26 L 57.348 121.200279 L 60.696 118.750605 L 64.044 122.489581 L 67.392 122.489581 L 70.74 117.203442 L 74.088 120.877953 L 77.436 119.330791 L 80.784 114.431442 L 84.132 116.236465 L 87.48 131.127907 L 90.828 136.285116 L 94.176 152.594791 L 97.524 223.957674 L 100.872 223.957674 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_50">    <path d="M 54 127.26 L 57.348 125.777302 L 60.696 125.777302 L 64.044 117.654698 L 67.392 125.648372 L 70.74 124.165674 L 74.088 113.593395 L 77.436 115.849674 L 80.784 127.969116 L 84.132 170.258233 L 87.48 174.190605 L 90.828 181.990884 L 94.176 187.599349 L 97.524 194.174791 L 100.872 193.078884 L 104.22 190.049023 L 107.568 190.693674 L 110.916 195.335163 L 114.264 199.654326 L 117.612 203.522233 L 120.96 217.511163 L 124.308 214.997023 L 127.656 205.520651 L 131.004 223.893209 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_51">    <path d="M 54 127.26 L 57.348 119.846512 L 60.696 121.58707 L 64.044 116.816651 L 67.392 124.61693 L 70.74 124.61693 L 74.088 129.129488 L 77.436 128.098047 L 80.784 123.585488 L 84.132 133.384186 L 87.48 127.775721 L 90.828 138.734791 L 94.176 151.434419 L 97.524 169.549116 L 100.872 182.248744 L 104.22 188.050605 L 107.568 185.536465 L 110.916 187.98614 L 114.264 184.633953 L 117.612 211.644837 L 120.96 200.814698 L 124.308 200.750233 L 127.656 206.874419 L 131.004 223.828744 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_52">    <path d="M 54 127.26 L 57.348 125.003721 L 60.696 124.35907 L 64.044 124.35907 L 67.392 124.165674 L 70.74 122.425116 L 74.088 123.649953 L 77.436 123.907814 L 80.784 124.23014 L 84.132 119.782047 L 87.48 164.262977 L 90.828 158.332186 L 94.176 168.001953 L 97.524 175.157581 L 100.872 171.74093 L 104.22 163.03814 L 107.568 163.16707 L 110.916 158.267721 L 114.264 181.410698 L 117.612 166.712651 L 120.96 181.475163 L 124.308 176.446884 L 127.656 168.324279 L 131.004 176.060093 L 134.352 169.420186 L 137.7 169.291256 L 141.048 180.830512 L 144.396 187.728279 L 147.744 185.536465 L 151.092 183.215721 L 154.44 190.306884 L 157.788 198.687349 L 161.136 208.486047 L 164.484 206.423163 L 167.832 212.160558 L 171.18 218.671535 L 174.528 210.67786 L 177.876 190.564744 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_53">    <path d="M 54 127.26 L 57.348 122.167256 L 60.696 118.041488 L 64.044 117.010047 L 67.392 114.560372 L 70.74 113.915721 L 74.088 121.909395 L 77.436 128.420372 L 80.784 137.703349 L 84.132 137.187628 L 87.48 128.420372 L 90.828 140.346419 L 94.176 143.376279 L 97.524 167.099442 L 100.872 170.902884 L 104.22 186.438977 L 107.568 194.432651 L 110.916 215.254884 L 114.264 199.718791 L 117.612 209.968744 L 120.96 202.490791 L 124.308 197.784837 L 127.656 214.674698 L 131.004 222.410512 L 134.352 210.097674 L 137.7 219.058326 L 141.048 223.828744 L 144.396 223.893209 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_54">    <path d="M 54 127.26 L 57.348 121.58707 L 60.696 119.330791 L 64.044 123.714419 L 67.392 125.841767 L 70.74 121.97386 L 74.088 121.200279 L 77.436 163.16707 L 80.784 154.786605 L 84.132 159.879349 L 87.48 167.292837 L 90.828 168.195349 L 94.176 158.332186 L 97.524 160.717395 L 100.872 169.678047 L 104.22 173.932744 L 107.568 173.868279 L 110.916 176.124558 L 114.264 173.481488 L 117.612 176.575814 L 120.96 183.086791 L 124.308 177.736186 L 127.656 175.028651 L 131.004 168.84 L 134.352 173.417023 L 137.7 180.443721 L 141.048 169.162326 L 144.396 173.868279 L 147.744 174.448465 L 151.092 183.409116 L 154.44 185.858791 L 157.788 195.979814 L 161.136 196.81786 L 164.484 223.893209 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_55">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 121.329209 L 64.044 122.811907 L 67.392 125.583907 L 70.74 119.07293 L 74.088 121.264744 L 77.436 122.425116 L 80.784 125.906233 L 84.132 125.068186 L 87.48 124.294605 L 90.828 123.456558 L 94.176 161.362047 L 97.524 163.811721 L 100.872 175.866698 L 104.22 174.964186 L 107.568 178.509767 L 110.916 180.314791 L 114.264 181.346233 L 117.612 187.98614 L 120.96 186.632372 L 124.308 196.624465 L 127.656 186.116651 L 131.004 190.693674 L 134.352 187.212558 L 137.7 185.407535 L 141.048 184.376093 L 144.396 189.984558 L 147.744 200.492372 L 151.092 198.880744 L 154.44 204.102419 L 157.788 205.198326 L 161.136 204.037953 L 164.484 197.978233 L 167.832 209.388558 L 171.18 198.751814 L 174.528 191.080465 L 177.876 187.019163 L 181.224 188.953116 L 184.572 187.019163 L 187.92 188.759721 L 191.268 188.759721 L 194.616 190.371349 L 197.964 189.533302 L 201.312 199.525395 L 204.66 198.107163 L 208.008 201.137023 L 211.356 195.206233 L 214.704 196.81786 L 218.052 196.68893 L 221.4 199.783256 L 224.748 198.107163 L 228.096 200.427907 L 231.444 197.591442 L 234.792 200.170047 L 238.14 198.300558 L 241.488 192.949953 L 244.836 200.427907 L 248.184 205.907442 L 251.532 204.876 L 254.88 179.863535 L 258.228 186.181116 L 261.576 199.654326 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_56">    <path d="M 54 127.26 L 57.348 122.167256 L 60.696 120.168837 L 64.044 122.425116 L 67.392 99.926791 L 70.74 94.12493 L 74.088 93.222419 L 77.436 102.18307 L 80.784 107.98493 L 84.132 110.821395 L 87.48 175.931163 L 90.828 173.352558 L 94.176 169.742512 L 97.524 178.896558 L 100.872 181.217302 L 104.22 173.868279 L 107.568 169.742512 L 110.916 168.324279 L 114.264 176.189023 L 117.612 162.973674 L 120.96 160.524 L 124.308 162.780279 L 127.656 166.841581 L 131.004 177.349395 L 134.352 173.997209 L 137.7 190.693674 L 141.048 182.506605 L 144.396 223.893209 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_57">    <path d="M 54 127.26 L 57.348 118.557209 L 60.696 122.102791 L 64.044 121.45814 L 67.392 121.58707 L 70.74 128.291442 L 74.088 151.563349 L 77.436 161.748837 L 80.784 178.638698 L 84.132 174.448465 L 87.48 176.962605 L 90.828 173.481488 L 94.176 178.380837 L 97.524 183.666977 L 100.872 187.921674 L 104.22 183.086791 L 107.568 183.860372 L 110.916 184.633953 L 114.264 189.210977 L 117.612 185.923256 L 120.96 190.242419 L 124.308 194.303721 L 127.656 184.827349 L 131.004 193.014419 L 134.352 192.949953 L 137.7 190.049023 L 141.048 179.928 L 144.396 179.412279 L 147.744 188.824186 L 151.092 200.492372 L 154.44 204.74707 L 157.788 204.553674 L 161.136 223.893209 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_58">    <path d="M 54 127.26 L 57.348 123.327628 L 60.696 124.488 L 64.044 122.489581 L 67.392 120.039907 L 70.74 125.454977 L 74.088 126.357488 L 77.436 135.060279 L 80.784 135.060279 L 84.132 133.255256 L 87.48 136.220651 L 90.828 135.640465 L 94.176 132.93293 L 97.524 134.673488 L 100.872 132.223814 L 104.22 133.642047 L 107.568 139.250512 L 110.916 150.596372 L 114.264 163.811721 L 117.612 169.355721 L 120.96 180.18586 L 124.308 179.025488 L 127.656 182.635535 L 131.004 177.542791 L 134.352 168.324279 L 137.7 168.066419 L 141.048 183.602512 L 144.396 183.602512 L 147.744 196.30214 L 151.092 189.597767 L 154.44 192.240837 L 157.788 191.789581 L 161.136 198.622884 L 164.484 203.844558 L 167.832 205.00493 L 171.18 205.00493 L 174.528 201.137023 L 177.876 189.791163 L 181.224 188.953116 L 184.572 188.179535 L 187.92 178.445302 L 191.268 184.247163 L 194.616 182.313209 L 197.964 180.766047 L 201.312 186.052186 L 204.66 175.415442 L 208.008 184.505023 L 211.356 192.434233 L 214.704 191.14493 L 218.052 193.53014 L 221.4 193.594605 L 224.748 200.814698 L 228.096 199.58986 L 231.444 199.58986 L 234.792 199.58986 L 238.14 198.042698 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_59">    <path d="M 54 127.26 L 57.348 125.583907 L 60.696 125.777302 L 64.044 124.61693 L 67.392 118.363814 L 70.74 112.948744 L 74.088 115.978605 L 77.436 149.436 L 80.784 145.181302 L 84.132 145.439163 L 87.48 146.92186 L 90.828 156.204837 L 94.176 183.086791 L 97.524 186.567907 L 100.872 198.880744 L 104.22 183.344651 L 107.568 185.020744 L 110.916 185.794326 L 114.264 180.894977 L 117.612 188.824186 L 120.96 195.335163 L 124.308 195.206233 L 127.656 204.360279 L 131.004 223.828744 L 134.352 204.74707 L 137.7 223.828744 L 141.048 223.893209 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_60">    <path d="M 54 127.26 L 57.348 122.102791 L 60.696 124.165674 L 64.044 121.651535 L 67.392 121.200279 L 70.74 120.297767 L 74.088 122.167256 L 77.436 114.238047 L 80.784 121.780465 L 84.132 156.656093 L 87.48 157.429674 L 90.828 161.490977 L 94.176 156.075907 L 97.524 156.333767 L 100.872 160.65293 L 104.22 165.616744 L 107.568 166.454791 L 110.916 177.091535 L 114.264 178.058512 L 117.612 178.896558 L 120.96 191.596186 L 124.308 210.806791 L 127.656 220.605488 L 131.004 220.347628 L 134.352 220.283163 L 137.7 217.124372 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_61">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 123.972279 L 64.044 124.810326 L 67.392 136.220651 L 70.74 141.635721 L 74.088 144.14986 L 77.436 147.759907 L 80.784 150.467442 L 84.132 147.502047 L 87.48 143.505209 L 90.828 151.369953 L 94.176 160.717395 L 97.524 161.362047 L 100.872 158.461116 L 104.22 186.438977 L 107.568 185.794326 L 110.916 192.434233 L 114.264 192.369767 L 117.612 191.789581 L 120.96 194.04586 L 124.308 188.11507 L 127.656 195.721953 L 131.004 197.913767 L 134.352 194.04586 L 137.7 190.177953 L 141.048 186.761302 L 144.396 190.951535 L 147.744 194.110326 L 151.092 197.849302 L 154.44 215.061488 L 157.788 196.366605 L 161.136 196.56 L 164.484 203.006512 L 167.832 223.893209 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_62">    <path d="M 54 127.26 L 57.348 122.231721 L 60.696 114.560372 L 64.044 122.102791 L 67.392 124.101209 L 70.74 125.906233 L 74.088 126.744279 L 77.436 133.190791 L 80.784 130.612186 L 84.132 131.192372 L 87.48 159.363628 L 90.828 162.586884 L 94.176 156.204837 L 97.524 172.643442 L 100.872 174.964186 L 104.22 173.674884 L 107.568 181.926419 L 110.916 183.924837 L 114.264 189.533302 L 117.612 187.599349 L 120.96 188.695256 L 124.308 205.262791 L 127.656 198.945209 L 131.004 199.07414 L 134.352 197.784837 L 137.7 178.574233 L 141.048 197.784837 L 144.396 194.561581 L 147.744 194.690512 L 151.092 199.718791 L 154.44 194.497116 L 157.788 194.432651 L 161.136 186.632372 L 164.484 175.157581 L 167.832 180.18586 L 171.18 191.338326 L 174.528 184.956279 L 177.876 193.401209 L 181.224 189.082047 L 184.572 194.690512 L 187.92 182.7 L 191.268 186.181116 L 194.616 177.671721 L 197.964 185.085209 L 201.312 194.239256 L 204.66 193.143349 L 208.008 197.204651 L 211.356 205.520651 L 214.704 204.876 L 218.052 223.893209 L 221.4 223.893209 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_63">    <path d="M 54 127.26 L 57.348 123.005302 L 60.696 123.069767 L 64.044 122.876372 L 67.392 122.102791 L 70.74 124.552465 L 74.088 127.26 L 77.436 131.256837 L 80.784 152.272465 L 84.132 158.654512 L 87.48 177.865116 L 90.828 179.154419 L 94.176 176.89814 L 97.524 166.068 L 100.872 158.976837 L 104.22 168.195349 L 107.568 174.448465 L 110.916 173.094698 L 114.264 181.539628 L 117.612 183.602512 L 120.96 186.825767 L 124.308 184.182698 L 127.656 179.218884 L 131.004 175.802233 L 134.352 163.876186 L 137.7 178.703163 L 141.048 186.374512 L 144.396 179.863535 L 147.744 188.824186 L 151.092 186.632372 L 154.44 194.368186 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #bcbd22; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_64">    <path d="M 54 127.26 L 57.348 123.424326 L 60.696 123.198698 L 64.044 122.167256 L 67.392 123.198698 L 70.74 124.35907 L 74.088 124.939256 L 77.436 129.838605 L 80.784 138.057907 L 84.132 145.53586 L 87.48 148.05 L 90.828 156.527163 L 94.176 159.041302 L 97.524 166.390326 L 100.872 170.322698 L 104.22 173.771581 L 107.568 178.445302 L 110.916 179.895767 L 114.264 187.405953 L 117.612 188.824186 L 120.96 191.112698 L 124.308 196.495535 L 127.656 200.266744 L 131.004 199.718791 L 134.352 205.649581 L 137.7 208.421581 L 141.048 209.614186 L 144.396 218.800465 L 147.744 223.893209 L 151.092 223.893209 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pcfa8e653e1)" style="fill: none; stroke: #000000; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="patch_3">    <path d="M 54 224.28 L 54 30.24 " style="fill: none"/>   </g>   <g id="patch_4">    <path d="M 388.8 224.28 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="patch_5">    <path d="M 54 224.28 L 388.8 224.28 " style="fill: none"/>   </g>   <g id="patch_6">    <path d="M 54 30.24 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="text_16">    <g id="patch_7">     <a xlink:href="https://dynomight.net/chess">      <path d="M 323.8664 225.676606 L 388.881244 225.676606 L 388.881244 213.343012 L 323.8664 213.343012 z" style="fill: #ffffff; opacity: 0.000001; stroke: #ffffff; stroke-linejoin: miter"/>     </a>    </g>    <a xlink:href="https://dynomight.net/chess">     <!-- dynomight.net/chess -->     <g style="opacity: 0.3" transform="translate(327.8664 220.734419) scale(0.05 -0.05)">      <defs>       <path id="CourierNewPSMT-64" d="M 3206 3922 L 3206 263 L 3556 263 Q 3653 263 3693 227 Q 3734 191 3734 131 Q 3734 75 3693 37 Q 3653 0 3556 0 L 2941 0 L 2941 522 Q 2484 -103 1781 -103 Q 1425 -103 1098 86 Q 772 275 583 625 Q 394 975 394 1347 Q 394 1722 583 2070 Q 772 2419 1098 2609 Q 1425 2800 1784 2800 Q 2472 2800 2941 2175 L 2941 3656 L 2591 3656 Q 2494 3656 2453 3693 Q 2413 3731 2413 3791 Q 2413 3847 2453 3884 Q 2494 3922 2591 3922 L 3206 3922 zM 2941 1347 Q 2941 1847 2603 2190 Q 2266 2534 1800 2534 Q 1331 2534 993 2190 Q 656 1847 656 1347 Q 656 850 993 504 Q 1331 159 1800 159 Q 2266 159 2603 504 Q 2941 850 2941 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-79" d="M 1938 0 L 719 2441 L 641 2441 Q 547 2441 506 2478 Q 466 2516 466 2572 Q 466 2613 486 2645 Q 506 2678 539 2692 Q 572 2706 641 2706 L 1359 2706 Q 1453 2706 1493 2668 Q 1534 2631 1534 2572 Q 1534 2516 1493 2478 Q 1453 2441 1359 2441 L 1006 2441 L 2078 288 L 3134 2441 L 2781 2441 Q 2688 2441 2647 2478 Q 2606 2516 2606 2575 Q 2606 2631 2647 2668 Q 2688 2706 2781 2706 L 3497 2706 Q 3594 2706 3634 2668 Q 3675 2631 3675 2572 Q 3675 2531 3650 2497 Q 3625 2463 3594 2452 Q 3563 2441 3419 2441 L 1756 -944 L 2166 -944 Q 2259 -944 2300 -980 Q 2341 -1016 2341 -1075 Q 2341 -1131 2300 -1168 Q 2259 -1206 2166 -1206 L 653 -1206 Q 559 -1206 518 -1170 Q 478 -1134 478 -1075 Q 478 -1016 518 -980 Q 559 -944 653 -944 L 1475 -944 L 1938 0 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6e" d="M 1072 2706 L 1072 2309 Q 1347 2588 1569 2694 Q 1791 2800 2069 2800 Q 2369 2800 2616 2672 Q 2791 2578 2933 2361 Q 3075 2144 3075 1916 L 3075 263 L 3297 263 Q 3391 263 3431 227 Q 3472 191 3472 131 Q 3472 75 3431 37 Q 3391 0 3297 0 L 2594 0 Q 2497 0 2456 37 Q 2416 75 2416 131 Q 2416 191 2456 227 Q 2497 263 2594 263 L 2813 263 L 2813 1872 Q 2813 2150 2609 2342 Q 2406 2534 2066 2534 Q 1806 2534 1615 2429 Q 1425 2325 1072 1909 L 1072 263 L 1369 263 Q 1463 263 1503 227 Q 1544 191 1544 131 Q 1544 75 1503 37 Q 1463 0 1369 0 L 513 0 Q 419 0 378 37 Q 338 75 338 131 Q 338 191 378 227 Q 419 263 513 263 L 809 263 L 809 2441 L 588 2441 Q 494 2441 453 2478 Q 413 2516 413 2575 Q 413 2631 453 2668 Q 494 2706 588 2706 L 1072 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6f" d="M 3388 1347 Q 3388 747 2958 322 Q 2528 -103 1922 -103 Q 1309 -103 881 323 Q 453 750 453 1347 Q 453 1947 881 2373 Q 1309 2800 1922 2800 Q 2528 2800 2958 2375 Q 3388 1950 3388 1347 zM 3122 1347 Q 3122 1841 2770 2187 Q 2419 2534 1919 2534 Q 1419 2534 1067 2186 Q 716 1838 716 1347 Q 716 859 1067 509 Q 1419 159 1919 159 Q 2419 159 2770 507 Q 3122 856 3122 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6d" d="M 725 2706 L 725 2441 Q 1059 2800 1397 2800 Q 1600 2800 1753 2692 Q 1906 2584 2009 2366 Q 2184 2584 2364 2692 Q 2544 2800 2725 2800 Q 3009 2800 3178 2616 Q 3400 2378 3400 2097 L 3400 263 L 3622 263 Q 3716 263 3756 227 Q 3797 191 3797 131 Q 3797 75 3756 37 Q 3716 0 3622 0 L 3138 0 L 3138 2072 Q 3138 2272 3016 2403 Q 2894 2534 2734 2534 Q 2591 2534 2431 2426 Q 2272 2319 2069 2003 L 2069 263 L 2288 263 Q 2381 263 2422 227 Q 2463 191 2463 131 Q 2463 75 2422 37 Q 2381 0 2288 0 L 1803 0 L 1803 2053 Q 1803 2263 1679 2398 Q 1556 2534 1403 2534 Q 1263 2534 1125 2444 Q 934 2316 725 2003 L 725 263 L 947 263 Q 1041 263 1081 227 Q 1122 191 1122 131 Q 1122 75 1081 37 Q 1041 0 947 0 L 241 0 Q 147 0 106 37 Q 66 75 66 131 Q 66 191 106 227 Q 147 263 241 263 L 463 263 L 463 2441 L 241 2441 Q 147 2441 106 2478 Q 66 2516 66 2575 Q 66 2631 106 2668 Q 147 2706 241 2706 L 725 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-69" d="M 2050 4053 L 2050 3375 L 1663 3375 L 1663 4053 L 2050 4053 zM 2059 2706 L 2059 263 L 3091 263 Q 3188 263 3228 227 Q 3269 191 3269 131 Q 3269 75 3228 37 Q 3188 0 3091 0 L 766 0 Q 672 0 631 37 Q 591 75 591 131 Q 591 191 631 227 Q 672 263 766 263 L 1797 263 L 1797 2441 L 1031 2441 Q 938 2441 895 2478 Q 853 2516 853 2572 Q 853 2631 893 2668 Q 934 2706 1031 2706 L 2059 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-67" d="M 2803 2225 L 2803 2706 L 3419 2706 Q 3513 2706 3553 2668 Q 3594 2631 3594 2572 Q 3594 2516 3553 2478 Q 3513 2441 3419 2441 L 3066 2441 L 3066 -181 Q 3066 -444 2953 -650 Q 2878 -788 2703 -934 Q 2528 -1081 2384 -1143 Q 2241 -1206 2000 -1206 L 1256 -1206 Q 1163 -1206 1122 -1170 Q 1081 -1134 1081 -1075 Q 1081 -1016 1122 -978 Q 1163 -941 1256 -941 L 2009 -944 Q 2241 -944 2427 -825 Q 2613 -706 2734 -478 Q 2803 -347 2803 -144 L 2803 644 Q 2381 66 1716 66 Q 1175 66 783 467 Q 391 869 391 1434 Q 391 2000 783 2400 Q 1175 2800 1716 2800 Q 2381 2800 2803 2225 zM 2803 1434 Q 2803 1897 2486 2215 Q 2169 2534 1728 2534 Q 1288 2534 972 2214 Q 656 1894 656 1434 Q 656 972 972 651 Q 1288 331 1728 331 Q 2169 331 2486 651 Q 2803 972 2803 1434 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-68" d="M 1056 3922 L 1056 2306 Q 1306 2578 1536 2689 Q 1766 2800 2050 2800 Q 2356 2800 2570 2692 Q 2784 2584 2928 2361 Q 3072 2138 3072 1891 L 3072 263 L 3366 263 Q 3463 263 3502 227 Q 3541 191 3541 131 Q 3541 75 3502 37 Q 3463 0 3366 0 L 2513 0 Q 2416 0 2375 37 Q 2334 75 2334 131 Q 2334 191 2375 227 Q 2416 263 2513 263 L 2806 263 L 2806 1872 Q 2806 2156 2601 2347 Q 2397 2538 2031 2538 Q 1744 2538 1541 2397 Q 1394 2297 1056 1925 L 1056 263 L 1353 263 Q 1447 263 1487 227 Q 1528 191 1528 131 Q 1528 75 1487 37 Q 1447 0 1353 0 L 497 0 Q 403 0 362 37 Q 322 75 322 131 Q 322 191 362 227 Q 403 263 497 263 L 794 263 L 794 3656 L 441 3656 Q 347 3656 306 3693 Q 266 3731 266 3791 Q 266 3847 306 3884 Q 347 3922 441 3922 L 1056 3922 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-74" d="M 1381 2706 L 2806 2706 Q 2900 2706 2940 2668 Q 2981 2631 2981 2572 Q 2981 2516 2940 2478 Q 2900 2441 2806 2441 L 1381 2441 L 1381 694 Q 1381 466 1564 312 Q 1747 159 2100 159 Q 2366 159 2675 239 Q 2984 319 3156 419 Q 3219 459 3259 459 Q 3309 459 3346 420 Q 3384 381 3384 328 Q 3384 281 3344 241 Q 3244 138 2858 17 Q 2472 -103 2119 -103 Q 1659 -103 1387 112 Q 1116 328 1116 694 L 1116 2441 L 631 2441 Q 538 2441 497 2478 Q 456 2516 456 2575 Q 456 2631 497 2668 Q 538 2706 631 2706 L 1116 2706 L 1116 3481 Q 1116 3575 1153 3615 Q 1191 3656 1247 3656 Q 1306 3656 1343 3615 Q 1381 3575 1381 3481 L 1381 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2e" d="M 1866 753 L 1978 753 Q 2153 753 2278 629 Q 2403 506 2403 331 Q 2403 150 2276 28 Q 2150 -94 1978 -94 L 1866 -94 Q 1691 -94 1566 29 Q 1441 153 1441 328 Q 1441 509 1567 631 Q 1694 753 1866 753 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-65" d="M 3341 1291 L 666 1291 Q 734 781 1092 470 Q 1450 159 1978 159 Q 2272 159 2594 256 Q 2916 353 3119 513 Q 3178 559 3222 559 Q 3272 559 3309 520 Q 3347 481 3347 428 Q 3347 375 3297 325 Q 3147 169 2764 33 Q 2381 -103 1978 -103 Q 1303 -103 851 339 Q 400 781 400 1409 Q 400 1981 823 2390 Q 1247 2800 1872 2800 Q 2516 2800 2931 2379 Q 3347 1959 3341 1291 zM 3075 1556 Q 2997 1991 2664 2262 Q 2331 2534 1872 2534 Q 1413 2534 1081 2265 Q 750 1997 669 1556 L 3075 1556 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2f" d="M 3100 4113 L 988 -413 Q 934 -528 850 -528 Q 797 -528 758 -489 Q 719 -450 719 -403 Q 719 -366 750 -300 L 2859 4225 Q 2894 4297 2925 4319 Q 2956 4341 3000 4341 Q 3053 4341 3092 4302 Q 3131 4263 3131 4216 Q 3131 4181 3100 4113 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-63" d="M 3016 2441 L 3016 2528 Q 3016 2625 3053 2665 Q 3091 2706 3147 2706 Q 3206 2706 3243 2665 Q 3281 2625 3281 2528 L 3281 1934 Q 3278 1838 3242 1797 Q 3206 1756 3147 1756 Q 3094 1756 3058 1792 Q 3022 1828 3016 1913 Q 2997 2134 2723 2334 Q 2450 2534 1988 2534 Q 1403 2534 1100 2168 Q 797 1803 797 1331 Q 797 822 1131 490 Q 1466 159 1997 159 Q 2303 159 2620 271 Q 2938 384 3194 634 Q 3259 697 3309 697 Q 3363 697 3398 661 Q 3434 625 3434 572 Q 3434 438 3119 231 Q 2609 -103 1984 -103 Q 1350 -103 942 301 Q 534 706 534 1328 Q 534 1963 951 2381 Q 1369 2800 2003 2800 Q 2606 2800 3016 2441 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-73" d="M 2788 2534 Q 2788 2625 2825 2665 Q 2863 2706 2919 2706 Q 2978 2706 3015 2665 Q 3053 2625 3053 2528 L 3053 2078 Q 3053 1984 3015 1943 Q 2978 1903 2919 1903 Q 2866 1903 2830 1937 Q 2794 1972 2788 2050 Q 2769 2238 2594 2359 Q 2338 2534 1916 2534 Q 1475 2534 1231 2356 Q 1047 2222 1047 2056 Q 1047 1869 1266 1744 Q 1416 1656 1834 1609 Q 2381 1550 2594 1475 Q 2897 1366 3045 1172 Q 3194 978 3194 753 Q 3194 419 2872 158 Q 2550 -103 1928 -103 Q 1306 -103 909 213 Q 909 106 896 75 Q 884 44 851 22 Q 819 0 778 0 Q 722 0 684 40 Q 647 81 647 175 L 647 716 Q 647 809 683 850 Q 719 891 778 891 Q 834 891 873 852 Q 913 813 913 747 Q 913 603 984 506 Q 1094 356 1333 257 Q 1572 159 1919 159 Q 2431 159 2681 350 Q 2931 541 2931 753 Q 2931 997 2678 1144 Q 2422 1291 1933 1341 Q 1444 1391 1231 1472 Q 1019 1553 900 1715 Q 781 1878 781 2066 Q 781 2403 1112 2601 Q 1444 2800 1903 2800 Q 2447 2800 2788 2534 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#CourierNewPSMT-64"/>      <use xlink:href="#CourierNewPSMT-79" x="60.009766"/>      <use xlink:href="#CourierNewPSMT-6e" x="120.019531"/>      <use xlink:href="#CourierNewPSMT-6f" x="180.029297"/>      <use xlink:href="#CourierNewPSMT-6d" x="240.039062"/>      <use xlink:href="#CourierNewPSMT-69" x="300.048828"/>      <use xlink:href="#CourierNewPSMT-67" x="360.058594"/>      <use xlink:href="#CourierNewPSMT-68" x="420.068359"/>      <use xlink:href="#CourierNewPSMT-74" x="480.078125"/>      <use xlink:href="#CourierNewPSMT-2e" x="540.087891"/>      <use xlink:href="#CourierNewPSMT-6e" x="600.097656"/>      <use xlink:href="#CourierNewPSMT-65" x="660.107422"/>      <use xlink:href="#CourierNewPSMT-74" x="720.117188"/>      <use xlink:href="#CourierNewPSMT-2f" x="780.126953"/>      <use xlink:href="#CourierNewPSMT-63" x="840.136719"/>      <use xlink:href="#CourierNewPSMT-68" x="900.146484"/>      <use xlink:href="#CourierNewPSMT-65" x="960.15625"/>      <use xlink:href="#CourierNewPSMT-73" x="1020.166016"/>      <use xlink:href="#CourierNewPSMT-73" x="1080.175781"/>     </g>    </a>   </g>   <g id="text_17">    <!-- gemma-2-27b vs stockfish -->    <g transform="translate(142.294688 24.24) scale(0.12 -0.12)">     <defs>      <path id="Montserrat-Regular-67" d="M 3750 3366 L 3750 410 Q 3750 -448 3331 -861 Q 2912 -1274 2067 -1274 Q 1600 -1274 1181 -1136 Q 762 -998 499 -755 L 730 -410 Q 973 -627 1321 -748 Q 1670 -870 2054 -870 Q 2694 -870 2995 -572 Q 3296 -275 3296 352 L 3296 781 Q 3085 461 2742 294 Q 2400 128 1984 128 Q 1510 128 1123 336 Q 736 544 515 918 Q 294 1293 294 1766 Q 294 2240 515 2611 Q 736 2982 1120 3187 Q 1504 3392 1984 3392 Q 2413 3392 2758 3219 Q 3104 3046 3315 2720 L 3315 3366 L 3750 3366 zM 2029 531 Q 2394 531 2688 688 Q 2982 845 3145 1126 Q 3309 1408 3309 1766 Q 3309 2125 3145 2403 Q 2982 2682 2691 2838 Q 2400 2995 2029 2995 Q 1664 2995 1373 2841 Q 1082 2688 918 2406 Q 755 2125 755 1766 Q 755 1408 918 1126 Q 1082 845 1373 688 Q 1664 531 2029 531 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6d" d="M 4832 3392 Q 5466 3392 5827 3027 Q 6189 2662 6189 1952 L 6189 0 L 5734 0 L 5734 1907 Q 5734 2432 5481 2707 Q 5229 2982 4768 2982 Q 4243 2982 3942 2659 Q 3642 2336 3642 1766 L 3642 0 L 3187 0 L 3187 1907 Q 3187 2432 2934 2707 Q 2682 2982 2214 2982 Q 1696 2982 1392 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2752 Q 1248 3059 1568 3225 Q 1888 3392 2304 3392 Q 2726 3392 3036 3213 Q 3347 3034 3501 2682 Q 3686 3014 4035 3203 Q 4384 3392 4832 3392 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-37" d="M 3494 4480 L 3494 4154 L 1619 0 L 1120 0 L 2963 4070 L 659 4070 L 659 3232 L 205 3232 L 205 4480 L 3494 4480 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-62" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1184 3827 793 Q 3610 403 3226 185 Q 2842 -32 2368 -32 Q 1946 -32 1610 147 Q 1274 326 1069 666 L 1069 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2726 Q 1299 3053 1628 3222 Q 1958 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2368 Q 3264 2669 2976 2832 Q 2688 2995 2336 2995 Q 1978 2995 1693 2832 Q 1408 2669 1245 2368 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-76" d="M 3462 3366 L 1965 0 L 1498 0 L 6 3366 L 480 3366 L 1741 499 L 3014 3366 L 3462 3366 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6f" d="M 2010 -32 Q 1523 -32 1132 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1132 3174 Q 1523 3392 2010 3392 Q 2496 3392 2886 3174 Q 3277 2957 3497 2566 Q 3718 2176 3718 1683 Q 3718 1190 3497 800 Q 3277 410 2886 189 Q 2496 -32 2010 -32 zM 2010 371 Q 2368 371 2653 534 Q 2938 698 3098 998 Q 3258 1299 3258 1683 Q 3258 2067 3098 2368 Q 2938 2669 2653 2832 Q 2368 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6b" d="M 1882 1690 L 1088 960 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 1530 L 3098 3366 L 3661 3366 L 2221 1990 L 3795 0 L 3238 0 L 1882 1690 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-68" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2778 Q 1274 3072 1600 3232 Q 1926 3392 2349 3392 z" transform="scale(0.015625)"/>     </defs>     <use xlink:href="#Montserrat-Regular-67"/>     <use xlink:href="#Montserrat-Regular-65" x="68.599991"/>     <use xlink:href="#Montserrat-Regular-6d" x="128.999985"/>     <use xlink:href="#Montserrat-Regular-6d" x="235.099976"/>     <use xlink:href="#Montserrat-Regular-61" x="341.199966"/>     <use xlink:href="#Montserrat-Regular-2d" x="400.199951"/>     <use xlink:href="#Montserrat-Regular-32" x="438.399948"/>     <use xlink:href="#Montserrat-Regular-2d" x="495.199936"/>     <use xlink:href="#Montserrat-Regular-32" x="533.399933"/>     <use xlink:href="#Montserrat-Regular-37" x="590.199921"/>     <use xlink:href="#Montserrat-Regular-62" x="649.099915"/>     <use xlink:href="#Montserrat-Regular-20" x="716.899902"/>     <use xlink:href="#Montserrat-Regular-76" x="743.099899"/>     <use xlink:href="#Montserrat-Regular-73" x="797.299896"/>     <use xlink:href="#Montserrat-Regular-20" x="846.19989"/>     <use xlink:href="#Montserrat-Regular-73" x="872.399887"/>     <use xlink:href="#Montserrat-Regular-74" x="921.299881"/>     <use xlink:href="#Montserrat-Regular-6f" x="961.899872"/>     <use xlink:href="#Montserrat-Regular-63" x="1024.599869"/>     <use xlink:href="#Montserrat-Regular-6b" x="1080.899857"/>     <use xlink:href="#Montserrat-Regular-66" x="1140.999847"/>     <use xlink:href="#Montserrat-Regular-69" x="1174.899841"/>     <use xlink:href="#Montserrat-Regular-73" x="1201.799835"/>     <use xlink:href="#Montserrat-Regular-68" x="1250.699829"/>    </g>   </g>  </g> </g> <defs>  <clipPath id="pcfa8e653e1">   <rect x="54" y="30.24" width="334.8" height="194.04"/>  </clipPath> </defs></svg>](https://dynomight.net/img/chess/gemma-2-27b_stockfish.pdf)

[Terrrible.](https://lichess.org/FeNVh8no)

And then I tried `gpt-3.5-turbo-instruct`. This is a closed OpenAI model, so details are very murky. I only ran 10 trials since AI companies have inexplicably neglected to send me free API keys and this was costing [The Automator](https://dynomight.net/automated/) money. The results were:

[<?xml version="1.0" encoding="utf-8" standalone="no"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"  "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg xmlns:xlink="http://www.w3.org/1999/xlink" width="432pt" height="252pt" viewBox="0 0 432 252" xmlns="http://www.w3.org/2000/svg" version="1.1"> <metadata>  <rdf:RDF xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:cc="http://creativecommons.org/ns#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">   <cc:Work>    <dc:type rdf:resource="http://purl.org/dc/dcmitype/StillImage"/>    <dc:date>2024-11-14T11:33:09.545234</dc:date>    <dc:format>image/svg+xml</dc:format>    <dc:creator>     <cc:Agent>      <dc:title>Matplotlib v3.8.4, https://matplotlib.org/</dc:title>     </cc:Agent>    </dc:creator>   </cc:Work>  </rdf:RDF> </metadata> <defs>  <style type="text/css">*{stroke-linejoin: round; stroke-linecap: butt}</style> </defs> <g id="figure_1">  <g id="patch_1">   <path d="M 0 252 L 432 252 L 432 0 L 0 0 z" style="fill: #ffffff"/>  </g>  <g id="axes_1">   <g id="patch_2">    <path d="M 54 224.28 L 388.8 224.28 L 388.8 30.24 L 54 30.24 z" style="fill: #ffffff"/>   </g>   <g id="matplotlib.axis_1">    <g id="xtick_1">     <g id="line2d_1"/>     <g id="text_1">      <!-- 0 -->      <g transform="translate(50.689844 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-30" d="M 2118 -38 Q 1600 -38 1193 237 Q 787 512 560 1027 Q 333 1542 333 2240 Q 333 2938 560 3453 Q 787 3968 1193 4243 Q 1600 4518 2118 4518 Q 2637 4518 3043 4243 Q 3450 3968 3680 3453 Q 3910 2938 3910 2240 Q 3910 1542 3680 1027 Q 3450 512 3043 237 Q 2637 -38 2118 -38 zM 2118 384 Q 2509 384 2806 601 Q 3104 819 3270 1235 Q 3437 1651 3437 2240 Q 3437 2829 3270 3245 Q 3104 3661 2806 3878 Q 2509 4096 2118 4096 Q 1728 4096 1430 3878 Q 1133 3661 966 3245 Q 800 2829 800 2240 Q 800 1651 966 1235 Q 1133 819 1430 601 Q 1728 384 2118 384 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="xtick_2">     <g id="line2d_2"/>     <g id="text_2">      <!-- 20 -->      <g transform="translate(114.81 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-32" d="M 3392 410 L 3392 0 L 250 0 L 250 326 L 2112 2150 Q 2464 2496 2589 2749 Q 2714 3002 2714 3258 Q 2714 3654 2442 3875 Q 2170 4096 1664 4096 Q 877 4096 442 3597 L 115 3878 Q 378 4186 784 4352 Q 1190 4518 1702 4518 Q 2387 4518 2787 4195 Q 3187 3872 3187 3309 Q 3187 2963 3033 2643 Q 2880 2323 2451 1907 L 928 410 L 3392 410 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-32"/>       <use xlink:href="#Montserrat-Regular-30" x="56.799988"/>      </g>     </g>    </g>    <g id="xtick_3">     <g id="line2d_3"/>     <g id="text_3">      <!-- 40 -->      <g transform="translate(181.305156 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-34" d="M 4141 1178 L 3245 1178 L 3245 0 L 2784 0 L 2784 1178 L 256 1178 L 256 1510 L 2630 4480 L 3142 4480 L 851 1587 L 2797 1587 L 2797 2624 L 3245 2624 L 3245 1587 L 4141 1587 L 4141 1178 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-34"/>       <use xlink:href="#Montserrat-Regular-30" x="66.099991"/>      </g>     </g>    </g>    <g id="xtick_4">     <g id="line2d_4"/>     <g id="text_4">      <!-- 60 -->      <g transform="translate(248.524531 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-36" d="M 2189 2682 Q 2630 2682 2972 2515 Q 3315 2349 3507 2045 Q 3699 1741 3699 1338 Q 3699 928 3500 614 Q 3302 301 2953 131 Q 2605 -38 2170 -38 Q 1280 -38 806 547 Q 333 1133 333 2202 Q 333 2950 589 3468 Q 845 3987 1305 4252 Q 1766 4518 2381 4518 Q 3046 4518 3443 4275 L 3258 3904 Q 2938 4115 2387 4115 Q 1645 4115 1222 3644 Q 800 3174 800 2278 Q 800 2118 819 1914 Q 986 2278 1350 2480 Q 1715 2682 2189 2682 zM 2150 352 Q 2637 352 2941 617 Q 3245 883 3245 1325 Q 3245 1766 2941 2028 Q 2637 2291 2118 2291 Q 1779 2291 1520 2163 Q 1261 2035 1113 1811 Q 966 1587 966 1312 Q 966 1056 1104 838 Q 1242 621 1510 486 Q 1779 352 2150 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-36"/>       <use xlink:href="#Montserrat-Regular-30" x="60.899994"/>      </g>     </g>    </g>    <g id="xtick_5">     <g id="line2d_5"/>     <g id="text_5">      <!-- 80 -->      <g transform="translate(315.34 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-38" d="M 2899 2349 Q 3322 2208 3549 1926 Q 3776 1645 3776 1248 Q 3776 858 3565 566 Q 3354 275 2960 118 Q 2566 -38 2035 -38 Q 1229 -38 768 307 Q 307 653 307 1248 Q 307 1651 528 1929 Q 749 2208 1171 2349 Q 826 2483 646 2736 Q 467 2989 467 3334 Q 467 3878 892 4198 Q 1318 4518 2035 4518 Q 2509 4518 2864 4374 Q 3219 4230 3417 3961 Q 3616 3693 3616 3334 Q 3616 2989 3430 2736 Q 3245 2483 2899 2349 zM 928 3322 Q 928 2950 1222 2732 Q 1517 2515 2035 2515 Q 2554 2515 2851 2732 Q 3149 2950 3149 3315 Q 3149 3693 2845 3910 Q 2541 4128 2035 4128 Q 1523 4128 1225 3910 Q 928 3693 928 3322 zM 2035 352 Q 2630 352 2966 595 Q 3302 838 3302 1254 Q 3302 1670 2966 1910 Q 2630 2150 2035 2150 Q 1446 2150 1110 1910 Q 774 1670 774 1254 Q 774 832 1107 592 Q 1440 352 2035 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-38"/>       <use xlink:href="#Montserrat-Regular-30" x="63.799988"/>      </g>     </g>    </g>    <g id="xtick_6">     <g id="line2d_6"/>     <g id="text_6">      <!-- 100 -->      <g transform="translate(380.375 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-31" d="M 1574 4480 L 1574 0 L 1114 0 L 1114 4070 L 58 4070 L 58 4480 L 1574 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>      </g>     </g>    </g>    <g id="text_7">     <!-- turns -->     <g transform="translate(208.170312 248.56125) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-74" d="M 2406 205 Q 2278 90 2089 29 Q 1901 -32 1696 -32 Q 1222 -32 966 224 Q 710 480 710 947 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 4102 L 1165 4102 L 1165 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 973 Q 1165 672 1315 515 Q 1466 358 1747 358 Q 1888 358 2019 403 Q 2150 448 2246 531 L 2406 205 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-75" d="M 3667 3366 L 3667 0 L 3232 0 L 3232 614 Q 3053 307 2739 137 Q 2426 -32 2022 -32 Q 1363 -32 982 336 Q 602 704 602 1414 L 602 3366 L 1056 3366 L 1056 1459 Q 1056 928 1318 653 Q 1581 378 2067 378 Q 2598 378 2905 701 Q 3213 1024 3213 1600 L 3213 3366 L 3667 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-72" d="M 1069 2707 Q 1229 3046 1545 3219 Q 1862 3392 2330 3392 L 2330 2950 L 2221 2957 Q 1690 2957 1389 2630 Q 1088 2304 1088 1715 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2707 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-6e" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2746 Q 1254 3053 1584 3222 Q 1914 3392 2349 3392 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-73" d="M 1536 -32 Q 1126 -32 752 86 Q 378 205 166 384 L 371 742 Q 582 576 902 470 Q 1222 365 1568 365 Q 2029 365 2249 509 Q 2470 653 2470 915 Q 2470 1101 2348 1206 Q 2227 1312 2041 1366 Q 1856 1421 1549 1472 Q 1139 1549 889 1629 Q 640 1709 464 1901 Q 288 2093 288 2432 Q 288 2854 640 3123 Q 992 3392 1619 3392 Q 1946 3392 2272 3305 Q 2598 3219 2810 3078 L 2611 2714 Q 2195 3002 1619 3002 Q 1184 3002 963 2848 Q 742 2694 742 2445 Q 742 2253 867 2137 Q 992 2022 1177 1968 Q 1363 1914 1690 1856 Q 2093 1779 2336 1702 Q 2579 1626 2752 1440 Q 2925 1254 2925 928 Q 2925 486 2557 227 Q 2189 -32 1536 -32 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-74"/>      <use xlink:href="#Montserrat-Regular-75" x="40.599991"/>      <use xlink:href="#Montserrat-Regular-72" x="107.899979"/>      <use xlink:href="#Montserrat-Regular-6e" x="147.999969"/>      <use xlink:href="#Montserrat-Regular-73" x="215.699951"/>     </g>    </g>   </g>   <g id="matplotlib.axis_2">    <g id="ytick_1">     <g id="line2d_7"/>     <g id="text_8">      <!-- LOSS -->      <g transform="translate(23.91875 227.667831) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-4c" d="M 730 4480 L 1203 4480 L 1203 410 L 3712 410 L 3712 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4f" d="M 2688 -38 Q 2022 -38 1481 259 Q 941 557 637 1075 Q 333 1594 333 2240 Q 333 2886 637 3404 Q 941 3923 1481 4220 Q 2022 4518 2688 4518 Q 3354 4518 3888 4224 Q 4422 3930 4729 3408 Q 5037 2886 5037 2240 Q 5037 1594 4729 1072 Q 4422 550 3888 256 Q 3354 -38 2688 -38 zM 2688 384 Q 3219 384 3648 624 Q 4077 864 4320 1289 Q 4563 1715 4563 2240 Q 4563 2765 4320 3190 Q 4077 3616 3648 3856 Q 3219 4096 2688 4096 Q 2157 4096 1725 3856 Q 1293 3616 1046 3190 Q 800 2765 800 2240 Q 800 1715 1046 1289 Q 1293 864 1725 624 Q 2157 384 2688 384 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-53" d="M 1965 -38 Q 1466 -38 1008 122 Q 550 282 301 544 L 486 909 Q 730 666 1133 515 Q 1536 365 1965 365 Q 2566 365 2867 585 Q 3168 806 3168 1158 Q 3168 1427 3005 1587 Q 2842 1747 2605 1833 Q 2368 1920 1946 2022 Q 1440 2150 1139 2268 Q 838 2387 624 2630 Q 410 2874 410 3290 Q 410 3629 589 3907 Q 768 4186 1139 4352 Q 1510 4518 2061 4518 Q 2445 4518 2813 4412 Q 3181 4307 3450 4122 L 3290 3744 Q 3008 3930 2688 4022 Q 2368 4115 2061 4115 Q 1472 4115 1174 3888 Q 877 3661 877 3302 Q 877 3034 1040 2870 Q 1203 2707 1449 2620 Q 1696 2534 2112 2432 Q 2605 2310 2909 2192 Q 3213 2074 3424 1834 Q 3635 1594 3635 1184 Q 3635 845 3452 566 Q 3270 288 2892 125 Q 2515 -38 1965 -38 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-4c"/>       <use xlink:href="#Montserrat-Regular-4f" x="58.899994"/>       <use xlink:href="#Montserrat-Regular-53" x="142.799988"/>       <use xlink:href="#Montserrat-Regular-53" x="204.299973"/>      </g>     </g>    </g>    <g id="ytick_2">     <g id="line2d_8"/>     <g id="text_9">      <!-- -1000 -->      <g transform="translate(23.209375 195.435273) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-2d" d="M 384 1933 L 2061 1933 L 2061 1536 L 384 1536 L 384 1933 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-31" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="74.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="140.499985"/>       <use xlink:href="#Montserrat-Regular-30" x="206.699982"/>      </g>     </g>    </g>    <g id="ytick_3">     <g id="line2d_9"/>     <g id="text_10">      <!-- -500 -->      <g transform="translate(27.779687 163.202714) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-35" d="M 1510 2643 Q 2483 2643 2928 2294 Q 3373 1946 3373 1318 Q 3373 928 3190 621 Q 3008 314 2643 138 Q 2278 -38 1747 -38 Q 1267 -38 835 118 Q 403 275 141 538 L 365 902 Q 589 672 953 528 Q 1318 384 1741 384 Q 2298 384 2602 633 Q 2906 883 2906 1299 Q 2906 1760 2563 1997 Q 2221 2234 1408 2234 L 454 2234 L 685 4480 L 3136 4480 L 3136 4070 L 1088 4070 L 934 2643 L 1510 2643 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-35" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="94.799988"/>       <use xlink:href="#Montserrat-Regular-30" x="160.999985"/>      </g>     </g>    </g>    <g id="ytick_4">     <g id="line2d_10"/>     <g id="text_11">      <!-- 0 -->      <g transform="translate(43.879688 130.970156) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="ytick_5">     <g id="line2d_11"/>     <g id="text_12">      <!-- 500 -->      <g transform="translate(31.6 98.737598) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-35"/>       <use xlink:href="#Montserrat-Regular-30" x="56.599991"/>       <use xlink:href="#Montserrat-Regular-30" x="122.799988"/>      </g>     </g>    </g>    <g id="ytick_6">     <g id="line2d_12"/>     <g id="text_13">      <!-- 1000 -->      <g transform="translate(27.029687 66.50504) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="168.499985"/>      </g>     </g>    </g>    <g id="ytick_7">     <g id="line2d_13"/>     <g id="text_14">      <!-- WIN -->      <g transform="translate(28.240625 34.272482) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-57" d="M 6880 4480 L 5370 0 L 4870 0 L 3558 3821 L 2240 0 L 1747 0 L 237 4480 L 723 4480 L 2016 627 L 3354 4480 L 3795 4480 L 5114 608 L 6426 4480 L 6880 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-49" d="M 730 4480 L 1203 4480 L 1203 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4e" d="M 4474 4480 L 4474 0 L 4083 0 L 1203 3635 L 1203 0 L 730 0 L 730 4480 L 1120 4480 L 4006 845 L 4006 4480 L 4474 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-57"/>       <use xlink:href="#Montserrat-Regular-49" x="111.099991"/>       <use xlink:href="#Montserrat-Regular-4e" x="141.299988"/>      </g>     </g>    </g>    <g id="text_15">     <!-- centipawn difference -->     <g transform="translate(17.26875 180.677188) rotate(-90) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-63" d="M 2029 -32 Q 1530 -32 1136 185 Q 742 403 518 796 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1136 3174 Q 1530 3392 2029 3392 Q 2464 3392 2806 3222 Q 3149 3053 3347 2726 L 3008 2496 Q 2842 2746 2586 2870 Q 2330 2995 2029 2995 Q 1664 2995 1373 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1293 918 995 Q 1082 698 1373 534 Q 1664 371 2029 371 Q 2330 371 2586 492 Q 2842 614 3008 864 L 3347 634 Q 3149 307 2803 137 Q 2458 -32 2029 -32 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-65" d="M 3565 1542 L 749 1542 Q 787 1018 1152 694 Q 1517 371 2074 371 Q 2387 371 2649 483 Q 2912 595 3104 813 L 3360 518 Q 3136 250 2800 109 Q 2464 -32 2061 -32 Q 1542 -32 1142 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 508 2566 Q 723 2957 1097 3174 Q 1472 3392 1939 3392 Q 2406 3392 2777 3174 Q 3149 2957 3360 2569 Q 3571 2182 3571 1683 L 3565 1542 zM 1939 3002 Q 1453 3002 1123 2691 Q 794 2381 749 1882 L 3136 1882 Q 3091 2381 2761 2691 Q 2432 3002 1939 3002 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-69" d="M 634 3366 L 1088 3366 L 1088 0 L 634 0 L 634 3366 zM 864 4102 Q 723 4102 627 4198 Q 531 4294 531 4429 Q 531 4557 627 4653 Q 723 4749 864 4749 Q 1005 4749 1101 4656 Q 1197 4563 1197 4435 Q 1197 4294 1101 4198 Q 1005 4102 864 4102 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-70" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1178 3827 787 Q 3610 397 3229 182 Q 2848 -32 2368 -32 Q 1958 -32 1628 137 Q 1299 307 1088 634 L 1088 -1242 L 634 -1242 L 634 3366 L 1069 3366 L 1069 2701 Q 1274 3034 1610 3213 Q 1946 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2364 Q 3264 2662 2976 2828 Q 2688 2995 2336 2995 Q 1978 2995 1693 2828 Q 1408 2662 1245 2364 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-61" d="M 1811 3392 Q 2470 3392 2822 3062 Q 3174 2733 3174 2086 L 3174 0 L 2739 0 L 2739 525 Q 2586 262 2288 115 Q 1990 -32 1581 -32 Q 1018 -32 685 237 Q 352 506 352 947 Q 352 1376 662 1638 Q 973 1901 1651 1901 L 2720 1901 L 2720 2106 Q 2720 2541 2477 2768 Q 2234 2995 1766 2995 Q 1446 2995 1152 2889 Q 858 2784 646 2598 L 442 2938 Q 698 3155 1056 3273 Q 1414 3392 1811 3392 zM 1651 326 Q 2035 326 2310 502 Q 2586 678 2720 1011 L 2720 1562 L 1664 1562 Q 800 1562 800 960 Q 800 666 1024 496 Q 1248 326 1651 326 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-77" d="M 5574 3366 L 4301 0 L 3872 0 L 2810 2758 L 1747 0 L 1318 0 L 51 3366 L 486 3366 L 1542 506 L 2624 3366 L 3014 3366 L 4090 506 L 5158 3366 L 5574 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-20" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-64" d="M 3706 4749 L 3706 0 L 3270 0 L 3270 666 Q 3066 326 2730 147 Q 2394 -32 1971 -32 Q 1498 -32 1114 185 Q 730 403 512 793 Q 294 1184 294 1683 Q 294 2182 512 2572 Q 730 2963 1114 3177 Q 1498 3392 1971 3392 Q 2381 3392 2710 3222 Q 3040 3053 3251 2726 L 3251 4749 L 3706 4749 zM 2010 371 Q 2362 371 2646 534 Q 2931 698 3094 998 Q 3258 1299 3258 1683 Q 3258 2067 3094 2368 Q 2931 2669 2646 2832 Q 2362 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-66" d="M 1747 4397 Q 1453 4397 1302 4237 Q 1152 4077 1152 3763 L 1152 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 0 L 710 0 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 3782 Q 710 4243 976 4512 Q 1242 4781 1728 4781 Q 1920 4781 2099 4726 Q 2278 4672 2400 4570 L 2246 4230 Q 2048 4397 1747 4397 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-63"/>      <use xlink:href="#Montserrat-Regular-65" x="56.299988"/>      <use xlink:href="#Montserrat-Regular-6e" x="116.699982"/>      <use xlink:href="#Montserrat-Regular-74" x="184.399963"/>      <use xlink:href="#Montserrat-Regular-69" x="224.999954"/>      <use xlink:href="#Montserrat-Regular-70" x="251.899948"/>      <use xlink:href="#Montserrat-Regular-61" x="319.699936"/>      <use xlink:href="#Montserrat-Regular-77" x="378.699921"/>      <use xlink:href="#Montserrat-Regular-6e" x="466.599915"/>      <use xlink:href="#Montserrat-Regular-20" x="534.299896"/>      <use xlink:href="#Montserrat-Regular-64" x="560.499893"/>      <use xlink:href="#Montserrat-Regular-69" x="628.299881"/>      <use xlink:href="#Montserrat-Regular-66" x="655.199875"/>      <use xlink:href="#Montserrat-Regular-66" x="689.099869"/>      <use xlink:href="#Montserrat-Regular-65" x="722.999863"/>      <use xlink:href="#Montserrat-Regular-72" x="783.399857"/>      <use xlink:href="#Montserrat-Regular-65" x="823.499847"/>      <use xlink:href="#Montserrat-Regular-6e" x="883.899841"/>      <use xlink:href="#Montserrat-Regular-63" x="951.599823"/>      <use xlink:href="#Montserrat-Regular-65" x="1007.899811"/>     </g>    </g>   </g>   <g id="line2d_14">    <path d="M 54 127.26 L 57.348 125.648372 L 60.696 124.294605 L 64.044 125.970698 L 67.392 126.744279 L 70.74 124.036744 L 74.088 124.036744 L 77.436 124.681395 L 80.784 124.423535 L 84.132 124.036744 L 87.48 125.197116 L 90.828 124.681395 L 94.176 124.488 L 97.524 117.010047 L 100.872 105.148465 L 104.22 101.022698 L 107.568 95.220837 L 110.916 90.450419 L 114.264 81.425302 L 117.612 89.483442 L 120.96 94.060465 L 124.308 93.157953 L 127.656 91.933116 L 131.004 97.799442 L 134.352 76.139163 L 137.7 83.230326 L 141.048 30.626791 L 144.396 30.626791 L 147.744 30.626791 L 151.092 30.626791 L 154.44 30.626791 L 157.788 30.626791 L 161.136 30.626791 L 164.484 30.626791 L 167.832 30.626791 L 171.18 30.626791 L 174.528 30.626791 L 177.876 30.626791 L 181.224 30.626791 L 184.572 30.626791 L 187.92 30.626791 L 191.268 30.626791 L 194.616 30.626791 L 197.964 30.626791 L 201.312 30.626791 L 204.66 30.626791 L 208.008 30.626791 L 211.356 30.626791 L 214.704 30.626791 L 218.052 30.626791 L 221.4 30.626791 L 224.748 30.626791 L 228.096 30.626791 L 231.444 30.626791 L 234.792 30.626791 L 238.14 30.626791 L 241.488 30.626791 L 244.836 30.626791 L 248.184 30.626791 L 251.532 30.626791 L 254.88 30.626791 L 258.228 30.626791 L 261.576 30.626791 L 264.924 30.626791 L 268.272 30.626791 L 271.62 30.626791 L 274.968 30.626791 L 278.316 30.626791 L 281.664 30.626791 L 285.012 30.626791 L 288.36 30.626791 L 291.708 30.626791 L 295.056 30.626791 L 298.404 30.626791 L 301.752 30.626791 L 305.1 30.626791 L 308.448 30.626791 L 311.796 30.626791 L 315.144 30.626791 L 318.492 30.626791 L 321.84 30.626791 L 325.188 30.626791 L 328.536 30.626791 L 331.884 30.626791 L 335.232 30.626791 L 338.58 30.626791 L 341.928 30.626791 L 345.276 30.626791 L 348.624 30.626791 L 351.972 30.626791 L 355.32 30.626791 L 358.668 30.626791 L 362.016 30.626791 L 365.364 30.626791 L 368.712 30.626791 L 372.06 30.626791 L 375.408 30.626791 L 378.756 30.626791 L 382.104 30.626791 L 385.452 30.626791 " clip-path="url(#pa80ca89b9d)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_15">    <path d="M 54 127.26 L 57.348 122.811907 L 60.696 124.101209 L 64.044 126.615349 L 67.392 127.711256 L 70.74 128.355907 L 74.088 118.68614 L 77.436 116.945581 L 80.784 115.527349 L 84.132 122.296186 L 87.48 125.648372 L 90.828 121.45814 L 94.176 118.041488 L 97.524 120.555628 L 100.872 118.621674 L 104.22 121.45814 L 107.568 149.17814 L 110.916 142.860558 L 114.264 137.767814 L 117.612 137.767814 L 120.96 137.703349 L 124.308 126.357488 L 127.656 130.28986 L 131.004 124.552465 L 134.352 104.439349 L 137.7 95.414233 L 141.048 109.209767 L 144.396 114.238047 L 147.744 79.426884 L 151.092 69.950512 L 154.44 70.595163 L 157.788 70.466233 L 161.136 30.626791 L 164.484 30.626791 L 167.832 30.626791 L 171.18 30.626791 L 174.528 30.626791 L 177.876 30.626791 L 181.224 30.626791 L 184.572 30.626791 L 187.92 30.626791 L 191.268 30.626791 L 194.616 30.626791 L 197.964 30.626791 L 201.312 30.626791 L 204.66 30.626791 L 208.008 30.626791 L 211.356 30.626791 L 214.704 30.626791 L 218.052 30.626791 L 221.4 30.626791 L 224.748 30.626791 L 228.096 30.626791 L 231.444 30.626791 L 234.792 30.626791 L 238.14 30.626791 L 241.488 30.626791 L 244.836 30.626791 L 248.184 30.626791 L 251.532 30.626791 L 254.88 30.626791 L 258.228 30.626791 L 261.576 30.626791 L 264.924 30.626791 L 268.272 30.626791 L 271.62 30.626791 L 274.968 30.626791 L 278.316 30.626791 L 281.664 30.626791 L 285.012 30.626791 L 288.36 30.626791 L 291.708 30.626791 L 295.056 30.626791 L 298.404 30.626791 L 301.752 30.626791 L 305.1 30.626791 L 308.448 30.626791 L 311.796 30.626791 L 315.144 30.626791 L 318.492 30.626791 L 321.84 30.626791 L 325.188 30.626791 L 328.536 30.626791 L 331.884 30.626791 L 335.232 30.626791 L 338.58 30.626791 L 341.928 30.626791 L 345.276 30.626791 L 348.624 30.626791 L 351.972 30.626791 L 355.32 30.626791 L 358.668 30.626791 L 362.016 30.626791 L 365.364 30.626791 L 368.712 30.626791 L 372.06 30.626791 L 375.408 30.626791 L 378.756 30.626791 L 382.104 30.626791 L 385.452 30.626791 " clip-path="url(#pa80ca89b9d)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_16">    <path d="M 54 127.26 L 57.348 118.750605 L 60.696 115.140558 L 64.044 118.557209 L 67.392 117.267907 L 70.74 115.269488 L 74.088 118.68614 L 77.436 115.333953 L 80.784 116.816651 L 84.132 114.947163 L 87.48 103.278977 L 90.828 92.577767 L 94.176 88.065209 L 97.524 85.228744 L 100.872 80.845116 L 104.22 79.104558 L 107.568 80.909581 L 110.916 71.94893 L 114.264 76.654884 L 117.612 77.49293 L 120.96 92.900093 L 124.308 85.873395 L 127.656 98.057302 L 131.004 93.222419 L 134.352 95.156372 L 137.7 96.703535 L 141.048 100.700372 L 144.396 108.242791 L 147.744 105.084 L 151.092 97.02586 L 154.44 99.79786 L 157.788 100.249116 L 161.136 96.89693 L 164.484 98.701953 L 167.832 95.156372 L 171.18 96.252279 L 174.528 100.055721 L 177.876 93.931535 L 181.224 95.801023 L 184.572 98.250698 L 187.92 97.412651 L 191.268 95.220837 L 194.616 85.42214 L 197.964 100.635907 L 201.312 94.640651 L 204.66 87.227163 L 208.008 77.428465 L 211.356 74.33414 L 214.704 67.952093 L 218.052 30.626791 L 221.4 30.626791 L 224.748 30.626791 L 228.096 30.626791 L 231.444 30.626791 L 234.792 30.626791 L 238.14 30.626791 L 241.488 30.626791 L 244.836 30.626791 L 248.184 30.626791 L 251.532 30.626791 L 254.88 30.626791 L 258.228 30.626791 L 261.576 30.626791 L 264.924 30.626791 L 268.272 30.626791 L 271.62 30.626791 L 274.968 30.626791 L 278.316 30.626791 L 281.664 30.626791 L 285.012 30.626791 L 288.36 30.626791 L 291.708 30.626791 L 295.056 30.626791 L 298.404 30.626791 L 301.752 30.626791 L 305.1 30.626791 L 308.448 30.626791 L 311.796 30.626791 L 315.144 30.626791 L 318.492 30.626791 L 321.84 30.626791 L 325.188 30.626791 L 328.536 30.626791 L 331.884 30.626791 L 335.232 30.626791 L 338.58 30.626791 L 341.928 30.626791 L 345.276 30.626791 L 348.624 30.626791 L 351.972 30.626791 L 355.32 30.626791 L 358.668 30.626791 L 362.016 30.626791 L 365.364 30.626791 L 368.712 30.626791 L 372.06 30.626791 L 375.408 30.626791 L 378.756 30.626791 L 382.104 30.626791 L 385.452 30.626791 " clip-path="url(#pa80ca89b9d)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_17">    <path d="M 54 127.26 L 57.348 119.782047 L 60.696 123.069767 L 64.044 122.489581 L 67.392 124.810326 L 70.74 124.294605 L 74.088 125.454977 L 77.436 124.23014 L 80.784 124.423535 L 84.132 123.392093 L 87.48 126.421953 L 90.828 126.421953 L 94.176 94.769581 L 97.524 114.495907 L 100.872 120.104372 L 104.22 90.837209 L 107.568 113.722326 L 110.916 113.206605 L 114.264 112.884279 L 117.612 112.110698 L 120.96 113.65786 L 124.308 86.840372 L 127.656 109.403163 L 131.004 100.249116 L 134.352 103.730233 L 137.7 108.694047 L 141.048 81.554233 L 144.396 69.30586 L 147.744 66.727256 L 151.092 30.626791 L 154.44 30.626791 L 157.788 30.626791 L 161.136 30.626791 L 164.484 30.626791 L 167.832 30.626791 L 171.18 30.626791 L 174.528 30.626791 L 177.876 30.626791 L 181.224 30.626791 L 184.572 30.626791 L 187.92 30.626791 L 191.268 30.626791 L 194.616 30.626791 L 197.964 30.626791 L 201.312 30.626791 L 204.66 30.626791 L 208.008 30.626791 L 211.356 30.626791 L 214.704 30.626791 L 218.052 30.626791 L 221.4 30.626791 L 224.748 30.626791 L 228.096 30.626791 L 231.444 30.626791 L 234.792 30.626791 L 238.14 30.626791 L 241.488 30.626791 L 244.836 30.626791 L 248.184 30.626791 L 251.532 30.626791 L 254.88 30.626791 L 258.228 30.626791 L 261.576 30.626791 L 264.924 30.626791 L 268.272 30.626791 L 271.62 30.626791 L 274.968 30.626791 L 278.316 30.626791 L 281.664 30.626791 L 285.012 30.626791 L 288.36 30.626791 L 291.708 30.626791 L 295.056 30.626791 L 298.404 30.626791 L 301.752 30.626791 L 305.1 30.626791 L 308.448 30.626791 L 311.796 30.626791 L 315.144 30.626791 L 318.492 30.626791 L 321.84 30.626791 L 325.188 30.626791 L 328.536 30.626791 L 331.884 30.626791 L 335.232 30.626791 L 338.58 30.626791 L 341.928 30.626791 L 345.276 30.626791 L 348.624 30.626791 L 351.972 30.626791 L 355.32 30.626791 L 358.668 30.626791 L 362.016 30.626791 L 365.364 30.626791 L 368.712 30.626791 L 372.06 30.626791 L 375.408 30.626791 L 378.756 30.626791 L 382.104 30.626791 L 385.452 30.626791 " clip-path="url(#pa80ca89b9d)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_18">    <path d="M 54 127.26 L 57.348 124.74586 L 60.696 123.069767 L 64.044 124.74586 L 67.392 120.877953 L 70.74 122.167256 L 74.088 115.333953 L 77.436 120.491163 L 80.784 119.008465 L 84.132 117.267907 L 87.48 117.848093 L 90.828 123.778884 L 94.176 121.909395 L 97.524 124.165674 L 100.872 113.65786 L 104.22 89.741302 L 107.568 88.452 L 110.916 95.607628 L 114.264 78.073116 L 117.612 84.777488 L 120.96 85.615535 L 124.308 78.911163 L 127.656 84.197302 L 131.004 76.203628 L 134.352 30.626791 L 137.7 30.626791 L 141.048 30.626791 L 144.396 30.626791 L 147.744 30.626791 L 151.092 30.626791 L 154.44 30.626791 L 157.788 30.626791 L 161.136 30.626791 L 164.484 30.626791 L 167.832 30.626791 L 171.18 30.626791 L 174.528 30.626791 L 177.876 30.626791 L 181.224 30.626791 L 184.572 30.626791 L 187.92 30.626791 L 191.268 30.626791 L 194.616 30.626791 L 197.964 30.626791 L 201.312 30.626791 L 204.66 30.626791 L 208.008 30.626791 L 211.356 30.626791 L 214.704 30.626791 L 218.052 30.626791 L 221.4 30.626791 L 224.748 30.626791 L 228.096 30.626791 L 231.444 30.626791 L 234.792 30.626791 L 238.14 30.626791 L 241.488 30.626791 L 244.836 30.626791 L 248.184 30.626791 L 251.532 30.626791 L 254.88 30.626791 L 258.228 30.626791 L 261.576 30.626791 L 264.924 30.626791 L 268.272 30.626791 L 271.62 30.626791 L 274.968 30.626791 L 278.316 30.626791 L 281.664 30.626791 L 285.012 30.626791 L 288.36 30.626791 L 291.708 30.626791 L 295.056 30.626791 L 298.404 30.626791 L 301.752 30.626791 L 305.1 30.626791 L 308.448 30.626791 L 311.796 30.626791 L 315.144 30.626791 L 318.492 30.626791 L 321.84 30.626791 L 325.188 30.626791 L 328.536 30.626791 L 331.884 30.626791 L 335.232 30.626791 L 338.58 30.626791 L 341.928 30.626791 L 345.276 30.626791 L 348.624 30.626791 L 351.972 30.626791 L 355.32 30.626791 L 358.668 30.626791 L 362.016 30.626791 L 365.364 30.626791 L 368.712 30.626791 L 372.06 30.626791 L 375.408 30.626791 L 378.756 30.626791 L 382.104 30.626791 L 385.452 30.626791 " clip-path="url(#pa80ca89b9d)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_19">    <path d="M 54 127.26 L 57.348 121.97386 L 60.696 116.04307 L 64.044 119.395256 L 67.392 118.879535 L 70.74 121.780465 L 74.088 121.58707 L 77.436 120.813488 L 80.784 117.590233 L 84.132 120.491163 L 87.48 122.425116 L 90.828 125.003721 L 94.176 126.679814 L 97.524 113.14214 L 100.872 117.267907 L 104.22 103.343442 L 107.568 100.829302 L 110.916 102.247535 L 114.264 103.407907 L 117.612 100.313581 L 120.96 100.313581 L 124.308 98.444093 L 127.656 97.863907 L 131.004 104.245953 L 134.352 105.34186 L 137.7 99.024279 L 141.048 30.626791 L 144.396 30.626791 L 147.744 30.626791 L 151.092 30.626791 L 154.44 30.626791 L 157.788 30.626791 L 161.136 30.626791 L 164.484 30.626791 L 167.832 30.626791 L 171.18 30.626791 L 174.528 30.626791 L 177.876 30.626791 L 181.224 30.626791 L 184.572 30.626791 L 187.92 30.626791 L 191.268 30.626791 L 194.616 30.626791 L 197.964 30.626791 L 201.312 30.626791 L 204.66 30.626791 L 208.008 30.626791 L 211.356 30.626791 L 214.704 30.626791 L 218.052 30.626791 L 221.4 30.626791 L 224.748 30.626791 L 228.096 30.626791 L 231.444 30.626791 L 234.792 30.626791 L 238.14 30.626791 L 241.488 30.626791 L 244.836 30.626791 L 248.184 30.626791 L 251.532 30.626791 L 254.88 30.626791 L 258.228 30.626791 L 261.576 30.626791 L 264.924 30.626791 L 268.272 30.626791 L 271.62 30.626791 L 274.968 30.626791 L 278.316 30.626791 L 281.664 30.626791 L 285.012 30.626791 L 288.36 30.626791 L 291.708 30.626791 L 295.056 30.626791 L 298.404 30.626791 L 301.752 30.626791 L 305.1 30.626791 L 308.448 30.626791 L 311.796 30.626791 L 315.144 30.626791 L 318.492 30.626791 L 321.84 30.626791 L 325.188 30.626791 L 328.536 30.626791 L 331.884 30.626791 L 335.232 30.626791 L 338.58 30.626791 L 341.928 30.626791 L 345.276 30.626791 L 348.624 30.626791 L 351.972 30.626791 L 355.32 30.626791 L 358.668 30.626791 L 362.016 30.626791 L 365.364 30.626791 L 368.712 30.626791 L 372.06 30.626791 L 375.408 30.626791 L 378.756 30.626791 L 382.104 30.626791 L 385.452 30.626791 " clip-path="url(#pa80ca89b9d)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_20">    <path d="M 54 127.26 L 57.348 125.261581 L 60.696 122.231721 L 64.044 96.961395 L 67.392 88.903256 L 70.74 90.514884 L 74.088 88.000744 L 77.436 95.220837 L 80.784 98.508558 L 84.132 95.285302 L 87.48 93.480279 L 90.828 94.12493 L 94.176 85.42214 L 97.524 72.529116 L 100.872 69.30586 L 104.22 74.84986 L 107.568 75.881302 L 110.916 78.395442 L 114.264 61.247721 L 117.612 59.765023 L 120.96 30.691256 L 124.308 48.805953 L 127.656 48.805953 L 131.004 30.626791 L 134.352 30.626791 L 137.7 30.626791 L 141.048 30.626791 L 144.396 30.626791 L 147.744 30.626791 L 151.092 30.626791 L 154.44 30.626791 L 157.788 30.626791 L 161.136 30.626791 L 164.484 30.626791 L 167.832 30.626791 L 171.18 30.626791 L 174.528 30.626791 L 177.876 30.626791 L 181.224 30.626791 L 184.572 30.626791 L 187.92 30.626791 L 191.268 30.626791 L 194.616 30.626791 L 197.964 30.626791 L 201.312 30.626791 L 204.66 30.626791 L 208.008 30.626791 L 211.356 30.626791 L 214.704 30.626791 L 218.052 30.626791 L 221.4 30.626791 L 224.748 30.626791 L 228.096 30.626791 L 231.444 30.626791 L 234.792 30.626791 L 238.14 30.626791 L 241.488 30.626791 L 244.836 30.626791 L 248.184 30.626791 L 251.532 30.626791 L 254.88 30.626791 L 258.228 30.626791 L 261.576 30.626791 L 264.924 30.626791 L 268.272 30.626791 L 271.62 30.626791 L 274.968 30.626791 L 278.316 30.626791 L 281.664 30.626791 L 285.012 30.626791 L 288.36 30.626791 L 291.708 30.626791 L 295.056 30.626791 L 298.404 30.626791 L 301.752 30.626791 L 305.1 30.626791 L 308.448 30.626791 L 311.796 30.626791 L 315.144 30.626791 L 318.492 30.626791 L 321.84 30.626791 L 325.188 30.626791 L 328.536 30.626791 L 331.884 30.626791 L 335.232 30.626791 L 338.58 30.626791 L 341.928 30.626791 L 345.276 30.626791 L 348.624 30.626791 L 351.972 30.626791 L 355.32 30.626791 L 358.668 30.626791 L 362.016 30.626791 L 365.364 30.626791 L 368.712 30.626791 L 372.06 30.626791 L 375.408 30.626791 L 378.756 30.626791 L 382.104 30.626791 L 385.452 30.626791 " clip-path="url(#pa80ca89b9d)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_21">    <path d="M 54 127.26 L 57.348 119.653116 L 60.696 115.720744 L 64.044 116.42986 L 67.392 111.917302 L 70.74 111.594977 L 74.088 118.68614 L 77.436 122.102791 L 80.784 113.52893 L 84.132 116.365395 L 87.48 115.140558 L 90.828 115.849674 L 94.176 117.783628 L 97.524 110.176744 L 100.872 119.910977 L 104.22 121.58707 L 107.568 126.421953 L 110.916 124.294605 L 114.264 122.167256 L 117.612 106.179907 L 120.96 110.305674 L 124.308 108.758512 L 127.656 101.151628 L 131.004 97.670512 L 134.352 95.801023 L 137.7 99.41107 L 141.048 84.777488 L 144.396 73.302698 L 147.744 81.102977 L 151.092 83.16586 L 154.44 82.327814 L 157.788 73.367163 L 161.136 68.79014 L 164.484 84.777488 L 167.832 79.233488 L 171.18 72.529116 L 174.528 50.482047 L 177.876 30.626791 L 181.224 30.626791 L 184.572 30.626791 L 187.92 30.626791 L 191.268 30.626791 L 194.616 30.626791 L 197.964 30.626791 L 201.312 30.626791 L 204.66 30.626791 L 208.008 30.626791 L 211.356 30.626791 L 214.704 30.626791 L 218.052 30.626791 L 221.4 30.626791 L 224.748 30.626791 L 228.096 30.626791 L 231.444 30.626791 L 234.792 30.626791 L 238.14 30.626791 L 241.488 30.626791 L 244.836 30.626791 L 248.184 30.626791 L 251.532 30.626791 L 254.88 30.626791 L 258.228 30.626791 L 261.576 30.626791 L 264.924 30.626791 L 268.272 30.626791 L 271.62 30.626791 L 274.968 30.626791 L 278.316 30.626791 L 281.664 30.626791 L 285.012 30.626791 L 288.36 30.626791 L 291.708 30.626791 L 295.056 30.626791 L 298.404 30.626791 L 301.752 30.626791 L 305.1 30.626791 L 308.448 30.626791 L 311.796 30.626791 L 315.144 30.626791 L 318.492 30.626791 L 321.84 30.626791 L 325.188 30.626791 L 328.536 30.626791 L 331.884 30.626791 L 335.232 30.626791 L 338.58 30.626791 L 341.928 30.626791 L 345.276 30.626791 L 348.624 30.626791 L 351.972 30.626791 L 355.32 30.626791 L 358.668 30.626791 L 362.016 30.626791 L 365.364 30.626791 L 368.712 30.626791 L 372.06 30.626791 L 375.408 30.626791 L 378.756 30.626791 L 382.104 30.626791 L 385.452 30.626791 " clip-path="url(#pa80ca89b9d)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_22">    <path d="M 54 127.26 L 57.348 123.456558 L 60.696 123.714419 L 64.044 124.488 L 67.392 125.068186 L 70.74 121.006884 L 74.088 119.653116 L 77.436 119.846512 L 80.784 121.071349 L 84.132 120.233302 L 87.48 122.618512 L 90.828 120.555628 L 94.176 112.755349 L 97.524 113.335535 L 100.872 93.093488 L 104.22 77.686326 L 107.568 84.068372 L 110.916 83.16586 L 114.264 94.25386 L 117.612 90.772744 L 120.96 97.348186 L 124.308 91.997581 L 127.656 91.417395 L 131.004 88.967721 L 134.352 93.157953 L 137.7 81.167442 L 141.048 83.16586 L 144.396 75.236651 L 147.744 78.653302 L 151.092 77.49293 L 154.44 84.326233 L 157.788 83.617116 L 161.136 80.909581 L 164.484 88.838791 L 167.832 64.793302 L 171.18 48.225767 L 174.528 62.214698 L 177.876 58.926977 L 181.224 30.626791 L 184.572 30.626791 L 187.92 30.626791 L 191.268 30.626791 L 194.616 30.626791 L 197.964 30.626791 L 201.312 30.626791 L 204.66 30.626791 L 208.008 30.626791 L 211.356 30.626791 L 214.704 30.626791 L 218.052 30.626791 L 221.4 30.626791 L 224.748 30.626791 L 228.096 30.626791 L 231.444 30.626791 L 234.792 30.626791 L 238.14 30.626791 L 241.488 30.626791 L 244.836 30.626791 L 248.184 30.626791 L 251.532 30.626791 L 254.88 30.626791 L 258.228 30.626791 L 261.576 30.626791 L 264.924 30.626791 L 268.272 30.626791 L 271.62 30.626791 L 274.968 30.626791 L 278.316 30.626791 L 281.664 30.626791 L 285.012 30.626791 L 288.36 30.626791 L 291.708 30.626791 L 295.056 30.626791 L 298.404 30.626791 L 301.752 30.626791 L 305.1 30.626791 L 308.448 30.626791 L 311.796 30.626791 L 315.144 30.626791 L 318.492 30.626791 L 321.84 30.626791 L 325.188 30.626791 L 328.536 30.626791 L 331.884 30.626791 L 335.232 30.626791 L 338.58 30.626791 L 341.928 30.626791 L 345.276 30.626791 L 348.624 30.626791 L 351.972 30.626791 L 355.32 30.626791 L 358.668 30.626791 L 362.016 30.626791 L 365.364 30.626791 L 368.712 30.626791 L 372.06 30.626791 L 375.408 30.626791 L 378.756 30.626791 L 382.104 30.626791 L 385.452 30.626791 " clip-path="url(#pa80ca89b9d)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_23">    <path d="M 54 127.26 L 57.348 124.681395 L 60.696 116.558791 L 64.044 118.041488 L 67.392 121.909395 L 70.74 111.143721 L 74.088 105.406326 L 77.436 96.445674 L 80.784 100.249116 L 84.132 106.437767 L 87.48 100.249116 L 90.828 86.324651 L 94.176 94.576186 L 97.524 105.019535 L 100.872 98.959814 L 104.22 94.962977 L 107.568 90.321488 L 110.916 91.739721 L 114.264 61.376651 L 117.612 68.016558 L 120.96 59.571628 L 124.308 68.274419 L 127.656 56.864093 L 131.004 64.019721 L 134.352 58.733581 L 137.7 58.346791 L 141.048 59.249302 L 144.396 58.024465 L 147.744 57.121953 L 151.092 30.626791 L 154.44 30.626791 L 157.788 30.626791 L 161.136 30.626791 L 164.484 30.626791 L 167.832 30.626791 L 171.18 30.626791 L 174.528 30.626791 L 177.876 30.626791 L 181.224 30.626791 L 184.572 30.626791 L 187.92 30.626791 L 191.268 30.626791 L 194.616 30.626791 L 197.964 30.626791 L 201.312 30.626791 L 204.66 30.626791 L 208.008 30.626791 L 211.356 30.626791 L 214.704 30.626791 L 218.052 30.626791 L 221.4 30.626791 L 224.748 30.626791 L 228.096 30.626791 L 231.444 30.626791 L 234.792 30.626791 L 238.14 30.626791 L 241.488 30.626791 L 244.836 30.626791 L 248.184 30.626791 L 251.532 30.626791 L 254.88 30.626791 L 258.228 30.626791 L 261.576 30.626791 L 264.924 30.626791 L 268.272 30.626791 L 271.62 30.626791 L 274.968 30.626791 L 278.316 30.626791 L 281.664 30.626791 L 285.012 30.626791 L 288.36 30.626791 L 291.708 30.626791 L 295.056 30.626791 L 298.404 30.626791 L 301.752 30.626791 L 305.1 30.626791 L 308.448 30.626791 L 311.796 30.626791 L 315.144 30.626791 L 318.492 30.626791 L 321.84 30.626791 L 325.188 30.626791 L 328.536 30.626791 L 331.884 30.626791 L 335.232 30.626791 L 338.58 30.626791 L 341.928 30.626791 L 345.276 30.626791 L 348.624 30.626791 L 351.972 30.626791 L 355.32 30.626791 L 358.668 30.626791 L 362.016 30.626791 L 365.364 30.626791 L 368.712 30.626791 L 372.06 30.626791 L 375.408 30.626791 L 378.756 30.626791 L 382.104 30.626791 L 385.452 30.626791 " clip-path="url(#pa80ca89b9d)" style="fill: none; stroke: #1f77b4; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_24">    <path d="M 54 127.26 L 57.348 123.134233 L 60.696 122.650744 L 64.044 120.942419 L 67.392 121.393674 L 70.74 121.393674 L 74.088 118.68614 L 77.436 120.168837 L 80.784 117.203442 L 84.132 118.750605 L 87.48 120.136605 L 90.828 121.006884 L 94.176 115.269488 L 97.524 113.238837 L 100.872 109.403163 L 104.22 92.900093 L 107.568 92.771163 L 110.916 93.673674 L 114.264 87.839581 L 117.612 90.128093 L 120.96 95.704326 L 124.308 89.418977 L 127.656 94.898512 L 131.004 95.446465 L 134.352 94.157163 L 137.7 89.322279 L 141.048 70.401767 L 144.396 63.665163 L 147.744 61.924605 L 151.092 30.626791 L 154.44 30.626791 L 157.788 30.626791 L 161.136 30.626791 L 164.484 30.626791 L 167.832 30.626791 L 171.18 30.626791 L 174.528 30.626791 L 177.876 30.626791 L 181.224 30.626791 L 184.572 30.626791 L 187.92 30.626791 L 191.268 30.626791 L 194.616 30.626791 L 197.964 30.626791 L 201.312 30.626791 L 204.66 30.626791 L 208.008 30.626791 L 211.356 30.626791 L 214.704 30.626791 L 218.052 30.626791 L 221.4 30.626791 L 224.748 30.626791 L 228.096 30.626791 L 231.444 30.626791 L 234.792 30.626791 L 238.14 30.626791 L 241.488 30.626791 L 244.836 30.626791 L 248.184 30.626791 L 251.532 30.626791 L 254.88 30.626791 L 258.228 30.626791 L 261.576 30.626791 L 264.924 30.626791 L 268.272 30.626791 L 271.62 30.626791 L 274.968 30.626791 L 278.316 30.626791 L 281.664 30.626791 L 285.012 30.626791 L 288.36 30.626791 L 291.708 30.626791 L 295.056 30.626791 L 298.404 30.626791 L 301.752 30.626791 L 305.1 30.626791 L 308.448 30.626791 L 311.796 30.626791 L 315.144 30.626791 L 318.492 30.626791 L 321.84 30.626791 L 325.188 30.626791 L 328.536 30.626791 L 331.884 30.626791 L 335.232 30.626791 L 338.58 30.626791 L 341.928 30.626791 L 345.276 30.626791 L 348.624 30.626791 L 351.972 30.626791 L 355.32 30.626791 L 358.668 30.626791 L 362.016 30.626791 L 365.364 30.626791 L 368.712 30.626791 L 372.06 30.626791 L 375.408 30.626791 L 378.756 30.626791 L 382.104 30.626791 L 385.452 30.626791 " clip-path="url(#pa80ca89b9d)" style="fill: none; stroke: #000000; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="patch_3">    <path d="M 54 224.28 L 54 30.24 " style="fill: none"/>   </g>   <g id="patch_4">    <path d="M 388.8 224.28 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="patch_5">    <path d="M 54 224.28 L 388.8 224.28 " style="fill: none"/>   </g>   <g id="patch_6">    <path d="M 54 30.24 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="text_16">    <g id="patch_7">     <a xlink:href="https://dynomight.net/chess">      <path d="M 323.8664 225.676606 L 388.881244 225.676606 L 388.881244 213.343012 L 323.8664 213.343012 z" style="fill: #ffffff; opacity: 0.000001; stroke: #ffffff; stroke-linejoin: miter"/>     </a>    </g>    <a xlink:href="https://dynomight.net/chess">     <!-- dynomight.net/chess -->     <g style="opacity: 0.3" transform="translate(327.8664 220.734419) scale(0.05 -0.05)">      <defs>       <path id="CourierNewPSMT-64" d="M 3206 3922 L 3206 263 L 3556 263 Q 3653 263 3693 227 Q 3734 191 3734 131 Q 3734 75 3693 37 Q 3653 0 3556 0 L 2941 0 L 2941 522 Q 2484 -103 1781 -103 Q 1425 -103 1098 86 Q 772 275 583 625 Q 394 975 394 1347 Q 394 1722 583 2070 Q 772 2419 1098 2609 Q 1425 2800 1784 2800 Q 2472 2800 2941 2175 L 2941 3656 L 2591 3656 Q 2494 3656 2453 3693 Q 2413 3731 2413 3791 Q 2413 3847 2453 3884 Q 2494 3922 2591 3922 L 3206 3922 zM 2941 1347 Q 2941 1847 2603 2190 Q 2266 2534 1800 2534 Q 1331 2534 993 2190 Q 656 1847 656 1347 Q 656 850 993 504 Q 1331 159 1800 159 Q 2266 159 2603 504 Q 2941 850 2941 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-79" d="M 1938 0 L 719 2441 L 641 2441 Q 547 2441 506 2478 Q 466 2516 466 2572 Q 466 2613 486 2645 Q 506 2678 539 2692 Q 572 2706 641 2706 L 1359 2706 Q 1453 2706 1493 2668 Q 1534 2631 1534 2572 Q 1534 2516 1493 2478 Q 1453 2441 1359 2441 L 1006 2441 L 2078 288 L 3134 2441 L 2781 2441 Q 2688 2441 2647 2478 Q 2606 2516 2606 2575 Q 2606 2631 2647 2668 Q 2688 2706 2781 2706 L 3497 2706 Q 3594 2706 3634 2668 Q 3675 2631 3675 2572 Q 3675 2531 3650 2497 Q 3625 2463 3594 2452 Q 3563 2441 3419 2441 L 1756 -944 L 2166 -944 Q 2259 -944 2300 -980 Q 2341 -1016 2341 -1075 Q 2341 -1131 2300 -1168 Q 2259 -1206 2166 -1206 L 653 -1206 Q 559 -1206 518 -1170 Q 478 -1134 478 -1075 Q 478 -1016 518 -980 Q 559 -944 653 -944 L 1475 -944 L 1938 0 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6e" d="M 1072 2706 L 1072 2309 Q 1347 2588 1569 2694 Q 1791 2800 2069 2800 Q 2369 2800 2616 2672 Q 2791 2578 2933 2361 Q 3075 2144 3075 1916 L 3075 263 L 3297 263 Q 3391 263 3431 227 Q 3472 191 3472 131 Q 3472 75 3431 37 Q 3391 0 3297 0 L 2594 0 Q 2497 0 2456 37 Q 2416 75 2416 131 Q 2416 191 2456 227 Q 2497 263 2594 263 L 2813 263 L 2813 1872 Q 2813 2150 2609 2342 Q 2406 2534 2066 2534 Q 1806 2534 1615 2429 Q 1425 2325 1072 1909 L 1072 263 L 1369 263 Q 1463 263 1503 227 Q 1544 191 1544 131 Q 1544 75 1503 37 Q 1463 0 1369 0 L 513 0 Q 419 0 378 37 Q 338 75 338 131 Q 338 191 378 227 Q 419 263 513 263 L 809 263 L 809 2441 L 588 2441 Q 494 2441 453 2478 Q 413 2516 413 2575 Q 413 2631 453 2668 Q 494 2706 588 2706 L 1072 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6f" d="M 3388 1347 Q 3388 747 2958 322 Q 2528 -103 1922 -103 Q 1309 -103 881 323 Q 453 750 453 1347 Q 453 1947 881 2373 Q 1309 2800 1922 2800 Q 2528 2800 2958 2375 Q 3388 1950 3388 1347 zM 3122 1347 Q 3122 1841 2770 2187 Q 2419 2534 1919 2534 Q 1419 2534 1067 2186 Q 716 1838 716 1347 Q 716 859 1067 509 Q 1419 159 1919 159 Q 2419 159 2770 507 Q 3122 856 3122 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6d" d="M 725 2706 L 725 2441 Q 1059 2800 1397 2800 Q 1600 2800 1753 2692 Q 1906 2584 2009 2366 Q 2184 2584 2364 2692 Q 2544 2800 2725 2800 Q 3009 2800 3178 2616 Q 3400 2378 3400 2097 L 3400 263 L 3622 263 Q 3716 263 3756 227 Q 3797 191 3797 131 Q 3797 75 3756 37 Q 3716 0 3622 0 L 3138 0 L 3138 2072 Q 3138 2272 3016 2403 Q 2894 2534 2734 2534 Q 2591 2534 2431 2426 Q 2272 2319 2069 2003 L 2069 263 L 2288 263 Q 2381 263 2422 227 Q 2463 191 2463 131 Q 2463 75 2422 37 Q 2381 0 2288 0 L 1803 0 L 1803 2053 Q 1803 2263 1679 2398 Q 1556 2534 1403 2534 Q 1263 2534 1125 2444 Q 934 2316 725 2003 L 725 263 L 947 263 Q 1041 263 1081 227 Q 1122 191 1122 131 Q 1122 75 1081 37 Q 1041 0 947 0 L 241 0 Q 147 0 106 37 Q 66 75 66 131 Q 66 191 106 227 Q 147 263 241 263 L 463 263 L 463 2441 L 241 2441 Q 147 2441 106 2478 Q 66 2516 66 2575 Q 66 2631 106 2668 Q 147 2706 241 2706 L 725 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-69" d="M 2050 4053 L 2050 3375 L 1663 3375 L 1663 4053 L 2050 4053 zM 2059 2706 L 2059 263 L 3091 263 Q 3188 263 3228 227 Q 3269 191 3269 131 Q 3269 75 3228 37 Q 3188 0 3091 0 L 766 0 Q 672 0 631 37 Q 591 75 591 131 Q 591 191 631 227 Q 672 263 766 263 L 1797 263 L 1797 2441 L 1031 2441 Q 938 2441 895 2478 Q 853 2516 853 2572 Q 853 2631 893 2668 Q 934 2706 1031 2706 L 2059 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-67" d="M 2803 2225 L 2803 2706 L 3419 2706 Q 3513 2706 3553 2668 Q 3594 2631 3594 2572 Q 3594 2516 3553 2478 Q 3513 2441 3419 2441 L 3066 2441 L 3066 -181 Q 3066 -444 2953 -650 Q 2878 -788 2703 -934 Q 2528 -1081 2384 -1143 Q 2241 -1206 2000 -1206 L 1256 -1206 Q 1163 -1206 1122 -1170 Q 1081 -1134 1081 -1075 Q 1081 -1016 1122 -978 Q 1163 -941 1256 -941 L 2009 -944 Q 2241 -944 2427 -825 Q 2613 -706 2734 -478 Q 2803 -347 2803 -144 L 2803 644 Q 2381 66 1716 66 Q 1175 66 783 467 Q 391 869 391 1434 Q 391 2000 783 2400 Q 1175 2800 1716 2800 Q 2381 2800 2803 2225 zM 2803 1434 Q 2803 1897 2486 2215 Q 2169 2534 1728 2534 Q 1288 2534 972 2214 Q 656 1894 656 1434 Q 656 972 972 651 Q 1288 331 1728 331 Q 2169 331 2486 651 Q 2803 972 2803 1434 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-68" d="M 1056 3922 L 1056 2306 Q 1306 2578 1536 2689 Q 1766 2800 2050 2800 Q 2356 2800 2570 2692 Q 2784 2584 2928 2361 Q 3072 2138 3072 1891 L 3072 263 L 3366 263 Q 3463 263 3502 227 Q 3541 191 3541 131 Q 3541 75 3502 37 Q 3463 0 3366 0 L 2513 0 Q 2416 0 2375 37 Q 2334 75 2334 131 Q 2334 191 2375 227 Q 2416 263 2513 263 L 2806 263 L 2806 1872 Q 2806 2156 2601 2347 Q 2397 2538 2031 2538 Q 1744 2538 1541 2397 Q 1394 2297 1056 1925 L 1056 263 L 1353 263 Q 1447 263 1487 227 Q 1528 191 1528 131 Q 1528 75 1487 37 Q 1447 0 1353 0 L 497 0 Q 403 0 362 37 Q 322 75 322 131 Q 322 191 362 227 Q 403 263 497 263 L 794 263 L 794 3656 L 441 3656 Q 347 3656 306 3693 Q 266 3731 266 3791 Q 266 3847 306 3884 Q 347 3922 441 3922 L 1056 3922 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-74" d="M 1381 2706 L 2806 2706 Q 2900 2706 2940 2668 Q 2981 2631 2981 2572 Q 2981 2516 2940 2478 Q 2900 2441 2806 2441 L 1381 2441 L 1381 694 Q 1381 466 1564 312 Q 1747 159 2100 159 Q 2366 159 2675 239 Q 2984 319 3156 419 Q 3219 459 3259 459 Q 3309 459 3346 420 Q 3384 381 3384 328 Q 3384 281 3344 241 Q 3244 138 2858 17 Q 2472 -103 2119 -103 Q 1659 -103 1387 112 Q 1116 328 1116 694 L 1116 2441 L 631 2441 Q 538 2441 497 2478 Q 456 2516 456 2575 Q 456 2631 497 2668 Q 538 2706 631 2706 L 1116 2706 L 1116 3481 Q 1116 3575 1153 3615 Q 1191 3656 1247 3656 Q 1306 3656 1343 3615 Q 1381 3575 1381 3481 L 1381 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2e" d="M 1866 753 L 1978 753 Q 2153 753 2278 629 Q 2403 506 2403 331 Q 2403 150 2276 28 Q 2150 -94 1978 -94 L 1866 -94 Q 1691 -94 1566 29 Q 1441 153 1441 328 Q 1441 509 1567 631 Q 1694 753 1866 753 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-65" d="M 3341 1291 L 666 1291 Q 734 781 1092 470 Q 1450 159 1978 159 Q 2272 159 2594 256 Q 2916 353 3119 513 Q 3178 559 3222 559 Q 3272 559 3309 520 Q 3347 481 3347 428 Q 3347 375 3297 325 Q 3147 169 2764 33 Q 2381 -103 1978 -103 Q 1303 -103 851 339 Q 400 781 400 1409 Q 400 1981 823 2390 Q 1247 2800 1872 2800 Q 2516 2800 2931 2379 Q 3347 1959 3341 1291 zM 3075 1556 Q 2997 1991 2664 2262 Q 2331 2534 1872 2534 Q 1413 2534 1081 2265 Q 750 1997 669 1556 L 3075 1556 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2f" d="M 3100 4113 L 988 -413 Q 934 -528 850 -528 Q 797 -528 758 -489 Q 719 -450 719 -403 Q 719 -366 750 -300 L 2859 4225 Q 2894 4297 2925 4319 Q 2956 4341 3000 4341 Q 3053 4341 3092 4302 Q 3131 4263 3131 4216 Q 3131 4181 3100 4113 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-63" d="M 3016 2441 L 3016 2528 Q 3016 2625 3053 2665 Q 3091 2706 3147 2706 Q 3206 2706 3243 2665 Q 3281 2625 3281 2528 L 3281 1934 Q 3278 1838 3242 1797 Q 3206 1756 3147 1756 Q 3094 1756 3058 1792 Q 3022 1828 3016 1913 Q 2997 2134 2723 2334 Q 2450 2534 1988 2534 Q 1403 2534 1100 2168 Q 797 1803 797 1331 Q 797 822 1131 490 Q 1466 159 1997 159 Q 2303 159 2620 271 Q 2938 384 3194 634 Q 3259 697 3309 697 Q 3363 697 3398 661 Q 3434 625 3434 572 Q 3434 438 3119 231 Q 2609 -103 1984 -103 Q 1350 -103 942 301 Q 534 706 534 1328 Q 534 1963 951 2381 Q 1369 2800 2003 2800 Q 2606 2800 3016 2441 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-73" d="M 2788 2534 Q 2788 2625 2825 2665 Q 2863 2706 2919 2706 Q 2978 2706 3015 2665 Q 3053 2625 3053 2528 L 3053 2078 Q 3053 1984 3015 1943 Q 2978 1903 2919 1903 Q 2866 1903 2830 1937 Q 2794 1972 2788 2050 Q 2769 2238 2594 2359 Q 2338 2534 1916 2534 Q 1475 2534 1231 2356 Q 1047 2222 1047 2056 Q 1047 1869 1266 1744 Q 1416 1656 1834 1609 Q 2381 1550 2594 1475 Q 2897 1366 3045 1172 Q 3194 978 3194 753 Q 3194 419 2872 158 Q 2550 -103 1928 -103 Q 1306 -103 909 213 Q 909 106 896 75 Q 884 44 851 22 Q 819 0 778 0 Q 722 0 684 40 Q 647 81 647 175 L 647 716 Q 647 809 683 850 Q 719 891 778 891 Q 834 891 873 852 Q 913 813 913 747 Q 913 603 984 506 Q 1094 356 1333 257 Q 1572 159 1919 159 Q 2431 159 2681 350 Q 2931 541 2931 753 Q 2931 997 2678 1144 Q 2422 1291 1933 1341 Q 1444 1391 1231 1472 Q 1019 1553 900 1715 Q 781 1878 781 2066 Q 781 2403 1112 2601 Q 1444 2800 1903 2800 Q 2447 2800 2788 2534 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#CourierNewPSMT-64"/>      <use xlink:href="#CourierNewPSMT-79" x="60.009766"/>      <use xlink:href="#CourierNewPSMT-6e" x="120.019531"/>      <use xlink:href="#CourierNewPSMT-6f" x="180.029297"/>      <use xlink:href="#CourierNewPSMT-6d" x="240.039062"/>      <use xlink:href="#CourierNewPSMT-69" x="300.048828"/>      <use xlink:href="#CourierNewPSMT-67" x="360.058594"/>      <use xlink:href="#CourierNewPSMT-68" x="420.068359"/>      <use xlink:href="#CourierNewPSMT-74" x="480.078125"/>      <use xlink:href="#CourierNewPSMT-2e" x="540.087891"/>      <use xlink:href="#CourierNewPSMT-6e" x="600.097656"/>      <use xlink:href="#CourierNewPSMT-65" x="660.107422"/>      <use xlink:href="#CourierNewPSMT-74" x="720.117188"/>      <use xlink:href="#CourierNewPSMT-2f" x="780.126953"/>      <use xlink:href="#CourierNewPSMT-63" x="840.136719"/>      <use xlink:href="#CourierNewPSMT-68" x="900.146484"/>      <use xlink:href="#CourierNewPSMT-65" x="960.15625"/>      <use xlink:href="#CourierNewPSMT-73" x="1020.166016"/>      <use xlink:href="#CourierNewPSMT-73" x="1080.175781"/>     </g>    </a>   </g>   <g id="text_17">    <!-- gpt-3.5-turbo-instruct vs stockfish -->    <g transform="translate(119.747813 24.24) scale(0.12 -0.12)">     <defs>      <path id="Montserrat-Regular-67" d="M 3750 3366 L 3750 410 Q 3750 -448 3331 -861 Q 2912 -1274 2067 -1274 Q 1600 -1274 1181 -1136 Q 762 -998 499 -755 L 730 -410 Q 973 -627 1321 -748 Q 1670 -870 2054 -870 Q 2694 -870 2995 -572 Q 3296 -275 3296 352 L 3296 781 Q 3085 461 2742 294 Q 2400 128 1984 128 Q 1510 128 1123 336 Q 736 544 515 918 Q 294 1293 294 1766 Q 294 2240 515 2611 Q 736 2982 1120 3187 Q 1504 3392 1984 3392 Q 2413 3392 2758 3219 Q 3104 3046 3315 2720 L 3315 3366 L 3750 3366 zM 2029 531 Q 2394 531 2688 688 Q 2982 845 3145 1126 Q 3309 1408 3309 1766 Q 3309 2125 3145 2403 Q 2982 2682 2691 2838 Q 2400 2995 2029 2995 Q 1664 2995 1373 2841 Q 1082 2688 918 2406 Q 755 2125 755 1766 Q 755 1408 918 1126 Q 1082 845 1373 688 Q 1664 531 2029 531 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-33" d="M 1856 2560 Q 2560 2522 2928 2179 Q 3296 1837 3296 1286 Q 3296 902 3110 604 Q 2925 307 2560 134 Q 2195 -38 1670 -38 Q 1184 -38 752 118 Q 320 275 58 538 L 282 902 Q 506 672 877 528 Q 1248 384 1670 384 Q 2221 384 2521 624 Q 2822 864 2822 1286 Q 2822 1702 2521 1939 Q 2221 2176 1613 2176 L 1293 2176 L 1293 2515 L 2534 4070 L 243 4070 L 243 4480 L 3123 4480 L 3123 4154 L 1856 2560 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-2e" d="M 678 -32 Q 538 -32 438 70 Q 339 173 339 320 Q 339 467 438 566 Q 538 666 678 666 Q 819 666 921 566 Q 1024 467 1024 320 Q 1024 173 921 70 Q 819 -32 678 -32 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-62" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1184 3827 793 Q 3610 403 3226 185 Q 2842 -32 2368 -32 Q 1946 -32 1610 147 Q 1274 326 1069 666 L 1069 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2726 Q 1299 3053 1628 3222 Q 1958 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2368 Q 3264 2669 2976 2832 Q 2688 2995 2336 2995 Q 1978 2995 1693 2832 Q 1408 2669 1245 2368 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6f" d="M 2010 -32 Q 1523 -32 1132 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1132 3174 Q 1523 3392 2010 3392 Q 2496 3392 2886 3174 Q 3277 2957 3497 2566 Q 3718 2176 3718 1683 Q 3718 1190 3497 800 Q 3277 410 2886 189 Q 2496 -32 2010 -32 zM 2010 371 Q 2368 371 2653 534 Q 2938 698 3098 998 Q 3258 1299 3258 1683 Q 3258 2067 3098 2368 Q 2938 2669 2653 2832 Q 2368 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-76" d="M 3462 3366 L 1965 0 L 1498 0 L 6 3366 L 480 3366 L 1741 499 L 3014 3366 L 3462 3366 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6b" d="M 1882 1690 L 1088 960 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 1530 L 3098 3366 L 3661 3366 L 2221 1990 L 3795 0 L 3238 0 L 1882 1690 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-68" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2778 Q 1274 3072 1600 3232 Q 1926 3392 2349 3392 z" transform="scale(0.015625)"/>     </defs>     <use xlink:href="#Montserrat-Regular-67"/>     <use xlink:href="#Montserrat-Regular-70" x="68.599991"/>     <use xlink:href="#Montserrat-Regular-74" x="136.399979"/>     <use xlink:href="#Montserrat-Regular-2d" x="176.999969"/>     <use xlink:href="#Montserrat-Regular-33" x="215.199966"/>     <use xlink:href="#Montserrat-Regular-2e" x="271.59996"/>     <use xlink:href="#Montserrat-Regular-35" x="292.799957"/>     <use xlink:href="#Montserrat-Regular-2d" x="349.399948"/>     <use xlink:href="#Montserrat-Regular-74" x="387.599945"/>     <use xlink:href="#Montserrat-Regular-75" x="428.199936"/>     <use xlink:href="#Montserrat-Regular-72" x="495.499924"/>     <use xlink:href="#Montserrat-Regular-62" x="535.599915"/>     <use xlink:href="#Montserrat-Regular-6f" x="603.399902"/>     <use xlink:href="#Montserrat-Regular-2d" x="666.099899"/>     <use xlink:href="#Montserrat-Regular-69" x="704.299896"/>     <use xlink:href="#Montserrat-Regular-6e" x="731.19989"/>     <use xlink:href="#Montserrat-Regular-73" x="798.899872"/>     <use xlink:href="#Montserrat-Regular-74" x="847.799866"/>     <use xlink:href="#Montserrat-Regular-72" x="888.399857"/>     <use xlink:href="#Montserrat-Regular-75" x="928.499847"/>     <use xlink:href="#Montserrat-Regular-63" x="995.799835"/>     <use xlink:href="#Montserrat-Regular-74" x="1052.099823"/>     <use xlink:href="#Montserrat-Regular-20" x="1092.699814"/>     <use xlink:href="#Montserrat-Regular-76" x="1118.899811"/>     <use xlink:href="#Montserrat-Regular-73" x="1173.099808"/>     <use xlink:href="#Montserrat-Regular-20" x="1221.999802"/>     <use xlink:href="#Montserrat-Regular-73" x="1248.199799"/>     <use xlink:href="#Montserrat-Regular-74" x="1297.099792"/>     <use xlink:href="#Montserrat-Regular-6f" x="1337.699783"/>     <use xlink:href="#Montserrat-Regular-63" x="1400.39978"/>     <use xlink:href="#Montserrat-Regular-6b" x="1456.699768"/>     <use xlink:href="#Montserrat-Regular-66" x="1516.799759"/>     <use xlink:href="#Montserrat-Regular-69" x="1550.699753"/>     <use xlink:href="#Montserrat-Regular-73" x="1577.599747"/>     <use xlink:href="#Montserrat-Regular-68" x="1626.499741"/>    </g>   </g>  </g> </g> <defs>  <clipPath id="pa80ca89b9d">   <rect x="54" y="30.24" width="334.8" height="194.04"/>  </clipPath> </defs></svg>](https://dynomight.net/img/chess/gpt-3.5-turbo-instruct_stockfish.pdf)

[Excellent](https://lichess.org/4CCCdinZ). Very, _very_, good.

Even if you raise Stockfish’s level a few clicks, this model will still win every game.

Moving on… I next tried `gpt-3.5-turbo`, a model that’s similar, except tuned to be more chatty and conversational.

[<?xml version="1.0" encoding="utf-8" standalone="no"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"  "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg xmlns:xlink="http://www.w3.org/1999/xlink" width="432pt" height="252pt" viewBox="0 0 432 252" xmlns="http://www.w3.org/2000/svg" version="1.1"> <metadata>  <rdf:RDF xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:cc="http://creativecommons.org/ns#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">   <cc:Work>    <dc:type rdf:resource="http://purl.org/dc/dcmitype/StillImage"/>    <dc:date>2024-11-14T11:33:57.830153</dc:date>    <dc:format>image/svg+xml</dc:format>    <dc:creator>     <cc:Agent>      <dc:title>Matplotlib v3.8.4, https://matplotlib.org/</dc:title>     </cc:Agent>    </dc:creator>   </cc:Work>  </rdf:RDF> </metadata> <defs>  <style type="text/css">*{stroke-linejoin: round; stroke-linecap: butt}</style> </defs> <g id="figure_1">  <g id="patch_1">   <path d="M 0 252 L 432 252 L 432 0 L 0 0 z" style="fill: #ffffff"/>  </g>  <g id="axes_1">   <g id="patch_2">    <path d="M 54 224.28 L 388.8 224.28 L 388.8 30.24 L 54 30.24 z" style="fill: #ffffff"/>   </g>   <g id="matplotlib.axis_1">    <g id="xtick_1">     <g id="line2d_1"/>     <g id="text_1">      <!-- 0 -->      <g transform="translate(50.689844 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-30" d="M 2118 -38 Q 1600 -38 1193 237 Q 787 512 560 1027 Q 333 1542 333 2240 Q 333 2938 560 3453 Q 787 3968 1193 4243 Q 1600 4518 2118 4518 Q 2637 4518 3043 4243 Q 3450 3968 3680 3453 Q 3910 2938 3910 2240 Q 3910 1542 3680 1027 Q 3450 512 3043 237 Q 2637 -38 2118 -38 zM 2118 384 Q 2509 384 2806 601 Q 3104 819 3270 1235 Q 3437 1651 3437 2240 Q 3437 2829 3270 3245 Q 3104 3661 2806 3878 Q 2509 4096 2118 4096 Q 1728 4096 1430 3878 Q 1133 3661 966 3245 Q 800 2829 800 2240 Q 800 1651 966 1235 Q 1133 819 1430 601 Q 1728 384 2118 384 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="xtick_2">     <g id="line2d_2"/>     <g id="text_2">      <!-- 20 -->      <g transform="translate(114.81 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-32" d="M 3392 410 L 3392 0 L 250 0 L 250 326 L 2112 2150 Q 2464 2496 2589 2749 Q 2714 3002 2714 3258 Q 2714 3654 2442 3875 Q 2170 4096 1664 4096 Q 877 4096 442 3597 L 115 3878 Q 378 4186 784 4352 Q 1190 4518 1702 4518 Q 2387 4518 2787 4195 Q 3187 3872 3187 3309 Q 3187 2963 3033 2643 Q 2880 2323 2451 1907 L 928 410 L 3392 410 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-32"/>       <use xlink:href="#Montserrat-Regular-30" x="56.799988"/>      </g>     </g>    </g>    <g id="xtick_3">     <g id="line2d_3"/>     <g id="text_3">      <!-- 40 -->      <g transform="translate(181.305156 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-34" d="M 4141 1178 L 3245 1178 L 3245 0 L 2784 0 L 2784 1178 L 256 1178 L 256 1510 L 2630 4480 L 3142 4480 L 851 1587 L 2797 1587 L 2797 2624 L 3245 2624 L 3245 1587 L 4141 1587 L 4141 1178 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-34"/>       <use xlink:href="#Montserrat-Regular-30" x="66.099991"/>      </g>     </g>    </g>    <g id="xtick_4">     <g id="line2d_4"/>     <g id="text_4">      <!-- 60 -->      <g transform="translate(248.524531 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-36" d="M 2189 2682 Q 2630 2682 2972 2515 Q 3315 2349 3507 2045 Q 3699 1741 3699 1338 Q 3699 928 3500 614 Q 3302 301 2953 131 Q 2605 -38 2170 -38 Q 1280 -38 806 547 Q 333 1133 333 2202 Q 333 2950 589 3468 Q 845 3987 1305 4252 Q 1766 4518 2381 4518 Q 3046 4518 3443 4275 L 3258 3904 Q 2938 4115 2387 4115 Q 1645 4115 1222 3644 Q 800 3174 800 2278 Q 800 2118 819 1914 Q 986 2278 1350 2480 Q 1715 2682 2189 2682 zM 2150 352 Q 2637 352 2941 617 Q 3245 883 3245 1325 Q 3245 1766 2941 2028 Q 2637 2291 2118 2291 Q 1779 2291 1520 2163 Q 1261 2035 1113 1811 Q 966 1587 966 1312 Q 966 1056 1104 838 Q 1242 621 1510 486 Q 1779 352 2150 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-36"/>       <use xlink:href="#Montserrat-Regular-30" x="60.899994"/>      </g>     </g>    </g>    <g id="xtick_5">     <g id="line2d_5"/>     <g id="text_5">      <!-- 80 -->      <g transform="translate(315.34 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-38" d="M 2899 2349 Q 3322 2208 3549 1926 Q 3776 1645 3776 1248 Q 3776 858 3565 566 Q 3354 275 2960 118 Q 2566 -38 2035 -38 Q 1229 -38 768 307 Q 307 653 307 1248 Q 307 1651 528 1929 Q 749 2208 1171 2349 Q 826 2483 646 2736 Q 467 2989 467 3334 Q 467 3878 892 4198 Q 1318 4518 2035 4518 Q 2509 4518 2864 4374 Q 3219 4230 3417 3961 Q 3616 3693 3616 3334 Q 3616 2989 3430 2736 Q 3245 2483 2899 2349 zM 928 3322 Q 928 2950 1222 2732 Q 1517 2515 2035 2515 Q 2554 2515 2851 2732 Q 3149 2950 3149 3315 Q 3149 3693 2845 3910 Q 2541 4128 2035 4128 Q 1523 4128 1225 3910 Q 928 3693 928 3322 zM 2035 352 Q 2630 352 2966 595 Q 3302 838 3302 1254 Q 3302 1670 2966 1910 Q 2630 2150 2035 2150 Q 1446 2150 1110 1910 Q 774 1670 774 1254 Q 774 832 1107 592 Q 1440 352 2035 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-38"/>       <use xlink:href="#Montserrat-Regular-30" x="63.799988"/>      </g>     </g>    </g>    <g id="xtick_6">     <g id="line2d_6"/>     <g id="text_6">      <!-- 100 -->      <g transform="translate(380.375 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-31" d="M 1574 4480 L 1574 0 L 1114 0 L 1114 4070 L 58 4070 L 58 4480 L 1574 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>      </g>     </g>    </g>    <g id="text_7">     <!-- turns -->     <g transform="translate(208.170312 248.56125) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-74" d="M 2406 205 Q 2278 90 2089 29 Q 1901 -32 1696 -32 Q 1222 -32 966 224 Q 710 480 710 947 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 4102 L 1165 4102 L 1165 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 973 Q 1165 672 1315 515 Q 1466 358 1747 358 Q 1888 358 2019 403 Q 2150 448 2246 531 L 2406 205 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-75" d="M 3667 3366 L 3667 0 L 3232 0 L 3232 614 Q 3053 307 2739 137 Q 2426 -32 2022 -32 Q 1363 -32 982 336 Q 602 704 602 1414 L 602 3366 L 1056 3366 L 1056 1459 Q 1056 928 1318 653 Q 1581 378 2067 378 Q 2598 378 2905 701 Q 3213 1024 3213 1600 L 3213 3366 L 3667 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-72" d="M 1069 2707 Q 1229 3046 1545 3219 Q 1862 3392 2330 3392 L 2330 2950 L 2221 2957 Q 1690 2957 1389 2630 Q 1088 2304 1088 1715 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2707 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-6e" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2746 Q 1254 3053 1584 3222 Q 1914 3392 2349 3392 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-73" d="M 1536 -32 Q 1126 -32 752 86 Q 378 205 166 384 L 371 742 Q 582 576 902 470 Q 1222 365 1568 365 Q 2029 365 2249 509 Q 2470 653 2470 915 Q 2470 1101 2348 1206 Q 2227 1312 2041 1366 Q 1856 1421 1549 1472 Q 1139 1549 889 1629 Q 640 1709 464 1901 Q 288 2093 288 2432 Q 288 2854 640 3123 Q 992 3392 1619 3392 Q 1946 3392 2272 3305 Q 2598 3219 2810 3078 L 2611 2714 Q 2195 3002 1619 3002 Q 1184 3002 963 2848 Q 742 2694 742 2445 Q 742 2253 867 2137 Q 992 2022 1177 1968 Q 1363 1914 1690 1856 Q 2093 1779 2336 1702 Q 2579 1626 2752 1440 Q 2925 1254 2925 928 Q 2925 486 2557 227 Q 2189 -32 1536 -32 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-74"/>      <use xlink:href="#Montserrat-Regular-75" x="40.599991"/>      <use xlink:href="#Montserrat-Regular-72" x="107.899979"/>      <use xlink:href="#Montserrat-Regular-6e" x="147.999969"/>      <use xlink:href="#Montserrat-Regular-73" x="215.699951"/>     </g>    </g>   </g>   <g id="matplotlib.axis_2">    <g id="ytick_1">     <g id="line2d_7"/>     <g id="text_8">      <!-- LOSS -->      <g transform="translate(23.91875 227.667831) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-4c" d="M 730 4480 L 1203 4480 L 1203 410 L 3712 410 L 3712 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4f" d="M 2688 -38 Q 2022 -38 1481 259 Q 941 557 637 1075 Q 333 1594 333 2240 Q 333 2886 637 3404 Q 941 3923 1481 4220 Q 2022 4518 2688 4518 Q 3354 4518 3888 4224 Q 4422 3930 4729 3408 Q 5037 2886 5037 2240 Q 5037 1594 4729 1072 Q 4422 550 3888 256 Q 3354 -38 2688 -38 zM 2688 384 Q 3219 384 3648 624 Q 4077 864 4320 1289 Q 4563 1715 4563 2240 Q 4563 2765 4320 3190 Q 4077 3616 3648 3856 Q 3219 4096 2688 4096 Q 2157 4096 1725 3856 Q 1293 3616 1046 3190 Q 800 2765 800 2240 Q 800 1715 1046 1289 Q 1293 864 1725 624 Q 2157 384 2688 384 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-53" d="M 1965 -38 Q 1466 -38 1008 122 Q 550 282 301 544 L 486 909 Q 730 666 1133 515 Q 1536 365 1965 365 Q 2566 365 2867 585 Q 3168 806 3168 1158 Q 3168 1427 3005 1587 Q 2842 1747 2605 1833 Q 2368 1920 1946 2022 Q 1440 2150 1139 2268 Q 838 2387 624 2630 Q 410 2874 410 3290 Q 410 3629 589 3907 Q 768 4186 1139 4352 Q 1510 4518 2061 4518 Q 2445 4518 2813 4412 Q 3181 4307 3450 4122 L 3290 3744 Q 3008 3930 2688 4022 Q 2368 4115 2061 4115 Q 1472 4115 1174 3888 Q 877 3661 877 3302 Q 877 3034 1040 2870 Q 1203 2707 1449 2620 Q 1696 2534 2112 2432 Q 2605 2310 2909 2192 Q 3213 2074 3424 1834 Q 3635 1594 3635 1184 Q 3635 845 3452 566 Q 3270 288 2892 125 Q 2515 -38 1965 -38 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-4c"/>       <use xlink:href="#Montserrat-Regular-4f" x="58.899994"/>       <use xlink:href="#Montserrat-Regular-53" x="142.799988"/>       <use xlink:href="#Montserrat-Regular-53" x="204.299973"/>      </g>     </g>    </g>    <g id="ytick_2">     <g id="line2d_8"/>     <g id="text_9">      <!-- -1000 -->      <g transform="translate(23.209375 195.435273) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-2d" d="M 384 1933 L 2061 1933 L 2061 1536 L 384 1536 L 384 1933 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-31" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="74.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="140.499985"/>       <use xlink:href="#Montserrat-Regular-30" x="206.699982"/>      </g>     </g>    </g>    <g id="ytick_3">     <g id="line2d_9"/>     <g id="text_10">      <!-- -500 -->      <g transform="translate(27.779687 163.202714) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-35" d="M 1510 2643 Q 2483 2643 2928 2294 Q 3373 1946 3373 1318 Q 3373 928 3190 621 Q 3008 314 2643 138 Q 2278 -38 1747 -38 Q 1267 -38 835 118 Q 403 275 141 538 L 365 902 Q 589 672 953 528 Q 1318 384 1741 384 Q 2298 384 2602 633 Q 2906 883 2906 1299 Q 2906 1760 2563 1997 Q 2221 2234 1408 2234 L 454 2234 L 685 4480 L 3136 4480 L 3136 4070 L 1088 4070 L 934 2643 L 1510 2643 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-35" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="94.799988"/>       <use xlink:href="#Montserrat-Regular-30" x="160.999985"/>      </g>     </g>    </g>    <g id="ytick_4">     <g id="line2d_10"/>     <g id="text_11">      <!-- 0 -->      <g transform="translate(43.879688 130.970156) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="ytick_5">     <g id="line2d_11"/>     <g id="text_12">      <!-- 500 -->      <g transform="translate(31.6 98.737598) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-35"/>       <use xlink:href="#Montserrat-Regular-30" x="56.599991"/>       <use xlink:href="#Montserrat-Regular-30" x="122.799988"/>      </g>     </g>    </g>    <g id="ytick_6">     <g id="line2d_12"/>     <g id="text_13">      <!-- 1000 -->      <g transform="translate(27.029687 66.50504) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="168.499985"/>      </g>     </g>    </g>    <g id="ytick_7">     <g id="line2d_13"/>     <g id="text_14">      <!-- WIN -->      <g transform="translate(28.240625 34.272482) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-57" d="M 6880 4480 L 5370 0 L 4870 0 L 3558 3821 L 2240 0 L 1747 0 L 237 4480 L 723 4480 L 2016 627 L 3354 4480 L 3795 4480 L 5114 608 L 6426 4480 L 6880 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-49" d="M 730 4480 L 1203 4480 L 1203 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4e" d="M 4474 4480 L 4474 0 L 4083 0 L 1203 3635 L 1203 0 L 730 0 L 730 4480 L 1120 4480 L 4006 845 L 4006 4480 L 4474 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-57"/>       <use xlink:href="#Montserrat-Regular-49" x="111.099991"/>       <use xlink:href="#Montserrat-Regular-4e" x="141.299988"/>      </g>     </g>    </g>    <g id="text_15">     <!-- centipawn difference -->     <g transform="translate(17.26875 180.677188) rotate(-90) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-63" d="M 2029 -32 Q 1530 -32 1136 185 Q 742 403 518 796 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1136 3174 Q 1530 3392 2029 3392 Q 2464 3392 2806 3222 Q 3149 3053 3347 2726 L 3008 2496 Q 2842 2746 2586 2870 Q 2330 2995 2029 2995 Q 1664 2995 1373 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1293 918 995 Q 1082 698 1373 534 Q 1664 371 2029 371 Q 2330 371 2586 492 Q 2842 614 3008 864 L 3347 634 Q 3149 307 2803 137 Q 2458 -32 2029 -32 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-65" d="M 3565 1542 L 749 1542 Q 787 1018 1152 694 Q 1517 371 2074 371 Q 2387 371 2649 483 Q 2912 595 3104 813 L 3360 518 Q 3136 250 2800 109 Q 2464 -32 2061 -32 Q 1542 -32 1142 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 508 2566 Q 723 2957 1097 3174 Q 1472 3392 1939 3392 Q 2406 3392 2777 3174 Q 3149 2957 3360 2569 Q 3571 2182 3571 1683 L 3565 1542 zM 1939 3002 Q 1453 3002 1123 2691 Q 794 2381 749 1882 L 3136 1882 Q 3091 2381 2761 2691 Q 2432 3002 1939 3002 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-69" d="M 634 3366 L 1088 3366 L 1088 0 L 634 0 L 634 3366 zM 864 4102 Q 723 4102 627 4198 Q 531 4294 531 4429 Q 531 4557 627 4653 Q 723 4749 864 4749 Q 1005 4749 1101 4656 Q 1197 4563 1197 4435 Q 1197 4294 1101 4198 Q 1005 4102 864 4102 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-70" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1178 3827 787 Q 3610 397 3229 182 Q 2848 -32 2368 -32 Q 1958 -32 1628 137 Q 1299 307 1088 634 L 1088 -1242 L 634 -1242 L 634 3366 L 1069 3366 L 1069 2701 Q 1274 3034 1610 3213 Q 1946 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2364 Q 3264 2662 2976 2828 Q 2688 2995 2336 2995 Q 1978 2995 1693 2828 Q 1408 2662 1245 2364 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-61" d="M 1811 3392 Q 2470 3392 2822 3062 Q 3174 2733 3174 2086 L 3174 0 L 2739 0 L 2739 525 Q 2586 262 2288 115 Q 1990 -32 1581 -32 Q 1018 -32 685 237 Q 352 506 352 947 Q 352 1376 662 1638 Q 973 1901 1651 1901 L 2720 1901 L 2720 2106 Q 2720 2541 2477 2768 Q 2234 2995 1766 2995 Q 1446 2995 1152 2889 Q 858 2784 646 2598 L 442 2938 Q 698 3155 1056 3273 Q 1414 3392 1811 3392 zM 1651 326 Q 2035 326 2310 502 Q 2586 678 2720 1011 L 2720 1562 L 1664 1562 Q 800 1562 800 960 Q 800 666 1024 496 Q 1248 326 1651 326 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-77" d="M 5574 3366 L 4301 0 L 3872 0 L 2810 2758 L 1747 0 L 1318 0 L 51 3366 L 486 3366 L 1542 506 L 2624 3366 L 3014 3366 L 4090 506 L 5158 3366 L 5574 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-20" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-64" d="M 3706 4749 L 3706 0 L 3270 0 L 3270 666 Q 3066 326 2730 147 Q 2394 -32 1971 -32 Q 1498 -32 1114 185 Q 730 403 512 793 Q 294 1184 294 1683 Q 294 2182 512 2572 Q 730 2963 1114 3177 Q 1498 3392 1971 3392 Q 2381 3392 2710 3222 Q 3040 3053 3251 2726 L 3251 4749 L 3706 4749 zM 2010 371 Q 2362 371 2646 534 Q 2931 698 3094 998 Q 3258 1299 3258 1683 Q 3258 2067 3094 2368 Q 2931 2669 2646 2832 Q 2362 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-66" d="M 1747 4397 Q 1453 4397 1302 4237 Q 1152 4077 1152 3763 L 1152 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 0 L 710 0 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 3782 Q 710 4243 976 4512 Q 1242 4781 1728 4781 Q 1920 4781 2099 4726 Q 2278 4672 2400 4570 L 2246 4230 Q 2048 4397 1747 4397 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-63"/>      <use xlink:href="#Montserrat-Regular-65" x="56.299988"/>      <use xlink:href="#Montserrat-Regular-6e" x="116.699982"/>      <use xlink:href="#Montserrat-Regular-74" x="184.399963"/>      <use xlink:href="#Montserrat-Regular-69" x="224.999954"/>      <use xlink:href="#Montserrat-Regular-70" x="251.899948"/>      <use xlink:href="#Montserrat-Regular-61" x="319.699936"/>      <use xlink:href="#Montserrat-Regular-77" x="378.699921"/>      <use xlink:href="#Montserrat-Regular-6e" x="466.599915"/>      <use xlink:href="#Montserrat-Regular-20" x="534.299896"/>      <use xlink:href="#Montserrat-Regular-64" x="560.499893"/>      <use xlink:href="#Montserrat-Regular-69" x="628.299881"/>      <use xlink:href="#Montserrat-Regular-66" x="655.199875"/>      <use xlink:href="#Montserrat-Regular-66" x="689.099869"/>      <use xlink:href="#Montserrat-Regular-65" x="722.999863"/>      <use xlink:href="#Montserrat-Regular-72" x="783.399857"/>      <use xlink:href="#Montserrat-Regular-65" x="823.499847"/>      <use xlink:href="#Montserrat-Regular-6e" x="883.899841"/>      <use xlink:href="#Montserrat-Regular-63" x="951.599823"/>      <use xlink:href="#Montserrat-Regular-65" x="1007.899811"/>     </g>    </g>   </g>   <g id="line2d_14">    <path d="M 54 127.26 L 57.348 125.777302 L 60.696 125.583907 L 64.044 123.327628 L 67.392 124.488 L 70.74 126.679814 L 74.088 128.033581 L 77.436 128.355907 L 80.784 128.420372 L 84.132 137.896744 L 87.48 134.093302 L 90.828 150.725302 L 94.176 158.203256 L 97.524 153.561767 L 100.872 143.440744 L 104.22 171.676465 L 107.568 176.253488 L 110.916 187.148093 L 114.264 189.339907 L 117.612 188.630791 L 120.96 195.850884 L 124.308 193.65907 L 127.656 199.718791 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p4419c81266)" style="fill: none; stroke-dasharray: 0.5,0.825; stroke-dashoffset: 0; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5"/>   </g>   <g id="line2d_15">    <path d="M 54 127.26 L 57.348 120.813488 L 60.696 120.168837 L 64.044 117.138977 L 67.392 133.513116 L 70.74 139.895163 L 74.088 148.726884 L 77.436 180.05693 L 80.784 183.151256 L 84.132 178.380837 L 87.48 188.953116 L 90.828 223.957674 L 94.176 223.957674 L 97.524 223.957674 L 100.872 223.957674 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p4419c81266)" style="fill: none; stroke-dasharray: 0.5,0.825; stroke-dashoffset: 0; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5"/>   </g>   <g id="line2d_16">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 123.649953 L 64.044 124.35907 L 67.392 129.451814 L 70.74 140.281953 L 74.088 164.005116 L 77.436 164.327442 L 80.784 167.099442 L 84.132 166.777116 L 87.48 163.682791 L 90.828 161.942233 L 94.176 152.46586 L 97.524 179.283349 L 100.872 177.28493 L 104.22 193.53014 L 107.568 195.786419 L 110.916 204.876 L 114.264 216.479721 L 117.612 222.410512 L 120.96 222.217116 L 124.308 222.539442 L 127.656 223.893209 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p4419c81266)" style="fill: none; stroke-dasharray: 0.5,0.825; stroke-dashoffset: 0; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5"/>   </g>   <g id="line2d_17">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 123.972279 L 64.044 126.357488 L 67.392 122.425116 L 70.74 160.588465 L 74.088 159.234698 L 77.436 165.616744 L 80.784 165.681209 L 84.132 161.426512 L 87.48 167.744093 L 90.828 163.876186 L 94.176 159.750419 L 97.524 144.730047 L 100.872 150.531907 L 104.22 165.294419 L 107.568 159.750419 L 110.916 177.865116 L 114.264 166.841581 L 117.612 172.127721 L 120.96 171.612 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p4419c81266)" style="fill: none; stroke-dasharray: 0.5,0.825; stroke-dashoffset: 0; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5"/>   </g>   <g id="line2d_18">    <path d="M 54 127.26 L 57.348 123.907814 L 60.696 121.006884 L 64.044 103.343442 L 67.392 112.626419 L 70.74 125.906233 L 74.088 132.223814 L 77.436 138.21907 L 80.784 138.60586 L 84.132 169.806977 L 87.48 164.649767 L 90.828 165.294419 L 94.176 167.873023 L 97.524 182.377674 L 100.872 173.674884 L 104.22 182.506605 L 107.568 184.311628 L 110.916 197.140186 L 114.264 198.493953 L 117.612 209.453023 L 120.96 210.742326 L 124.308 214.352372 L 127.656 198.751814 L 131.004 184.891814 L 134.352 223.893209 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p4419c81266)" style="fill: none; stroke-dasharray: 0.5,0.825; stroke-dashoffset: 0; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5"/>   </g>   <g id="line2d_19">    <path d="M 54 127.26 L 57.348 123.134233 L 60.696 118.041488 L 64.044 125.197116 L 67.392 120.620093 L 70.74 116.172 L 74.088 157.171814 L 77.436 155.818047 L 80.784 160.846326 L 84.132 163.03814 L 87.48 164.649767 L 90.828 159.750419 L 94.176 163.296 L 97.524 175.544372 L 100.872 182.635535 L 104.22 196.946791 L 107.568 200.621302 L 110.916 193.401209 L 114.264 196.173209 L 117.612 193.723535 L 120.96 190.435814 L 124.308 170.258233 L 127.656 183.215721 L 131.004 199.912186 L 134.352 195.979814 L 137.7 204.682605 L 141.048 207.712465 L 144.396 201.072558 L 147.744 198.171628 L 151.092 208.228186 L 154.44 205.198326 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p4419c81266)" style="fill: none; stroke-dasharray: 0.5,0.825; stroke-dashoffset: 0; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5"/>   </g>   <g id="line2d_20">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 122.940837 L 64.044 122.940837 L 67.392 121.071349 L 70.74 118.557209 L 74.088 145.374698 L 77.436 155.560186 L 80.784 158.074326 L 84.132 169.549116 L 87.48 165.358884 L 90.828 174.964186 L 94.176 176.704744 L 97.524 179.863535 L 100.872 176.382419 L 104.22 179.476744 L 107.568 183.666977 L 110.916 179.992465 L 114.264 183.022326 L 117.612 176.446884 L 120.96 177.736186 L 124.308 165.487814 L 127.656 175.350977 L 131.004 174.061674 L 134.352 171.289674 L 137.7 183.731442 L 141.048 183.344651 L 144.396 184.569488 L 147.744 187.341488 L 151.092 194.561581 L 154.44 190.500279 L 157.788 199.07414 L 161.136 201.394884 L 164.484 223.893209 L 167.832 203.715628 L 171.18 223.893209 L 174.528 223.893209 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p4419c81266)" style="fill: none; stroke-dasharray: 0.5,0.825; stroke-dashoffset: 0; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5"/>   </g>   <g id="line2d_21">    <path d="M 54 127.26 L 57.348 124.165674 L 60.696 123.972279 L 64.044 119.266326 L 67.392 115.269488 L 70.74 114.882698 L 74.088 123.585488 L 77.436 124.488 L 80.784 125.841767 L 84.132 127.453395 L 87.48 128.484837 L 90.828 133.642047 L 94.176 159.750419 L 97.524 162.006698 L 100.872 168.130884 L 104.22 146.53507 L 107.568 171.676465 L 110.916 197.462512 L 114.264 188.953116 L 117.612 187.019163 L 120.96 185.085209 L 124.308 185.536465 L 127.656 190.75814 L 131.004 197.913767 L 134.352 196.56 L 137.7 198.429488 L 141.048 194.497116 L 144.396 209.195163 L 147.744 191.402791 L 151.092 200.234512 L 154.44 208.357116 L 157.788 223.828744 L 161.136 213.192 L 164.484 223.828744 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p4419c81266)" style="fill: none; stroke-dasharray: 0.5,0.825; stroke-dashoffset: 0; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5"/>   </g>   <g id="line2d_22">    <path d="M 54 127.26 L 57.348 122.811907 L 60.696 114.753767 L 64.044 115.785209 L 67.392 142.280372 L 70.74 153.819628 L 74.088 154.141953 L 77.436 165.616744 L 80.784 157.365209 L 84.132 165.745674 L 87.48 141.506791 L 90.828 158.074326 L 94.176 167.486233 L 97.524 167.292837 L 100.872 181.088372 L 104.22 178.767628 L 107.568 205.778512 L 110.916 205.778512 L 114.264 200.170047 L 117.612 188.824186 L 120.96 182.377674 L 124.308 183.795907 L 127.656 186.310047 L 131.004 168.195349 L 134.352 168.259814 L 137.7 168.130884 L 141.048 167.679628 L 144.396 162.715814 L 147.744 178.380837 L 151.092 182.055349 L 154.44 187.857209 L 157.788 189.275442 L 161.136 196.30214 L 164.484 192.434233 L 167.832 199.654326 L 171.18 198.816279 L 174.528 199.976651 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p4419c81266)" style="fill: none; stroke-dasharray: 0.5,0.825; stroke-dashoffset: 0; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5"/>   </g>   <g id="line2d_23">    <path d="M 54 127.26 L 57.348 120.813488 L 60.696 120.813488 L 64.044 125.003721 L 67.392 132.094884 L 70.74 147.566512 L 74.088 152.401395 L 77.436 184.118233 L 80.784 181.797488 L 84.132 186.438977 L 87.48 193.078884 L 90.828 187.019163 L 94.176 185.085209 L 97.524 198.945209 L 100.872 203.780093 L 104.22 217.382233 L 107.568 224.086605 L 110.916 217.382233 L 114.264 210.613395 L 117.612 225.182512 L 120.96 223.828744 L 124.308 223.893209 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p4419c81266)" style="fill: none; stroke-dasharray: 0.5,0.825; stroke-dashoffset: 0; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5"/>   </g>   <g id="line2d_24">    <path d="M 54 127.26 L 57.348 120.104372 L 60.696 118.879535 L 64.044 123.521023 L 67.392 134.544558 L 70.74 134.931349 L 74.088 131.708093 L 77.436 138.09014 L 80.784 137.316558 L 84.132 142.344837 L 87.48 145.374698 L 90.828 169.09786 L 94.176 178.058512 L 97.524 206.745488 L 100.872 203.070977 L 104.22 200.556837 L 107.568 208.743907 L 110.916 205.649581 L 114.264 201.137023 L 117.612 214.416837 L 120.96 213.836651 L 124.308 213.643256 L 127.656 220.863349 L 131.004 223.893209 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p4419c81266)" style="fill: none; stroke-dasharray: 0.5,0.825; stroke-dashoffset: 0; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5"/>   </g>   <g id="line2d_25">    <path d="M 54 127.26 L 57.348 123.907814 L 60.696 121.006884 L 64.044 123.327628 L 67.392 124.488 L 70.74 134.931349 L 74.088 148.726884 L 77.436 155.818047 L 80.784 158.074326 L 84.132 165.745674 L 87.48 164.649767 L 90.828 163.876186 L 94.176 167.486233 L 97.524 179.283349 L 100.872 177.28493 L 104.22 182.506605 L 107.568 195.786419 L 110.916 197.462512 L 114.264 198.493953 L 117.612 193.723535 L 120.96 195.850884 L 124.308 213.643256 L 127.656 199.718791 L 131.004 223.893209 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p4419c81266)" style="fill: none; stroke: #000000; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="patch_3">    <path d="M 54 224.28 L 54 30.24 " style="fill: none"/>   </g>   <g id="patch_4">    <path d="M 388.8 224.28 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="patch_5">    <path d="M 54 224.28 L 388.8 224.28 " style="fill: none"/>   </g>   <g id="patch_6">    <path d="M 54 30.24 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="text_16">    <g id="patch_7">     <a xlink:href="https://dynomight.net/chess">      <path d="M 323.8664 225.676606 L 388.881244 225.676606 L 388.881244 213.343012 L 323.8664 213.343012 z" style="fill: #ffffff; opacity: 0.000001; stroke: #ffffff; stroke-linejoin: miter"/>     </a>    </g>    <a xlink:href="https://dynomight.net/chess">     <!-- dynomight.net/chess -->     <g style="opacity: 0.3" transform="translate(327.8664 220.734419) scale(0.05 -0.05)">      <defs>       <path id="CourierNewPSMT-64" d="M 3206 3922 L 3206 263 L 3556 263 Q 3653 263 3693 227 Q 3734 191 3734 131 Q 3734 75 3693 37 Q 3653 0 3556 0 L 2941 0 L 2941 522 Q 2484 -103 1781 -103 Q 1425 -103 1098 86 Q 772 275 583 625 Q 394 975 394 1347 Q 394 1722 583 2070 Q 772 2419 1098 2609 Q 1425 2800 1784 2800 Q 2472 2800 2941 2175 L 2941 3656 L 2591 3656 Q 2494 3656 2453 3693 Q 2413 3731 2413 3791 Q 2413 3847 2453 3884 Q 2494 3922 2591 3922 L 3206 3922 zM 2941 1347 Q 2941 1847 2603 2190 Q 2266 2534 1800 2534 Q 1331 2534 993 2190 Q 656 1847 656 1347 Q 656 850 993 504 Q 1331 159 1800 159 Q 2266 159 2603 504 Q 2941 850 2941 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-79" d="M 1938 0 L 719 2441 L 641 2441 Q 547 2441 506 2478 Q 466 2516 466 2572 Q 466 2613 486 2645 Q 506 2678 539 2692 Q 572 2706 641 2706 L 1359 2706 Q 1453 2706 1493 2668 Q 1534 2631 1534 2572 Q 1534 2516 1493 2478 Q 1453 2441 1359 2441 L 1006 2441 L 2078 288 L 3134 2441 L 2781 2441 Q 2688 2441 2647 2478 Q 2606 2516 2606 2575 Q 2606 2631 2647 2668 Q 2688 2706 2781 2706 L 3497 2706 Q 3594 2706 3634 2668 Q 3675 2631 3675 2572 Q 3675 2531 3650 2497 Q 3625 2463 3594 2452 Q 3563 2441 3419 2441 L 1756 -944 L 2166 -944 Q 2259 -944 2300 -980 Q 2341 -1016 2341 -1075 Q 2341 -1131 2300 -1168 Q 2259 -1206 2166 -1206 L 653 -1206 Q 559 -1206 518 -1170 Q 478 -1134 478 -1075 Q 478 -1016 518 -980 Q 559 -944 653 -944 L 1475 -944 L 1938 0 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6e" d="M 1072 2706 L 1072 2309 Q 1347 2588 1569 2694 Q 1791 2800 2069 2800 Q 2369 2800 2616 2672 Q 2791 2578 2933 2361 Q 3075 2144 3075 1916 L 3075 263 L 3297 263 Q 3391 263 3431 227 Q 3472 191 3472 131 Q 3472 75 3431 37 Q 3391 0 3297 0 L 2594 0 Q 2497 0 2456 37 Q 2416 75 2416 131 Q 2416 191 2456 227 Q 2497 263 2594 263 L 2813 263 L 2813 1872 Q 2813 2150 2609 2342 Q 2406 2534 2066 2534 Q 1806 2534 1615 2429 Q 1425 2325 1072 1909 L 1072 263 L 1369 263 Q 1463 263 1503 227 Q 1544 191 1544 131 Q 1544 75 1503 37 Q 1463 0 1369 0 L 513 0 Q 419 0 378 37 Q 338 75 338 131 Q 338 191 378 227 Q 419 263 513 263 L 809 263 L 809 2441 L 588 2441 Q 494 2441 453 2478 Q 413 2516 413 2575 Q 413 2631 453 2668 Q 494 2706 588 2706 L 1072 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6f" d="M 3388 1347 Q 3388 747 2958 322 Q 2528 -103 1922 -103 Q 1309 -103 881 323 Q 453 750 453 1347 Q 453 1947 881 2373 Q 1309 2800 1922 2800 Q 2528 2800 2958 2375 Q 3388 1950 3388 1347 zM 3122 1347 Q 3122 1841 2770 2187 Q 2419 2534 1919 2534 Q 1419 2534 1067 2186 Q 716 1838 716 1347 Q 716 859 1067 509 Q 1419 159 1919 159 Q 2419 159 2770 507 Q 3122 856 3122 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6d" d="M 725 2706 L 725 2441 Q 1059 2800 1397 2800 Q 1600 2800 1753 2692 Q 1906 2584 2009 2366 Q 2184 2584 2364 2692 Q 2544 2800 2725 2800 Q 3009 2800 3178 2616 Q 3400 2378 3400 2097 L 3400 263 L 3622 263 Q 3716 263 3756 227 Q 3797 191 3797 131 Q 3797 75 3756 37 Q 3716 0 3622 0 L 3138 0 L 3138 2072 Q 3138 2272 3016 2403 Q 2894 2534 2734 2534 Q 2591 2534 2431 2426 Q 2272 2319 2069 2003 L 2069 263 L 2288 263 Q 2381 263 2422 227 Q 2463 191 2463 131 Q 2463 75 2422 37 Q 2381 0 2288 0 L 1803 0 L 1803 2053 Q 1803 2263 1679 2398 Q 1556 2534 1403 2534 Q 1263 2534 1125 2444 Q 934 2316 725 2003 L 725 263 L 947 263 Q 1041 263 1081 227 Q 1122 191 1122 131 Q 1122 75 1081 37 Q 1041 0 947 0 L 241 0 Q 147 0 106 37 Q 66 75 66 131 Q 66 191 106 227 Q 147 263 241 263 L 463 263 L 463 2441 L 241 2441 Q 147 2441 106 2478 Q 66 2516 66 2575 Q 66 2631 106 2668 Q 147 2706 241 2706 L 725 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-69" d="M 2050 4053 L 2050 3375 L 1663 3375 L 1663 4053 L 2050 4053 zM 2059 2706 L 2059 263 L 3091 263 Q 3188 263 3228 227 Q 3269 191 3269 131 Q 3269 75 3228 37 Q 3188 0 3091 0 L 766 0 Q 672 0 631 37 Q 591 75 591 131 Q 591 191 631 227 Q 672 263 766 263 L 1797 263 L 1797 2441 L 1031 2441 Q 938 2441 895 2478 Q 853 2516 853 2572 Q 853 2631 893 2668 Q 934 2706 1031 2706 L 2059 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-67" d="M 2803 2225 L 2803 2706 L 3419 2706 Q 3513 2706 3553 2668 Q 3594 2631 3594 2572 Q 3594 2516 3553 2478 Q 3513 2441 3419 2441 L 3066 2441 L 3066 -181 Q 3066 -444 2953 -650 Q 2878 -788 2703 -934 Q 2528 -1081 2384 -1143 Q 2241 -1206 2000 -1206 L 1256 -1206 Q 1163 -1206 1122 -1170 Q 1081 -1134 1081 -1075 Q 1081 -1016 1122 -978 Q 1163 -941 1256 -941 L 2009 -944 Q 2241 -944 2427 -825 Q 2613 -706 2734 -478 Q 2803 -347 2803 -144 L 2803 644 Q 2381 66 1716 66 Q 1175 66 783 467 Q 391 869 391 1434 Q 391 2000 783 2400 Q 1175 2800 1716 2800 Q 2381 2800 2803 2225 zM 2803 1434 Q 2803 1897 2486 2215 Q 2169 2534 1728 2534 Q 1288 2534 972 2214 Q 656 1894 656 1434 Q 656 972 972 651 Q 1288 331 1728 331 Q 2169 331 2486 651 Q 2803 972 2803 1434 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-68" d="M 1056 3922 L 1056 2306 Q 1306 2578 1536 2689 Q 1766 2800 2050 2800 Q 2356 2800 2570 2692 Q 2784 2584 2928 2361 Q 3072 2138 3072 1891 L 3072 263 L 3366 263 Q 3463 263 3502 227 Q 3541 191 3541 131 Q 3541 75 3502 37 Q 3463 0 3366 0 L 2513 0 Q 2416 0 2375 37 Q 2334 75 2334 131 Q 2334 191 2375 227 Q 2416 263 2513 263 L 2806 263 L 2806 1872 Q 2806 2156 2601 2347 Q 2397 2538 2031 2538 Q 1744 2538 1541 2397 Q 1394 2297 1056 1925 L 1056 263 L 1353 263 Q 1447 263 1487 227 Q 1528 191 1528 131 Q 1528 75 1487 37 Q 1447 0 1353 0 L 497 0 Q 403 0 362 37 Q 322 75 322 131 Q 322 191 362 227 Q 403 263 497 263 L 794 263 L 794 3656 L 441 3656 Q 347 3656 306 3693 Q 266 3731 266 3791 Q 266 3847 306 3884 Q 347 3922 441 3922 L 1056 3922 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-74" d="M 1381 2706 L 2806 2706 Q 2900 2706 2940 2668 Q 2981 2631 2981 2572 Q 2981 2516 2940 2478 Q 2900 2441 2806 2441 L 1381 2441 L 1381 694 Q 1381 466 1564 312 Q 1747 159 2100 159 Q 2366 159 2675 239 Q 2984 319 3156 419 Q 3219 459 3259 459 Q 3309 459 3346 420 Q 3384 381 3384 328 Q 3384 281 3344 241 Q 3244 138 2858 17 Q 2472 -103 2119 -103 Q 1659 -103 1387 112 Q 1116 328 1116 694 L 1116 2441 L 631 2441 Q 538 2441 497 2478 Q 456 2516 456 2575 Q 456 2631 497 2668 Q 538 2706 631 2706 L 1116 2706 L 1116 3481 Q 1116 3575 1153 3615 Q 1191 3656 1247 3656 Q 1306 3656 1343 3615 Q 1381 3575 1381 3481 L 1381 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2e" d="M 1866 753 L 1978 753 Q 2153 753 2278 629 Q 2403 506 2403 331 Q 2403 150 2276 28 Q 2150 -94 1978 -94 L 1866 -94 Q 1691 -94 1566 29 Q 1441 153 1441 328 Q 1441 509 1567 631 Q 1694 753 1866 753 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-65" d="M 3341 1291 L 666 1291 Q 734 781 1092 470 Q 1450 159 1978 159 Q 2272 159 2594 256 Q 2916 353 3119 513 Q 3178 559 3222 559 Q 3272 559 3309 520 Q 3347 481 3347 428 Q 3347 375 3297 325 Q 3147 169 2764 33 Q 2381 -103 1978 -103 Q 1303 -103 851 339 Q 400 781 400 1409 Q 400 1981 823 2390 Q 1247 2800 1872 2800 Q 2516 2800 2931 2379 Q 3347 1959 3341 1291 zM 3075 1556 Q 2997 1991 2664 2262 Q 2331 2534 1872 2534 Q 1413 2534 1081 2265 Q 750 1997 669 1556 L 3075 1556 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2f" d="M 3100 4113 L 988 -413 Q 934 -528 850 -528 Q 797 -528 758 -489 Q 719 -450 719 -403 Q 719 -366 750 -300 L 2859 4225 Q 2894 4297 2925 4319 Q 2956 4341 3000 4341 Q 3053 4341 3092 4302 Q 3131 4263 3131 4216 Q 3131 4181 3100 4113 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-63" d="M 3016 2441 L 3016 2528 Q 3016 2625 3053 2665 Q 3091 2706 3147 2706 Q 3206 2706 3243 2665 Q 3281 2625 3281 2528 L 3281 1934 Q 3278 1838 3242 1797 Q 3206 1756 3147 1756 Q 3094 1756 3058 1792 Q 3022 1828 3016 1913 Q 2997 2134 2723 2334 Q 2450 2534 1988 2534 Q 1403 2534 1100 2168 Q 797 1803 797 1331 Q 797 822 1131 490 Q 1466 159 1997 159 Q 2303 159 2620 271 Q 2938 384 3194 634 Q 3259 697 3309 697 Q 3363 697 3398 661 Q 3434 625 3434 572 Q 3434 438 3119 231 Q 2609 -103 1984 -103 Q 1350 -103 942 301 Q 534 706 534 1328 Q 534 1963 951 2381 Q 1369 2800 2003 2800 Q 2606 2800 3016 2441 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-73" d="M 2788 2534 Q 2788 2625 2825 2665 Q 2863 2706 2919 2706 Q 2978 2706 3015 2665 Q 3053 2625 3053 2528 L 3053 2078 Q 3053 1984 3015 1943 Q 2978 1903 2919 1903 Q 2866 1903 2830 1937 Q 2794 1972 2788 2050 Q 2769 2238 2594 2359 Q 2338 2534 1916 2534 Q 1475 2534 1231 2356 Q 1047 2222 1047 2056 Q 1047 1869 1266 1744 Q 1416 1656 1834 1609 Q 2381 1550 2594 1475 Q 2897 1366 3045 1172 Q 3194 978 3194 753 Q 3194 419 2872 158 Q 2550 -103 1928 -103 Q 1306 -103 909 213 Q 909 106 896 75 Q 884 44 851 22 Q 819 0 778 0 Q 722 0 684 40 Q 647 81 647 175 L 647 716 Q 647 809 683 850 Q 719 891 778 891 Q 834 891 873 852 Q 913 813 913 747 Q 913 603 984 506 Q 1094 356 1333 257 Q 1572 159 1919 159 Q 2431 159 2681 350 Q 2931 541 2931 753 Q 2931 997 2678 1144 Q 2422 1291 1933 1341 Q 1444 1391 1231 1472 Q 1019 1553 900 1715 Q 781 1878 781 2066 Q 781 2403 1112 2601 Q 1444 2800 1903 2800 Q 2447 2800 2788 2534 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#CourierNewPSMT-64"/>      <use xlink:href="#CourierNewPSMT-79" x="60.009766"/>      <use xlink:href="#CourierNewPSMT-6e" x="120.019531"/>      <use xlink:href="#CourierNewPSMT-6f" x="180.029297"/>      <use xlink:href="#CourierNewPSMT-6d" x="240.039062"/>      <use xlink:href="#CourierNewPSMT-69" x="300.048828"/>      <use xlink:href="#CourierNewPSMT-67" x="360.058594"/>      <use xlink:href="#CourierNewPSMT-68" x="420.068359"/>      <use xlink:href="#CourierNewPSMT-74" x="480.078125"/>      <use xlink:href="#CourierNewPSMT-2e" x="540.087891"/>      <use xlink:href="#CourierNewPSMT-6e" x="600.097656"/>      <use xlink:href="#CourierNewPSMT-65" x="660.107422"/>      <use xlink:href="#CourierNewPSMT-74" x="720.117188"/>      <use xlink:href="#CourierNewPSMT-2f" x="780.126953"/>      <use xlink:href="#CourierNewPSMT-63" x="840.136719"/>      <use xlink:href="#CourierNewPSMT-68" x="900.146484"/>      <use xlink:href="#CourierNewPSMT-65" x="960.15625"/>      <use xlink:href="#CourierNewPSMT-73" x="1020.166016"/>      <use xlink:href="#CourierNewPSMT-73" x="1080.175781"/>     </g>    </a>   </g>   <g id="text_17">    <!-- gpt-3.5-turbo vs stockfish -->    <g transform="translate(145.343437 24.24) scale(0.12 -0.12)">     <defs>      <path id="Montserrat-Regular-67" d="M 3750 3366 L 3750 410 Q 3750 -448 3331 -861 Q 2912 -1274 2067 -1274 Q 1600 -1274 1181 -1136 Q 762 -998 499 -755 L 730 -410 Q 973 -627 1321 -748 Q 1670 -870 2054 -870 Q 2694 -870 2995 -572 Q 3296 -275 3296 352 L 3296 781 Q 3085 461 2742 294 Q 2400 128 1984 128 Q 1510 128 1123 336 Q 736 544 515 918 Q 294 1293 294 1766 Q 294 2240 515 2611 Q 736 2982 1120 3187 Q 1504 3392 1984 3392 Q 2413 3392 2758 3219 Q 3104 3046 3315 2720 L 3315 3366 L 3750 3366 zM 2029 531 Q 2394 531 2688 688 Q 2982 845 3145 1126 Q 3309 1408 3309 1766 Q 3309 2125 3145 2403 Q 2982 2682 2691 2838 Q 2400 2995 2029 2995 Q 1664 2995 1373 2841 Q 1082 2688 918 2406 Q 755 2125 755 1766 Q 755 1408 918 1126 Q 1082 845 1373 688 Q 1664 531 2029 531 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-33" d="M 1856 2560 Q 2560 2522 2928 2179 Q 3296 1837 3296 1286 Q 3296 902 3110 604 Q 2925 307 2560 134 Q 2195 -38 1670 -38 Q 1184 -38 752 118 Q 320 275 58 538 L 282 902 Q 506 672 877 528 Q 1248 384 1670 384 Q 2221 384 2521 624 Q 2822 864 2822 1286 Q 2822 1702 2521 1939 Q 2221 2176 1613 2176 L 1293 2176 L 1293 2515 L 2534 4070 L 243 4070 L 243 4480 L 3123 4480 L 3123 4154 L 1856 2560 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-2e" d="M 678 -32 Q 538 -32 438 70 Q 339 173 339 320 Q 339 467 438 566 Q 538 666 678 666 Q 819 666 921 566 Q 1024 467 1024 320 Q 1024 173 921 70 Q 819 -32 678 -32 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-62" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1184 3827 793 Q 3610 403 3226 185 Q 2842 -32 2368 -32 Q 1946 -32 1610 147 Q 1274 326 1069 666 L 1069 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2726 Q 1299 3053 1628 3222 Q 1958 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2368 Q 3264 2669 2976 2832 Q 2688 2995 2336 2995 Q 1978 2995 1693 2832 Q 1408 2669 1245 2368 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6f" d="M 2010 -32 Q 1523 -32 1132 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1132 3174 Q 1523 3392 2010 3392 Q 2496 3392 2886 3174 Q 3277 2957 3497 2566 Q 3718 2176 3718 1683 Q 3718 1190 3497 800 Q 3277 410 2886 189 Q 2496 -32 2010 -32 zM 2010 371 Q 2368 371 2653 534 Q 2938 698 3098 998 Q 3258 1299 3258 1683 Q 3258 2067 3098 2368 Q 2938 2669 2653 2832 Q 2368 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-76" d="M 3462 3366 L 1965 0 L 1498 0 L 6 3366 L 480 3366 L 1741 499 L 3014 3366 L 3462 3366 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6b" d="M 1882 1690 L 1088 960 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 1530 L 3098 3366 L 3661 3366 L 2221 1990 L 3795 0 L 3238 0 L 1882 1690 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-68" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2778 Q 1274 3072 1600 3232 Q 1926 3392 2349 3392 z" transform="scale(0.015625)"/>     </defs>     <use xlink:href="#Montserrat-Regular-67"/>     <use xlink:href="#Montserrat-Regular-70" x="68.599991"/>     <use xlink:href="#Montserrat-Regular-74" x="136.399979"/>     <use xlink:href="#Montserrat-Regular-2d" x="176.999969"/>     <use xlink:href="#Montserrat-Regular-33" x="215.199966"/>     <use xlink:href="#Montserrat-Regular-2e" x="271.59996"/>     <use xlink:href="#Montserrat-Regular-35" x="292.799957"/>     <use xlink:href="#Montserrat-Regular-2d" x="349.399948"/>     <use xlink:href="#Montserrat-Regular-74" x="387.599945"/>     <use xlink:href="#Montserrat-Regular-75" x="428.199936"/>     <use xlink:href="#Montserrat-Regular-72" x="495.499924"/>     <use xlink:href="#Montserrat-Regular-62" x="535.599915"/>     <use xlink:href="#Montserrat-Regular-6f" x="603.399902"/>     <use xlink:href="#Montserrat-Regular-20" x="666.099899"/>     <use xlink:href="#Montserrat-Regular-76" x="692.299896"/>     <use xlink:href="#Montserrat-Regular-73" x="746.499893"/>     <use xlink:href="#Montserrat-Regular-20" x="795.399887"/>     <use xlink:href="#Montserrat-Regular-73" x="821.599884"/>     <use xlink:href="#Montserrat-Regular-74" x="870.499878"/>     <use xlink:href="#Montserrat-Regular-6f" x="911.099869"/>     <use xlink:href="#Montserrat-Regular-63" x="973.799866"/>     <use xlink:href="#Montserrat-Regular-6b" x="1030.099854"/>     <use xlink:href="#Montserrat-Regular-66" x="1090.199844"/>     <use xlink:href="#Montserrat-Regular-69" x="1124.099838"/>     <use xlink:href="#Montserrat-Regular-73" x="1150.999832"/>     <use xlink:href="#Montserrat-Regular-68" x="1199.899826"/>    </g>   </g>  </g> </g> <defs>  <clipPath id="p4419c81266">   <rect x="54" y="30.24" width="334.8" height="194.04"/>  </clipPath> </defs></svg>](https://dynomight.net/img/chess/gpt-3.5-turbo_stockfish.pdf)

[Terrible](https://lichess.org/Q7QwEBcn).

And then I tried `gpt-4o-mini`, which is a newer chat model.

[<?xml version="1.0" encoding="utf-8" standalone="no"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"  "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg xmlns:xlink="http://www.w3.org/1999/xlink" width="432pt" height="252pt" viewBox="0 0 432 252" xmlns="http://www.w3.org/2000/svg" version="1.1"> <metadata>  <rdf:RDF xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:cc="http://creativecommons.org/ns#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">   <cc:Work>    <dc:type rdf:resource="http://purl.org/dc/dcmitype/StillImage"/>    <dc:date>2024-11-14T11:33:20.106872</dc:date>    <dc:format>image/svg+xml</dc:format>    <dc:creator>     <cc:Agent>      <dc:title>Matplotlib v3.8.4, https://matplotlib.org/</dc:title>     </cc:Agent>    </dc:creator>   </cc:Work>  </rdf:RDF> </metadata> <defs>  <style type="text/css">*{stroke-linejoin: round; stroke-linecap: butt}</style> </defs> <g id="figure_1">  <g id="patch_1">   <path d="M 0 252 L 432 252 L 432 0 L 0 0 z" style="fill: #ffffff"/>  </g>  <g id="axes_1">   <g id="patch_2">    <path d="M 54 224.28 L 388.8 224.28 L 388.8 30.24 L 54 30.24 z" style="fill: #ffffff"/>   </g>   <g id="matplotlib.axis_1">    <g id="xtick_1">     <g id="line2d_1"/>     <g id="text_1">      <!-- 0 -->      <g transform="translate(50.689844 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-30" d="M 2118 -38 Q 1600 -38 1193 237 Q 787 512 560 1027 Q 333 1542 333 2240 Q 333 2938 560 3453 Q 787 3968 1193 4243 Q 1600 4518 2118 4518 Q 2637 4518 3043 4243 Q 3450 3968 3680 3453 Q 3910 2938 3910 2240 Q 3910 1542 3680 1027 Q 3450 512 3043 237 Q 2637 -38 2118 -38 zM 2118 384 Q 2509 384 2806 601 Q 3104 819 3270 1235 Q 3437 1651 3437 2240 Q 3437 2829 3270 3245 Q 3104 3661 2806 3878 Q 2509 4096 2118 4096 Q 1728 4096 1430 3878 Q 1133 3661 966 3245 Q 800 2829 800 2240 Q 800 1651 966 1235 Q 1133 819 1430 601 Q 1728 384 2118 384 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="xtick_2">     <g id="line2d_2"/>     <g id="text_2">      <!-- 20 -->      <g transform="translate(114.81 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-32" d="M 3392 410 L 3392 0 L 250 0 L 250 326 L 2112 2150 Q 2464 2496 2589 2749 Q 2714 3002 2714 3258 Q 2714 3654 2442 3875 Q 2170 4096 1664 4096 Q 877 4096 442 3597 L 115 3878 Q 378 4186 784 4352 Q 1190 4518 1702 4518 Q 2387 4518 2787 4195 Q 3187 3872 3187 3309 Q 3187 2963 3033 2643 Q 2880 2323 2451 1907 L 928 410 L 3392 410 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-32"/>       <use xlink:href="#Montserrat-Regular-30" x="56.799988"/>      </g>     </g>    </g>    <g id="xtick_3">     <g id="line2d_3"/>     <g id="text_3">      <!-- 40 -->      <g transform="translate(181.305156 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-34" d="M 4141 1178 L 3245 1178 L 3245 0 L 2784 0 L 2784 1178 L 256 1178 L 256 1510 L 2630 4480 L 3142 4480 L 851 1587 L 2797 1587 L 2797 2624 L 3245 2624 L 3245 1587 L 4141 1587 L 4141 1178 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-34"/>       <use xlink:href="#Montserrat-Regular-30" x="66.099991"/>      </g>     </g>    </g>    <g id="xtick_4">     <g id="line2d_4"/>     <g id="text_4">      <!-- 60 -->      <g transform="translate(248.524531 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-36" d="M 2189 2682 Q 2630 2682 2972 2515 Q 3315 2349 3507 2045 Q 3699 1741 3699 1338 Q 3699 928 3500 614 Q 3302 301 2953 131 Q 2605 -38 2170 -38 Q 1280 -38 806 547 Q 333 1133 333 2202 Q 333 2950 589 3468 Q 845 3987 1305 4252 Q 1766 4518 2381 4518 Q 3046 4518 3443 4275 L 3258 3904 Q 2938 4115 2387 4115 Q 1645 4115 1222 3644 Q 800 3174 800 2278 Q 800 2118 819 1914 Q 986 2278 1350 2480 Q 1715 2682 2189 2682 zM 2150 352 Q 2637 352 2941 617 Q 3245 883 3245 1325 Q 3245 1766 2941 2028 Q 2637 2291 2118 2291 Q 1779 2291 1520 2163 Q 1261 2035 1113 1811 Q 966 1587 966 1312 Q 966 1056 1104 838 Q 1242 621 1510 486 Q 1779 352 2150 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-36"/>       <use xlink:href="#Montserrat-Regular-30" x="60.899994"/>      </g>     </g>    </g>    <g id="xtick_5">     <g id="line2d_5"/>     <g id="text_5">      <!-- 80 -->      <g transform="translate(315.34 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-38" d="M 2899 2349 Q 3322 2208 3549 1926 Q 3776 1645 3776 1248 Q 3776 858 3565 566 Q 3354 275 2960 118 Q 2566 -38 2035 -38 Q 1229 -38 768 307 Q 307 653 307 1248 Q 307 1651 528 1929 Q 749 2208 1171 2349 Q 826 2483 646 2736 Q 467 2989 467 3334 Q 467 3878 892 4198 Q 1318 4518 2035 4518 Q 2509 4518 2864 4374 Q 3219 4230 3417 3961 Q 3616 3693 3616 3334 Q 3616 2989 3430 2736 Q 3245 2483 2899 2349 zM 928 3322 Q 928 2950 1222 2732 Q 1517 2515 2035 2515 Q 2554 2515 2851 2732 Q 3149 2950 3149 3315 Q 3149 3693 2845 3910 Q 2541 4128 2035 4128 Q 1523 4128 1225 3910 Q 928 3693 928 3322 zM 2035 352 Q 2630 352 2966 595 Q 3302 838 3302 1254 Q 3302 1670 2966 1910 Q 2630 2150 2035 2150 Q 1446 2150 1110 1910 Q 774 1670 774 1254 Q 774 832 1107 592 Q 1440 352 2035 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-38"/>       <use xlink:href="#Montserrat-Regular-30" x="63.799988"/>      </g>     </g>    </g>    <g id="xtick_6">     <g id="line2d_6"/>     <g id="text_6">      <!-- 100 -->      <g transform="translate(380.375 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-31" d="M 1574 4480 L 1574 0 L 1114 0 L 1114 4070 L 58 4070 L 58 4480 L 1574 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>      </g>     </g>    </g>    <g id="text_7">     <!-- turns -->     <g transform="translate(208.170312 248.56125) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-74" d="M 2406 205 Q 2278 90 2089 29 Q 1901 -32 1696 -32 Q 1222 -32 966 224 Q 710 480 710 947 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 4102 L 1165 4102 L 1165 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 973 Q 1165 672 1315 515 Q 1466 358 1747 358 Q 1888 358 2019 403 Q 2150 448 2246 531 L 2406 205 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-75" d="M 3667 3366 L 3667 0 L 3232 0 L 3232 614 Q 3053 307 2739 137 Q 2426 -32 2022 -32 Q 1363 -32 982 336 Q 602 704 602 1414 L 602 3366 L 1056 3366 L 1056 1459 Q 1056 928 1318 653 Q 1581 378 2067 378 Q 2598 378 2905 701 Q 3213 1024 3213 1600 L 3213 3366 L 3667 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-72" d="M 1069 2707 Q 1229 3046 1545 3219 Q 1862 3392 2330 3392 L 2330 2950 L 2221 2957 Q 1690 2957 1389 2630 Q 1088 2304 1088 1715 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2707 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-6e" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2746 Q 1254 3053 1584 3222 Q 1914 3392 2349 3392 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-73" d="M 1536 -32 Q 1126 -32 752 86 Q 378 205 166 384 L 371 742 Q 582 576 902 470 Q 1222 365 1568 365 Q 2029 365 2249 509 Q 2470 653 2470 915 Q 2470 1101 2348 1206 Q 2227 1312 2041 1366 Q 1856 1421 1549 1472 Q 1139 1549 889 1629 Q 640 1709 464 1901 Q 288 2093 288 2432 Q 288 2854 640 3123 Q 992 3392 1619 3392 Q 1946 3392 2272 3305 Q 2598 3219 2810 3078 L 2611 2714 Q 2195 3002 1619 3002 Q 1184 3002 963 2848 Q 742 2694 742 2445 Q 742 2253 867 2137 Q 992 2022 1177 1968 Q 1363 1914 1690 1856 Q 2093 1779 2336 1702 Q 2579 1626 2752 1440 Q 2925 1254 2925 928 Q 2925 486 2557 227 Q 2189 -32 1536 -32 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-74"/>      <use xlink:href="#Montserrat-Regular-75" x="40.599991"/>      <use xlink:href="#Montserrat-Regular-72" x="107.899979"/>      <use xlink:href="#Montserrat-Regular-6e" x="147.999969"/>      <use xlink:href="#Montserrat-Regular-73" x="215.699951"/>     </g>    </g>   </g>   <g id="matplotlib.axis_2">    <g id="ytick_1">     <g id="line2d_7"/>     <g id="text_8">      <!-- LOSS -->      <g transform="translate(23.91875 227.667831) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-4c" d="M 730 4480 L 1203 4480 L 1203 410 L 3712 410 L 3712 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4f" d="M 2688 -38 Q 2022 -38 1481 259 Q 941 557 637 1075 Q 333 1594 333 2240 Q 333 2886 637 3404 Q 941 3923 1481 4220 Q 2022 4518 2688 4518 Q 3354 4518 3888 4224 Q 4422 3930 4729 3408 Q 5037 2886 5037 2240 Q 5037 1594 4729 1072 Q 4422 550 3888 256 Q 3354 -38 2688 -38 zM 2688 384 Q 3219 384 3648 624 Q 4077 864 4320 1289 Q 4563 1715 4563 2240 Q 4563 2765 4320 3190 Q 4077 3616 3648 3856 Q 3219 4096 2688 4096 Q 2157 4096 1725 3856 Q 1293 3616 1046 3190 Q 800 2765 800 2240 Q 800 1715 1046 1289 Q 1293 864 1725 624 Q 2157 384 2688 384 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-53" d="M 1965 -38 Q 1466 -38 1008 122 Q 550 282 301 544 L 486 909 Q 730 666 1133 515 Q 1536 365 1965 365 Q 2566 365 2867 585 Q 3168 806 3168 1158 Q 3168 1427 3005 1587 Q 2842 1747 2605 1833 Q 2368 1920 1946 2022 Q 1440 2150 1139 2268 Q 838 2387 624 2630 Q 410 2874 410 3290 Q 410 3629 589 3907 Q 768 4186 1139 4352 Q 1510 4518 2061 4518 Q 2445 4518 2813 4412 Q 3181 4307 3450 4122 L 3290 3744 Q 3008 3930 2688 4022 Q 2368 4115 2061 4115 Q 1472 4115 1174 3888 Q 877 3661 877 3302 Q 877 3034 1040 2870 Q 1203 2707 1449 2620 Q 1696 2534 2112 2432 Q 2605 2310 2909 2192 Q 3213 2074 3424 1834 Q 3635 1594 3635 1184 Q 3635 845 3452 566 Q 3270 288 2892 125 Q 2515 -38 1965 -38 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-4c"/>       <use xlink:href="#Montserrat-Regular-4f" x="58.899994"/>       <use xlink:href="#Montserrat-Regular-53" x="142.799988"/>       <use xlink:href="#Montserrat-Regular-53" x="204.299973"/>      </g>     </g>    </g>    <g id="ytick_2">     <g id="line2d_8"/>     <g id="text_9">      <!-- -1000 -->      <g transform="translate(23.209375 195.435273) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-2d" d="M 384 1933 L 2061 1933 L 2061 1536 L 384 1536 L 384 1933 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-31" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="74.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="140.499985"/>       <use xlink:href="#Montserrat-Regular-30" x="206.699982"/>      </g>     </g>    </g>    <g id="ytick_3">     <g id="line2d_9"/>     <g id="text_10">      <!-- -500 -->      <g transform="translate(27.779687 163.202714) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-35" d="M 1510 2643 Q 2483 2643 2928 2294 Q 3373 1946 3373 1318 Q 3373 928 3190 621 Q 3008 314 2643 138 Q 2278 -38 1747 -38 Q 1267 -38 835 118 Q 403 275 141 538 L 365 902 Q 589 672 953 528 Q 1318 384 1741 384 Q 2298 384 2602 633 Q 2906 883 2906 1299 Q 2906 1760 2563 1997 Q 2221 2234 1408 2234 L 454 2234 L 685 4480 L 3136 4480 L 3136 4070 L 1088 4070 L 934 2643 L 1510 2643 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-35" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="94.799988"/>       <use xlink:href="#Montserrat-Regular-30" x="160.999985"/>      </g>     </g>    </g>    <g id="ytick_4">     <g id="line2d_10"/>     <g id="text_11">      <!-- 0 -->      <g transform="translate(43.879688 130.970156) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="ytick_5">     <g id="line2d_11"/>     <g id="text_12">      <!-- 500 -->      <g transform="translate(31.6 98.737598) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-35"/>       <use xlink:href="#Montserrat-Regular-30" x="56.599991"/>       <use xlink:href="#Montserrat-Regular-30" x="122.799988"/>      </g>     </g>    </g>    <g id="ytick_6">     <g id="line2d_12"/>     <g id="text_13">      <!-- 1000 -->      <g transform="translate(27.029687 66.50504) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="168.499985"/>      </g>     </g>    </g>    <g id="ytick_7">     <g id="line2d_13"/>     <g id="text_14">      <!-- WIN -->      <g transform="translate(28.240625 34.272482) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-57" d="M 6880 4480 L 5370 0 L 4870 0 L 3558 3821 L 2240 0 L 1747 0 L 237 4480 L 723 4480 L 2016 627 L 3354 4480 L 3795 4480 L 5114 608 L 6426 4480 L 6880 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-49" d="M 730 4480 L 1203 4480 L 1203 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4e" d="M 4474 4480 L 4474 0 L 4083 0 L 1203 3635 L 1203 0 L 730 0 L 730 4480 L 1120 4480 L 4006 845 L 4006 4480 L 4474 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-57"/>       <use xlink:href="#Montserrat-Regular-49" x="111.099991"/>       <use xlink:href="#Montserrat-Regular-4e" x="141.299988"/>      </g>     </g>    </g>    <g id="text_15">     <!-- centipawn difference -->     <g transform="translate(17.26875 180.677188) rotate(-90) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-63" d="M 2029 -32 Q 1530 -32 1136 185 Q 742 403 518 796 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1136 3174 Q 1530 3392 2029 3392 Q 2464 3392 2806 3222 Q 3149 3053 3347 2726 L 3008 2496 Q 2842 2746 2586 2870 Q 2330 2995 2029 2995 Q 1664 2995 1373 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1293 918 995 Q 1082 698 1373 534 Q 1664 371 2029 371 Q 2330 371 2586 492 Q 2842 614 3008 864 L 3347 634 Q 3149 307 2803 137 Q 2458 -32 2029 -32 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-65" d="M 3565 1542 L 749 1542 Q 787 1018 1152 694 Q 1517 371 2074 371 Q 2387 371 2649 483 Q 2912 595 3104 813 L 3360 518 Q 3136 250 2800 109 Q 2464 -32 2061 -32 Q 1542 -32 1142 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 508 2566 Q 723 2957 1097 3174 Q 1472 3392 1939 3392 Q 2406 3392 2777 3174 Q 3149 2957 3360 2569 Q 3571 2182 3571 1683 L 3565 1542 zM 1939 3002 Q 1453 3002 1123 2691 Q 794 2381 749 1882 L 3136 1882 Q 3091 2381 2761 2691 Q 2432 3002 1939 3002 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-69" d="M 634 3366 L 1088 3366 L 1088 0 L 634 0 L 634 3366 zM 864 4102 Q 723 4102 627 4198 Q 531 4294 531 4429 Q 531 4557 627 4653 Q 723 4749 864 4749 Q 1005 4749 1101 4656 Q 1197 4563 1197 4435 Q 1197 4294 1101 4198 Q 1005 4102 864 4102 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-70" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1178 3827 787 Q 3610 397 3229 182 Q 2848 -32 2368 -32 Q 1958 -32 1628 137 Q 1299 307 1088 634 L 1088 -1242 L 634 -1242 L 634 3366 L 1069 3366 L 1069 2701 Q 1274 3034 1610 3213 Q 1946 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2364 Q 3264 2662 2976 2828 Q 2688 2995 2336 2995 Q 1978 2995 1693 2828 Q 1408 2662 1245 2364 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-61" d="M 1811 3392 Q 2470 3392 2822 3062 Q 3174 2733 3174 2086 L 3174 0 L 2739 0 L 2739 525 Q 2586 262 2288 115 Q 1990 -32 1581 -32 Q 1018 -32 685 237 Q 352 506 352 947 Q 352 1376 662 1638 Q 973 1901 1651 1901 L 2720 1901 L 2720 2106 Q 2720 2541 2477 2768 Q 2234 2995 1766 2995 Q 1446 2995 1152 2889 Q 858 2784 646 2598 L 442 2938 Q 698 3155 1056 3273 Q 1414 3392 1811 3392 zM 1651 326 Q 2035 326 2310 502 Q 2586 678 2720 1011 L 2720 1562 L 1664 1562 Q 800 1562 800 960 Q 800 666 1024 496 Q 1248 326 1651 326 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-77" d="M 5574 3366 L 4301 0 L 3872 0 L 2810 2758 L 1747 0 L 1318 0 L 51 3366 L 486 3366 L 1542 506 L 2624 3366 L 3014 3366 L 4090 506 L 5158 3366 L 5574 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-20" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-64" d="M 3706 4749 L 3706 0 L 3270 0 L 3270 666 Q 3066 326 2730 147 Q 2394 -32 1971 -32 Q 1498 -32 1114 185 Q 730 403 512 793 Q 294 1184 294 1683 Q 294 2182 512 2572 Q 730 2963 1114 3177 Q 1498 3392 1971 3392 Q 2381 3392 2710 3222 Q 3040 3053 3251 2726 L 3251 4749 L 3706 4749 zM 2010 371 Q 2362 371 2646 534 Q 2931 698 3094 998 Q 3258 1299 3258 1683 Q 3258 2067 3094 2368 Q 2931 2669 2646 2832 Q 2362 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-66" d="M 1747 4397 Q 1453 4397 1302 4237 Q 1152 4077 1152 3763 L 1152 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 0 L 710 0 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 3782 Q 710 4243 976 4512 Q 1242 4781 1728 4781 Q 1920 4781 2099 4726 Q 2278 4672 2400 4570 L 2246 4230 Q 2048 4397 1747 4397 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-63"/>      <use xlink:href="#Montserrat-Regular-65" x="56.299988"/>      <use xlink:href="#Montserrat-Regular-6e" x="116.699982"/>      <use xlink:href="#Montserrat-Regular-74" x="184.399963"/>      <use xlink:href="#Montserrat-Regular-69" x="224.999954"/>      <use xlink:href="#Montserrat-Regular-70" x="251.899948"/>      <use xlink:href="#Montserrat-Regular-61" x="319.699936"/>      <use xlink:href="#Montserrat-Regular-77" x="378.699921"/>      <use xlink:href="#Montserrat-Regular-6e" x="466.599915"/>      <use xlink:href="#Montserrat-Regular-20" x="534.299896"/>      <use xlink:href="#Montserrat-Regular-64" x="560.499893"/>      <use xlink:href="#Montserrat-Regular-69" x="628.299881"/>      <use xlink:href="#Montserrat-Regular-66" x="655.199875"/>      <use xlink:href="#Montserrat-Regular-66" x="689.099869"/>      <use xlink:href="#Montserrat-Regular-65" x="722.999863"/>      <use xlink:href="#Montserrat-Regular-72" x="783.399857"/>      <use xlink:href="#Montserrat-Regular-65" x="823.499847"/>      <use xlink:href="#Montserrat-Regular-6e" x="883.899841"/>      <use xlink:href="#Montserrat-Regular-63" x="951.599823"/>      <use xlink:href="#Montserrat-Regular-65" x="1007.899811"/>     </g>    </g>   </g>   <g id="line2d_14">    <path d="M 54 127.26 L 57.348 118.363814 L 60.696 114.173581 L 64.044 97.734977 L 67.392 97.799442 L 70.74 100.120186 L 74.088 96.51014 L 77.436 102.505395 L 80.784 105.535256 L 84.132 108.11386 L 87.48 108.049395 L 90.828 114.624837 L 94.176 121.071349 L 97.524 114.238047 L 100.872 99.475535 L 104.22 93.480279 L 107.568 94.189395 L 110.916 138.60586 L 114.264 138.60586 L 117.612 165.229953 L 120.96 161.362047 L 124.308 163.811721 L 127.656 178.251907 L 131.004 186.632372 L 134.352 176.446884 L 137.7 154.85107 L 141.048 169.806977 L 144.396 165.81014 L 147.744 174.51293 L 151.092 175.544372 L 154.44 171.096279 L 157.788 192.692093 L 161.136 171.096279 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pa9ce699240)" style="fill: none; stroke: #d62728; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_15">    <path d="M 54 127.26 L 57.348 122.618512 L 60.696 122.167256 L 64.044 123.263163 L 67.392 121.97386 L 70.74 122.360651 L 74.088 121.45814 L 77.436 111.143721 L 80.784 111.723907 L 84.132 121.135814 L 87.48 123.649953 L 90.828 125.841767 L 94.176 154.013023 L 97.524 172.063256 L 100.872 176.382419 L 104.22 171.289674 L 107.568 166.970512 L 110.916 169.226791 L 114.264 164.907628 L 117.612 184.311628 L 120.96 188.824186 L 124.308 182.506605 L 127.656 187.921674 L 131.004 193.336744 L 134.352 195.464093 L 137.7 194.561581 L 141.048 195.464093 L 144.396 195.721953 L 147.744 191.209395 L 151.092 191.531721 L 154.44 194.948372 L 157.788 188.630791 L 161.136 187.148093 L 164.484 191.402791 L 167.832 185.536465 L 171.18 185.085209 L 174.528 177.607256 L 177.876 178.445302 L 181.224 178.445302 L 184.572 179.79907 L 187.92 174.706326 L 191.268 168.001953 L 194.616 177.994047 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pa9ce699240)" style="fill: none; stroke: #d62728; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_16">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 124.488 L 64.044 122.811907 L 67.392 122.554047 L 70.74 118.750605 L 74.088 127.969116 L 77.436 132.93293 L 80.784 132.804 L 84.132 137.123163 L 87.48 223.957674 L 90.828 223.957674 L 94.176 223.957674 L 97.524 223.957674 L 100.872 223.957674 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pa9ce699240)" style="fill: none; stroke: #d62728; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_17">    <path d="M 54 127.26 L 57.348 125.003721 L 60.696 119.07293 L 64.044 126.164093 L 67.392 126.164093 L 70.74 121.651535 L 74.088 120.749023 L 77.436 121.716 L 80.784 124.939256 L 84.132 115.076093 L 87.48 122.876372 L 90.828 120.877953 L 94.176 122.811907 L 97.524 126.293023 L 100.872 130.418791 L 104.22 154.013023 L 107.568 154.915535 L 110.916 157.945395 L 114.264 154.335349 L 117.612 166.19693 L 120.96 181.668558 L 124.308 177.929581 L 127.656 176.446884 L 131.004 175.093116 L 134.352 183.215721 L 137.7 197.011256 L 141.048 196.882326 L 144.396 184.376093 L 147.744 189.726698 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pa9ce699240)" style="fill: none; stroke: #d62728; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_18">    <path d="M 54 127.26 L 57.348 125.068186 L 60.696 124.874791 L 64.044 124.939256 L 67.392 144.858977 L 70.74 138.09014 L 74.088 141.700186 L 77.436 152.07907 L 80.784 153.432837 L 84.132 154.915535 L 87.48 151.434419 L 90.828 157.558605 L 94.176 157.107349 L 97.524 157.88093 L 100.872 149.500465 L 104.22 150.080651 L 107.568 160.588465 L 110.916 169.162326 L 114.264 160.846326 L 117.612 160.459535 L 120.96 166.068 L 124.308 156.591628 L 127.656 160.008279 L 131.004 171.86986 L 134.352 192.240837 L 137.7 223.893209 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pa9ce699240)" style="fill: none; stroke: #d62728; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_19">    <path d="M 54 127.26 L 57.348 119.07293 L 60.696 122.360651 L 64.044 121.716 L 67.392 122.231721 L 70.74 122.102791 L 74.088 117.074512 L 77.436 117.525767 L 80.784 115.656279 L 84.132 110.112279 L 87.48 108.049395 L 90.828 106.373302 L 94.176 114.624837 L 97.524 118.105953 L 100.872 110.75693 L 104.22 108.307256 L 107.568 107.856 L 110.916 112.948744 L 114.264 149.30707 L 117.612 154.85107 L 120.96 151.241023 L 124.308 152.014605 L 127.656 148.146698 L 131.004 171.547535 L 134.352 173.223628 L 137.7 165.294419 L 141.048 181.668558 L 144.396 184.891814 L 147.744 192.949953 L 151.092 194.883907 L 154.44 197.978233 L 157.788 194.690512 L 161.136 201.201488 L 164.484 200.943628 L 167.832 223.893209 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pa9ce699240)" style="fill: none; stroke: #d62728; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_20">    <path d="M 54 127.26 L 57.348 118.363814 L 60.696 100.958233 L 64.044 125.390512 L 67.392 128.291442 L 70.74 123.714419 L 74.088 133.964372 L 77.436 134.351163 L 80.784 131.256837 L 84.132 130.225395 L 87.48 137.445488 L 90.828 109.016372 L 94.176 134.737953 L 97.524 158.074326 L 100.872 143.956465 L 104.22 154.98 L 107.568 158.590047 L 110.916 175.608837 L 114.264 173.932744 L 117.612 177.478326 L 120.96 189.017581 L 124.308 187.921674 L 127.656 193.594605 L 131.004 198.171628 L 134.352 196.044279 L 137.7 223.893209 L 141.048 223.893209 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pa9ce699240)" style="fill: none; stroke: #d62728; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_21">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 124.35907 L 64.044 124.488 L 67.392 124.488 L 70.74 126.937674 L 74.088 133.255256 L 77.436 165.552279 L 80.784 181.861953 L 84.132 182.635535 L 87.48 179.541209 L 90.828 178.961023 L 94.176 154.270884 L 97.524 150.596372 L 100.872 173.223628 L 104.22 168.066419 L 107.568 177.994047 L 110.916 182.184279 L 114.264 179.928 L 117.612 176.382419 L 120.96 171.998791 L 124.308 170.516093 L 127.656 186.890233 L 131.004 183.344651 L 134.352 192.563163 L 137.7 185.536465 L 141.048 223.893209 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pa9ce699240)" style="fill: none; stroke: #d62728; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_22">    <path d="M 54 127.26 L 57.348 118.363814 L 60.696 114.624837 L 64.044 102.892186 L 67.392 116.623256 L 70.74 120.104372 L 74.088 122.682977 L 77.436 120.620093 L 80.784 140.604279 L 84.132 135.31814 L 87.48 143.376279 L 90.828 168.904465 L 94.176 175.093116 L 97.524 169.742512 L 100.872 176.382419 L 104.22 174.12614 L 107.568 184.633953 L 110.916 182.055349 L 114.264 194.303721 L 117.612 195.721953 L 120.96 200.879163 L 124.308 200.943628 L 127.656 194.497116 L 131.004 197.204651 L 134.352 182.82893 L 137.7 191.982977 L 141.048 191.080465 L 144.396 191.854047 L 147.744 193.078884 L 151.092 195.979814 L 154.44 198.236093 L 157.788 199.267535 L 161.136 193.401209 L 164.484 194.04586 L 167.832 205.13386 L 171.18 223.893209 L 174.528 223.893209 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pa9ce699240)" style="fill: none; stroke: #d62728; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_23">    <path d="M 54 127.26 L 57.348 126.164093 L 60.696 125.390512 L 64.044 124.165674 L 67.392 123.263163 L 70.74 124.810326 L 74.088 125.132651 L 77.436 122.618512 L 80.784 123.649953 L 84.132 120.039907 L 87.48 127.26 L 90.828 122.038326 L 94.176 125.326047 L 97.524 120.749023 L 100.872 136.671907 L 104.22 158.00986 L 107.568 157.816465 L 110.916 156.462698 L 114.264 147.953302 L 117.612 147.050791 L 120.96 134.737953 L 124.308 135.83386 L 127.656 169.355721 L 131.004 169.226791 L 134.352 188.695256 L 137.7 183.924837 L 141.048 171.48307 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pa9ce699240)" style="fill: none; stroke: #d62728; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_24">    <path d="M 54 127.26 L 57.348 123.553256 L 60.696 122.263953 L 64.044 123.714419 L 67.392 122.908605 L 70.74 122.231721 L 74.088 123.907814 L 77.436 122.167256 L 80.784 128.098047 L 84.132 125.680605 L 87.48 132.352744 L 90.828 123.940047 L 94.176 144.375488 L 97.524 154.238651 L 100.872 146.728465 L 104.22 156.49493 L 107.568 159.589256 L 110.916 169.194558 L 114.264 162.876977 L 117.612 171.289674 L 120.96 176.833674 L 124.308 174.222837 L 127.656 182.57107 L 131.004 184.988512 L 134.352 190.468047 L 137.7 193.272279 L 141.048 196.173209 L 144.396 209.839814 L 147.744 208.518279 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#pa9ce699240)" style="fill: none; stroke: #000000; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="patch_3">    <path d="M 54 224.28 L 54 30.24 " style="fill: none"/>   </g>   <g id="patch_4">    <path d="M 388.8 224.28 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="patch_5">    <path d="M 54 224.28 L 388.8 224.28 " style="fill: none"/>   </g>   <g id="patch_6">    <path d="M 54 30.24 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="text_16">    <g id="patch_7">     <a xlink:href="https://dynomight.net/chess">      <path d="M 323.8664 225.676606 L 388.881244 225.676606 L 388.881244 213.343012 L 323.8664 213.343012 z" style="fill: #ffffff; opacity: 0.000001; stroke: #ffffff; stroke-linejoin: miter"/>     </a>    </g>    <a xlink:href="https://dynomight.net/chess">     <!-- dynomight.net/chess -->     <g style="opacity: 0.3" transform="translate(327.8664 220.734419) scale(0.05 -0.05)">      <defs>       <path id="CourierNewPSMT-64" d="M 3206 3922 L 3206 263 L 3556 263 Q 3653 263 3693 227 Q 3734 191 3734 131 Q 3734 75 3693 37 Q 3653 0 3556 0 L 2941 0 L 2941 522 Q 2484 -103 1781 -103 Q 1425 -103 1098 86 Q 772 275 583 625 Q 394 975 394 1347 Q 394 1722 583 2070 Q 772 2419 1098 2609 Q 1425 2800 1784 2800 Q 2472 2800 2941 2175 L 2941 3656 L 2591 3656 Q 2494 3656 2453 3693 Q 2413 3731 2413 3791 Q 2413 3847 2453 3884 Q 2494 3922 2591 3922 L 3206 3922 zM 2941 1347 Q 2941 1847 2603 2190 Q 2266 2534 1800 2534 Q 1331 2534 993 2190 Q 656 1847 656 1347 Q 656 850 993 504 Q 1331 159 1800 159 Q 2266 159 2603 504 Q 2941 850 2941 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-79" d="M 1938 0 L 719 2441 L 641 2441 Q 547 2441 506 2478 Q 466 2516 466 2572 Q 466 2613 486 2645 Q 506 2678 539 2692 Q 572 2706 641 2706 L 1359 2706 Q 1453 2706 1493 2668 Q 1534 2631 1534 2572 Q 1534 2516 1493 2478 Q 1453 2441 1359 2441 L 1006 2441 L 2078 288 L 3134 2441 L 2781 2441 Q 2688 2441 2647 2478 Q 2606 2516 2606 2575 Q 2606 2631 2647 2668 Q 2688 2706 2781 2706 L 3497 2706 Q 3594 2706 3634 2668 Q 3675 2631 3675 2572 Q 3675 2531 3650 2497 Q 3625 2463 3594 2452 Q 3563 2441 3419 2441 L 1756 -944 L 2166 -944 Q 2259 -944 2300 -980 Q 2341 -1016 2341 -1075 Q 2341 -1131 2300 -1168 Q 2259 -1206 2166 -1206 L 653 -1206 Q 559 -1206 518 -1170 Q 478 -1134 478 -1075 Q 478 -1016 518 -980 Q 559 -944 653 -944 L 1475 -944 L 1938 0 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6e" d="M 1072 2706 L 1072 2309 Q 1347 2588 1569 2694 Q 1791 2800 2069 2800 Q 2369 2800 2616 2672 Q 2791 2578 2933 2361 Q 3075 2144 3075 1916 L 3075 263 L 3297 263 Q 3391 263 3431 227 Q 3472 191 3472 131 Q 3472 75 3431 37 Q 3391 0 3297 0 L 2594 0 Q 2497 0 2456 37 Q 2416 75 2416 131 Q 2416 191 2456 227 Q 2497 263 2594 263 L 2813 263 L 2813 1872 Q 2813 2150 2609 2342 Q 2406 2534 2066 2534 Q 1806 2534 1615 2429 Q 1425 2325 1072 1909 L 1072 263 L 1369 263 Q 1463 263 1503 227 Q 1544 191 1544 131 Q 1544 75 1503 37 Q 1463 0 1369 0 L 513 0 Q 419 0 378 37 Q 338 75 338 131 Q 338 191 378 227 Q 419 263 513 263 L 809 263 L 809 2441 L 588 2441 Q 494 2441 453 2478 Q 413 2516 413 2575 Q 413 2631 453 2668 Q 494 2706 588 2706 L 1072 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6f" d="M 3388 1347 Q 3388 747 2958 322 Q 2528 -103 1922 -103 Q 1309 -103 881 323 Q 453 750 453 1347 Q 453 1947 881 2373 Q 1309 2800 1922 2800 Q 2528 2800 2958 2375 Q 3388 1950 3388 1347 zM 3122 1347 Q 3122 1841 2770 2187 Q 2419 2534 1919 2534 Q 1419 2534 1067 2186 Q 716 1838 716 1347 Q 716 859 1067 509 Q 1419 159 1919 159 Q 2419 159 2770 507 Q 3122 856 3122 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6d" d="M 725 2706 L 725 2441 Q 1059 2800 1397 2800 Q 1600 2800 1753 2692 Q 1906 2584 2009 2366 Q 2184 2584 2364 2692 Q 2544 2800 2725 2800 Q 3009 2800 3178 2616 Q 3400 2378 3400 2097 L 3400 263 L 3622 263 Q 3716 263 3756 227 Q 3797 191 3797 131 Q 3797 75 3756 37 Q 3716 0 3622 0 L 3138 0 L 3138 2072 Q 3138 2272 3016 2403 Q 2894 2534 2734 2534 Q 2591 2534 2431 2426 Q 2272 2319 2069 2003 L 2069 263 L 2288 263 Q 2381 263 2422 227 Q 2463 191 2463 131 Q 2463 75 2422 37 Q 2381 0 2288 0 L 1803 0 L 1803 2053 Q 1803 2263 1679 2398 Q 1556 2534 1403 2534 Q 1263 2534 1125 2444 Q 934 2316 725 2003 L 725 263 L 947 263 Q 1041 263 1081 227 Q 1122 191 1122 131 Q 1122 75 1081 37 Q 1041 0 947 0 L 241 0 Q 147 0 106 37 Q 66 75 66 131 Q 66 191 106 227 Q 147 263 241 263 L 463 263 L 463 2441 L 241 2441 Q 147 2441 106 2478 Q 66 2516 66 2575 Q 66 2631 106 2668 Q 147 2706 241 2706 L 725 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-69" d="M 2050 4053 L 2050 3375 L 1663 3375 L 1663 4053 L 2050 4053 zM 2059 2706 L 2059 263 L 3091 263 Q 3188 263 3228 227 Q 3269 191 3269 131 Q 3269 75 3228 37 Q 3188 0 3091 0 L 766 0 Q 672 0 631 37 Q 591 75 591 131 Q 591 191 631 227 Q 672 263 766 263 L 1797 263 L 1797 2441 L 1031 2441 Q 938 2441 895 2478 Q 853 2516 853 2572 Q 853 2631 893 2668 Q 934 2706 1031 2706 L 2059 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-67" d="M 2803 2225 L 2803 2706 L 3419 2706 Q 3513 2706 3553 2668 Q 3594 2631 3594 2572 Q 3594 2516 3553 2478 Q 3513 2441 3419 2441 L 3066 2441 L 3066 -181 Q 3066 -444 2953 -650 Q 2878 -788 2703 -934 Q 2528 -1081 2384 -1143 Q 2241 -1206 2000 -1206 L 1256 -1206 Q 1163 -1206 1122 -1170 Q 1081 -1134 1081 -1075 Q 1081 -1016 1122 -978 Q 1163 -941 1256 -941 L 2009 -944 Q 2241 -944 2427 -825 Q 2613 -706 2734 -478 Q 2803 -347 2803 -144 L 2803 644 Q 2381 66 1716 66 Q 1175 66 783 467 Q 391 869 391 1434 Q 391 2000 783 2400 Q 1175 2800 1716 2800 Q 2381 2800 2803 2225 zM 2803 1434 Q 2803 1897 2486 2215 Q 2169 2534 1728 2534 Q 1288 2534 972 2214 Q 656 1894 656 1434 Q 656 972 972 651 Q 1288 331 1728 331 Q 2169 331 2486 651 Q 2803 972 2803 1434 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-68" d="M 1056 3922 L 1056 2306 Q 1306 2578 1536 2689 Q 1766 2800 2050 2800 Q 2356 2800 2570 2692 Q 2784 2584 2928 2361 Q 3072 2138 3072 1891 L 3072 263 L 3366 263 Q 3463 263 3502 227 Q 3541 191 3541 131 Q 3541 75 3502 37 Q 3463 0 3366 0 L 2513 0 Q 2416 0 2375 37 Q 2334 75 2334 131 Q 2334 191 2375 227 Q 2416 263 2513 263 L 2806 263 L 2806 1872 Q 2806 2156 2601 2347 Q 2397 2538 2031 2538 Q 1744 2538 1541 2397 Q 1394 2297 1056 1925 L 1056 263 L 1353 263 Q 1447 263 1487 227 Q 1528 191 1528 131 Q 1528 75 1487 37 Q 1447 0 1353 0 L 497 0 Q 403 0 362 37 Q 322 75 322 131 Q 322 191 362 227 Q 403 263 497 263 L 794 263 L 794 3656 L 441 3656 Q 347 3656 306 3693 Q 266 3731 266 3791 Q 266 3847 306 3884 Q 347 3922 441 3922 L 1056 3922 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-74" d="M 1381 2706 L 2806 2706 Q 2900 2706 2940 2668 Q 2981 2631 2981 2572 Q 2981 2516 2940 2478 Q 2900 2441 2806 2441 L 1381 2441 L 1381 694 Q 1381 466 1564 312 Q 1747 159 2100 159 Q 2366 159 2675 239 Q 2984 319 3156 419 Q 3219 459 3259 459 Q 3309 459 3346 420 Q 3384 381 3384 328 Q 3384 281 3344 241 Q 3244 138 2858 17 Q 2472 -103 2119 -103 Q 1659 -103 1387 112 Q 1116 328 1116 694 L 1116 2441 L 631 2441 Q 538 2441 497 2478 Q 456 2516 456 2575 Q 456 2631 497 2668 Q 538 2706 631 2706 L 1116 2706 L 1116 3481 Q 1116 3575 1153 3615 Q 1191 3656 1247 3656 Q 1306 3656 1343 3615 Q 1381 3575 1381 3481 L 1381 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2e" d="M 1866 753 L 1978 753 Q 2153 753 2278 629 Q 2403 506 2403 331 Q 2403 150 2276 28 Q 2150 -94 1978 -94 L 1866 -94 Q 1691 -94 1566 29 Q 1441 153 1441 328 Q 1441 509 1567 631 Q 1694 753 1866 753 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-65" d="M 3341 1291 L 666 1291 Q 734 781 1092 470 Q 1450 159 1978 159 Q 2272 159 2594 256 Q 2916 353 3119 513 Q 3178 559 3222 559 Q 3272 559 3309 520 Q 3347 481 3347 428 Q 3347 375 3297 325 Q 3147 169 2764 33 Q 2381 -103 1978 -103 Q 1303 -103 851 339 Q 400 781 400 1409 Q 400 1981 823 2390 Q 1247 2800 1872 2800 Q 2516 2800 2931 2379 Q 3347 1959 3341 1291 zM 3075 1556 Q 2997 1991 2664 2262 Q 2331 2534 1872 2534 Q 1413 2534 1081 2265 Q 750 1997 669 1556 L 3075 1556 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2f" d="M 3100 4113 L 988 -413 Q 934 -528 850 -528 Q 797 -528 758 -489 Q 719 -450 719 -403 Q 719 -366 750 -300 L 2859 4225 Q 2894 4297 2925 4319 Q 2956 4341 3000 4341 Q 3053 4341 3092 4302 Q 3131 4263 3131 4216 Q 3131 4181 3100 4113 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-63" d="M 3016 2441 L 3016 2528 Q 3016 2625 3053 2665 Q 3091 2706 3147 2706 Q 3206 2706 3243 2665 Q 3281 2625 3281 2528 L 3281 1934 Q 3278 1838 3242 1797 Q 3206 1756 3147 1756 Q 3094 1756 3058 1792 Q 3022 1828 3016 1913 Q 2997 2134 2723 2334 Q 2450 2534 1988 2534 Q 1403 2534 1100 2168 Q 797 1803 797 1331 Q 797 822 1131 490 Q 1466 159 1997 159 Q 2303 159 2620 271 Q 2938 384 3194 634 Q 3259 697 3309 697 Q 3363 697 3398 661 Q 3434 625 3434 572 Q 3434 438 3119 231 Q 2609 -103 1984 -103 Q 1350 -103 942 301 Q 534 706 534 1328 Q 534 1963 951 2381 Q 1369 2800 2003 2800 Q 2606 2800 3016 2441 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-73" d="M 2788 2534 Q 2788 2625 2825 2665 Q 2863 2706 2919 2706 Q 2978 2706 3015 2665 Q 3053 2625 3053 2528 L 3053 2078 Q 3053 1984 3015 1943 Q 2978 1903 2919 1903 Q 2866 1903 2830 1937 Q 2794 1972 2788 2050 Q 2769 2238 2594 2359 Q 2338 2534 1916 2534 Q 1475 2534 1231 2356 Q 1047 2222 1047 2056 Q 1047 1869 1266 1744 Q 1416 1656 1834 1609 Q 2381 1550 2594 1475 Q 2897 1366 3045 1172 Q 3194 978 3194 753 Q 3194 419 2872 158 Q 2550 -103 1928 -103 Q 1306 -103 909 213 Q 909 106 896 75 Q 884 44 851 22 Q 819 0 778 0 Q 722 0 684 40 Q 647 81 647 175 L 647 716 Q 647 809 683 850 Q 719 891 778 891 Q 834 891 873 852 Q 913 813 913 747 Q 913 603 984 506 Q 1094 356 1333 257 Q 1572 159 1919 159 Q 2431 159 2681 350 Q 2931 541 2931 753 Q 2931 997 2678 1144 Q 2422 1291 1933 1341 Q 1444 1391 1231 1472 Q 1019 1553 900 1715 Q 781 1878 781 2066 Q 781 2403 1112 2601 Q 1444 2800 1903 2800 Q 2447 2800 2788 2534 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#CourierNewPSMT-64"/>      <use xlink:href="#CourierNewPSMT-79" x="60.009766"/>      <use xlink:href="#CourierNewPSMT-6e" x="120.019531"/>      <use xlink:href="#CourierNewPSMT-6f" x="180.029297"/>      <use xlink:href="#CourierNewPSMT-6d" x="240.039062"/>      <use xlink:href="#CourierNewPSMT-69" x="300.048828"/>      <use xlink:href="#CourierNewPSMT-67" x="360.058594"/>      <use xlink:href="#CourierNewPSMT-68" x="420.068359"/>      <use xlink:href="#CourierNewPSMT-74" x="480.078125"/>      <use xlink:href="#CourierNewPSMT-2e" x="540.087891"/>      <use xlink:href="#CourierNewPSMT-6e" x="600.097656"/>      <use xlink:href="#CourierNewPSMT-65" x="660.107422"/>      <use xlink:href="#CourierNewPSMT-74" x="720.117188"/>      <use xlink:href="#CourierNewPSMT-2f" x="780.126953"/>      <use xlink:href="#CourierNewPSMT-63" x="840.136719"/>      <use xlink:href="#CourierNewPSMT-68" x="900.146484"/>      <use xlink:href="#CourierNewPSMT-65" x="960.15625"/>      <use xlink:href="#CourierNewPSMT-73" x="1020.166016"/>      <use xlink:href="#CourierNewPSMT-73" x="1080.175781"/>     </g>    </a>   </g>   <g id="text_17">    <!-- gpt-4o-mini vs stockfish -->    <g transform="translate(148.720313 24.24) scale(0.12 -0.12)">     <defs>      <path id="Montserrat-Regular-67" d="M 3750 3366 L 3750 410 Q 3750 -448 3331 -861 Q 2912 -1274 2067 -1274 Q 1600 -1274 1181 -1136 Q 762 -998 499 -755 L 730 -410 Q 973 -627 1321 -748 Q 1670 -870 2054 -870 Q 2694 -870 2995 -572 Q 3296 -275 3296 352 L 3296 781 Q 3085 461 2742 294 Q 2400 128 1984 128 Q 1510 128 1123 336 Q 736 544 515 918 Q 294 1293 294 1766 Q 294 2240 515 2611 Q 736 2982 1120 3187 Q 1504 3392 1984 3392 Q 2413 3392 2758 3219 Q 3104 3046 3315 2720 L 3315 3366 L 3750 3366 zM 2029 531 Q 2394 531 2688 688 Q 2982 845 3145 1126 Q 3309 1408 3309 1766 Q 3309 2125 3145 2403 Q 2982 2682 2691 2838 Q 2400 2995 2029 2995 Q 1664 2995 1373 2841 Q 1082 2688 918 2406 Q 755 2125 755 1766 Q 755 1408 918 1126 Q 1082 845 1373 688 Q 1664 531 2029 531 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6f" d="M 2010 -32 Q 1523 -32 1132 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1132 3174 Q 1523 3392 2010 3392 Q 2496 3392 2886 3174 Q 3277 2957 3497 2566 Q 3718 2176 3718 1683 Q 3718 1190 3497 800 Q 3277 410 2886 189 Q 2496 -32 2010 -32 zM 2010 371 Q 2368 371 2653 534 Q 2938 698 3098 998 Q 3258 1299 3258 1683 Q 3258 2067 3098 2368 Q 2938 2669 2653 2832 Q 2368 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6d" d="M 4832 3392 Q 5466 3392 5827 3027 Q 6189 2662 6189 1952 L 6189 0 L 5734 0 L 5734 1907 Q 5734 2432 5481 2707 Q 5229 2982 4768 2982 Q 4243 2982 3942 2659 Q 3642 2336 3642 1766 L 3642 0 L 3187 0 L 3187 1907 Q 3187 2432 2934 2707 Q 2682 2982 2214 2982 Q 1696 2982 1392 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2752 Q 1248 3059 1568 3225 Q 1888 3392 2304 3392 Q 2726 3392 3036 3213 Q 3347 3034 3501 2682 Q 3686 3014 4035 3203 Q 4384 3392 4832 3392 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-76" d="M 3462 3366 L 1965 0 L 1498 0 L 6 3366 L 480 3366 L 1741 499 L 3014 3366 L 3462 3366 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6b" d="M 1882 1690 L 1088 960 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 1530 L 3098 3366 L 3661 3366 L 2221 1990 L 3795 0 L 3238 0 L 1882 1690 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-68" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2778 Q 1274 3072 1600 3232 Q 1926 3392 2349 3392 z" transform="scale(0.015625)"/>     </defs>     <use xlink:href="#Montserrat-Regular-67"/>     <use xlink:href="#Montserrat-Regular-70" x="68.599991"/>     <use xlink:href="#Montserrat-Regular-74" x="136.399979"/>     <use xlink:href="#Montserrat-Regular-2d" x="176.999969"/>     <use xlink:href="#Montserrat-Regular-34" x="215.199966"/>     <use xlink:href="#Montserrat-Regular-6f" x="281.299957"/>     <use xlink:href="#Montserrat-Regular-2d" x="343.999954"/>     <use xlink:href="#Montserrat-Regular-6d" x="382.199951"/>     <use xlink:href="#Montserrat-Regular-69" x="488.299942"/>     <use xlink:href="#Montserrat-Regular-6e" x="515.199936"/>     <use xlink:href="#Montserrat-Regular-69" x="582.899918"/>     <use xlink:href="#Montserrat-Regular-20" x="609.799911"/>     <use xlink:href="#Montserrat-Regular-76" x="635.999908"/>     <use xlink:href="#Montserrat-Regular-73" x="690.199905"/>     <use xlink:href="#Montserrat-Regular-20" x="739.099899"/>     <use xlink:href="#Montserrat-Regular-73" x="765.299896"/>     <use xlink:href="#Montserrat-Regular-74" x="814.19989"/>     <use xlink:href="#Montserrat-Regular-6f" x="854.799881"/>     <use xlink:href="#Montserrat-Regular-63" x="917.499878"/>     <use xlink:href="#Montserrat-Regular-6b" x="973.799866"/>     <use xlink:href="#Montserrat-Regular-66" x="1033.899857"/>     <use xlink:href="#Montserrat-Regular-69" x="1067.79985"/>     <use xlink:href="#Montserrat-Regular-73" x="1094.699844"/>     <use xlink:href="#Montserrat-Regular-68" x="1143.599838"/>    </g>   </g>  </g> </g> <defs>  <clipPath id="pa9ce699240">   <rect x="54" y="30.24" width="334.8" height="194.04"/>  </clipPath> </defs></svg>](https://dynomight.net/img/chess/gpt-4o-mini_stockfish.pdf)

[Terrible](https://lichess.org/z9PyJVSE).

And then I tried `gpt-4o`, a bigger chat model.

[<?xml version="1.0" encoding="utf-8" standalone="no"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"  "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg xmlns:xlink="http://www.w3.org/1999/xlink" width="432pt" height="252pt" viewBox="0 0 432 252" xmlns="http://www.w3.org/2000/svg" version="1.1"> <metadata>  <rdf:RDF xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:cc="http://creativecommons.org/ns#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">   <cc:Work>    <dc:type rdf:resource="http://purl.org/dc/dcmitype/StillImage"/>    <dc:date>2024-11-14T11:33:11.685626</dc:date>    <dc:format>image/svg+xml</dc:format>    <dc:creator>     <cc:Agent>      <dc:title>Matplotlib v3.8.4, https://matplotlib.org/</dc:title>     </cc:Agent>    </dc:creator>   </cc:Work>  </rdf:RDF> </metadata> <defs>  <style type="text/css">*{stroke-linejoin: round; stroke-linecap: butt}</style> </defs> <g id="figure_1">  <g id="patch_1">   <path d="M 0 252 L 432 252 L 432 0 L 0 0 z" style="fill: #ffffff"/>  </g>  <g id="axes_1">   <g id="patch_2">    <path d="M 54 224.28 L 388.8 224.28 L 388.8 30.24 L 54 30.24 z" style="fill: #ffffff"/>   </g>   <g id="matplotlib.axis_1">    <g id="xtick_1">     <g id="line2d_1"/>     <g id="text_1">      <!-- 0 -->      <g transform="translate(50.689844 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-30" d="M 2118 -38 Q 1600 -38 1193 237 Q 787 512 560 1027 Q 333 1542 333 2240 Q 333 2938 560 3453 Q 787 3968 1193 4243 Q 1600 4518 2118 4518 Q 2637 4518 3043 4243 Q 3450 3968 3680 3453 Q 3910 2938 3910 2240 Q 3910 1542 3680 1027 Q 3450 512 3043 237 Q 2637 -38 2118 -38 zM 2118 384 Q 2509 384 2806 601 Q 3104 819 3270 1235 Q 3437 1651 3437 2240 Q 3437 2829 3270 3245 Q 3104 3661 2806 3878 Q 2509 4096 2118 4096 Q 1728 4096 1430 3878 Q 1133 3661 966 3245 Q 800 2829 800 2240 Q 800 1651 966 1235 Q 1133 819 1430 601 Q 1728 384 2118 384 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="xtick_2">     <g id="line2d_2"/>     <g id="text_2">      <!-- 20 -->      <g transform="translate(114.81 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-32" d="M 3392 410 L 3392 0 L 250 0 L 250 326 L 2112 2150 Q 2464 2496 2589 2749 Q 2714 3002 2714 3258 Q 2714 3654 2442 3875 Q 2170 4096 1664 4096 Q 877 4096 442 3597 L 115 3878 Q 378 4186 784 4352 Q 1190 4518 1702 4518 Q 2387 4518 2787 4195 Q 3187 3872 3187 3309 Q 3187 2963 3033 2643 Q 2880 2323 2451 1907 L 928 410 L 3392 410 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-32"/>       <use xlink:href="#Montserrat-Regular-30" x="56.799988"/>      </g>     </g>    </g>    <g id="xtick_3">     <g id="line2d_3"/>     <g id="text_3">      <!-- 40 -->      <g transform="translate(181.305156 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-34" d="M 4141 1178 L 3245 1178 L 3245 0 L 2784 0 L 2784 1178 L 256 1178 L 256 1510 L 2630 4480 L 3142 4480 L 851 1587 L 2797 1587 L 2797 2624 L 3245 2624 L 3245 1587 L 4141 1587 L 4141 1178 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-34"/>       <use xlink:href="#Montserrat-Regular-30" x="66.099991"/>      </g>     </g>    </g>    <g id="xtick_4">     <g id="line2d_4"/>     <g id="text_4">      <!-- 60 -->      <g transform="translate(248.524531 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-36" d="M 2189 2682 Q 2630 2682 2972 2515 Q 3315 2349 3507 2045 Q 3699 1741 3699 1338 Q 3699 928 3500 614 Q 3302 301 2953 131 Q 2605 -38 2170 -38 Q 1280 -38 806 547 Q 333 1133 333 2202 Q 333 2950 589 3468 Q 845 3987 1305 4252 Q 1766 4518 2381 4518 Q 3046 4518 3443 4275 L 3258 3904 Q 2938 4115 2387 4115 Q 1645 4115 1222 3644 Q 800 3174 800 2278 Q 800 2118 819 1914 Q 986 2278 1350 2480 Q 1715 2682 2189 2682 zM 2150 352 Q 2637 352 2941 617 Q 3245 883 3245 1325 Q 3245 1766 2941 2028 Q 2637 2291 2118 2291 Q 1779 2291 1520 2163 Q 1261 2035 1113 1811 Q 966 1587 966 1312 Q 966 1056 1104 838 Q 1242 621 1510 486 Q 1779 352 2150 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-36"/>       <use xlink:href="#Montserrat-Regular-30" x="60.899994"/>      </g>     </g>    </g>    <g id="xtick_5">     <g id="line2d_5"/>     <g id="text_5">      <!-- 80 -->      <g transform="translate(315.34 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-38" d="M 2899 2349 Q 3322 2208 3549 1926 Q 3776 1645 3776 1248 Q 3776 858 3565 566 Q 3354 275 2960 118 Q 2566 -38 2035 -38 Q 1229 -38 768 307 Q 307 653 307 1248 Q 307 1651 528 1929 Q 749 2208 1171 2349 Q 826 2483 646 2736 Q 467 2989 467 3334 Q 467 3878 892 4198 Q 1318 4518 2035 4518 Q 2509 4518 2864 4374 Q 3219 4230 3417 3961 Q 3616 3693 3616 3334 Q 3616 2989 3430 2736 Q 3245 2483 2899 2349 zM 928 3322 Q 928 2950 1222 2732 Q 1517 2515 2035 2515 Q 2554 2515 2851 2732 Q 3149 2950 3149 3315 Q 3149 3693 2845 3910 Q 2541 4128 2035 4128 Q 1523 4128 1225 3910 Q 928 3693 928 3322 zM 2035 352 Q 2630 352 2966 595 Q 3302 838 3302 1254 Q 3302 1670 2966 1910 Q 2630 2150 2035 2150 Q 1446 2150 1110 1910 Q 774 1670 774 1254 Q 774 832 1107 592 Q 1440 352 2035 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-38"/>       <use xlink:href="#Montserrat-Regular-30" x="63.799988"/>      </g>     </g>    </g>    <g id="xtick_6">     <g id="line2d_6"/>     <g id="text_6">      <!-- 100 -->      <g transform="translate(380.375 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-31" d="M 1574 4480 L 1574 0 L 1114 0 L 1114 4070 L 58 4070 L 58 4480 L 1574 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>      </g>     </g>    </g>    <g id="text_7">     <!-- turns -->     <g transform="translate(208.170312 248.56125) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-74" d="M 2406 205 Q 2278 90 2089 29 Q 1901 -32 1696 -32 Q 1222 -32 966 224 Q 710 480 710 947 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 4102 L 1165 4102 L 1165 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 973 Q 1165 672 1315 515 Q 1466 358 1747 358 Q 1888 358 2019 403 Q 2150 448 2246 531 L 2406 205 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-75" d="M 3667 3366 L 3667 0 L 3232 0 L 3232 614 Q 3053 307 2739 137 Q 2426 -32 2022 -32 Q 1363 -32 982 336 Q 602 704 602 1414 L 602 3366 L 1056 3366 L 1056 1459 Q 1056 928 1318 653 Q 1581 378 2067 378 Q 2598 378 2905 701 Q 3213 1024 3213 1600 L 3213 3366 L 3667 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-72" d="M 1069 2707 Q 1229 3046 1545 3219 Q 1862 3392 2330 3392 L 2330 2950 L 2221 2957 Q 1690 2957 1389 2630 Q 1088 2304 1088 1715 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2707 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-6e" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2746 Q 1254 3053 1584 3222 Q 1914 3392 2349 3392 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-73" d="M 1536 -32 Q 1126 -32 752 86 Q 378 205 166 384 L 371 742 Q 582 576 902 470 Q 1222 365 1568 365 Q 2029 365 2249 509 Q 2470 653 2470 915 Q 2470 1101 2348 1206 Q 2227 1312 2041 1366 Q 1856 1421 1549 1472 Q 1139 1549 889 1629 Q 640 1709 464 1901 Q 288 2093 288 2432 Q 288 2854 640 3123 Q 992 3392 1619 3392 Q 1946 3392 2272 3305 Q 2598 3219 2810 3078 L 2611 2714 Q 2195 3002 1619 3002 Q 1184 3002 963 2848 Q 742 2694 742 2445 Q 742 2253 867 2137 Q 992 2022 1177 1968 Q 1363 1914 1690 1856 Q 2093 1779 2336 1702 Q 2579 1626 2752 1440 Q 2925 1254 2925 928 Q 2925 486 2557 227 Q 2189 -32 1536 -32 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-74"/>      <use xlink:href="#Montserrat-Regular-75" x="40.599991"/>      <use xlink:href="#Montserrat-Regular-72" x="107.899979"/>      <use xlink:href="#Montserrat-Regular-6e" x="147.999969"/>      <use xlink:href="#Montserrat-Regular-73" x="215.699951"/>     </g>    </g>   </g>   <g id="matplotlib.axis_2">    <g id="ytick_1">     <g id="line2d_7"/>     <g id="text_8">      <!-- LOSS -->      <g transform="translate(23.91875 227.667831) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-4c" d="M 730 4480 L 1203 4480 L 1203 410 L 3712 410 L 3712 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4f" d="M 2688 -38 Q 2022 -38 1481 259 Q 941 557 637 1075 Q 333 1594 333 2240 Q 333 2886 637 3404 Q 941 3923 1481 4220 Q 2022 4518 2688 4518 Q 3354 4518 3888 4224 Q 4422 3930 4729 3408 Q 5037 2886 5037 2240 Q 5037 1594 4729 1072 Q 4422 550 3888 256 Q 3354 -38 2688 -38 zM 2688 384 Q 3219 384 3648 624 Q 4077 864 4320 1289 Q 4563 1715 4563 2240 Q 4563 2765 4320 3190 Q 4077 3616 3648 3856 Q 3219 4096 2688 4096 Q 2157 4096 1725 3856 Q 1293 3616 1046 3190 Q 800 2765 800 2240 Q 800 1715 1046 1289 Q 1293 864 1725 624 Q 2157 384 2688 384 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-53" d="M 1965 -38 Q 1466 -38 1008 122 Q 550 282 301 544 L 486 909 Q 730 666 1133 515 Q 1536 365 1965 365 Q 2566 365 2867 585 Q 3168 806 3168 1158 Q 3168 1427 3005 1587 Q 2842 1747 2605 1833 Q 2368 1920 1946 2022 Q 1440 2150 1139 2268 Q 838 2387 624 2630 Q 410 2874 410 3290 Q 410 3629 589 3907 Q 768 4186 1139 4352 Q 1510 4518 2061 4518 Q 2445 4518 2813 4412 Q 3181 4307 3450 4122 L 3290 3744 Q 3008 3930 2688 4022 Q 2368 4115 2061 4115 Q 1472 4115 1174 3888 Q 877 3661 877 3302 Q 877 3034 1040 2870 Q 1203 2707 1449 2620 Q 1696 2534 2112 2432 Q 2605 2310 2909 2192 Q 3213 2074 3424 1834 Q 3635 1594 3635 1184 Q 3635 845 3452 566 Q 3270 288 2892 125 Q 2515 -38 1965 -38 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-4c"/>       <use xlink:href="#Montserrat-Regular-4f" x="58.899994"/>       <use xlink:href="#Montserrat-Regular-53" x="142.799988"/>       <use xlink:href="#Montserrat-Regular-53" x="204.299973"/>      </g>     </g>    </g>    <g id="ytick_2">     <g id="line2d_8"/>     <g id="text_9">      <!-- -1000 -->      <g transform="translate(23.209375 195.435273) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-2d" d="M 384 1933 L 2061 1933 L 2061 1536 L 384 1536 L 384 1933 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-31" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="74.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="140.499985"/>       <use xlink:href="#Montserrat-Regular-30" x="206.699982"/>      </g>     </g>    </g>    <g id="ytick_3">     <g id="line2d_9"/>     <g id="text_10">      <!-- -500 -->      <g transform="translate(27.779687 163.202714) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-35" d="M 1510 2643 Q 2483 2643 2928 2294 Q 3373 1946 3373 1318 Q 3373 928 3190 621 Q 3008 314 2643 138 Q 2278 -38 1747 -38 Q 1267 -38 835 118 Q 403 275 141 538 L 365 902 Q 589 672 953 528 Q 1318 384 1741 384 Q 2298 384 2602 633 Q 2906 883 2906 1299 Q 2906 1760 2563 1997 Q 2221 2234 1408 2234 L 454 2234 L 685 4480 L 3136 4480 L 3136 4070 L 1088 4070 L 934 2643 L 1510 2643 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-35" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="94.799988"/>       <use xlink:href="#Montserrat-Regular-30" x="160.999985"/>      </g>     </g>    </g>    <g id="ytick_4">     <g id="line2d_10"/>     <g id="text_11">      <!-- 0 -->      <g transform="translate(43.879688 130.970156) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="ytick_5">     <g id="line2d_11"/>     <g id="text_12">      <!-- 500 -->      <g transform="translate(31.6 98.737598) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-35"/>       <use xlink:href="#Montserrat-Regular-30" x="56.599991"/>       <use xlink:href="#Montserrat-Regular-30" x="122.799988"/>      </g>     </g>    </g>    <g id="ytick_6">     <g id="line2d_12"/>     <g id="text_13">      <!-- 1000 -->      <g transform="translate(27.029687 66.50504) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="168.499985"/>      </g>     </g>    </g>    <g id="ytick_7">     <g id="line2d_13"/>     <g id="text_14">      <!-- WIN -->      <g transform="translate(28.240625 34.272482) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-57" d="M 6880 4480 L 5370 0 L 4870 0 L 3558 3821 L 2240 0 L 1747 0 L 237 4480 L 723 4480 L 2016 627 L 3354 4480 L 3795 4480 L 5114 608 L 6426 4480 L 6880 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-49" d="M 730 4480 L 1203 4480 L 1203 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4e" d="M 4474 4480 L 4474 0 L 4083 0 L 1203 3635 L 1203 0 L 730 0 L 730 4480 L 1120 4480 L 4006 845 L 4006 4480 L 4474 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-57"/>       <use xlink:href="#Montserrat-Regular-49" x="111.099991"/>       <use xlink:href="#Montserrat-Regular-4e" x="141.299988"/>      </g>     </g>    </g>    <g id="text_15">     <!-- centipawn difference -->     <g transform="translate(17.26875 180.677188) rotate(-90) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-63" d="M 2029 -32 Q 1530 -32 1136 185 Q 742 403 518 796 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1136 3174 Q 1530 3392 2029 3392 Q 2464 3392 2806 3222 Q 3149 3053 3347 2726 L 3008 2496 Q 2842 2746 2586 2870 Q 2330 2995 2029 2995 Q 1664 2995 1373 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1293 918 995 Q 1082 698 1373 534 Q 1664 371 2029 371 Q 2330 371 2586 492 Q 2842 614 3008 864 L 3347 634 Q 3149 307 2803 137 Q 2458 -32 2029 -32 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-65" d="M 3565 1542 L 749 1542 Q 787 1018 1152 694 Q 1517 371 2074 371 Q 2387 371 2649 483 Q 2912 595 3104 813 L 3360 518 Q 3136 250 2800 109 Q 2464 -32 2061 -32 Q 1542 -32 1142 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 508 2566 Q 723 2957 1097 3174 Q 1472 3392 1939 3392 Q 2406 3392 2777 3174 Q 3149 2957 3360 2569 Q 3571 2182 3571 1683 L 3565 1542 zM 1939 3002 Q 1453 3002 1123 2691 Q 794 2381 749 1882 L 3136 1882 Q 3091 2381 2761 2691 Q 2432 3002 1939 3002 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-69" d="M 634 3366 L 1088 3366 L 1088 0 L 634 0 L 634 3366 zM 864 4102 Q 723 4102 627 4198 Q 531 4294 531 4429 Q 531 4557 627 4653 Q 723 4749 864 4749 Q 1005 4749 1101 4656 Q 1197 4563 1197 4435 Q 1197 4294 1101 4198 Q 1005 4102 864 4102 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-70" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1178 3827 787 Q 3610 397 3229 182 Q 2848 -32 2368 -32 Q 1958 -32 1628 137 Q 1299 307 1088 634 L 1088 -1242 L 634 -1242 L 634 3366 L 1069 3366 L 1069 2701 Q 1274 3034 1610 3213 Q 1946 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2364 Q 3264 2662 2976 2828 Q 2688 2995 2336 2995 Q 1978 2995 1693 2828 Q 1408 2662 1245 2364 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-61" d="M 1811 3392 Q 2470 3392 2822 3062 Q 3174 2733 3174 2086 L 3174 0 L 2739 0 L 2739 525 Q 2586 262 2288 115 Q 1990 -32 1581 -32 Q 1018 -32 685 237 Q 352 506 352 947 Q 352 1376 662 1638 Q 973 1901 1651 1901 L 2720 1901 L 2720 2106 Q 2720 2541 2477 2768 Q 2234 2995 1766 2995 Q 1446 2995 1152 2889 Q 858 2784 646 2598 L 442 2938 Q 698 3155 1056 3273 Q 1414 3392 1811 3392 zM 1651 326 Q 2035 326 2310 502 Q 2586 678 2720 1011 L 2720 1562 L 1664 1562 Q 800 1562 800 960 Q 800 666 1024 496 Q 1248 326 1651 326 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-77" d="M 5574 3366 L 4301 0 L 3872 0 L 2810 2758 L 1747 0 L 1318 0 L 51 3366 L 486 3366 L 1542 506 L 2624 3366 L 3014 3366 L 4090 506 L 5158 3366 L 5574 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-20" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-64" d="M 3706 4749 L 3706 0 L 3270 0 L 3270 666 Q 3066 326 2730 147 Q 2394 -32 1971 -32 Q 1498 -32 1114 185 Q 730 403 512 793 Q 294 1184 294 1683 Q 294 2182 512 2572 Q 730 2963 1114 3177 Q 1498 3392 1971 3392 Q 2381 3392 2710 3222 Q 3040 3053 3251 2726 L 3251 4749 L 3706 4749 zM 2010 371 Q 2362 371 2646 534 Q 2931 698 3094 998 Q 3258 1299 3258 1683 Q 3258 2067 3094 2368 Q 2931 2669 2646 2832 Q 2362 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-66" d="M 1747 4397 Q 1453 4397 1302 4237 Q 1152 4077 1152 3763 L 1152 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 0 L 710 0 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 3782 Q 710 4243 976 4512 Q 1242 4781 1728 4781 Q 1920 4781 2099 4726 Q 2278 4672 2400 4570 L 2246 4230 Q 2048 4397 1747 4397 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-63"/>      <use xlink:href="#Montserrat-Regular-65" x="56.299988"/>      <use xlink:href="#Montserrat-Regular-6e" x="116.699982"/>      <use xlink:href="#Montserrat-Regular-74" x="184.399963"/>      <use xlink:href="#Montserrat-Regular-69" x="224.999954"/>      <use xlink:href="#Montserrat-Regular-70" x="251.899948"/>      <use xlink:href="#Montserrat-Regular-61" x="319.699936"/>      <use xlink:href="#Montserrat-Regular-77" x="378.699921"/>      <use xlink:href="#Montserrat-Regular-6e" x="466.599915"/>      <use xlink:href="#Montserrat-Regular-20" x="534.299896"/>      <use xlink:href="#Montserrat-Regular-64" x="560.499893"/>      <use xlink:href="#Montserrat-Regular-69" x="628.299881"/>      <use xlink:href="#Montserrat-Regular-66" x="655.199875"/>      <use xlink:href="#Montserrat-Regular-66" x="689.099869"/>      <use xlink:href="#Montserrat-Regular-65" x="722.999863"/>      <use xlink:href="#Montserrat-Regular-72" x="783.399857"/>      <use xlink:href="#Montserrat-Regular-65" x="823.499847"/>      <use xlink:href="#Montserrat-Regular-6e" x="883.899841"/>      <use xlink:href="#Montserrat-Regular-63" x="951.599823"/>      <use xlink:href="#Montserrat-Regular-65" x="1007.899811"/>     </g>    </g>   </g>   <g id="line2d_14">    <path d="M 54 127.26 L 57.348 122.231721 L 60.696 115.527349 L 64.044 115.720744 L 67.392 97.412651 L 70.74 94.189395 L 74.088 101.087163 L 77.436 95.929953 L 80.784 98.186233 L 84.132 94.640651 L 87.48 95.156372 L 90.828 87.033767 L 94.176 82.198884 L 97.524 82.263349 L 100.872 94.25386 L 104.22 91.675256 L 107.568 100.829302 L 110.916 101.860744 L 114.264 101.022698 L 117.612 86.969302 L 120.96 88.32307 L 124.308 82.714605 L 127.656 87.033767 L 131.004 91.868651 L 134.352 98.444093 L 137.7 140.088558 L 141.048 138.863721 L 144.396 156.720558 L 147.744 138.863721 L 151.092 154.077488 L 154.44 154.077488 L 157.788 163.360465 L 161.136 158.203256 L 164.484 223.893209 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#padec282533)" style="fill: none; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_15">    <path d="M 54 127.26 L 57.348 123.069767 L 60.696 117.267907 L 64.044 113.52893 L 67.392 116.558791 L 70.74 119.975442 L 74.088 129.000558 L 77.436 130.225395 L 80.784 132.804 L 84.132 134.802419 L 87.48 129.193953 L 90.828 131.579163 L 94.176 127.38893 L 97.524 134.609023 L 100.872 135.31814 L 104.22 155.689116 L 107.568 166.519256 L 110.916 160.008279 L 114.264 161.877767 L 117.612 162.715814 L 120.96 151.305488 L 124.308 164.843163 L 127.656 159.879349 L 131.004 158.332186 L 134.352 168.904465 L 137.7 168.066419 L 141.048 168.904465 L 144.396 174.448465 L 147.744 174.448465 L 151.092 152.401395 L 154.44 181.990884 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#padec282533)" style="fill: none; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_16">    <path d="M 54 127.26 L 57.348 123.198698 L 60.696 124.294605 L 64.044 121.780465 L 67.392 122.554047 L 70.74 124.165674 L 74.088 122.747442 L 77.436 122.038326 L 80.784 114.238047 L 84.132 116.42986 L 87.48 114.560372 L 90.828 116.494326 L 94.176 110.692465 L 97.524 120.104372 L 100.872 120.168837 L 104.22 119.459721 L 107.568 120.684558 L 110.916 125.648372 L 114.264 126.099628 L 117.612 126.615349 L 120.96 124.681395 L 124.308 121.135814 L 127.656 124.036744 L 131.004 130.870047 L 134.352 147.824372 L 137.7 151.047628 L 141.048 144.923442 L 144.396 160.78186 L 147.744 164.649767 L 151.092 174.577395 L 154.44 173.288093 L 157.788 163.618326 L 161.136 159.492558 L 164.484 184.956279 L 167.832 201.201488 L 171.18 165.487814 L 174.528 180.830512 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#padec282533)" style="fill: none; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_17">    <path d="M 54 127.26 L 57.348 124.552465 L 60.696 125.132651 L 64.044 119.782047 L 67.392 119.008465 L 70.74 154.72214 L 74.088 148.017767 L 77.436 145.374698 L 80.784 142.538233 L 84.132 148.726884 L 87.48 148.662419 L 90.828 143.76307 L 94.176 141.571256 L 97.524 133.899907 L 100.872 144.407721 L 104.22 142.409302 L 107.568 129.967535 L 110.916 142.731628 L 114.264 153.239442 L 117.612 154.013023 L 120.96 152.401395 L 124.308 154.657674 L 127.656 174.51293 L 131.004 149.887256 L 134.352 158.525581 L 137.7 157.62307 L 141.048 150.080651 L 144.396 156.527163 L 147.744 153.239442 L 151.092 156.656093 L 154.44 157.171814 L 157.788 164.391907 L 161.136 168.388744 L 164.484 169.291256 L 167.832 167.615163 L 171.18 169.871442 L 174.528 169.162326 L 177.876 171.031814 L 181.224 172.901302 L 184.572 171.289674 L 187.92 172.643442 L 191.268 173.932744 L 194.616 174.964186 L 197.964 178.638698 L 201.312 179.992465 L 204.66 182.893395 L 208.008 189.146512 L 211.356 182.635535 L 214.704 184.569488 L 218.052 177.02707 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#padec282533)" style="fill: none; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_18">    <path d="M 54 127.26 L 57.348 120.104372 L 60.696 124.488 L 64.044 120.491163 L 67.392 113.593395 L 70.74 120.297767 L 74.088 129.387349 L 77.436 127.38893 L 80.784 131.450233 L 84.132 131.708093 L 87.48 132.094884 L 90.828 132.54614 L 94.176 127.904651 L 97.524 127.26 L 100.872 129.451814 L 104.22 130.032 L 107.568 123.649953 L 110.916 137.767814 L 114.264 132.868465 L 117.612 152.33693 L 120.96 152.659256 L 124.308 148.662419 L 127.656 183.989302 L 131.004 177.994047 L 134.352 173.288093 L 137.7 168.259814 L 141.048 160.072744 L 144.396 161.426512 L 147.744 175.544372 L 151.092 176.446884 L 154.44 175.286512 L 157.788 184.376093 L 161.136 178.187442 L 164.484 169.678047 L 167.832 223.828744 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#padec282533)" style="fill: none; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_19">    <path d="M 54 127.26 L 57.348 122.554047 L 60.696 115.527349 L 64.044 108.242791 L 67.392 107.59814 L 70.74 95.543163 L 74.088 90.772744 L 77.436 85.357674 L 80.784 82.005488 L 84.132 82.77907 L 87.48 74.205209 L 90.828 80.329395 L 94.176 75.881302 L 97.524 87.291628 L 100.872 94.25386 L 104.22 87.485023 L 107.568 84.584093 L 110.916 67.823163 L 114.264 72.851442 L 117.612 97.02586 L 120.96 95.607628 L 124.308 97.090326 L 127.656 105.084 L 131.004 106.566698 L 134.352 103.407907 L 137.7 102.312 L 141.048 103.601302 L 144.396 125.712837 L 147.744 150.531907 L 151.092 142.796093 L 154.44 149.17814 L 157.788 155.624651 L 161.136 157.236279 L 164.484 148.404558 L 167.832 159.299163 L 171.18 161.104186 L 174.528 173.932744 L 177.876 169.871442 L 181.224 171.48307 L 184.572 174.12614 L 187.92 175.931163 L 191.268 223.893209 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#padec282533)" style="fill: none; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_20">    <path d="M 54 127.26 L 57.348 123.069767 L 60.696 119.717581 L 64.044 122.618512 L 67.392 117.203442 L 70.74 119.459721 L 74.088 120.362233 L 77.436 121.651535 L 80.784 118.944 L 84.132 120.555628 L 87.48 116.365395 L 90.828 128.871628 L 94.176 144.923442 L 97.524 145.310233 L 100.872 144.407721 L 104.22 145.568093 L 107.568 145.954884 L 110.916 149.951721 L 114.264 120.426698 L 117.612 155.431256 L 120.96 157.107349 L 124.308 160.26614 L 127.656 162.780279 L 131.004 159.879349 L 134.352 162.071163 L 137.7 168.130884 L 141.048 171.547535 L 144.396 168.388744 L 147.744 176.060093 L 151.092 177.607256 L 154.44 175.608837 L 157.788 163.682791 L 161.136 182.055349 L 164.484 181.281767 L 167.832 178.380837 L 171.18 178.187442 L 174.528 179.992465 L 177.876 174.51293 L 181.224 181.023907 L 184.572 185.987721 L 187.92 181.604093 L 191.268 185.72986 L 194.616 187.083628 L 197.964 177.542791 L 201.312 180.508186 L 204.66 223.893209 L 208.008 185.858791 L 211.356 223.893209 L 214.704 223.828744 L 218.052 223.893209 L 221.4 183.280186 L 224.748 223.893209 L 228.096 223.893209 L 231.444 223.893209 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#padec282533)" style="fill: none; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_21">    <path d="M 54 127.26 L 57.348 120.039907 L 60.696 118.621674 L 64.044 121.716 L 67.392 118.234884 L 70.74 115.205023 L 74.088 116.365395 L 77.436 116.881116 L 80.784 85.164279 L 84.132 74.978791 L 87.48 73.882884 L 90.828 71.304279 L 94.176 77.815256 L 97.524 75.881302 L 100.872 80.071535 L 104.22 84.970884 L 107.568 81.683163 L 110.916 79.749209 L 114.264 83.16586 L 117.612 74.785395 L 120.96 97.992837 L 124.308 70.143907 L 127.656 80.00707 L 131.004 75.301116 L 134.352 66.276 L 137.7 71.755535 L 141.048 72.786977 L 144.396 87.227163 L 147.744 80.845116 L 151.092 78.395442 L 154.44 76.397023 L 157.788 75.623442 L 161.136 74.011814 L 164.484 223.893209 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#padec282533)" style="fill: none; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_22">    <path d="M 54 127.26 L 57.348 125.003721 L 60.696 123.907814 L 64.044 123.069767 L 67.392 116.945581 L 70.74 118.621674 L 74.088 120.620093 L 77.436 120.620093 L 80.784 113.65786 L 84.132 115.269488 L 87.48 114.753767 L 90.828 111.981767 L 94.176 88.838791 L 97.524 93.802605 L 100.872 94.318326 L 104.22 99.79786 L 107.568 93.673674 L 110.916 97.090326 L 114.264 101.989674 L 117.612 87.162698 L 120.96 90.385953 L 124.308 79.813674 L 127.656 89.096651 L 131.004 97.799442 L 134.352 116.945581 L 137.7 144.278791 L 141.048 144.085395 L 144.396 138.348 L 147.744 145.116837 L 151.092 137.832279 L 154.44 155.431256 L 157.788 159.041302 L 161.136 153.755163 L 164.484 171.289674 L 167.832 178.767628 L 171.18 168.84 L 174.528 177.220465 L 177.876 174.899721 L 181.224 170.451628 L 184.572 170.193767 L 187.92 177.607256 L 191.268 181.281767 L 194.616 187.277023 L 197.964 194.110326 L 201.312 195.206233 L 204.66 204.166884 L 208.008 215.254884 L 211.356 204.231349 L 214.704 201.330419 L 218.052 222.152651 L 221.4 221.508 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#padec282533)" style="fill: none; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_23">    <path d="M 54 127.26 L 57.348 123.327628 L 60.696 121.006884 L 64.044 120.297767 L 67.392 116.107535 L 70.74 127.775721 L 74.088 114.173581 L 77.436 115.591814 L 80.784 117.138977 L 84.132 116.752186 L 87.48 117.267907 L 90.828 223.957674 L 94.176 223.957674 L 97.524 223.957674 L 100.872 223.957674 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#padec282533)" style="fill: none; stroke: #ff7f0e; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_24">    <path d="M 54 127.26 L 57.348 123.069767 L 60.696 120.362233 L 64.044 120.394465 L 67.392 116.752186 L 70.74 119.717581 L 74.088 120.491163 L 77.436 121.135814 L 80.784 115.688512 L 84.132 116.591023 L 87.48 115.559581 L 90.828 122.682977 L 94.176 119.040698 L 97.524 123.682186 L 100.872 124.810326 L 104.22 124.74586 L 107.568 122.167256 L 110.916 131.708093 L 114.264 123.263163 L 117.612 139.47614 L 120.96 137.993442 L 124.308 134.899116 L 127.656 141.958047 L 131.004 140.378651 L 134.352 153.174977 L 137.7 154.335349 L 141.048 147.502047 L 144.396 158.751209 L 147.744 158.944605 L 151.092 155.366791 L 154.44 165.229953 L 157.788 163.650558 L 161.136 163.940651 L 164.484 183.119023 L 167.832 212.515116 L 171.18 201.072558 L 174.528 202.394093 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#padec282533)" style="fill: none; stroke: #000000; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="patch_3">    <path d="M 54 224.28 L 54 30.24 " style="fill: none"/>   </g>   <g id="patch_4">    <path d="M 388.8 224.28 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="patch_5">    <path d="M 54 224.28 L 388.8 224.28 " style="fill: none"/>   </g>   <g id="patch_6">    <path d="M 54 30.24 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="text_16">    <g id="patch_7">     <a xlink:href="https://dynomight.net/chess">      <path d="M 323.8664 225.676606 L 388.881244 225.676606 L 388.881244 213.343012 L 323.8664 213.343012 z" style="fill: #ffffff; opacity: 0.000001; stroke: #ffffff; stroke-linejoin: miter"/>     </a>    </g>    <a xlink:href="https://dynomight.net/chess">     <!-- dynomight.net/chess -->     <g style="opacity: 0.3" transform="translate(327.8664 220.734419) scale(0.05 -0.05)">      <defs>       <path id="CourierNewPSMT-64" d="M 3206 3922 L 3206 263 L 3556 263 Q 3653 263 3693 227 Q 3734 191 3734 131 Q 3734 75 3693 37 Q 3653 0 3556 0 L 2941 0 L 2941 522 Q 2484 -103 1781 -103 Q 1425 -103 1098 86 Q 772 275 583 625 Q 394 975 394 1347 Q 394 1722 583 2070 Q 772 2419 1098 2609 Q 1425 2800 1784 2800 Q 2472 2800 2941 2175 L 2941 3656 L 2591 3656 Q 2494 3656 2453 3693 Q 2413 3731 2413 3791 Q 2413 3847 2453 3884 Q 2494 3922 2591 3922 L 3206 3922 zM 2941 1347 Q 2941 1847 2603 2190 Q 2266 2534 1800 2534 Q 1331 2534 993 2190 Q 656 1847 656 1347 Q 656 850 993 504 Q 1331 159 1800 159 Q 2266 159 2603 504 Q 2941 850 2941 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-79" d="M 1938 0 L 719 2441 L 641 2441 Q 547 2441 506 2478 Q 466 2516 466 2572 Q 466 2613 486 2645 Q 506 2678 539 2692 Q 572 2706 641 2706 L 1359 2706 Q 1453 2706 1493 2668 Q 1534 2631 1534 2572 Q 1534 2516 1493 2478 Q 1453 2441 1359 2441 L 1006 2441 L 2078 288 L 3134 2441 L 2781 2441 Q 2688 2441 2647 2478 Q 2606 2516 2606 2575 Q 2606 2631 2647 2668 Q 2688 2706 2781 2706 L 3497 2706 Q 3594 2706 3634 2668 Q 3675 2631 3675 2572 Q 3675 2531 3650 2497 Q 3625 2463 3594 2452 Q 3563 2441 3419 2441 L 1756 -944 L 2166 -944 Q 2259 -944 2300 -980 Q 2341 -1016 2341 -1075 Q 2341 -1131 2300 -1168 Q 2259 -1206 2166 -1206 L 653 -1206 Q 559 -1206 518 -1170 Q 478 -1134 478 -1075 Q 478 -1016 518 -980 Q 559 -944 653 -944 L 1475 -944 L 1938 0 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6e" d="M 1072 2706 L 1072 2309 Q 1347 2588 1569 2694 Q 1791 2800 2069 2800 Q 2369 2800 2616 2672 Q 2791 2578 2933 2361 Q 3075 2144 3075 1916 L 3075 263 L 3297 263 Q 3391 263 3431 227 Q 3472 191 3472 131 Q 3472 75 3431 37 Q 3391 0 3297 0 L 2594 0 Q 2497 0 2456 37 Q 2416 75 2416 131 Q 2416 191 2456 227 Q 2497 263 2594 263 L 2813 263 L 2813 1872 Q 2813 2150 2609 2342 Q 2406 2534 2066 2534 Q 1806 2534 1615 2429 Q 1425 2325 1072 1909 L 1072 263 L 1369 263 Q 1463 263 1503 227 Q 1544 191 1544 131 Q 1544 75 1503 37 Q 1463 0 1369 0 L 513 0 Q 419 0 378 37 Q 338 75 338 131 Q 338 191 378 227 Q 419 263 513 263 L 809 263 L 809 2441 L 588 2441 Q 494 2441 453 2478 Q 413 2516 413 2575 Q 413 2631 453 2668 Q 494 2706 588 2706 L 1072 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6f" d="M 3388 1347 Q 3388 747 2958 322 Q 2528 -103 1922 -103 Q 1309 -103 881 323 Q 453 750 453 1347 Q 453 1947 881 2373 Q 1309 2800 1922 2800 Q 2528 2800 2958 2375 Q 3388 1950 3388 1347 zM 3122 1347 Q 3122 1841 2770 2187 Q 2419 2534 1919 2534 Q 1419 2534 1067 2186 Q 716 1838 716 1347 Q 716 859 1067 509 Q 1419 159 1919 159 Q 2419 159 2770 507 Q 3122 856 3122 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6d" d="M 725 2706 L 725 2441 Q 1059 2800 1397 2800 Q 1600 2800 1753 2692 Q 1906 2584 2009 2366 Q 2184 2584 2364 2692 Q 2544 2800 2725 2800 Q 3009 2800 3178 2616 Q 3400 2378 3400 2097 L 3400 263 L 3622 263 Q 3716 263 3756 227 Q 3797 191 3797 131 Q 3797 75 3756 37 Q 3716 0 3622 0 L 3138 0 L 3138 2072 Q 3138 2272 3016 2403 Q 2894 2534 2734 2534 Q 2591 2534 2431 2426 Q 2272 2319 2069 2003 L 2069 263 L 2288 263 Q 2381 263 2422 227 Q 2463 191 2463 131 Q 2463 75 2422 37 Q 2381 0 2288 0 L 1803 0 L 1803 2053 Q 1803 2263 1679 2398 Q 1556 2534 1403 2534 Q 1263 2534 1125 2444 Q 934 2316 725 2003 L 725 263 L 947 263 Q 1041 263 1081 227 Q 1122 191 1122 131 Q 1122 75 1081 37 Q 1041 0 947 0 L 241 0 Q 147 0 106 37 Q 66 75 66 131 Q 66 191 106 227 Q 147 263 241 263 L 463 263 L 463 2441 L 241 2441 Q 147 2441 106 2478 Q 66 2516 66 2575 Q 66 2631 106 2668 Q 147 2706 241 2706 L 725 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-69" d="M 2050 4053 L 2050 3375 L 1663 3375 L 1663 4053 L 2050 4053 zM 2059 2706 L 2059 263 L 3091 263 Q 3188 263 3228 227 Q 3269 191 3269 131 Q 3269 75 3228 37 Q 3188 0 3091 0 L 766 0 Q 672 0 631 37 Q 591 75 591 131 Q 591 191 631 227 Q 672 263 766 263 L 1797 263 L 1797 2441 L 1031 2441 Q 938 2441 895 2478 Q 853 2516 853 2572 Q 853 2631 893 2668 Q 934 2706 1031 2706 L 2059 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-67" d="M 2803 2225 L 2803 2706 L 3419 2706 Q 3513 2706 3553 2668 Q 3594 2631 3594 2572 Q 3594 2516 3553 2478 Q 3513 2441 3419 2441 L 3066 2441 L 3066 -181 Q 3066 -444 2953 -650 Q 2878 -788 2703 -934 Q 2528 -1081 2384 -1143 Q 2241 -1206 2000 -1206 L 1256 -1206 Q 1163 -1206 1122 -1170 Q 1081 -1134 1081 -1075 Q 1081 -1016 1122 -978 Q 1163 -941 1256 -941 L 2009 -944 Q 2241 -944 2427 -825 Q 2613 -706 2734 -478 Q 2803 -347 2803 -144 L 2803 644 Q 2381 66 1716 66 Q 1175 66 783 467 Q 391 869 391 1434 Q 391 2000 783 2400 Q 1175 2800 1716 2800 Q 2381 2800 2803 2225 zM 2803 1434 Q 2803 1897 2486 2215 Q 2169 2534 1728 2534 Q 1288 2534 972 2214 Q 656 1894 656 1434 Q 656 972 972 651 Q 1288 331 1728 331 Q 2169 331 2486 651 Q 2803 972 2803 1434 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-68" d="M 1056 3922 L 1056 2306 Q 1306 2578 1536 2689 Q 1766 2800 2050 2800 Q 2356 2800 2570 2692 Q 2784 2584 2928 2361 Q 3072 2138 3072 1891 L 3072 263 L 3366 263 Q 3463 263 3502 227 Q 3541 191 3541 131 Q 3541 75 3502 37 Q 3463 0 3366 0 L 2513 0 Q 2416 0 2375 37 Q 2334 75 2334 131 Q 2334 191 2375 227 Q 2416 263 2513 263 L 2806 263 L 2806 1872 Q 2806 2156 2601 2347 Q 2397 2538 2031 2538 Q 1744 2538 1541 2397 Q 1394 2297 1056 1925 L 1056 263 L 1353 263 Q 1447 263 1487 227 Q 1528 191 1528 131 Q 1528 75 1487 37 Q 1447 0 1353 0 L 497 0 Q 403 0 362 37 Q 322 75 322 131 Q 322 191 362 227 Q 403 263 497 263 L 794 263 L 794 3656 L 441 3656 Q 347 3656 306 3693 Q 266 3731 266 3791 Q 266 3847 306 3884 Q 347 3922 441 3922 L 1056 3922 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-74" d="M 1381 2706 L 2806 2706 Q 2900 2706 2940 2668 Q 2981 2631 2981 2572 Q 2981 2516 2940 2478 Q 2900 2441 2806 2441 L 1381 2441 L 1381 694 Q 1381 466 1564 312 Q 1747 159 2100 159 Q 2366 159 2675 239 Q 2984 319 3156 419 Q 3219 459 3259 459 Q 3309 459 3346 420 Q 3384 381 3384 328 Q 3384 281 3344 241 Q 3244 138 2858 17 Q 2472 -103 2119 -103 Q 1659 -103 1387 112 Q 1116 328 1116 694 L 1116 2441 L 631 2441 Q 538 2441 497 2478 Q 456 2516 456 2575 Q 456 2631 497 2668 Q 538 2706 631 2706 L 1116 2706 L 1116 3481 Q 1116 3575 1153 3615 Q 1191 3656 1247 3656 Q 1306 3656 1343 3615 Q 1381 3575 1381 3481 L 1381 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2e" d="M 1866 753 L 1978 753 Q 2153 753 2278 629 Q 2403 506 2403 331 Q 2403 150 2276 28 Q 2150 -94 1978 -94 L 1866 -94 Q 1691 -94 1566 29 Q 1441 153 1441 328 Q 1441 509 1567 631 Q 1694 753 1866 753 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-65" d="M 3341 1291 L 666 1291 Q 734 781 1092 470 Q 1450 159 1978 159 Q 2272 159 2594 256 Q 2916 353 3119 513 Q 3178 559 3222 559 Q 3272 559 3309 520 Q 3347 481 3347 428 Q 3347 375 3297 325 Q 3147 169 2764 33 Q 2381 -103 1978 -103 Q 1303 -103 851 339 Q 400 781 400 1409 Q 400 1981 823 2390 Q 1247 2800 1872 2800 Q 2516 2800 2931 2379 Q 3347 1959 3341 1291 zM 3075 1556 Q 2997 1991 2664 2262 Q 2331 2534 1872 2534 Q 1413 2534 1081 2265 Q 750 1997 669 1556 L 3075 1556 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2f" d="M 3100 4113 L 988 -413 Q 934 -528 850 -528 Q 797 -528 758 -489 Q 719 -450 719 -403 Q 719 -366 750 -300 L 2859 4225 Q 2894 4297 2925 4319 Q 2956 4341 3000 4341 Q 3053 4341 3092 4302 Q 3131 4263 3131 4216 Q 3131 4181 3100 4113 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-63" d="M 3016 2441 L 3016 2528 Q 3016 2625 3053 2665 Q 3091 2706 3147 2706 Q 3206 2706 3243 2665 Q 3281 2625 3281 2528 L 3281 1934 Q 3278 1838 3242 1797 Q 3206 1756 3147 1756 Q 3094 1756 3058 1792 Q 3022 1828 3016 1913 Q 2997 2134 2723 2334 Q 2450 2534 1988 2534 Q 1403 2534 1100 2168 Q 797 1803 797 1331 Q 797 822 1131 490 Q 1466 159 1997 159 Q 2303 159 2620 271 Q 2938 384 3194 634 Q 3259 697 3309 697 Q 3363 697 3398 661 Q 3434 625 3434 572 Q 3434 438 3119 231 Q 2609 -103 1984 -103 Q 1350 -103 942 301 Q 534 706 534 1328 Q 534 1963 951 2381 Q 1369 2800 2003 2800 Q 2606 2800 3016 2441 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-73" d="M 2788 2534 Q 2788 2625 2825 2665 Q 2863 2706 2919 2706 Q 2978 2706 3015 2665 Q 3053 2625 3053 2528 L 3053 2078 Q 3053 1984 3015 1943 Q 2978 1903 2919 1903 Q 2866 1903 2830 1937 Q 2794 1972 2788 2050 Q 2769 2238 2594 2359 Q 2338 2534 1916 2534 Q 1475 2534 1231 2356 Q 1047 2222 1047 2056 Q 1047 1869 1266 1744 Q 1416 1656 1834 1609 Q 2381 1550 2594 1475 Q 2897 1366 3045 1172 Q 3194 978 3194 753 Q 3194 419 2872 158 Q 2550 -103 1928 -103 Q 1306 -103 909 213 Q 909 106 896 75 Q 884 44 851 22 Q 819 0 778 0 Q 722 0 684 40 Q 647 81 647 175 L 647 716 Q 647 809 683 850 Q 719 891 778 891 Q 834 891 873 852 Q 913 813 913 747 Q 913 603 984 506 Q 1094 356 1333 257 Q 1572 159 1919 159 Q 2431 159 2681 350 Q 2931 541 2931 753 Q 2931 997 2678 1144 Q 2422 1291 1933 1341 Q 1444 1391 1231 1472 Q 1019 1553 900 1715 Q 781 1878 781 2066 Q 781 2403 1112 2601 Q 1444 2800 1903 2800 Q 2447 2800 2788 2534 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#CourierNewPSMT-64"/>      <use xlink:href="#CourierNewPSMT-79" x="60.009766"/>      <use xlink:href="#CourierNewPSMT-6e" x="120.019531"/>      <use xlink:href="#CourierNewPSMT-6f" x="180.029297"/>      <use xlink:href="#CourierNewPSMT-6d" x="240.039062"/>      <use xlink:href="#CourierNewPSMT-69" x="300.048828"/>      <use xlink:href="#CourierNewPSMT-67" x="360.058594"/>      <use xlink:href="#CourierNewPSMT-68" x="420.068359"/>      <use xlink:href="#CourierNewPSMT-74" x="480.078125"/>      <use xlink:href="#CourierNewPSMT-2e" x="540.087891"/>      <use xlink:href="#CourierNewPSMT-6e" x="600.097656"/>      <use xlink:href="#CourierNewPSMT-65" x="660.107422"/>      <use xlink:href="#CourierNewPSMT-74" x="720.117188"/>      <use xlink:href="#CourierNewPSMT-2f" x="780.126953"/>      <use xlink:href="#CourierNewPSMT-63" x="840.136719"/>      <use xlink:href="#CourierNewPSMT-68" x="900.146484"/>      <use xlink:href="#CourierNewPSMT-65" x="960.15625"/>      <use xlink:href="#CourierNewPSMT-73" x="1020.166016"/>      <use xlink:href="#CourierNewPSMT-73" x="1080.175781"/>     </g>    </a>   </g>   <g id="text_17">    <!-- gpt-4o vs stockfish -->    <g transform="translate(164.669062 24.24) scale(0.12 -0.12)">     <defs>      <path id="Montserrat-Regular-67" d="M 3750 3366 L 3750 410 Q 3750 -448 3331 -861 Q 2912 -1274 2067 -1274 Q 1600 -1274 1181 -1136 Q 762 -998 499 -755 L 730 -410 Q 973 -627 1321 -748 Q 1670 -870 2054 -870 Q 2694 -870 2995 -572 Q 3296 -275 3296 352 L 3296 781 Q 3085 461 2742 294 Q 2400 128 1984 128 Q 1510 128 1123 336 Q 736 544 515 918 Q 294 1293 294 1766 Q 294 2240 515 2611 Q 736 2982 1120 3187 Q 1504 3392 1984 3392 Q 2413 3392 2758 3219 Q 3104 3046 3315 2720 L 3315 3366 L 3750 3366 zM 2029 531 Q 2394 531 2688 688 Q 2982 845 3145 1126 Q 3309 1408 3309 1766 Q 3309 2125 3145 2403 Q 2982 2682 2691 2838 Q 2400 2995 2029 2995 Q 1664 2995 1373 2841 Q 1082 2688 918 2406 Q 755 2125 755 1766 Q 755 1408 918 1126 Q 1082 845 1373 688 Q 1664 531 2029 531 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6f" d="M 2010 -32 Q 1523 -32 1132 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1132 3174 Q 1523 3392 2010 3392 Q 2496 3392 2886 3174 Q 3277 2957 3497 2566 Q 3718 2176 3718 1683 Q 3718 1190 3497 800 Q 3277 410 2886 189 Q 2496 -32 2010 -32 zM 2010 371 Q 2368 371 2653 534 Q 2938 698 3098 998 Q 3258 1299 3258 1683 Q 3258 2067 3098 2368 Q 2938 2669 2653 2832 Q 2368 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-76" d="M 3462 3366 L 1965 0 L 1498 0 L 6 3366 L 480 3366 L 1741 499 L 3014 3366 L 3462 3366 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6b" d="M 1882 1690 L 1088 960 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 1530 L 3098 3366 L 3661 3366 L 2221 1990 L 3795 0 L 3238 0 L 1882 1690 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-68" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2778 Q 1274 3072 1600 3232 Q 1926 3392 2349 3392 z" transform="scale(0.015625)"/>     </defs>     <use xlink:href="#Montserrat-Regular-67"/>     <use xlink:href="#Montserrat-Regular-70" x="68.599991"/>     <use xlink:href="#Montserrat-Regular-74" x="136.399979"/>     <use xlink:href="#Montserrat-Regular-2d" x="176.999969"/>     <use xlink:href="#Montserrat-Regular-34" x="215.199966"/>     <use xlink:href="#Montserrat-Regular-6f" x="281.299957"/>     <use xlink:href="#Montserrat-Regular-20" x="343.999954"/>     <use xlink:href="#Montserrat-Regular-76" x="370.199951"/>     <use xlink:href="#Montserrat-Regular-73" x="424.399948"/>     <use xlink:href="#Montserrat-Regular-20" x="473.299942"/>     <use xlink:href="#Montserrat-Regular-73" x="499.499939"/>     <use xlink:href="#Montserrat-Regular-74" x="548.399933"/>     <use xlink:href="#Montserrat-Regular-6f" x="588.999924"/>     <use xlink:href="#Montserrat-Regular-63" x="651.699921"/>     <use xlink:href="#Montserrat-Regular-6b" x="707.999908"/>     <use xlink:href="#Montserrat-Regular-66" x="768.099899"/>     <use xlink:href="#Montserrat-Regular-69" x="801.999893"/>     <use xlink:href="#Montserrat-Regular-73" x="828.899887"/>     <use xlink:href="#Montserrat-Regular-68" x="877.799881"/>    </g>   </g>  </g> </g> <defs>  <clipPath id="padec282533">   <rect x="54" y="30.24" width="334.8" height="194.04"/>  </clipPath> </defs></svg>](https://dynomight.net/img/chess/gpt-4o_stockfish.pdf)

[Terrible](https://lichess.org/govbFo94).

It lost every single game, though it lost slightly slower.

Finally, I tried `o1-mini`, a model that’s supposed to be able to solve complex tasks. (I’m too poor for `o1`.)

[<?xml version="1.0" encoding="utf-8" standalone="no"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"  "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg xmlns:xlink="http://www.w3.org/1999/xlink" width="432pt" height="252pt" viewBox="0 0 432 252" xmlns="http://www.w3.org/2000/svg" version="1.1"> <metadata>  <rdf:RDF xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:cc="http://creativecommons.org/ns#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">   <cc:Work>    <dc:type rdf:resource="http://purl.org/dc/dcmitype/StillImage"/>    <dc:date>2024-11-14T11:33:21.659140</dc:date>    <dc:format>image/svg+xml</dc:format>    <dc:creator>     <cc:Agent>      <dc:title>Matplotlib v3.8.4, https://matplotlib.org/</dc:title>     </cc:Agent>    </dc:creator>   </cc:Work>  </rdf:RDF> </metadata> <defs>  <style type="text/css">*{stroke-linejoin: round; stroke-linecap: butt}</style> </defs> <g id="figure_1">  <g id="patch_1">   <path d="M 0 252 L 432 252 L 432 0 L 0 0 z" style="fill: #ffffff"/>  </g>  <g id="axes_1">   <g id="patch_2">    <path d="M 54 224.28 L 388.8 224.28 L 388.8 30.24 L 54 30.24 z" style="fill: #ffffff"/>   </g>   <g id="matplotlib.axis_1">    <g id="xtick_1">     <g id="line2d_1"/>     <g id="text_1">      <!-- 0 -->      <g transform="translate(50.689844 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-30" d="M 2118 -38 Q 1600 -38 1193 237 Q 787 512 560 1027 Q 333 1542 333 2240 Q 333 2938 560 3453 Q 787 3968 1193 4243 Q 1600 4518 2118 4518 Q 2637 4518 3043 4243 Q 3450 3968 3680 3453 Q 3910 2938 3910 2240 Q 3910 1542 3680 1027 Q 3450 512 3043 237 Q 2637 -38 2118 -38 zM 2118 384 Q 2509 384 2806 601 Q 3104 819 3270 1235 Q 3437 1651 3437 2240 Q 3437 2829 3270 3245 Q 3104 3661 2806 3878 Q 2509 4096 2118 4096 Q 1728 4096 1430 3878 Q 1133 3661 966 3245 Q 800 2829 800 2240 Q 800 1651 966 1235 Q 1133 819 1430 601 Q 1728 384 2118 384 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="xtick_2">     <g id="line2d_2"/>     <g id="text_2">      <!-- 20 -->      <g transform="translate(114.81 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-32" d="M 3392 410 L 3392 0 L 250 0 L 250 326 L 2112 2150 Q 2464 2496 2589 2749 Q 2714 3002 2714 3258 Q 2714 3654 2442 3875 Q 2170 4096 1664 4096 Q 877 4096 442 3597 L 115 3878 Q 378 4186 784 4352 Q 1190 4518 1702 4518 Q 2387 4518 2787 4195 Q 3187 3872 3187 3309 Q 3187 2963 3033 2643 Q 2880 2323 2451 1907 L 928 410 L 3392 410 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-32"/>       <use xlink:href="#Montserrat-Regular-30" x="56.799988"/>      </g>     </g>    </g>    <g id="xtick_3">     <g id="line2d_3"/>     <g id="text_3">      <!-- 40 -->      <g transform="translate(181.305156 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-34" d="M 4141 1178 L 3245 1178 L 3245 0 L 2784 0 L 2784 1178 L 256 1178 L 256 1510 L 2630 4480 L 3142 4480 L 851 1587 L 2797 1587 L 2797 2624 L 3245 2624 L 3245 1587 L 4141 1587 L 4141 1178 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-34"/>       <use xlink:href="#Montserrat-Regular-30" x="66.099991"/>      </g>     </g>    </g>    <g id="xtick_4">     <g id="line2d_4"/>     <g id="text_4">      <!-- 60 -->      <g transform="translate(248.524531 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-36" d="M 2189 2682 Q 2630 2682 2972 2515 Q 3315 2349 3507 2045 Q 3699 1741 3699 1338 Q 3699 928 3500 614 Q 3302 301 2953 131 Q 2605 -38 2170 -38 Q 1280 -38 806 547 Q 333 1133 333 2202 Q 333 2950 589 3468 Q 845 3987 1305 4252 Q 1766 4518 2381 4518 Q 3046 4518 3443 4275 L 3258 3904 Q 2938 4115 2387 4115 Q 1645 4115 1222 3644 Q 800 3174 800 2278 Q 800 2118 819 1914 Q 986 2278 1350 2480 Q 1715 2682 2189 2682 zM 2150 352 Q 2637 352 2941 617 Q 3245 883 3245 1325 Q 3245 1766 2941 2028 Q 2637 2291 2118 2291 Q 1779 2291 1520 2163 Q 1261 2035 1113 1811 Q 966 1587 966 1312 Q 966 1056 1104 838 Q 1242 621 1510 486 Q 1779 352 2150 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-36"/>       <use xlink:href="#Montserrat-Regular-30" x="60.899994"/>      </g>     </g>    </g>    <g id="xtick_5">     <g id="line2d_5"/>     <g id="text_5">      <!-- 80 -->      <g transform="translate(315.34 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-38" d="M 2899 2349 Q 3322 2208 3549 1926 Q 3776 1645 3776 1248 Q 3776 858 3565 566 Q 3354 275 2960 118 Q 2566 -38 2035 -38 Q 1229 -38 768 307 Q 307 653 307 1248 Q 307 1651 528 1929 Q 749 2208 1171 2349 Q 826 2483 646 2736 Q 467 2989 467 3334 Q 467 3878 892 4198 Q 1318 4518 2035 4518 Q 2509 4518 2864 4374 Q 3219 4230 3417 3961 Q 3616 3693 3616 3334 Q 3616 2989 3430 2736 Q 3245 2483 2899 2349 zM 928 3322 Q 928 2950 1222 2732 Q 1517 2515 2035 2515 Q 2554 2515 2851 2732 Q 3149 2950 3149 3315 Q 3149 3693 2845 3910 Q 2541 4128 2035 4128 Q 1523 4128 1225 3910 Q 928 3693 928 3322 zM 2035 352 Q 2630 352 2966 595 Q 3302 838 3302 1254 Q 3302 1670 2966 1910 Q 2630 2150 2035 2150 Q 1446 2150 1110 1910 Q 774 1670 774 1254 Q 774 832 1107 592 Q 1440 352 2035 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-38"/>       <use xlink:href="#Montserrat-Regular-30" x="63.799988"/>      </g>     </g>    </g>    <g id="xtick_6">     <g id="line2d_6"/>     <g id="text_6">      <!-- 100 -->      <g transform="translate(380.375 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-31" d="M 1574 4480 L 1574 0 L 1114 0 L 1114 4070 L 58 4070 L 58 4480 L 1574 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>      </g>     </g>    </g>    <g id="text_7">     <!-- turns -->     <g transform="translate(208.170312 248.56125) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-74" d="M 2406 205 Q 2278 90 2089 29 Q 1901 -32 1696 -32 Q 1222 -32 966 224 Q 710 480 710 947 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 4102 L 1165 4102 L 1165 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 973 Q 1165 672 1315 515 Q 1466 358 1747 358 Q 1888 358 2019 403 Q 2150 448 2246 531 L 2406 205 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-75" d="M 3667 3366 L 3667 0 L 3232 0 L 3232 614 Q 3053 307 2739 137 Q 2426 -32 2022 -32 Q 1363 -32 982 336 Q 602 704 602 1414 L 602 3366 L 1056 3366 L 1056 1459 Q 1056 928 1318 653 Q 1581 378 2067 378 Q 2598 378 2905 701 Q 3213 1024 3213 1600 L 3213 3366 L 3667 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-72" d="M 1069 2707 Q 1229 3046 1545 3219 Q 1862 3392 2330 3392 L 2330 2950 L 2221 2957 Q 1690 2957 1389 2630 Q 1088 2304 1088 1715 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2707 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-6e" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2746 Q 1254 3053 1584 3222 Q 1914 3392 2349 3392 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-73" d="M 1536 -32 Q 1126 -32 752 86 Q 378 205 166 384 L 371 742 Q 582 576 902 470 Q 1222 365 1568 365 Q 2029 365 2249 509 Q 2470 653 2470 915 Q 2470 1101 2348 1206 Q 2227 1312 2041 1366 Q 1856 1421 1549 1472 Q 1139 1549 889 1629 Q 640 1709 464 1901 Q 288 2093 288 2432 Q 288 2854 640 3123 Q 992 3392 1619 3392 Q 1946 3392 2272 3305 Q 2598 3219 2810 3078 L 2611 2714 Q 2195 3002 1619 3002 Q 1184 3002 963 2848 Q 742 2694 742 2445 Q 742 2253 867 2137 Q 992 2022 1177 1968 Q 1363 1914 1690 1856 Q 2093 1779 2336 1702 Q 2579 1626 2752 1440 Q 2925 1254 2925 928 Q 2925 486 2557 227 Q 2189 -32 1536 -32 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-74"/>      <use xlink:href="#Montserrat-Regular-75" x="40.599991"/>      <use xlink:href="#Montserrat-Regular-72" x="107.899979"/>      <use xlink:href="#Montserrat-Regular-6e" x="147.999969"/>      <use xlink:href="#Montserrat-Regular-73" x="215.699951"/>     </g>    </g>   </g>   <g id="matplotlib.axis_2">    <g id="ytick_1">     <g id="line2d_7"/>     <g id="text_8">      <!-- LOSS -->      <g transform="translate(23.91875 227.667831) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-4c" d="M 730 4480 L 1203 4480 L 1203 410 L 3712 410 L 3712 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4f" d="M 2688 -38 Q 2022 -38 1481 259 Q 941 557 637 1075 Q 333 1594 333 2240 Q 333 2886 637 3404 Q 941 3923 1481 4220 Q 2022 4518 2688 4518 Q 3354 4518 3888 4224 Q 4422 3930 4729 3408 Q 5037 2886 5037 2240 Q 5037 1594 4729 1072 Q 4422 550 3888 256 Q 3354 -38 2688 -38 zM 2688 384 Q 3219 384 3648 624 Q 4077 864 4320 1289 Q 4563 1715 4563 2240 Q 4563 2765 4320 3190 Q 4077 3616 3648 3856 Q 3219 4096 2688 4096 Q 2157 4096 1725 3856 Q 1293 3616 1046 3190 Q 800 2765 800 2240 Q 800 1715 1046 1289 Q 1293 864 1725 624 Q 2157 384 2688 384 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-53" d="M 1965 -38 Q 1466 -38 1008 122 Q 550 282 301 544 L 486 909 Q 730 666 1133 515 Q 1536 365 1965 365 Q 2566 365 2867 585 Q 3168 806 3168 1158 Q 3168 1427 3005 1587 Q 2842 1747 2605 1833 Q 2368 1920 1946 2022 Q 1440 2150 1139 2268 Q 838 2387 624 2630 Q 410 2874 410 3290 Q 410 3629 589 3907 Q 768 4186 1139 4352 Q 1510 4518 2061 4518 Q 2445 4518 2813 4412 Q 3181 4307 3450 4122 L 3290 3744 Q 3008 3930 2688 4022 Q 2368 4115 2061 4115 Q 1472 4115 1174 3888 Q 877 3661 877 3302 Q 877 3034 1040 2870 Q 1203 2707 1449 2620 Q 1696 2534 2112 2432 Q 2605 2310 2909 2192 Q 3213 2074 3424 1834 Q 3635 1594 3635 1184 Q 3635 845 3452 566 Q 3270 288 2892 125 Q 2515 -38 1965 -38 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-4c"/>       <use xlink:href="#Montserrat-Regular-4f" x="58.899994"/>       <use xlink:href="#Montserrat-Regular-53" x="142.799988"/>       <use xlink:href="#Montserrat-Regular-53" x="204.299973"/>      </g>     </g>    </g>    <g id="ytick_2">     <g id="line2d_8"/>     <g id="text_9">      <!-- -1000 -->      <g transform="translate(23.209375 195.435273) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-2d" d="M 384 1933 L 2061 1933 L 2061 1536 L 384 1536 L 384 1933 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-31" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="74.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="140.499985"/>       <use xlink:href="#Montserrat-Regular-30" x="206.699982"/>      </g>     </g>    </g>    <g id="ytick_3">     <g id="line2d_9"/>     <g id="text_10">      <!-- -500 -->      <g transform="translate(27.779687 163.202714) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-35" d="M 1510 2643 Q 2483 2643 2928 2294 Q 3373 1946 3373 1318 Q 3373 928 3190 621 Q 3008 314 2643 138 Q 2278 -38 1747 -38 Q 1267 -38 835 118 Q 403 275 141 538 L 365 902 Q 589 672 953 528 Q 1318 384 1741 384 Q 2298 384 2602 633 Q 2906 883 2906 1299 Q 2906 1760 2563 1997 Q 2221 2234 1408 2234 L 454 2234 L 685 4480 L 3136 4480 L 3136 4070 L 1088 4070 L 934 2643 L 1510 2643 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-35" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="94.799988"/>       <use xlink:href="#Montserrat-Regular-30" x="160.999985"/>      </g>     </g>    </g>    <g id="ytick_4">     <g id="line2d_10"/>     <g id="text_11">      <!-- 0 -->      <g transform="translate(43.879688 130.970156) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="ytick_5">     <g id="line2d_11"/>     <g id="text_12">      <!-- 500 -->      <g transform="translate(31.6 98.737598) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-35"/>       <use xlink:href="#Montserrat-Regular-30" x="56.599991"/>       <use xlink:href="#Montserrat-Regular-30" x="122.799988"/>      </g>     </g>    </g>    <g id="ytick_6">     <g id="line2d_12"/>     <g id="text_13">      <!-- 1000 -->      <g transform="translate(27.029687 66.50504) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="168.499985"/>      </g>     </g>    </g>    <g id="ytick_7">     <g id="line2d_13"/>     <g id="text_14">      <!-- WIN -->      <g transform="translate(28.240625 34.272482) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-57" d="M 6880 4480 L 5370 0 L 4870 0 L 3558 3821 L 2240 0 L 1747 0 L 237 4480 L 723 4480 L 2016 627 L 3354 4480 L 3795 4480 L 5114 608 L 6426 4480 L 6880 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-49" d="M 730 4480 L 1203 4480 L 1203 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4e" d="M 4474 4480 L 4474 0 L 4083 0 L 1203 3635 L 1203 0 L 730 0 L 730 4480 L 1120 4480 L 4006 845 L 4006 4480 L 4474 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-57"/>       <use xlink:href="#Montserrat-Regular-49" x="111.099991"/>       <use xlink:href="#Montserrat-Regular-4e" x="141.299988"/>      </g>     </g>    </g>    <g id="text_15">     <!-- centipawn difference -->     <g transform="translate(17.26875 180.677188) rotate(-90) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-63" d="M 2029 -32 Q 1530 -32 1136 185 Q 742 403 518 796 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1136 3174 Q 1530 3392 2029 3392 Q 2464 3392 2806 3222 Q 3149 3053 3347 2726 L 3008 2496 Q 2842 2746 2586 2870 Q 2330 2995 2029 2995 Q 1664 2995 1373 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1293 918 995 Q 1082 698 1373 534 Q 1664 371 2029 371 Q 2330 371 2586 492 Q 2842 614 3008 864 L 3347 634 Q 3149 307 2803 137 Q 2458 -32 2029 -32 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-65" d="M 3565 1542 L 749 1542 Q 787 1018 1152 694 Q 1517 371 2074 371 Q 2387 371 2649 483 Q 2912 595 3104 813 L 3360 518 Q 3136 250 2800 109 Q 2464 -32 2061 -32 Q 1542 -32 1142 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 508 2566 Q 723 2957 1097 3174 Q 1472 3392 1939 3392 Q 2406 3392 2777 3174 Q 3149 2957 3360 2569 Q 3571 2182 3571 1683 L 3565 1542 zM 1939 3002 Q 1453 3002 1123 2691 Q 794 2381 749 1882 L 3136 1882 Q 3091 2381 2761 2691 Q 2432 3002 1939 3002 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-69" d="M 634 3366 L 1088 3366 L 1088 0 L 634 0 L 634 3366 zM 864 4102 Q 723 4102 627 4198 Q 531 4294 531 4429 Q 531 4557 627 4653 Q 723 4749 864 4749 Q 1005 4749 1101 4656 Q 1197 4563 1197 4435 Q 1197 4294 1101 4198 Q 1005 4102 864 4102 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-70" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1178 3827 787 Q 3610 397 3229 182 Q 2848 -32 2368 -32 Q 1958 -32 1628 137 Q 1299 307 1088 634 L 1088 -1242 L 634 -1242 L 634 3366 L 1069 3366 L 1069 2701 Q 1274 3034 1610 3213 Q 1946 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2364 Q 3264 2662 2976 2828 Q 2688 2995 2336 2995 Q 1978 2995 1693 2828 Q 1408 2662 1245 2364 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-61" d="M 1811 3392 Q 2470 3392 2822 3062 Q 3174 2733 3174 2086 L 3174 0 L 2739 0 L 2739 525 Q 2586 262 2288 115 Q 1990 -32 1581 -32 Q 1018 -32 685 237 Q 352 506 352 947 Q 352 1376 662 1638 Q 973 1901 1651 1901 L 2720 1901 L 2720 2106 Q 2720 2541 2477 2768 Q 2234 2995 1766 2995 Q 1446 2995 1152 2889 Q 858 2784 646 2598 L 442 2938 Q 698 3155 1056 3273 Q 1414 3392 1811 3392 zM 1651 326 Q 2035 326 2310 502 Q 2586 678 2720 1011 L 2720 1562 L 1664 1562 Q 800 1562 800 960 Q 800 666 1024 496 Q 1248 326 1651 326 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-77" d="M 5574 3366 L 4301 0 L 3872 0 L 2810 2758 L 1747 0 L 1318 0 L 51 3366 L 486 3366 L 1542 506 L 2624 3366 L 3014 3366 L 4090 506 L 5158 3366 L 5574 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-20" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-64" d="M 3706 4749 L 3706 0 L 3270 0 L 3270 666 Q 3066 326 2730 147 Q 2394 -32 1971 -32 Q 1498 -32 1114 185 Q 730 403 512 793 Q 294 1184 294 1683 Q 294 2182 512 2572 Q 730 2963 1114 3177 Q 1498 3392 1971 3392 Q 2381 3392 2710 3222 Q 3040 3053 3251 2726 L 3251 4749 L 3706 4749 zM 2010 371 Q 2362 371 2646 534 Q 2931 698 3094 998 Q 3258 1299 3258 1683 Q 3258 2067 3094 2368 Q 2931 2669 2646 2832 Q 2362 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-66" d="M 1747 4397 Q 1453 4397 1302 4237 Q 1152 4077 1152 3763 L 1152 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 0 L 710 0 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 3782 Q 710 4243 976 4512 Q 1242 4781 1728 4781 Q 1920 4781 2099 4726 Q 2278 4672 2400 4570 L 2246 4230 Q 2048 4397 1747 4397 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-63"/>      <use xlink:href="#Montserrat-Regular-65" x="56.299988"/>      <use xlink:href="#Montserrat-Regular-6e" x="116.699982"/>      <use xlink:href="#Montserrat-Regular-74" x="184.399963"/>      <use xlink:href="#Montserrat-Regular-69" x="224.999954"/>      <use xlink:href="#Montserrat-Regular-70" x="251.899948"/>      <use xlink:href="#Montserrat-Regular-61" x="319.699936"/>      <use xlink:href="#Montserrat-Regular-77" x="378.699921"/>      <use xlink:href="#Montserrat-Regular-6e" x="466.599915"/>      <use xlink:href="#Montserrat-Regular-20" x="534.299896"/>      <use xlink:href="#Montserrat-Regular-64" x="560.499893"/>      <use xlink:href="#Montserrat-Regular-69" x="628.299881"/>      <use xlink:href="#Montserrat-Regular-66" x="655.199875"/>      <use xlink:href="#Montserrat-Regular-66" x="689.099869"/>      <use xlink:href="#Montserrat-Regular-65" x="722.999863"/>      <use xlink:href="#Montserrat-Regular-72" x="783.399857"/>      <use xlink:href="#Montserrat-Regular-65" x="823.499847"/>      <use xlink:href="#Montserrat-Regular-6e" x="883.899841"/>      <use xlink:href="#Montserrat-Regular-63" x="951.599823"/>      <use xlink:href="#Montserrat-Regular-65" x="1007.899811"/>     </g>    </g>   </g>   <g id="line2d_14">    <path d="M 54 127.26 L 57.348 122.360651 L 60.696 122.167256 L 64.044 122.876372 L 67.392 121.909395 L 70.74 121.97386 L 74.088 117.138977 L 77.436 116.623256 L 80.784 115.076093 L 84.132 113.27107 L 87.48 111.079256 L 90.828 151.112093 L 94.176 164.778698 L 97.524 176.962605 L 100.872 176.446884 L 104.22 181.088372 L 107.568 170.258233 L 110.916 171.096279 L 114.264 181.926419 L 117.612 180.637116 L 120.96 172.901302 L 124.308 168.84 L 127.656 176.060093 L 131.004 175.608837 L 134.352 178.638698 L 137.7 167.744093 L 141.048 167.937488 L 144.396 174.577395 L 147.744 187.857209 L 151.092 211.967163 L 154.44 211.967163 L 157.788 214.094512 L 161.136 223.893209 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p1a701acc29)" style="fill: none; stroke: #9467bd; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_15">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 123.456558 L 64.044 121.135814 L 67.392 123.069767 L 70.74 119.20186 L 74.088 115.656279 L 77.436 115.91414 L 80.784 118.879535 L 84.132 118.041488 L 87.48 116.623256 L 90.828 120.555628 L 94.176 143.569674 L 97.524 145.890419 L 100.872 148.404558 L 104.22 147.824372 L 107.568 171.225209 L 110.916 163.03814 L 114.264 178.058512 L 117.612 164.391907 L 120.96 166.068 L 124.308 170.516093 L 127.656 184.440558 L 131.004 187.148093 L 134.352 194.754977 L 137.7 196.753395 L 141.048 193.594605 L 144.396 197.913767 L 147.744 194.04586 L 151.092 188.11507 L 154.44 190.75814 L 157.788 195.979814 L 161.136 223.828744 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p1a701acc29)" style="fill: none; stroke: #9467bd; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_16">    <path d="M 54 127.26 L 57.348 125.068186 L 60.696 126.035163 L 64.044 125.970698 L 67.392 112.110698 L 70.74 151.498884 L 74.088 149.758326 L 77.436 155.624651 L 80.784 149.69386 L 84.132 136.027256 L 87.48 140.733209 L 90.828 158.074326 L 94.176 166.712651 L 97.524 171.096279 L 100.872 181.281767 L 104.22 176.89814 L 107.568 179.089953 L 110.916 184.376093 L 114.264 174.706326 L 117.612 176.382419 L 120.96 180.314791 L 124.308 179.79907 L 127.656 186.438977 L 131.004 188.566326 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p1a701acc29)" style="fill: none; stroke: #9467bd; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_17">    <path d="M 54 127.26 L 57.348 122.167256 L 60.696 124.488 L 64.044 121.97386 L 67.392 127.51786 L 70.74 134.737953 L 74.088 135.640465 L 77.436 139.121581 L 80.784 135.31814 L 84.132 134.737953 L 87.48 153.174977 L 90.828 154.657674 L 94.176 162.393488 L 97.524 164.649767 L 100.872 172.836837 L 104.22 182.44214 L 107.568 175.737767 L 110.916 182.119814 L 114.264 185.149674 L 117.612 179.79907 L 120.96 179.541209 L 124.308 181.926419 L 127.656 190.564744 L 131.004 187.341488 L 134.352 206.874419 L 137.7 178.638698 L 141.048 198.558419 L 144.396 177.091535 L 147.744 201.588279 L 151.092 223.828744 L 154.44 223.893209 L 157.788 223.893209 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p1a701acc29)" style="fill: none; stroke: #9467bd; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_18">    <path d="M 54 127.26 L 57.348 116.365395 L 60.696 115.398419 L 64.044 117.010047 L 67.392 116.816651 L 70.74 151.821209 L 74.088 150.274047 L 77.436 156.785023 L 80.784 152.981581 L 84.132 149.758326 L 87.48 152.014605 L 90.828 155.624651 L 94.176 158.138791 L 97.524 160.459535 L 100.872 158.912372 L 104.22 156.785023 L 107.568 151.112093 L 110.916 157.49414 L 114.264 154.528744 L 117.612 152.788186 L 120.96 155.044465 L 124.308 155.10893 L 127.656 160.008279 L 131.004 145.116837 L 134.352 162.973674 L 137.7 169.678047 L 141.048 171.225209 L 144.396 166.068 L 147.744 160.78186 L 151.092 160.330605 L 154.44 178.187442 L 157.788 181.668558 L 161.136 164.520837 L 164.484 174.899721 L 167.832 169.678047 L 171.18 165.294419 L 174.528 186.954698 L 177.876 185.536465 L 181.224 181.475163 L 184.572 171.74093 L 187.92 180.766047 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p1a701acc29)" style="fill: none; stroke: #9467bd; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_19">    <path d="M 54 127.26 L 57.348 121.329209 L 60.696 121.135814 L 64.044 118.750605 L 67.392 122.618512 L 70.74 116.881116 L 74.088 123.778884 L 77.436 120.749023 L 80.784 113.013209 L 84.132 120.297767 L 87.48 113.65786 L 90.828 122.038326 L 94.176 142.344837 L 97.524 143.118419 L 100.872 141.700186 L 104.22 147.050791 L 107.568 142.860558 L 110.916 156.913953 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p1a701acc29)" style="fill: none; stroke: #9467bd; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_20">    <path d="M 54 127.26 L 57.348 125.068186 L 60.696 122.940837 L 64.044 124.488 L 67.392 114.173581 L 70.74 123.263163 L 74.088 115.591814 L 77.436 120.297767 L 80.784 120.297767 L 84.132 159.879349 L 87.48 164.262977 L 90.828 223.957674 L 94.176 223.957674 L 97.524 223.957674 L 100.872 223.957674 L 104.22 223.957674 L 107.568 223.957674 L 110.916 223.957674 L 114.264 223.957674 L 117.612 223.957674 L 120.96 223.957674 L 124.308 223.957674 L 127.656 223.957674 L 131.004 223.957674 L 134.352 223.957674 L 137.7 223.957674 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p1a701acc29)" style="fill: none; stroke: #9467bd; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_21">    <path d="M 54 127.26 L 57.348 125.068186 L 60.696 121.780465 L 64.044 119.137395 L 67.392 124.552465 L 70.74 117.719163 L 74.088 123.069767 L 77.436 119.588651 L 80.784 125.068186 L 84.132 121.97386 L 87.48 121.58707 L 90.828 121.97386 L 94.176 120.297767 L 97.524 118.879535 L 100.872 122.296186 L 104.22 134.480093 L 107.568 142.473767 L 110.916 128.420372 L 114.264 158.396651 L 117.612 162.909209 L 120.96 157.752 L 124.308 164.972093 L 127.656 223.893209 L 131.004 223.893209 L 134.352 223.828744 L 137.7 223.893209 L 141.048 223.957674 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p1a701acc29)" style="fill: none; stroke: #9467bd; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_22">    <path d="M 54 127.26 L 57.348 125.068186 L 60.696 123.521023 L 64.044 120.684558 L 67.392 115.269488 L 70.74 119.910977 L 74.088 121.45814 L 77.436 120.039907 L 80.784 114.109116 L 84.132 114.624837 L 87.48 117.396837 L 90.828 117.267907 L 94.176 115.398419 L 97.524 115.333953 L 100.872 145.310233 L 104.22 131.965953 L 107.568 147.179721 L 110.916 156.527163 L 114.264 146.92186 L 117.612 157.558605 L 120.96 159.363628 L 124.308 151.176558 L 127.656 151.563349 L 131.004 149.758326 L 134.352 171.289674 L 137.7 185.278605 L 141.048 191.402791 L 144.396 223.957674 L 147.744 223.957674 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p1a701acc29)" style="fill: none; stroke: #9467bd; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_23">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 124.35907 L 64.044 122.554047 L 67.392 117.267907 L 70.74 115.849674 L 74.088 112.175163 L 77.436 110.563535 L 80.784 114.560372 L 84.132 119.782047 L 87.48 123.456558 L 90.828 142.538233 L 94.176 146.470605 L 97.524 155.624651 L 100.872 164.972093 L 104.22 167.744093 L 107.568 169.291256 L 110.916 174.448465 L 114.264 180.830512 L 117.612 184.440558 L 120.96 180.701581 L 124.308 187.728279 L 127.656 203.135442 L 131.004 187.599349 L 134.352 205.13386 L 137.7 198.558419 L 141.048 187.277023 L 144.396 181.604093 L 147.744 201.394884 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p1a701acc29)" style="fill: none; stroke: #9467bd; stroke-opacity: 0.33; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_24">    <path d="M 54 127.26 L 57.348 124.488 L 60.696 123.198698 L 64.044 121.554837 L 67.392 119.588651 L 70.74 120.942419 L 74.088 122.263953 L 77.436 120.168837 L 80.784 119.588651 L 84.132 121.135814 L 87.48 122.521814 L 90.828 146.825163 L 94.176 152.304698 L 97.524 158.042093 L 100.872 161.942233 L 104.22 162.264558 L 107.568 169.774744 L 110.916 167.067209 L 114.264 179.444512 L 117.612 178.090744 L 120.96 176.221256 L 124.308 175.157581 L 127.656 188.50186 L 131.004 187.470419 L 134.352 206.00414 L 137.7 197.655907 L 141.048 196.076512 L 144.396 210.935721 L 147.744 212.772977 L 151.092 223.957674 L 154.44 223.957674 L 157.788 223.957674 L 161.136 223.957674 L 164.484 223.957674 L 167.832 223.957674 L 171.18 223.957674 L 174.528 223.957674 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p1a701acc29)" style="fill: none; stroke: #000000; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="patch_3">    <path d="M 54 224.28 L 54 30.24 " style="fill: none"/>   </g>   <g id="patch_4">    <path d="M 388.8 224.28 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="patch_5">    <path d="M 54 224.28 L 388.8 224.28 " style="fill: none"/>   </g>   <g id="patch_6">    <path d="M 54 30.24 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="text_16">    <g id="patch_7">     <a xlink:href="https://dynomight.net/chess">      <path d="M 323.8664 225.676606 L 388.881244 225.676606 L 388.881244 213.343012 L 323.8664 213.343012 z" style="fill: #ffffff; opacity: 0.000001; stroke: #ffffff; stroke-linejoin: miter"/>     </a>    </g>    <a xlink:href="https://dynomight.net/chess">     <!-- dynomight.net/chess -->     <g style="opacity: 0.3" transform="translate(327.8664 220.734419) scale(0.05 -0.05)">      <defs>       <path id="CourierNewPSMT-64" d="M 3206 3922 L 3206 263 L 3556 263 Q 3653 263 3693 227 Q 3734 191 3734 131 Q 3734 75 3693 37 Q 3653 0 3556 0 L 2941 0 L 2941 522 Q 2484 -103 1781 -103 Q 1425 -103 1098 86 Q 772 275 583 625 Q 394 975 394 1347 Q 394 1722 583 2070 Q 772 2419 1098 2609 Q 1425 2800 1784 2800 Q 2472 2800 2941 2175 L 2941 3656 L 2591 3656 Q 2494 3656 2453 3693 Q 2413 3731 2413 3791 Q 2413 3847 2453 3884 Q 2494 3922 2591 3922 L 3206 3922 zM 2941 1347 Q 2941 1847 2603 2190 Q 2266 2534 1800 2534 Q 1331 2534 993 2190 Q 656 1847 656 1347 Q 656 850 993 504 Q 1331 159 1800 159 Q 2266 159 2603 504 Q 2941 850 2941 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-79" d="M 1938 0 L 719 2441 L 641 2441 Q 547 2441 506 2478 Q 466 2516 466 2572 Q 466 2613 486 2645 Q 506 2678 539 2692 Q 572 2706 641 2706 L 1359 2706 Q 1453 2706 1493 2668 Q 1534 2631 1534 2572 Q 1534 2516 1493 2478 Q 1453 2441 1359 2441 L 1006 2441 L 2078 288 L 3134 2441 L 2781 2441 Q 2688 2441 2647 2478 Q 2606 2516 2606 2575 Q 2606 2631 2647 2668 Q 2688 2706 2781 2706 L 3497 2706 Q 3594 2706 3634 2668 Q 3675 2631 3675 2572 Q 3675 2531 3650 2497 Q 3625 2463 3594 2452 Q 3563 2441 3419 2441 L 1756 -944 L 2166 -944 Q 2259 -944 2300 -980 Q 2341 -1016 2341 -1075 Q 2341 -1131 2300 -1168 Q 2259 -1206 2166 -1206 L 653 -1206 Q 559 -1206 518 -1170 Q 478 -1134 478 -1075 Q 478 -1016 518 -980 Q 559 -944 653 -944 L 1475 -944 L 1938 0 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6e" d="M 1072 2706 L 1072 2309 Q 1347 2588 1569 2694 Q 1791 2800 2069 2800 Q 2369 2800 2616 2672 Q 2791 2578 2933 2361 Q 3075 2144 3075 1916 L 3075 263 L 3297 263 Q 3391 263 3431 227 Q 3472 191 3472 131 Q 3472 75 3431 37 Q 3391 0 3297 0 L 2594 0 Q 2497 0 2456 37 Q 2416 75 2416 131 Q 2416 191 2456 227 Q 2497 263 2594 263 L 2813 263 L 2813 1872 Q 2813 2150 2609 2342 Q 2406 2534 2066 2534 Q 1806 2534 1615 2429 Q 1425 2325 1072 1909 L 1072 263 L 1369 263 Q 1463 263 1503 227 Q 1544 191 1544 131 Q 1544 75 1503 37 Q 1463 0 1369 0 L 513 0 Q 419 0 378 37 Q 338 75 338 131 Q 338 191 378 227 Q 419 263 513 263 L 809 263 L 809 2441 L 588 2441 Q 494 2441 453 2478 Q 413 2516 413 2575 Q 413 2631 453 2668 Q 494 2706 588 2706 L 1072 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6f" d="M 3388 1347 Q 3388 747 2958 322 Q 2528 -103 1922 -103 Q 1309 -103 881 323 Q 453 750 453 1347 Q 453 1947 881 2373 Q 1309 2800 1922 2800 Q 2528 2800 2958 2375 Q 3388 1950 3388 1347 zM 3122 1347 Q 3122 1841 2770 2187 Q 2419 2534 1919 2534 Q 1419 2534 1067 2186 Q 716 1838 716 1347 Q 716 859 1067 509 Q 1419 159 1919 159 Q 2419 159 2770 507 Q 3122 856 3122 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6d" d="M 725 2706 L 725 2441 Q 1059 2800 1397 2800 Q 1600 2800 1753 2692 Q 1906 2584 2009 2366 Q 2184 2584 2364 2692 Q 2544 2800 2725 2800 Q 3009 2800 3178 2616 Q 3400 2378 3400 2097 L 3400 263 L 3622 263 Q 3716 263 3756 227 Q 3797 191 3797 131 Q 3797 75 3756 37 Q 3716 0 3622 0 L 3138 0 L 3138 2072 Q 3138 2272 3016 2403 Q 2894 2534 2734 2534 Q 2591 2534 2431 2426 Q 2272 2319 2069 2003 L 2069 263 L 2288 263 Q 2381 263 2422 227 Q 2463 191 2463 131 Q 2463 75 2422 37 Q 2381 0 2288 0 L 1803 0 L 1803 2053 Q 1803 2263 1679 2398 Q 1556 2534 1403 2534 Q 1263 2534 1125 2444 Q 934 2316 725 2003 L 725 263 L 947 263 Q 1041 263 1081 227 Q 1122 191 1122 131 Q 1122 75 1081 37 Q 1041 0 947 0 L 241 0 Q 147 0 106 37 Q 66 75 66 131 Q 66 191 106 227 Q 147 263 241 263 L 463 263 L 463 2441 L 241 2441 Q 147 2441 106 2478 Q 66 2516 66 2575 Q 66 2631 106 2668 Q 147 2706 241 2706 L 725 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-69" d="M 2050 4053 L 2050 3375 L 1663 3375 L 1663 4053 L 2050 4053 zM 2059 2706 L 2059 263 L 3091 263 Q 3188 263 3228 227 Q 3269 191 3269 131 Q 3269 75 3228 37 Q 3188 0 3091 0 L 766 0 Q 672 0 631 37 Q 591 75 591 131 Q 591 191 631 227 Q 672 263 766 263 L 1797 263 L 1797 2441 L 1031 2441 Q 938 2441 895 2478 Q 853 2516 853 2572 Q 853 2631 893 2668 Q 934 2706 1031 2706 L 2059 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-67" d="M 2803 2225 L 2803 2706 L 3419 2706 Q 3513 2706 3553 2668 Q 3594 2631 3594 2572 Q 3594 2516 3553 2478 Q 3513 2441 3419 2441 L 3066 2441 L 3066 -181 Q 3066 -444 2953 -650 Q 2878 -788 2703 -934 Q 2528 -1081 2384 -1143 Q 2241 -1206 2000 -1206 L 1256 -1206 Q 1163 -1206 1122 -1170 Q 1081 -1134 1081 -1075 Q 1081 -1016 1122 -978 Q 1163 -941 1256 -941 L 2009 -944 Q 2241 -944 2427 -825 Q 2613 -706 2734 -478 Q 2803 -347 2803 -144 L 2803 644 Q 2381 66 1716 66 Q 1175 66 783 467 Q 391 869 391 1434 Q 391 2000 783 2400 Q 1175 2800 1716 2800 Q 2381 2800 2803 2225 zM 2803 1434 Q 2803 1897 2486 2215 Q 2169 2534 1728 2534 Q 1288 2534 972 2214 Q 656 1894 656 1434 Q 656 972 972 651 Q 1288 331 1728 331 Q 2169 331 2486 651 Q 2803 972 2803 1434 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-68" d="M 1056 3922 L 1056 2306 Q 1306 2578 1536 2689 Q 1766 2800 2050 2800 Q 2356 2800 2570 2692 Q 2784 2584 2928 2361 Q 3072 2138 3072 1891 L 3072 263 L 3366 263 Q 3463 263 3502 227 Q 3541 191 3541 131 Q 3541 75 3502 37 Q 3463 0 3366 0 L 2513 0 Q 2416 0 2375 37 Q 2334 75 2334 131 Q 2334 191 2375 227 Q 2416 263 2513 263 L 2806 263 L 2806 1872 Q 2806 2156 2601 2347 Q 2397 2538 2031 2538 Q 1744 2538 1541 2397 Q 1394 2297 1056 1925 L 1056 263 L 1353 263 Q 1447 263 1487 227 Q 1528 191 1528 131 Q 1528 75 1487 37 Q 1447 0 1353 0 L 497 0 Q 403 0 362 37 Q 322 75 322 131 Q 322 191 362 227 Q 403 263 497 263 L 794 263 L 794 3656 L 441 3656 Q 347 3656 306 3693 Q 266 3731 266 3791 Q 266 3847 306 3884 Q 347 3922 441 3922 L 1056 3922 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-74" d="M 1381 2706 L 2806 2706 Q 2900 2706 2940 2668 Q 2981 2631 2981 2572 Q 2981 2516 2940 2478 Q 2900 2441 2806 2441 L 1381 2441 L 1381 694 Q 1381 466 1564 312 Q 1747 159 2100 159 Q 2366 159 2675 239 Q 2984 319 3156 419 Q 3219 459 3259 459 Q 3309 459 3346 420 Q 3384 381 3384 328 Q 3384 281 3344 241 Q 3244 138 2858 17 Q 2472 -103 2119 -103 Q 1659 -103 1387 112 Q 1116 328 1116 694 L 1116 2441 L 631 2441 Q 538 2441 497 2478 Q 456 2516 456 2575 Q 456 2631 497 2668 Q 538 2706 631 2706 L 1116 2706 L 1116 3481 Q 1116 3575 1153 3615 Q 1191 3656 1247 3656 Q 1306 3656 1343 3615 Q 1381 3575 1381 3481 L 1381 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2e" d="M 1866 753 L 1978 753 Q 2153 753 2278 629 Q 2403 506 2403 331 Q 2403 150 2276 28 Q 2150 -94 1978 -94 L 1866 -94 Q 1691 -94 1566 29 Q 1441 153 1441 328 Q 1441 509 1567 631 Q 1694 753 1866 753 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-65" d="M 3341 1291 L 666 1291 Q 734 781 1092 470 Q 1450 159 1978 159 Q 2272 159 2594 256 Q 2916 353 3119 513 Q 3178 559 3222 559 Q 3272 559 3309 520 Q 3347 481 3347 428 Q 3347 375 3297 325 Q 3147 169 2764 33 Q 2381 -103 1978 -103 Q 1303 -103 851 339 Q 400 781 400 1409 Q 400 1981 823 2390 Q 1247 2800 1872 2800 Q 2516 2800 2931 2379 Q 3347 1959 3341 1291 zM 3075 1556 Q 2997 1991 2664 2262 Q 2331 2534 1872 2534 Q 1413 2534 1081 2265 Q 750 1997 669 1556 L 3075 1556 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2f" d="M 3100 4113 L 988 -413 Q 934 -528 850 -528 Q 797 -528 758 -489 Q 719 -450 719 -403 Q 719 -366 750 -300 L 2859 4225 Q 2894 4297 2925 4319 Q 2956 4341 3000 4341 Q 3053 4341 3092 4302 Q 3131 4263 3131 4216 Q 3131 4181 3100 4113 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-63" d="M 3016 2441 L 3016 2528 Q 3016 2625 3053 2665 Q 3091 2706 3147 2706 Q 3206 2706 3243 2665 Q 3281 2625 3281 2528 L 3281 1934 Q 3278 1838 3242 1797 Q 3206 1756 3147 1756 Q 3094 1756 3058 1792 Q 3022 1828 3016 1913 Q 2997 2134 2723 2334 Q 2450 2534 1988 2534 Q 1403 2534 1100 2168 Q 797 1803 797 1331 Q 797 822 1131 490 Q 1466 159 1997 159 Q 2303 159 2620 271 Q 2938 384 3194 634 Q 3259 697 3309 697 Q 3363 697 3398 661 Q 3434 625 3434 572 Q 3434 438 3119 231 Q 2609 -103 1984 -103 Q 1350 -103 942 301 Q 534 706 534 1328 Q 534 1963 951 2381 Q 1369 2800 2003 2800 Q 2606 2800 3016 2441 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-73" d="M 2788 2534 Q 2788 2625 2825 2665 Q 2863 2706 2919 2706 Q 2978 2706 3015 2665 Q 3053 2625 3053 2528 L 3053 2078 Q 3053 1984 3015 1943 Q 2978 1903 2919 1903 Q 2866 1903 2830 1937 Q 2794 1972 2788 2050 Q 2769 2238 2594 2359 Q 2338 2534 1916 2534 Q 1475 2534 1231 2356 Q 1047 2222 1047 2056 Q 1047 1869 1266 1744 Q 1416 1656 1834 1609 Q 2381 1550 2594 1475 Q 2897 1366 3045 1172 Q 3194 978 3194 753 Q 3194 419 2872 158 Q 2550 -103 1928 -103 Q 1306 -103 909 213 Q 909 106 896 75 Q 884 44 851 22 Q 819 0 778 0 Q 722 0 684 40 Q 647 81 647 175 L 647 716 Q 647 809 683 850 Q 719 891 778 891 Q 834 891 873 852 Q 913 813 913 747 Q 913 603 984 506 Q 1094 356 1333 257 Q 1572 159 1919 159 Q 2431 159 2681 350 Q 2931 541 2931 753 Q 2931 997 2678 1144 Q 2422 1291 1933 1341 Q 1444 1391 1231 1472 Q 1019 1553 900 1715 Q 781 1878 781 2066 Q 781 2403 1112 2601 Q 1444 2800 1903 2800 Q 2447 2800 2788 2534 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#CourierNewPSMT-64"/>      <use xlink:href="#CourierNewPSMT-79" x="60.009766"/>      <use xlink:href="#CourierNewPSMT-6e" x="120.019531"/>      <use xlink:href="#CourierNewPSMT-6f" x="180.029297"/>      <use xlink:href="#CourierNewPSMT-6d" x="240.039062"/>      <use xlink:href="#CourierNewPSMT-69" x="300.048828"/>      <use xlink:href="#CourierNewPSMT-67" x="360.058594"/>      <use xlink:href="#CourierNewPSMT-68" x="420.068359"/>      <use xlink:href="#CourierNewPSMT-74" x="480.078125"/>      <use xlink:href="#CourierNewPSMT-2e" x="540.087891"/>      <use xlink:href="#CourierNewPSMT-6e" x="600.097656"/>      <use xlink:href="#CourierNewPSMT-65" x="660.107422"/>      <use xlink:href="#CourierNewPSMT-74" x="720.117188"/>      <use xlink:href="#CourierNewPSMT-2f" x="780.126953"/>      <use xlink:href="#CourierNewPSMT-63" x="840.136719"/>      <use xlink:href="#CourierNewPSMT-68" x="900.146484"/>      <use xlink:href="#CourierNewPSMT-65" x="960.15625"/>      <use xlink:href="#CourierNewPSMT-73" x="1020.166016"/>      <use xlink:href="#CourierNewPSMT-73" x="1080.175781"/>     </g>    </a>   </g>   <g id="text_17">    <!-- o1-mini vs stockfish -->    <g transform="translate(163.431563 24.24) scale(0.12 -0.12)">     <defs>      <path id="Montserrat-Regular-6f" d="M 2010 -32 Q 1523 -32 1132 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1132 3174 Q 1523 3392 2010 3392 Q 2496 3392 2886 3174 Q 3277 2957 3497 2566 Q 3718 2176 3718 1683 Q 3718 1190 3497 800 Q 3277 410 2886 189 Q 2496 -32 2010 -32 zM 2010 371 Q 2368 371 2653 534 Q 2938 698 3098 998 Q 3258 1299 3258 1683 Q 3258 2067 3098 2368 Q 2938 2669 2653 2832 Q 2368 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6d" d="M 4832 3392 Q 5466 3392 5827 3027 Q 6189 2662 6189 1952 L 6189 0 L 5734 0 L 5734 1907 Q 5734 2432 5481 2707 Q 5229 2982 4768 2982 Q 4243 2982 3942 2659 Q 3642 2336 3642 1766 L 3642 0 L 3187 0 L 3187 1907 Q 3187 2432 2934 2707 Q 2682 2982 2214 2982 Q 1696 2982 1392 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2752 Q 1248 3059 1568 3225 Q 1888 3392 2304 3392 Q 2726 3392 3036 3213 Q 3347 3034 3501 2682 Q 3686 3014 4035 3203 Q 4384 3392 4832 3392 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-76" d="M 3462 3366 L 1965 0 L 1498 0 L 6 3366 L 480 3366 L 1741 499 L 3014 3366 L 3462 3366 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-6b" d="M 1882 1690 L 1088 960 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 1530 L 3098 3366 L 3661 3366 L 2221 1990 L 3795 0 L 3238 0 L 1882 1690 z" transform="scale(0.015625)"/>      <path id="Montserrat-Regular-68" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2778 Q 1274 3072 1600 3232 Q 1926 3392 2349 3392 z" transform="scale(0.015625)"/>     </defs>     <use xlink:href="#Montserrat-Regular-6f"/>     <use xlink:href="#Montserrat-Regular-31" x="62.699997"/>     <use xlink:href="#Montserrat-Regular-2d" x="98.799988"/>     <use xlink:href="#Montserrat-Regular-6d" x="136.999985"/>     <use xlink:href="#Montserrat-Regular-69" x="243.099976"/>     <use xlink:href="#Montserrat-Regular-6e" x="269.999969"/>     <use xlink:href="#Montserrat-Regular-69" x="337.699951"/>     <use xlink:href="#Montserrat-Regular-20" x="364.599945"/>     <use xlink:href="#Montserrat-Regular-76" x="390.799942"/>     <use xlink:href="#Montserrat-Regular-73" x="444.999939"/>     <use xlink:href="#Montserrat-Regular-20" x="493.899933"/>     <use xlink:href="#Montserrat-Regular-73" x="520.09993"/>     <use xlink:href="#Montserrat-Regular-74" x="568.999924"/>     <use xlink:href="#Montserrat-Regular-6f" x="609.599915"/>     <use xlink:href="#Montserrat-Regular-63" x="672.299911"/>     <use xlink:href="#Montserrat-Regular-6b" x="728.599899"/>     <use xlink:href="#Montserrat-Regular-66" x="788.69989"/>     <use xlink:href="#Montserrat-Regular-69" x="822.599884"/>     <use xlink:href="#Montserrat-Regular-73" x="849.499878"/>     <use xlink:href="#Montserrat-Regular-68" x="898.399872"/>    </g>   </g>  </g> </g> <defs>  <clipPath id="p1a701acc29">   <rect x="54" y="30.24" width="334.8" height="194.04"/>  </clipPath> </defs></svg>](https://dynomight.net/img/chess/o1-mini_stockfish.pdf)

[Terrible](https://lichess.org/d44jrD2E).

So, umm:

| Model                    | Quality   |
| ------------------------ | --------- |
| `Llama-3.2-3b`           | Terrible  |
| `Llama-3.2-3b-instruct`  | Terrible  |
| `Llama-3.1-70b`          | Terrible  |
| `Llama-3.1-70b-instruct` | Terrible  |
| `Qwen-2.5`               | Terrible  |
| `command-r-v01`          | Terrible  |
| `gemma-2-27b`            | Terrible  |
| `gemma-2-27b-it`         | Terrible  |
| `gpt-3.5-turbo-instruct` | Excellent |
| `gpt-3.5-turbo`          | Terrible  |
| `gpt-4o-mini`            | Terrible  |
| `gpt-4o`                 | Terrible  |
| `o1-mini`                | Terrible  |

And, uhh:

[<?xml version="1.0" encoding="utf-8" standalone="no"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"  "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg xmlns:xlink="http://www.w3.org/1999/xlink" width="432pt" height="252pt" viewBox="0 0 432 252" xmlns="http://www.w3.org/2000/svg" version="1.1"> <metadata>  <rdf:RDF xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:cc="http://creativecommons.org/ns#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">   <cc:Work>    <dc:type rdf:resource="http://purl.org/dc/dcmitype/StillImage"/>    <dc:date>2024-11-14T11:35:26.144596</dc:date>    <dc:format>image/svg+xml</dc:format>    <dc:creator>     <cc:Agent>      <dc:title>Matplotlib v3.8.4, https://matplotlib.org/</dc:title>     </cc:Agent>    </dc:creator>   </cc:Work>  </rdf:RDF> </metadata> <defs>  <style type="text/css">*{stroke-linejoin: round; stroke-linecap: butt}</style> </defs> <g id="figure_1">  <g id="patch_1">   <path d="M 0 252 L 432 252 L 432 0 L 0 0 z" style="fill: #ffffff"/>  </g>  <g id="axes_1">   <g id="patch_2">    <path d="M 54 224.28 L 388.8 224.28 L 388.8 30.24 L 54 30.24 z" style="fill: #ffffff"/>   </g>   <g id="matplotlib.axis_1">    <g id="xtick_1">     <g id="line2d_1"/>     <g id="text_1">      <!-- 0 -->      <g transform="translate(50.689844 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-30" d="M 2118 -38 Q 1600 -38 1193 237 Q 787 512 560 1027 Q 333 1542 333 2240 Q 333 2938 560 3453 Q 787 3968 1193 4243 Q 1600 4518 2118 4518 Q 2637 4518 3043 4243 Q 3450 3968 3680 3453 Q 3910 2938 3910 2240 Q 3910 1542 3680 1027 Q 3450 512 3043 237 Q 2637 -38 2118 -38 zM 2118 384 Q 2509 384 2806 601 Q 3104 819 3270 1235 Q 3437 1651 3437 2240 Q 3437 2829 3270 3245 Q 3104 3661 2806 3878 Q 2509 4096 2118 4096 Q 1728 4096 1430 3878 Q 1133 3661 966 3245 Q 800 2829 800 2240 Q 800 1651 966 1235 Q 1133 819 1430 601 Q 1728 384 2118 384 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="xtick_2">     <g id="line2d_2"/>     <g id="text_2">      <!-- 20 -->      <g transform="translate(114.81 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-32" d="M 3392 410 L 3392 0 L 250 0 L 250 326 L 2112 2150 Q 2464 2496 2589 2749 Q 2714 3002 2714 3258 Q 2714 3654 2442 3875 Q 2170 4096 1664 4096 Q 877 4096 442 3597 L 115 3878 Q 378 4186 784 4352 Q 1190 4518 1702 4518 Q 2387 4518 2787 4195 Q 3187 3872 3187 3309 Q 3187 2963 3033 2643 Q 2880 2323 2451 1907 L 928 410 L 3392 410 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-32"/>       <use xlink:href="#Montserrat-Regular-30" x="56.799988"/>      </g>     </g>    </g>    <g id="xtick_3">     <g id="line2d_3"/>     <g id="text_3">      <!-- 40 -->      <g transform="translate(181.305156 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-34" d="M 4141 1178 L 3245 1178 L 3245 0 L 2784 0 L 2784 1178 L 256 1178 L 256 1510 L 2630 4480 L 3142 4480 L 851 1587 L 2797 1587 L 2797 2624 L 3245 2624 L 3245 1587 L 4141 1587 L 4141 1178 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-34"/>       <use xlink:href="#Montserrat-Regular-30" x="66.099991"/>      </g>     </g>    </g>    <g id="xtick_4">     <g id="line2d_4"/>     <g id="text_4">      <!-- 60 -->      <g transform="translate(248.524531 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-36" d="M 2189 2682 Q 2630 2682 2972 2515 Q 3315 2349 3507 2045 Q 3699 1741 3699 1338 Q 3699 928 3500 614 Q 3302 301 2953 131 Q 2605 -38 2170 -38 Q 1280 -38 806 547 Q 333 1133 333 2202 Q 333 2950 589 3468 Q 845 3987 1305 4252 Q 1766 4518 2381 4518 Q 3046 4518 3443 4275 L 3258 3904 Q 2938 4115 2387 4115 Q 1645 4115 1222 3644 Q 800 3174 800 2278 Q 800 2118 819 1914 Q 986 2278 1350 2480 Q 1715 2682 2189 2682 zM 2150 352 Q 2637 352 2941 617 Q 3245 883 3245 1325 Q 3245 1766 2941 2028 Q 2637 2291 2118 2291 Q 1779 2291 1520 2163 Q 1261 2035 1113 1811 Q 966 1587 966 1312 Q 966 1056 1104 838 Q 1242 621 1510 486 Q 1779 352 2150 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-36"/>       <use xlink:href="#Montserrat-Regular-30" x="60.899994"/>      </g>     </g>    </g>    <g id="xtick_5">     <g id="line2d_5"/>     <g id="text_5">      <!-- 80 -->      <g transform="translate(315.34 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-38" d="M 2899 2349 Q 3322 2208 3549 1926 Q 3776 1645 3776 1248 Q 3776 858 3565 566 Q 3354 275 2960 118 Q 2566 -38 2035 -38 Q 1229 -38 768 307 Q 307 653 307 1248 Q 307 1651 528 1929 Q 749 2208 1171 2349 Q 826 2483 646 2736 Q 467 2989 467 3334 Q 467 3878 892 4198 Q 1318 4518 2035 4518 Q 2509 4518 2864 4374 Q 3219 4230 3417 3961 Q 3616 3693 3616 3334 Q 3616 2989 3430 2736 Q 3245 2483 2899 2349 zM 928 3322 Q 928 2950 1222 2732 Q 1517 2515 2035 2515 Q 2554 2515 2851 2732 Q 3149 2950 3149 3315 Q 3149 3693 2845 3910 Q 2541 4128 2035 4128 Q 1523 4128 1225 3910 Q 928 3693 928 3322 zM 2035 352 Q 2630 352 2966 595 Q 3302 838 3302 1254 Q 3302 1670 2966 1910 Q 2630 2150 2035 2150 Q 1446 2150 1110 1910 Q 774 1670 774 1254 Q 774 832 1107 592 Q 1440 352 2035 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-38"/>       <use xlink:href="#Montserrat-Regular-30" x="63.799988"/>      </g>     </g>    </g>    <g id="xtick_6">     <g id="line2d_6"/>     <g id="text_6">      <!-- 100 -->      <g transform="translate(380.375 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-31" d="M 1574 4480 L 1574 0 L 1114 0 L 1114 4070 L 58 4070 L 58 4480 L 1574 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>      </g>     </g>    </g>    <g id="text_7">     <!-- turns -->     <g transform="translate(208.170312 248.56125) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-74" d="M 2406 205 Q 2278 90 2089 29 Q 1901 -32 1696 -32 Q 1222 -32 966 224 Q 710 480 710 947 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 4102 L 1165 4102 L 1165 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 973 Q 1165 672 1315 515 Q 1466 358 1747 358 Q 1888 358 2019 403 Q 2150 448 2246 531 L 2406 205 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-75" d="M 3667 3366 L 3667 0 L 3232 0 L 3232 614 Q 3053 307 2739 137 Q 2426 -32 2022 -32 Q 1363 -32 982 336 Q 602 704 602 1414 L 602 3366 L 1056 3366 L 1056 1459 Q 1056 928 1318 653 Q 1581 378 2067 378 Q 2598 378 2905 701 Q 3213 1024 3213 1600 L 3213 3366 L 3667 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-72" d="M 1069 2707 Q 1229 3046 1545 3219 Q 1862 3392 2330 3392 L 2330 2950 L 2221 2957 Q 1690 2957 1389 2630 Q 1088 2304 1088 1715 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2707 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-6e" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2746 Q 1254 3053 1584 3222 Q 1914 3392 2349 3392 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-73" d="M 1536 -32 Q 1126 -32 752 86 Q 378 205 166 384 L 371 742 Q 582 576 902 470 Q 1222 365 1568 365 Q 2029 365 2249 509 Q 2470 653 2470 915 Q 2470 1101 2348 1206 Q 2227 1312 2041 1366 Q 1856 1421 1549 1472 Q 1139 1549 889 1629 Q 640 1709 464 1901 Q 288 2093 288 2432 Q 288 2854 640 3123 Q 992 3392 1619 3392 Q 1946 3392 2272 3305 Q 2598 3219 2810 3078 L 2611 2714 Q 2195 3002 1619 3002 Q 1184 3002 963 2848 Q 742 2694 742 2445 Q 742 2253 867 2137 Q 992 2022 1177 1968 Q 1363 1914 1690 1856 Q 2093 1779 2336 1702 Q 2579 1626 2752 1440 Q 2925 1254 2925 928 Q 2925 486 2557 227 Q 2189 -32 1536 -32 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-74"/>      <use xlink:href="#Montserrat-Regular-75" x="40.599991"/>      <use xlink:href="#Montserrat-Regular-72" x="107.899979"/>      <use xlink:href="#Montserrat-Regular-6e" x="147.999969"/>      <use xlink:href="#Montserrat-Regular-73" x="215.699951"/>     </g>    </g>   </g>   <g id="matplotlib.axis_2">    <g id="ytick_1">     <g id="line2d_7"/>     <g id="text_8">      <!-- LOSS -->      <g transform="translate(23.91875 227.667831) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-4c" d="M 730 4480 L 1203 4480 L 1203 410 L 3712 410 L 3712 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4f" d="M 2688 -38 Q 2022 -38 1481 259 Q 941 557 637 1075 Q 333 1594 333 2240 Q 333 2886 637 3404 Q 941 3923 1481 4220 Q 2022 4518 2688 4518 Q 3354 4518 3888 4224 Q 4422 3930 4729 3408 Q 5037 2886 5037 2240 Q 5037 1594 4729 1072 Q 4422 550 3888 256 Q 3354 -38 2688 -38 zM 2688 384 Q 3219 384 3648 624 Q 4077 864 4320 1289 Q 4563 1715 4563 2240 Q 4563 2765 4320 3190 Q 4077 3616 3648 3856 Q 3219 4096 2688 4096 Q 2157 4096 1725 3856 Q 1293 3616 1046 3190 Q 800 2765 800 2240 Q 800 1715 1046 1289 Q 1293 864 1725 624 Q 2157 384 2688 384 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-53" d="M 1965 -38 Q 1466 -38 1008 122 Q 550 282 301 544 L 486 909 Q 730 666 1133 515 Q 1536 365 1965 365 Q 2566 365 2867 585 Q 3168 806 3168 1158 Q 3168 1427 3005 1587 Q 2842 1747 2605 1833 Q 2368 1920 1946 2022 Q 1440 2150 1139 2268 Q 838 2387 624 2630 Q 410 2874 410 3290 Q 410 3629 589 3907 Q 768 4186 1139 4352 Q 1510 4518 2061 4518 Q 2445 4518 2813 4412 Q 3181 4307 3450 4122 L 3290 3744 Q 3008 3930 2688 4022 Q 2368 4115 2061 4115 Q 1472 4115 1174 3888 Q 877 3661 877 3302 Q 877 3034 1040 2870 Q 1203 2707 1449 2620 Q 1696 2534 2112 2432 Q 2605 2310 2909 2192 Q 3213 2074 3424 1834 Q 3635 1594 3635 1184 Q 3635 845 3452 566 Q 3270 288 2892 125 Q 2515 -38 1965 -38 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-4c"/>       <use xlink:href="#Montserrat-Regular-4f" x="58.899994"/>       <use xlink:href="#Montserrat-Regular-53" x="142.799988"/>       <use xlink:href="#Montserrat-Regular-53" x="204.299973"/>      </g>     </g>    </g>    <g id="ytick_2">     <g id="line2d_8"/>     <g id="text_9">      <!-- -1000 -->      <g transform="translate(23.209375 195.435273) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-2d" d="M 384 1933 L 2061 1933 L 2061 1536 L 384 1536 L 384 1933 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-31" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="74.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="140.499985"/>       <use xlink:href="#Montserrat-Regular-30" x="206.699982"/>      </g>     </g>    </g>    <g id="ytick_3">     <g id="line2d_9"/>     <g id="text_10">      <!-- -500 -->      <g transform="translate(27.779687 163.202714) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-35" d="M 1510 2643 Q 2483 2643 2928 2294 Q 3373 1946 3373 1318 Q 3373 928 3190 621 Q 3008 314 2643 138 Q 2278 -38 1747 -38 Q 1267 -38 835 118 Q 403 275 141 538 L 365 902 Q 589 672 953 528 Q 1318 384 1741 384 Q 2298 384 2602 633 Q 2906 883 2906 1299 Q 2906 1760 2563 1997 Q 2221 2234 1408 2234 L 454 2234 L 685 4480 L 3136 4480 L 3136 4070 L 1088 4070 L 934 2643 L 1510 2643 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-35" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="94.799988"/>       <use xlink:href="#Montserrat-Regular-30" x="160.999985"/>      </g>     </g>    </g>    <g id="ytick_4">     <g id="line2d_10"/>     <g id="text_11">      <!-- 0 -->      <g transform="translate(43.879688 130.970156) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="ytick_5">     <g id="line2d_11"/>     <g id="text_12">      <!-- 500 -->      <g transform="translate(31.6 98.737598) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-35"/>       <use xlink:href="#Montserrat-Regular-30" x="56.599991"/>       <use xlink:href="#Montserrat-Regular-30" x="122.799988"/>      </g>     </g>    </g>    <g id="ytick_6">     <g id="line2d_12"/>     <g id="text_13">      <!-- 1000 -->      <g transform="translate(27.029687 66.50504) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="168.499985"/>      </g>     </g>    </g>    <g id="ytick_7">     <g id="line2d_13"/>     <g id="text_14">      <!-- WIN -->      <g transform="translate(28.240625 34.272482) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-57" d="M 6880 4480 L 5370 0 L 4870 0 L 3558 3821 L 2240 0 L 1747 0 L 237 4480 L 723 4480 L 2016 627 L 3354 4480 L 3795 4480 L 5114 608 L 6426 4480 L 6880 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-49" d="M 730 4480 L 1203 4480 L 1203 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4e" d="M 4474 4480 L 4474 0 L 4083 0 L 1203 3635 L 1203 0 L 730 0 L 730 4480 L 1120 4480 L 4006 845 L 4006 4480 L 4474 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-57"/>       <use xlink:href="#Montserrat-Regular-49" x="111.099991"/>       <use xlink:href="#Montserrat-Regular-4e" x="141.299988"/>      </g>     </g>    </g>    <g id="text_15">     <!-- mean centipawn difference -->     <g transform="translate(17.26875 196.6475) rotate(-90) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-6d" d="M 4832 3392 Q 5466 3392 5827 3027 Q 6189 2662 6189 1952 L 6189 0 L 5734 0 L 5734 1907 Q 5734 2432 5481 2707 Q 5229 2982 4768 2982 Q 4243 2982 3942 2659 Q 3642 2336 3642 1766 L 3642 0 L 3187 0 L 3187 1907 Q 3187 2432 2934 2707 Q 2682 2982 2214 2982 Q 1696 2982 1392 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2752 Q 1248 3059 1568 3225 Q 1888 3392 2304 3392 Q 2726 3392 3036 3213 Q 3347 3034 3501 2682 Q 3686 3014 4035 3203 Q 4384 3392 4832 3392 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-65" d="M 3565 1542 L 749 1542 Q 787 1018 1152 694 Q 1517 371 2074 371 Q 2387 371 2649 483 Q 2912 595 3104 813 L 3360 518 Q 3136 250 2800 109 Q 2464 -32 2061 -32 Q 1542 -32 1142 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 508 2566 Q 723 2957 1097 3174 Q 1472 3392 1939 3392 Q 2406 3392 2777 3174 Q 3149 2957 3360 2569 Q 3571 2182 3571 1683 L 3565 1542 zM 1939 3002 Q 1453 3002 1123 2691 Q 794 2381 749 1882 L 3136 1882 Q 3091 2381 2761 2691 Q 2432 3002 1939 3002 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-61" d="M 1811 3392 Q 2470 3392 2822 3062 Q 3174 2733 3174 2086 L 3174 0 L 2739 0 L 2739 525 Q 2586 262 2288 115 Q 1990 -32 1581 -32 Q 1018 -32 685 237 Q 352 506 352 947 Q 352 1376 662 1638 Q 973 1901 1651 1901 L 2720 1901 L 2720 2106 Q 2720 2541 2477 2768 Q 2234 2995 1766 2995 Q 1446 2995 1152 2889 Q 858 2784 646 2598 L 442 2938 Q 698 3155 1056 3273 Q 1414 3392 1811 3392 zM 1651 326 Q 2035 326 2310 502 Q 2586 678 2720 1011 L 2720 1562 L 1664 1562 Q 800 1562 800 960 Q 800 666 1024 496 Q 1248 326 1651 326 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-20" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-63" d="M 2029 -32 Q 1530 -32 1136 185 Q 742 403 518 796 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1136 3174 Q 1530 3392 2029 3392 Q 2464 3392 2806 3222 Q 3149 3053 3347 2726 L 3008 2496 Q 2842 2746 2586 2870 Q 2330 2995 2029 2995 Q 1664 2995 1373 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1293 918 995 Q 1082 698 1373 534 Q 1664 371 2029 371 Q 2330 371 2586 492 Q 2842 614 3008 864 L 3347 634 Q 3149 307 2803 137 Q 2458 -32 2029 -32 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-69" d="M 634 3366 L 1088 3366 L 1088 0 L 634 0 L 634 3366 zM 864 4102 Q 723 4102 627 4198 Q 531 4294 531 4429 Q 531 4557 627 4653 Q 723 4749 864 4749 Q 1005 4749 1101 4656 Q 1197 4563 1197 4435 Q 1197 4294 1101 4198 Q 1005 4102 864 4102 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-70" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1178 3827 787 Q 3610 397 3229 182 Q 2848 -32 2368 -32 Q 1958 -32 1628 137 Q 1299 307 1088 634 L 1088 -1242 L 634 -1242 L 634 3366 L 1069 3366 L 1069 2701 Q 1274 3034 1610 3213 Q 1946 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2364 Q 3264 2662 2976 2828 Q 2688 2995 2336 2995 Q 1978 2995 1693 2828 Q 1408 2662 1245 2364 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-77" d="M 5574 3366 L 4301 0 L 3872 0 L 2810 2758 L 1747 0 L 1318 0 L 51 3366 L 486 3366 L 1542 506 L 2624 3366 L 3014 3366 L 4090 506 L 5158 3366 L 5574 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-64" d="M 3706 4749 L 3706 0 L 3270 0 L 3270 666 Q 3066 326 2730 147 Q 2394 -32 1971 -32 Q 1498 -32 1114 185 Q 730 403 512 793 Q 294 1184 294 1683 Q 294 2182 512 2572 Q 730 2963 1114 3177 Q 1498 3392 1971 3392 Q 2381 3392 2710 3222 Q 3040 3053 3251 2726 L 3251 4749 L 3706 4749 zM 2010 371 Q 2362 371 2646 534 Q 2931 698 3094 998 Q 3258 1299 3258 1683 Q 3258 2067 3094 2368 Q 2931 2669 2646 2832 Q 2362 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-66" d="M 1747 4397 Q 1453 4397 1302 4237 Q 1152 4077 1152 3763 L 1152 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 0 L 710 0 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 3782 Q 710 4243 976 4512 Q 1242 4781 1728 4781 Q 1920 4781 2099 4726 Q 2278 4672 2400 4570 L 2246 4230 Q 2048 4397 1747 4397 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-6d"/>      <use xlink:href="#Montserrat-Regular-65" x="106.099991"/>      <use xlink:href="#Montserrat-Regular-61" x="166.499985"/>      <use xlink:href="#Montserrat-Regular-6e" x="225.499969"/>      <use xlink:href="#Montserrat-Regular-20" x="293.199951"/>      <use xlink:href="#Montserrat-Regular-63" x="319.399948"/>      <use xlink:href="#Montserrat-Regular-65" x="375.699936"/>      <use xlink:href="#Montserrat-Regular-6e" x="436.09993"/>      <use xlink:href="#Montserrat-Regular-74" x="503.799911"/>      <use xlink:href="#Montserrat-Regular-69" x="544.399902"/>      <use xlink:href="#Montserrat-Regular-70" x="571.299896"/>      <use xlink:href="#Montserrat-Regular-61" x="639.099884"/>      <use xlink:href="#Montserrat-Regular-77" x="698.099869"/>      <use xlink:href="#Montserrat-Regular-6e" x="785.999863"/>      <use xlink:href="#Montserrat-Regular-20" x="853.699844"/>      <use xlink:href="#Montserrat-Regular-64" x="879.899841"/>      <use xlink:href="#Montserrat-Regular-69" x="947.699829"/>      <use xlink:href="#Montserrat-Regular-66" x="974.599823"/>      <use xlink:href="#Montserrat-Regular-66" x="1008.499817"/>      <use xlink:href="#Montserrat-Regular-65" x="1042.399811"/>      <use xlink:href="#Montserrat-Regular-72" x="1102.799805"/>      <use xlink:href="#Montserrat-Regular-65" x="1142.899796"/>      <use xlink:href="#Montserrat-Regular-6e" x="1203.299789"/>      <use xlink:href="#Montserrat-Regular-63" x="1270.999771"/>      <use xlink:href="#Montserrat-Regular-65" x="1327.299759"/>     </g>    </g>   </g>   <g id="line2d_14">    <path d="M 54 127.26 L 57.348 123.102 L 60.696 121.960967 L 64.044 119.272772 L 67.392 119.504847 L 70.74 116.571684 L 74.088 117.02294 L 77.436 115.217916 L 80.784 115.662726 L 84.132 115.018074 L 87.48 114.760214 L 90.828 114.760214 L 94.176 104.310419 L 97.524 106.167014 L 100.872 98.869563 L 104.22 100.94534 L 107.568 98.031516 L 110.916 98.811544 L 114.264 93.01613 L 117.612 89.760642 L 120.96 91.862205 L 124.308 90.862995 L 127.656 90.347274 L 131.004 85.69934 L 134.352 79.220595 L 137.7 70.704753 L 141.048 65.547544 L 144.396 61.112344 L 147.744 57.360474 L 151.092 51.171823 L 154.44 51.004214 L 157.788 51.687544 L 161.136 46.382065 L 164.484 47.742279 L 167.832 46.053293 L 171.18 38.81386 L 174.528 41.269981 L 177.876 39.278009 L 181.224 37.260251 L 184.572 37.292484 L 187.92 35.023312 L 191.268 37.163553 L 194.616 35.197367 L 197.964 38.046726 L 201.312 36.725191 L 204.66 36.31906 L 208.008 35.706642 L 211.356 34.333535 L 214.704 34.333535 L 218.052 30.626791 L 221.4 30.626791 L 224.748 30.626791 L 228.096 30.626791 L 231.444 30.626791 L 234.792 30.626791 L 238.14 30.626791 L 241.488 30.626791 L 244.836 30.626791 L 248.184 30.626791 L 251.532 30.626791 L 254.88 30.626791 L 258.228 30.626791 L 261.576 30.626791 L 264.924 30.626791 L 268.272 30.626791 L 271.62 30.626791 L 274.968 30.626791 L 278.316 30.626791 L 281.664 30.626791 L 285.012 30.626791 L 288.36 30.626791 L 291.708 30.626791 L 295.056 30.626791 L 298.404 30.626791 L 301.752 30.626791 L 305.1 30.626791 L 308.448 30.626791 L 311.796 30.626791 L 315.144 30.626791 L 318.492 30.626791 L 321.84 30.626791 L 325.188 30.626791 L 328.536 30.626791 L 331.884 30.626791 L 335.232 30.626791 L 338.58 30.626791 L 341.928 30.626791 L 345.276 30.626791 L 348.624 30.626791 L 351.972 30.626791 L 355.32 30.626791 L 358.668 30.626791 L 362.016 30.626791 L 365.364 30.626791 L 368.712 30.626791 L 372.06 30.626791 L 375.408 30.626791 L 378.756 30.626791 L 382.104 30.626791 L 385.452 30.626791 " clip-path="url(#p3f4a6a0093)" style="fill: none; stroke: #1f77b4; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_15">    <path d="M 54 127.26 L 57.348 122.753888 L 60.696 120.710344 L 64.044 116.184893 L 67.392 115.018074 L 70.74 118.641014 L 74.088 119.20186 L 77.436 118.582995 L 80.784 114.431442 L 84.132 113.735219 L 87.48 112.220288 L 90.828 125.210009 L 94.176 122.263953 L 97.524 124.056084 L 100.872 126.312363 L 104.22 127.363144 L 107.568 128.66534 L 110.916 128.143172 L 114.264 130.373665 L 117.612 133.513116 L 120.96 132.37853 L 124.308 132.191581 L 127.656 139.076456 L 131.004 136.749265 L 134.352 142.203014 L 137.7 149.249051 L 141.048 149.062102 L 144.396 157.816465 L 147.744 157.513479 L 151.092 156.398233 L 154.44 165.848819 L 157.788 166.777116 L 161.136 168.865786 L 164.484 190.081256 L 167.832 197.178865 L 171.18 199.428698 L 174.528 205.249898 L 177.876 204.083079 L 181.224 205.095181 L 184.572 203.438428 L 187.92 205.462633 L 191.268 211.135563 L 194.616 211.67707 L 197.964 211.12267 L 201.312 216.905191 L 204.66 216.756921 L 208.008 213.959135 L 211.356 217.981758 L 214.704 218.697321 L 218.052 223.757833 L 221.4 223.738493 L 224.748 223.951228 L 228.096 223.951228 L 231.444 221.198567 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p3f4a6a0093)" style="fill: none; stroke: #ff7f0e; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_16">    <path d="M 54 127.26 L 57.348 123.099421 L 60.696 122.083451 L 64.044 120.683269 L 67.392 122.003514 L 70.74 121.775308 L 74.088 122.884108 L 77.436 123.63706 L 80.784 124.708471 L 84.132 128.502887 L 87.48 132.069098 L 90.828 136.269645 L 94.176 141.019434 L 97.524 142.814143 L 100.872 147.600033 L 104.22 151.622657 L 107.568 155.333269 L 110.916 160.313844 L 114.264 161.421354 L 117.612 166.356804 L 120.96 171.266467 L 124.308 173.582054 L 127.656 177.22949 L 131.004 179.118318 L 134.352 183.340783 L 137.7 186.038004 L 141.048 187.111993 L 144.396 191.843732 L 147.744 195.848305 L 151.092 200.895924 L 154.44 202.75123 L 157.788 205.216376 L 161.136 206.029926 L 164.484 207.103914 L 167.832 208.743907 L 171.18 209.84755 L 174.528 212.088357 L 177.876 213.162346 L 181.224 214.646333 L 184.572 215.989786 L 187.92 217.92374 L 191.268 217.668458 L 194.616 218.204807 L 197.964 217.99594 L 201.312 218.688296 L 204.66 218.369838 L 208.008 218.720528 L 211.356 219.78936 L 214.704 220.715079 L 218.052 220.837563 L 221.4 221.522182 L 224.748 221.457717 L 228.096 221.849665 L 231.444 222.018564 L 234.792 222.023721 L 238.14 222.973937 L 241.488 223.080949 L 244.836 223.00488 L 248.184 223.015194 L 251.532 222.999723 L 254.88 223.200854 L 258.228 223.172489 L 261.576 223.215036 L 264.924 223.24598 L 268.272 223.132521 L 271.62 223.216326 L 274.968 223.296262 L 278.316 223.289816 L 281.664 223.342677 L 285.012 223.157018 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p3f4a6a0093)" style="fill: none; stroke: #2ca02c; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_17">    <path d="M 54 127.26 L 57.348 122.67653 L 60.696 122.328419 L 64.044 119.878744 L 67.392 122.663637 L 70.74 122.470242 L 74.088 123.882028 L 77.436 128.974772 L 80.784 132.288279 L 84.132 134.003051 L 87.48 141.081321 L 90.828 141.326288 L 94.176 147.444028 L 97.524 152.865544 L 100.872 149.468233 L 104.22 157.236279 L 107.568 159.363628 L 110.916 164.353228 L 114.264 167.943935 L 117.612 174.190605 L 120.96 175.531479 L 124.308 174.164819 L 127.656 179.618567 L 131.004 185.639609 L 134.352 191.286753 L 137.7 196.392391 L 141.048 199.976651 L 144.396 204.373172 L 147.744 207.016242 L 151.092 212.624707 L 154.44 215.802837 L 157.788 215.583656 L 161.136 215.860856 L 164.484 215.093721 L 167.832 220.238037 L 171.18 219.651405 L 174.528 219.638512 L 177.876 219.116344 L 181.224 219.380651 L 184.572 219.664298 L 187.92 219.561153 L 191.268 219.135684 L 194.616 219.393544 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p3f4a6a0093)" style="fill: none; stroke: #d62728; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_18">    <path d="M 54 127.26 L 57.348 123.688633 L 60.696 122.450902 L 64.044 121.890056 L 67.392 119.279219 L 70.74 127.640344 L 74.088 127.472735 L 77.436 128.401033 L 80.784 124.752307 L 84.132 128.510623 L 87.48 131.347088 L 90.828 146.92186 L 94.176 153.838967 L 97.524 156.540056 L 100.872 163.386251 L 104.22 166.88026 L 107.568 167.544251 L 110.916 168.395191 L 114.264 179.160865 L 117.612 180.566205 L 120.96 181.662112 L 124.308 180.604884 L 127.656 192.414893 L 131.004 188.740381 L 134.352 202.23293 L 137.7 197.720372 L 141.048 201.665637 L 144.396 204.437637 L 147.744 206.13307 L 151.092 213.972028 L 154.44 214.790735 L 157.788 215.055042 L 161.136 218.091349 L 164.484 219.238828 L 167.832 218.529712 L 171.18 218.800465 L 174.528 219.612726 L 177.876 220.599042 L 181.224 219.593386 L 184.572 223.951228 L 187.92 218.426567 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p3f4a6a0093)" style="fill: none; stroke: #9467bd; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_19">    <path d="M 54 127.26 L 57.348 123.522313 L 60.696 122.91634 L 64.044 121.579334 L 67.392 125.102997 L 70.74 126.376828 L 74.088 125.715416 L 77.436 127.914966 L 80.784 129.798636 L 84.132 135.912508 L 87.48 138.175233 L 90.828 147.050791 L 94.176 150.677598 L 97.524 157.815176 L 100.872 161.00362 L 104.22 166.350357 L 107.568 170.276283 L 110.916 174.261516 L 114.264 178.388573 L 117.612 183.030061 L 120.96 190.243708 L 124.308 192.69725 L 127.656 194.935479 L 131.004 194.845228 L 134.352 200.34797 L 137.7 203.867766 L 141.048 206.357408 L 144.396 207.934225 L 147.744 210.503805 L 151.092 211.429524 L 154.44 213.527219 L 157.788 215.3735 L 161.136 215.092432 L 164.484 217.050882 L 167.832 217.27522 L 171.18 218.572259 L 174.528 218.711503 L 177.876 219.802253 L 181.224 220.397911 L 184.572 220.180019 L 187.92 221.08382 L 191.268 220.752469 L 194.616 221.513157 L 197.964 221.598251 L 201.312 221.854822 L 204.66 222.503341 L 208.008 222.392461 L 211.356 222.558781 L 214.704 222.6168 L 218.052 222.507209 L 221.4 222.46853 L 224.748 222.511077 L 228.096 222.3035 L 231.444 223.306577 L 234.792 223.102867 L 238.14 223.240822 L 241.488 223.213747 L 244.836 223.164753 L 248.184 223.222772 L 251.532 223.956385 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p3f4a6a0093)" style="fill: none; stroke: #8c564b; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_20">    <path d="M 54 127.26 L 57.348 123.312156 L 60.696 123.016906 L 64.044 123.13939 L 67.392 122.706184 L 70.74 122.515367 L 74.088 126.543148 L 77.436 130.158352 L 80.784 134.271226 L 84.132 139.563812 L 87.48 143.157098 L 90.828 148.551539 L 94.176 154.579027 L 97.524 158.04725 L 100.872 162.720971 L 104.22 166.247213 L 107.568 171.744798 L 110.916 177.259144 L 114.264 180.508186 L 117.612 184.133704 L 120.96 187.734726 L 124.308 191.245496 L 127.656 195.047648 L 131.004 196.968709 L 134.352 198.878166 L 137.7 201.773939 L 141.048 205.408482 L 144.396 208.62658 L 147.744 210.23563 L 151.092 213.133981 L 154.44 216.134188 L 157.788 216.033622 L 161.136 218.260247 L 164.484 218.452353 L 167.832 218.768233 L 171.18 219.791939 L 174.528 219.758417 L 177.876 221.104448 L 181.224 220.351496 L 184.572 221.930891 L 187.92 222.089475 L 191.268 222.493027 L 194.616 222.444033 L 197.964 222.473687 L 201.312 223.013905 L 204.66 222.952019 L 208.008 223.46645 L 211.356 223.956385 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p3f4a6a0093)" style="fill: none; stroke: #e377c2; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_21">    <path d="M 54 127.26 L 57.348 123.918128 L 60.696 122.738417 L 64.044 122.757756 L 67.392 124.695578 L 70.74 128.576378 L 74.088 131.209133 L 77.436 137.044515 L 80.784 140.276796 L 84.132 143.930679 L 87.48 146.777459 L 90.828 149.791847 L 94.176 157.347159 L 97.524 161.296292 L 100.872 166.877682 L 104.22 173.926298 L 107.568 178.673509 L 110.916 185.296655 L 114.264 186.130833 L 117.612 186.690391 L 120.96 190.061916 L 124.308 192.409736 L 127.656 195.117271 L 131.004 195.808337 L 134.352 200.417593 L 137.7 203.664056 L 141.048 204.571725 L 144.396 207.927779 L 147.744 210.243366 L 151.092 211.995527 L 154.44 214.550925 L 157.788 214.299511 L 161.136 215.600417 L 164.484 216.898744 L 167.832 218.345341 L 171.18 219.191124 L 174.528 219.409016 L 177.876 220.792437 L 181.224 221.559572 L 184.572 221.522182 L 187.92 222.588435 L 191.268 223.199565 L 194.616 223.953807 L 197.964 223.956385 L 201.312 223.956385 L 204.66 223.955096 L 208.008 223.956385 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p3f4a6a0093)" style="fill: none; stroke: #7f7f7f; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_22">    <path d="M 54 127.26 L 57.348 122.881529 L 60.696 123.457847 L 64.044 122.848007 L 67.392 124.310076 L 70.74 126.63211 L 74.088 129.374456 L 77.436 136.459172 L 80.784 142.922445 L 84.132 145.693155 L 87.48 151.43184 L 90.828 153.609472 L 94.176 159.608595 L 97.524 166.276867 L 100.872 170.361377 L 104.22 173.221049 L 107.568 176.810467 L 110.916 179.762969 L 114.264 187.06042 L 117.612 189.136197 L 120.96 193.165267 L 124.308 195.82123 L 127.656 198.461721 L 131.004 200.955232 L 134.352 202.884028 L 137.7 204.82056 L 141.048 206.180774 L 144.396 207.275392 L 147.744 208.669127 L 151.092 210.238208 L 154.44 213.626495 L 157.788 214.884854 L 161.136 216.459092 L 164.484 217.618175 L 167.832 219.499267 L 171.18 220.506212 L 174.528 220.717658 L 177.876 221.17536 L 181.224 221.264322 L 184.572 221.34168 L 187.92 221.15602 L 191.268 220.961336 L 194.616 221.132813 L 197.964 221.306869 L 201.312 221.584069 L 204.66 221.790357 L 208.008 221.80454 L 211.356 222.094633 L 214.704 222.182305 L 218.052 222.503341 L 221.4 222.558781 L 224.748 222.335732 L 228.096 222.952019 L 231.444 223.431639 L 234.792 223.472897 L 238.14 223.441953 L 241.488 223.471607 L 244.836 223.542519 L 248.184 223.54123 L 251.532 223.570884 L 254.88 223.573462 L 258.228 223.35557 L 261.576 223.474186 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p3f4a6a0093)" style="fill: none; stroke: #bcbd22; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_23">    <path d="M 54 127.26 L 57.348 123.58249 L 60.696 123.006802 L 64.044 122.432612 L 67.392 122.87937 L 70.74 124.04424 L 74.088 128.552301 L 77.436 133.388684 L 80.784 136.219152 L 84.132 139.632805 L 87.48 148.635433 L 90.828 157.772989 L 94.176 162.415976 L 97.524 170.600048 L 100.872 173.931245 L 104.22 177.653731 L 107.568 178.413819 L 110.916 184.654942 L 114.264 188.524348 L 117.612 189.206479 L 120.96 193.338243 L 124.308 197.269116 L 127.656 199.585363 L 131.004 200.954122 L 134.352 204.492208 L 137.7 205.289776 L 141.048 206.139816 L 144.396 209.530981 L 147.744 211.333006 L 151.092 214.560759 L 154.44 216.373279 L 157.788 214.251926 L 161.136 216.124413 L 164.484 216.788554 L 167.832 218.008893 L 171.18 217.635595 L 174.528 218.017889 L 177.876 219.287701 L 181.224 219.989322 L 184.572 219.908366 L 187.92 219.819913 L 191.268 219.593536 L 194.616 220.674451 L 197.964 220.885837 L 201.312 221.672911 L 204.66 221.1392 L 208.008 221.585958 L 211.356 221.831825 L 214.704 223.101638 L 218.052 223.098639 L 221.4 222.957716 L 224.748 223.956175 L 228.096 223.956175 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p3f4a6a0093)" style="fill: none; stroke: #17becf; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_24">    <path d="M 54 127.26 L 57.348 122.653323 L 60.696 122.410934 L 64.044 122.216249 L 67.392 124.294605 L 70.74 125.995194 L 74.088 128.914175 L 77.436 133.977265 L 80.784 139.746893 L 84.132 144.509576 L 87.48 151.168822 L 90.828 155.115377 L 94.176 161.594121 L 97.524 165.224796 L 100.872 169.974586 L 104.22 173.12822 L 107.568 177.682035 L 110.916 182.408618 L 114.264 188.985349 L 117.612 194.831046 L 120.96 197.769366 L 124.308 204.620718 L 127.656 204.959805 L 131.004 205.870052 L 134.352 207.342435 L 137.7 209.0701 L 141.048 209.673494 L 144.396 213.442125 L 147.744 215.311613 L 151.092 217.730344 L 154.44 218.042355 L 157.788 220.75118 L 161.136 221.154731 L 164.484 221.462874 L 167.832 221.801961 L 171.18 221.76715 L 174.528 222.877239 L 177.876 223.327206 L 181.224 223.342677 L 184.572 223.297552 L 187.92 223.285948 L 191.268 223.221483 L 194.616 223.162175 L 197.964 223.141546 L 201.312 223.363306 L 204.66 223.408432 L 208.008 223.403274 L 211.356 223.497393 L 214.704 223.382646 L 218.052 223.002301 L 221.4 222.887553 L 224.748 222.388593 L 228.096 222.942993 L 231.444 222.959754 L 234.792 223.069345 L 238.14 223.108024 L 241.488 223.044848 L 244.836 223.04098 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p3f4a6a0093)" style="fill: none; stroke: #1f77b4; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_25">    <path d="M 54 127.26 L 57.348 122.976 L 60.696 122.085209 L 64.044 123.685116 L 67.392 126.931814 L 70.74 135.745953 L 74.088 146.675721 L 77.436 153.110512 L 80.784 149.951721 L 84.132 160.746698 L 87.48 160.670512 L 90.828 168.634884 L 94.176 173.135721 L 97.524 180.666419 L 100.872 182.922698 L 104.22 187.282884 L 107.568 193.735256 L 110.916 197.521116 L 114.264 200.070419 L 117.612 202.514233 L 120.96 205.87814 L 124.308 204.190326 L 127.656 205.315535 L 131.004 203.903163 L 134.352 209.453023 L 137.7 211.111535 L 141.048 208.562233 L 144.396 211.691721 L 147.744 211.644837 L 151.092 213.865953 L 154.44 214.164837 L 157.788 218.431256 L 161.136 218.214419 L 164.484 221.619349 L 167.832 220.007721 L 171.18 221.039163 L 174.528 221.771721 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p3f4a6a0093)" style="fill: none; stroke-dasharray: 0.5,0.825; stroke-dashoffset: 0; stroke: #ff7f0e; stroke-width: 0.5"/>   </g>   <g id="patch_3">    <path d="M 54 224.28 L 54 30.24 " style="fill: none"/>   </g>   <g id="patch_4">    <path d="M 388.8 224.28 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="patch_5">    <path d="M 54 224.28 L 388.8 224.28 " style="fill: none"/>   </g>   <g id="patch_6">    <path d="M 54 30.24 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="text_16">    <g id="patch_7">     <a xlink:href="https://dynomight.net/chess">      <path d="M 323.8664 225.676606 L 388.881244 225.676606 L 388.881244 213.343012 L 323.8664 213.343012 z" style="fill: #ffffff; opacity: 0.000001; stroke: #ffffff; stroke-linejoin: miter"/>     </a>    </g>    <a xlink:href="https://dynomight.net/chess">     <!-- dynomight.net/chess -->     <g style="opacity: 0.3" transform="translate(327.8664 220.734419) scale(0.05 -0.05)">      <defs>       <path id="CourierNewPSMT-64" d="M 3206 3922 L 3206 263 L 3556 263 Q 3653 263 3693 227 Q 3734 191 3734 131 Q 3734 75 3693 37 Q 3653 0 3556 0 L 2941 0 L 2941 522 Q 2484 -103 1781 -103 Q 1425 -103 1098 86 Q 772 275 583 625 Q 394 975 394 1347 Q 394 1722 583 2070 Q 772 2419 1098 2609 Q 1425 2800 1784 2800 Q 2472 2800 2941 2175 L 2941 3656 L 2591 3656 Q 2494 3656 2453 3693 Q 2413 3731 2413 3791 Q 2413 3847 2453 3884 Q 2494 3922 2591 3922 L 3206 3922 zM 2941 1347 Q 2941 1847 2603 2190 Q 2266 2534 1800 2534 Q 1331 2534 993 2190 Q 656 1847 656 1347 Q 656 850 993 504 Q 1331 159 1800 159 Q 2266 159 2603 504 Q 2941 850 2941 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-79" d="M 1938 0 L 719 2441 L 641 2441 Q 547 2441 506 2478 Q 466 2516 466 2572 Q 466 2613 486 2645 Q 506 2678 539 2692 Q 572 2706 641 2706 L 1359 2706 Q 1453 2706 1493 2668 Q 1534 2631 1534 2572 Q 1534 2516 1493 2478 Q 1453 2441 1359 2441 L 1006 2441 L 2078 288 L 3134 2441 L 2781 2441 Q 2688 2441 2647 2478 Q 2606 2516 2606 2575 Q 2606 2631 2647 2668 Q 2688 2706 2781 2706 L 3497 2706 Q 3594 2706 3634 2668 Q 3675 2631 3675 2572 Q 3675 2531 3650 2497 Q 3625 2463 3594 2452 Q 3563 2441 3419 2441 L 1756 -944 L 2166 -944 Q 2259 -944 2300 -980 Q 2341 -1016 2341 -1075 Q 2341 -1131 2300 -1168 Q 2259 -1206 2166 -1206 L 653 -1206 Q 559 -1206 518 -1170 Q 478 -1134 478 -1075 Q 478 -1016 518 -980 Q 559 -944 653 -944 L 1475 -944 L 1938 0 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6e" d="M 1072 2706 L 1072 2309 Q 1347 2588 1569 2694 Q 1791 2800 2069 2800 Q 2369 2800 2616 2672 Q 2791 2578 2933 2361 Q 3075 2144 3075 1916 L 3075 263 L 3297 263 Q 3391 263 3431 227 Q 3472 191 3472 131 Q 3472 75 3431 37 Q 3391 0 3297 0 L 2594 0 Q 2497 0 2456 37 Q 2416 75 2416 131 Q 2416 191 2456 227 Q 2497 263 2594 263 L 2813 263 L 2813 1872 Q 2813 2150 2609 2342 Q 2406 2534 2066 2534 Q 1806 2534 1615 2429 Q 1425 2325 1072 1909 L 1072 263 L 1369 263 Q 1463 263 1503 227 Q 1544 191 1544 131 Q 1544 75 1503 37 Q 1463 0 1369 0 L 513 0 Q 419 0 378 37 Q 338 75 338 131 Q 338 191 378 227 Q 419 263 513 263 L 809 263 L 809 2441 L 588 2441 Q 494 2441 453 2478 Q 413 2516 413 2575 Q 413 2631 453 2668 Q 494 2706 588 2706 L 1072 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6f" d="M 3388 1347 Q 3388 747 2958 322 Q 2528 -103 1922 -103 Q 1309 -103 881 323 Q 453 750 453 1347 Q 453 1947 881 2373 Q 1309 2800 1922 2800 Q 2528 2800 2958 2375 Q 3388 1950 3388 1347 zM 3122 1347 Q 3122 1841 2770 2187 Q 2419 2534 1919 2534 Q 1419 2534 1067 2186 Q 716 1838 716 1347 Q 716 859 1067 509 Q 1419 159 1919 159 Q 2419 159 2770 507 Q 3122 856 3122 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6d" d="M 725 2706 L 725 2441 Q 1059 2800 1397 2800 Q 1600 2800 1753 2692 Q 1906 2584 2009 2366 Q 2184 2584 2364 2692 Q 2544 2800 2725 2800 Q 3009 2800 3178 2616 Q 3400 2378 3400 2097 L 3400 263 L 3622 263 Q 3716 263 3756 227 Q 3797 191 3797 131 Q 3797 75 3756 37 Q 3716 0 3622 0 L 3138 0 L 3138 2072 Q 3138 2272 3016 2403 Q 2894 2534 2734 2534 Q 2591 2534 2431 2426 Q 2272 2319 2069 2003 L 2069 263 L 2288 263 Q 2381 263 2422 227 Q 2463 191 2463 131 Q 2463 75 2422 37 Q 2381 0 2288 0 L 1803 0 L 1803 2053 Q 1803 2263 1679 2398 Q 1556 2534 1403 2534 Q 1263 2534 1125 2444 Q 934 2316 725 2003 L 725 263 L 947 263 Q 1041 263 1081 227 Q 1122 191 1122 131 Q 1122 75 1081 37 Q 1041 0 947 0 L 241 0 Q 147 0 106 37 Q 66 75 66 131 Q 66 191 106 227 Q 147 263 241 263 L 463 263 L 463 2441 L 241 2441 Q 147 2441 106 2478 Q 66 2516 66 2575 Q 66 2631 106 2668 Q 147 2706 241 2706 L 725 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-69" d="M 2050 4053 L 2050 3375 L 1663 3375 L 1663 4053 L 2050 4053 zM 2059 2706 L 2059 263 L 3091 263 Q 3188 263 3228 227 Q 3269 191 3269 131 Q 3269 75 3228 37 Q 3188 0 3091 0 L 766 0 Q 672 0 631 37 Q 591 75 591 131 Q 591 191 631 227 Q 672 263 766 263 L 1797 263 L 1797 2441 L 1031 2441 Q 938 2441 895 2478 Q 853 2516 853 2572 Q 853 2631 893 2668 Q 934 2706 1031 2706 L 2059 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-67" d="M 2803 2225 L 2803 2706 L 3419 2706 Q 3513 2706 3553 2668 Q 3594 2631 3594 2572 Q 3594 2516 3553 2478 Q 3513 2441 3419 2441 L 3066 2441 L 3066 -181 Q 3066 -444 2953 -650 Q 2878 -788 2703 -934 Q 2528 -1081 2384 -1143 Q 2241 -1206 2000 -1206 L 1256 -1206 Q 1163 -1206 1122 -1170 Q 1081 -1134 1081 -1075 Q 1081 -1016 1122 -978 Q 1163 -941 1256 -941 L 2009 -944 Q 2241 -944 2427 -825 Q 2613 -706 2734 -478 Q 2803 -347 2803 -144 L 2803 644 Q 2381 66 1716 66 Q 1175 66 783 467 Q 391 869 391 1434 Q 391 2000 783 2400 Q 1175 2800 1716 2800 Q 2381 2800 2803 2225 zM 2803 1434 Q 2803 1897 2486 2215 Q 2169 2534 1728 2534 Q 1288 2534 972 2214 Q 656 1894 656 1434 Q 656 972 972 651 Q 1288 331 1728 331 Q 2169 331 2486 651 Q 2803 972 2803 1434 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-68" d="M 1056 3922 L 1056 2306 Q 1306 2578 1536 2689 Q 1766 2800 2050 2800 Q 2356 2800 2570 2692 Q 2784 2584 2928 2361 Q 3072 2138 3072 1891 L 3072 263 L 3366 263 Q 3463 263 3502 227 Q 3541 191 3541 131 Q 3541 75 3502 37 Q 3463 0 3366 0 L 2513 0 Q 2416 0 2375 37 Q 2334 75 2334 131 Q 2334 191 2375 227 Q 2416 263 2513 263 L 2806 263 L 2806 1872 Q 2806 2156 2601 2347 Q 2397 2538 2031 2538 Q 1744 2538 1541 2397 Q 1394 2297 1056 1925 L 1056 263 L 1353 263 Q 1447 263 1487 227 Q 1528 191 1528 131 Q 1528 75 1487 37 Q 1447 0 1353 0 L 497 0 Q 403 0 362 37 Q 322 75 322 131 Q 322 191 362 227 Q 403 263 497 263 L 794 263 L 794 3656 L 441 3656 Q 347 3656 306 3693 Q 266 3731 266 3791 Q 266 3847 306 3884 Q 347 3922 441 3922 L 1056 3922 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-74" d="M 1381 2706 L 2806 2706 Q 2900 2706 2940 2668 Q 2981 2631 2981 2572 Q 2981 2516 2940 2478 Q 2900 2441 2806 2441 L 1381 2441 L 1381 694 Q 1381 466 1564 312 Q 1747 159 2100 159 Q 2366 159 2675 239 Q 2984 319 3156 419 Q 3219 459 3259 459 Q 3309 459 3346 420 Q 3384 381 3384 328 Q 3384 281 3344 241 Q 3244 138 2858 17 Q 2472 -103 2119 -103 Q 1659 -103 1387 112 Q 1116 328 1116 694 L 1116 2441 L 631 2441 Q 538 2441 497 2478 Q 456 2516 456 2575 Q 456 2631 497 2668 Q 538 2706 631 2706 L 1116 2706 L 1116 3481 Q 1116 3575 1153 3615 Q 1191 3656 1247 3656 Q 1306 3656 1343 3615 Q 1381 3575 1381 3481 L 1381 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2e" d="M 1866 753 L 1978 753 Q 2153 753 2278 629 Q 2403 506 2403 331 Q 2403 150 2276 28 Q 2150 -94 1978 -94 L 1866 -94 Q 1691 -94 1566 29 Q 1441 153 1441 328 Q 1441 509 1567 631 Q 1694 753 1866 753 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-65" d="M 3341 1291 L 666 1291 Q 734 781 1092 470 Q 1450 159 1978 159 Q 2272 159 2594 256 Q 2916 353 3119 513 Q 3178 559 3222 559 Q 3272 559 3309 520 Q 3347 481 3347 428 Q 3347 375 3297 325 Q 3147 169 2764 33 Q 2381 -103 1978 -103 Q 1303 -103 851 339 Q 400 781 400 1409 Q 400 1981 823 2390 Q 1247 2800 1872 2800 Q 2516 2800 2931 2379 Q 3347 1959 3341 1291 zM 3075 1556 Q 2997 1991 2664 2262 Q 2331 2534 1872 2534 Q 1413 2534 1081 2265 Q 750 1997 669 1556 L 3075 1556 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2f" d="M 3100 4113 L 988 -413 Q 934 -528 850 -528 Q 797 -528 758 -489 Q 719 -450 719 -403 Q 719 -366 750 -300 L 2859 4225 Q 2894 4297 2925 4319 Q 2956 4341 3000 4341 Q 3053 4341 3092 4302 Q 3131 4263 3131 4216 Q 3131 4181 3100 4113 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-63" d="M 3016 2441 L 3016 2528 Q 3016 2625 3053 2665 Q 3091 2706 3147 2706 Q 3206 2706 3243 2665 Q 3281 2625 3281 2528 L 3281 1934 Q 3278 1838 3242 1797 Q 3206 1756 3147 1756 Q 3094 1756 3058 1792 Q 3022 1828 3016 1913 Q 2997 2134 2723 2334 Q 2450 2534 1988 2534 Q 1403 2534 1100 2168 Q 797 1803 797 1331 Q 797 822 1131 490 Q 1466 159 1997 159 Q 2303 159 2620 271 Q 2938 384 3194 634 Q 3259 697 3309 697 Q 3363 697 3398 661 Q 3434 625 3434 572 Q 3434 438 3119 231 Q 2609 -103 1984 -103 Q 1350 -103 942 301 Q 534 706 534 1328 Q 534 1963 951 2381 Q 1369 2800 2003 2800 Q 2606 2800 3016 2441 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-73" d="M 2788 2534 Q 2788 2625 2825 2665 Q 2863 2706 2919 2706 Q 2978 2706 3015 2665 Q 3053 2625 3053 2528 L 3053 2078 Q 3053 1984 3015 1943 Q 2978 1903 2919 1903 Q 2866 1903 2830 1937 Q 2794 1972 2788 2050 Q 2769 2238 2594 2359 Q 2338 2534 1916 2534 Q 1475 2534 1231 2356 Q 1047 2222 1047 2056 Q 1047 1869 1266 1744 Q 1416 1656 1834 1609 Q 2381 1550 2594 1475 Q 2897 1366 3045 1172 Q 3194 978 3194 753 Q 3194 419 2872 158 Q 2550 -103 1928 -103 Q 1306 -103 909 213 Q 909 106 896 75 Q 884 44 851 22 Q 819 0 778 0 Q 722 0 684 40 Q 647 81 647 175 L 647 716 Q 647 809 683 850 Q 719 891 778 891 Q 834 891 873 852 Q 913 813 913 747 Q 913 603 984 506 Q 1094 356 1333 257 Q 1572 159 1919 159 Q 2431 159 2681 350 Q 2931 541 2931 753 Q 2931 997 2678 1144 Q 2422 1291 1933 1341 Q 1444 1391 1231 1472 Q 1019 1553 900 1715 Q 781 1878 781 2066 Q 781 2403 1112 2601 Q 1444 2800 1903 2800 Q 2447 2800 2788 2534 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#CourierNewPSMT-64"/>      <use xlink:href="#CourierNewPSMT-79" x="60.009766"/>      <use xlink:href="#CourierNewPSMT-6e" x="120.019531"/>      <use xlink:href="#CourierNewPSMT-6f" x="180.029297"/>      <use xlink:href="#CourierNewPSMT-6d" x="240.039062"/>      <use xlink:href="#CourierNewPSMT-69" x="300.048828"/>      <use xlink:href="#CourierNewPSMT-67" x="360.058594"/>      <use xlink:href="#CourierNewPSMT-68" x="420.068359"/>      <use xlink:href="#CourierNewPSMT-74" x="480.078125"/>      <use xlink:href="#CourierNewPSMT-2e" x="540.087891"/>      <use xlink:href="#CourierNewPSMT-6e" x="600.097656"/>      <use xlink:href="#CourierNewPSMT-65" x="660.107422"/>      <use xlink:href="#CourierNewPSMT-74" x="720.117188"/>      <use xlink:href="#CourierNewPSMT-2f" x="780.126953"/>      <use xlink:href="#CourierNewPSMT-63" x="840.136719"/>      <use xlink:href="#CourierNewPSMT-68" x="900.146484"/>      <use xlink:href="#CourierNewPSMT-65" x="960.15625"/>      <use xlink:href="#CourierNewPSMT-73" x="1020.166016"/>      <use xlink:href="#CourierNewPSMT-73" x="1080.175781"/>     </g>    </a>   </g>   <g id="legend_1">    <g id="line2d_26">     <path d="M 242.532813 43.160313 L 252.532813 43.160313 L 262.532812 43.160313 " style="fill: none; stroke: #1f77b4; stroke-width: 0.5; stroke-linecap: square"/>    </g>    <g id="text_17">     <!-- gpt-3.5-turbo-instruct -->     <g style="fill: #1f77b4" transform="translate(270.532812 46.660313) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-67" d="M 3750 3366 L 3750 410 Q 3750 -448 3331 -861 Q 2912 -1274 2067 -1274 Q 1600 -1274 1181 -1136 Q 762 -998 499 -755 L 730 -410 Q 973 -627 1321 -748 Q 1670 -870 2054 -870 Q 2694 -870 2995 -572 Q 3296 -275 3296 352 L 3296 781 Q 3085 461 2742 294 Q 2400 128 1984 128 Q 1510 128 1123 336 Q 736 544 515 918 Q 294 1293 294 1766 Q 294 2240 515 2611 Q 736 2982 1120 3187 Q 1504 3392 1984 3392 Q 2413 3392 2758 3219 Q 3104 3046 3315 2720 L 3315 3366 L 3750 3366 zM 2029 531 Q 2394 531 2688 688 Q 2982 845 3145 1126 Q 3309 1408 3309 1766 Q 3309 2125 3145 2403 Q 2982 2682 2691 2838 Q 2400 2995 2029 2995 Q 1664 2995 1373 2841 Q 1082 2688 918 2406 Q 755 2125 755 1766 Q 755 1408 918 1126 Q 1082 845 1373 688 Q 1664 531 2029 531 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-33" d="M 1856 2560 Q 2560 2522 2928 2179 Q 3296 1837 3296 1286 Q 3296 902 3110 604 Q 2925 307 2560 134 Q 2195 -38 1670 -38 Q 1184 -38 752 118 Q 320 275 58 538 L 282 902 Q 506 672 877 528 Q 1248 384 1670 384 Q 2221 384 2521 624 Q 2822 864 2822 1286 Q 2822 1702 2521 1939 Q 2221 2176 1613 2176 L 1293 2176 L 1293 2515 L 2534 4070 L 243 4070 L 243 4480 L 3123 4480 L 3123 4154 L 1856 2560 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-2e" d="M 678 -32 Q 538 -32 438 70 Q 339 173 339 320 Q 339 467 438 566 Q 538 666 678 666 Q 819 666 921 566 Q 1024 467 1024 320 Q 1024 173 921 70 Q 819 -32 678 -32 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-62" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1184 3827 793 Q 3610 403 3226 185 Q 2842 -32 2368 -32 Q 1946 -32 1610 147 Q 1274 326 1069 666 L 1069 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2726 Q 1299 3053 1628 3222 Q 1958 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2368 Q 3264 2669 2976 2832 Q 2688 2995 2336 2995 Q 1978 2995 1693 2832 Q 1408 2669 1245 2368 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-6f" d="M 2010 -32 Q 1523 -32 1132 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1132 3174 Q 1523 3392 2010 3392 Q 2496 3392 2886 3174 Q 3277 2957 3497 2566 Q 3718 2176 3718 1683 Q 3718 1190 3497 800 Q 3277 410 2886 189 Q 2496 -32 2010 -32 zM 2010 371 Q 2368 371 2653 534 Q 2938 698 3098 998 Q 3258 1299 3258 1683 Q 3258 2067 3098 2368 Q 2938 2669 2653 2832 Q 2368 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-67"/>      <use xlink:href="#Montserrat-Regular-70" x="68.599991"/>      <use xlink:href="#Montserrat-Regular-74" x="136.399979"/>      <use xlink:href="#Montserrat-Regular-2d" x="176.999969"/>      <use xlink:href="#Montserrat-Regular-33" x="215.199966"/>      <use xlink:href="#Montserrat-Regular-2e" x="271.59996"/>      <use xlink:href="#Montserrat-Regular-35" x="292.799957"/>      <use xlink:href="#Montserrat-Regular-2d" x="349.399948"/>      <use xlink:href="#Montserrat-Regular-74" x="387.599945"/>      <use xlink:href="#Montserrat-Regular-75" x="428.199936"/>      <use xlink:href="#Montserrat-Regular-72" x="495.499924"/>      <use xlink:href="#Montserrat-Regular-62" x="535.599915"/>      <use xlink:href="#Montserrat-Regular-6f" x="603.399902"/>      <use xlink:href="#Montserrat-Regular-2d" x="666.099899"/>      <use xlink:href="#Montserrat-Regular-69" x="704.299896"/>      <use xlink:href="#Montserrat-Regular-6e" x="731.19989"/>      <use xlink:href="#Montserrat-Regular-73" x="798.899872"/>      <use xlink:href="#Montserrat-Regular-74" x="847.799866"/>      <use xlink:href="#Montserrat-Regular-72" x="888.399857"/>      <use xlink:href="#Montserrat-Regular-75" x="928.499847"/>      <use xlink:href="#Montserrat-Regular-63" x="995.799835"/>      <use xlink:href="#Montserrat-Regular-74" x="1052.099823"/>     </g>    </g>    <g id="line2d_27">     <path d="M 242.532813 57.57125 L 252.532813 57.57125 L 262.532812 57.57125 " style="fill: none; stroke: #ff7f0e; stroke-width: 0.5; stroke-linecap: square"/>    </g>    <g id="text_18">     <!-- gpt-4o -->     <g style="fill: #ff7f0e" transform="translate(270.532812 61.07125) scale(0.1 -0.1)">      <use xlink:href="#Montserrat-Regular-67"/>      <use xlink:href="#Montserrat-Regular-70" x="68.599991"/>      <use xlink:href="#Montserrat-Regular-74" x="136.399979"/>      <use xlink:href="#Montserrat-Regular-2d" x="176.999969"/>      <use xlink:href="#Montserrat-Regular-34" x="215.199966"/>      <use xlink:href="#Montserrat-Regular-6f" x="281.299957"/>     </g>    </g>    <g id="line2d_28">     <path d="M 242.532813 71.982188 L 252.532813 71.982188 L 262.532812 71.982188 " style="fill: none; stroke: #2ca02c; stroke-width: 0.5; stroke-linecap: square"/>    </g>    <g id="text_19">     <!-- llama-3.1-70b -->     <g style="fill: #2ca02c" transform="translate(270.532812 75.482188) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-6c" d="M 634 4749 L 1088 4749 L 1088 0 L 634 0 L 634 4749 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-37" d="M 3494 4480 L 3494 4154 L 1619 0 L 1120 0 L 2963 4070 L 659 4070 L 659 3232 L 205 3232 L 205 4480 L 3494 4480 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-6c"/>      <use xlink:href="#Montserrat-Regular-6c" x="26.899994"/>      <use xlink:href="#Montserrat-Regular-61" x="53.799988"/>      <use xlink:href="#Montserrat-Regular-6d" x="112.799973"/>      <use xlink:href="#Montserrat-Regular-61" x="218.899963"/>      <use xlink:href="#Montserrat-Regular-2d" x="277.899948"/>      <use xlink:href="#Montserrat-Regular-33" x="316.099945"/>      <use xlink:href="#Montserrat-Regular-2e" x="372.499939"/>      <use xlink:href="#Montserrat-Regular-31" x="393.699936"/>      <use xlink:href="#Montserrat-Regular-2d" x="429.799927"/>      <use xlink:href="#Montserrat-Regular-37" x="467.999924"/>      <use xlink:href="#Montserrat-Regular-30" x="526.899918"/>      <use xlink:href="#Montserrat-Regular-62" x="593.099915"/>     </g>    </g>    <g id="line2d_29">     <path d="M 242.532813 86.343125 L 252.532813 86.343125 L 262.532812 86.343125 " style="fill: none; stroke: #d62728; stroke-width: 0.5; stroke-linecap: square"/>    </g>    <g id="text_20">     <!-- gpt-4o-mini -->     <g style="fill: #d62728" transform="translate(270.532812 89.843125) scale(0.1 -0.1)">      <use xlink:href="#Montserrat-Regular-67"/>      <use xlink:href="#Montserrat-Regular-70" x="68.599991"/>      <use xlink:href="#Montserrat-Regular-74" x="136.399979"/>      <use xlink:href="#Montserrat-Regular-2d" x="176.999969"/>      <use xlink:href="#Montserrat-Regular-34" x="215.199966"/>      <use xlink:href="#Montserrat-Regular-6f" x="281.299957"/>      <use xlink:href="#Montserrat-Regular-2d" x="343.999954"/>      <use xlink:href="#Montserrat-Regular-6d" x="382.199951"/>      <use xlink:href="#Montserrat-Regular-69" x="488.299942"/>      <use xlink:href="#Montserrat-Regular-6e" x="515.199936"/>      <use xlink:href="#Montserrat-Regular-69" x="582.899918"/>     </g>    </g>    <g id="line2d_30">     <path d="M 242.532813 100.754063 L 252.532813 100.754063 L 262.532812 100.754063 " style="fill: none; stroke: #9467bd; stroke-width: 0.5; stroke-linecap: square"/>    </g>    <g id="text_21">     <!-- o1-mini -->     <g style="fill: #9467bd" transform="translate(270.532812 104.254063) scale(0.1 -0.1)">      <use xlink:href="#Montserrat-Regular-6f"/>      <use xlink:href="#Montserrat-Regular-31" x="62.699997"/>      <use xlink:href="#Montserrat-Regular-2d" x="98.799988"/>      <use xlink:href="#Montserrat-Regular-6d" x="136.999985"/>      <use xlink:href="#Montserrat-Regular-69" x="243.099976"/>      <use xlink:href="#Montserrat-Regular-6e" x="269.999969"/>      <use xlink:href="#Montserrat-Regular-69" x="337.699951"/>     </g>    </g>    <g id="line2d_31">     <path d="M 242.532813 115.115 L 252.532813 115.115 L 262.532812 115.115 " style="fill: none; stroke: #8c564b; stroke-width: 0.5; stroke-linecap: square"/>    </g>    <g id="text_22">     <!-- llama-3.1-70b-instruct -->     <g style="fill: #8c564b" transform="translate(270.532812 118.615) scale(0.1 -0.1)">      <use xlink:href="#Montserrat-Regular-6c"/>      <use xlink:href="#Montserrat-Regular-6c" x="26.899994"/>      <use xlink:href="#Montserrat-Regular-61" x="53.799988"/>      <use xlink:href="#Montserrat-Regular-6d" x="112.799973"/>      <use xlink:href="#Montserrat-Regular-61" x="218.899963"/>      <use xlink:href="#Montserrat-Regular-2d" x="277.899948"/>      <use xlink:href="#Montserrat-Regular-33" x="316.099945"/>      <use xlink:href="#Montserrat-Regular-2e" x="372.499939"/>      <use xlink:href="#Montserrat-Regular-31" x="393.699936"/>      <use xlink:href="#Montserrat-Regular-2d" x="429.799927"/>      <use xlink:href="#Montserrat-Regular-37" x="467.999924"/>      <use xlink:href="#Montserrat-Regular-30" x="526.899918"/>      <use xlink:href="#Montserrat-Regular-62" x="593.099915"/>      <use xlink:href="#Montserrat-Regular-2d" x="660.899902"/>      <use xlink:href="#Montserrat-Regular-69" x="699.099899"/>      <use xlink:href="#Montserrat-Regular-6e" x="725.999893"/>      <use xlink:href="#Montserrat-Regular-73" x="793.699875"/>      <use xlink:href="#Montserrat-Regular-74" x="842.599869"/>      <use xlink:href="#Montserrat-Regular-72" x="883.19986"/>      <use xlink:href="#Montserrat-Regular-75" x="923.29985"/>      <use xlink:href="#Montserrat-Regular-63" x="990.599838"/>      <use xlink:href="#Montserrat-Regular-74" x="1046.899826"/>     </g>    </g>    <g id="line2d_32">     <path d="M 242.532813 129.475938 L 252.532813 129.475938 L 262.532812 129.475938 " style="fill: none; stroke: #e377c2; stroke-width: 0.5; stroke-linecap: square"/>    </g>    <g id="text_23">     <!-- qwen-2.5-72b -->     <g style="fill: #e377c2" transform="translate(270.532812 132.975938) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-71" d="M 3706 3366 L 3706 -1242 L 3251 -1242 L 3251 634 Q 3040 307 2710 137 Q 2381 -32 1971 -32 Q 1491 -32 1110 182 Q 730 397 512 787 Q 294 1178 294 1683 Q 294 2182 512 2572 Q 730 2963 1114 3177 Q 1498 3392 1971 3392 Q 2394 3392 2730 3213 Q 3066 3034 3270 2701 L 3270 3366 L 3706 3366 zM 2010 371 Q 2362 371 2646 534 Q 2931 698 3094 998 Q 3258 1299 3258 1683 Q 3258 2067 3094 2364 Q 2931 2662 2646 2828 Q 2362 2995 2010 2995 Q 1651 2995 1366 2828 Q 1082 2662 918 2364 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-71"/>      <use xlink:href="#Montserrat-Regular-77" x="67.799988"/>      <use xlink:href="#Montserrat-Regular-65" x="155.699982"/>      <use xlink:href="#Montserrat-Regular-6e" x="216.099976"/>      <use xlink:href="#Montserrat-Regular-2d" x="283.799957"/>      <use xlink:href="#Montserrat-Regular-32" x="321.999954"/>      <use xlink:href="#Montserrat-Regular-2e" x="378.799942"/>      <use xlink:href="#Montserrat-Regular-35" x="399.999939"/>      <use xlink:href="#Montserrat-Regular-2d" x="456.59993"/>      <use xlink:href="#Montserrat-Regular-37" x="494.799927"/>      <use xlink:href="#Montserrat-Regular-32" x="553.699921"/>      <use xlink:href="#Montserrat-Regular-62" x="610.499908"/>     </g>    </g>    <g id="line2d_33">     <path d="M 242.532813 143.836875 L 252.532813 143.836875 L 262.532812 143.836875 " style="fill: none; stroke: #7f7f7f; stroke-width: 0.5; stroke-linecap: square"/>    </g>    <g id="text_24">     <!-- llama-3.2-3b -->     <g style="fill: #7f7f7f" transform="translate(270.532812 147.336875) scale(0.1 -0.1)">      <use xlink:href="#Montserrat-Regular-6c"/>      <use xlink:href="#Montserrat-Regular-6c" x="26.899994"/>      <use xlink:href="#Montserrat-Regular-61" x="53.799988"/>      <use xlink:href="#Montserrat-Regular-6d" x="112.799973"/>      <use xlink:href="#Montserrat-Regular-61" x="218.899963"/>      <use xlink:href="#Montserrat-Regular-2d" x="277.899948"/>      <use xlink:href="#Montserrat-Regular-33" x="316.099945"/>      <use xlink:href="#Montserrat-Regular-2e" x="372.499939"/>      <use xlink:href="#Montserrat-Regular-32" x="393.699936"/>      <use xlink:href="#Montserrat-Regular-2d" x="450.499924"/>      <use xlink:href="#Montserrat-Regular-33" x="488.699921"/>      <use xlink:href="#Montserrat-Regular-62" x="545.099915"/>     </g>    </g>    <g id="line2d_34">     <path d="M 242.532813 158.197813 L 252.532813 158.197813 L 262.532812 158.197813 " style="fill: none; stroke: #bcbd22; stroke-width: 0.5; stroke-linecap: square"/>    </g>    <g id="text_25">     <!-- gemma-2-27b -->     <g style="fill: #bcbd22" transform="translate(270.532812 161.697813) scale(0.1 -0.1)">      <use xlink:href="#Montserrat-Regular-67"/>      <use xlink:href="#Montserrat-Regular-65" x="68.599991"/>      <use xlink:href="#Montserrat-Regular-6d" x="128.999985"/>      <use xlink:href="#Montserrat-Regular-6d" x="235.099976"/>      <use xlink:href="#Montserrat-Regular-61" x="341.199966"/>      <use xlink:href="#Montserrat-Regular-2d" x="400.199951"/>      <use xlink:href="#Montserrat-Regular-32" x="438.399948"/>      <use xlink:href="#Montserrat-Regular-2d" x="495.199936"/>      <use xlink:href="#Montserrat-Regular-32" x="533.399933"/>      <use xlink:href="#Montserrat-Regular-37" x="590.199921"/>      <use xlink:href="#Montserrat-Regular-62" x="649.099915"/>     </g>    </g>    <g id="line2d_35">     <path d="M 242.532813 172.60875 L 252.532813 172.60875 L 262.532812 172.60875 " style="fill: none; stroke: #17becf; stroke-width: 0.5; stroke-linecap: square"/>    </g>    <g id="text_26">     <!-- gemma-2-27b-it -->     <g style="fill: #17becf" transform="translate(270.532812 176.10875) scale(0.1 -0.1)">      <use xlink:href="#Montserrat-Regular-67"/>      <use xlink:href="#Montserrat-Regular-65" x="68.599991"/>      <use xlink:href="#Montserrat-Regular-6d" x="128.999985"/>      <use xlink:href="#Montserrat-Regular-6d" x="235.099976"/>      <use xlink:href="#Montserrat-Regular-61" x="341.199966"/>      <use xlink:href="#Montserrat-Regular-2d" x="400.199951"/>      <use xlink:href="#Montserrat-Regular-32" x="438.399948"/>      <use xlink:href="#Montserrat-Regular-2d" x="495.199936"/>      <use xlink:href="#Montserrat-Regular-32" x="533.399933"/>      <use xlink:href="#Montserrat-Regular-37" x="590.199921"/>      <use xlink:href="#Montserrat-Regular-62" x="649.099915"/>      <use xlink:href="#Montserrat-Regular-2d" x="716.899902"/>      <use xlink:href="#Montserrat-Regular-69" x="755.099899"/>      <use xlink:href="#Montserrat-Regular-74" x="781.999893"/>     </g>    </g>    <g id="line2d_36">     <path d="M 242.532813 187.019688 L 252.532813 187.019688 L 262.532812 187.019688 " style="fill: none; stroke: #1f77b4; stroke-width: 0.5; stroke-linecap: square"/>    </g>    <g id="text_27">     <!-- command-r-v01 -->     <g style="fill: #1f77b4" transform="translate(270.532812 190.519688) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-76" d="M 3462 3366 L 1965 0 L 1498 0 L 6 3366 L 480 3366 L 1741 499 L 3014 3366 L 3462 3366 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-63"/>      <use xlink:href="#Montserrat-Regular-6f" x="56.299988"/>      <use xlink:href="#Montserrat-Regular-6d" x="118.999985"/>      <use xlink:href="#Montserrat-Regular-6d" x="225.099976"/>      <use xlink:href="#Montserrat-Regular-61" x="331.199966"/>      <use xlink:href="#Montserrat-Regular-6e" x="390.199951"/>      <use xlink:href="#Montserrat-Regular-64" x="457.899933"/>      <use xlink:href="#Montserrat-Regular-2d" x="525.699921"/>      <use xlink:href="#Montserrat-Regular-72" x="563.899918"/>      <use xlink:href="#Montserrat-Regular-2d" x="603.999908"/>      <use xlink:href="#Montserrat-Regular-76" x="642.199905"/>      <use xlink:href="#Montserrat-Regular-30" x="696.399902"/>      <use xlink:href="#Montserrat-Regular-31" x="762.599899"/>     </g>    </g>    <g id="line2d_37">     <path d="M 242.532813 201.380625 L 252.532813 201.380625 L 262.532812 201.380625 " style="fill: none; stroke-dasharray: 0.5,0.825; stroke-dashoffset: 0; stroke: #ff7f0e; stroke-width: 0.5"/>    </g>    <g id="text_28">     <!-- gpt-3.5-turbo -->     <g style="fill: #ff7f0e" transform="translate(270.532812 204.880625) scale(0.1 -0.1)">      <use xlink:href="#Montserrat-Regular-67"/>      <use xlink:href="#Montserrat-Regular-70" x="68.599991"/>      <use xlink:href="#Montserrat-Regular-74" x="136.399979"/>      <use xlink:href="#Montserrat-Regular-2d" x="176.999969"/>      <use xlink:href="#Montserrat-Regular-33" x="215.199966"/>      <use xlink:href="#Montserrat-Regular-2e" x="271.59996"/>      <use xlink:href="#Montserrat-Regular-35" x="292.799957"/>      <use xlink:href="#Montserrat-Regular-2d" x="349.399948"/>      <use xlink:href="#Montserrat-Regular-74" x="387.599945"/>      <use xlink:href="#Montserrat-Regular-75" x="428.199936"/>      <use xlink:href="#Montserrat-Regular-72" x="495.499924"/>      <use xlink:href="#Montserrat-Regular-62" x="535.599915"/>      <use xlink:href="#Montserrat-Regular-6f" x="603.399902"/>     </g>    </g>   </g>  </g> </g> <defs>  <clipPath id="p3f4a6a0093">   <rect x="54" y="30.24" width="334.8" height="194.04"/>  </clipPath> </defs></svg>](https://dynomight.net/img/chess/all_mean.pdf)

Notice anything? Any patterns jump out at you?

## Discussion

There are lots of people on the internet who have tried to get LLMs to play chess. The history seems to go something like this:

- **Before September 2023**: Wow, recent LLMs can sort of play chess! They fall apart after the early game, but they can do something! Amazing!

- **September-October 2023**: Wow! LLMs can now play chess at an advanced amateur level! Amazing!

- **(Year of silence.)**

- **Recently:** Wow, recent LLMs can sort of play chess! They fall apart after the early game, but they can do something! Amazing!

I can only assume that lots of other people are experimenting with recent models, getting terrible results, and then mostly not saying anything. I haven’t seen anyone say explicitly that only `gpt-3.5-turbo-instruct` is good at chess. No other LLM is remotely close.

To be fair, a year ago, many people did notice that `gpt-3.5-turbo-instruct` was much better than `gpt-3.5-turbo`. Many speculated at the time that this is because `gpt-3.5-turbo` was subject to additional tuning to be good at chatting.

That might be true. Here’s a comparison of three models where we have similar versions with or without additional chat tuning.

[<?xml version="1.0" encoding="utf-8" standalone="no"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"  "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg xmlns:xlink="http://www.w3.org/1999/xlink" width="432pt" height="252pt" viewBox="0 0 432 252" xmlns="http://www.w3.org/2000/svg" version="1.1"> <metadata>  <rdf:RDF xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:cc="http://creativecommons.org/ns#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">   <cc:Work>    <dc:type rdf:resource="http://purl.org/dc/dcmitype/StillImage"/>    <dc:date>2024-11-14T11:37:09.825337</dc:date>    <dc:format>image/svg+xml</dc:format>    <dc:creator>     <cc:Agent>      <dc:title>Matplotlib v3.8.4, https://matplotlib.org/</dc:title>     </cc:Agent>    </dc:creator>   </cc:Work>  </rdf:RDF> </metadata> <defs>  <style type="text/css">*{stroke-linejoin: round; stroke-linecap: butt}</style> </defs> <g id="figure_1">  <g id="patch_1">   <path d="M 0 252 L 432 252 L 432 0 L 0 0 z" style="fill: #ffffff"/>  </g>  <g id="axes_1">   <g id="patch_2">    <path d="M 54 224.28 L 388.8 224.28 L 388.8 30.24 L 54 30.24 z" style="fill: #ffffff"/>   </g>   <g id="matplotlib.axis_1">    <g id="xtick_1">     <g id="line2d_1"/>     <g id="text_1">      <!-- 0 -->      <g transform="translate(50.689844 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-30" d="M 2118 -38 Q 1600 -38 1193 237 Q 787 512 560 1027 Q 333 1542 333 2240 Q 333 2938 560 3453 Q 787 3968 1193 4243 Q 1600 4518 2118 4518 Q 2637 4518 3043 4243 Q 3450 3968 3680 3453 Q 3910 2938 3910 2240 Q 3910 1542 3680 1027 Q 3450 512 3043 237 Q 2637 -38 2118 -38 zM 2118 384 Q 2509 384 2806 601 Q 3104 819 3270 1235 Q 3437 1651 3437 2240 Q 3437 2829 3270 3245 Q 3104 3661 2806 3878 Q 2509 4096 2118 4096 Q 1728 4096 1430 3878 Q 1133 3661 966 3245 Q 800 2829 800 2240 Q 800 1651 966 1235 Q 1133 819 1430 601 Q 1728 384 2118 384 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="xtick_2">     <g id="line2d_2"/>     <g id="text_2">      <!-- 20 -->      <g transform="translate(114.81 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-32" d="M 3392 410 L 3392 0 L 250 0 L 250 326 L 2112 2150 Q 2464 2496 2589 2749 Q 2714 3002 2714 3258 Q 2714 3654 2442 3875 Q 2170 4096 1664 4096 Q 877 4096 442 3597 L 115 3878 Q 378 4186 784 4352 Q 1190 4518 1702 4518 Q 2387 4518 2787 4195 Q 3187 3872 3187 3309 Q 3187 2963 3033 2643 Q 2880 2323 2451 1907 L 928 410 L 3392 410 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-32"/>       <use xlink:href="#Montserrat-Regular-30" x="56.799988"/>      </g>     </g>    </g>    <g id="xtick_3">     <g id="line2d_3"/>     <g id="text_3">      <!-- 40 -->      <g transform="translate(181.305156 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-34" d="M 4141 1178 L 3245 1178 L 3245 0 L 2784 0 L 2784 1178 L 256 1178 L 256 1510 L 2630 4480 L 3142 4480 L 851 1587 L 2797 1587 L 2797 2624 L 3245 2624 L 3245 1587 L 4141 1587 L 4141 1178 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-34"/>       <use xlink:href="#Montserrat-Regular-30" x="66.099991"/>      </g>     </g>    </g>    <g id="xtick_4">     <g id="line2d_4"/>     <g id="text_4">      <!-- 60 -->      <g transform="translate(248.524531 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-36" d="M 2189 2682 Q 2630 2682 2972 2515 Q 3315 2349 3507 2045 Q 3699 1741 3699 1338 Q 3699 928 3500 614 Q 3302 301 2953 131 Q 2605 -38 2170 -38 Q 1280 -38 806 547 Q 333 1133 333 2202 Q 333 2950 589 3468 Q 845 3987 1305 4252 Q 1766 4518 2381 4518 Q 3046 4518 3443 4275 L 3258 3904 Q 2938 4115 2387 4115 Q 1645 4115 1222 3644 Q 800 3174 800 2278 Q 800 2118 819 1914 Q 986 2278 1350 2480 Q 1715 2682 2189 2682 zM 2150 352 Q 2637 352 2941 617 Q 3245 883 3245 1325 Q 3245 1766 2941 2028 Q 2637 2291 2118 2291 Q 1779 2291 1520 2163 Q 1261 2035 1113 1811 Q 966 1587 966 1312 Q 966 1056 1104 838 Q 1242 621 1510 486 Q 1779 352 2150 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-36"/>       <use xlink:href="#Montserrat-Regular-30" x="60.899994"/>      </g>     </g>    </g>    <g id="xtick_5">     <g id="line2d_5"/>     <g id="text_5">      <!-- 80 -->      <g transform="translate(315.34 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-38" d="M 2899 2349 Q 3322 2208 3549 1926 Q 3776 1645 3776 1248 Q 3776 858 3565 566 Q 3354 275 2960 118 Q 2566 -38 2035 -38 Q 1229 -38 768 307 Q 307 653 307 1248 Q 307 1651 528 1929 Q 749 2208 1171 2349 Q 826 2483 646 2736 Q 467 2989 467 3334 Q 467 3878 892 4198 Q 1318 4518 2035 4518 Q 2509 4518 2864 4374 Q 3219 4230 3417 3961 Q 3616 3693 3616 3334 Q 3616 2989 3430 2736 Q 3245 2483 2899 2349 zM 928 3322 Q 928 2950 1222 2732 Q 1517 2515 2035 2515 Q 2554 2515 2851 2732 Q 3149 2950 3149 3315 Q 3149 3693 2845 3910 Q 2541 4128 2035 4128 Q 1523 4128 1225 3910 Q 928 3693 928 3322 zM 2035 352 Q 2630 352 2966 595 Q 3302 838 3302 1254 Q 3302 1670 2966 1910 Q 2630 2150 2035 2150 Q 1446 2150 1110 1910 Q 774 1670 774 1254 Q 774 832 1107 592 Q 1440 352 2035 352 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-38"/>       <use xlink:href="#Montserrat-Regular-30" x="63.799988"/>      </g>     </g>    </g>    <g id="xtick_6">     <g id="line2d_6"/>     <g id="text_6">      <!-- 100 -->      <g transform="translate(380.375 235.200312) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-31" d="M 1574 4480 L 1574 0 L 1114 0 L 1114 4070 L 58 4070 L 58 4480 L 1574 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>      </g>     </g>    </g>    <g id="text_7">     <!-- turns -->     <g transform="translate(208.170312 248.56125) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-74" d="M 2406 205 Q 2278 90 2089 29 Q 1901 -32 1696 -32 Q 1222 -32 966 224 Q 710 480 710 947 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 4102 L 1165 4102 L 1165 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 973 Q 1165 672 1315 515 Q 1466 358 1747 358 Q 1888 358 2019 403 Q 2150 448 2246 531 L 2406 205 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-75" d="M 3667 3366 L 3667 0 L 3232 0 L 3232 614 Q 3053 307 2739 137 Q 2426 -32 2022 -32 Q 1363 -32 982 336 Q 602 704 602 1414 L 602 3366 L 1056 3366 L 1056 1459 Q 1056 928 1318 653 Q 1581 378 2067 378 Q 2598 378 2905 701 Q 3213 1024 3213 1600 L 3213 3366 L 3667 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-72" d="M 1069 2707 Q 1229 3046 1545 3219 Q 1862 3392 2330 3392 L 2330 2950 L 2221 2957 Q 1690 2957 1389 2630 Q 1088 2304 1088 1715 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2707 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-6e" d="M 2349 3392 Q 2982 3392 3356 3024 Q 3731 2656 3731 1952 L 3731 0 L 3277 0 L 3277 1907 Q 3277 2432 3014 2707 Q 2752 2982 2266 2982 Q 1722 2982 1405 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2746 Q 1254 3053 1584 3222 Q 1914 3392 2349 3392 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-73" d="M 1536 -32 Q 1126 -32 752 86 Q 378 205 166 384 L 371 742 Q 582 576 902 470 Q 1222 365 1568 365 Q 2029 365 2249 509 Q 2470 653 2470 915 Q 2470 1101 2348 1206 Q 2227 1312 2041 1366 Q 1856 1421 1549 1472 Q 1139 1549 889 1629 Q 640 1709 464 1901 Q 288 2093 288 2432 Q 288 2854 640 3123 Q 992 3392 1619 3392 Q 1946 3392 2272 3305 Q 2598 3219 2810 3078 L 2611 2714 Q 2195 3002 1619 3002 Q 1184 3002 963 2848 Q 742 2694 742 2445 Q 742 2253 867 2137 Q 992 2022 1177 1968 Q 1363 1914 1690 1856 Q 2093 1779 2336 1702 Q 2579 1626 2752 1440 Q 2925 1254 2925 928 Q 2925 486 2557 227 Q 2189 -32 1536 -32 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-74"/>      <use xlink:href="#Montserrat-Regular-75" x="40.599991"/>      <use xlink:href="#Montserrat-Regular-72" x="107.899979"/>      <use xlink:href="#Montserrat-Regular-6e" x="147.999969"/>      <use xlink:href="#Montserrat-Regular-73" x="215.699951"/>     </g>    </g>   </g>   <g id="matplotlib.axis_2">    <g id="ytick_1">     <g id="line2d_7"/>     <g id="text_8">      <!-- LOSS -->      <g transform="translate(23.91875 227.667831) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-4c" d="M 730 4480 L 1203 4480 L 1203 410 L 3712 410 L 3712 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4f" d="M 2688 -38 Q 2022 -38 1481 259 Q 941 557 637 1075 Q 333 1594 333 2240 Q 333 2886 637 3404 Q 941 3923 1481 4220 Q 2022 4518 2688 4518 Q 3354 4518 3888 4224 Q 4422 3930 4729 3408 Q 5037 2886 5037 2240 Q 5037 1594 4729 1072 Q 4422 550 3888 256 Q 3354 -38 2688 -38 zM 2688 384 Q 3219 384 3648 624 Q 4077 864 4320 1289 Q 4563 1715 4563 2240 Q 4563 2765 4320 3190 Q 4077 3616 3648 3856 Q 3219 4096 2688 4096 Q 2157 4096 1725 3856 Q 1293 3616 1046 3190 Q 800 2765 800 2240 Q 800 1715 1046 1289 Q 1293 864 1725 624 Q 2157 384 2688 384 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-53" d="M 1965 -38 Q 1466 -38 1008 122 Q 550 282 301 544 L 486 909 Q 730 666 1133 515 Q 1536 365 1965 365 Q 2566 365 2867 585 Q 3168 806 3168 1158 Q 3168 1427 3005 1587 Q 2842 1747 2605 1833 Q 2368 1920 1946 2022 Q 1440 2150 1139 2268 Q 838 2387 624 2630 Q 410 2874 410 3290 Q 410 3629 589 3907 Q 768 4186 1139 4352 Q 1510 4518 2061 4518 Q 2445 4518 2813 4412 Q 3181 4307 3450 4122 L 3290 3744 Q 3008 3930 2688 4022 Q 2368 4115 2061 4115 Q 1472 4115 1174 3888 Q 877 3661 877 3302 Q 877 3034 1040 2870 Q 1203 2707 1449 2620 Q 1696 2534 2112 2432 Q 2605 2310 2909 2192 Q 3213 2074 3424 1834 Q 3635 1594 3635 1184 Q 3635 845 3452 566 Q 3270 288 2892 125 Q 2515 -38 1965 -38 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-4c"/>       <use xlink:href="#Montserrat-Regular-4f" x="58.899994"/>       <use xlink:href="#Montserrat-Regular-53" x="142.799988"/>       <use xlink:href="#Montserrat-Regular-53" x="204.299973"/>      </g>     </g>    </g>    <g id="ytick_2">     <g id="line2d_8"/>     <g id="text_9">      <!-- -1000 -->      <g transform="translate(23.209375 195.435273) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-2d" d="M 384 1933 L 2061 1933 L 2061 1536 L 384 1536 L 384 1933 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-31" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="74.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="140.499985"/>       <use xlink:href="#Montserrat-Regular-30" x="206.699982"/>      </g>     </g>    </g>    <g id="ytick_3">     <g id="line2d_9"/>     <g id="text_10">      <!-- -500 -->      <g transform="translate(27.779687 163.202714) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-35" d="M 1510 2643 Q 2483 2643 2928 2294 Q 3373 1946 3373 1318 Q 3373 928 3190 621 Q 3008 314 2643 138 Q 2278 -38 1747 -38 Q 1267 -38 835 118 Q 403 275 141 538 L 365 902 Q 589 672 953 528 Q 1318 384 1741 384 Q 2298 384 2602 633 Q 2906 883 2906 1299 Q 2906 1760 2563 1997 Q 2221 2234 1408 2234 L 454 2234 L 685 4480 L 3136 4480 L 3136 4070 L 1088 4070 L 934 2643 L 1510 2643 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-2d"/>       <use xlink:href="#Montserrat-Regular-35" x="38.199997"/>       <use xlink:href="#Montserrat-Regular-30" x="94.799988"/>       <use xlink:href="#Montserrat-Regular-30" x="160.999985"/>      </g>     </g>    </g>    <g id="ytick_4">     <g id="line2d_10"/>     <g id="text_11">      <!-- 0 -->      <g transform="translate(43.879688 130.970156) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-30"/>      </g>     </g>    </g>    <g id="ytick_5">     <g id="line2d_11"/>     <g id="text_12">      <!-- 500 -->      <g transform="translate(31.6 98.737598) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-35"/>       <use xlink:href="#Montserrat-Regular-30" x="56.599991"/>       <use xlink:href="#Montserrat-Regular-30" x="122.799988"/>      </g>     </g>    </g>    <g id="ytick_6">     <g id="line2d_12"/>     <g id="text_13">      <!-- 1000 -->      <g transform="translate(27.029687 66.50504) scale(0.1 -0.1)">       <use xlink:href="#Montserrat-Regular-31"/>       <use xlink:href="#Montserrat-Regular-30" x="36.099991"/>       <use xlink:href="#Montserrat-Regular-30" x="102.299988"/>       <use xlink:href="#Montserrat-Regular-30" x="168.499985"/>      </g>     </g>    </g>    <g id="ytick_7">     <g id="line2d_13"/>     <g id="text_14">      <!-- WIN -->      <g transform="translate(28.240625 34.272482) scale(0.1 -0.1)">       <defs>        <path id="Montserrat-Regular-57" d="M 6880 4480 L 5370 0 L 4870 0 L 3558 3821 L 2240 0 L 1747 0 L 237 4480 L 723 4480 L 2016 627 L 3354 4480 L 3795 4480 L 5114 608 L 6426 4480 L 6880 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-49" d="M 730 4480 L 1203 4480 L 1203 0 L 730 0 L 730 4480 z" transform="scale(0.015625)"/>        <path id="Montserrat-Regular-4e" d="M 4474 4480 L 4474 0 L 4083 0 L 1203 3635 L 1203 0 L 730 0 L 730 4480 L 1120 4480 L 4006 845 L 4006 4480 L 4474 4480 z" transform="scale(0.015625)"/>       </defs>       <use xlink:href="#Montserrat-Regular-57"/>       <use xlink:href="#Montserrat-Regular-49" x="111.099991"/>       <use xlink:href="#Montserrat-Regular-4e" x="141.299988"/>      </g>     </g>    </g>    <g id="text_15">     <!-- mean centipawn difference -->     <g transform="translate(17.26875 196.6475) rotate(-90) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-6d" d="M 4832 3392 Q 5466 3392 5827 3027 Q 6189 2662 6189 1952 L 6189 0 L 5734 0 L 5734 1907 Q 5734 2432 5481 2707 Q 5229 2982 4768 2982 Q 4243 2982 3942 2659 Q 3642 2336 3642 1766 L 3642 0 L 3187 0 L 3187 1907 Q 3187 2432 2934 2707 Q 2682 2982 2214 2982 Q 1696 2982 1392 2659 Q 1088 2336 1088 1766 L 1088 0 L 634 0 L 634 3366 L 1069 3366 L 1069 2752 Q 1248 3059 1568 3225 Q 1888 3392 2304 3392 Q 2726 3392 3036 3213 Q 3347 3034 3501 2682 Q 3686 3014 4035 3203 Q 4384 3392 4832 3392 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-65" d="M 3565 1542 L 749 1542 Q 787 1018 1152 694 Q 1517 371 2074 371 Q 2387 371 2649 483 Q 2912 595 3104 813 L 3360 518 Q 3136 250 2800 109 Q 2464 -32 2061 -32 Q 1542 -32 1142 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 508 2566 Q 723 2957 1097 3174 Q 1472 3392 1939 3392 Q 2406 3392 2777 3174 Q 3149 2957 3360 2569 Q 3571 2182 3571 1683 L 3565 1542 zM 1939 3002 Q 1453 3002 1123 2691 Q 794 2381 749 1882 L 3136 1882 Q 3091 2381 2761 2691 Q 2432 3002 1939 3002 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-61" d="M 1811 3392 Q 2470 3392 2822 3062 Q 3174 2733 3174 2086 L 3174 0 L 2739 0 L 2739 525 Q 2586 262 2288 115 Q 1990 -32 1581 -32 Q 1018 -32 685 237 Q 352 506 352 947 Q 352 1376 662 1638 Q 973 1901 1651 1901 L 2720 1901 L 2720 2106 Q 2720 2541 2477 2768 Q 2234 2995 1766 2995 Q 1446 2995 1152 2889 Q 858 2784 646 2598 L 442 2938 Q 698 3155 1056 3273 Q 1414 3392 1811 3392 zM 1651 326 Q 2035 326 2310 502 Q 2586 678 2720 1011 L 2720 1562 L 1664 1562 Q 800 1562 800 960 Q 800 666 1024 496 Q 1248 326 1651 326 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-20" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-63" d="M 2029 -32 Q 1530 -32 1136 185 Q 742 403 518 796 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1136 3174 Q 1530 3392 2029 3392 Q 2464 3392 2806 3222 Q 3149 3053 3347 2726 L 3008 2496 Q 2842 2746 2586 2870 Q 2330 2995 2029 2995 Q 1664 2995 1373 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1293 918 995 Q 1082 698 1373 534 Q 1664 371 2029 371 Q 2330 371 2586 492 Q 2842 614 3008 864 L 3347 634 Q 3149 307 2803 137 Q 2458 -32 2029 -32 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-69" d="M 634 3366 L 1088 3366 L 1088 0 L 634 0 L 634 3366 zM 864 4102 Q 723 4102 627 4198 Q 531 4294 531 4429 Q 531 4557 627 4653 Q 723 4749 864 4749 Q 1005 4749 1101 4656 Q 1197 4563 1197 4435 Q 1197 4294 1101 4198 Q 1005 4102 864 4102 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-70" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1178 3827 787 Q 3610 397 3229 182 Q 2848 -32 2368 -32 Q 1958 -32 1628 137 Q 1299 307 1088 634 L 1088 -1242 L 634 -1242 L 634 3366 L 1069 3366 L 1069 2701 Q 1274 3034 1610 3213 Q 1946 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2364 Q 3264 2662 2976 2828 Q 2688 2995 2336 2995 Q 1978 2995 1693 2828 Q 1408 2662 1245 2364 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-77" d="M 5574 3366 L 4301 0 L 3872 0 L 2810 2758 L 1747 0 L 1318 0 L 51 3366 L 486 3366 L 1542 506 L 2624 3366 L 3014 3366 L 4090 506 L 5158 3366 L 5574 3366 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-64" d="M 3706 4749 L 3706 0 L 3270 0 L 3270 666 Q 3066 326 2730 147 Q 2394 -32 1971 -32 Q 1498 -32 1114 185 Q 730 403 512 793 Q 294 1184 294 1683 Q 294 2182 512 2572 Q 730 2963 1114 3177 Q 1498 3392 1971 3392 Q 2381 3392 2710 3222 Q 3040 3053 3251 2726 L 3251 4749 L 3706 4749 zM 2010 371 Q 2362 371 2646 534 Q 2931 698 3094 998 Q 3258 1299 3258 1683 Q 3258 2067 3094 2368 Q 2931 2669 2646 2832 Q 2362 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-66" d="M 1747 4397 Q 1453 4397 1302 4237 Q 1152 4077 1152 3763 L 1152 3366 L 2189 3366 L 2189 2982 L 1165 2982 L 1165 0 L 710 0 L 710 2982 L 109 2982 L 109 3366 L 710 3366 L 710 3782 Q 710 4243 976 4512 Q 1242 4781 1728 4781 Q 1920 4781 2099 4726 Q 2278 4672 2400 4570 L 2246 4230 Q 2048 4397 1747 4397 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-6d"/>      <use xlink:href="#Montserrat-Regular-65" x="106.099991"/>      <use xlink:href="#Montserrat-Regular-61" x="166.499985"/>      <use xlink:href="#Montserrat-Regular-6e" x="225.499969"/>      <use xlink:href="#Montserrat-Regular-20" x="293.199951"/>      <use xlink:href="#Montserrat-Regular-63" x="319.399948"/>      <use xlink:href="#Montserrat-Regular-65" x="375.699936"/>      <use xlink:href="#Montserrat-Regular-6e" x="436.09993"/>      <use xlink:href="#Montserrat-Regular-74" x="503.799911"/>      <use xlink:href="#Montserrat-Regular-69" x="544.399902"/>      <use xlink:href="#Montserrat-Regular-70" x="571.299896"/>      <use xlink:href="#Montserrat-Regular-61" x="639.099884"/>      <use xlink:href="#Montserrat-Regular-77" x="698.099869"/>      <use xlink:href="#Montserrat-Regular-6e" x="785.999863"/>      <use xlink:href="#Montserrat-Regular-20" x="853.699844"/>      <use xlink:href="#Montserrat-Regular-64" x="879.899841"/>      <use xlink:href="#Montserrat-Regular-69" x="947.699829"/>      <use xlink:href="#Montserrat-Regular-66" x="974.599823"/>      <use xlink:href="#Montserrat-Regular-66" x="1008.499817"/>      <use xlink:href="#Montserrat-Regular-65" x="1042.399811"/>      <use xlink:href="#Montserrat-Regular-72" x="1102.799805"/>      <use xlink:href="#Montserrat-Regular-65" x="1142.899796"/>      <use xlink:href="#Montserrat-Regular-6e" x="1203.299789"/>      <use xlink:href="#Montserrat-Regular-63" x="1270.999771"/>      <use xlink:href="#Montserrat-Regular-65" x="1327.299759"/>     </g>    </g>   </g>   <g id="line2d_14">    <path d="M 54 127.26 L 57.348 123.430772 L 60.696 121.219619 L 64.044 120.03346 L 67.392 118.3896 L 70.74 118.821516 L 74.088 118.13174 L 77.436 115.411312 L 80.784 115.108326 L 84.132 113.335535 L 87.48 116.629702 L 90.828 111.897963 L 94.176 103.646428 L 97.524 107.591693 L 100.872 99.462642 L 104.22 101.151628 L 107.568 98.308716 L 110.916 98.199126 L 114.264 94.756688 L 117.612 89.831553 L 120.96 89.921805 L 124.308 91.591451 L 127.656 88.774326 L 131.004 86.666316 L 134.352 79.697637 L 137.7 68.906177 L 141.048 64.696605 L 144.396 58.694902 L 147.744 57.70214 L 151.092 49.985665 L 154.44 50.926856 L 157.788 51.539274 L 161.136 45.859898 L 164.484 47.806744 L 167.832 44.029088 L 171.18 41.998437 L 174.528 39.361814 L 177.876 38.826753 L 181.224 37.228019 L 184.572 35.074884 L 187.92 35.074884 L 191.268 37.163553 L 194.616 37.537451 L 197.964 38.13053 L 201.312 36.74453 L 204.66 36.422205 L 208.008 36.435098 L 211.356 35.08133 L 214.704 35.08133 L 218.052 30.626791 L 221.4 30.626791 L 224.748 30.626791 L 228.096 30.626791 L 231.444 30.626791 L 234.792 30.626791 L 238.14 30.626791 L 241.488 30.626791 L 244.836 30.626791 L 248.184 30.626791 L 251.532 30.626791 L 254.88 30.626791 L 258.228 30.626791 L 261.576 30.626791 L 264.924 30.626791 L 268.272 30.626791 L 271.62 30.626791 L 274.968 30.626791 L 278.316 30.626791 L 281.664 30.626791 L 285.012 30.626791 L 288.36 30.626791 L 291.708 30.626791 L 295.056 30.626791 L 298.404 30.626791 L 301.752 30.626791 L 305.1 30.626791 L 308.448 30.626791 L 311.796 30.626791 L 315.144 30.626791 L 318.492 30.626791 L 321.84 30.626791 L 325.188 30.626791 L 328.536 30.626791 L 331.884 30.626791 L 335.232 30.626791 L 338.58 30.626791 L 341.928 30.626791 L 345.276 30.626791 L 348.624 30.626791 L 351.972 30.626791 L 355.32 30.626791 L 358.668 30.626791 L 362.016 30.626791 L 365.364 30.626791 L 368.712 30.626791 L 372.06 30.626791 L 375.408 30.626791 L 378.756 30.626791 L 382.104 30.626791 L 385.452 30.626791 " clip-path="url(#p41ad9ac113)" style="fill: none; stroke: #1f77b4; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_15">    <path d="M 54 127.26 L 57.348 123.243823 L 60.696 122.249771 L 64.044 121.766283 L 67.392 122.659769 L 70.74 122.628826 L 74.088 122.863479 L 77.436 123.668004 L 80.784 125.229349 L 84.132 129.306123 L 87.48 131.482465 L 90.828 136.18584 L 94.176 141.827827 L 97.524 143.783699 L 100.872 149.02858 L 104.22 150.948352 L 107.568 155.303615 L 110.916 160.086927 L 114.264 161.612171 L 117.612 166.99243 L 120.96 171.312882 L 124.308 174.153215 L 127.656 178.088166 L 131.004 181.052272 L 134.352 184.186566 L 137.7 186.863157 L 141.048 187.98614 L 144.396 191.735431 L 147.744 198.13166 L 151.092 201.996988 L 154.44 203.91547 L 157.788 205.198326 L 161.136 206.750646 L 164.484 207.921332 L 167.832 209.306043 L 171.18 209.420791 L 174.528 212.310117 L 177.876 214.452938 L 181.224 215.290984 L 184.572 216.239911 L 187.92 218.048802 L 191.268 218.31053 L 194.616 218.326002 L 197.964 218.207386 L 201.312 218.1455 L 204.66 218.475561 L 208.008 218.596755 L 211.356 219.82675 L 214.704 221.283661 L 218.052 221.384227 L 221.4 221.364887 L 224.748 221.435799 L 228.096 221.826458 L 231.444 222.003092 L 234.792 222.041771 L 238.14 222.973937 L 241.488 223.105446 L 244.836 223.002301 L 248.184 223.02422 L 251.532 223.035823 L 254.88 223.217615 L 258.228 223.215036 L 261.576 223.215036 L 264.924 223.24598 L 268.272 223.291105 L 271.62 223.216326 L 274.968 223.309155 L 278.316 223.297552 L 281.664 223.488368 L 285.012 223.020352 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p41ad9ac113)" style="fill: none; stroke: #2ca02c; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_16">    <path d="M 54 127.26 L 57.348 123.812406 L 60.696 122.934391 L 64.044 122.254928 L 67.392 125.41243 L 70.74 126.365224 L 74.088 126.468368 L 77.436 128.478391 L 80.784 130.135144 L 84.132 136.015652 L 87.48 138.363472 L 90.828 147.756039 L 94.176 152.4852 L 97.524 158.3064 L 100.872 160.267429 L 104.22 166.291049 L 107.568 170.68757 L 110.916 175.019626 L 114.264 178.79986 L 117.612 182.653585 L 120.96 190.7388 L 124.308 193.071148 L 127.656 195.11856 L 131.004 196.299561 L 134.352 200.935892 L 137.7 203.973488 L 141.048 206.339358 L 144.396 208.250104 L 147.744 210.666257 L 151.092 211.608737 L 154.44 213.808287 L 157.788 215.474065 L 161.136 215.746108 L 164.484 217.624621 L 167.832 217.739369 L 171.18 218.879113 L 174.528 218.603202 L 177.876 219.891215 L 181.224 220.131025 L 184.572 220.605488 L 187.92 221.644666 L 191.268 221.631773 L 194.616 221.439667 L 197.964 221.750389 L 201.312 221.919287 L 204.66 222.308657 L 208.008 222.570385 L 211.356 222.679976 L 214.704 222.682554 L 218.052 222.535574 L 221.4 222.558781 L 224.748 222.664504 L 228.096 222.557492 L 231.444 223.313023 L 234.792 223.102867 L 238.14 223.199565 L 241.488 223.225351 L 244.836 223.167332 L 248.184 223.225351 L 251.532 223.956385 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p41ad9ac113)" style="fill: none; stroke: #8c564b; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_17">    <path d="M 54 127.26 L 57.348 123.247691 L 60.696 123.820141 L 64.044 123.135522 L 67.392 124.259793 L 70.74 126.1512 L 74.088 129.964956 L 77.436 137.335898 L 80.784 142.979174 L 84.132 145.698313 L 87.48 152.379477 L 90.828 154.879434 L 94.176 159.300452 L 97.524 167.283812 L 100.872 170.482571 L 104.22 172.826523 L 107.568 178.353762 L 110.916 180.258061 L 114.264 187.128753 L 117.612 190.438393 L 120.96 193.655202 L 124.308 195.80189 L 127.656 198.107163 L 131.004 201.32784 L 134.352 203.278554 L 137.7 204.953358 L 141.048 206.466999 L 144.396 207.918753 L 147.744 209.523935 L 151.092 210.359403 L 154.44 213.712878 L 157.788 215.066646 L 161.136 217.084404 L 164.484 217.759998 L 167.832 220.27027 L 171.18 220.673821 L 174.528 220.925235 L 177.876 221.312026 L 181.224 220.932971 L 184.572 221.442246 L 187.92 221.141838 L 191.268 220.904607 L 194.616 221.17536 L 197.964 221.309447 L 201.312 221.683345 L 204.66 221.791647 L 208.008 221.697527 L 211.356 221.916709 L 214.704 222.224852 L 218.052 222.211959 L 221.4 222.474977 L 224.748 222.70963 L 228.096 223.146703 L 231.444 223.420035 L 234.792 223.46774 L 238.14 222.997144 L 241.488 223.471607 L 244.836 223.555412 L 248.184 223.547676 L 251.532 223.570884 L 254.88 223.576041 L 258.228 223.203433 L 261.576 223.476765 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p41ad9ac113)" style="fill: none; stroke: #bcbd22; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_18">    <path d="M 54 127.26 L 57.348 124.147684 L 60.696 123.140229 L 64.044 122.636502 L 67.392 122.946834 L 70.74 123.846347 L 74.088 129.490793 L 77.436 133.342209 L 80.784 136.842814 L 84.132 139.926646 L 87.48 149.782313 L 90.828 157.434172 L 94.176 162.435466 L 97.524 170.640526 L 100.872 173.910256 L 104.22 178.373341 L 107.568 178.676177 L 110.916 185.689382 L 114.264 188.711747 L 117.612 190.323375 L 120.96 194.554085 L 124.308 200.474382 L 127.656 200.029123 L 131.004 201.283944 L 134.352 205.426202 L 137.7 205.511656 L 141.048 205.943422 L 144.396 209.426038 L 147.744 212.530858 L 151.092 215.344835 L 154.44 216.836528 L 157.788 214.715176 L 161.136 216.38827 L 164.484 217.470685 L 167.832 217.316268 L 171.18 217.685069 L 174.528 218.608569 L 177.876 219.776437 L 181.224 219.885878 L 184.572 219.945845 L 187.92 219.791429 L 191.268 220.401599 L 194.616 220.493049 L 197.964 220.863349 L 201.312 221.596452 L 204.66 221.472019 L 208.008 221.839321 L 211.356 221.777854 L 214.704 223.185592 L 218.052 223.299531 L 221.4 223.954676 L 224.748 223.956175 L 228.096 223.956175 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p41ad9ac113)" style="fill: none; stroke: #17becf; stroke-width: 0.5; stroke-linecap: square"/>   </g>   <g id="line2d_19">    <path d="M 54 127.26 L 57.348 123.925395 L 60.696 121.956279 L 64.044 124.575907 L 67.392 126.855628 L 70.74 136.753953 L 74.088 148.222884 L 77.436 153.913395 L 80.784 155.073767 L 84.132 161.227256 L 87.48 161.057302 L 90.828 170.22307 L 94.176 170.393023 L 97.524 179.717023 L 100.872 183.883814 L 104.22 187.652093 L 107.568 194.514698 L 110.916 198.745953 L 114.264 199.800837 L 117.612 202.824837 L 120.96 206.300093 L 124.308 202.467349 L 127.656 206.440744 L 131.004 205.667163 L 134.352 209.329953 L 137.7 210.261767 L 141.048 210.203163 L 144.396 212.629395 L 147.744 212.184 L 151.092 212.664558 L 154.44 214.20586 L 157.788 218.144093 L 161.136 218.99386 L 164.484 221.373209 L 167.832 220.013581 L 171.18 221.039163 L 174.528 223.945953 L 177.876 223.957674 L 181.224 223.957674 L 184.572 223.957674 L 187.92 223.957674 L 191.268 223.957674 L 194.616 223.957674 L 197.964 223.957674 L 201.312 223.957674 L 204.66 223.957674 L 208.008 223.957674 L 211.356 223.957674 L 214.704 223.957674 L 218.052 223.957674 L 221.4 223.957674 L 224.748 223.957674 L 228.096 223.957674 L 231.444 223.957674 L 234.792 223.957674 L 238.14 223.957674 L 241.488 223.957674 L 244.836 223.957674 L 248.184 223.957674 L 251.532 223.957674 L 254.88 223.957674 L 258.228 223.957674 L 261.576 223.957674 L 264.924 223.957674 L 268.272 223.957674 L 271.62 223.957674 L 274.968 223.957674 L 278.316 223.957674 L 281.664 223.957674 L 285.012 223.957674 L 288.36 223.957674 L 291.708 223.957674 L 295.056 223.957674 L 298.404 223.957674 L 301.752 223.957674 L 305.1 223.957674 L 308.448 223.957674 L 311.796 223.957674 L 315.144 223.957674 L 318.492 223.957674 L 321.84 223.957674 L 325.188 223.957674 L 328.536 223.957674 L 331.884 223.957674 L 335.232 223.957674 L 338.58 223.957674 L 341.928 223.957674 L 345.276 223.957674 L 348.624 223.957674 L 351.972 223.957674 L 355.32 223.957674 L 358.668 223.957674 L 362.016 223.957674 L 365.364 223.957674 L 368.712 223.957674 L 372.06 223.957674 L 375.408 223.957674 L 378.756 223.957674 L 382.104 223.957674 L 385.452 223.957674 " clip-path="url(#p41ad9ac113)" style="fill: none; stroke-dasharray: 0.5,0.825; stroke-dashoffset: 0; stroke: #ff7f0e; stroke-width: 0.5"/>   </g>   <g id="patch_3">    <path d="M 54 224.28 L 54 30.24 " style="fill: none"/>   </g>   <g id="patch_4">    <path d="M 388.8 224.28 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="patch_5">    <path d="M 54 224.28 L 388.8 224.28 " style="fill: none"/>   </g>   <g id="patch_6">    <path d="M 54 30.24 L 388.8 30.24 " style="fill: none"/>   </g>   <g id="text_16">    <g id="patch_7">     <a xlink:href="https://dynomight.net/chess">      <path d="M 323.8664 225.676606 L 388.881244 225.676606 L 388.881244 213.343012 L 323.8664 213.343012 z" style="fill: #ffffff; opacity: 0.000001; stroke: #ffffff; stroke-linejoin: miter"/>     </a>    </g>    <a xlink:href="https://dynomight.net/chess">     <!-- dynomight.net/chess -->     <g style="opacity: 0.3" transform="translate(327.8664 220.734419) scale(0.05 -0.05)">      <defs>       <path id="CourierNewPSMT-64" d="M 3206 3922 L 3206 263 L 3556 263 Q 3653 263 3693 227 Q 3734 191 3734 131 Q 3734 75 3693 37 Q 3653 0 3556 0 L 2941 0 L 2941 522 Q 2484 -103 1781 -103 Q 1425 -103 1098 86 Q 772 275 583 625 Q 394 975 394 1347 Q 394 1722 583 2070 Q 772 2419 1098 2609 Q 1425 2800 1784 2800 Q 2472 2800 2941 2175 L 2941 3656 L 2591 3656 Q 2494 3656 2453 3693 Q 2413 3731 2413 3791 Q 2413 3847 2453 3884 Q 2494 3922 2591 3922 L 3206 3922 zM 2941 1347 Q 2941 1847 2603 2190 Q 2266 2534 1800 2534 Q 1331 2534 993 2190 Q 656 1847 656 1347 Q 656 850 993 504 Q 1331 159 1800 159 Q 2266 159 2603 504 Q 2941 850 2941 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-79" d="M 1938 0 L 719 2441 L 641 2441 Q 547 2441 506 2478 Q 466 2516 466 2572 Q 466 2613 486 2645 Q 506 2678 539 2692 Q 572 2706 641 2706 L 1359 2706 Q 1453 2706 1493 2668 Q 1534 2631 1534 2572 Q 1534 2516 1493 2478 Q 1453 2441 1359 2441 L 1006 2441 L 2078 288 L 3134 2441 L 2781 2441 Q 2688 2441 2647 2478 Q 2606 2516 2606 2575 Q 2606 2631 2647 2668 Q 2688 2706 2781 2706 L 3497 2706 Q 3594 2706 3634 2668 Q 3675 2631 3675 2572 Q 3675 2531 3650 2497 Q 3625 2463 3594 2452 Q 3563 2441 3419 2441 L 1756 -944 L 2166 -944 Q 2259 -944 2300 -980 Q 2341 -1016 2341 -1075 Q 2341 -1131 2300 -1168 Q 2259 -1206 2166 -1206 L 653 -1206 Q 559 -1206 518 -1170 Q 478 -1134 478 -1075 Q 478 -1016 518 -980 Q 559 -944 653 -944 L 1475 -944 L 1938 0 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6e" d="M 1072 2706 L 1072 2309 Q 1347 2588 1569 2694 Q 1791 2800 2069 2800 Q 2369 2800 2616 2672 Q 2791 2578 2933 2361 Q 3075 2144 3075 1916 L 3075 263 L 3297 263 Q 3391 263 3431 227 Q 3472 191 3472 131 Q 3472 75 3431 37 Q 3391 0 3297 0 L 2594 0 Q 2497 0 2456 37 Q 2416 75 2416 131 Q 2416 191 2456 227 Q 2497 263 2594 263 L 2813 263 L 2813 1872 Q 2813 2150 2609 2342 Q 2406 2534 2066 2534 Q 1806 2534 1615 2429 Q 1425 2325 1072 1909 L 1072 263 L 1369 263 Q 1463 263 1503 227 Q 1544 191 1544 131 Q 1544 75 1503 37 Q 1463 0 1369 0 L 513 0 Q 419 0 378 37 Q 338 75 338 131 Q 338 191 378 227 Q 419 263 513 263 L 809 263 L 809 2441 L 588 2441 Q 494 2441 453 2478 Q 413 2516 413 2575 Q 413 2631 453 2668 Q 494 2706 588 2706 L 1072 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6f" d="M 3388 1347 Q 3388 747 2958 322 Q 2528 -103 1922 -103 Q 1309 -103 881 323 Q 453 750 453 1347 Q 453 1947 881 2373 Q 1309 2800 1922 2800 Q 2528 2800 2958 2375 Q 3388 1950 3388 1347 zM 3122 1347 Q 3122 1841 2770 2187 Q 2419 2534 1919 2534 Q 1419 2534 1067 2186 Q 716 1838 716 1347 Q 716 859 1067 509 Q 1419 159 1919 159 Q 2419 159 2770 507 Q 3122 856 3122 1347 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-6d" d="M 725 2706 L 725 2441 Q 1059 2800 1397 2800 Q 1600 2800 1753 2692 Q 1906 2584 2009 2366 Q 2184 2584 2364 2692 Q 2544 2800 2725 2800 Q 3009 2800 3178 2616 Q 3400 2378 3400 2097 L 3400 263 L 3622 263 Q 3716 263 3756 227 Q 3797 191 3797 131 Q 3797 75 3756 37 Q 3716 0 3622 0 L 3138 0 L 3138 2072 Q 3138 2272 3016 2403 Q 2894 2534 2734 2534 Q 2591 2534 2431 2426 Q 2272 2319 2069 2003 L 2069 263 L 2288 263 Q 2381 263 2422 227 Q 2463 191 2463 131 Q 2463 75 2422 37 Q 2381 0 2288 0 L 1803 0 L 1803 2053 Q 1803 2263 1679 2398 Q 1556 2534 1403 2534 Q 1263 2534 1125 2444 Q 934 2316 725 2003 L 725 263 L 947 263 Q 1041 263 1081 227 Q 1122 191 1122 131 Q 1122 75 1081 37 Q 1041 0 947 0 L 241 0 Q 147 0 106 37 Q 66 75 66 131 Q 66 191 106 227 Q 147 263 241 263 L 463 263 L 463 2441 L 241 2441 Q 147 2441 106 2478 Q 66 2516 66 2575 Q 66 2631 106 2668 Q 147 2706 241 2706 L 725 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-69" d="M 2050 4053 L 2050 3375 L 1663 3375 L 1663 4053 L 2050 4053 zM 2059 2706 L 2059 263 L 3091 263 Q 3188 263 3228 227 Q 3269 191 3269 131 Q 3269 75 3228 37 Q 3188 0 3091 0 L 766 0 Q 672 0 631 37 Q 591 75 591 131 Q 591 191 631 227 Q 672 263 766 263 L 1797 263 L 1797 2441 L 1031 2441 Q 938 2441 895 2478 Q 853 2516 853 2572 Q 853 2631 893 2668 Q 934 2706 1031 2706 L 2059 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-67" d="M 2803 2225 L 2803 2706 L 3419 2706 Q 3513 2706 3553 2668 Q 3594 2631 3594 2572 Q 3594 2516 3553 2478 Q 3513 2441 3419 2441 L 3066 2441 L 3066 -181 Q 3066 -444 2953 -650 Q 2878 -788 2703 -934 Q 2528 -1081 2384 -1143 Q 2241 -1206 2000 -1206 L 1256 -1206 Q 1163 -1206 1122 -1170 Q 1081 -1134 1081 -1075 Q 1081 -1016 1122 -978 Q 1163 -941 1256 -941 L 2009 -944 Q 2241 -944 2427 -825 Q 2613 -706 2734 -478 Q 2803 -347 2803 -144 L 2803 644 Q 2381 66 1716 66 Q 1175 66 783 467 Q 391 869 391 1434 Q 391 2000 783 2400 Q 1175 2800 1716 2800 Q 2381 2800 2803 2225 zM 2803 1434 Q 2803 1897 2486 2215 Q 2169 2534 1728 2534 Q 1288 2534 972 2214 Q 656 1894 656 1434 Q 656 972 972 651 Q 1288 331 1728 331 Q 2169 331 2486 651 Q 2803 972 2803 1434 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-68" d="M 1056 3922 L 1056 2306 Q 1306 2578 1536 2689 Q 1766 2800 2050 2800 Q 2356 2800 2570 2692 Q 2784 2584 2928 2361 Q 3072 2138 3072 1891 L 3072 263 L 3366 263 Q 3463 263 3502 227 Q 3541 191 3541 131 Q 3541 75 3502 37 Q 3463 0 3366 0 L 2513 0 Q 2416 0 2375 37 Q 2334 75 2334 131 Q 2334 191 2375 227 Q 2416 263 2513 263 L 2806 263 L 2806 1872 Q 2806 2156 2601 2347 Q 2397 2538 2031 2538 Q 1744 2538 1541 2397 Q 1394 2297 1056 1925 L 1056 263 L 1353 263 Q 1447 263 1487 227 Q 1528 191 1528 131 Q 1528 75 1487 37 Q 1447 0 1353 0 L 497 0 Q 403 0 362 37 Q 322 75 322 131 Q 322 191 362 227 Q 403 263 497 263 L 794 263 L 794 3656 L 441 3656 Q 347 3656 306 3693 Q 266 3731 266 3791 Q 266 3847 306 3884 Q 347 3922 441 3922 L 1056 3922 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-74" d="M 1381 2706 L 2806 2706 Q 2900 2706 2940 2668 Q 2981 2631 2981 2572 Q 2981 2516 2940 2478 Q 2900 2441 2806 2441 L 1381 2441 L 1381 694 Q 1381 466 1564 312 Q 1747 159 2100 159 Q 2366 159 2675 239 Q 2984 319 3156 419 Q 3219 459 3259 459 Q 3309 459 3346 420 Q 3384 381 3384 328 Q 3384 281 3344 241 Q 3244 138 2858 17 Q 2472 -103 2119 -103 Q 1659 -103 1387 112 Q 1116 328 1116 694 L 1116 2441 L 631 2441 Q 538 2441 497 2478 Q 456 2516 456 2575 Q 456 2631 497 2668 Q 538 2706 631 2706 L 1116 2706 L 1116 3481 Q 1116 3575 1153 3615 Q 1191 3656 1247 3656 Q 1306 3656 1343 3615 Q 1381 3575 1381 3481 L 1381 2706 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2e" d="M 1866 753 L 1978 753 Q 2153 753 2278 629 Q 2403 506 2403 331 Q 2403 150 2276 28 Q 2150 -94 1978 -94 L 1866 -94 Q 1691 -94 1566 29 Q 1441 153 1441 328 Q 1441 509 1567 631 Q 1694 753 1866 753 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-65" d="M 3341 1291 L 666 1291 Q 734 781 1092 470 Q 1450 159 1978 159 Q 2272 159 2594 256 Q 2916 353 3119 513 Q 3178 559 3222 559 Q 3272 559 3309 520 Q 3347 481 3347 428 Q 3347 375 3297 325 Q 3147 169 2764 33 Q 2381 -103 1978 -103 Q 1303 -103 851 339 Q 400 781 400 1409 Q 400 1981 823 2390 Q 1247 2800 1872 2800 Q 2516 2800 2931 2379 Q 3347 1959 3341 1291 zM 3075 1556 Q 2997 1991 2664 2262 Q 2331 2534 1872 2534 Q 1413 2534 1081 2265 Q 750 1997 669 1556 L 3075 1556 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-2f" d="M 3100 4113 L 988 -413 Q 934 -528 850 -528 Q 797 -528 758 -489 Q 719 -450 719 -403 Q 719 -366 750 -300 L 2859 4225 Q 2894 4297 2925 4319 Q 2956 4341 3000 4341 Q 3053 4341 3092 4302 Q 3131 4263 3131 4216 Q 3131 4181 3100 4113 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-63" d="M 3016 2441 L 3016 2528 Q 3016 2625 3053 2665 Q 3091 2706 3147 2706 Q 3206 2706 3243 2665 Q 3281 2625 3281 2528 L 3281 1934 Q 3278 1838 3242 1797 Q 3206 1756 3147 1756 Q 3094 1756 3058 1792 Q 3022 1828 3016 1913 Q 2997 2134 2723 2334 Q 2450 2534 1988 2534 Q 1403 2534 1100 2168 Q 797 1803 797 1331 Q 797 822 1131 490 Q 1466 159 1997 159 Q 2303 159 2620 271 Q 2938 384 3194 634 Q 3259 697 3309 697 Q 3363 697 3398 661 Q 3434 625 3434 572 Q 3434 438 3119 231 Q 2609 -103 1984 -103 Q 1350 -103 942 301 Q 534 706 534 1328 Q 534 1963 951 2381 Q 1369 2800 2003 2800 Q 2606 2800 3016 2441 z" transform="scale(0.015625)"/>       <path id="CourierNewPSMT-73" d="M 2788 2534 Q 2788 2625 2825 2665 Q 2863 2706 2919 2706 Q 2978 2706 3015 2665 Q 3053 2625 3053 2528 L 3053 2078 Q 3053 1984 3015 1943 Q 2978 1903 2919 1903 Q 2866 1903 2830 1937 Q 2794 1972 2788 2050 Q 2769 2238 2594 2359 Q 2338 2534 1916 2534 Q 1475 2534 1231 2356 Q 1047 2222 1047 2056 Q 1047 1869 1266 1744 Q 1416 1656 1834 1609 Q 2381 1550 2594 1475 Q 2897 1366 3045 1172 Q 3194 978 3194 753 Q 3194 419 2872 158 Q 2550 -103 1928 -103 Q 1306 -103 909 213 Q 909 106 896 75 Q 884 44 851 22 Q 819 0 778 0 Q 722 0 684 40 Q 647 81 647 175 L 647 716 Q 647 809 683 850 Q 719 891 778 891 Q 834 891 873 852 Q 913 813 913 747 Q 913 603 984 506 Q 1094 356 1333 257 Q 1572 159 1919 159 Q 2431 159 2681 350 Q 2931 541 2931 753 Q 2931 997 2678 1144 Q 2422 1291 1933 1341 Q 1444 1391 1231 1472 Q 1019 1553 900 1715 Q 781 1878 781 2066 Q 781 2403 1112 2601 Q 1444 2800 1903 2800 Q 2447 2800 2788 2534 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#CourierNewPSMT-64"/>      <use xlink:href="#CourierNewPSMT-79" x="60.009766"/>      <use xlink:href="#CourierNewPSMT-6e" x="120.019531"/>      <use xlink:href="#CourierNewPSMT-6f" x="180.029297"/>      <use xlink:href="#CourierNewPSMT-6d" x="240.039062"/>      <use xlink:href="#CourierNewPSMT-69" x="300.048828"/>      <use xlink:href="#CourierNewPSMT-67" x="360.058594"/>      <use xlink:href="#CourierNewPSMT-68" x="420.068359"/>      <use xlink:href="#CourierNewPSMT-74" x="480.078125"/>      <use xlink:href="#CourierNewPSMT-2e" x="540.087891"/>      <use xlink:href="#CourierNewPSMT-6e" x="600.097656"/>      <use xlink:href="#CourierNewPSMT-65" x="660.107422"/>      <use xlink:href="#CourierNewPSMT-74" x="720.117188"/>      <use xlink:href="#CourierNewPSMT-2f" x="780.126953"/>      <use xlink:href="#CourierNewPSMT-63" x="840.136719"/>      <use xlink:href="#CourierNewPSMT-68" x="900.146484"/>      <use xlink:href="#CourierNewPSMT-65" x="960.15625"/>      <use xlink:href="#CourierNewPSMT-73" x="1020.166016"/>      <use xlink:href="#CourierNewPSMT-73" x="1080.175781"/>     </g>    </a>   </g>   <g id="legend_1">    <g id="line2d_20">     <path d="M 242.532813 43.160313 L 252.532813 43.160313 L 262.532812 43.160313 " style="fill: none; stroke: #1f77b4; stroke-width: 0.5; stroke-linecap: square"/>    </g>    <g id="text_17">     <!-- gpt-3.5-turbo-instruct -->     <g style="fill: #1f77b4" transform="translate(270.532812 46.660313) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-67" d="M 3750 3366 L 3750 410 Q 3750 -448 3331 -861 Q 2912 -1274 2067 -1274 Q 1600 -1274 1181 -1136 Q 762 -998 499 -755 L 730 -410 Q 973 -627 1321 -748 Q 1670 -870 2054 -870 Q 2694 -870 2995 -572 Q 3296 -275 3296 352 L 3296 781 Q 3085 461 2742 294 Q 2400 128 1984 128 Q 1510 128 1123 336 Q 736 544 515 918 Q 294 1293 294 1766 Q 294 2240 515 2611 Q 736 2982 1120 3187 Q 1504 3392 1984 3392 Q 2413 3392 2758 3219 Q 3104 3046 3315 2720 L 3315 3366 L 3750 3366 zM 2029 531 Q 2394 531 2688 688 Q 2982 845 3145 1126 Q 3309 1408 3309 1766 Q 3309 2125 3145 2403 Q 2982 2682 2691 2838 Q 2400 2995 2029 2995 Q 1664 2995 1373 2841 Q 1082 2688 918 2406 Q 755 2125 755 1766 Q 755 1408 918 1126 Q 1082 845 1373 688 Q 1664 531 2029 531 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-33" d="M 1856 2560 Q 2560 2522 2928 2179 Q 3296 1837 3296 1286 Q 3296 902 3110 604 Q 2925 307 2560 134 Q 2195 -38 1670 -38 Q 1184 -38 752 118 Q 320 275 58 538 L 282 902 Q 506 672 877 528 Q 1248 384 1670 384 Q 2221 384 2521 624 Q 2822 864 2822 1286 Q 2822 1702 2521 1939 Q 2221 2176 1613 2176 L 1293 2176 L 1293 2515 L 2534 4070 L 243 4070 L 243 4480 L 3123 4480 L 3123 4154 L 1856 2560 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-2e" d="M 678 -32 Q 538 -32 438 70 Q 339 173 339 320 Q 339 467 438 566 Q 538 666 678 666 Q 819 666 921 566 Q 1024 467 1024 320 Q 1024 173 921 70 Q 819 -32 678 -32 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-62" d="M 2368 3392 Q 2842 3392 3226 3177 Q 3610 2963 3827 2572 Q 4045 2182 4045 1683 Q 4045 1184 3827 793 Q 3610 403 3226 185 Q 2842 -32 2368 -32 Q 1946 -32 1610 147 Q 1274 326 1069 666 L 1069 0 L 634 0 L 634 4749 L 1088 4749 L 1088 2726 Q 1299 3053 1628 3222 Q 1958 3392 2368 3392 zM 2336 371 Q 2688 371 2976 534 Q 3264 698 3427 998 Q 3590 1299 3590 1683 Q 3590 2067 3427 2368 Q 3264 2669 2976 2832 Q 2688 2995 2336 2995 Q 1978 2995 1693 2832 Q 1408 2669 1245 2368 Q 1082 2067 1082 1683 Q 1082 1299 1245 998 Q 1408 698 1693 534 Q 1978 371 2336 371 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-6f" d="M 2010 -32 Q 1523 -32 1132 189 Q 742 410 518 800 Q 294 1190 294 1683 Q 294 2176 518 2566 Q 742 2957 1132 3174 Q 1523 3392 2010 3392 Q 2496 3392 2886 3174 Q 3277 2957 3497 2566 Q 3718 2176 3718 1683 Q 3718 1190 3497 800 Q 3277 410 2886 189 Q 2496 -32 2010 -32 zM 2010 371 Q 2368 371 2653 534 Q 2938 698 3098 998 Q 3258 1299 3258 1683 Q 3258 2067 3098 2368 Q 2938 2669 2653 2832 Q 2368 2995 2010 2995 Q 1651 2995 1366 2832 Q 1082 2669 918 2368 Q 755 2067 755 1683 Q 755 1299 918 998 Q 1082 698 1366 534 Q 1651 371 2010 371 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-67"/>      <use xlink:href="#Montserrat-Regular-70" x="68.599991"/>      <use xlink:href="#Montserrat-Regular-74" x="136.399979"/>      <use xlink:href="#Montserrat-Regular-2d" x="176.999969"/>      <use xlink:href="#Montserrat-Regular-33" x="215.199966"/>      <use xlink:href="#Montserrat-Regular-2e" x="271.59996"/>      <use xlink:href="#Montserrat-Regular-35" x="292.799957"/>      <use xlink:href="#Montserrat-Regular-2d" x="349.399948"/>      <use xlink:href="#Montserrat-Regular-74" x="387.599945"/>      <use xlink:href="#Montserrat-Regular-75" x="428.199936"/>      <use xlink:href="#Montserrat-Regular-72" x="495.499924"/>      <use xlink:href="#Montserrat-Regular-62" x="535.599915"/>      <use xlink:href="#Montserrat-Regular-6f" x="603.399902"/>      <use xlink:href="#Montserrat-Regular-2d" x="666.099899"/>      <use xlink:href="#Montserrat-Regular-69" x="704.299896"/>      <use xlink:href="#Montserrat-Regular-6e" x="731.19989"/>      <use xlink:href="#Montserrat-Regular-73" x="798.899872"/>      <use xlink:href="#Montserrat-Regular-74" x="847.799866"/>      <use xlink:href="#Montserrat-Regular-72" x="888.399857"/>      <use xlink:href="#Montserrat-Regular-75" x="928.499847"/>      <use xlink:href="#Montserrat-Regular-63" x="995.799835"/>      <use xlink:href="#Montserrat-Regular-74" x="1052.099823"/>     </g>    </g>    <g id="line2d_21">     <path d="M 242.532813 57.57125 L 252.532813 57.57125 L 262.532812 57.57125 " style="fill: none; stroke: #2ca02c; stroke-width: 0.5; stroke-linecap: square"/>    </g>    <g id="text_18">     <!-- llama-3.1-70b -->     <g style="fill: #2ca02c" transform="translate(270.532812 61.07125) scale(0.1 -0.1)">      <defs>       <path id="Montserrat-Regular-6c" d="M 634 4749 L 1088 4749 L 1088 0 L 634 0 L 634 4749 z" transform="scale(0.015625)"/>       <path id="Montserrat-Regular-37" d="M 3494 4480 L 3494 4154 L 1619 0 L 1120 0 L 2963 4070 L 659 4070 L 659 3232 L 205 3232 L 205 4480 L 3494 4480 z" transform="scale(0.015625)"/>      </defs>      <use xlink:href="#Montserrat-Regular-6c"/>      <use xlink:href="#Montserrat-Regular-6c" x="26.899994"/>      <use xlink:href="#Montserrat-Regular-61" x="53.799988"/>      <use xlink:href="#Montserrat-Regular-6d" x="112.799973"/>      <use xlink:href="#Montserrat-Regular-61" x="218.899963"/>      <use xlink:href="#Montserrat-Regular-2d" x="277.899948"/>      <use xlink:href="#Montserrat-Regular-33" x="316.099945"/>      <use xlink:href="#Montserrat-Regular-2e" x="372.499939"/>      <use xlink:href="#Montserrat-Regular-31" x="393.699936"/>      <use xlink:href="#Montserrat-Regular-2d" x="429.799927"/>      <use xlink:href="#Montserrat-Regular-37" x="467.999924"/>      <use xlink:href="#Montserrat-Regular-30" x="526.899918"/>      <use xlink:href="#Montserrat-Regular-62" x="593.099915"/>     </g>    </g>    <g id="line2d_22">     <path d="M 242.532813 71.932188 L 252.532813 71.932188 L 262.532812 71.932188 " style="fill: none; stroke: #8c564b; stroke-width: 0.5; stroke-linecap: square"/>    </g>    <g id="text_19">     <!-- llama-3.1-70b-instruct -->     <g style="fill: #8c564b" transform="translate(270.532812 75.432188) scale(0.1 -0.1)">      <use xlink:href="#Montserrat-Regular-6c"/>      <use xlink:href="#Montserrat-Regular-6c" x="26.899994"/>      <use xlink:href="#Montserrat-Regular-61" x="53.799988"/>      <use xlink:href="#Montserrat-Regular-6d" x="112.799973"/>      <use xlink:href="#Montserrat-Regular-61" x="218.899963"/>      <use xlink:href="#Montserrat-Regular-2d" x="277.899948"/>      <use xlink:href="#Montserrat-Regular-33" x="316.099945"/>      <use xlink:href="#Montserrat-Regular-2e" x="372.499939"/>      <use xlink:href="#Montserrat-Regular-31" x="393.699936"/>      <use xlink:href="#Montserrat-Regular-2d" x="429.799927"/>      <use xlink:href="#Montserrat-Regular-37" x="467.999924"/>      <use xlink:href="#Montserrat-Regular-30" x="526.899918"/>      <use xlink:href="#Montserrat-Regular-62" x="593.099915"/>      <use xlink:href="#Montserrat-Regular-2d" x="660.899902"/>      <use xlink:href="#Montserrat-Regular-69" x="699.099899"/>      <use xlink:href="#Montserrat-Regular-6e" x="725.999893"/>      <use xlink:href="#Montserrat-Regular-73" x="793.699875"/>      <use xlink:href="#Montserrat-Regular-74" x="842.599869"/>      <use xlink:href="#Montserrat-Regular-72" x="883.19986"/>      <use xlink:href="#Montserrat-Regular-75" x="923.29985"/>      <use xlink:href="#Montserrat-Regular-63" x="990.599838"/>      <use xlink:href="#Montserrat-Regular-74" x="1046.899826"/>     </g>    </g>    <g id="line2d_23">     <path d="M 242.532813 86.293125 L 252.532813 86.293125 L 262.532812 86.293125 " style="fill: none; stroke: #bcbd22; stroke-width: 0.5; stroke-linecap: square"/>    </g>    <g id="text_20">     <!-- gemma-2-27b -->     <g style="fill: #bcbd22" transform="translate(270.532812 89.793125) scale(0.1 -0.1)">      <use xlink:href="#Montserrat-Regular-67"/>      <use xlink:href="#Montserrat-Regular-65" x="68.599991"/>      <use xlink:href="#Montserrat-Regular-6d" x="128.999985"/>      <use xlink:href="#Montserrat-Regular-6d" x="235.099976"/>      <use xlink:href="#Montserrat-Regular-61" x="341.199966"/>      <use xlink:href="#Montserrat-Regular-2d" x="400.199951"/>      <use xlink:href="#Montserrat-Regular-32" x="438.399948"/>      <use xlink:href="#Montserrat-Regular-2d" x="495.199936"/>      <use xlink:href="#Montserrat-Regular-32" x="533.399933"/>      <use xlink:href="#Montserrat-Regular-37" x="590.199921"/>      <use xlink:href="#Montserrat-Regular-62" x="649.099915"/>     </g>    </g>    <g id="line2d_24">     <path d="M 242.532813 100.704063 L 252.532813 100.704063 L 262.532812 100.704063 " style="fill: none; stroke: #17becf; stroke-width: 0.5; stroke-linecap: square"/>    </g>    <g id="text_21">     <!-- gemma-2-27b-it -->     <g style="fill: #17becf" transform="translate(270.532812 104.204063) scale(0.1 -0.1)">      <use xlink:href="#Montserrat-Regular-67"/>      <use xlink:href="#Montserrat-Regular-65" x="68.599991"/>      <use xlink:href="#Montserrat-Regular-6d" x="128.999985"/>      <use xlink:href="#Montserrat-Regular-6d" x="235.099976"/>      <use xlink:href="#Montserrat-Regular-61" x="341.199966"/>      <use xlink:href="#Montserrat-Regular-2d" x="400.199951"/>      <use xlink:href="#Montserrat-Regular-32" x="438.399948"/>      <use xlink:href="#Montserrat-Regular-2d" x="495.199936"/>      <use xlink:href="#Montserrat-Regular-32" x="533.399933"/>      <use xlink:href="#Montserrat-Regular-37" x="590.199921"/>      <use xlink:href="#Montserrat-Regular-62" x="649.099915"/>      <use xlink:href="#Montserrat-Regular-2d" x="716.899902"/>      <use xlink:href="#Montserrat-Regular-69" x="755.099899"/>      <use xlink:href="#Montserrat-Regular-74" x="781.999893"/>     </g>    </g>    <g id="line2d_25">     <path d="M 242.532813 115.115 L 252.532813 115.115 L 262.532812 115.115 " style="fill: none; stroke-dasharray: 0.5,0.825; stroke-dashoffset: 0; stroke: #ff7f0e; stroke-width: 0.5"/>    </g>    <g id="text_22">     <!-- gpt-3.5-turbo -->     <g style="fill: #ff7f0e" transform="translate(270.532812 118.615) scale(0.1 -0.1)">      <use xlink:href="#Montserrat-Regular-67"/>      <use xlink:href="#Montserrat-Regular-70" x="68.599991"/>      <use xlink:href="#Montserrat-Regular-74" x="136.399979"/>      <use xlink:href="#Montserrat-Regular-2d" x="176.999969"/>      <use xlink:href="#Montserrat-Regular-33" x="215.199966"/>      <use xlink:href="#Montserrat-Regular-2e" x="271.59996"/>      <use xlink:href="#Montserrat-Regular-35" x="292.799957"/>      <use xlink:href="#Montserrat-Regular-2d" x="349.399948"/>      <use xlink:href="#Montserrat-Regular-74" x="387.599945"/>      <use xlink:href="#Montserrat-Regular-75" x="428.199936"/>      <use xlink:href="#Montserrat-Regular-72" x="495.499924"/>      <use xlink:href="#Montserrat-Regular-62" x="535.599915"/>      <use xlink:href="#Montserrat-Regular-6f" x="603.399902"/>     </g>    </g>   </g>  </g> </g> <defs>  <clipPath id="p41ad9ac113">   <rect x="54" y="30.24" width="334.8" height="194.04"/>  </clipPath> </defs></svg>](https://dynomight.net/img/chess/instruct_mean.pdf)

(Again, do not be confused by the name `gpt-3.5-turbo-instruct`. I stress that this is more like a base model than `gpt-3.5-turbo`. This is the opposite of the naming scheme everyone else uses where “`instruct`” or “`it`” means more tuning to be good at chatting.)

In all cases, additional instruction tuning makes the model worse. But the difference is very small in two cases, and enormous in the other.

## Possible theories

I can think of four possible explanations.

**Theory 1: Base models _at sufficient scale_ can play chess, but instruction tuning destroys it.**

This would be consistent with our data. But I did manage to get [`llama-3.1-405b`](https://huggingface.co/meta-llama/Llama-3.1-405B) to play a couple games. Despite being larger than `gpt-3.5-turbo`, it was still terrible.

**Theory 2: GPT-3.5-instruct was trained on more chess games.**

All models were clearly trained on a _lot_ of chess games. But it’s hard to know exactly how many.

**Theory 3: There’s something particular about different transformer architectures.**

I doubt this, but it could be that for some reason, Llama type models are uniquely bad at chess.

**Theory 4: There’s “competition” between different types of data.**

We know that transformers trained specifically on chess games can be [extremely good](https://github.com/sgrvinod/chess-transformers) at chess. Maybe `gpt-3.5-turbo-instruct` happens to have been trained on a higher _fraction_ of chess games, so it decided to dedicate a larger fraction of its parameters to chess.

That is, maybe LLMs sort of have little “chess subnetworks” hidden inside of them, but the size of the subnetworks depends on the fraction of data that was chess. (If this theory were true, we should probably expect that _big enough_ models should become good at chess, provided they are trained on enough chess games, even if the _fraction_ of chess games is low.)

## Details

I did things this way (i.e. by working with standard algebraic notation) because this is how people got good results two years ago, and in preliminary experiments I also found it to work best.

If you want to know _exactly_ how I did things, here are some words: I ran all the open models (anything _not_ from OpenAI, meaning anything that doesn’t start with `gpt` or `o1`) myself using `Q5_K_M` quantization, whatever that is. For the open models I manually generated the set of legal moves and then used [grammars](https://github.com/ggerganov/llama.cpp/blob/master/grammars/README.md) to constrain the models, so they always generated legal moves. Since OpenAI is lame and doesn’t support full grammars, for the closed (OpenAI) models I tried generating up to 10 times and if it still couldn’t come up with a legal move, I just chose one randomly. For the chat models `llama-3.1-70b-instruct`, `gemma-2-27b-it`, `gpt-3.5-turbo`, `gpt-4o-mini`, and `gpt-4o` I changed the system prompt to “You are a chess grandmaster. You will be given a partially completed game. After seeing it, you should choose the next move.” It’s impossible to change the system prompt for `o1-mini`, so I didn’t. I used a temperature of 0.7 for all the open models and the default for the closed (OpenAI) models. The fact that OpenAI has “open” as part of their name sure made this paragraph hard to write.

## Token weirdness

One _extremely_ strange thing I noticed was that if I gave a prompt like “`1. e4 e5 2. `” (with a space at the end), the open models would play much, _much_ worse than if I gave a prompt like “`1 e4 e5 2.`” (without a space) and let the model generate the space itself. Huh?

After some confusion, I’m pretty sure this is because of the tokenizer. Look at how the [Llama tokenizer](https://belladoreai.github.io/llama-tokenizer-js/example-demo/build/) breaks up a string of moves:

![_resources/Something weird is happening with LLMs and chess/9b268124f98d2bbc36708b8c87928965_MD5.png](/img/user/_resources/Something%20weird%20is%20happening%20with%20LLMs%20and%20chess/9b268124f98d2bbc36708b8c87928965_MD5.png)

After the “`1.`”, it generates “` e`” as a _single token_. That’s not the same as having a space followed by an `e`. So putting in the space and asking models to generate tokens presents the model with a confusing situation and leads to bad predictions.

The right way to deal with this is “token healing”—to delete the last token of the input and then do constrained generation over all strings that start with the deleted stuff. But I couldn’t figure out any easy way to do that. So, instead I left the space out and modified the grammar so that the model could generate a space (or not), then one of the current legal moves, and then another space (or not).

## P.S.

Some people have asked to see all the games from `gpt-3.5-turbo-instruct`. Behold: [1](https://lichess.org/iwi02kUm)[2](https://lichess.org/bfaDMlVm)[3](https://lichess.org/lNE5mKPO)[4](https://lichess.org/C3xV1uAz)[5](https://lichess.org/YpzT2KQS)[6](https://lichess.org/dy9m2DsU)[7](https://lichess.org/4YDx633U)[8](https://lichess.org/ZM9ZbDfo)[9](https://lichess.org/59cTZSjs)[10](https://lichess.org/DmeQQx7Y)
